<?php

/* Reject incoming call
*/
class ActForward extends RuleHandlerBase
{
  var $main; // ruler
  // if true we accept, which is needed for reading phone number
  var $DoNotRoute = false;
  // if false, then call needs answering
  var $ActiveCall = false;
  // if special option is on to forwarded to phone number read from DTMF
  // the number ends with #, and timeout applies
  var $ReadNum = false;
  // the number read from DTMF if ReadNum is enabled
  var $Num = '';
  // have we started calling the receiver?
  // is now stored in main->callingReceiver
  // because shared between this and act_notify
  // how many times disconnected?
  var $selfDisconnects = 0;

  var $ForwardProt = '';
  
  function Init($defMain)
  {
    $this->main = $defMain;
  }
  
  function RoutingCall($ev)
  {
    $this->Determine($ev->params);
    
    // We only change parameters (above),
    // but not do the routing.
    // Otherwise routing for registered phones does not work at all,
    // and for SIP lines, authentication does not work.
    
    
    if ($this->DoNotRoute) return false;
    
    return true;
  }
  
  function Determine(& $params)
  {
    $forwDst = $this->main->GetThisRuleCfg("ForwDst");
    $forwDstInt = (int) $forwDst;
    $determDst = $this->main->GetThisRuleCfg("DetermDst");
    
    $prot = $this->main->GetThisRuleCfg("ForwProt");

    if($prot == '' || $prot=='0')
    {
      $this->ForwardProt = 'UDP';
    }
    else if($prot=='1')
    {
      $this->ForwardProt = 'TCP';
    }
    else if($prot=='2')
    {
      $this->ForwardProt = 'TLS';
    }
    else if($prot=='3')
    {
      if($forwDstInt > 0 && $this->main->GetCfg("Sip" . $forwDstInt . 'Reg')!=0)
      {
        $this->ForwardProt = "";
      }
      else
      {
        $this->ForwardProt =  $this->main->originalData['ip_transport'];
      }
    }
    else
    {
      $this->ForwardProt = 'UDP';
    }

    $dstnum = '';
    if ($determDst=='ori')
    {
      $dstnum = $this->main->originalData['called'];
      $toremove = $this->main->GetThisRuleCfg("DstPrefix");
      if ($toremove!='' && strpos($dstnum,$toremove)===0)
      {
        $dstnum = substr($dstnum,strlen($toremove));
      }
      $toadd = $this->main->GetThisRuleCfg("DstAPrefix");
      if ($toadd!='')
      {
        $dstnum = $toadd . $dstnum;
      }
    } else if ($determDst=='fixed')
    {
      $dstnum = $this->main->GetThisRuleCfg("DstNum");
    } else if ($determDst=='read')
    {
      // handled elsewhere
      $this->ReadNum = true;
      $this->DoNotRoute = true;
      $dstnum = '?';
    }
    
    $caller = $this->main->originalData['caller'];
    if ($this->main->GetThisRuleCfg("UseUsername")==1)
    {
      // otherwise PBX might not allow
      // needed for Asterisk
      if ( $forwDstInt>0
      && $this->main->GetCfg("Sip" . $forwDstInt . 'Reg')!=0)
      {
        $caller = $this->main->GetCfg("Sip" . $forwDstInt . 'Usr');
      }
    }
    
    $srcConv = $this->main->GetThisRuleCfg("SrcConv");
    $srcConvParts = explode(',',$srcConv);
    foreach ($srcConvParts as $conv)
    {
      $parts = explode('=',$conv);
      if (count($parts)!=2) continue;
      if ($this->main->MatchesPattern($parts[0],$caller))
      {
        $caller = $parts[1];
      }
    }
    
    $InLoop = ((int) $this->main->GetThisRuleCfg('InLoop'))!=0;
    $ip = "";
    if ($forwDstInt > 0)
    {
      $ip = $this->main->GetCfg('Sip' . $forwDstInt . 'Ip');
      // does not work with auth: $params["callto"] = 'sip/sip:' . $dstnum. "@" . $ip;
      $params["context"] = "forwSip" . $forwDst;
    }
    
    $logNum = $dstnum;
    if ($this->ReadNum) $logNum = '[number to read]';
    dbglog("Forward to $logNum @ $ip with source $caller using SIP line $forwDst over " . (strlen($this->ForwardProt) ? $this->ForwardProt : "line settings"));
    
    
    $params["rtp_forward"] = $InLoop ? "no" : "yes";
    $params["caller"] = $caller;
    $params["called"] = $dstnum;
    if ( ($this->main->GetThisRuleCfg("SetPAsserted")==1)
    && isset( $this->main->originalData['sip_p-asserted-identity'] ) )
    {
      $params["osip_P-Asserted-Identity"] = $this->main->originalData['sip_p-asserted-identity'];
    }

    if(!$this->DoNotRoute && strlen($this->ForwardProt) > 0)
    {
      $params['oip_transport'] = $this->ForwardProt;
    }
    
  }
  
  function UnansweredCall()
  {
    $params = array();
    $this->Determine($params);
    
    if ( $this->ReadNum
    || ($this->main->addOnRuleHandler!=null
      && $this->main->addOnRuleHandler->AnswerCallBeforeRouting() )
    )
    {
      if ($this->ActiveCall==false)
      {
        $m = new Yate("call.answered");
        foreach ($params as $key=>$value)
        {
          $m->params[$key] = $value;
        }
        $m->params["id"] = $this->main->instanceId;
        $m->params["targetid"] = $this->main->originalData['id'];
        $m->Dispatch();
      }
    }
    if ($this->ReadNum) return;
    
    dbglog("forward using route");
    $m = new Yate("call.route");
    $m->params["id"] = $this->main->originalData['id'];
    foreach ($params as $key=>$value)
    {
      $m->params[$key] = $value;
      // verbose: dbglog($key . '=' . $value);
    }
    $m->Dispatch();
  }
  
  function ActiveCall()
  {
    $this->ActiveCall = true;
    $this->UnansweredCall();
  }
  
  function InitiateTransfer($num)
  {
    dbglog("forward using route to $num");
    $m = new Yate("call.route");
    $this->Determine($m->params);
    $m->params['called'] = $num;
    $m->params["id"] = $this->main->originalData['id'];
    $m->Dispatch();
  }
  
  function Dtmf($digit)
  {
    if ($this->ReadNum==false) return;
    
    dbglog("Reading DTMF $digit");
    if ($digit=='#')
    {
      if (strlen($this->Num)>=1)
      {
        $this->InitiateTransfer( $this->Num );
      }
    } else
    {
      $this->Num .= $digit;
      dbglog("Number: " . $this->Num );
    }
    
  }
  
  function ChannelDisco($ev)
  {
    if($ev->GetValue('id') == $this->main->instanceId)
    {
      $this->selfDisconnects += 1;
      if($this->selfDisconnects == 2 && $this->main->callingReceiver)
      {
        dbglog("Receiver disconnected");

        $m = new Yate("call.drop");
        $m->params["id"] = $this->main->originalData['id'];
        $m->params["reason"] = "603";
        $m->Dispatch();        
      }
    } elseif($ev->GetValue('lastpeerid') == $this->main->originalData['id']
    && $this->main->callingReceiver)
    {
      dbglog("Caller disconnected while calling receiver");
      $this->main->callingReceiver = false;
      if (isset($this->receiverId))
      {
        $m = new Yate("call.drop");
        // call drop with empty id seems to drop *all* calls,
        // that is not good
        $m->params["id"] = $this->receiverId;
        $m->params["reason"] = "487";
        $m->Dispatch();
      }
    }
  }

  function ChannelStarted($ev)
  {
    if ($this->main->callingReceiver)
    {
      $this->receiverId = $ev->params['id'];
    }
  }

  
  function Routed( $ev )
  {
    dbglog('Forward action routed');
    
    dbglog('Connect caller to ringback tone');
    // connect caller to ringback tone
    $m = new Yate("chan.masquerade");
    $m->params['message'] = 'call.execute';
    $m->params['callto'] = "tone/ring";
    $m->params['id'] = $this->main->originalData['id'];
    $m->Dispatch();
    
    dbglog('Call destination');
    $m = new Yate("call.execute");
    $m->params['originatingId'] = $this->main->instanceId;
    $m->params['copyparams'] = 'originatingId';
    $m->params['callto'] = $ev->retval;
    
    if(strlen($this->ForwardProt) > 0)
    {
      $m->params['oip_transport'] = $this->ForwardProt;
    }
    
    foreach ($ev->params as $key=>$value)
    {
      if($key != "id")
      {
        $m->params[$key] = $value;
      }
    }
    $m->Dispatch();
    $this->main->callingReceiver = true;
  }
  
  function CallAnswered( $ev )
  {
    if($ev->GetValue("lastpeerid") == $this->main->instanceId)
    {
      dbglog("Forwarding target has answered");
      $this->receiverId = $ev->GetValue("id");
      $this->main->callingReceiver = false;

      $m = new Yate("chan.connect");
      $m->params["id"] = $this->main->originalData['id'];
      $m->params["targetid"] = $this->receiverId;
      $m->Dispatch();
      
    }
  }
  
};


?>
