<?php

/* Actions involving the Recording Engine
*/
class ActRec extends RuleHandlerBase
{
  var $main; // ruler
  var $act; // the action (could be 'keep')
  
  function __construct($act)
  {
    $this->act = $act;
  }
  
  function Init($defMain)
  {
    $this->main = $defMain;
  }
  
  function IsCallAction()
  {
    // this action does not act on the call itself
    return false;
  }
  
  function Redo($ev)
  {
    dbglog('Needs to redo ' . $this->act);
    $callid = $ev->GetValue('sip_callid');
    if ($callid=='') return;
    $this->PerformOnCall($callid);
  }
  
  function PerformOnCall($callid)
  {
    dbglog('Performing action ' . $this->act . ' on ' . $callid);
    $cmd = $this->act . '-callid ' . $callid;
    $this->SendMsgToOrk( $cmd );
  }

  function Perform()
  {
    $this->PerformOnCall( $this->main->originalData['sip_callid'] );
  }
 
  function SendMsgToOrk($msg)
  {
    $s = new socket_client();
    $s->write($msg);
    $s->close();
  }
  
}

// socket helper function

function socketTryRead(&$socket)
{
	socket_set_nonblock($socket);
	$read = array($socket);
	$write = null;
	$except = null;
  $seconds = 1;
  $flags = PHP_NORMAL_READ;
  $size = 2 * 1024;
	if (socket_select($read,$write,$except,$seconds)==1)
	{
  	return socket_read($socket,$size,$flags);
 	} 
  return '';
}

class socket_client
{
	var $socket = FALSE;
	var $connected = FALSE;
	
	function close()
	{
    if ($this->socket !== false)
    {
      // verbose: dbglog("closing socket");
      socket_close($this->socket);
    }
		$this->socket = FALSE;
		$this->connected = FALSE;
	}
	
	function create_socket()
	{
		$this->socket = @socket_create(AF_UNIX, SOCK_STREAM, 0);
		if($this->socket === FALSE)
		{
			dbglog("Could not create socket: " . socket_strerror(socket_last_error()));
		}
		else
		{
			// verbose: dbglog("Created socket");
		}
	}
	
	function connect()
	{
		$res = @socket_connect($this->socket, '/tmp/apresa_ork');
		if($res)
		{
			// verbose: dbglog("Connected to socket");
			$this->connected = TRUE;
		}
		else
		{
			dbglog("Could not connect socket: " . socket_strerror(socket_last_error()));
			$this->connected = FALSE;
		}
	}
	
	function write_real($msg)
	{
    $buf = $msg; // . "\r\n";
		$res = @socket_write($this->socket, $buf, strlen($buf) );
		
		if($res === FALSE)
		{
			dbglog("Could not write to socket: " . socket_strerror(socket_last_error()));
			$this->close();
		}
		else
		{
			// verbose: dbglog("Wrote " . $res . " bytes to socket");
      $data = socketTryRead( $this->socket );
      dbglog("Reply from ork: " . trim($data) );
			return TRUE;
		}
	}
	
	function create_connection()
	{
		if($this->socket === FALSE)
		{
			$this->create_socket();
			if($this->socket === FALSE) return FALSE;
		}
		if($this->connected === FALSE)
		{
			$this->connect();
			if($this->connected == FALSE) return FALSE;
		}
		return TRUE;
	}
	
	
	function write($msg)
	{
		dbglog("to-ork: " . $msg);
		if(!($this->create_connection() && $this->write_real($msg)))
		{
			dbglog("Failed to send, reestablish connection and try again");
      $this->close();
			if(!($this->create_connection() && $this->write_real($msg)))
			{
				dbglog("Still failure to send, giving up");
			}
		}
	}
}
