#!/usr/bin/php
<?php
// The first line is needed to make executable

include("libyate.php");

function dbglog($t)
{
    Yate::Debug('DtmfDialout: '.$t);
}

class DtmfDialout
{
    // ID of this endpoint instance
    var $instanceId;
    // the communicated telephone number
    var $number = '';
    // start time of the call
    var $startTime = null;
    // the channel id of the other side
    var $otherSideId;
    // the telephone number or ID of the caller
    var $caller;
		var $callername;
		// Context for forwarding
		var $forwContext;
		var $useOriginalCallerId;

    // if true, transfer to '300' after 2 seconds
    var $debugAutoTransfer = false;

    function Init()
    {
			global $argc, $argv;
			
			Yate::Init();
			Yate::Debug(true);

			$this->forwIpAddr = '';
			$this->forwSipLine = '';
			$this->useOriginalCallerId = true;
			if ($argc>=2)
			{
				$parts = explode(',',$argv[1]);
				$this->forwContext = $parts[0];
				$this->useOriginalCallerId = $parts[1]==1;
				dbglog('Forward to context: ' . $this->forwContext);
			}
			
			$this->instanceId = "DtmfDialout/".uniqid();
			Yate::Install("chan.dtmf",100,"targetid",$this->instanceId);
			Yate::Install("engine.timer");
    }
    
    function Main()
    {
        $this->Init();

        // main loop
        while (true)
        {
            $ev = Yate::GetEvent();
            if ($ev === false)
                break;
            if ($ev === true)
                continue;
            if ($ev->name!='engine.timer')
            {
                dbglog("Event received ".$ev->type.' '.$ev->name);
            }
            $this->HandleEvent($ev);
            
            // timing
            // currently, only for debugging
            if ($ev->name=='engine.timer')
            {
                $this->OnNewSecond();
            }
            
            
        }
    }

    function OnNewSecond()
    {
        if ($this->startTime===null) return;
        
        if ($this->debugAutoTransfer)
        {
            if (time() - $this->startTime>2)
            {
                dbglog('Longer than 2 seconds!');
                // prevent doing it again
                $this->startTime = null;
                
                $this->InitiateTransfer('300');
            }
        }
        
        
    }
    
    function InitiateTransfer($target)
    {
        dbglog('Transfer to '.$target);
        $m = new Yate("call.route");
        $m->params["id"] = $this->otherSideId;
        
				// user the other as caller might cause
        // an authentication problem,
        // it might be prefered or not,
        // so it must be an option.
				if ($this->useOriginalCallerId)
				{
					$m->params["caller"] = $this->caller;
					$m->params["callername"] = $this->callername;
				}
				
        $m->params["called"] = $target;
				$m->params["context"] = $this->forwContext;
        $m->Dispatch();
				return;
    }
    
    function HandleIncomingEvent($ev)
    {
        $acked = false;
        if ($ev->name=="call.execute")
        {
            // A new incoming call arriving
            
            $this->caller = $ev->GetValue("caller");
						$this->callername = $ev->GetValue("callername");
            dbglog("Caller id: ".$this->caller.' name: '.$this->callername);
            $this->otherSideId = $ev->GetValue("id");
            
            $ev->params["targetid"] = $this->instanceId;
            $ev->handled = true;
            $ev->Acknowledge();
            $acked = true;
            
            $this->startTime = time();
            
            /* Signal we are answering the call */
            $m = new Yate("call.answered");
            $m->params["id"] = $this->instanceId;
            $m->params["targetid"] = $this->otherSideId;
            $m->Dispatch();
        }
        if ($ev->name=="chan.dtmf")
        {
            $digit = $ev->params['text'];
            dbglog('Received DTMF: '.$digit);
            if ($digit=='#')
            {
                if (strlen($this->number)>=1)
                {
                    $ev->Acknowledge();
                    $acked = true;
                    
                    $this->InitiateTransfer($this->number);
                } else
                {
                    $this->number = '';
                }
            } else
            {
                $this->number .= $digit;
                dbglog('Number: '.$this->number);
            }
        }

        // we must ACK all events of type incoming
        if (!$acked) $ev->Acknowledge();
        $acked = true;
    }
    
    function HandleEvent($ev)
    {
        if ($ev->type=="incoming")
        {
            $this->HandleIncomingEvent($ev);
        }
        if ($ev->type=="answer")
        {
            if ($ev->name=="call.route")
            {
                // This is after the transfer is routed.
                dbglog('Routed, now transfer');
								dbglog("To: ".$ev->retval);

                $m = new Yate("chan.masquerade");
                $m->params['message'] = 'call.execute';
                $m->params['callto'] = $ev->retval;
                foreach ($ev->params as $key=>$value)
                {
									dbglog("Param[".$key."]".$value);
                  $m->params[$key] = $value;
                }
                $m->Dispatch();
            }
        }
    }
    
}

$t = new DtmfDialout();
$t->Main();

dbglog("Goodbye");



?>