#!/usr/bin/php
<?php // The first line is needed to make executable

// Checks messages for encrypted SIPREC SIP signaling
// and passes this on to orkaudio
include("libyate.php");

function dbglog($t) {
    Yate::Output( date('D M G:i:s Y') . ' EncryptionHandler: '.$t);
}

class socket_client
{
	var $socket = FALSE;
	var $connected = FALSE;
	
		
	function close()
	{
		socket_close($this->socket);
		$this->socket = FALSE;
		$this->connected = FALSE;
	}
	
	function create_socket()
	{
		$this->socket = @socket_create(AF_UNIX, SOCK_STREAM, 0);
		if($this->socket === FALSE)
		{
			dbglog("Could not create socket: " . socket_strerror(socket_last_error()));
		}
		else
		{
			dbglog("Created socket");
		}
	}
	
	function connect()
	{
		$res = @socket_connect($this->socket, '/tmp/apresa_yate');
		if($res)
		{
			dbglog("Connected to socket");
			$this->connected = TRUE;
		}
		else
		{
			dbglog("Could not connect socket: " . socket_strerror(socket_last_error()));
			$this->connected = FALSE;
		}
	}
	
	function write_real($msg)
	{
		$len = strlen($msg) + 4;
		$buf = pack('L', $len) . $msg;
		
		$res = @socket_write($this->socket, $buf);
		
		if($res === FALSE)
		{
			dbglog("Could not write to socket: " . socket_strerror(socket_last_error()));
			$this->close();
		}
		else
		{
			dbglog("Wrote " . $res . " bytes to socket");
			return TRUE;
		}
	}
	
	function create_connection()
	{
		if($this->socket === FALSE)
		{
			$this->create_socket();
			if($this->socket === FALSE) return FALSE;
		}
		
		if($this->connected === FALSE)
		{
			$this->connect();
			if($this->connected == FALSE) return FALSE;
		}
		
		return TRUE;
	}
	
	
	function write($msg)
	{
		
		if(!($this->create_connection() && $this->write_real($msg)))
		{
			dbglog("Failed to send, reestablish connection and try again");
			if(!($this->create_connection() && $this->write_real($msg)))
			{
				dbglog("Still failure to send, giving up");
			}
		}
	}
}

class SIPRECEncryptionHandler {
	
	var $calls = array();
	
	var $local_ip; 
	
	var $sock;
	
	var $delim_char = "\x1e";
	
	function Init()
	{
		global $argc, $argv;
    
		Yate::Init();
		Yate::Debug(true);

		dbglog('init');
		
		$this->instanceId = "SIPRECEncryptionHandler/".uniqid();
		// without next line, problem.
		Yate::Install("engine.timer");
    Yate::Install("call.rawsip");
		
		$this->local_ip = getHostByName(getHostName());
		$this->sock = new socket_client();
	}
	
  
  function BuildMessageSIP($sip, $localip, $localport, $remoteip, $remoteport, $outgoing)
  {
    $msg = "type=" . "SIP" . $this->delim_char;
    $msg .= "sip=" . $sip . $this->delim_char;
    $msg .= "localaddr=" . $localip . ":" . $localport . $this->delim_char;
    $msg .= "remoteaddr=" . $remoteip . ":" . $remoteport . $this->delim_char;
    $msg .= "outgoing=" . $outgoing . $this->delim_char;
    
    return $msg;
  }
	
	function HandleEvent($ev)
	{
    if($ev->name === "call.rawsip")
    {
      $ev->handled = true;
      $sip = $ev->GetValue('sip');
      if($sip === null) return;
      
      $remote_ip = $ev->GetValue("remoteip", "0.0.0.0");
      $remote_port = $ev->GetValue("remoteport", "5060");
      
      $local_ip = $ev->GetValue("localip", "127.0.0.1");
      $local_port = $ev->GetValue("localport", "5060");
      
      $outgoing = $ev->GetValue("outgoing", "yes");
      
      $out = $this->BuildMessageSIP($sip, $local_ip, $local_port, $remote_ip, $remote_port, $outgoing);
      dbglog("Sending message of type SIP");
      $this->sock->write($out);      
    }
	}
	
	function YateEventHandling()
	{
		while (true)
		{
			$ev = Yate::GetEvent();
			if ($ev === false)
					return false; // YATE closed probably
			if ($ev === true)
					break; // no messages now
			if ($ev->name!='engine.timer')
			{
				$this->HandleEvent($ev);
			}
			
			if ($ev->type=='incoming')
			{
				$ev->Acknowledge();
			}
		}
		return true;
	}
	
	function Main()
	{
		$this->Init();
		
		while (true)
		{
			if ($this->YateEventHandling()==false)
			{
				break;
			}
		}
	}
}

$t = new SIPRECEncryptionHandler();
$t->Main();
dbglog("Goodbye");
?>
