#!/usr/bin/php
<?php // The first line is needed to make executable
require_once("libyate.php");
require_once("ruler.php");

function dbglog($t) {
  Yate::Debug( date('D M G:i:s Y') . ' global: '.$t);
}

class VidiGlobal {

	var $calls = array();
  // loaded configuration from database
  var $configs_t = null;
  var $instanceId;
  var $originalData = array();

	function Init()
	{
		global $argc, $argv;

		Yate::Init();
		Yate::Debug(true);

		dbglog('init');

		$this->instanceId = "VidiGlobal/".uniqid();
		Yate::Install("call.route",80);
    Yate::Install("engine.init"); // for settings reload
    
    $this->LoadSettings();
	}
	
	function YateEventHandling()
	{
		while (true)
		{
			$ev = Yate::GetEvent();
			if ($ev === false)
					return false; // YATE closed probably
			if ($ev === true)
					break; // no messages now
			if ($ev->name!='engine.timer')
			{
					dbglog("Event received (" . $ev->type.') ' . $ev->name );
			}
			$this->HandleEvent($ev);
		}
		return true;
	}
	
	function Main()
	{
		$this->Init();
		$this->YateEventHandling();
	}
	
  // virtual: if true we do not actually perform anything
  function Process($virt, $ev)
  {
    // arrays are assigned by copy
    $paramsCopy = $ev->params;
    $originalData = array();
    $ruler = new Ruler();
    $ruler->Init($virt, $this->configs_t);
    $toCopy = $ruler->toCopy;
    // copy selected parameters of this call
    foreach ($toCopy as $key)
    {
      $value = $ev->GetValue($key);
      $this->originalData[$key] = $value;
      // verbose: dbglog("route $key = $value");
    }
    $ruler->originalData = $this->originalData;
    
    $ruler->processRules();
    
    $directRouting = $ruler->ruleHandler->RoutingCall($ev);
    if ($ruler->addOnRuleHandler!=null
    && $ruler->addOnRuleHandler->DirectRoutingAllowed()==false)
    {
      $directRouting = false;
    }
    if ($virt)
    {
      // restore
      $ev->params = $paramsCopy;
    }
    return $directRouting;
  }
  
  function EnsureSettingsLoaded()
  {
    if ($this->configs_t == null)
    {
      dbglog('Try load settings');
      $this->LoadSettings();
    }
  }
  
  function LoadSettings()
  {
    try
    {
      $dbintf = new DBInterface(DATA_PATH_ABS,'apresa',true);
      $this->configs_t = $dbintf->LoadConfigsPart('t');
      $dbintf = null;
      dbglog('Settings loaded');
    } catch(Exception $e)
    {
      dbglog('Failed to load settings');
    }
  }
  
  function EngineInit()
  {
    dbglog("Reload settings");
    $this->LoadSettings();
  }
  
  function HandleIncomingEvent($ev)
  {
    $acked = false;

    $this->EnsureSettingsLoaded();
    if ($this->configs_t==null)
    {
      $ev->Acknowledge();
      return;
    }
    
    if ($ev->name=='engine.init')
    {
      $this->EngineInit();
    }
    
    if ($ev->name=='call.route'
    && isset($ev->params['module']) // dont handle our own messages
    )
    {
      // Check if any new calls should be rejected
      if(isset($this->configs_t["TelsysMaintenance"]) && $this->configs_t["TelsysMaintenance"] == "1" ) {
        Yate::Debug("[APRESA]"
        . "\tTi:".time()
        . "\tFr:".$ev->GetValue('caller')
        . "\tTo:".$ev->GetValue('called')
        . "\tRu:Currently not accepting calls because we are in maintenance mode"
        );   
        $ev->handled = true;
        $ev->retval = "-";

        $code = $this->configs_t["TelsysMaintenanceCode"] ?? "503";
        $msg = $this->configs_t["TelsysMaintenanceMsg"] ?? "Service Unavailable";

        $ev->setParam("error", $code);
        $ev->setParam("reason", $msg);
        $ev->Acknowledge();
        return;
      }

      // SIPREC special
      $sip_require = $ev->GetValue('sip_require');
      $sip_contact = $ev->GetValue('sip_contact');
      if ( (is_string($sip_require) && strpos($sip_require,'siprec') !== false)
      ||   (is_string($sip_contact) && strpos($sip_contact,';+sip.src') !== false)
      ||   (is_string($sip_contact) && strpos($sip_contact,';src') !== false ))
      {
				dbglog('SIPREC call');
				if(!isset($this->configs_t['SipRec']) || $this->configs_t['SipRec'] == '0')
				{
					Yate::Debug("[APRESA-SIPREC]"
					. "\tTi:".time()
					. "\tFr:".$ev->GetValue('caller')
					. "\tTo:".$ev->GetValue('called')
					. "\tRu:SIPREC Disabled, Reject"
					);   
					$ev->handled = true;
	        $ev->retval = "-";
					$ev->setParam("error", "403");
					$ev->setParam("reason", "Forbidden");
	        $ev->Acknowledge();
	        return;
				}
				else 
				{
	        Yate::Debug("[APRESA-SIPREC]"
	          . "\tTi:".time()
	          . "\tFr:".$ev->GetValue('caller')
	          . "\tTo:".$ev->GetValue('called')
	          . "\tRu:SIPREC"
	          );        
	        $ev->handled = true;
	        $ev->retval = "simplyaccept/";
	        $ev->Acknowledge();
	        return;
				}
      }
      
      $directRouting = $this->Process(true,$ev);
      
      if ($directRouting)
      {
        // routed to other module
        $this->Process(false,$ev);
        if ($ev->handled) dbglog("Routed to " . $ev->retval);
        else dbglog("Routing to be determined elsewhere");
      } else
      {
        // otherwise we handle ourselves
        dbglog("Routing to main php");
        $ev->handled = true;
        $ev->retval = "external/nodata/main.php";
      }

    }
    // we must ACK all events of type incoming
    if (!$acked) $ev->Acknowledge();
    $acked = true;
  }
  
	function HandleEvent($ev)
	{
		if ($ev->type=='incoming')
		{
      $this->HandleIncomingEvent($ev);
    }
	}

}

$t = new VidiGlobal();
$t->Main();
dbglog("Goodbye");

?>
