#!/usr/bin/php
<?php
/*
Main entry point for handling

*/

require_once("libyate.php");
require_once("ruler.php");

function dbglog($t)
{
  global $TheVidiMain;
  if (isset($TheVidiMain->instanceId)) $id = $TheVidiMain->instanceId;
  else $id = 'VidiMain';
  $tt = date('D M G:i:s Y') . ' ' . $id . ': ' . $t;
  Yate::Debug( $tt );
}


class VidiMain
{
    // ID of this endpoint instance
    var $instanceId;
    // start time of the call
    var $startTime = null;
    // data of incoming call.execute call
    var $originalData = array();
    // helper obj
    var $ruler = null;
    // loaded configuration from database
    var $configs_t = null;

    function Init()
    {
			global $argc, $argv;
      
			Yate::Init();
			Yate::Debug(true);

      $this->LoadSettings();
      
      $this->ruler = new Ruler();
      $this->ruler->Init(false, $this->configs_t);
      
			$this->instanceId = "VidiMain/".uniqid();
      $this->ruler->instanceId = $this->instanceId;
      
      Yate::SetLocal("id",$this->instanceId);
      // next line is needed when playing back a notification message
      // to the receiver of the call,
      // to detect when the receiver declines the call
			Yate::SetLocal("disconnected","true");
      
			Yate::Install("engine.timer");

			Yate::Install("chan.dtmf",100,"targetid",$this->instanceId);
      Yate::Install("chan.startup",100);
      Yate::Install("chan.notify",100,"targetid",$this->instanceId);
      // filtering seems seems broken for next
      // so not: Yate::Install("chan.disconnected",100,"id",$this->instanceId);
      // but:
      Yate::Install("chan.disconnected");
      // call.answered is now installed on demand
      // it seems we cannot install two handlers for one type of message
      
    }
    
    function LoadSettings()
    {
      dbglog("Load settings");
      $dbintf = new DBInterface(DATA_PATH_ABS);
      $this->configs_t = $dbintf->LoadConfigsPart('t');
      $dbintf = null;
    }
    
    function Run()
    {
        $this->Init();

        // main loop
        while (true)
        {
            $ev = Yate::GetEvent();
            if ($ev === false)
                break;
            if ($ev === true)
                continue;
            if ($ev->name!='engine.timer'
            && $ev->type!='installed'
            && $ev->type!='setlocal'
            )
            {
              
                dbglog( "Event received ".$ev->type.' '.$ev->name);
                //dbglog( var_export($ev,true) );
            }
            $this->HandleEvent($ev);
            
            if ($ev->name=='engine.timer')
            {
                $this->OnNewSecond();
            }
            
        }
    }

    function OnNewSecond()
    {
      if ($this->ruler->ruleHandler!=null)
      {
        $this->ruler->ruleHandler->OnNewSecond();
      }
    }
    
    function HandleIncomingEvent($ev)
    {
      $acked = false;
      if ($ev->name=="call.execute")
      {
        // A new incoming call arriving
        $toCopy = $this->ruler->toCopy;
        foreach ($ev->params as $key=>$value)
        {
          // verbose dbglog("ev params: $key = $value");
          // to copy all params use: $this->originalData[$key] = $value;
        }
        // copy selected parameters of this call
        foreach ($toCopy as $key)
        {
          $value = $ev->GetValue($key);
          $this->originalData[$key] = $value;
          // verbose: dbglog("$key = $value");
        }
        $this->ruler->originalData = $this->originalData;
        
        $ev->params["targetid"] = $this->instanceId;
        $ev->handled = true;
        $ev->Acknowledge();
        $acked = true;
        
        $this->startTime = time();

        $this->ruler->processRules();
        
        $this->ruler->ruleHandler->UnansweredCall();
      }
      if ($ev->name=='chan.startup')
      {
        $newid = $ev->params['id'];
        $oid = $ev->GetValue('originatingId');
        // verbose: dbglog('Startup of channel with ID ' . $newid . ' vid: ' . $oid );
        if ($oid == $this->instanceId)
        {
          dbglog('Startup of channel with ID ' . $newid);
          // dbglog('originated here');
          Yate::Install("call.answered",100,"id",$newid);
          //Yate::Install("chan.disconnected",100,"id",$newid);
          // we would like to re-do call actions earlier,
          // during call setup,
          // but SIP CallID is then not yet available for us
          $this->ruler->RedoCallActions($ev);
          $this->ruler->ruleHandler->ChannelStarted($ev);
        } else
        {
          // dbglog('originated elsewhere');
        }
      }
      if ($ev->name=="chan.notify")
      {
        if ($this->ruler->ruleHandler)
        {
          // reason = eof
          if (isset($ev->params['reason']) && $ev->params['reason']=='eof')
          {
            if ($this->ruler->addOnRuleHandler!=null
            && $this->ruler->addOnRuleHandler->AddOnChannelEof( $ev->params['reason'] ))
            { ; }
            else $this->ruler->ruleHandler->ChannelEof( $ev->params['reason'] );
          }
        }
      }
      if ($ev->name=="chan.dtmf")
      {
        if ($this->ruler->ruleHandler)
        {
          $this->ruler->ruleHandler->Dtmf( $ev->params['text'] );
        }
      }
      if ($ev->name=="call.answered")
      {
        // the add-on ruler decides if the main rule handler
        // must still be called or not
        $doDef = true;
        if ($this->ruler->addOnRuleHandler)
        {
          $doDef = $this->ruler->addOnRuleHandler->CallAnswered( $ev );
        }
        if ($doDef)
        {
          $this->ruler->ruleHandler->CallAnswered( $ev );
        }
      }
      if ($ev->name=="chan.disconnected")
      {
        if ($ev->GetValue('id')==$this->instanceId
        || $ev->GetValue('lastpeerid') == $this->originalData['id']
        )
        {
          $this->ruler->ruleHandler->ChannelDisco( $ev );
        }
      }
      
      // we must ACK all events of type incoming
      if (!$acked) $ev->Acknowledge();
      $acked = true;
    }
    
    function HandleEvent($ev)
    {
      if ($ev->type=="incoming")
      {
          $this->HandleIncomingEvent($ev);
      }
      if ($ev->type=="answer")
      {
        // next block only works when given a routing 'context'
        // that is present in regexroute.conf
        if ($ev->name=="call.route")
        {
          if ($ev->retval=='external/nodata/main.php')
          {
            dbglog("Route to myself?");
            return;
          }
          if ($this->ruler->addOnRuleHandler)
          {
            $this->ruler->addOnRuleHandler->Routed( $ev );
          }
          $this->ruler->ruleHandler->Routed( $ev );
        }
      }
    }

}

$TheVidiMain = new VidiMain();
$TheVidiMain->Run();

dbglog("Goodbye");



?>