#!/usr/bin/php
<?php
// Menu reacting on dial codes
// It can play a message at the same time.
// It redirects to another rule based on the DTMF input,
// or when a timeout is reached, it redirects to the default rule.
//
// Parameters when calling from regexroute.php
// Parameter 1. Dial code mapping, specifying which rule to perform,
//   when detecting a pressed digit.
//   Digit '=' RuleNumber ('.' Digit '=' RuleNumber) *
//   Digit: can be *#0123456789ABCDd
//   d means default, and is used on timeout
//   Example: d=3.*=2.9=4
//   This means, go to rule 2 when detecting *,
//   go to rule 4 when detecting 9,
//   or go to rule 3 when the timeout is reached.
// Parameter 2. Timeout
//   if zero (or less), then there is never a timeout
//   when the timeout is reached, the default action is performed
// Parameter 3. Notification message file

include("libyate.php");

function dbglog($t)
{
    Yate::Debug( date('D M G:i:s Y') . ' VidiMenu: '.$t);
}

class VidiMenu
{
    // ID of this endpoint instance
    var $instanceId;
    // start time of the call
    var $startTime = null;
    // data of incoming call.execute call
    var $originalData = array();
    // dial code mapping (from command line parameter 1)
    var $targets = array();
    // timeout in seconds
    var $timeout = 0;
    // notification message filename
    var $filePlayback = '';

    function Init()
    {
			global $argc, $argv;
			
			Yate::Init();
			Yate::Debug(true);

			$this->forwContext = '';
			if ($argc>=2)
			{
				$parts = explode(',',$argv[1]);
				$targetParts = explode('.',$parts[0]);
        foreach ($targetParts as $targetPart)
        {
          $subparts = explode('=',$targetPart);
          if (count($subparts)!=2) continue;
          $key = $subparts[0];
          $value = $subparts[1];
          $this->targets[ $key ] = $value;
          dbglog("On dial code $key forward to rule $value");
        }
        $this->timeout = (int) $parts[1];
        $this->filePlayback = $parts[2];
			}      
      
			$this->instanceId = "VidiMenu/".uniqid();
			Yate::Install("chan.dtmf",100,"targetid",$this->instanceId);
			Yate::Install("engine.timer");
    }
    
    function Main()
    {
        $this->Init();

        // main loop
        while (true)
        {
            $ev = Yate::GetEvent();
            if ($ev === false)
                break;
            if ($ev === true)
                continue;
            if ($ev->name!='engine.timer')
            {
                dbglog("Event received ".$ev->type.' '.$ev->name);
            }
            $this->HandleEvent($ev);
            
            if ($ev->name=='engine.timer')
            {
                $this->OnNewSecond();
            }
            
        }
    }

    function OnNewSecond()
    {
      if ($this->timeout>0)
      {
        $age = time() - $this->startTime;
        if ($age>=$this->timeout)
        {
          $targetRule = $this->targets['d'];
          if ($targetRule!='')
          {
            dbglog("Timeout : Move to rule " . $targetRule);
            $this->InitiateTransfer('r'.$targetRule);
            //$this->InitiateTransfer('r'.$targetRule);
          } else
          {
            dbglog("Timeout : No action defined");
          }
        }
      }
    }

    function HandleIncomingEvent($ev)
    {
        $acked = false;
        if ($ev->name=="call.execute")
        {
            // A new incoming call arriving
            $toCopy = array('caller','callername','id','called');
            foreach ($ev->params as $key=>$value)
            {
              // verbose dbglog("ev params: $key = $value");
              // to copy all params use: $this->originalData[$key] = $value;
            }
            // copy selected parameters of this call
            foreach ($toCopy as $key)
            {
              $value = $ev->GetValue($key);
              $this->originalData[$key] = $value;
              // verbose: dbglog("$key = $value");
            }
            
            $ev->params["targetid"] = $this->instanceId;
            $ev->handled = true;
            $ev->Acknowledge();
            $acked = true;
            
            $this->startTime = time();
            
            /* Signal we are answering the call */
            $m = new Yate("call.answered");
            $m->params["id"] = $this->instanceId;
            $m->params["targetid"] = $this->originalData['id'];
            $m->Dispatch();
        }
        if ($ev->name=="chan.dtmf")
        {
            $digit = $ev->params['text'];

            // choose next rule depending on DTMF input
            $targetRule = '';
            if (isset($this->targets[$digit]))
            {
              $targetRule = $this->targets[$digit];
            }
            if ($targetRule!='')
            {
              dbglog('DTMF ' . $digit . ' : Move to rule ' . $targetRule);
              $targetContext = 'r' . $targetRule;
              $this->InitiateTransfer($targetContext);
            } else
            {
              dbglog('DTMF ' . $digit . ' : No action defined');
            }
        }

        // we must ACK all events of type incoming
        if (!$acked) $ev->Acknowledge();
        $acked = true;
    }
    
    function InitiateTransfer($targetContext)
    {
        dbglog("Transfer to context $targetContext");
        $m = new Yate("call.route");
        foreach ($this->originalData as $key=>$value)
        {
          $m->params[$key] = $value;
        }
				$m->params["context"] = $targetContext;
        foreach ($m->params as $key=>$value)
        {
          // verbose: dbglog("m params: $key = $value");
        }
        $m->Dispatch();
				return;
    }
    
    function HandleEvent($ev)
    {
      if ($ev->type=="incoming")
      {
          $this->HandleIncomingEvent($ev);
      }
      if ($ev->type=="answer")
      {
        if ($ev->name=="call.route")
        {
          // This is after the transfer is routed.
          dbglog('Routed, now transfer to: '.$ev->retval);

          $m = new Yate("chan.masquerade");
          $m->params['message'] = 'call.execute';
          $m->params['callto'] = $ev->retval;
          foreach ($ev->params as $key=>$value)
          {
            // verbose: dbglog("Param[".$key."]".$value);
            $m->params[$key] = $value;
          }
          $m->Dispatch();

        }
        if ($ev->name=="call.answered")
        {
          if ($this->filePlayback!='')
          {
            dbglog("Playback notification message");
            $m = new Yate("chan.attach");
            $m->params["id"] = $this->instanceId;
            $m->params["notify"] = $this->instanceId;
            $m->params["source"] = "wave/play/".$this->filePlayback;
            $m->params["single"] = true;
            $m->Dispatch();
          }
        }
      }
    }
    
}

$t = new VidiMenu();
$t->Main();

dbglog("Goodbye");



?>