<?php

/*
	Non-blocking AF_UNIX style file socket.
	Accepts only one connection at a time, but can have 10 in back log.
*/
class NonBlockingSocket
{
	// Server socket is used to wait for new connections and accept them
	var $serverSock;
	// Client socket is used once a connection has been made
	var $clientSock;
	var $onData;
	
	function Create($sockFn , $onData)
	{
		$this->serverSock = socket_create(AF_UNIX, SOCK_STREAM, 0);

		@unlink( $sockFn );
		socket_bind( $this->serverSock, $sockFn) or die('could not bind');
		socket_set_nonblock( $this->serverSock );
		socket_listen( $this->serverSock, 10 ); // 10 = backlog

		$this->clientSock = NULL;
		
		$this->onData = $onData;
		
	}

	// This function should be called periodically to handle the socket
	function Perform( & $refAny )
	{
		// can we accept new connections? (for now only one)
		if ($this->clientSock==NULL)
		{
			// is there a new incoming connection?
			$read = array( $this->serverSock );
			$write = NULL;
			$except = NULL;
			$ret = socket_select($read,$write,$except,0,0);
			if ($ret>0)
			{
				$this->clientSock = socket_accept( $this->serverSock );
				if ($this->clientSock===false)
				{
					$this->clientSock = NULL;
				} else
				{
					//lg("New client");
				}
			}
		}
		// is there a client to service?
		if ($this->clientSock!=NULL)
		{
			// can we read from client?
			$read = array( $this->clientSock );
			$except = array( $this->clientSock );
			$ret = socket_select($read,$write,$except,0,0);
			if (count($read)>0)
			{
				//lg("We could read from client");
				$data = socket_read( $this->clientSock , 2*1024 );
				if ($data!==false && $data!='')
				{
					$reply = call_user_func_array( $this->onData , array( $data ) );
					socket_write( $this->clientSock , $reply );
					socket_close( $this->clientSock );
					$this->clientSock = NULL;
					$refAny = true;
				}
			}
			if (count($except)>0)
			{
				//lg('fail');
				socket_close( $this->clientSock );
				$this->clientSock = NULL;
			}
		}
	}
	
	function Close()
	{
		if ($this->clientSock != NULL)
		{
			socket_close( $this->clientSock );
			$this->clientSock = NULL;
		}
		if ($this->serverSock != NULL)
		{
			socket_close( $this->serverSock );
			$this->serverSock = NULL;
		}
	}
	
}

?>