#!/usr/bin/php
<?php // The first line is needed to make executable
include("libyate.php");

function dbglog($t) {
    Yate::Debug('NotifyReceiver: '.$t);
}

class NotifyReceiver{

    function Init()
    {
			global $argc, $argv;
			Yate::Init();
			Yate::Debug(true);

			dbglog('init');

			$this->useOriginalCallerId = true;
			$this->forwContext = '';
			
			if ($argc>=2)
			{
				$parts = explode(',',$argv[1]);
				$this->forwNum = $parts[0];
				$this->forwContext = $parts[1];
				$this->filePlayback = $parts[2];
				dbglog('Audio file: ' . $this->filePlayback);
				dbglog('Forward to context: ' . $this->forwContext);
			}
			
			$this->instanceId = "NotifyReceiver/".uniqid();
			// without next line, problem.
			Yate::Install("engine.timer");
			Yate::Install("chan.notify");
			Yate::Install("call.answered");
			Yate::SetLocal("id",$this->instanceId);
			Yate::SetLocal("disconnected","true");
			Yate::Install("chan.disconnected");
			
			$this->selfDisconnects = 0;
			$this->callingReceiver = false;
			$this->receiverAnswered = false;
		}

	function Main()
	{
		$this->Init();

		// main loop
		while (true)
		{
				$ev = Yate::GetEvent();
				if ($ev === false)
						break;
				if ($ev === true)
						continue;
				if ($ev->name!='engine.timer')
				{
						dbglog("Event received ".$ev->type.' '.$ev->name);
						/*foreach ($ev->params as $key=>$value)
						{
							dbglog("Param[".$key."]".$value);
						}*/
				}
				$this->HandleEvent($ev);
				
				// timing
				// currently, only for debugging
				if ($ev->name=='engine.timer')
				{
						//dbglog('another second');
				}
		}
	}
	
    function HandleIncomingEvent($ev)
    {
        $acked = false;
        if ($ev->name=="call.execute")
        {
            // A new incoming call arriving
            
            $this->caller = $ev->GetValue("caller");
						$this->callername = $ev->GetValue("callername");
            $this->otherSideId = $ev->GetValue("id");
            dbglog("Caller id: ".$this->caller.' name: '.$this->callername);
            
            $ev->params["targetid"] = $this->instanceId;
            $ev->handled = true;
            $ev->Acknowledge();
            $acked = true;
            
            $this->startTime = time();
            
            // answer the call
            $m = new Yate("call.answered");
            $m->params["id"] = $this->instanceId;
            $m->params["targetid"] = $this->otherSideId;
            $m->Dispatch();
        }
				
				if ($ev->name=="chan.notify")
				{
					// playback completed
					dbglog("Playback completed, connecting " . $this->otherSideId . " with " . $this->receiverId);
					$m = new Yate("chan.connect");
					$m->params["id"] = $this->otherSideId;
					$m->params["targetid"] = $this->receiverId;
					$m->Dispatch();
					
					$this->callingReceiver = false;
					//$this->InitiateTransfer($this->forwNum);
				}
				
				if ($ev->name=="call.answered")
				{
					if($ev->GetValue("lastpeerid") == $this->instanceId)
					{
						dbglog("Connected to other, playback notification");
						$this->receiverAnswered = true;
						
						$m = new Yate("chan.attach");
						$m->params["id"] = $this->instanceId;
						$m->params["notify"] = $this->instanceId;
						$m->params["source"] = "wave/play/".$this->filePlayback;
						$m->params["single"] = true;
						$m->Dispatch();
					
						$this->receiverId = $ev->GetValue("id");
					}
				}
				
				if($ev->name=="chan.disconnected")
				{
					if($ev->GetValue('id') == $this->instanceId)
					{
						// Connecting otherSideId to tone/ring causes a disconnect message to ourself
						// So ignore the first one
						$this->selfDisconnects += 1;
						if($this->selfDisconnects == 2 && $this->callingReceiver)
						{
							dbglog("Receiver disconnected");
							$m = new Yate("chan.masquerade");
							$m->params['message'] = 'call.execute';
							$m->params['callto'] = "tone/busy";
							$m->params['id'] = $this->otherSideId;
							$m->Dispatch();
						}
					}
					elseif($ev->GetValue('lastpeerid') == $this->otherSideId && $this->callingReceiver)
					{
						if($this->receiverAnswered)
						{
							dbglog("Caller disconnected while playing message");
							$m = new Yate("chan.masquerade");
							$m->params['message'] = 'call.execute';
							$m->params['callto'] = "tone/busy";
							$m->params['id'] = $this->receiverId;
							$m->Dispatch();
							
							$this->callingReceiver = false;
						}
						else
						{
							dbglog("Caller disconnected while ringing other");
							dbglog("Goodbye");
							exit(0);
						}
					}
				}
				
        // we must ACK all events of type incoming
        if (!$acked) $ev->Acknowledge();
        $acked = true;
    }
    	
    function InitiateTransfer($target)
    {
        dbglog('Transfer to '.$target);
        $m = new Yate("call.route");
        $m->params["id"] = $this->otherSideId;
        
				// user the other as caller might cause
        // an authentication problem,
        // it might be prefered or not,
        // so it must be an option.
				if ($this->useOriginalCallerId)
				{
					$m->params["caller"] = $this->caller;
					$m->params["callername"] = $this->callername;
				}
				
        $m->params["called"] = $target;
				$m->params["context"] = $this->forwContext;
        $m->Dispatch();
				return;
    }
    			
    function HandleEvent($ev)
    {
        if ($ev->type=="incoming")
        {
            $this->HandleIncomingEvent($ev);
        }
				
        if ($ev->type=="answer")
        {
					if ($ev->name=="call.route")
					{
						// This is after the transfer is routed.
						dbglog('Routed, now transfer to: '.$ev->retval);
						if(strpos($ev->retval, 'sip') === 0)
						{
						
							$m = new Yate("chan.masquerade");
							$m->params['message'] = 'call.execute';
							$m->params['callto'] = "tone/ring";
							$m->params['id'] = $this->otherSideId;
							$m->Dispatch();
						
							$m = new Yate("call.execute");
							$m->params['callto'] = $ev->retval;
							foreach ($ev->params as $key=>$value)
							{
								if($key != "id")
								{
									//dbglog("Param[".$key."]".$value);
									$m->params[$key] = $value;
								}
							}
							$m->Dispatch();
							$this->callingReceiver = true;
						}
						else
						{
							dbglog("Not routed to a sip channel, simply connect instead");
							$m = new Yate("chan.masquerade");
							$m->params['message'] = 'call.execute';
							$m->params['callto'] = $ev->retval;
							foreach ($ev->params as $key=>$value)
							{
								//dbglog("Param[".$key."]".$value);
								$m->params[$key] = $value;
							}
							
							$m->Dispatch(); 
						}
					}

					if ($ev->name=="call.answered")
					{
						$this->InitiateTransfer($this->forwNum);
					}
        } // type=="answer"
				
				
    } // function HandleEvent
		
} // class Notify

$t = new NotifyReceiver();
$t->Main();
dbglog("Goodbye");

?>