<?php

require_once("act_reject.php");
require_once("act_accept.php");
require_once("act_menu.php");
require_once("act_forward.php");
require_once("act_notify.php");
require_once("act_rec.php");

require_once("/var/apresa/conf/voip_config.php");
define('DATA_PATH_ABS','/var/apresa/data');
require_once("/var/www/db.php");
require_once("/var/www/utility.php");

class RuleHandlerBase
{
  function RoutingCall($ev) { return false; }
  function UnansweredCall() {}
  function ActiveCall() {}
  function Dtmf($digit) {}
  function OnNewSecond() {}
  function ChannelEof() {}
  function IsAddOn() { return false; }
  function IsCallAction() { return true; }
  function Routed($ev) {}
  function CallAnswered($ev) {}
  function ChannelDisco($ev) {}
  function ChannelStarted($ev) {}
}

class Ruler
{
    // ID of this endpoint instance (COPIED)
    var $instanceId;  
    // data of incoming call.execute call (COPIED)
    var $originalData = array();
    // loaded configuration from database
    var $configs_t = null;
    // current rule number to execute
    var $ruleNr = 1;
    // configuration for current rule
    var $ruleCfg = null;
    // pointer to object who will apply the rule
    var $ruleHandler = null;
    // additional rule applied together (notification message)
    var $addOnRuleHandler = null;
    // should match voippbx.php
    var $maxRules = 15;
    // true when calling the receiver of the call, false after receiver has answered
    var $callingReceiver = false;
    // fields to copy
    var $toCopy = array('caller','callername','id','called'
      ,'sip_history-info', 'sip_p-asserted-identity', 'sip_callid', 'ip_transport' );
    // if virt (virtual) then nothing that has outside effect is performed
    var $virt = false;
    // list of call actions that might have to be redone,
    // when another call leg is created
    var $callActions = array();

    function Init($virt, $configs_t)
    {
      $this->virt = $virt;
      $this->configs_t = $configs_t;
    }
    
    function GetThisRuleCfg($what)
    {
      return $this->GetRuleCfg($this->ruleNr,$what);
    }
    
    function GetCfg($what)
    {
      $param = 'Telsys' . $what;
      if (isset($this->configs_t[$param])==false) return '';
      return $this->configs_t[$param];
    }
    
    function GetRuleCfg($ruleNr, $what)
    {
      return $this->GetCfg('R' . $ruleNr . $what);
    }
    
    function GetRuleCfgs($ruleNr)
    {
      $arr = array();
      foreach ($this->configs_t as $param=>$value)
      {
        $prefix = 'TelsysR' . $ruleNr;
        if (startsWith($param,$prefix))
        {
          $postfix = substr($param,strlen($prefix));
          if ($postfix=='') continue;
          // For Rule 1, skip Rule 13 etc.
          if ( ctype_digit($postfix[0]) ) continue;
          //dbglog( $postfix );
          $arr[ $postfix ] = $value;
        }
      }
      return $arr;
    }
    
    // ruleNr : next possible rule is
    // returns : next rule for which to execute the action
    function GetNextToAct($ruleNr)
    {
      for (;$ruleNr<=$this->maxRules;$ruleNr++)
      {
        $cnd = $this->GetRuleCfg($ruleNr,'Cnd');
        if ($cnd=='dstNum')
        {
          // lets check the destination phone number
          $numlist = $this->GetRuleCfg($ruleNr,'NumDst');
          $num = $this->originalData['called'];
          if ($this->CheckInNumList( $numlist , $num ))
          {
            return $ruleNr;
          }
        } else if ($cnd=='srcNum')
        {
          $numlist = $this->GetRuleCfg($ruleNr,'NumSrc');
          $num = $this->originalData['caller'];
          if ($this->CheckInNumList( $numlist , $num ))
          {
            return $ruleNr;
          }
        } else if ($cnd=='dirIn')
        {
          // for InnovaPhone, check History-Info field
          if ( array_key_exists( 'sip_history-info' , $this->originalData )
            && strpos($this->originalData['sip_history-info'],'3D486')!==false )
          {
            // for InnovaPhone, we read the direction form the SIP History-Info field
            // it contains 3D486 for incoming
            // and 3D480 for outgoing
            return $ruleNr;
          }
        } else return $ruleNr;
      }
      return $ruleNr;
    }
    
    function ProcessRules()
    {
      $this->ruleNr = $this->GetNextToAct($this->ruleNr);
      $this->ruleCfg = $this->GetRuleCfgs($this->ruleNr);
      $act = $this->GetThisRuleCfg('Act');
      dbglog( 'Rule ' . $this->ruleNr . ' Act: ' . $act . ($this->virt?' (virt)':'') );
      if ($this->virt==false)
      {
        Yate::Debug("[APRESA]"
          . "\tTi:".time()
          . "\tFr:".$this->originalData['caller']
          . "\tTo:".$this->originalData['called']
          . "\tRu:".$this->ruleNr
          );
      }
      $this->ruleHandler = null;
      if ($act=='reject') $this->ruleHandler = new ActRejectCall();
      if ($act=='accept') $this->ruleHandler = new ActAcceptCall();
      if ($act=='menu'  ) $this->ruleHandler = new ActMenu();
      if ($act=='forward')$this->ruleHandler = new ActForward();
      if ($act=='notify') $this->ruleHandler = new ActNotify();
      if ($act=='keep'
      || $act=='delete'
      || $act=='startrec'
      )
      {
        $this->ruleHandler = new ActRec($act);
      }
      if ($this->ruleHandler == null)
      {
        $this->ruleHandler = new ActRejectCall();
      }
      $this->ruleHandler->Init($this);
      if ($this->ruleHandler->IsAddOn())
      {
        // it is an add-on rule that must be executed together with the next
        dbglog("add-on rule together with next");
        $this->addOnRuleHandler = $this->ruleHandler;
        $this->addOnRuleNr = $this->ruleNr;
        $this->ruleNr ++;
        $this->ProcessRules();
      }
      if ($this->ruleHandler->IsCallAction()==false)
      {
        if ($this->virt==false)
        {
          $this->ruleHandler->Perform();
          $this->callActions[] = $this->ruleHandler;
        }
        $this->ruleNr ++;
        $this->ProcessRules();
      }
    }

    function RedoCallActions($ev)
    {
      foreach ($this->callActions as $handler)
      {
        $handler->Redo($ev);
      }
    }
    
    function ToRule( $ruleNr )
    {
      $this->ruleNr = $ruleNr;
      $this->ProcessRules();
      $this->ruleHandler->ActiveCall();
    }
    
    function MatchesPattern($pattern,$numToFind)
    {
      // input quoting
      $pattern = str_replace('+','\\+',$pattern);
      $pattern = str_replace('.','\\.',$pattern);
      // replacement
      $pattern = str_replace('?','.',$pattern);
      $pattern = str_replace('*','.*',$pattern);
      $pattern = '/^' . $pattern . '$/';
      if (preg_match($pattern,$numToFind)>0)
      {
        dbglog($numToFind . ' matches ' . $pattern );
        return true;
      }
      return false;
    }
        
    function CheckInNumList($numlist,$numToFind)
    {
      $numbers = explode(',',$numlist);
      foreach ($numbers as $num)
      {
        if ($this->MatchesPattern($num,$numToFind)) return true;
      }
      dbglog( 'no match for ' . $numToFind );
      return false;
    }
        
}


?>