#!/bin/bash
# Node handling for high availibility cluster

function lg(){
	echo "$1"
	logger -t apresa_cmd "$1"
}

function adaptPgsqlConfig()
{
	lg "Begin of adaptPgsqlConfig"
	pgetc=/etc/postgresql/9.4/main/postgresql.conf
	echo "listen_addresses = '*'" >> $pgetc
	echo "wal_level = hot_standby" >> $pgetc
	echo "synchronous_commit = on" >> $pgetc
	echo "archive_mode = on" >> $pgetc
	echo "archive_command = 'cp %p /database/db/pg_archive/%f'" >> $pgetc
	echo "max_wal_senders=5" >> $pgetc
	echo "wal_keep_segments = 32" >> $pgetc
	echo "hot_standby = on" >> $pgetc
	echo "restart_after_crash = off" >> $pgetc
	echo "wal_receiver_status_interval = 2   # seconds" >> $pgetc
	echo "max_standby_streaming_delay = -1" >> $pgetc
	echo "max_standby_archive_delay = -1" >> $pgetc
	echo "synchronous_commit = on" >> $pgetc
	echo "restart_after_crash = off" >> $pgetc
	echo "hot_standby_feedback = on" >> $pgetc
	lg "End of adaptPgsqlConfig"
}

function createDrbdDev(){
	#recmnt=/dev/sda5
	lg "Adjust global conf"
	sed -i -e 's/usage-count.*/usage-count no;/' /etc/drbd.d/global_common.conf
	lg "Load DRBD driver"
	modprobe drbd
	lg "Create DRBD resource"
	echo yes | drbdadm create-md r0
	lg "Start DRBD"
	service drbd start
	lg "Set as DRBD primary"
	drbdadm -- --overwrite-data-of-peer primary r0
	lg "End of DRBD primary initialisation"
}

function initDrbdSecondary(){
	lg "Adjust global conf"
	sed -i -e 's/usage-count.*/usage-count no;/' /etc/drbd.d/global_common.conf
	lg "Load DRBD driver"
	modprobe drbd
	lg "Create DRBD resource"
	echo yes | drbdadm create-md r0
	lg "Start DRBD"
	service drbd start
	lg "Connect DRBD"
	drbdadm connect r0
	lg "End of DRBD secondary initialisation"
}

function initRecordings(){
	lg "Remove apresa_first mounting"
	systemctl disable apresa_first
	umount /var/upload
	umount /var/upload
	umount /var/upload
	umount /var/upload
	lg "Create ext4 partition"
	mkfs -t ext4 /dev/drbd1
	lg "Mount it"
	mount /dev/drbd1 /recordings
	lg "Create log subdir"
	mkdir /recordings/log
	chmod 777 /recordings/log
	lg "End of recordings initialisation"
}

function initServices(){
	service corosync restart
	# from now on, start automatically
	#update-rc.d corosync defaults

	#service pacemaker restart
	# configuration is done in php
	# from now on, start automatically
	#update-rc.d pacemaker defaults
	
	# do not auto-start
	# it will be managed by pacemaker
	systemctl disable postgresql
}

function initServicesSecondary(){
	# from now on, start automatically
	#update-rc.d corosync defaults
	#update-rc.d pacemaker defaults

	service corosync restart
	# but do not yet start pacemaker,
	# because it would cause failure initialy
	#service pacemaker stop

	# do not auto-start
	# it will be managed bij pacemaker
	systemctl disable postgresql
}

function node_initboth(){
	touch /var/apresa/conf/opt_cluster
	mkdir /database/db/pg_archive
	chown postgres:postgres /database/db/pg_archive/
	adaptPgsqlConfig
}

function node_init1(){
	lg "node $1"
	# notify user and block web interf.
	echo "node-init1" > /var/apresa/data/busy
	# general init for both nodes
	node_initboth

	createDrbdDev
	initRecordings
	initServices
	
	# further php initialisation
	cd /var/apresa/cmd;/usr/bin/php ./clusternode.php $1
	# give some extra time to really start
	sleep 20
	# signal to user that finished
	rm /var/apresa/data/busy
}

function node_init2(){
	lg "node $1"
	# notify user and block web interf.
	echo "node-init2" > /var/apresa/data/busy
	# general init for both nodes
	node_initboth
	
	initDrbdSecondary
	initServicesSecondary
	
	# further php initialisation
	cd /var/apresa/cmd;/usr/bin/php ./clusternode.php $1
	# give some extra time to really start
	sleep 20
	# signal to user that finished
	rm /var/apresa/data/busy
}

function node_param(){
	lg "node $1"
	cd /var/apresa/cmd;/usr/bin/php ./clusternode.php $1
}

case $1 in
	start)
		node_param $1
		;;
	stop)
		node_param $1
		;;
	init1)
		node_init1 $1
		;;
	init2)
		node_init2 $1
		;;
	*)
		echo "Not a supported parameter"
		;;
esac


