#!/bin/bash
# This script is run during abnormal exit of recording engine

autodir=/recordings/log/auto

# Parameters:
# 1. The configuration param
# 2. The default value if empty
function GetIntCfg()
{
  local r=`psql apresa -t -P pager -c "select value from configs where param='$1'"`
  if [ -z "$r" ]; then
    # use default
    return $2
  fi
  return $r
}

# Parameters:
# 1. The configuration param
# 2. The default value if empty
function GetBoolCfg()
{
  local r=`psql apresa -t -P pager -c "select value from configs where param='$1'"`
  if [ -z "$r" ]; then
    # use default
    if [ "$2" -eq "1" ]; then
      return 1
    else
      return 0
    fi
    return 0
  fi
  if [ "$r" -eq "1" ]; then
    return 1
  fi
  # in all other cases (empty or zero)
  return 0
}

# Check if this is enabled

GetBoolCfg "CollectLogs" "0"
if [ "$?" -eq "0" ]
then
  echo "Collecting logs is disabled"
  exit 0
fi

echo "Collecting logs automatically"

GetIntCfg "CollectLogsKeep" "5"
keep=$?
echo "Keep $keep tgz"

# Move to directory where auto logs are stored
cd $autodir

# Find the existing tgz (that we might need to delete afterwards)
items=( $(find "apresa_logs_auto_"* ) )
      
# Rotate, we keep the configured items max
for ((n=1;n<keep;n++))
do
  # we start at the end at keep-1, and will rename keep-1 to keep (4 to 5)
  i=$(( keep - n ))
  # normally only one item with number i, but just in case, we make array
  itemsOfI=( $(find "apresa_logs_auto_$i""_"* 2>/dev/null ) )
  # of none found then skip
  if [ "${#itemsOfI[@]}" = "0" ]; then
    continue
  fi
  # take the first one found
  item="${itemsOfI[0]}"
  # check if not empty
  if [ -z "$item" ]; then
    continue
  fi
  # construct the new name to rename to
  # we replace the number with one higher
  # this also works with keep larger than 10
  # because take into account the length of i
  fromix=$(( ${#i} + 18))
  newname="${item:0:17}$((i+1))_${item:$fromix}"
  # perform the rename
  echo "Rename ${item} to ${newname}"
  mv "${item}" "${newname}"
  # dont delete renamed items
  # this is very rare (two logs in the same second)
  # the ! seems to cause ix to be the index instead of the value
  for ix in "${!items[@]}"
  do
    if [ "$newname" = "${items[ix]}" ]
    then
      unset 'items[ix]'
    fi
    # also do not delete the original (in case mv failed)
    if [ "$item" = "${items[ix]}" ]
    then
      unset 'items[ix]'
    fi
  done
done

# delete every other auto log item in the directory
for item in "${items[@]}"
do
  fullfn="$autodir/$item"
  if [ -f "$fullfn" ]; then
    echo "Remove $fullfn"
    rm $item
  fi
done

# pack normal logs
echo "Packing logs"
/usr/share/apresa/packlog.sh 0 1>/dev/null 2>/dev/null

# determine file name
stamp=$(date +%Y%m%d_%H%M%S)
fn="apresa_logs_auto_1_$stamp.tgz"
echo "Storing logs as $fn"

# store with this filename
mkdir -p /recordings/log/auto
chmod 777 /recordings/log/auto
mv /recordings/log/aprlogs.tgz /recordings/log/auto/$fn

