#!/bin/bash

# If any of these have been disabled, do no generate any alarms
if [[ $1 = "apresanet" && ! -f /var/apresa/conf/apresanet ]]; then
  exit 0
fi

if [[ $1 = "apresaoig" && ! -f /var/apresa/conf/MitelOIG ]]; then
  exit 0
fi

if [[ $1 = "apresaav32" && ! -f /var/apresa/conf/AvayaCSTA ]]; then
  exit 0
fi

if [[ $1 = "apresaavnet" && ! -f /var/apresa/conf/AvayaDMCC ]]; then
  exit 0
fi

if [[ $1 = "apresacj" && ! -f /var/apresa/conf/CJtapi ]]; then
  exit 0
fi

if [[ $1 = "ed137" && ! -f /var/apresa/conf/ed137 ]]; then
  exit 0
fi

if [ -z "$SERVICE_RESULT" ]; then
        # Debian 8 (systemd 219) does not pass the exit info via the enviroment variab$
        # So try to retrieve them ourselves

        ExecMainStatus=$(systemctl show --property ExecMainStatus $1)
        ExecMainCode=$(systemctl show --property ExecMainCode $1)

        #logger -t apresa_cmd "$1 stop check, retrieved status $ExecMainStatus $ExecMainCode"

        ExecMainStatus=${ExecMainStatus:15}
        ExecMainCode=${ExecMainCode:13}

        if [ "$ExecMainCode" = "1" ]; then
                if [ "$ExecMainStatus" = "0" ]; then
                        SERVICE_RESULT="success"
                else
                        SERVICE_RESULT="exit-code"
                fi
                EXIT_CODE="exited"
                EXIT_STATUS=$ExecMainStatus
        elif [ "$ExecMainCode" = "2" ]; then
                SERVICE_RESULT="signal"
                EXIT_CODE="killed"
                EXIT_STATUS=`kill -l $ExecMainStatus`
        else
                #Still running ?
                exit 0
        fi
fi

logger -t apresa_cmd "$1 stop check, SERVICE_RESULT=$SERVICE_RESULT EXIT_CODE=$EXIT_CODE EXIT_STATUS=$EXIT_STATUS"
DESCRIPTION=`/usr/share/apresa/get_service_description.sh $1`

if [ "$SERVICE_RESULT" = "success" ]; then
        #logger -t apresa_cmd "$1 has exitted succesfully, start a timer to generate an alarm if it does not startup again later"
        systemctl start apresacheck@$1.timer
else
        #logger -t apresa_cmd "$1 has not exitted successfully, create alarm"
        if [[ ("$EXIT_CODE" = "killed") || ("$EXIT_CODE" = "dumped") ]]; then
                # Exitted via a signal, dumped also means that systemd has collected a$
                # but this requires a specific package to be installed
                echo "$DESCRIPTION exited because of signal $EXIT_STATUS" > /var/apresa/data/err_proc_$1.dat
        elif [ "$EXIT_CODE" = "exited" ]; then
                echo "$DESCRIPTION exited with code $EXIT_STATUS" > /var/apresa/data/err_proc_$1.dat
        else 
                echo "$DESCRIPTION exited" > /var/apresa/data/err_proc_$1.dat
        fi

        if [[ ($1 == "orkaudio") || ($1 == "crec") ]]; then
          echo 'Collect logs'
          /usr/share/apresa/collect_logs.sh
        fi

        /usr/share/apresa/check_health.sh
fi