#!/bin/bash

on_term()
{
	echo "service_up terminated..."
	logger -t apresa_cmd "service_up terminated..."
	exit 0
}

trap 'on_term' TERM

# too much: logger -t apresa_cmd "service_up script"

function StartService {
	echo "Check if $3 is running"
	if pgrep "$1" > /dev/null
	then
		# Already running
		echo "It is running"
	else
		# This is an error, it is not running
		echo "It is NOT running"
		afail=1
		# store in log file
		logger -t apresa_cmd "$3 is inactive, try to restart it"
		# store in health state file
		echo "$3 is inactive" > /var/apresa/data/err_proc.dat
		# inform customer
		# not immediately: /usr/share/apresa/check_health.sh
		# restart service, it might help
		if [ "$1" = "orkaudio" ];
		then
			# notify, orkaudio is going to (re)start
			/usr/share/apresa/apresa_startup.sh -v
		fi
		if [ "$1" = "apresa_crec" ];
		then
			# notify, crec is going to (re)start
			/usr/share/apresa/apresa_startup.sh -c
		fi
		echo "Try restarting $3"
		service $2 restart
		sleep 1
		if pgrep "$1" > /dev/null
    then
			logger -t apresa_cmd "$3 has become active again"
		else
			logger -t apresa_cmd "$3 is inactive, even after restart"
			echo "$3 is inactive, even after restart"
			# it has stayed in active, so it is really bad
			allok=0
			if [ "$1" = "orkaudio" ];
			then
				FailVoipStopTelsys=`sudo -u "www-data" psql apresa -t -P pager -c "select value from configs where param='FailVoipStopTelsys'"`
				FailVoipStopTelsys=`echo $FailVoipStopTelsys | tr -d ' '`
				if [ "$FailVoipStopTelsys" = "1" ];
				then
					echo "Stop telsys"
					service yate stop
					service eyate stop
				fi
			fi
		fi
	fi
}


if [ -f /var/apresa/data/shutting_down ]
then
	logger -t apresa_cmd "Shutdown is active"
	exit
fi


if [ -f /var/apresa/data/busy ]
then
	# update or other intensive process is busy
	logger -t apresa_cmd "A system action is busy: `cat /var/apresa/data/busy`"
	echo "A system action is busy: `cat /var/apresa/data/busy`"
	busycnt=0
	. /var/apresa/data/busycnt
	let "busycnt = $busycnt + 1"
	# echo "Busy cnt:" $busycnt
	echo "busycnt=$busycnt" > /var/apresa/data/busycnt
	if [ $busycnt -lt 15 ] ; then
		logger -t apresa_cmd "Busy (cnt: $busycnt), not checking."
	  echo "Busy (cnt: $busycnt), not checking."
		exit
	fi
fi

rm -f /var/apresa/data/busycnt

afail=0
allok=1


# Recording card service
cardlist=`sudo -u "www-data" psql apresa -t -P pager -c "select value from configs where param='CardList'"`
cardlist=`echo $cardlist | tr -d ' '`
echo "CardList: '$cardlist'"
if [ -n "$cardlist" ]
then
	if [ -f /var/apresa/conf/crec_disabled ]; then
		echo "Card recording service is disabled (for tracing?)"
		logger -t apresa_cmd "Card recording service is disabled (for tracing?)"
		echo "Card recording service is disabled (for tracing?)" > /var/apresa/data/err_proc.dat
		allok=0
		afail=1
	else
		StartService apresa_crec apresacrec "Card recording service"
	fi
fi

StartService orkaudio orkaudio "VoIP recording service"
StartService apresa_remote apresaremote "Apresa Remote Transfer"
StartService apresa_net apresanet "Apresa NET service"
StartService apresa_handler apresa_handler "Command handler"
StartService apresa_fifo apresafifo "Apresa FIFO service"

# echo $allok $afail
if [ $allok == 1 ]
then
	echo "All processes are running"
	rm -f /var/apresa/data/err_proc.dat
	if [ $afail == 1 ]
	then
	  echo "One or more processes have become active again."
		# inform customer
		# not immediately: /usr/share/apresa/check_health.sh
	fi
fi

