<?php

function AranaImport()
{
  global $dbintf;
  lg("Arana import start");
  $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
  try
  {
    $importer = new AranaImporter();
    $importer->Run();
  } catch(Exception $e)
  {
    lg("Arana import. Exception occured: ".$e);
    addToSayUniq(MSG_ERR,"Error occured in Arana import: " . $e->getMessage());
  }
  $dbintf->SetConfigsPart( array('AranaImporting'=>0) , 'io' );
  lg("Arana import end");
}

class AranaImporter
{
  function __construct()
  {
  }
  
  function OnDbErr()
  {
    $einfo = $this->db->errorInfo();
    $errmsg = "DB error: " . $einfo[2];
    //lg( $errmsg );
    throw new Exception( $errmsg );
  }

  // NOTE: This function is not so reliable (see below)
  function ConvertVariantDateToTimestamp($variantDate)
  {
    // variant_date_to_timestamp not supported
    // Variant is the number of days since midnight Dec 30 1899
    $variantDate = $variantDate;
    // lg( $vardt );
    // convert to Jan 1 1970 offset (for Unix timestamp)
    // and convert to seconds
    $timestampDate = (int) ( ($variantDate - 25569) * 24 * 60 * 60 );
    // determine timezone at that time
    // convert timestamp to DateTime object (GMT timezone)
    $dateObj = new DateTime("@$timestampDate");
    // create DateTime object with Apresa current timezone
    $dateTmp = new DateTime();
    // get offset of current timezone at the moment of the datetime
    // however we can only do that accurate when we know the offset
    // so we do it two times
    $offset = $dateTmp->getTimezone()->getOffset($dateObj);
    $timeTmp = $timestampDate - $offset;
    $dateObj = new DateTime("@$timeTmp");
    $offset = $dateTmp->getTimezone()->getOffset($dateObj);
    // apply offset
    $timestampDate -= $offset;
    //lg( date('D M G:i:s Y',$timestampDate) );
    return $timestampDate;
  }
  
  function Decompact($ch)
  {
    if ($ch>='0' && $ch<='9')
    {
      return ord($ch) - ord('0');
    }
    if ($ch>='A' && $ch<='Z')
    {
      return ord($ch)-ord('A')+10;
    }
    return -1;
  }
  
  function ConvertFilenameToTimestamp($filename)
  {
    $Year = 2000 + $this->Decompact($filename[2]);
    $Month = $this->Decompact($filename[3]);
    $Day = $this->Decompact($filename[4]);
    $Hour = $this->Decompact($filename[5]);
    $Minute = $this->Decompact($filename[6])*10+$this->Decompact($filename[7]);
    $Second = $this->Decompact($filename[8])*2;
    $ts = mktime($Hour,$Minute,$Second,$Month,$Day,$Year);
    return $ts;
  }

  function Run()
  {
    global $dbintf, $lng;
    // Load configuration
    LoadBasicSettings();

    $options = array();

    $cfgPrefix = 'Arana';
    $cfgPart = 'io';
    $cfgPartLoaded = $dbintf->LoadConfigsPart($cfgPart);
      
    $options['Mounting'] = true;
    $options['Unmount'] = false;
    $options['ImportPath'] = '/mnt/drive_Arana';
    
    $BckTargetType = $cfgPartLoaded[ $cfgPrefix . 'BckTargetType' ];
    if ($BckTargetType==1)
    {
      // backup to internal or USB disk
      $options['ImportPath'] = $cfgPartLoaded[$cfgPrefix . 'BckPath'];
      $options['Mounting'] = false;
      // check if a mount point
      $mp = $options['ImportPath'];
      exec('mountpoint '.escapeshellarg($mp),$dummy,$retval);
      if ($retval != 0)
      {
        addToSayUniq(MSG_ERR,"Arana import: $mp is not a mount point.");
        return;
      }
    } else if ($BckTargetType==2)
    {
      // backup to any directory
      $options['ImportPath'] = $cfgPartLoaded[$cfgPrefix . 'BckPath'];
      $options['Mounting'] = false;
    } else if ($BckTargetType==3)
    {
      $subdir = $cfgPartLoaded[$cfgPrefix . 'BckSubdir'];
      if ($subdir!='')
      {
        $options['ImportPath'] .= '/' . $subdir;
      }
    }
    

    if ($options['Mounting'])
    {
      $this->signal_required('mnt Arana io');
      $options['Unmount'] = true;
    }
    
    try
    {
      $this->MainRun($options);
    } catch(Exception $e)
    {
      lg("Arana import. Exception occured: ".$e);
      addToSayUniq(MSG_ERR,"Error occured in Arana import: " . $e->getMessage());
    }

    if ($options['Unmount'])
    {
      $this->signal_required('umnt Arana');
    }
  }

  function signal_required($msg)
  {
    $reply = 1;
    $cmd_timeout = 20;
    $err = signal_service($msg,$reply,$cmd_timeout);
    if ($err!='')
    {
      $this->onerror('Socket error: '.$err);
    }
    $msg_firstpart = explode(' ',$msg);
    $msg_firstpart = $msg_firstpart[0];

    $where = $msg_firstpart;
    if ($where=='mnt')
    {
      $where = 'mounting directory';
    }
    // more user friendly
    if ($reply=='OK')
    {
      // OK
    } else if ($reply[0]=='E')
    {
      $this->onerror($where . ' failed: '.$reply);
    } else
    {
      $this->onerror($where . ' failed, unexpected reply: '.$reply.' LEN: '.strlen($reply));
    }
  }  
  
  function onerror($msg)
  {
    throw new Exception($msg);
  }
 
  function MainRun($options)
  {
    global $dbintf, $lng;
    
    $options['ALE'] = false;
    if ($dbintf->GetConfigValue('ALE')==1)
    {
      $options['ALE'] = true;
      lg( 'ALE is on');
    }
    
    //$options['ImportPath']
    $dbfn = $options['ImportPath'] . '/' . 'callinfo.db';
    if (file_exists($dbfn)==false)
    {
      onerror('Cannot find callinfo.db in the import directory');
    }
    $this->db = new PDO('sqlite:'.$dbfn);

    $total = 0;
    $sql = 'SELECT count(id) FROM Archive ORDER BY id DESC';
    $stmt = $this->db->prepare($sql);
    if ($stmt===false)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $row = $stmt->fetch();
    if ($row)
    {
      $total = $row[0];
    }
    lg('Total: ' . $total);
    
    $sql = 'SELECT * FROM Archive ORDER BY startdate ASC';
    $stmt = $this->db->prepare($sql);
    if ($stmt===false)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $counting = 0;
    $cancelled = false;
    while ($row = $stmt->fetch())
    {
      $this->ImportCall( $options, $row );
      $counting ++;
      if ( ($counting % 100) == 0)
      {
        if ($dbintf->GetConfigValue('AranaStop')==1)
        {
          $cancelled = true;
          lg('Cancelled');
          break;
        }
        if ( ($counting % 1000) == 0)
        {
          lg( 'Arana import progress ' . $counting . ' / ' . $total );
        }
        $dbintf->SetConfigsPart( array('AranaProgress' => $lng->Get('Main.NumRecords')
         . ' ' . $counting . ' / ' . $total) , 'io' );
      }
    }
    
    $this->db = null;
    
    $msg = "Arana import finished";
    if ($cancelled) $msg = "Arana import cancelled";
    if ($options['SrcFileMissing']>0) $msg .= '. Missing files at the source: ' . $options['SrcFileMissing'];
    if ($options['FilePresent']>0)    $msg .= '. Duplicate recordings (not copied): ' . $options['FilePresent'];
    if ($options['FileCopyErr']>0)    $msg .= '. Failed to copy recordings: ' . $options['FileCopyErr'];
    if ($options['FileCopied']>0)     $msg .= '. Recordings copied: ' . $options['FileCopied'];
    if ($options['DbEntryPresent']>0) $msg .= '. Duplicate entries (not added): ' . $options['DbEntryPresent'];
    if ($options['DbEntryAdded']>0)   $msg .= '. Call detail records added: ' . $options['DbEntryAdded'];
    if ($options['NotCall']>0)        $msg .= '. Not a call recording: ' . $options['NotCall'];
    addToSay(MSG_INF,$msg);
  }
  
  function ImportCall( & $options, $row )
  {
    global $dbintf;
    // too verbose: lg("Import " . $row['filename']);
    
    if ($row['doctype']!=0)
    {
      $options['NotCall'] ++;
      // it is a fax
      // too verbose: lg("Not doctype call recording. Skip.");
      return;
    }
    
    $data = array();
    $data['localfull'] = $row['local'];
    $data['local'] = $data['localfull'];
    $data['remotefull'] = $row['remote'];
    $data['remote'] = $data['remotefull'];
    $data['connected'] = $row['connected'];
    
    $data['local2'] = '';
    $data['remote2'] = '';
    $data['direction'] = $row['direction'];
    $data['notes'] = $row['notes'];
    $data['notes2'] = '';
    $data['category'] = 0;
    
    $aranaUtf8 = "Ara\xC3\xB1a"; // UTF-8
    $data['recorderid'] = $dbintf->GetOrCreateRecorderId("Arana",$aranaUtf8,48);

    // OLD: $startdate = $this->ConvertVariantDateToTimestamp( $row['startdate'] );
    $startdate = $this->ConvertFilenameToTimestamp( $row['filename'] );
    $data['startdate'] = $startdate;

    $data['duration'] = $row['length'];
    $flags = 0;
    if ($options['ALE']) $flags |= 16;
    $data['flags'] = $flags;
    
    $archivestate = (int) $row['archivestate'];
    $data['isarchived'] = ($archivestate==1)?1:0;
    $data['callextra'] = '';
    
    $fn = $row['filename'];
    // remove & disk seek part
    if (substr($fn,64,1)=='&')
    {
      $fn = substr($fn,0,64) . '____' . substr($fn,68);
    }
    // if no extension, then add .wav
    if (strrchr($fn,'.')===false)
    {
      $fn .= '.wav';
    }
    $data['filename'] = $fn;
    
    $data['directory'] = date('Y/m/d/H',$startdate);
    $data['remoteid'] = substr($data['filename'],0,20);

    $data['waittime'] = $row['waittime'];

    $data['source'] = 0; // unknown
    
    $encoding = 51; // unknown
    if (strlen($row['filename'])>16)
    {
      $encoding = 500 + ord($row['filename'][15]);
    }
    $data['encoding'] = $encoding;
    $data['encryption'] = 0; // no (not used for ALE)
    $data['processingstate'] = 0; // audio processing
    $data['lineid'] = 0; // not applicable

    $local_is_remote = false;
    $remote_is_local = false;
    $connected_is_local = ($data['direction']==1);
    
    $FullSipId = true;
    $cid = GetSipIdToUse($data['localfull'],$FullSipId);
    $ret = $dbintf->GetOrCreateCallerID($cid,$data['local2' ],$local_is_remote?0:1);
    $data['localcallerid'] = $ret["id"];
    $data['localcontact'] = $ret["contactid"];
    $cid = GetSipIdToUse($data['remotefull'],$FullSipId);
    $ret = $dbintf->GetOrCreateCallerID($cid,$data['remote2'],$remote_is_local?1:0);
    $data['remotecallerid'] = $ret["id"];
    $data['remotecontact'] = $ret["contactid"];
    $cid = GetSipIdToUse($data['connected'],$FullSipId);
    $ret = $dbintf->GetOrCreateCallerID($cid,'',$connected_is_local?1:0);
    $data['connectedcallerid'] = $ret["id"];
    $data['connectedcontact'] = $ret["contactid"];
    
    $tenantid = CalcTenant($data);
    $data['tenantid'] = $tenantid;
    
    //var_dump( $data );
    
    $subpath = $row['subpath'];
    if ($subpath===NULL)
    {
      // some older Arana version didnt have subpath filled in
      $subpath = idate('Y',$startdate) . '\\' . idate('m',$startdate) . '\\' . idate('d',$startdate) . '\\';
    }
    
    $srcPath = $options['ImportPath'] . '/' . str_replace( '\\', '/' , $subpath);
    if ($srcPath[ strlen($srcPath)-1 ]!='/')
    {
      $srcPath .= '/';
    }
    $srcFn = $srcPath . $row['filename'];
    $extraSize = 0;
    
    if ($data['isarchived']==1)
    {
      $dstDir = '/recordings/' . $data['directory'];
      $dstFn = $dstDir . '/' . $data['filename'];
      if (file_exists($srcFn)==false)
      {
        lg( 'source file missing ' . $srcFn );
        $data['isarchived'] = 0;
        $options['SrcFileMissing'] ++;
      } else if (file_exists( $dstFn ))
      {
        // too verbose: lg( 'Destination file already exists' );
        $options['FilePresent'] ++;
      } else 
      {
        @mkdir( $dstDir , 0777 , true );
        $result = false;
        if ($options['ALE'])
        {
						$cmd = 'apresa_crypt -e -i ' . escapeshellarg($srcFn)
								 . ' -o ' . escapeshellarg($dstFn) . ' ';
						$encrypt_result = 0;
						$encrypt_output = array();
						exec( $cmd , $encrypt_output , $encrypt_result);
						if ($encrypt_result == 0)
						{
							$result = true;
						}
        } else
        {
          $result = copy( $srcFn , $dstFn );
        }
        if ($result==false)
        {
          lg( 'failed to copy to ' . $dstFn );
          $data['isarchived'] = 0;
          $options['FileCopyErr'] ++;
        } else
        {
          $extraSize = filesize($dstFn);
          // too verbose: lg( 'copied to ' . $dstFn );
          chmod( $dstFn , 0777 );
          $options['FileCopied'] ++;
        }
      }
    }
    
    $extraRecords = 0;
    if ($dbintf->GetCallInfoRM($data['remoteid'])!==false)
    {
      // too verbose: lg( 'database entry already exists' );
      $options['DbEntryPresent'] ++;
    } else
    {
      $extraRecords = 1;
      $dbintf->InsertCall($data);
      $options['DbEntryAdded'] ++;
    }
    
    if ($tenantid!=NULL)
    {
      if ($extraSize>0 || $extraRecords>0)
      {
				UpdateTnDiskUsage($tenantid, $extraSize, $extraRecords);
      }
    }
    
    
  }
  
}










?>