<?php
// Configuration of Asterisk

function Aste($param)
{
	global $dbintf;
	$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
  
  if ($param=='apply')
  {
    ApplyAsteCfg();
  }
  if ($param=='restart')
  {
    RestartAste();
  }
  
}

function IsIP4($id)
{
  return preg_match("/^([0-9]{1,3}\.){3}[0-9]{1,3}$/",$id)>0;
}

class AsteCfg
{
  function __construct()
  {
    // file content arrays
    // - rtp contains STUN configuration
    // - extensions contains dial plan
    // - pjsip contains SIP registrations etc.
    $this->rtp_fc = array();
    $this->extensions_fc = array();
    $this->pjsip_fc = array();
    $this->sip_fc = array();
    // do we use chan_sip or pjsip
    $this->pjsip = false;
  }

  function TrunkPjsip($pre,$name)
  {
    $Server = $this->cfg[$pre . 'Server'];
    $Id  = $this->cfg[$pre . 'Id'];
    $Pwd = decrypt( $this->cfg[$pre . 'Pwd'] );
    if (str_ensure_defined($Server)=='') return;
    
    // we have a server
    $serverUri = "sip:$Server";
    $clientUri = "sip:$Id@$Server";
    $contactUser = "$Id";
    
    // configure registration
    $this->pjsip_fc[] = "[$name-reg]";
    $this->pjsip_fc[] = "type=registration";
    $this->pjsip_fc[] = "outbound_auth=$name";
    $this->pjsip_fc[] = "endpoint=$name";
    $this->pjsip_fc[] = "line=yes";
    $this->pjsip_fc[] = "transport=aptransport";
    $this->pjsip_fc[] = "server_uri=$serverUri";
    $this->pjsip_fc[] = "client_uri=$clientUri";
    $this->pjsip_fc[] = "contact_user=$contactUser";
    $this->pjsip_fc[] = "";
    // configure endpoint
    $this->pjsip_fc[] = "[$name](apcodecs)";
    $this->pjsip_fc[] = "type=endpoint";
    $this->pjsip_fc[] = "context=from-$name"; // points to dial plan
    $this->pjsip_fc[] = "aors=$name";
    $this->pjsip_fc[] = "outbound_auth=$name";
    $this->pjsip_fc[] = "";
    // configure authentication
    $this->pjsip_fc[] = "[$name]";
    $this->pjsip_fc[] = "type=auth";
    $this->pjsip_fc[] = "auth_type=userpass";
    $this->pjsip_fc[] = "username=$Id";
    $this->pjsip_fc[] = "password=$Pwd";
    $this->pjsip_fc[] = "";
    // configure AOR = address of record
    $this->pjsip_fc[] = "[$name]";
    $this->pjsip_fc[] = "type=aor";
    $this->pjsip_fc[] = "contact=$serverUri";
    $this->pjsip_fc[] = "";
  }
  
  function Phone($pre, $name)
  {
    $Id  = $this->cfg[$pre . 'Id'];
    $Pwd = decrypt( $this->cfg[$pre . 'Pwd'] );
    if (str_ensure_defined($Id)=='') return;
    
    // we have to use the ID as section name,
    // otherwise SIP register get this error:
    // No matching endpoint found
    $name = $Id;

    // configure endpoint
    $this->pjsip_fc[] = "[$name](apcodecs)";
    $this->pjsip_fc[] = "type=endpoint";
    $this->pjsip_fc[] = "transport=aptransport";
    $this->pjsip_fc[] = "context=from-phones"; // points to dial plan
    $this->pjsip_fc[] = "aors=$name";
    $this->pjsip_fc[] = "auth=$name";
    $this->pjsip_fc[] = "";
    // configure authentication
    $this->pjsip_fc[] = "[$name]";
    $this->pjsip_fc[] = "type=auth";
    $this->pjsip_fc[] = "auth_type=userpass";
    $this->pjsip_fc[] = "username=$Id";
    $this->pjsip_fc[] = "password=$Pwd";
    $this->pjsip_fc[] = "";
    // configure AOR = address of record
    $this->pjsip_fc[] = "[$name]";
    $this->pjsip_fc[] = "type=aor";
    $this->pjsip_fc[] = "max_contacts=1";
    $this->pjsip_fc[] = "remove_existing=yes";
    $this->pjsip_fc[] = "";
    
  }
  
  function Prepare()
  {
    $list = array('pjsip','extensions','rtp','sip');
    
    foreach ($list as $item)
    {
      if (file_exists( "/etc/asterisk/$item-apresa.conf" )) continue;
      file_put_contents( "/etc/asterisk/$item.conf"
      , "#include \"$item-apresa.conf\"\n" );
      file_put_contents( "/etc/asterisk/$item-apresa.conf" , "" );
    }
    
  }
  
  function DialPlan()
  {
    $this->extensions_fc[] = '; This file is generated by Apresa, do not edit';
    $this->extensions_fc[] = '';
    
    $PhoneTelNum = $this->cfg[$this->prefix . 'InReg1_Id'];
    
    if ($this->pjsip) $chan = 'pjsip';
    else $chan = 'sip';
    
    $MainAct = $this->cfg[$this->prefix . 'MainAct'];
    if ($MainAct=='proxy')
    {
      $this->extensions_fc[] = '[from-trunk1]';
      $this->extensions_fc[] = "exten => _X.,1,Dial($chan/$PhoneTelNum)";
      $this->extensions_fc[] = '';
      $this->extensions_fc[] = '[from-phones]';
      $this->extensions_fc[] = "exten => _X.,1,Dial($chan/\${FILTER(0-9,\${EXTEN})}@trunk1)";
      $this->extensions_fc[] = '';
    } else if ($MainAct=='accept')
    {
      $this->extensions_fc[] = '[from-trunk1]';
      $this->extensions_fc[] = 'exten => _X.,1,NoOp()';
      $this->extensions_fc[] = 'same => n,Answer()';
      $this->extensions_fc[] = 'same => n,While(true)';
      // tones? $this->extensions_fc[] = 'same => n,  Playtones(info)';
      $this->extensions_fc[] = 'same => n,  Wait(4)';
      $this->extensions_fc[] = 'same => n,EndWhile()';

      $this->extensions_fc[] = '';
      $this->extensions_fc[] = '[from-phones]';
      $this->extensions_fc[] = 'exten => _X.,1,NoOp()';
      $this->extensions_fc[] = 'same => n,Answer()';
      $this->extensions_fc[] = 'same => n,While(true)';
      // tones? $this->extensions_fc[] = 'same => n,  Playtones(info)';
      $this->extensions_fc[] = 'same => n,  Wait(4)';
      $this->extensions_fc[] = 'same => n,EndWhile()';
      $this->extensions_fc[] = '';
    } else { // reject
      $this->extensions_fc[] = '[from-trunk1]';
      $this->extensions_fc[] = '';
      $this->extensions_fc[] = '[from-phones]';
      $this->extensions_fc[] = '';
    }
    
  }
  
  function NetworkSip()
  {
    // network configuration
    $this->sip_fc[] = 'localnet=10.0.0.0/8,172.16.0.0/12,192.168.0.0/16';
    $PublicIp = $this->cfg[$this->prefix . 'PublicIp'];
    if ($PublicIp!='')
    {
      if (IsIP4($PublicIp))
      {
        $this->sip_fc[] = 'externaddr=' . $PublicIp;
      } else
      {
        // it is a host name, not an IP
        $this->sip_fc[] = 'externhost=' . $PublicIp;
        $this->sip_fc[] = 'externrefresh=60';
      }
    }
  }
  
  function Rtp()
  {
    $this->rtp_fc[] = '; This file is generated by Apresa, do not edit';
    $this->rtp_fc[] = '';
    $this->rtp_fc[] = "[general]";
    $StunIp = $this->cfg[$this->prefix . 'StunIp'];
    if ($StunIp!='')
    {
      $this->rtp_fc[] = "icesupport=yes";
      $this->rtp_fc[] = "stunaddr=$StunIp";
    }
  }
  
  function MainSip()
  {
    $this->sip_fc[] = '; This file is generated by Apresa, do not edit';
    $this->sip_fc[] = '';
    $this->sip_fc[] = '[general]';
    $this->NetworkSip();
    $outCount = 1;
    for ($ix=1; $ix <= $outCount; $ix++)
    {
      $pre = $this->prefix . 'OutReg' . $ix . '_';
      $Server = $this->cfg[$pre . 'Server'];
      $Id  = $this->cfg[$pre . 'Id'];
      $Pwd = decrypt( $this->cfg[$pre . 'Pwd'] );
      if ($Server=='') continue;
      if ($Id=='') continue;
      $this->sip_fc[] = 'register => ' . $Id . ':' . $Pwd . '@' . $Server . '/' . $Id;
    }
    $this->sip_fc[] = "icesupport=yes";
    $this->sip_fc[] = "srvlookup=yes"; // DNS SRV lookup
    $this->sip_fc[] = "keepalive=60"; // guessing this is for tcp only
    $this->sip_fc[] = '';
    $this->sip_fc[] = "[apcodecs](!)";
    $this->sip_fc[] = "disallow=all";
    $this->sip_fc[] = "allow=ulaw,alaw";
    $this->sip_fc[] = '';
    
    for ($ix=1; $ix <= $outCount; $ix++)
    {
      $pre = $this->prefix . 'OutReg' . $ix . '_';
      $Server = $this->cfg[$pre . 'Server'];
      $Id  = $this->cfg[$pre . 'Id'];
      $Pwd = decrypt( $this->cfg[$pre . 'Pwd'] );
      if ($Server=='') continue;
      if ($Id=='') continue;
      $name = 'trunk' . $ix;
      $this->sip_fc[] = "[$name](apcodecs)";
      $this->sip_fc[] = "type=peer";
      $this->sip_fc[] = "host=$Server";
      $this->sip_fc[] = "fromdomain=$Server";
      $this->sip_fc[] = "defaultuser=$Id";
      $this->sip_fc[] = "fromuser=$Id";
      $this->sip_fc[] = "remotesecret=$Pwd";
      $this->sip_fc[] = "context=from-$name";
      $this->sip_fc[] = "insecure=port,invite";
      $this->sip_fc[] = "directmedia=no"; // otherwise no rec
      $this->sip_fc[] = "qualify=yes"; // helps with NAT issues
      $this->sip_fc[] = "qualifyfreq=60"; // maybe should be a setting
      $this->sip_fc[] = "";
    }
    
    $inCount = 1;
    for ($ix=1; $ix <= $inCount; $ix++)
    {
      $pre = $this->prefix . 'InReg' . $ix . '_';
      $Id  = $this->cfg[$pre . 'Id'];
      $Pwd = decrypt( $this->cfg[$pre . 'Pwd'] );
      if ($Id=='') continue;
      $this->sip_fc[] = "[$Id](apcodecs)";
      $this->sip_fc[] = "type=friend";
      $this->sip_fc[] = "secret=$Pwd";
      $this->sip_fc[] = "host=dynamic";
      $this->sip_fc[] = "context=from-phones";
      $this->sip_fc[] = "";
    }    
  }
  
  function MainPjsip()
  {
    $this->pjsip_fc[] = '; This file is generated by Apresa, do not edit';
    $this->pjsip_fc[] = '';
    
    $this->pjsip_fc[] = "[aptransport]";
    $this->pjsip_fc[] = "type=transport";
    $this->pjsip_fc[] = "protocol=udp";
    $this->pjsip_fc[] = "bind=0.0.0.0";
    $this->pjsip_fc[] = "";
    
    $this->pjsip_fc[] = "[apcodecs]";
    $this->pjsip_fc[] = "disallow=all";
    $this->pjsip_fc[] = "allow=ulaw,alaw";
    $this->pjsip_fc[] = "";
    
    $outCount = 1;
    for ($ix=1; $ix <= $outCount; $ix++)
    {
      $pre = $this->prefix . 'OutReg' . $ix . '_';
      $this->Trunk($pre, 'trunk' . $ix );
    }
    
    $inCount = 1;
    for ($ix=1; $ix <= $inCount; $ix++)
    {
      $pre = $this->prefix . 'InReg' . $ix . '_';
      $this->Phone($pre, 'phone' . $ix );
    }
  }
  
  function Perform()
  {
    global $dbintf;

    $prefix = 'Aste_';
    $this->prefix = $prefix;
    
    $this->cfg = $dbintf->LoadConfigsPart('aste');
    $defaultCfgs = array
    ( 'MainAct' => 'reject'
    );
    foreach ($defaultCfgs as $key=>$value)
    {
      $cfgKey = $prefix . $key;
      if (isset($this->cfg[$cfgKey])==false)
      {
        $this->cfg[$cfgKey] = $value;
      }
    }    
    // debug: var_dump($this->cfg);
    
    $this->Prepare();
    
    $this->Rtp();
    if ($this->pjsip)
    {
      $this->MainPjsip();
    } else
    {
      $this->MainSip();
    }
    $this->DialPlan();
    
  }

  function ArrayWrite($arr, $fn)
  {
    $h = fopen($fn,"w");
    if ($h===false)
    {
      lg("Failed to open $fn");
      return;
    }
    foreach ($arr as $line)
    {
      fwrite($h,"$line\n");
    }
    fclose($h);
  }
  
  function Write()
  {
    $this->ArrayWrite( $this->pjsip_fc,      "/etc/asterisk/pjsip-apresa.conf"      );
    $this->ArrayWrite( $this->extensions_fc, "/etc/asterisk/extensions-apresa.conf" );
    $this->ArrayWrite( $this->rtp_fc,        "/etc/asterisk/rtp-apresa.conf"        );
    $this->ArrayWrite( $this->sip_fc,        "/etc/asterisk/sip-apresa.conf"        );
    
    // these extra things in the dial plan
    // not sure what they do
    // remove them
    $clearList = array
      ( 'extensions.ael'
      , 'extensions.lua'
      );
    foreach ($clearList as $item)
    {
      unlink("/etc/asterisk/$item");
    }    
  }
  
  function Apply()
  {
    system('rasterisk -x "pjsip reload"');
    system('rasterisk -x "dialplan reload"');
  }
  
  function DebugOutput()
  {
    echo "pjsip conf:\n";
    foreach ($this->pjsip_fc as $line)
    {
      echo $line . "\n";
    }
  }
  
}

function RestartAste()
{
  global $dbintf;
  lg('Restart Asterisk');
  system('service asterisk restart');
}

function ApplyAsteCfg()
{
  global $dbintf;
  lg('Apply configuration to Asterisk');
  
  $obj = new AsteCfg();
  $obj->Perform();
  $obj->Write();
  // $obj->DebugOutput();
  $obj->Apply();
  
}


