<?php
/*
  To be used in multi-tenant setup,
		by user apresapush.
		
	methods:
	- upload : called by sender
	- scan : called by receiver
*/
define('CMD',1);
define('CMD_REMOTE',1);

require_once __DIR__ . "/" . "../conf/voip_config.php";
require_once __DIR__ . "/" . CMD_WEB_RELPATH . "db.php";
require_once __DIR__ . "/" . CMD_WEB_RELPATH . "signaling.php";
require_once __DIR__ . "/" . CMD_WEB_RELPATH . "datafile.php";
require_once __DIR__ . "/" . CMD_WEB_RELPATH . "utility.php";
require_once __DIR__ . "/" . CMD_WEB_RELPATH . "tenancy.php";

function GetSerialNumber()
{
	return trim(file_get_contents(CMD_WEB_RELPATH . '../apresa/conf/serial'));
}

// TransferTarget . status
define('STATUS_DONE',-3);
define('STATUS_LASTFULLDONE',-2);
define('STATUS_LASTFULL',-1);
define('STATUS_OK',0);
//define('STATUS_ERR',1);

$globReturnCode = 0;

if (!defined('NO_GO'))
{
  Go();

  exit( $globReturnCode );
}

function vlg($t)
{
  // verbose logging
  if (defined('VERBOSE_LOGING'))
  {
    lg($t);
  }
  return;
}

function Go()
{
	global $dbintf, $argc, $argv, $recid_map;

	// Interface to database
	$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
	// no longer: $dbintf->SetAdmin();

	// Parse options
	$options = array();
	for ($i=1;$i<$argc;$i++)
	{
		if ($argv[$i]=='--debug')
		{
			define('DEBUG_LOGING',1);
		}
		if ($argv[$i]=='--scan'
		|| $argv[$i]=='--upload'
		)
		{
			$options['act'] = substr($argv[$i],2);
		}
		if ( ( $argv[$i]=='--sn'
		|| $argv[$i]=='--accid'
		|| $argv[$i]=='--target'
		|| $argv[$i]=='--id'
		) && $i+1<$argv)
		{
			$options[substr($argv[$i],2)] = $argv[$i+1];
		}
	}

	$retobj = array();
	$retobj['errcode'] = 0;
	if ($options['act']=='scan')
	{
		HandleScan( $options , $retobj );
	} else if ($options['act']=='upload')
	{
		HandleUpload( $options , $retobj );
	} else
	{
		err( $retobj, 1, 'Unsupported action');
	}
	$json = json_encode( $retobj );
	lg($options['act'].': '.$json);
	//echo $json . "\n";
}

function err( & $ref_retobj, $code, $msg )
{
	$ref_retobj['errcode'] = $code;
	$ref_retobj['errmsg'] = $msg;
	return false;
}

function SnAppend( $text , $sn )
{
	$pos = strpos($text, '.');
	if ($pos===false)
	{
		$pos = strlen($text);
	}
	$before = substr($text,0,$pos);
	$toadd = '_SN' . $sn;
	// already has this postfix?
	if (substr($before,-strlen($toadd))==$toadd) return $text;
	return $before . $toadd . substr($text,$pos);
}


// = = = = = = = = = = = = = = = = = = = = = = = =

class Uploader
{
	var $workdir;
	var $options;
	var $ref_retobj;
	var $accid_filter;
	var $push; // the JSON object to upload
	var $notFoundCnt;
	var $TriedId;
  var $target;
  var $mySN;
  var $this_is_all;
  

	function Init($param_options, & $param_ref_retobj)
	{
		global $dbintf;
		$this->options = $param_options;
		$this->ref_retobj = & $param_ref_retobj;
		$this->notFoundCnt = 0;
		
		if (isset($this->options['id'])==false)
		{
			return err($this->ref_retobj,3,'No target ID specified');
		}
		$targetid = $this->options['id'];
		
		$this->target = $dbintf->GetTransferTarget( $targetid );
		if ($this->target === FALSE)
		{
			return err($this->ref_retobj,3,'Target does not exist');
		}
		
		if ( $this->target['enabled'] == 0 )
		{
			return err($this->ref_retobj,3,'Target is disabled');
		}
		if ( $this->target['ip'] == '' )
		{
			return err($this->ref_retobj,3,'Target not fully configured');
		}
		
		//var_dump( $this->target );
		
		$this->mySN = GetSerialNumber();
		
		// LastId: ID of last uploaded and received recording
		// TriedId: ID of last uploaded recording (maybe not processed yet)
		
		if ($this->target['status']=='')
		{
			$this->target['status'] = STATUS_OK;
		}

		return true;
	}

	function lg($t)
	{
		if (isset($this->target) && isset($this->target['id']))
		{
			$t = 't'.$this->target['id'] . '. ' . $t;
		} else
		{
			$t = 'upload: '.$t;
		}
		lg($t);
	}

	var $decrypt_todel = array();
	// returns a new filename with decrypted content,
	// or an empty string if decryption failed
	function Decrypt($oldfn, & $ref_errcode)
	{
		//$this->lg('Decrypt '.$oldfn);
		$parts = pathinfo($oldfn);
		$memdir = '/dev/shm/apresa/'.substr($parts['dirname'],12);
		@mkdir($memdir,0777,TRUE);
		//$this->lg('memdir: '.$memdir);
		$memfn = tempnam($memdir,'exp');
		//$this->lg('memfn: '.$memfn);
		$this->decrypt_todel[] = $memfn;
		
		$cmdToExec = 'apresa_crypt -d -i '.escapeshellarg($oldfn).' -o '.escapeshellarg($memfn);
		$retValue = 999;
		$dummyOutput = array();
		//$this->lg('Decrypt cmd: '.$cmdToExec);
		exec($cmdToExec,$dummyOutput,$retValue);
		$ref_errcode = $retValue;
		if ($retValue!=0)
		{
			$this->lg('Decryption error '.$retValue);
			return '';
		}
		return $memfn;
	}
	function CleanupDecrypt()
	{
		foreach ($this->decrypt_todel as $fn)
		{
			DeleteFn($fn);
		}
	}

	function GetSftpCmdLineOptions()
	{
		$portOpt = '';
    $ip = $this->target['ip'];
    $pos = strpos($ip,':');
    if ($pos !== false)
    {
      $port = substr($ip,$pos+1);
      $ip = substr($ip,0,$pos);
      $portOpt = "-P " . $port;
    }
    return ' -i /var/apresa/data/connector'
			// automatically accept all hosts
			. ' -o StrictHostKeyChecking=no ' 
			// but do not write it to a file
			. ' -o UserKnownHostsFile=/dev/null ' 
      . ' ' . $portOpt
			. ' ' . escapeshellarg('apresapush@'.$ip)
			;
	}
	
	function filterErrMsgs($arr)
	{
		$ret = array();
		foreach ($arr as $line)
		{
			if (strpos($line,'Warning: Permanently added')!==false
			&& strpos($line,'to the list of known hosts')!==false)
			{
				continue;
			}
			$ret[] = $line;
		}
		return implode('',$ret);
	}
	
	function GetRemoteStatus()
	{
		$workdir = $this->workdir;
		$ref_retobj = & $this->ref_retobj;
		$mySN = $this->mySN;
		
		// is checking of remote status necessary?
		if ($this->target['triedid']==$this->target['lastid'])
		{
			// there are not unprocessed uploads
			// are there new to upload
			$calls = $this->GetToUpload( 1 );
			if (count($calls)==0)
			{
				lg("Not needed to check remote status");
				return;
			}
		}
		
		//echo 'Local working dir: ' . $workdir . "\n";
		
		@unlink( "$workdir/push.flag" );
		@unlink( "$workdir/proc.json" );
		
		$batchCmds = '';
		// move to directory where we should upload
		// actually this is in /var/upload/data (chroot jail)
		$batchCmds .= "cd /data\n";
		// create a directory if not exists
		// the - prefix means, do not stop if failed
		$batchCmds .= "-mkdir SN$mySN\n";
		// move into the directory for our SN
		$batchCmds .= "cd SN$mySN\n";
		// get instructions from other side
		$batchCmds .= "-get proc.json $workdir/proc.json\n";
		// see if our previous upload has been processed
		$batchCmds .= "-get push.flag $workdir/push.flag\n";
		
		// Create a batch file to be used with SFTP
		$batchfn = tempnam('/tmp', "apresa_upl_");
		$errfn = tempnam('/tmp', "apresa_errupl_");
		file_put_contents( $batchfn, $batchCmds );
		// create command to execute
		$cmd = 'sftp -b '.$batchfn
			.$this->GetSftpCmdLineOptions()
			.' 2>'.$errfn;
		//echo "CMD: $cmd\n";
		$this->lg('SFTP to '.$this->target['ip'] . ' , get status');
		// execute the SFTP batch file
		//system( $cmd );
		$sftp_status = '';
		$sftp_output = array();
		$last = exec( $cmd , $sftp_output , $sftp_status );
		//system( $cmd );
		// remove the SFTP batch file
		unlink( $batchfn );
		$errmsgarr = file($errfn);
		$errmsg = $this->filterErrMsgs( $errmsgarr );
		unlink( $errfn );

		//echo "Status: $sftp_status $last\n";
		
		if ($sftp_status!=0)
		{
			return err($this->ref_retobj,4,'SFTP error code '.$sftp_status.'. '.$errmsg);
		} else
		{
			$this->lg("SFTP OK");
		}
		
		// debug:
		//echo "Working dir\n";
		//system( "ls -l $workdir" );

		if ( file_exists("$workdir/push.flag") == true )
		{
			// Previous upload not processed (flag file still there)
			return err($ref_retobj,5,'Previous upload not processed');
		}
		
		if ( $this->target['triedid']!=0
		&& file_exists("$workdir/proc.json") == true )
		{
			// process this file
			//echo "PROC\n";
			$this->lg("Reading message file of receiver");
			$proc = json_decode( file_get_contents("$workdir/proc.json") , true );
			//var_dump( $proc );
			
			if (isset($proc['processed']))
			{
				foreach ($proc['processed'] as $id)
				{
					$id = (int) $id;
					if ($id > $this->target['lastid'])
					{
						$this->target['lastid'] = $id;
					}
				}
				if ($this->target['lastid'] == $this->target['triedid']
				&& $this->target['status'] == STATUS_LASTFULL)
				{
					$this->target['status'] = STATUS_LASTFULLDONE;
				}
			}
			//echo "LastId: " . $this->target['lastid'] . "\n";
			$this->SetProperty('lastid',$this->target['lastid']);
		}	
		
		return true;
	}

	function GetToUpload( $max )
	{
		global $dbintf;
		
		// maybe use backlog
		// but at the moment it seems unnecessary complicated

		$listconfig = array();
		$listconfig['allrecords'] = false;
		$listconfig['loadrange'] = $max;
		$listconfig['startfrom'] = 0;
		$listconfig['accidfilter'] = $this->target['accid'];
		$listconfig['afterid'] = $this->target['lastid'];
    if ($this->target['minage']>0)
    {
      // we dont filter on database level,
      // because that could lead to skipping of records,
      // after updating the lastid
      // dont: $listconfig['minage'] = $this->target['minage'] * 60;
    }
		//$listconfig['callback'] = 'CallbackCall';
		$calls = $dbintf->GetCallsForExport($listconfig);
		unset($calls['cnt']);
		return $calls;
	}
	
	function DetermineUpload()
	{
		global $dbintf;
		//echo "UPLOAD\n";
		
		// Determine what to upload by
		// [x] what the remote side has said what has been received (lastid)
		// [x] local reset time frame (lastid)
		// [x] filtering to accid
		// [x] block limit size

		$byte_limit = 100 * 1024*1024;
		//$byte_limit = 100 * 1024;
		
		$block_limit = 500;
		//$block_limit = 50;
		//$block_limit = 2;
		
		$byte_count = 0;
		
		$calls = $this->GetToUpload( $block_limit+1 );
		$cnt = count( $calls );
		$this->lg("Call export count: $cnt (unfiltered)");

		// Load list of recorders
		$recorderMap = array();
		$recorders = $dbintf->GetRecorders();
		foreach ($recorders as $recorder)
		{
			$recorderMap[ $recorder['id'] ] = $recorder;
		}
		
		// Contact list
		// We do not load it completely, because it is too large
		$contactlist = array();
		$cidlist = array();
		$linelist = array();
    $tenantlist = array();
		
		// pointers to other tables, they are useless for the other side
		$to_unset = array
		( 'recorderid'
		, 'localcallerid'
		, 'remotecallerid'
		, 'connectedcallerid'
		, 'localcontact'
		, 'remotecontact'
		, 'connectedcontact'
		, 'tenantid'
		, 'agent'
		);
		
		// we need to create a push.json
		// based on this call listing
		// there might also be other things that must in it (SN etc.)
		$this->push = array();
    $this->push['SN'] = $this->mySN;
    $this->push['pubkey'] = trim( file_get_contents('/var/apresa/data/connector.pub') );
		$this->push['data'] = array();
		$cnt = 0;
		$this->this_is_all = true;
    $now = time();
		foreach ($calls as $call)
		{
      if ($this->target['minage']>0)
      {
        $finishTime = $call['startdate'] + $call['duration'];
        $age = $now - $finishTime;
        $minage = $this->target['minage'] * 60;
        if ($age<$minage)
        {
          $this->lg('too early for call finished at: ' . date('Y-m-d H:i:s',$finishTime) . ' age: ' . $age . ' s minage.:' . $minage . ' s');
          break;
        }
      }
      
      $cnt ++;
			if ($cnt > $block_limit)
			{
				$this->this_is_all = false;
				break;
			}
			$call['recorder_sn']   = $recorderMap[ $call['recorderid'] ]['recid'];
			$call['recorder_name'] = $recorderMap[ $call['recorderid'] ]['name'];
			$call['recorder_type'] = $recorderMap[ $call['recorderid'] ]['rectype'];
			$call['local_cid']       = $this->GetCallerID( $cidlist, $call['localcallerid'] );
			$call['remote_cid']      = $this->GetCallerID( $cidlist, $call['remotecallerid'] );
			$call['connected_cid']   = $this->GetCallerID( $cidlist, $call['connectedcallerid'] );
			$call['local_name']      = $this->GetContactName( $contactlist, $call['localcontact'] );
			$call['remote_name']     = $this->GetContactName( $contactlist, $call['remotecontact'] );
			$call['connected_name']  = $this->GetContactName( $contactlist, $call['connectedcontact'] );
			$call['line_name']       = $this->GetLineName( $linelist, $call['lineid'] );
      $call['tenant_name']     = $this->GetTenantName( $tenantlist, $call['tenantid'] );
			foreach ($to_unset as $key)
			{
				unset( $call[$key] );
			}

			// check existence of file(s)
			$call['files'] = array();
			$local_path = AUDIO_PATH . '/' . $call['directory'] . '/' . $call['filename'];
			if ($this->AddFileTransfer( $call, $call['filename'] )==false)
			{
				$call['isarchived'] = 0;
			}
			// screen recording
			if (HasScreenRecording($call))
			{
				$this->AddFileTransfer( $call, GetScreenFileName($call) );
			}
			
			// do preprocessing on files
			$giveup = array();
			foreach ($call['files'] as $filekey=>$filetrans)
			{
				$onlyfn = $filetrans['fn'];
				$srcdir = AUDIO_PATH . '/' . $call['directory'];
				$local_path = $srcdir . '/' . $onlyfn;
				$thisOneEncrypted = ($call['flags']&16)!=0;
				if ($thisOneEncrypted)
				{
					$errcode = 0;
					$local_path = $this->Decrypt( $local_path , $errcode );
					if ($local_path=='')
					{
						if ($errcode==41)
						{
							// this decryption error should be solved by the user
							return err($this->ref_retobj,7,'Decryption failed, first unlock playback');
						} else
						{
							// skip this file
							$giveup[] = $filekey;
							continue;
						}
					}
					$fz = filesize($local_path);
					// update filesize (it is a little smaller)
					$call['files'][$filekey]['sz'] = filesize($local_path);
					$call['files'][$filekey]['replacefn'] = $local_path;
					// remove encryption flag
					$call['flags'] ^= 16;
				}
				$byte_count += $call['files'][$filekey]['sz'];
			}
			foreach ($giveup as $filekey)
			{
				unset( $call['files'][$filekey] );
			}
			//$this->lg('Bytes to transfer: ' . round($byte_count/1024) . ' KB');
			// first add, THEN check limit
			// otherwise, if one file is too big, nothing gets uploaded anymore!
			$this->push['data'][] = $call;
			if ($byte_count > $byte_limit)
			{
				$this->lg('Reached byte limit');
				break;
			}
		}
		
		// debug
		//file_put_contents('/home/lucas/apresa/tmp/a.json', json_encode($this->push['data']) );
		//echo "END\n";
		//exit;
		
    $this->lg("Call export count: " . count($this->push['data']) . ' (filtered)' );
		$this->lg('Recording bytes to transfer: ' . round($byte_count/1024) . ' KB');
	}

	function AddFileTransfer( & $ref_call , $fn )
	{
		$local_path = AUDIO_PATH . '/' . $ref_call['directory'] . '/' . $fn;
		if (file_exists($local_path)==false)
		{
			$this->notFoundCnt ++;
			// only log the first three
			if ($this->notFoundCnt <= 3)
			{
				$this->lg('Not found ' . $fn);
			}
			return false;
		}
		$ref_call['files'][] = array
		(	'fn' => $fn
		,	'sz' => filesize($local_path)
		);
		return true;
	}			
	
  function SftpBatchLineQuote($t)
  {
    $t = str_replace( '\\' , '\\\\' , $t );
    $t = str_replace( '#' , '\\#' , $t );
    return $t;
  }
  
	function PerformUpload()
	{
		$workdir = $this->workdir;
		$ref_retobj = & $this->ref_retobj;
		$mySN = $this->mySN;
		
		if (count( $this->push['data'] )==0)
		{
			// nothing to upload (ready)
			$this->target['status'] = STATUS_DONE;
			return;
		}

		$batchCmds = '';
		// move to directory where we should upload
		$batchCmds .= "cd /data\n";
		// create a directory if not exists
		// the - prefix means, do not stop if failed
		$batchCmds .= "-mkdir SN$mySN\n";
		// move into the directory for our SN
		$batchCmds .= "cd SN$mySN\n";
		// upload our call list
		$batchCmds .= "put $workdir/push.json\n";
		// remove remote json if exists
		$batchCmds .= "-rm proc.json\n";
		// upload recordings in separate directory
		$batchCmds .= "-mkdir recs\n";
		$batchCmds .= "cd recs\n";
		$recordings_upload = 0;
		$triedid = 0;
		foreach ($this->push['data'] as $callkey=>$call)
		{
			$triedid = (int) $call['id'];
			foreach ($call['files'] as $filekey=>$filetrans)
			{
				$onlyfn = $filetrans['fn'];
				//echo $local_fn . "\n";
				$srcdir = AUDIO_PATH . '/' . $call['directory'];
				$local_path = $srcdir . '/' . $onlyfn;
				// handle encryption
				if (isset($filetrans['replacefn']))
				{
					$local_path = $filetrans['replacefn'];
					// unset (don't transfer)
					unset( $this->push['data'][$callkey]['files'][$filekey]['replacefn'] );
				}
				$batchCmds .= "put " . $this->SftpBatchLineQuote($local_path)
          . " " . $this->SftpBatchLineQuote($onlyfn) . "\n";
				$recordings_upload++;
				$batchCmds .= "chmod 200 " . $this->SftpBatchLineQuote($onlyfn) . "\n";
			}
		}
		// signal we are ready
		$batchCmds .= "cd ..\n";
		$batchCmds .= "put $workdir/push.flag\n";

		// create push.json
		file_put_contents( $this->workdir.'/push.json' , json_encode( $this->push ));
		
		// create empty flag file
		file_put_contents("$workdir/push.flag", '');
		
		// Create a batch file to be used with SFTP
		$batchfn = tempnam('/tmp', "apresa_upl");
		$errfn = tempnam('/tmp', "apresa_errupl_");
		file_put_contents( $batchfn, $batchCmds );
		// create command to execute
		$cmd = 'sftp -b '.$batchfn
			.$this->GetSftpCmdLineOptions()
			.' 2>'.$errfn;
		//echo "CMD: $cmd\n";
		$this->lg('SFTP to '.$this->target['ip'] . ' , upload');
		// execute the SFTP batch file
		//system( $cmd );
		$sftp_status = '';
		$sftp_output = array();
		$last = exec( $cmd , $sftp_output , $sftp_status );
		//system( $cmd );
		// remove the SFTP batch file
		unlink( $batchfn );
		$errmsgarr = file($errfn);
		$errmsg = $this->filterErrMsgs( $errmsgarr );
		unlink( $errfn );
		
		//echo "Status: $sftp_status $last\n";
		
		if ($sftp_status!=0)
		{
			return err($this->ref_retobj,6,'SFTP error code '.$sftp_status.' '.$errmsg);
		} else
		{
			$this->lg("SFTP OK, $recordings_upload recordings uploaded.");
			$this->SetProperty('triedid', $triedid );
			if ($this->this_is_all)
			{
				$this->target['status'] = STATUS_LASTFULL;
			}
		}
		
	}

	function SetProperty( $key , $value )
	{
		global $dbintf;
		$dbintf->EditTransferTarget( $this->target['id'] 
		, array ( $key => $value )
		);
	}
	
	function GetContactName( & $ref_contactlist , $id )
	{
		global $dbintf;
		if ($id==0) return null;
		if (isset($ref_contactlist[$id])==false)
		{
			$ref_contactlist[$id] = $dbintf->GetContactName( $id );
		} 
		return $ref_contactlist[$id];
	}
	
	function GetLineName( & $ref_list , $id )
	{
		global $dbintf;
		if ($id==0) return null;
		if (isset($ref_list[$id])==false)
		{
			$ref_list[$id] = $dbintf->GetLineName( $id );
		} 
		return $ref_list[$id];
	}
  
  function GetTenantName( & $ref_list , $id )
	{
		global $dbintf;
		if ($id==null) return null;
		if (isset($ref_list[$id])==false)
		{
      $group = $dbintf->GetGroupInfo( $id );
      $groupname = null;
      if ($group !== false) $groupname = $group['name'];
			$ref_list[$id] = $groupname;
		} 
		return $ref_list[$id];
  }

	function GetCallerID( & $ref_cidlist , $id )
	{
		global $dbintf;
		if ($id==0) return null;
		if (isset($ref_cidlist[$id])==false)
		{
			$obj = $dbintf->GetCallerID( $id );
			if ($obj==false) return '';
			$ref_cidlist[$id] = $obj['callerid'];
		} 
		return $ref_cidlist[$id];
	}
	
	function RunInDir()
	{
		if ($this->GetRemoteStatus()==false)
		{
			// connection error, or previous upload not processed yet
			return false;
		}
		
		$this->push = null;
		$this->DetermineUpload();
		//var_dump( $this->push['data'][ count($this->push['data'])-1 ] );
		if ($this->push!=null)
		{
			$this->PerformUpload();
			$this->CleanupDecrypt();
		}
		// this must be done at the end
		// the other side can start only after we have finished
		$this->SetProperty( 'lasttry', time() );
		
	}
	
	function Run()
	{
		global $globReturnCode;
		$this->workdir = exec('mktemp -d --tmpdir=/tmp');
		//$this->workdir = '/home/lucas/apresa/tmp2';
    
		$this->RunInDir();

		if ( isset($this->ref_retobj['errcode']) )
		{
			$errcode = $this->ref_retobj['errcode'];
			$globReturnCode = $errcode;
			if ($errcode!=0)
			{
				if ($errcode==5)
				{
					$this->target['status'] += 7;
					if ($this->target['status']>100)
					{
						$this->target['status'] = 100;
					}
				} else
				{
					$this->target['status'] = 100;
				}
			} else
			{
				if ($this->target['status'] > STATUS_OK)
				{
					$this->target['status'] = STATUS_OK;
				}
			}
		}
		if ( isset($this->ref_retobj['errmsg']) )
		{
			$this->SetProperty( 'errmsg', $this->ref_retobj['errmsg'] );
		}
		$this->SetProperty( 'status', $this->target['status'] );
		$this->SetProperty( 'lasttry', time() );
		
		exec( "rm -Rf ".escapeshellarg($this->workdir) );
	}
	
}


function HandleUpload( $options, & $ref_retobj )
{
	global $dbintf;
	
	$uploader = new Uploader();
	
	if ($uploader->Init( $options, $ref_retobj )==false)
	{
		return;
	}
	
	$uploader->Run();
	
	return;
}

// = = = = = = = = = = = = = = = = = = = = = = = =


class Scanner
{
	var $options;
	var $ref_retobj;
	var $mySN;
	var $uploaddir;
	
	function Init($param_options, & $param_ref_retobj)
	{
		$this->options = $param_options;
		$this->ref_retobj = & $param_ref_retobj;
		$this->mySN = GetSerialNumber();
		$this->uploaddir = '/recordings/upload/data';
		return true;
	}
	
	function lg($t)
	{
		lg('scan: '.$t);
	}
	
	function Scan()
	{
		$this->lg("Start");
		
		// Scanning is for all SN.
		// We do not know who will upload.
		
		$sndirs = scandir( $this->uploaddir );
		//var_dump( $files );
		
		foreach ($sndirs as $sndir)
		{
			if ($sndir=='.' || $sndir=='..') continue;
			$fulldir = $this->uploaddir . '/' . $sndir;
			if (is_dir($fulldir)==false)
			{
				continue;
			}
			// directory must start with SN
			if (substr($sndir,0,2)!='SN') continue;
			$sn = substr($sndir,2);
			//echo $sn . "\n";
			
			$this->ScanForSN( $sn , $fulldir );
		}
		return;
	}
	
	function Cleanup( $snpath )
	{
		// complete erase all content in recs subdirectory (the recordings)
		// all recordings should already been handled
		if (is_dir("$snpath/recs"))
		{
			system("rm -Rf " . escapeshellarg("$snpath/recs"));
		}
		// delete files in the main directory,
		// except the following that we want to keep
		// - proc.json , because generated by us
		// - push.flag , because it must be deleted at the last moment
		$exclude = array ( 'proc.json'=>1, 'push.flag'=>1 );
		$files = scandir( $snpath );
		foreach ($files as $file)
		{
			$path = $snpath . '/' . $file;
			if (is_file($path)==false)
			{
				continue;
			}
			if (isset($exclude[$file]))
			{
				continue;
			}
			DeleteFn( $path );
		}
	}
	
	function ScanForSN( $sn , $snpath )
	{
		global $globReturnCode;
		
		// check for the flag file that signals a completed upload
		if (file_exists($snpath . '/push.flag')==false)
		{
			// nothing to here
			return;
		}
		
		//echo $snpath . "\n";
		
		// something found
		$globReturnCode = 1;
		
		// build a reply
		$reply = array();
		$reply['sender'] = $this->mySN;
		
		$this->ReadSN( $reply , $sn, $snpath );
		
		$this->Cleanup( $snpath );

		file_put_contents( "$snpath/proc.json", json_encode($reply) );
		
		// remove handled files (allow next round)
		@unlink( $snpath . '/' . 'push.json' );
		@unlink( $snpath . '/' . 'push.flag' );
	}
	
	function ReadSN( & $reply, $sn, $snpath )
	{
		global $dbintf;
	
		clearstatcache();
	
		$pushfn = $snpath . '/' . 'push.json';
		if (file_exists($pushfn)==false)
		{
			return err($reply,1,'no push.json');
		}
		
		$pushcontent = file_get_contents( $pushfn );
		$pushobj = json_decode( $pushcontent , true );
		//var_dump( $pushobj );
		if (isset($pushobj['data'])==false)
		{
			return err($reply,2,'no data');
		}
		$callcnt = count( $pushobj['data'] );
		$this->lg("Importing $callcnt calls from SN $sn");
    
    // load additional options
    $options = [];
    // default:
    $options['compare'] = 3;
    if (isset($pushobj['pubkey']))
    {
      $authkeys = $dbintf->GetAuthKeys();
      foreach ($authkeys as $authobj)
      {
        if ( trim($pushobj['pubkey']) == trim($authobj['pubkey']) )
        {
          $options['undouble'] = $authobj['undouble'];
          if (isset( $authobj['compare'] ))
          {
            $options['compare'] = $authobj['compare'];
          }
        }
      }
    }
    $undouble = isset($options['undouble']) && ($options['undouble']==1);
    $this->lg("Undoubling " . ($undouble?'enabled':'disabled') 
      . ' compare: ' . $options['compare'] );
		
		$accepted_cols = $dbintf->GetTableColumns( 'Calls' );
		unset( $accepted_cols['id'] );
		//var_dump( $accepted_cols );
		//exit;
		
		$ALE = (int) $dbintf->GetConfigValue( 'ALE' );
		$said_already = false;
		//$this->lg('Call content encryption is: ' . (($ALE!=0)?'ON':'OFF') );
    
    $SpinServer = (int) $dbintf->GetConfigValue('SpinServer');
		$DetermineAgent = $dbintf->GetConfigValue('DetermineAgent');
		
		foreach( $pushobj['data'] as $data)
		{
			// add this call to our database (import)
			
			/*
			// in JSON, empty things are set as NULL sometimes
			// but this needs to be corrected before entering into the database
			// CORRECTION: not true (it seems)
			foreach ($data as $key=>&$ref_value)
			{
				if ($ref_value===null)
				{
					$ref_value = '';
				}
			}
			*/

			$ori_id = $data['id'];
			unset($data['id']);

			$call_sn = $data['recorder_sn'];
			
			if ($call_sn == $this->mySN)
			{
				// this prevents duplication, and cannot delete problems
				$reply['already'][] = $ori_id;
				$reply['processed'][] = $ori_id;
				if ($said_already == false)
				{
					$this->lg("Call (".$data['remoteid'].") originated from this server");
					$said_already = true;
				}
				continue;
			}
			
			$ori_filename = $data['filename'];
			$data['remoteid'] = SnAppend( $data['remoteid'], $call_sn );
			$data['filename'] = SnAppend( $data['filename'], $call_sn );
      // in PHP, isset means that it is set *and* that it is not NULL
      // that is what we want here
      if (isset($data['screenfn']))
      {
        $data['screenfn'] = SnAppend( $data['screenfn'], $call_sn );
      }

			if ($dbintf->GetCallInfoRM( $data['remoteid'] ) !== false)
			{
				// call already exists
				$reply['already'][] = $ori_id;
				$reply['processed'][] = $ori_id;
				// prevent creating too many log entries
				if ($said_already == false)
				{
					$this->lg("Entry in DB (".$data['remoteid'].") is already there!");
					$said_already = true;
				}
				continue;
			}
      
			$recorderName = 'Apresa'.$call_sn;
			if (isset($data['recorder_name']) )
			{
				$recorderName = $data['recorder_name'];
			}
			$recorderType = 52;
			if (isset($data['recorder_type']) )
			{
				$recorderType = $data['recorder_type'];
				// do not add main Apresa
				if ($recorderType==50) $recorderType = 52;
			}
			
			// Note: Use $data['recorderid'] and not SN,
			// because a server might contain recordings from other recorders.
			$data['recorderid'] = $dbintf->GetOrCreateRecorderId
				(	$call_sn
				, $recorderName
				,	$recorderType);
			unset( $data['recorder_sn'] );
			unset( $data['recorder_name'] );
			unset( $data['recorder_type'] );
			
      if ($undouble && isset($data['files'][0]))
      {
        $audioFilePath = $snpath . '/recs/' . $data['files'][0]['fn'];
        if ($this->FindDuplicate($data,$audioFilePath,$options['compare']))
        {
          // skip this call
          $reply['already'][] = $ori_id;
          $reply['processed'][] = $ori_id;
          continue;
        }
      }
			
			$direction = $data['direction'];
			$remote_is_local = ($direction==2);
			$local_is_remote = ($direction==3);
			$connected_is_local = ($direction==0) || ($direction==2);
			
			if ($data['local_cid']!==null)
			{
				$ret = $dbintf->GetOrCreateCallerID($data['local_cid'],$data['local2']
					,$local_is_remote?0:1,$data['local_name']);
        if ($data['local_name']!='' && $dbintf->GetContactName($ret['contactid']) == '')
        {
          $dbintf->EditContact($ret['contactid'],$data['local_name']);
        }      
				$data['localcallerid'] = $ret["id"];
				$data['localcontact'] = $ret["contactid"];
			} else
			{
				$data['localcallerid'] = 0;
				$data['localcontact'] = 0;
			}
			unset($data['local_cid']);
			unset($data['local_name']);
			
			if ($data['remote_cid']!==null)
			{
				$ret = $dbintf->GetOrCreateCallerID($data['remote_cid'],$data['remote2']
					,$remote_is_local?1:0,$data['remote_name']);
        if ($data['remote_name']!='' && $dbintf->GetContactName($ret['contactid']) == '')
        {
          $dbintf->EditContact($ret['contactid'],$data['remote_name']);
        }      
				$data['remotecallerid'] = $ret["id"];
				$data['remotecontact'] = $ret["contactid"];
			} else
			{
				$data['remotecallerid'] = 0;
				$data['remotecontact'] = 0;
			}
			unset($data['remote_cid']);
			unset($data['remote_name']);
			
			if ($data['connected_cid']!==null)
			{
				$ret = $dbintf->GetOrCreateCallerID($data['connected_cid'],''
					,$connected_is_local?1:0,$data['connected_name']);
				$data['connectedcallerid'] = $ret["id"];
				$data['connectedcontact'] = $ret["contactid"];
			} else
			{
				$data['connectedcallerid'] = 0;
				$data['connectedcontact'] = 0;
			}
			unset($data['connected_cid']);
			unset($data['connected_name']);
			
			// testing block
			//$data['linenr'] = 6;
			//$data['line_name'] = 'TestLine6';
			// end block
			if ($data['linenr']!=0)
			{
				$ct = $dbintf->GetOrCreateLineContact
				( $data['linenr']
				, $data['recorderid']
				, $data['line_name']
				);
				$data['lineid'] = $ct['id'];
				if ($data['localcontact']==0)
				{
					$data['localcontact'] = $ct['contactid'];
				} else if ($data['remotecontact']==0)
				{
					$data['remotecontact'] = $ct['contactid'];
				}
			}
			unset($data['line_name']);
			
			unset($data['retention']);

			$data['processingstate'] = 0;
			// flags is copied as is

			$extrasize = 0;
			$file_err = false;
			foreach ($data['files'] as $filetrans)
			{
				$srcpath = $snpath . '/recs/' . $filetrans['fn'];
				if (file_exists($srcpath)==false)
				{
					$this->lg('Import file not found: '.$srcpath);
					$file_err = true;
					continue;
				}
				if (filesize($srcpath)==$filetrans['sz'])
				{
					$dstdir = AUDIO_PATH . '/' . $data['directory'];
					@mkdir( $dstdir , 0777 , true );
					$dstfn = SnAppend( $filetrans['fn'], $call_sn );
					//$this->lg('Import: '.$dstfn);
					$dstpath = $dstdir . '/' . $dstfn;
					if ($ALE)
					{
						$cmd = 'apresa_crypt -e -i ' . escapeshellarg($srcpath)
								 . ' -o ' . escapeshellarg($dstpath) . ' ';
						// too verbose: $this->lg( $cmd );
						$encrypt_result = 0;
						$encrypt_output = array();
						exec( $cmd , $encrypt_output , $encrypt_result);
						if ($encrypt_result == 0)
						{
							$data['flags'] |= 16;
						} else
						{
							// fall back to no encryption
							$this->lg('Encryption error: '.$encrypt_result);
							rename( $srcpath , $dstpath );
						}
					} else
					{
						rename( $srcpath , $dstpath );
					}
					chmod( $dstpath , 0777 );
					$extrasize += filesize($dstpath);
				} else
				{
					$file_err = true;
					$this->lg('Size mismatch '.$filetrans['sz'].' vs '.filesize($srcpath).' '.$srcpath);
				}
			}
			unset( $data['files'] );
			if ($file_err)
			{
				// incomplete received
				// stop import here
				break;
			}

      $tenantid = null;
      if ($data['tenant_name'] != null)
      {
        $matchingTenants = $dbintf->GetGroupInfoOnName( $data['tenant_name'] );
        if (count($matchingTenants) >= 1)
        {
          $tenant = $matchingTenants[0];
          if ($tenant['is_tenant'])
          {
            $tenantid = $tenant['id'];
          }
        }
      }
      if ($tenantid==null)
      {
        // apply based on local tenant configuration (its user phones)
        $tenantid = CalcTenant($data);
      }
			$data['tenantid'] = $tenantid;
			if ($tenantid != NULL)
			{
				UpdateTnDiskUsage($tenantid, $extrasize, +1);
			}
			if ($DetermineAgent !== '0')
			{
				$data['agent'] = GetAgentOfCall($data);
			}
      
			// Remove any columns that are not in our Calls table
			$to_unset = array();
			foreach ($data as $key=>$value)
			{
				if (isset( $accepted_cols[$key] )==false)
				{
					//echo 'Warning: Not accepted column: '. $key . "\n";
					$to_unset[] = $key;
				}
			}
			foreach ($to_unset as $col)
			{
				unset( $data[$col] );
			}
      
      // Merge data if SpinServer is used
      if ($SpinServer!=0)
      {
        $activeCall = $dbintf->GetActiveCallInfoRM( $data['remoteid'] );
        if ($activeCall !== false)
        {
          lg('Merging with active call data (spin)');
          $dbintf->RemoveActiveCall( $data['remoteid'] );
          if ($activeCall['category']!= 0) $data['category'] = $activeCall['category'];
          $tocopy = array( 'notes', 'notes2', 'notes3', 'notes4', 'notes5'
          , 'data1', 'data2', 'data3', 'data4', 'data5' );
          foreach ($tocopy as $field)
          {
            if ($activeCall[$field]   !='') $data[$field] = $activeCall[$field];
          }
        }
      }
			
			// Output data for debugging / testing
			//var_dump( $data );
			/*
			$dbg = array( 'localcallerid' , 'recorderid', 'lineid'
			, 'remoteid', 'filename' );
			foreach ($dbg as $key)
			{
				echo $key . ": " . $data[$key] . "\n";
			}
			*/
			
			$call_id = $dbintf->InsertCall( $data );
			$reply['processed'][] = $ori_id;
			
		}
		
	}
  
  function analyseAudio($path)
  {
    vlg("analyse $path");
    $output = shell_exec("sox $path -n stat 2>&1");
    $lines = explode("\n", $output);
    $data = [];
    $data['filesize'] = filesize($path);
    foreach ($lines as $line)
    {
      $colonPos = strpos($line, ':');
      if ($colonPos===false) continue;
      $key = trim(substr($line,0,$colonPos));
      $value = trim(substr($line,$colonPos+1));
      if ($key=='Length (seconds)')
      {
        $data['len'] = floatval($value);
      }
      if (strpos($key,'RMS')!==false && strpos($key,'amplitude')!==false)
      {
        $data['RMSamp'] = floatval($value);
      }
      if (strpos($key,'RMS')!==false && strpos($key,'delta')!==false)
      {
        $data['RMSdelta'] = floatval($value);
      }
    }
    foreach ($data as $k=>$v)
    {
      vlg("- $k: $v");
    }
    return $data;
  }
  
  function percentageDiff($a,$b)
  {
    if ($b==0 && $a==0) return 0;
    if ($b==0) return 100 * abs($a-$b)/$a;
    else return 100 * abs($a-$b)/$b;
  }
  
  function FindDuplicate($data, $newPath, $compare)
  {
    global $dbintf;
    vlg("FindDuplicate");
    
    $timeRange = 2; // in seconds
    $fd = $data['startdate'] - $timeRange;
    $ld = $data['startdate'] + $timeRange;
    
    $calls = $dbintf->GetCallsBetween($fd,$ld,$data['recorderid']);
    vlg("Found " . count($calls) . " calls in time range");
    
    $newAnalysed = false;
    
    // check for each if a duplicate
    foreach ($calls as $call)
    {
      vlg("Checking call " . $call['remoteid'] );
      $exactCompare = ['local','remote','direction'];
      $good = true;
      foreach ($exactCompare as $col)
      {
        if ($data[$col] !== $call[$col])
        {
          vlg("mismatch of $col");
          $good = false;
          break;
        }
      }
      if ($good==false) continue;
      
      $durationDiff = abs( $call['duration'] - $data['duration'] );
      vlg('Duration diff ' . $durationDiff);
      if ($durationDiff > 2)
      {
        vlg("Duration mismatch");
      }

      // Now check the audio
      
      if ($newAnalysed==false)
      {
        $newAudioData = $this->analyseAudio($newPath);
        //var_dump( $newAudioData );
      }
      
			$fullfn = AUDIO_PATH . '/' . $call['directory'] . '/' . $call['filename'];
      $existingAudioData = $this->analyseAudio($fullfn);
      
      $fsPDiff = $this->percentageDiff($newAudioData['filesize'],$existingAudioData['filesize']);
      $fsDiff = abs($newAudioData['filesize'] - $existingAudioData['filesize']);
      vlg('Filesize diff: ' . $fsPDiff . '% ' . ($fsDiff/1024) . ' kB');
      if ($fsDiff>2*1024 && $fsPDiff>5)
      {
        vlg('Filesize mismatch');
        return false;
      }
      
      $lenDiff = abs($newAudioData['len'] - $existingAudioData['len']);
      vlg('Duration diff: ' . $lenDiff);
      if ($lenDiff>2)
      {
        vlg('Duration mismatch');
        return false;
      }

      $rmsDeltaDiff = abs($newAudioData['RMSdelta'] - $existingAudioData['RMSdelta']);
      vlg('RMS delta diff: ' . $rmsDeltaDiff);
      $rmsAmpDiff = abs($newAudioData['RMSamp'] - $existingAudioData['RMSamp']);
      vlg('RMS amp diff: ' . $rmsAmpDiff);
      $rmsDiff = ($rmsAmpDiff + $rmsDeltaDiff * 3) / 2.0;
      vlg('RMS diff: ' . $rmsDiff);
      $th = (0.001 / 27) * pow($compare,3);
      vlg('Threshold: ' . $th);
      if ($rmsDiff>$th)
      {
        lg("Almost dup, but audio diff $rmsDiff > $th");
        return false;
      }
      
      // we found a duplicate
      lg('Duplicate found ' . $call['remoteid']);
      
      return true;
    }
    return false;
  }
	
}

function HandleScan( $options, & $ref_retobj )
{
	global $dbintf;

  // needed permission
  $dbintf->AllowEditNames();
  
  // when run stand-alone
  // otherwise apresa-remote dit
  umask(0);
	
	$scanner = new Scanner();
	if ($scanner->Init( $options, $ref_retobj )==false)
	{
		return;
	}
	$scanner->Scan();
	return;
}



?>