<?php
/*
Client connector.
This provides an interface for client.php to get the information it needs.
It sits in between to provide caching.
It is started and managed by apresa-remote.
*/

// without CMD it cannot load language
define('CMD',1);
// to get logging in other remote log file
define('CMD_REMOTE',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "utility.php";
require_once CMD_WEB_RELPATH . "actions.php";
require_once CMD_WEB_RELPATH . "language.php";

// this var is checked during logon for free-seating
// which is only available for Apresa Client
// for which this PHP is a front-end
$CLIENT = 1;

if (str_ensure_defined($argv[1])=='debug')
{
	// logging to console instead of apresa.log
	define('DEBUG_LOGING',1);
}

$exit = false;

#region Signal handling

function sig_handler($signo)
{
  global $exit;
  lg("CC Signal $signo");
  if ($signo==SIGINT
  || $signo==SIGTERM)
  {
    $exit = true;
  }
}

pcntl_signal(SIGINT, 'sig_handler');
pcntl_signal(SIGTERM, 'sig_handler');

#endregion

// A user session will survive multiple connections
class UserSession
{
  // the username and password for logon
  var $usr;
  var $pwd;
  // cached password,
  // to check if it is the same later,
  // and cache previous result
  var $cached_pwd;
  var $login_time;
  var $login_ok;
  // link to database object
  var $db;
}

class ClientCon
{
  var $sock;
  var $userSession;
  var $toSend;
  var $rcv;
  
  function __construct()
  {
    $this->rcv = '';
    $this->oldCnt = 0;
    $this->createdAt = time();
  }
  
  function errReply($msg)
  {
    $reply = array
    ( 'r' => 1
    , 'm' => $msg
    );
    $this->replyObj($reply);
  }
  
  function replyObj($obj)
  {
    $this->toSend = json_encode($obj) . "\0";
  }
  
}

class ClientConnector
{
  var $activeCalls = array();
  var $activeCallsTime = 0;
  
  var $users = array();
  // connection with clients
  var $clients = array();
  
  var $maindb = null;
  var $cfgCache = array();
  var $cfgCacheTime = 0;
  
  function GetActive()
  {
    //lg('DB: get full active calls list');
    $this->activeCalls = $this->maindb->GetActiveCallsForClientCon();
    $this->activeCallsTime = microtime(true);
  }
  
  function chkUpdateActiveCalls()
  {
    $UPDATE_SECONDS = 0.5;
    $age = microtime(true) - $this->activeCallsTime;
    //lg('active call list is ' . $age . ' s old');
    if ( $age >= $UPDATE_SECONDS)
    {
      $this->GetActive();
    }
  }
  
  function BuildCfgCache()
  {
    LoadBasicSettings();
    $toload = array( 'UsrFilterLocal', 'MultiTenancy', 'RecOnDemand' );
    foreach ($toload as $key)
    {
      $this->cfgCache[ $key ] = $this->maindb->GetConfigValue($key);
    }
    $this->cfgCacheTime = time();
  }
  
  function chkBuildCfgCache()
  {
    $UPDATE_SECONDS = 70;
    $age = time() - $this->cfgCacheTime;
    if ( $age >= $UPDATE_SECONDS)
    {
      $this->BuildCfgCache();
    }
  }
  
  function GetOrCreateUserSessionUid($uid)
  {
    foreach ($this->users as $uc)
    {
      if ($uc->uid == $uid)
      {
        return $uc;
      }
    }
    $uc = new UserSession();
    $uc->uid = $uid;
    $this->users[] = $uc;
    return $uc;
  }
  
  function GetOrCreateUserSession($usr)
  {
    foreach ($this->users as $uc)
    {
      if ($uc->usr == $usr)
      {
        return $uc;
      }
    }
    $uc = new UserSession();
    $uc->usr = $usr;
    $uc->dispusr = $usr;
    $this->users[] = $uc;
    return $uc;
  }
  
  function UserRq($uid,$usr,$pwd,$pc)
  {
    if (isset($uid))
    {
      $uc = $this->GetOrCreateUserSessionUid($uid);
    } else
    {
      $uc = $this->GetOrCreateUserSession($usr);
    }
    $uc->pwd = $pwd;
    $uc->pc = $pc;

    // this has to be less than the expiry time
    // for custom local id free-seating
    $LOGIN_CACHE_TIME = 20;
    
    if ($uc->cached_pwd !== null
    && $uc->cached_pwd == $pwd
    && time() - $uc->login_time < $LOGIN_CACHE_TIME
    )
    {
      // use login_ok (boolean)
      //lg('Login ' . ($uc->login_ok?'OK':'failed') . ' cached for ' . $uc->usr);
    }
    else
    {
      if ($uc->db === null
      || $uc->db->db == null)
      {
        //lg('setup database connection');
        $uc->db = new DBInterface('');
        $uc->db->cloneInit( $this->maindb );
        $uc->db->cachedsettings = $this->cfgCache;
      }
      // we need to check fresh
      $uc->cached_pwd = $uc->pwd;
      $uc->login_time = time();
      if (isset($uid))
      {
        // web client
        $uinfo = $uc->db->GetUserInfo($uid);
        if ($uinfo === false)
        {
          $uc->login_ok = false;
        } else
        {
          $uc->dispusr = $uinfo['usr'] . '(W)';
          //lg('Login through sessions OK for ' . $uc->dispusr);
          $uc->db->SetCurrentUserId($uid);
          $uc->login_ok = true;
        }
      } else
      {
        // Apresa Client on PC
        $uid = null;
        if (GenTryLogin($uc->db, $uc->usr, $uc->pwd, $uc->pc, $uid))
        {
          //lg('Login OK for ' . $uc->usr);
          $uc->db->SetCurrentUserId($uid);
          $uc->login_ok = true;
          
          // check setting
          if ($uc->db->actuinfo['customlocalid']!=0)
          {
            $uaccid = $uc->db->actuinfo['accid'];
            $phone_list = $uc->db->GetUserPhones($uaccid);
            $DstContactId = $uc->db->GetCustomLocalContactId();
            $uc->db->AddOrUpdateLocalIdMapping($phone_list, $DstContactId );
          }
          
        } else
        {
          //lg('Login failed for ' . $uc->usr);
          $uc->login_ok = false;
        }
      }
      // we keep the database connection for a short time
      $uc->db->close();
    }
    return $uc;
  }
  
  function EndSocket($sock)
  {
    @socket_shutdown($sock);
    @socket_close($sock);
    foreach ($this->clients as $key => $client)
    {
      if ($client->sock == $sock)
      {
        //lg('remove from clients array');
        unset($this->clients[$key]);
      }
    }
  }

  function ClientMsg( $client )
  {
    global $configs;
    //lg('Received message: ' . $client->rcv);
    $msg = json_decode( $client->rcv );
    if ($msg==NULL)
    {
      //lg('Could not decode message as JSON');
      $client->errReply('json_decode');
      return;
    }
    $webclient = isset($msg->uid);
    $session = $this->UserRq( $msg->uid, $msg->usr, $msg->pwd, $msg->pc );
    if ($session->login_ok==false)
    {
      $client->errReply('login');
      return;
    }
		if ($webclient)
		{
			if ( ($session->db->actuinfo['permissions']&PM_WEBCLIENT) == 0 )
			{
				$client->errReply('NoWebclientPm');
				return;
			}
		}
    //lg('we have a session and login');
    
    $phones = $session->db->GetMyPhones();
    
    $reply = array();
    $reply['r'] = '0';
    $reply['ph'] = $phones;
    if ( isset($msg->info) && $msg->info=='ActiveCalls' )
    {
      // determine active calls for user

      // possibly update or use global cache
      $this->chkUpdateActiveCalls();
      //lg("Total Count: " . count($this->activeCalls) );
      $this->chkBuildCfgCache();
      $StoreOnDemand = $this->cfgCache['RecOnDemand'];
      if($session->db->actuinfo['rod'] == 1)
      {
        $StoreOnDemand = true;
      }
      else if($session->db->actuinfo['rod'] == 2)
      {
        $StoreOnDemand = false;
      }      
      
      $calls = array();
      foreach ($this->activeCalls as $call)
      {
        // only own phones
        // at the same time determine if we are local or remote
        $side = $session->db->IsCallWithMe($call);
        if ($side==0) continue;
        $call['side'] = $side;
        // get permission
        $pm = $session->db->GetCallPermissions($call);
        // - for the normal Apresa client (PC), we need listing permission
        // - for web client not, because it should be possible to only do
        // web client things without being allowed to see the listing.
        // Above we already filtered out calls not with this user.
        if ( ($webclient==false) && (($pm & PM_LIST) == 0) ) continue;
        $call['pm'] = $pm;
        if ($webclient)
        {
          $callextra = StringToArray($call['callextra']);
          $call['sd']  = date($configs['datetimef'],$call['startdate']);
          $call['sdf'] = date($configs['datef'],$call['startdate']);
          $call['stf'] = date($configs['timef'],$call['startdate']);
          if($StoreOnDemand)
          {
            if(!$callextra['to-keep'])
            {
              $call['sod'] = 0; // Will not be stored
            }
            else
            {
              $call['sod'] = 1; // Will be stored
            }
          }
          else
          {
            $call['sod'] = 2; // Off, will be stored
          }
        } else
        {
          unset($call['notes']);
          unset($call['notes2']);
          unset($call['category']);
        }
        $calls[] = $call;
      }
      if ($session->oldCnt != count( $calls ) )
      {
        lg('CC ' . $session->dispusr . " : " . count( $calls ) . " calls");
      }
      $reply['list'] = $calls;
      $session->oldCnt = count( $calls );
    }
    $client->replyObj($reply);
    //lg('To send back reply with length: ' . strlen($client->toSend) );
  }
  
  function Init()
  {
    global $dbintf;
		$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
    $this->maindb = $dbintf;
    // it must be $dbintf, otherwise LoadBasicSettings will not work
    $this->BuildCfgCache();
  }
  
  function Run() 
  {
    global $exit;
    $mainLoopCnt = 0;

    lg("CC Started");
    
    $masterSocket = socket_create(AF_UNIX,SOCK_STREAM,0);
    if ($masterSocket === false)
    {
      LogSocketError();
      return;
    }
    $ipc_path = '/tmp/apresa_cc';
    @unlink( $ipc_path );
    if (socket_bind($masterSocket, $ipc_path) === false)
    {
      LogSocketError();
      return;
    }
    // otherwise clients cannot write to it
    chmod($ipc_path,0766);
    
    if (socket_listen($masterSocket) === false)
    {
      LogSocketError();
      return;
    }
    
    // not needed anymore: socket_set_nonblock($masterSocket);
    
    while ($exit==false)
    {
      $mainLoopCnt ++;

      $readSocks = array($masterSocket);
      foreach ($this->clients as $client)
      {
        $readSocks[] = $client->sock;
      }        
      
      $writeSocks = array();
      foreach ($this->clients as $client)
      {
        if ($client->toSend != '')
        {
          $writeSocks[] = $client->sock;
        }
      }        
      
      $excSocks = array();
      
      //lg('waiting on ' . count($readSocks) . ' sockets');
      
      $result = @socket_select($readSocks,$writeSocks,$excSocks,NULL);
      if ($result === false)
      {
        if (socket_last_error()==4)
        {
          // interrupted is ok
          //lg('CC socket waiting interrupted');
        }
        else LogSocketError('socket_select');
      }
      if ($result>0)
      {
        //lg('data for ' . $result . ' sockets');
        foreach ($readSocks as $readSock)
        {
          //lg('something to read on a socket');
          if ($readSock == $masterSocket)
          {
            //lg('new incoming connection');
            $sock = socket_accept($masterSocket);
            if ($sock===false)
            {
              LogSocketError('socket_accept');
            } else
            {
              $cc = new ClientCon();
              $cc->sock = $sock;
              $this->clients[] = $cc;
            }
          } else
          {
            //lg('something not on master');
            $client = null;
            foreach ($this->clients as $loopClient)
            {
              if ($loopClient->sock == $readSock)
              {
                $client = $loopClient;
              }
            }
            $buf = socket_read($readSock,4*1024);
            if ($buf === false)
            {
              LogSocketError('socket_read');
            } else if ($buf === '')
            {
              //lg('end of socket (empty read)');
              $this->EndSocket($readSock);
            } else
            {
              // received data from client
              $client->rcv .= $buf;
              // lg('RCV: ' . $client->rcv);
              if (strlen($client->rcv)>0 && $client->rcv[ strlen($client->rcv)-1 ]=="\0")
              {
                $client->rcv = substr($client->rcv,0,strlen($client->rcv)-1);
                $this->ClientMsg( $client );
              }
            }
          }
        }
        foreach ($writeSocks as $writeSock)
        {
          //lg('to write to socket');
          $client = null;
          foreach ($this->clients as $loopClient)
          {
            if ($loopClient->sock == $writeSock)
            {
              $client = $loopClient;
            }
          }
          $result = @socket_write($writeSock,$client->toSend);
          if ($result===false)
          {
            LogSocketError('socket_write');
          } else if ($result>0)
          {
            $client->toSend = substr($client->toSend,$result);
            $toSendCnt = strlen($client->toSend);
            if ($toSendCnt==0)
            {
              // we could disconnect here
              // but socket then stays in use
              // better wait for the other side to disconnect
            } else
            {
              //lg('Still ' . $toSendCnt . ' bytes to send');
            }
          }
        }
      }
      
      // debug: 
      //lg("$mainLoopCnt Time " . time());
      //$this->GetActive();
      //lg("Active count " . count($this->activeCalls) );
      
      // debug
      //$this->UserRq('5010','5010');

      // no sleep needed anymore! usleep(500 * 1000);
      pcntl_signal_dispatch();
      
      //break; // debug
    }
    
    lg('CC Cleanup');
    foreach ($this->clients as $client)
    {
      socket_shutdown($client->sock);
      socket_close($client->sock);
    }
    
    socket_shutdown($masterSocket);
    socket_close($masterSocket);
  
  } // end of Run function
  
  
}

function LogSocketError($t)
{
  $err = socket_last_error();
  lg('CC Socket error ' . $err . ' : ' . socket_strerror($err) . ' with ' . $t);
}

$cc = new ClientConnector();
$cc->Init();
$cc->Run();

lg("CC End");

