<?php
/*
	Handler of starting and stopping of cluster node (high availability)
*/
define('CMD',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";

// Interface to database
$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);

Main();

function nlg($t)
{
	lg('clusternode.php '.$t);
}

function systemlg($cmd)
{
	nlg("CMD: $cmd");
	return system($cmd);
}

function Main()
{
	global $argv;
	
	if (isset($argv[1])==false)
	{
		nlg('no arguments');
		exit(2);
	}

	// debug: echo "ARGV[1] " . $argv[1] . "\n";
	$arg1 = $argv[1];
	if ($arg1=='start') Start();
	if ($arg1=='stop') Stop();
	if ($arg1=='init1') Init1();
	if ($arg1=='init2') Init2();
}

function wait_signal_service($cmd, $timeout)
{
	$reply = true;
	signal_service($cmd, $reply, $timeout);
}

function Start()
{
	global $dbintf;
	$dbintf->ClearActiveCalls();
}


function Stop()
{
  // doing something with crm might result in timeout err
}

// initialisation that is needed on both nodes
function InitBoth()
{
	global $dbintf;

	$ipaddr1 = $dbintf->GetConfigValue('cluster_ipaddr1');
	$ipaddr2 = $dbintf->GetConfigValue('cluster_ipaddr2');
	$pgoptfn = '/etc/postgresql/9.4/main/pg_hba.conf';
	system("echo \"hostssl replication postgres $ipaddr1/32 trust\" >> $pgoptfn");
	system("echo \"hostssl replication postgres $ipaddr2/32 trust\" >> $pgoptfn");
	
}

// wait for text to appear in the crm status
function WaitCrm($text,$invert=false,$cmd='')
{
        $es = 0;
				if ($invert) nlg("Waiting for crm text $text to disappear");
        else nlg("Waiting for crm text $text");
        while (true)
        {
								if ($cmd!='')
								{
									systemlg($cmd);
								}
                //nlg("Checking...");
                system("crm_mon -A -1 | grep $text > /dev/null", $es);
                if ($invert)
                {
                        if ($es!=0) break;
                } else
                {
                        if ($es==0) break;
                }
                system("sleep 1");
        }
        nlg("Done waiting");
}

function WaitSysZero($cmd)
{
	nlg("Waiting until $cmd returns zero");
	$es = 0;
	while (true)
	{
		system($cmd,$es);
		if ($es==0) break;
		system("sleep 1");
	}
}

function Init1()
{
	global $dbintf;
	
	InitBoth();
	
	// Initialisation of the primary.
	
	// clear configuration
	systemlg('cibadmin -E --force');
	systemlg('service pacemaker restart');
	// wait until node at least online
	sleep(3);
	WaitCrm("Online");

	// put in maintenance mode first
	systemlg('crm configure property maintenance-mode=on');
	
	$rpl = array();
	$rpl['[floating_addr]'] = $dbintf->GetConfigValue('cluster_vip');
	$rpl['[nodenames]'] = $dbintf->GetConfigValue('cluster_ipname1')
	. ' ' . $dbintf->GetConfigValue('cluster_ipname2');

	// if maybe still exists from old round
	systemlg('crm cib delete newconf');
	
	// newconf will have node existance configuration copied
	// that is required for proper functionality
	systemlg('crm cib new newconf');
	
	// most is done in .sh script
	// but some must be done here
	// specifically, the pacemaker configuration
	$lines = file( '/var/apresa/conf/pacemaker.dist' );
	$cmd = '';
	foreach ($lines as $line)
	{
		$line = trim($line);
		if (substr($line,0,1)=='#')
		{
			continue;
		}
		if (substr($line,-1)=='\\')
		{
			$cmd .= substr($line,0,strlen($line)-1) . ' ';
			// will continue on next line
			continue;
		}
		$cmd .= $line;
		// apply replacements
		foreach ($rpl as $key => $value)
		{
			//echo $value;
			$cmd = str_replace($key,$value,$cmd);
		}
		if (trim($cmd)=='') continue;
		// -F = force, means do not prompt
		// if invalid (duplicate), it will still not do it (which is good)
		if (substr($cmd,0,strlen('configure'))=='configure')
		{
			$cmd = 'crm -F -c newconf ' . $cmd;
		} else
		{
			$cmd = 'crm -F ' . $cmd;
		}
		systemlg($cmd);
		$cmd = '';
		//WaitCrm("Online");
	}
	systemlg('crm cib commit newconf');
	systemlg('crm cib delete newconf');

	systemlg('crm configure property maintenance-mode=off');

	// wait until we are online with resources loaded
	WaitCrm("Online");
	WaitCrm("anode");
	WaitCrm("PRI");
	sleep(1);

	// restart
  systemlg('service pacemaker restart');
	WaitCrm("Online");
	WaitCrm("anode");
	// wait until this file is created
	// we use async, so not created anymore: WaitSysZero('ls /var/lib/pgsql/tmp/rep_mode.conf');
	systemlg('sleep 1');
	
	// recover database
	WaitCrm("unmanaged",false,'crm configure property maintenance-mode=on');
	systemlg('sleep 1');
	systemlg('service postgresql stop');
	systemlg('rm -f /database/db/main/recovery.conf');
	systemlg('rm -f /var/lib/pgsql/tmp/PGSQL.lock');
	systemlg('service postgresql start');
	// wait untils started
	WaitSysZero('psql apresa -c \'select now()\'');
	systemlg('crm resource cleanup postgresql');
	WaitCrm("unmanaged",true,'crm configure property maintenance-mode=off');
	// wait until DB is OK and node is started
	WaitCrm("ms_postgresql");
	WaitCrm("PRI");
	WaitCrm("anode | grep Started");
	systemlg('crm resource cleanup postgresql');
	
	nlg("End of function Init1");
	
}

function Init2()
{
	global $dbintf;
	
	// Initialisation of the secondary.
	
	// do things needed for both
	InitBoth();

	// load IP address of master before we stop the database
	// NOTE: VIP is not known (it is not filled in at secondary)
	$DbIp = $dbintf->GetConfigValue('cluster_ipaddr1');
	
	// We are going to start with a fresh copy from the primary.
	// The primary must be running now!
	
	// make sure pacemaker is not running
	system('service pacemaker stop');
	// also stop postgresql manually
	system('service postgresql stop');
	// completely erase local database
	system('rm -r /database/db/main/*');
	// load database from primary
	system("sudo -u postgres pg_basebackup -h $DbIp -U postgres -D /database/db/main -X stream -P");
	// only the certificate/key needs to be added
	system('ln -s /etc/ssl/certs/ssl-cert-snakeoil.pem /database/db/main/server.crt');
	system('ln -s /etc/ssl/private/ssl-cert-snakeoil.key /database/db/main/server.key');
	
	// pgsql will create recovery.conf
	// no need to do ourselves

	// after fail-over, the following is also needed:
	// system('rm /var/lib/pgsql/tmp/PGSQL.lock');
	// system('crm resource cleanup ms_postgresql');
	
	// ready, only now start pacemaker
	system('service pacemaker start');
}


function OldStart()
{
	// first perform any pending updates
	// check /recordings/updates/updateX.tgz
	lg('Check for updates');
	$do_update = file_exists('/recordings/updates/do_update');
	if ($do_update)
	{
		for ($i=1;$i<=10;$i++)
		{
			$fn = '/recordings/updates/update' . $i . '.tgz';
			if (file_exists($fn))
			{
				lg('Performing update '.$i);
				// perform update (waits for completion)
				system("cp /usr/share/apresa/install_update.sh /usr/share/apresa/install_update_run.sh");
				system("/usr/share/apresa/install_update_run.sh ".escapeshellarg($fn)
				."  > /var/log/apresa/node_update".$i.".log"
				." 2> /var/log/apresa/node_update".$i.".errlog"
				);
				// clean up
				system("rm /usr/share/apresa/install_update_run.sh");
				system("rm ".escapeshellarg($fn));
				// updating starts orkaudio,
				// which is not what we want yet at this point
				system("service orkaudio stop");
			}
		}
	}

  // we should now have connection to the database
	// the other node might have changed the settings
	// so apply:
	lg('Applying settings');
	$timeout = 10;
	wait_signal_service('ns',$timeout);
  wait_signal_service('ss',$timeout);
  wait_signal_service('os',$timeout);
	wait_signal_service('bs',$timeout);
	// - restart of rec. comp. is not be needed,
	// because it is not yet started
	// - restart of network/system comp. should not be needed
	// because those that require it should be per node settings
	
	system('service apresaremote restart');
	system('service apresacrec restart');
	system('service apresanet restart');
}

function OldStop()
{
	system('service apresaremote stop');
	system('service apresacrec stop');
	system('service apresanet stop');
}


