<?php
define('CMD',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "encryption.php";
require_once CMD_WEB_RELPATH . "utility.php";
require_once CMD_WEB_RELPATH . "emailing.php";
require_once CMD_WEB_RELPATH . "tenancy.php";
require_once CMD_WEB_RELPATH . "certificates_general.php";

define('MSG_INF',0);
define('MSG_ERR',1);
define('MSG_WRN',2);


$FlagToBackup = 1;
$FlagHandled = 2;
$options = array();
$options['ShowCopy'] = false;
$options['DoSecondaryReset'] = false;
$options['CmdLineTarget'] = false;
$options['Mounting'] = true;
$options['UseCallsArchive'] = false;
$BackupDest = BACKUP_MOUNT;
for ($i=1;$i<$argc;$i++)
{
	$param = $argv[$i];
	if ($param=='debug')
	{
		$options['ShowCopy'] = true;
		$options['debug'] = true;
		define('DEBUG_LOGING',1);
	} else if ($param=='nodb')
  {
    $options['nodb'] = true;
  } else if ($param=='reset')
	{
		$options['DoSecondaryReset'] = true;
		$options['CmdLineTarget'] = true;
		$options['Mounting'] = false;
		$FlagToBackup = 16;
		$FlagHandled = 32;
	} else
	{
		$options['CmdLineTarget'] = true;
		$options['Mounting'] = false;
		$BackupDest = $param;
		$FlagToBackup = 16;
		$FlagHandled = 32;
	}
}


// Start-up
DoBackup();

function DoBackup()
{
  global $options, $BackupDest, $FlagToBackup, $FlagHandled, $dbintf;

	//echo "Backup\n";
	dbglg("dobackup start");
	$begintime = time();

  // Interface to database
  $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
  // so we can detect and report it
  $dbintf->errorAsException = true;
	
	if ($dbintf->nodePassive)
	{
		// only do at primary
		return;
	}

	if ($options['DoSecondaryReset'])
	{
		// no more: $dbintf->RemoveProcessingFlagForAll($FlagToBackup+$FlagHandled);
    $configs_set = array();
    $configs_set['Bck2IdHandled'] = '0';
    $configs_set['Bck2ProcVersion'] = '2';
    $dbintf->SetConfigsPart($configs_set,'b');
		echo "Reset secondary backup timeframe. Next secondary backup will again copy all.\n";
	  exit;
	}

	if ($options['CmdLineTarget']==false)
	{
	  // Load configuration
	  $configs = $dbintf->LoadConfigsPart('b');
	  if ($configs['BckAct']==0)
	  {
	    // No backup to do, so stop
	    exit;
	  }
	} else
  {
    $tmp_configs = $dbintf->GetConfigsWithPrefix('Bck2');
    $configs = array();
    foreach ($tmp_configs as $key=>$value)
    {
      $configs['Bck2' . $key] = $value;
    }
  }
  
	if ($configs['BckTargetType']==1)
	{
		// backup to internal or USB disk
		$BackupDest = $configs['BckPath'];
		$options['Mounting'] = false;
		// check if a mount point
		exec('mountpoint '.escapeshellarg($BackupDest),$dummy,$retval);
		if ($retval != 0)
		{
			$smsg = "Backup destination $BackupDest is not a mount point. Please check if the backup disk is inserted";
			$msg = "Backup failed: $smsg";
			lg($msg);
			setBackupStatus("1 $smsg");
			addToSayUniq(MSG_ERR,$msg);
			reportToAdmin($msg);
			exit;
		}
	}
	if ($configs['BckTargetType']==2)
	{
		// backup to any directory
		$BackupDest = $configs['BckPath'];
		$options['Mounting'] = false;
	}

	dbglg('Server:      '. str_ensure_defined($configs['BckServer'])  );
  //echo 'Share name:  '. $configs['BckShare' ]  . "\n";
  //echo 'Directory:   '. $configs['BckSubdir']  . "\n";

  //echo 'Source:      '. AUDIO_PATH . "\n";
  //echo 'Destination: '. $BackupDest . "\n";

  
	if ($options['CmdLineTarget']==false)
	{
    $options['UseCallsArchive'] = ($dbintf->GetConfigValue('UseCallsArchive')!=0);
	  if ($configs['BckRepeat']==1)
	  {
	    // weekly
	    // Check if correct day of week
	    if ($configs['BckDow']!=date("w"))
	    {
	      //echo 'No backup activity this day of the week.';
				dbglg('No backup activity this day of the week.');
	      exit;
	    }
	  }
	}

  $bckTypeDescr = 'regular';
  if ($options['CmdLineTarget']) $bckTypeDescr = 'CmdLineTarget';
  lg("Starting $bckTypeDescr backup to ".$BackupDest);
	$needunmount = false;
	$isDoneOK = false;
  try
  {
		if ($options['Mounting'])
		{
	  	// backup mount, + password
	  	lg('Mount');
			// will throw exception on error
	  	signal_required('bm');
			lg('Mount finished');
			$needunmount = true;
	 	}

    // things to do after mounting
    
    if ($configs['BckTargetType']==3)
    {
      if ($configs['BckSubdir']!='')
      {
        // we will backup into a subdirectory
        // create it if not already existing
        $BackupDest .= '/' . $configs['BckSubdir'];
        $backup_perm = fileperms($BackupDest);
        mkdir($BackupDest,$backup_perm,true);
      }
    }
    
    if (BackupDatabase($configs))
    {
      if ($configs['BckData']=='db')
      {
        lg("Only backup the database");
        // check if this is not a repeating backup
        if ($options['CmdLineTarget']==false)
        {
          if ($configs['BckRepeat']==0)
          {
            // notify web interface that backup has stopped
            $dbintf->SetConfigsPart(
              array('BckAct'=>0
              ,'BckProgress'=>0
              ,'BckCopied'=>0 // no recordings copied
              ,'BckCount'=>0 // no recordings to copy
              ),'b');
          }
        }
      }
      else
      {
        BackupMain($configs);
        BackupRetentionCheck($configs);
      }
    }
		
		// otherwise we would throw an exception
		$isDoneOK = true;
		
  } catch(Exception $e)
  {
    // maybe not allowed: fwrite(STDERR,$e->getMessage()."\n");
    // maybe not allowed: //syslog(LOG_ERR,$e->getMessage());
    lg('Backup failed: '.$e->getMessage());
		setBackupStatus("1 ".$e->getMessage());
    addToSayUniq(MSG_ERR,'Backup failed: '.$e->getMessage());
    reportToAdmin('Backup failed: '.$e->getMessage());
  }
  // this is done always, even when an exception occurs
  // that is important
  // it is no problem to unmount when not mounted
  try
  {
		if ($options['Mounting'] && $needunmount)
		{
      lg("sync");
      system("sync");
			lg("Unmount");
  		signal_required('bu');
  	} else
		{
			dbglg('No unmount needed');
		}
	} catch(Exception $e)
	{
    lg($e->getMessage());
 	}
  
	if ($options['CmdLineTarget']==false)
	{
    $dbintf->SetConfigsPart(array('BckFinished'=>1),'b');
    if ($isDoneOK)
    {
      $elapsed = time() - $begintime;
      setBackupStatus("0 Completed in " . $elapsed . ' s');
    }
  }
  lg("End of $bckTypeDescr backup");

  // end
}

function FileEntryCompare($a, $b)
{
	if ($a['fn'] == $b['fn']) return 0;
	if ($a['dt'] < $b['dt']) return 1;
	if ($a['dt'] > $b['dt']) return -1;
	// if date/time is equal sort on filename
	return ($a['fn'] < $b['fn']) ? 1 : -1;
}

function GetDirFileList(&$arr,$dir)
{
	if ($handle = @opendir($dir))
	{
		while (false !== ($fn = readdir($handle)))
		{
			if ($fn=='.' || $fn=='..' || is_dir($dir.$fn)==1) continue;
			$info = stat($dir . $fn);
			$arr[] = array( 'fn'=>$fn
				,'sz'=>$info['size']
				,'dt'=>$info['mtime']
				,'dir'=>$dir
				 );
		}
	}
  closedir($handle);
	usort($arr, "FileEntryCompare");
	return $arr;
}

function BackupDatabase($configs)
{
  global $options, $BackupDest, $FlagToBackup, $FlagHandled, $dbintf;
  if (isset($options['nodb']) && $options['nodb']) return true;

  $BckDbDays = (int) $configs['BckDbDays'];
  if ($BckDbDays >= 1)
  {
    lg('Database days to keep: ' . $BckDbDays);
    // find all database files
    $now = time();
    $threshold = strtotime('-' . $BckDbDays . ' days', $now);
    lg( 'Delete older than ' . date('r',$threshold) );
    $list = array();
    GetDirFileList($list,$BackupDest . '/');
    foreach ($list as $item)
    {
      $fn = $item['fn'];
      //lg( $fn );
      if ( substr($fn,0,8)=='apresadb'
      && substr($fn,16)=='.sql.gz' )
      {
        $datestr = substr($fn,8,8);
        $y = substr($datestr,0,4);
        $m = substr($datestr,4,2);
        $d = substr($datestr,6,2);
        $time = mktime(0,0,0,$m,$d,$y);
        //lg( date('r',$time) );
        if ( $time < $threshold )
        {
          // database is older than the number of days, so needs to be deleted

          $delfn = $BackupDest . '/' . $fn;
          lg('to delete: ' . $delfn);
          unlink( $delfn );

          LogAction("Deleted backup database " . $fn. " older than " . $BckDbDays . " days");
        }
      }
    }
  } else
  {
    lg('No limit on database retention to apply');
  }

  lg('Make database dump');
  //system("pg_dump apresa | gzip > /var/apresa/data/apresadb.sql.gz");
  // store on same partition for safety
  // so that all data is on one partition
  system("pg_dump apresa | gzip > ".AUDIO_PATH."/apresadb.sql.gz");

  // Start with copying the database
  lg('Copy database file');

	//lg("Copy database");
  if (DB_TYPE=='sqlite3')
  {
    $dbintf->close();
    if (copy(CMD_WEB_RELPATH . DATA_PATH . '/vicalls.db'
    , $BackupDest . '/vicalls.db')===false)
    {
      $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
      onerror("Failed to copy database file");
    } else
    {
      $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
    }
  }
  if (DB_TYPE=='postgresql')
  {
		$dstfn = 'apresadb' . date('Ymd') . '.sql.gz';
    if (copy(AUDIO_PATH . '/apresadb.sql.gz'
    , $BackupDest . '/' . $dstfn)===false)
    {
      onerror("Failed to copy database file");
    }
  }
  
  return true;
}

function TryTempWrite($filename, $size)
{
	$h = fopen($filename,'w');
	if ($h===false)
	{
		return false;
	}
	$blksz = 64 * 1024;
	while ($size > 0)
	{
		if ($size > $blksz)
		{
			$towrite = $blksz;
		} else
		{
			$towrite = $size;
		}
		$written = fwrite( $h , str_repeat(' ',$towrite) );
		if ($written != $towrite)
		{
			fclose($h);
			unlink($filename);
			return false;
		}
		$size -= $written;
	}
	if (fclose($h)===false)
	{
		unlink($filename);
		return false;
	}
	unlink($filename);
	return true;
}

// tenantid will be null if stats are not to be updated,
// even when the call can have a tenant
function TnPerformCopy($tenantid,$src,$dst,$srcMissingExpected,$cntPlus)
{
  $copyok = PerformCopy($src,$dst,$srcMissingExpected);
  if ($copyok)
  {
    if (isset($tenantid))
    {
      UpdateTnBackupDiskUsage($tenantid,filesize($src),$cntPlus);
    }
  }
  return $copyok;
}

function PerformCopy($src,$dst,$srcMissingExpected)
{
	// we need to know where is the problem,
	// at the src or at the dst
	// if at the src, we continue with other files
	// if at the dst, we stop the process (backup dest has a problem)
	if (file_exists($src))
	{
		if (copy($src,$dst)===false)
		{
			// copy failed, now what to do?
			if ($srcMissingExpected)
			{
				// active calls not properly finished
				// ignore copying error of these
			} else 
			{
				// check if dst is writable
				// write same size, to mimick the same problems
				// If the dst file already exist, we cannot check if backup is writable.
				// Then we assume it is not, for safety reasons, and stop the process.
				// This can happen if unlink failed in a previous attempt.
				if (file_exists($dst) || TryTempWrite($dst,filesize($src))==false)
				{
					// it is not, so failure is at the destination
					// stop whole backup process
					onerror("Backup failed to write to $dst");
				} else
				{
					// failure is at the source
					// we will continue
					lg("Backup failed to copy file $src");
				}
			}
		} else {
			// copy worked
			return true;
		}
	} else
	{
		// source file does not exist
		if ($srcMissingExpected)
		{
			// expected to not exist, do not even log
		} else {
			lg("Backup skipping file $src because it does not exist");
		}
	}
	return false;
}

function BackupRetentionCheck($configs)
{
  global $dbintf;
  $DelBckDelMain = (int) $dbintf->GetConfigValue('DelBckDelMain');
  
  $pols = $dbintf->GetRetentionPolicies();
  array_unshift($pols, GetDefaultRetentionPolicy($configs));
  
  foreach ($pols as $pol)
  {
    $PolNr = $pol['nr'];
    // disable filtering for speed if only one policy
    if (count($pols)==1) $PolNr = false;
    
    $BckRecDays = $pol['bckrecdays'];
    if ($BckRecDays>0)
    {
      DeleteOlder($BckRecDays
        ,false // no tenant
        ,true // backup
        ,true // force delete meta data
        ,$PolNr
        );
      if ($DelBckDelMain!=0)
      {
        DeleteOlder($BckRecDays
          ,false // no tenant
          ,false // main storage
          ,true // force delete meta data
          ,$PolNr
          );
      }
    }
		$ScreenRecDaysKeep = $pol['bckscreenrecdays'];
		DeleteOlderDataType( $ScreenRecDaysKeep , 'screenrec'
			, false // no tenant
			, true // backup
      , $PolNr
			);
  }

  // For tenants, we do not apply retention policy
  $all_configs = $dbintf->LoadConfigs();
	$tenants = $dbintf->GetTenantList();
	foreach ($tenants as $tenant)
	{
		$prefix = 'TN' . $tenant['id'] . '_';
		$RecBckDaysKeep = int_ensure_defined($all_configs[$prefix . 'RecBckDaysKeep']);
		DeleteOlder($RecBckDaysKeep
      , $tenant // a tenant
      , true // backup storage
      , true // force delete meta data
      );
    if ($DelBckDelMain!=0)
    {
      DeleteOlder($RecBckDaysKeep
        , $tenant // a tenant
        , false // main storage
        , true // force delete meta data
        );
    }
		$ScreenRecDaysKeep = int_ensure_defined($all_configs[$prefix . 'ScreenRecBckDays']);
		DeleteOlderDataType( $ScreenRecDaysKeep , 'screenrec'
			, $tenant
			, true // backup
			);
	}
  
}

function BackupMain($configs)
{
  global $options, $BackupDest, $FlagToBackup, $FlagHandled, $dbintf;
  
  lg("Preparing to perform backup of recordings");

  // settings
  // If the chunck are too small, SQLite commits makes it slow
  $chuck_size = 64;

  $cfg = array();
  // IdHandled = up to which ID everything is backed up (handled)
  // there could be higher IDs that are already handled, but no ID equal or lower
  $IdHandled = 0;
  // IdSeen = up to which ID everything is included in count (seen)
  // similarly there could be higher IDs included in count,
  // but these would be marked in the database with a flag so we can skip them during count update
  $IdSeen = 0;
  
  if ($options['CmdLineTarget'])
  {
    // not needed for cmd backup
    $IdSeen = -1;

    $BckProcVersion = int_ensure_defined($configs['Bck2ProcVersion']);
    // new start is handled somewhere else (see DoSecondaryReset)
    $is_new_start = false;

    // no longer: $dbintf->SetProcessingFlagOnForAll($FlagToBackup);
    if (isset($configs['Bck2IdHandled']))
    {
      $IdHandled = $configs['Bck2IdHandled'];
    } else
    {
      // handling upgrade situation
      lg('upgrade convert from other procedure (secondary)');
      $res = $dbintf->GetBackupFrame($FlagToBackup,$FlagHandled,$cfg);
      $IdHandled = $res['IdHandled'];
      // store in database at once
      $configs_set = array();
      $configs_set['Bck2IdHandled'] = $IdHandled;
      $dbintf->SetConfigsPart($configs_set,'b');
      lg("upgrade result: IdHandled: $IdHandled IdSeen: $IdSeen");
    }

    if ($BckProcVersion==0)
    {
      $cfg['FlagHandled']  = $FlagHandled;
      $cfg['FlagToBackup'] = $FlagToBackup;
    }
    
    // UpdateBackupRange is not needed
    
	} else
	{
    // settings dont apply to command line, so do here
    $from_ts = $configs['BckFrom'];
    $to_ts = $configs['BckTo'];
    if ($configs['BckRepeat']>0)
    {
      $to_ts = 0; // 0 means to end
    }
    if ($to_ts!=0)
    {
      // include last day
      $to_ts += 24*60*60;
    }
    $cfg['from_ts'] = $from_ts;
    $cfg['to_ts'  ] = $to_ts  ;

    $BckProcVersion = int_ensure_defined($configs['BckProcVersion']);

    $is_new_start = false;
	  if ($configs['BckNew']==1)
	  {
	    $configs_set = array();
	    $configs_set['BckNew'] = 0;
	    $dbintf->SetConfigsPart($configs_set,'b');

			// no longer: lg('RemoveProcessingFlagForAll');
	    // no longer: $dbintf->RemoveProcessingFlagForAll(1+2);
      $IdSeen = 0;
      $IdHandled = 0;
      $BckProcVersion = 2;

	    $is_new_start = true;
	  } else
    {
      // not a new start
      // normally we should have BckIdSeen
      if (isset($configs['BckIdSeen']) && isset($configs['BckIdHandled']))
      {
        $IdSeen = $configs['BckIdSeen'];
        $IdHandled = $configs['BckIdHandled'];
      } else
      {
        // handling upgrade situation
        lg('upgrade convert from other procedure');
        $res = $dbintf->GetBackupFrame($FlagToBackup,$FlagHandled,$cfg);
        $IdSeen = $res['IdSeen'];
        $IdHandled = $res['IdHandled'];
        // store in database at once
        $configs_set = array();
        $configs_set['BckIdSeen'] = $IdSeen;
        $configs_set['BckIdHandled'] = $IdHandled;
        $dbintf->SetConfigsPart($configs_set,'b');
        lg("upgrade result: IdHandled: $IdHandled IdSeen: $IdSeen");
      }
    }
    
	  $BckProgress = $configs['BckProgress'];
	  $BckCopied = (int) $configs['BckCopied'];
    if ($BckProcVersion==0)
    {
      $cfg['FlagHandled']  = $FlagHandled;
      $cfg['FlagToBackup'] = $FlagToBackup;
    }
    
	  if ($is_new_start || $configs['BckRepeat']>0)
	  {
	    // determine which records must be backuped
	    $configs_set = array();
      $res = $dbintf->UpdateBackupRange($IdSeen,$cfg);
	    $cnt = $res['Cnt'];
      $IdSeen = $res['IdSeen'];
      $configs_set['BckIdSeen'] = $IdSeen;
      // dont handle calls added after we calculated backup range
      // to avoid going beyond 100 percent
      // those new calls will be handled in the next round
      $cfg['IdMax'] = $IdSeen;
      if ($cnt>0)
      {
        lg("Newly seen to backup: $cnt");
      }
	    if ($is_new_start==false)
	    {
	      // these are extra to do in repeating backup
	      $cnt += $configs['BckCount'];
	    } else {
        // it is a new start
	      $configs_set['BckProgress'] = 0;
	      $configs_set['BckCopied'] = 0;
        $configs_set['BckProcVersion'] = 2;
	      $BckProgress = 0;
	      $BckCopied = 0;
	    }
	    $configs_set['BckCount'] = $cnt;
      $dbintf->SetConfigsPart($configs_set,'b');
	  }
	}

  
  lg("IdHandled: $IdHandled IdSeen: $IdSeen v: $BckProcVersion");

  $backup_perm = fileperms($BackupDest);
  $prev_first_id = false;

	lg("Running backup...");

  // Doing in chunck, not all at once
  // because otherwise maybe too large a list for memory
  while (true)
  {

    if ($options['CmdLineTarget']==false)
    {
	    // reload config, maybe stop command is given
	    $configs = $dbintf->LoadConfigsPart('b');
	    if ($configs['BckAct']==0)
	    {
	      //echo 'Stopped in configuration';
	      break;
	    }
	  }

    // Get a list of files that need backup
    // Do in chuncks
		//lg("GetToBackupList");
    $testtime1 = microtime(true);
    $calls = $dbintf->GetToHandleList($IdHandled,$chuck_size,$cfg);
    // old: $calls = $dbintf->GetToBackupList($chuck_size,$FlagToBackup,$FlagHandled);
    $testtime2 = microtime(true);
    $db_time = (int) (($testtime2-$testtime1)*1000);
    if ($db_time>500)
    {
      lg( 'Get backup list query took '.$db_time. ' ms' );
    }
    $cnt_calls = count($calls);
		//lg("ToBackupList cnt.: ".$cnt_calls);
    if ($cnt_calls==0) {
      // ready
      // check if this is not a repeating backup
      if ($options['CmdLineTarget']==false)
      {
	      if ($configs['BckRepeat']==0)
	      {
	        // notify web interface that backup has stopped
	        $dbintf->SetConfigsPart(array('BckAct'=>0),'b');
	      }
	    }
      break;
    }
    $first_id = $calls[0]['id'];
    if ($first_id==$prev_first_id)
    {
      onerror("Double ID in backup");
    }
    $prev_first_id = $first_id;
    
    // list of items already in backup
    $alreadyInBck = array();
    if ($options['UseCallsArchive']==true)
    {
      $ids = array();
      foreach ($calls as &$call)
      {
        $ids[] = $call['id'];
      }
      $alreadyInBck = $dbintf->GetInArchiveList($ids);
      dbglg( 'Already in backup: '. count($alreadyInBck) );
    }

    foreach ($calls as &$call)
    {
      $dirfn = $call['directory'] . '/' . $call['filename'];
      $src = AUDIO_PATH . '/' . $dirfn;
      $dst = $BackupDest . '/' . $dirfn;
      // no echo "Copying " . $call['filename'] . "\n";
      $dstdir = $BackupDest . '/' . $call['directory'];
      if (is_dir($dstdir)===false)
      {
        if (mkdir($dstdir,$backup_perm,true)===false)
        {
          onerror("Failed to make directory $dstdir");
        }
      }
			$srcMissingExpected = (($call['flags']&1)!=0);
      $isRefresh = false;
      $tenantStats = $call['tenantid'];
      if (isset($alreadyInBck[ $call['id'] ]))
      {
        $tenantStats = null;
      }
      if ($options['CmdLineTarget'])
      {
        $tenantStats = null;
      }
      $anyCopied = false;
			if (TnPerformCopy($tenantStats,$src,$dst,$srcMissingExpected,0))
			{
				if ($options['ShowCopy'])
				{
					lg("Copied: ".$src);
				}
				$BckCopied ++;
        $anyCopied = true;
			}
      if (HasScreenRecording($call))
      {
				$dirfn = $call['directory'] . '/' . GetScreenFileName($call);
				$src = AUDIO_PATH   . '/' . $dirfn;
				$dst = $BackupDest . '/' . $dirfn;
				if (TnPerformCopy($tenantStats,$src,$dst,$srcMissingExpected,0))
        {
          $anyCopied = true;
        }
			} // if has screen recording
      if ($anyCopied)
      {
        if (isset($tenantStats))
        {
          UpdateTnBackupDiskUsage($tenantStats,0,1);
        }
      }
      $call['seen'] = 1;
      $BckProgress ++;
      $IdHandled = $call['id'];
    }

    $seen_id_list = array();
    foreach ($calls as &$call)
    {
      if ($call['seen']==1)
      {
        $seen_id_list[] = $call['id'];
      }
    }
    // not anymore: $dbintf->SetProcessingFlagOnForIDs($seen_id_list,$FlagHandled);
    if ($options['UseCallsArchive']==true)
    {
      $dbintf->CopyCallsToArchive($seen_id_list);
    }
    if ($options['CmdLineTarget']==false)
    {
    	$dbintf->SetConfigsPart(array
        ('BckProgress'=>$BckProgress
        ,'BckCopied'=>$BckCopied
        ,'BckIdHandled'=>$IdHandled
        ),'b');
    } else
    {
    	$dbintf->SetConfigsPart(array
        ('Bck2IdHandled'=>$IdHandled
        ),'b');
    }

  }
  
  lg("Status: $BckProgress ($BckCopied) / " . $configs['BckCount'] . " , IdHandled: $IdHandled IdSeen: $IdSeen" );
  
  // Check if there are additional files to copy
  $toCopy = array();
  // Telsys
  $configsT = $dbintf->LoadConfigsPart('t');
  $maxRules = 10;
  $basedir = '/recordings/telsys/audio';
  $targetDir = 'telsys/audio';
  for ($i=1;$i<=$maxRules;$i++)
  {
    $key = 'TelsysR' . $i . 'Act';
    $act = $configsT[$key];
    $key = 'TelsysR' . $i . 'Cnd';
    $cnd = $configsT[$key];
    // debug: lg("cnd: $cnd");
    if ($cnd=='') continue;
    // debug: lg("act for $key = $act");
    if ($act=='notify')
    {
      $basefn = $basedir . '/r' . $i;
      $fn = $basefn . '.slin';
      $toCopy[] = array('src'=>$fn,'dstdir'=>$targetDir);
      $fn = $basefn . '.wav';
      $toCopy[] = array('src'=>$fn,'dstdir'=>$targetDir);
      // debug: lg($fn);
    }
    if ($cnd=='always')
    {
      // no further rules
      break;
    }
  }

  if($configs['BckCertificates']==1)
  {
    $targetDir = 'certificates';

    $certificates = $dbintf->GetCertificateFiles();
    foreach($certificates as $certificate)
    {
      $toCopy[] = array('src' => CERTDIR . $certificate['cert_location'], 'dstdir' => $targetDir);
      
      if(strlen(str_ensure_defined($certificate['key_location'])) > 0)
      {
        $toCopy[] = array('src' => CERTDIR . $certificate['key_location'], 'dstdir' => $targetDir);
      }

      if(strlen(str_ensure_defined($certificate['intermediate_location'])) > 0)
      {
        $toCopy[] = array('src' => CERTDIR . $certificate['intermediate_location'], 'dstdir' => $targetDir);
      }
    }

    $requests = $dbintf->GetCertificateRequestFiles();
    foreach($requests as $request)
    {
      $toCopy[] = array('src' => CERTDIR . $request['request_location'], 'dstdir' => $targetDir);
      $toCopy[] = array('src' => CERTDIR . $request['key_location'], 'dstdir' => $targetDir);
    }  
  } 

  foreach ($toCopy as $item)
  {
    $src = $item['src'];
    $dstdir = $BackupDest . '/' . $item['dstdir'];
    if (file_exists($src))
    {
      lg('to copy: ' . $src . ' to ' . $dstdir);
      $pi = pathinfo($src);
      // ensure directory
      if (is_dir($dstdir)===false)
      {
        if (mkdir($dstdir,$backup_perm,true)===false)
        {
          onerror("Failed to make directory $dstdir");
        }
      }
      $dst = $dstdir . '/' . $pi['filename'] . '.' . $pi['extension'];
      // debug: lg('dst: ' . $dst);
      PerformCopy($src,$dst,false);
    }
  }
  lg('BackupMain finished');
}

function signal_required($msg)
{
  //echo 'Signaling ' . substr($msg,0,2) . "\n";
  $reply = 1;
	$cmd_timeout = 20;
  $err = signal_service($msg,$reply,$cmd_timeout);
  if ($err!='')
  {
    onerror('Socket error: '.$err);
  }
  $msg_firstpart = explode(' ',$msg);
  $msg_firstpart = $msg_firstpart[0];

	$where = $msg_firstpart;
	if ($where=='bm')
	{
		$where = 'mounting backup directory';
	}
	// more user friendly
  if ($reply=='OK')
  {
    // OK
  } else if ($reply[0]=='E')
  {
		onerror($where . ' failed: '.$reply);
  } else
  {
    onerror($where . ' failed, unexpected reply: '.$reply.' LEN: '.strlen($reply));
  }

}

function onerror($msg)
{
  throw new Exception($msg);
}

function setBackupStatus($status)
{
	file_put_contents('/var/apresa/data/backup_status',$status . "\n");
}

function reportToAdmin($msg)
{
	global $dbintf;
  $dat = array();
  $configs_n = $dbintf->LoadConfigsPart('n');

	if ($configs_n['SMTPhost']=='')
	{
  	lg('E-mail is not configured');
		return false;
	}
	if ($configs_n['EmAdmin']=='')
	{
  	lg('No admin email address is specified');
		return false;
	}
	$dat['To'] = array( $configs_n['EmAdmin'] , $configs_n['EmAdmin2'] );
  $dat['Subject'] = GetEmailSubjPrefix($dbintf) . 'Backup FAILED';
  $body = "";
  $body .= "\n";
  $body .= $msg . "\n";
  $dat['Body'] = $body;
	return SendEmailData($configs_n,$dat);
}

function dbglg($t)
{
	global $options;
	if (isset($options['debug']) && $options['debug']==true)
	{
		lg($t);
	}
}

?>