<?php
/*
  Script for use by underlying system to report problems etc.
*/
define('CMD',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "emailing.php";
require_once CMD_WEB_RELPATH . "datafile.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "actions.php";

// Interface to database
$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);

// Load configuration
LoadBasicSettings();
$configs = array_merge($configs,$dbintf->LoadConfigsPart('n'));

if (isset($argv[1])==false)
{
  // No file name specified
  lg('doemail.php no filename specified.');
  exit(2);
}

// debug: echo "ARGV[1] " . $argv[1] . "\n";

$arg1 = $argv[1];

if ($arg1[0]=='-')
{
	// switch
	// [1] type switch
	// if -recid
	// [2] recid
	// [3] user acc-id (for safety not email address, because user filled in)
	if ($argv[1]=='-recid')
	{
		$recid = $argv[2];
		// email address
		$accid = $argv[3];

		lg("Email recording $recid to user $accid");
		$userinfo = $dbintf->GetUserInfoOnAccId($accid);
		if ($userinfo==false)
		{
			lg("Invalid accid");
			return;
		}
		$addressee = $userinfo["email"];
		if ($addressee=='')
		{
			lg("No email address specified for user " . $userinfo['usr']);
			return;
		}

		//echo 'Name: ' . $userinfo['name'] . "\n";
		//echo "Email: " . $addressee ."\n";

		$dbintf->SetCurrentUserId($userinfo['id']);
		
		if ($dbintf->actuinfo['enabled']==0)
		{
			lg("User account is disabled for user " . $userinfo['usr']);
			return;
		}

		EmailRecordings( array($recid), $addressee );
	}
	if ($argv[1]=='-loud')
	{
		// argv[2] identifies the recording (remote id)
		LoudVoiceDetected($argv[2]);
		exit(0);
	}
} else
{
	// normal (data file name)
	$filename = $argv[1];
	lg('doemail.php filename: '.$filename);
	$dat = ReadDataFile($filename);

	if (DeleteFn($filename)==false)
	{
	  lg('Failed to delete: '.$filename);
	}

	// add defaults:
	if (isset($dat['To'])==false)
	{
	  $dat['To'] = array( $configs['EmAdmin'] , $configs['EmAdmin2'] );
	}
	$res = SendEmailData($configs,$dat);
	if ($res==1 || $res==2)
	{
	  // OK
	  exit(0);
	} else
	{
	  exit(1);
	}
}

function LoudVoiceDetected($remoteid)
{
	global $dbintf, $lng, $configs;

	// get call information
  $info = $dbintf->GetActiveCallInfoRM($remoteid);
  // call found?
  if (isset($info['id'])==false) return;

	$dat = array();
  $dat['To'] = array( $configs['EmAdmin'] , $configs['EmAdmin2'] );
	if (isset($configs['EmailAddrLoud']) && $configs['EmailAddrLoud']!='')
	{
		$dat['To'] = $configs['EmailAddrLoud'];
	}
	
  $dat['Subject'] = GetEmailSubjPrefix($dbintf) . $lng->Get('Options.LoudVoiceDetect');

	$body = '';
	$body .= $lng->Get('Options.LoudVoiceDetect') . "\n";
  $body .= "\n";

  $body .= $lng->Get("Cols.startdate") . ": ".date($configs['datetimef'],$info['startdate'])."\n";
  $body .= $lng->Get("Cols.direction") . ": ".formatDirection($info['direction']) . "\n";
  $body .= $lng->Get("Cols.rname") . ": " . $dbintf->GetContactName($info['remotecontact']) . "\n";
  $body .= $lng->Get("Cols.remoteID") . ": ".$info['remote'] . "\n";
  $lname = $dbintf->GetContactName($info['localcontact']);
  $body .= $lng->Get("Cols.lname") . ": " . $lname . "\n";
  $body .= $lng->Get("Cols.localID") . ": " .$info['local'] . "\n";
  if ($info['linenr']!=0)
  {
   	$body .= $lng->Get("Cols.linenr") . ": " .$info['linenr'] . "\n";
 	}
 	if ($lname!='')
 	{
   	$dat['Subject'] .= ' (' . $lname . ')';
	}

  $dat['Body'] = $body;

	if (SendEmailData($configs,$dat)==1)
	{
	  // OK
	  exit(0);
	} else
	{
	  exit(1);
	}

}


?>