<?php
error_reporting(E_ALL ^ E_NOTICE);
define('CMD',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "encryption.php";
require_once CMD_WEB_RELPATH . "utility.php";
require_once CMD_WEB_RELPATH . "actions.php";
require_once CMD_WEB_RELPATH . "tenancy.php";
require_once CMD_WEB_RELPATH . "certificates_general.php";

BasicConstants();

$debug_mode = false;
for ($i=1;$i<$argc;$i++)
{
	if ($argv[$i]=='debug')
	{
		$debug_mode = true;
	}
}

dbglog('Starting restore backup');
DoRestore();
dbglog('End restore backup');

// --------------

function DoRestore()
{
	global $debug_mode, $options;

  // Interface to database
  $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);

  // Load configuration
  $configs = $dbintf->LoadConfigsPart('rb');
  // ready with existing database
  // do this to avoid a lock on the database
  $dbintf = null;

	if ($configs['RBckAct']==0)
	{
	  // No restore backup to do, so stop
	  dbglog('No restore backup to do');
	  return;
	}

  $WinPwd = decrypt($configs['RWinPwd']);
  
  $options = array();
  $options['NeedMounting'] = false;
  $options['NeedUnmounting'] = false;

	if ($configs['RBckRestoreMethod']==0) dbglog('Restore Calls!');
	else if ($configs['RBckRestoreMethod']==1)
  {
    dbglog('Full Restore!');
    stop_services();
  } 
  else if ($configs['RBckRestoreMethod']==2)
  {
    dbglog('Restore Database!');
    stop_services();
  }
  else
  {
    dbglog("Unknown restore method");
    return;
  }
  dbglog('Restoring backup from: (type='.$configs['RBckSrcType'].')');
  $RBckSrcType = (int) $configs['RBckSrcType'];
  if ($RBckSrcType==0) // Network
  {
    dbglog('Restore from SMB network drive');
	  dbglog('Server: ' . $configs['RBckServer']);
	  dbglog('IP:     ' . $configs['RBckServerIP']);
	  dbglog('Share:  ' . $configs['RBckShare']);
	  dbglog('Dir.:   ' . $configs['RBckSubdir']);
	  dbglog('File:   ' . $configs['RBckFile']);
  	$options['BackupDir'] = BACKUP_MOUNT;
    $options['NeedMounting'] = true;
  } else if ($RBckSrcType==3) // local USB disk
  {
    dbglog('Restore from local disk');
    dbglog('Disk: ' . $configs['RBckDisk']);
    $options['BackupDir'] = BACKUP_MOUNT;
    $options['NeedMounting'] = true;
	} else if ($RBckSrcType==1 || $RBckSrcType==2) // directory
	{
    dbglog('Restore from directory');
	  dbglog('Path: ' . $configs['RBckPath']);
	  $options['BackupDir'] = $configs['RBckPath'];
  } else if ($RBckSrcType==4) // SFTP
  {
    dbglog('Restore from SFTP server');
	  dbglog('Server: ' . $configs['RBckServer']);
    $options['BackupDir'] = BACKUP_MOUNT;
    $options['NeedMounting'] = true;
	} else
  {
    dbglog("Unknown source type");
    return;
  }

	if ($debug_mode)
	{
		dbglog('Before mount:');
		passthru('ls -altr ' . $options['BackupDir']);
	}

  try
  {
  	if ($options['NeedMounting'])
  	{
			dbglog('Mounting remote backup source');
			signal_required('restore mount '.$WinPwd);
			dbglog('Mounting finished');
      $options['NeedUnmounting'] = true;
		}

    // things to do after mounting
    if ($configs['RBckSrcType']==3)
    {
      if ($configs['RBckSubdir']!='')
      {
        // we will restore backup from a subdirectory
        // for mount type 3
        $options['BackupDir'] .= '/' . $configs['RBckSubdir'];
      }
    }

		if ($debug_mode)
		{
			dbglog('After mount:');
			passthru('ls -altr ' . $options['BackupDir']);
		}

		if ($configs['RBckAct']>=2)
		{
			file_put_contents('/var/apresa/data/busy','restore');
			RestoreDb($configs);
			$FullRestore = ($configs['RBckRestoreMethod']==1);
			ClearAudioAndPrepareRestore( $FullRestore );

      $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
			$restored_configs = $dbintf->LoadConfigs();

			$newconfigs = array();
      /*
      // Auto-delete is now more complicated (separate policies)
      // so disabling all of that would destroy information
      // about what the policies are.
      // The disabling of only this (HdDaysKeep) is now partial,
      // and the message might be misleading, so do not do anymore.
			if (isset($restored_configs['HdDaysKeep']))
			{
			  $HdDaysKeep = (int) $restored_configs['HdDaysKeep'];
			  if ($HdDaysKeep>0)
			  {
        	$newconfigs['HdDaysKeep'] = -1;
        	addToSay(MSG_WRN, "~Backup.RestoreAutodelInfo");
				}
			}
      */

			if (isset($restored_configs['BckAct']) && isset($restored_configs['BckRepeat']))
			{
				if ($restored_configs['BckAct']=='1' && $restored_configs['BckRepeat']=='0')
				{
					// back-up must have been completed,
					// otherwise we cannot restore it.
					$newconfigs['BckAct'] = '0';
				}
			}

			if (isset($restored_configs['ApresaSerial']))
			{
				$NewSerial = $restored_configs['ApresaSerial'];
				if (strlen($NewSerial)>1)
				{
					// Write new serial number
					file_put_contents('/var/apresa/conf/serial', $NewSerial . "\n");
				}
			}

			if (count($newconfigs)!=0)
			{
      	$dbintf->SetConfigsPart($newconfigs,'s');
			}

			$dbintf = null;
			
			if ($configs['RBckRestoreMethod']==1		//Full restore!!
      || $configs['RBckRestoreMethod']==2) // or database restore
			{
        // let the handler process the new settings
        // save with fresh orkaudio xml
        // o = recording engine, f = save fresh configuration
        signal_required('of');
        signal_required('os');
        signal_required('ns');
        signal_required('ss');
        signal_required('spin apply');
        signal_required('telsys apply');

        start_services();

        addToSay(MSG_INF, "~Backup.RestoredDb");
				addToSay(MSG_WRN, "~Backup.VerifyRestore");
			}
  		
			unlink('/var/apresa/data/busy');
		}

    if ($configs['RBckRestoreMethod']==0 // restore calls
    || $configs['RBckRestoreMethod']==1) // full restore
    {
      dbglog('Restore audio');		
      RestoreAudio($configs);
      dbglog('Restore audio finished');		
    }
    
    if ($configs['RBckRestoreMethod']==1)		//Full restore!!
		{
      dbglog('Restore additional files');
      RestoreAdditionalFiles();
      FixCertificates();
      dbglog('Restore additional files finished');
    }

    if ($configs['RBckRestoreMethod']==0 // full restore
    || $configs['RBckRestoreMethod']==1) // database restore
    {
      addToSay(MSG_INF, "~Backup.RecordingsRestored");
    }
		
  } catch(Exception $e)
  {
    addToSay(MSG_ERR,"~Backup.BackupRestored");	//Backup restore failed!!
    dbglog($e->getMessage());
  }
  unlink('/var/apresa/data/busy');

 	if ($options['NeedUnmounting'])
 	{
  	signal_required('restore unmount');
  }

	if ($debug_mode)
	{
		dbglog('After unmount:');
		passthru('ls -altr ' . $options['BackupDir']);
	}
}

function RestoreDb(&$configs)
{
	global $options, $dbintf;
	// backup directory is mounted now

	$dbfile = AUDIO_PATH . '/' . $configs['RBckFile'];

	$dbsrcfile = $options['BackupDir'] . '/' . $configs['RBckFile'];
	if (@copy($dbsrcfile , $dbfile)==false)
	{
		onerror('Could not copy database file from ' . $dbsrcfile);
	}

	// determine current psql encoding
	// since not easy to change (utf8 might not even be installed)
	$retval = 0;
	$dbenc = '';
	$dummy_output = null;
	exec('psql -P pager=off -l | grep LATIN1',$dummy_output,$retval);
	if ($retval==0)
	{
		$dbenc = 'LATIN1';
	}
	exec('psql -P pager=off -l | grep UTF8',$dummy_output,$retval);
	if ($retval==0)
	{
		$dbenc = 'UTF8';
	}
	dbglog('Database encoding is ' . $dbenc);
	// extract SQL
	$sqlfile = AUDIO_PATH . '/' . 'restoring.sql';
  $sqlfile_ori = '';
	exec('gunzip -c ' . escapeshellarg($dbfile) . ' > ' . escapeshellarg($sqlfile));
	// correct SQL for encoding (if we know it)
	if ($dbenc!='')
	{
		$cmd = 'sed -i "s/SET client_encoding = \'.*\'/SET client_encoding = \''.$dbenc.'\'/" ' . escapeshellarg($sqlfile);
		exec($cmd);
    
    if ($dbenc=='UTF8')
    {
      // remove any invalid characters
      // otherwise it will fail badly
      $sqlfile_ori = $sqlfile;
      $sqlfile = AUDIO_PATH . '/' . 'restoring2.sql';
      // -c ignore invalid in source
      $cmd = 'iconv -c -f utf-8 -t utf-8 ' . escapeshellarg($sqlfile_ori) . ' > ' . escapeshellarg($sqlfile);
      exec($cmd);
    }
	}
	
	// We now have the database backup that must be restored, on our local disk.
	// The next step is to restore this into the database.
	// (This might close our existing database connection, better is it to reconnect)
	if ($configs['RBckRestoreMethod']==1		//Full Restore!!! Delete DB first!
  || $configs['RBckRestoreMethod']==2)  // database restore
	{
		// empty the current database
		// the next line delete the DB
		// not: exec('sudo -u postgres psql postgres -f /var/apresa/data/dropcreate.sql');
		// but better is keep the database, and only delete all tables:
		exec('psql apresa -c "DROP SCHEMA public CASCADE"');
		exec('psql apresa -c "CREATE SCHEMA public"');
    exec('psql apresa -c "GRANT ALL ON SCHEMA public TO PUBLIC"');
		// import data from backup
		exec('psql apresa < '.escapeshellarg($sqlfile));

		// The database is now restored.
		// Still we have to copy the files (recordings).
		
		// we might need to upgrade the database
    exec('/usr/bin/apresa_upgrade root');
		exec('sudo -u www-data /usr/bin/apresa_upgrade');
	}
	else				//Merge the Calls tables!
	{
		lg("Drop old backup database.");
		exec('sudo -u postgres psql postgres -c "DROP DATABASE apresabu;"');
		lg("Create new backup database.");
		exec('sudo -u postgres psql postgres -c "CREATE DATABASE apresabu OWNER postgres;"');
		lg("Import data from backup.");
		exec('psql apresabu < '.escapeshellarg($sqlfile));
		
		$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
		lg("Just before the merge.");
    $RBckTo = int_ensure_defined($configs['RBckTo']);
    $RBckFrom = int_ensure_defined($configs['RBckFrom']);
    // add one day (move to end of day)
    if ($RBckTo != 0) $RBckTo += 60 * 60 * 24;
		$number = $dbintf->MergeCallsTables('RestoredNewCallCB',$RBckFrom,$RBckTo);
		lg("Number of records restored: ".$number);
    addToSayRaw(MSG_INF, "<%Backup.RestoreCalls%>: <%Main.CapRecordings%>: " . $number);
		$dbintf = null;
		
		//Clean up the mess.
		exec('sudo -u postgres psql postgres -c "DROP DATABASE apresabu;"');
	}
  
  if ($sqlfile_ori != '') unlink($sqlfile_ori);
  if ($sqlfile     != '') unlink($sqlfile);
  unlink($dbfile);
}

function RestoredNewCallCB($call)
{
	if ($call['tenantid']!=NULL)
	{
		// no audio data at this point yet
		UpdateTnDiskUsage($call['tenantid'], 0, +1);
	}
}

function ClearAudioAndPrepareRestore($FullRestore)
{
  $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
	//For the case of encryption, remove some things
	//that are cached in memory (should not be, but anyway)
	system('rm -R /dev/shm/apresa/2???');
	
	if ($FullRestore)
	{
		//First we empty our complete archive (dangerous!)
		//Deleting is easy.
		exec('rm -R /recordings/2*');
		$dbintf->SetProcessingFlagOnForAll(8);
		//If doing a complete restore, the backup should replace these keys
		system('rm /dev/shm/apresa/*.ekey');
		system('rm /var/apresa/data/*.ekey');
	}
	
	//This prevents rerunning to do only the copying again
  $configs_rb = array('RBckAct'=>1);
  $dbintf->SetConfigsPart($configs_rb,'rb');
  $dbintf = null;
}

function RestoreAdditionalFiles()
{
	global $options, $dbintf;

  $copyDirs = array
  ( array 
    ( 'backupdir' => 'telsys/audio'
    , 'localdir' => '/recordings/telsys/audio'
    ),
    array
    ( 'backupdir' => 'certificates'
    , 'localdir' => CERTDIR
    )
  );

  foreach ($copyDirs as $copyDirItem)
  {
    $backupdir = $options['BackupDir'] . '/' . $copyDirItem['backupdir'];
    $localdir  = $copyDirItem['localdir'];

    if (file_exists($backupdir)==false) continue;
    
    dbglog('Restoring files from ' . $backupdir . ' to ' . $localdir);
    
    $backup_perm = 0777;
    mkdir($localdir,$backup_perm,true);
    
    // the star wildcard should not be espaced/quoted,
    // otherwise it is not interpreted
    $cmd = 'cp ' . escapeshellarg($backupdir) . '/* ' . escapeshellarg($localdir) . '/';
    // debug: dbglog($cmd);
    system($cmd);
    
  }  
}

function FixCertificates()
{
  global $dbintf;
  $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
  $certificates = $dbintf->GetCertificateFiles();

  foreach($certificates as $cert)
  {
    chown(CERTDIR . $cert['cert_location'], 'www-data');
    chgrp(CERTDIR . $cert['cert_location'], 'www-data');

    if(strlen(str_ensure_defined($cert['intermediate_location'])))
    {
      chown(CERTDIR . $cert['intermediate_location'], 'www-data');
      chgrp(CERTDIR . $cert['intermediate_location'], 'www-data');
    }

    if(strlen(str_ensure_defined($cert['key_location'])))
    {
      if($cert['webkey']) $owner = 'www-data';
      else $owner = 'root';
      chown(CERTDIR . $cert['key_location'], $owner);
      chgrp(CERTDIR . $cert['key_location'], $owner);
      chmod(CERTDIR . $cert['key_location'], 0400);
    }

    if($cert['trusted'])
    {
      copy(CERTDIR . $cert['cert_location'], '/usr/local/share/ca-certificates/' . $cert['cert_location']);
    }
    else 
    {
      @unlink(CERTDIR . '/usr/local/share/ca-certificates/' . $cert['cert_location']);  
    }
  }
  
  system('update-ca-certificates');

  $requests = $dbintf->GetCertificateRequestFiles();

  foreach($requests as $request)
  {
    chown(CERTDIR . $request['request_location'], 'www-data');
    chgrp(CERTDIR . $request['request_location'], 'www-data');

    chown(CERTDIR . $request['key_location'], $root);
    chgrp(CERTDIR . $request['key_location'], $root);
    chmod(CERTDIR . $request['key_location'], 0400);
  }

  $dbintf=null;
}

function RestoreAudio($configs)
{
	global $options, $dbintf;

	// We copy the files in the backup database (just restored)
	// starting with the newest
  $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);

  // Do not load configuration,
  // but use the configuration before the restore
	
  $chuck_size = 64;
  $prev_first_id = false;

	// next two vars not used yet
  $RBckCopied = 0;
	$RBckProgress = 0;

	$backup_perm = 0777;

  // Doing in chunck, not all at once
  // because otherwise maybe too large a list for memory
  while (true)
  {
    // Get a list of files that need to be restore from backup
    // Do in chuncks
    $calls = $dbintf->GetToRestoreList($chuck_size);
    $cnt_calls = count($calls);
    if ($cnt_calls==0) {
      // ready
      break;
    }
    $first_id = $calls[0]['id'];
    if ($first_id==$prev_first_id)
    {
      onerror("Double ID in restore backup");
    }
    $prev_first_id = $first_id;

    foreach ($calls as &$call)
    {
      if ($call['isarchived']!=0)
      {
        $dirfn = $call['directory'] . '/' . $call['filename'];
        $src = $options['BackupDir'] . '/' . $dirfn;
        $dst = AUDIO_PATH . '/' . $dirfn;
        $dstdir = AUDIO_PATH . '/' . $call['directory'];
        if (is_dir($dstdir)===false)
        {
          if (mkdir($dstdir,$backup_perm,true)===false)
          {
            onerror("Failed to make directory $dstdir");
          }
        }
        $szplus = 0;
        if (isCopyNeeded($src,$dst))
        {
          if (@copy($src,$dst)===false)
          {
            // failed to copy, but ignore error
            // we must continue
            dbglog("Failed to copy " . $call['filename']);
          } else {
            $RBckCopied ++;
            $szplus += filesize($dst);
          }
        }
        if (HasScreenRecording($call))
        {
          $dirfn = $call['directory'] . '/' . GetScreenFileName($call);
          $src = $options['BackupDir'] . '/' . $dirfn;
          $dst = AUDIO_PATH   . '/' . $dirfn;
          if (isCopyNeeded($src,$dst))
          {
            if (@copy($src,$dst)===false)
            {
              // failed to copy, but ignore error
              // we must continue
            } else // IF copy
            {
              $szplus += filesize($dst);
            }
          }
        } // if has screen recording
        // check if tenant size must be updated
        // only for Restore Call (not for full DB, since it includes stats)
        if ($call['tenantid']!=NULL && $configs['RBckRestoreMethod']==0)
        {
          // only added audio, listing already there
          UpdateTnDiskUsage($call['tenantid'], $szplus, +0);
        }
      }
      $call['seen'] = 1;
      $RBckProgress ++;
    }

    $seen_id_list = array();
    foreach ($calls as &$call)
    {
      if ($call['seen']==1)
      {
        $seen_id_list[] = $call['id'];
      }
    }
    $dbintf->RemoveProcessingFlagForIDs($seen_id_list,8);
  }

	// We have finished
  $configs_rb = array('RBckAct'=>0);
  $dbintf->SetConfigsPart($configs_rb,'rb');
  $dbintf = null;
}

function isCopyNeeded($src, $dst)
{
	//Is the source file ok?
	if (!file_exists($src)) return false;
	$srcfilesize = filesize($src);
	if (!$srcfilesize) return false;
	
	if (!file_exists($dst)) return true;
	$dstfilesize = filesize($dst);
	if ($srcfilesize == $dstfilesize) return false;
	return true;
}

// stop and start conflicting services during full restore

function stop_services()
{
	system("service orkaudio stop");
  // handler should remain active
  system("service apresafifo stop");
	system("service apresacrec stop");
	system("service apresaremote stop");
	system("service apresaav32 stop");
	system("service apresaavnet stop");
  system("service apresa_ed137 stop");
}

function start_services()
{
  system("/usr/share/apresa/apresa_startup.sh -o");
  system("service orkaudio start");
  system("service apresafifo start");
  system("service apresacrec start");
  system("service apresaremote start");
  system("service apresaav32 start");
  system("service apresaavnet start");
  system("service apresa_ed137 start");
}

function dbglog($text)
{
	global $debug_mode;
	if (isset($debug_mode) && $debug_mode)
	{
		echo $text . "\n";
	} else
	{
		lg($text);
	}
}

function signal_required($msg)
{
  //echo 'Signaling ' . substr($msg,0,2) . "\n";
  $reply = 1;
  $err = signal_service($msg,$reply);
  if ($err!='')
  {
    onerror('Socket error: '.$err);
  }
  if ($reply=='OK')
  {
    // OK
  } else if ($reply[0]=='E')
  {
    onerror($msg . ' failed: '.$reply);
  } else
  {
    onerror($msg . ' failed, unexpected reply: '.$reply.' LEN: '.strlen($reply));
  }

}

function onerror($msg)
{
  throw new Exception($msg);
}

?>