<?php
/*
	Frequent tasks.
	This is to be run from a cron job, each minute.
*/

error_reporting(E_ALL ^ E_NOTICE);
define('CMD',1);
// processing flag in the database
define('EXPORT_FLAG',64);
define('EXPORT_MNT','/mnt/export');

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "encryption.php";
require_once CMD_WEB_RELPATH . "datafile.php";
require_once CMD_WEB_RELPATH . "utility.php";
require_once CMD_WEB_RELPATH . "emailing.php";
require_once CMD_WEB_RELPATH . "actions.php";
require_once CMD_WEB_RELPATH . "ldapfunc.php";
require_once CMD_WEB_RELPATH . "smbgen.php";
require_once CMD_WEB_RELPATH . "tenancy.php";
require_once "transcribe.php";
require_once "micc.php";

if ($argv[1]=='debug')
{
	// logging to console instead of apresa.log
	define('DEBUG_LOGING',1);
}

$f = new FrequentTask();
$f->Run();

function verbose_lg($t)
{
  if (defined('DEBUG_LOGING'))
  {
    lg($t);
  }
}

class FrequentTask
{
	var $errmsg = '';
	var $errflag = false;
	var $runcnt = 0;

	var $importBusyCount = 0;
	var $importBusyMax;
	var $startImportTime;

	
	function Run()
	{
		global $dbintf;
		$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);

		if ($dbintf->nodePassive)
		{
			// only do at primary
			return;
		}

		$counter = (int) $dbintf->GetConfigValue('FreqTaskCounter');
		$counter ++;
		if ($counter>=10000) $counter = 0;
		// too often: lg("Frequent Task [$counter]");
		$dbintf->SetConfigsPart(array('FreqTaskCounter'=>$counter),'dat');
		$this->runcnt = $counter;
		
		$this->RunTask('ExportRecs','Export recordings');
		$this->RunTask('SyncLdapGroups','Synchronize LDAP Groups');
		$this->RunTask('SyncSamwinGroups','Synchronize Samwin groups');
    $this->RunTask('SyncAzureGroups','Synchronize Azure Groups');
    $this->RunTask('ExportTenantRecs','Export tenant recordings');
    $this->RunTask('CheckCrec','Check card recording');
    $this->RunTask('Transcribe','Transcribe recordings');
    $this->RunTask('ExpiryTmpFiles','Delete expired temporary files');
		$this->RunTask('CheckTeamsBot','Check Teams Bot');
		$this->RunTask('Import3CX','Import 3CX recordings');
		$this->RunTask('ImportWildix','Import Wildix recordings');
		$this->RunTask('ReadMiccAgents','Read MICC agents');
		$this->RunTask('ExpireStoreOnDmd','Remove calls not stored on demand');
		// additional tasks can be added

		$dbintf->close();
	}
	
	// fn: function (so a dynamic call)
	function RunTask($fn,$fname)
	{
		global $dbintf;
		$this->errflag = false;
		$anydone = false;
		try
		{
			$anydone = $this->$fn();
		} catch(Exception $e)
		{
			$this->AddErrMsg("Exception: ".$e->getMessage());
		}
		// Error handling
		$cfgkey = 'TaskFailCnt_'.$fn;
		$failcnt = (int) $dbintf->GetConfigValue($cfgkey);
		if ($this->errflag)
		{
			lg($this->errmsg);
			lg("Task $fname failed");
			$failcnt ++;
			$dbintf->SetConfigsPart(array($cfgkey => $failcnt),'dat');
			if ($failcnt==4)
			{
				$this->reportToAdmin('Task ' . $fname . ' failed'
				, 'Task ' . $fname . ' failed 4 times.'
					. "\nError description:"
					. "\n\n" . $this->errmsg);
			}
		} else
		{
			// Only when something is done may we consider it as succeeded.
			if ($anydone)
			{
				$dbintf->SetConfigsPart(array('TaskFailCnt_'.$fn=>'0'),'dat');
				if ($failcnt>=4)
				{
					$this->reportToAdmin('Task ' . $fname . ' succeeded'
					, 'Task ' . $fname . ' completed successful after previous failure.');
				}
			}
		}
	}
  
  function CrecToRun()
  {
    global $dbintf;

    $crec = $dbintf->GetConfigValue('crec');
    verbose_lg("crec: " . $crec);
    if ($crec!='1') return false;

    $cardlist = $dbintf->GetConfigValue('CardList');
    verbose_lg("CardList: " . $cardlist);
    if ($cardlist=='') return false;

    if (file_exists('/var/apresa/conf/crec_disabled')) return false;
    
    return true;
  }
  
  function GetCrecStartTime()
  {
		// no more PID files
		$service = 'apresacrec';
		$service = 'orkaudio';
		$output = exec("systemctl show --property=ExecMainStartTimestamp orkaudio");
		$pos = strpos($output,'=');
		if ($pos===false) return 0;
		$timeStr = substr($output,$pos+1);
		$timeStamp = strtotime($timeStr);
		return $timeStamp;
  }
  
  function GetCrecEventTime()
  {
    $fn = '/var/apresa/data/crec_event';
    $fileInfo = @stat($fn);
    if ($fileInfo===false) return 0;
    return (int) $fileInfo['mtime'];
  }
  
  function KernLogShowsCardFail($kernlogfn)
  {
    $errcnt = (int) exec("grep FIFO_TryAppendMsg $kernlogfn 2>/dev/null | wc -l");
    verbose_lg('Number of errors: ' . $errcnt);
    if ($errcnt < 1000) return false;
    
    $now = time();
    
    $last = exec("grep FIFO_TryAppendMsg $kernlogfn 2>/dev/null | tail -n1 ");
    $last = strtotime( substr($last,0,15) );
    $earlier  = exec("grep FIFO_TryAppendMsg $kernlogfn 2>/dev/null | tail -n500 2>/dev/null | head -n1");
    $earlier  = strtotime( substr($earlier,0,15) );
    
    verbose_lg( 'Earlier error: ' . date('D M G:i:s Y', $earlier ) );
    verbose_lg( 'Last error   : ' . date('D M G:i:s Y', $last ) );
    
    if ($earlier < $now - 60)
    {
      verbose_lg('Earlier error is more than a minute ago');
      // we expect much more than 10 errors per second
      return false;
    }

    if ($last < $now - 5)
    {
      verbose_lg('Last error is more than 5 seconds ago');
      // last error must be within last 5 seconds
      return false;
    }
    
    return true;
  }
  
  function CrecCheckEnabled()
  {
		// there was never an gui option yet
    global $dbintf;
    $DoCrecCheck = $dbintf->GetConfigValue('DoCrecCheck');
    verbose_lg("DoCrecCheck: " . $DoCrecCheck);
		// empty or undefined is not really equal to zero
		// it can be disabled if needed
		// but by default it is done
    if ($DoCrecCheck==='0') return true;
    return false;
  }
  
  function Transcribe()
  {
    global $dbintf;
    verbose_lg('check Transcribe');
    
    $ScribeType = $dbintf->GetConfigValue('Scribe_Type');
    if ($ScribeType=='') return;
    
    $dbintf->SetAdmin();
    
    $obj = new Transcriber();
    $obj->Run();
    
  }
  
	function GetHostFromUrl($url)
	{
		$t = $url;
		$pos = strpos($t,'://');
		if ($pos !== false)
		{
			$t = substr($t,$pos+3);
		}
		$pos = strpos($t,'/');
		if ($pos !== false)
		{
			$t = substr($t,0,$pos);
		}
		return $t;
	}
	
	function Unescape3cx($fn)
	{
		// some characters are escaped using %FF
		return rawurldecode($fn);
	}
	
	function GetAudioDuration($fn)
	{
		$cmd = 'soxi -d ' . escapeshellarg($fn);
		$output = array();
		exec($cmd,$output);
		if (count($output)<1) return -1;
		$line = $output[0];
		$colonPos1 = strpos($line,':');
		if ($colonPos1===false) return -1;
		$colonPos2 = strpos($line,':',$colonPos1+1);
		if ($colonPos2===false) return -1;
		$hours = (int) substr($line,0,$colonPos1);
		$minu = (int) substr($line,$colonPos1+1,$colonPos2-$colonPos1);
		$seconds = round( floatval( substr($line,$colonPos2+1) ) );
		$duration = $hours*60*60 + $minu*60 + $seconds;
		return $duration;
	}
	
	function IsInExternalPhonesList($local)
	{
		global $dbintf;
		$mobs = $dbintf->GetMobileDevices();
		$mobcnt = count($mobs);
		$moblic = (int) $dbintf->GetConfigValue('MobileChCnt');
		//var_dump( $mobs );
		// match on local phone number
		for ($i=0;$i<$moblic && $i<$mobcnt;$i++)
		{
			if ($local!=''
			&& $local==$mobs[$i]['phonenum'] )
			{
				return true;
			}
		}
		return false;
	}
	
	function ImportWildixFile($file,$fullfn,$source)
	{
		global $dbintf;
		lg('File to import: ' . $fullfn );
		// Format:
		// yyyy.mm.dd-hh.mm.ss-caller-receiver[?]-callid-?.wav
		// examples:
		// 2023.09.06-13.26.08-+447883977691-+447883977691[pbx]-294475a8205533d82be4-0.mp3
		$fn = $file;
		$year  = (int) substr($fn,0,4);
		$month = (int) substr($fn,5,2);
		$day   = (int) substr($fn,8,2);
		$hour  = (int) substr($fn,11,2);
		$minu  = (int) substr($fn,14,2);
		$sec   = (int) substr($fn,17,2);
		$startdate = mktime($hour,$minu,$sec,$month,$day,$year);		
		if ($startdate===false) return false;
		// verbose lg( date('c',$startdate) );
		// split based on -
		$parts = explode('-',$fn);
		if ($parts===false) return false;
		if (count($parts)<5) return false;
		$caller = $parts[2];
		$receiver = $parts[3];
		// remove [name] part form receiver
		$pos = strpos($receiver,'[');
		if ($pos!==false)
		{
			$receiver = substr($receiver,0,$pos);
		}
		if ($source['delay']>0)
		{
			// determine age in minutes
			$age = (time() - $startdate) / 60;
			// debug:
			if ($age < $source['delay'])
			{
				lg('Delay import of this file, age is ' . $age . ' minutes');
				return true;
			}
		}
		$callIdWildix = $parts[4];
		
    $pi = pathinfo($fullfn);
		$directory = $pi['dirname'];
		
		$ConfRecEnc = $dbintf->GetConfigValue('RecEnc');
		if ($ConfRecEnc=='mp3') $storageExtension = 'mp3';
		else $storageExtension = 'wav';
		
		if ( strlen($caller) > strlen($receiver) )
		{
			// incoming
			$direction = 0;
			$local = $receiver;
			$remote = $caller;
		} else if ( strlen($caller) < strlen($receiver) )
		{
			// outbound
			$direction = 1;
			$local = $caller;
			$remote = $receiver;
		} else
		{
			// internal?
			$direction = 2;
			$local = $caller;
			$remote = $receiver;
		}
		
		lg('Import Wildix call of ' . date('c',$startdate) );
		
		// check if Local ID is in external phones
		if ($this->IsInExternalPhonesList($local)==false
		&& $this->IsInExternalPhonesList($remote)==false )
		{
			lg('Local number ' . $local . ' is not licensed for import');
			return false;
		}
		
		// determine duration:
		$duration = $this->GetAudioDuration($fullfn);
		if ($duration == -1)
		{
			lg('Could not determine duration');
			return false;
		}
		
    $data = array();

    $data['localfull']  = $local;
    $data['remotefull'] = $remote;
		$data['connected'] = '';
		
    $data['local'] = $local;
    $data['remote'] = $remote;

    $data['local2'] = '';
    $data['remote2'] = '';
		
		$data['direction'] = $direction;
		$remote_is_local = ($direction==2);
		$local_is_remote = ($direction==3);
	
    $data['startdate'] = $startdate;
    $data['duration'] = $duration;

		$data['flags'] = 0;
		$data['isarchived'] = 1;
		
		$rec_uniqifier = $dbintf->GetRecIdsNext();
		
		$data['filename'] = strftime('%Y%m%d_%H%M%S_x'.$rec_uniqifier,$startdate) . '.' . $storageExtension;
		$data['directory'] = strftime('%Y/%m/%d/%H',$startdate);
		$data['remoteid'] = strftime('%Y%m%d_%H%M%S_x'.$rec_uniqifier,$startdate);
		
		// use other recorder
		$recorderId = 'Wildix_' . $source['id'];
		$recorderName = $source['description'];
		$data['recorderid'] = $dbintf->GetOrCreateRecorderId($recorderId,$recorderName,60);
			
		$data['waittime'] = -1;

		$data['source'] = 0; // unknown
		$data['encoding'] = 51; // unknown, GSM or G.711
		$data['encryption'] = 0; // no (not used for ALE)
		$data['processingstate'] = 4; // audio processing
		$data['lineid'] = 0; // not applicable

		$data['localcontact'] = 0;
		$data['remotecontact'] = 0;
		
		$localname = '';
		$remotename = '';

		$cid = $local;
		$ret = $dbintf->GetOrCreateCallerID($cid,$data['local2' ],$local_is_remote?1:0,$localname);
		$data['localcallerid'] = $ret["id"];
		$data['localcontact'] = $ret["contactid"];
		
		$cid = $remote;
		$ret = $dbintf->GetOrCreateCallerID($cid,$data['remote2'],$remote_is_local?1:0,$remotename);
		$data['remotecallerid'] = $ret["id"];
		$data['remotecontact'] = $ret["contactid"];
		
		$convert = true;
		
		$tenantid = CalcTenant($data);
		$data['tenantid'] = $tenantid;
		
		$ALE = (int) $dbintf->GetConfigValue('ALE');
		$AleDisk = (int) $dbintf->GetConfigValue('AleDisk');
		
		if($tenantid !== null)
		{
			$group = $dbintf->GetGroupInfo( $data['tenantid'] );
			$key = $dbintf->GetCallEncryptionKey($group['accid']);
			if($key && $key['inuse'])
			{
				$ALE = 2;
			}
		}

		if ($ALE==1)
		{ 
			$data['flags'] |= 16;
		}
		else if($ALE==2)
		{
			$data['flags'] |= 2048;
		}
		
		// Because original file could contain spaces,
		// which is not handled well by the handler,
		// we first rename.
		// The prefix busy_ is used to detect if it is a busy file,
		// so we skip it during the next round if it is still there then.
		$tmpfn = $directory . '/busy_' . $data['filename'];
		if (rename($fullfn,$tmpfn)==false)
		{
			lg('Could not rename file');
			return false;
		}
		
		$handler_act = 'copyrec';
		if ($convert) $handler_act = 'copyconvrec';
		if (signal_service($handler_act
			.' '.$tmpfn
			.' '.$data['directory'].'/'.$data['filename']
			.' '.$data['remoteid']
			.' '.$ConfRecEnc
			.' '.$ALE
			.' '.$AleDisk
			)!='')
		{
			lg('Failed to ' . $handler_act);
			DeleteFn($tmpfn);
			return false;
		}
		
		$dbintf->InsertActiveCall($data);		
		// debug: lg('active processing count = ' . $dbintf->GetActiveCallsProcessing() );
		$this->importBusyCount++;
		
		return true;
	}

	function Import3CXFile($file,$fullfn,$source)
	{
		global $dbintf;
		lg('File to import: ' . $fullfn );
		// Format seems to be:
		// [DisplayName]_Caller-Receiver_YearMonthDayHourMinuteSecond(InternalCallIdentifier)
		// examples:
		// [1126]_1126-07554253592_20180717120127(1004).wav
		// [Suppliers Queue%3A01212701800%3AMain incoming number]_01212701800-015_20191003130152(115).wav
		$fn = $this->Unescape3cx($file);
		// verbose lg($fn);
		$posSqbStart = strpos($fn,'[');
		$posSqbStop  = strpos($fn,']');
		if ($posSqbStart===false) return false;
		if ($posSqbStop===false) return false;
		$DisplayName = substr($fn,$posSqbStart+1,$posSqbStop-$posSqbStart-1);
		// verbose lg('Display name = ' . $DisplayName);
		$posUn1 = strpos($fn,'_');
		if ($posUn1===false) return false;
		$posUn2 = strpos($fn,'_',$posUn1+1);
		if ($posUn2===false) return false;
		$FromTo = substr($fn,$posUn1+1,$posUn2-$posUn1-1);
		// verbose lg('FromTo = ' . $FromTo );
		$parts = explode('-',$FromTo);
		if ($parts===false) return false;
		if (count($parts)!=2) return false;
		$caller = $parts[0];
		$receiver = $parts[1];
		// verbose lg("3CX call from $caller to $receiver");
		$posRbStart = strpos($fn,'(');
		if ($posRbStart===false) return false;
		$datePosSize = $posRbStart - $posUn2 - 1;
		if ($datePosSize!=14) return false;
		$year  = (int) substr($fn,$posUn2+1,4);
		$month = (int) substr($fn,$posUn2+5,2);
		$day   = (int) substr($fn,$posUn2+7,2);
		$hour  = (int) substr($fn,$posUn2+9,2);
		$minu  = (int) substr($fn,$posUn2+11,2);
		$sec   = (int) substr($fn,$posUn2+13,2);
		$startdate = mktime($hour,$minu,$sec,$month,$day,$year);		
		if ($startdate===false) return false;
		// verbose lg( date('c',$startdate) );
		if ($source['delay']>0)
		{
			// determine age in minutes
			$age = (time() - $startdate) / 60;
			// debug:
			if ($age < $source['delay'])
			{
				lg('Delay import of this file, age is ' . $age . ' minutes');
				return true;
			}
		}
		$posRbStop = strpos($fn,')');
		if ($posRbStop===false) return false;
		$callId3cx = substr($fn,$posRbStart+1,$posRbStop-$posRbStart-1);
		
    $pi = pathinfo($fullfn);
		$directory = $pi['dirname'];
		
		$ConfRecEnc = $dbintf->GetConfigValue('RecEnc');
		if ($ConfRecEnc=='mp3') $storageExtension = 'mp3';
		else $storageExtension = 'wav';
		
		if ( strlen($caller) > strlen($receiver) )
		{
			// incoming
			$direction = 0;
			$local = $receiver;
			$remote = $caller;
		} else if ( strlen($caller) < strlen($receiver) )
		{
			// outbound
			$direction = 1;
			$local = $caller;
			$remote = $receiver;
		} else
		{
			// internal?
			$direction = 2;
			$local = $caller;
			$remote = $receiver;
		}
		
		lg('Import 3CX call of ' . date('c',$startdate) );
		
		// check if Local ID is in external phones
		if ($this->IsInExternalPhonesList($local)==false
		&& $this->IsInExternalPhonesList($remote)==false )
		{
			lg('Local number ' . $local . ' is not licensed for import');
			return false;
		}
		
		// determine duration:
		$duration = $this->GetAudioDuration($fullfn);
		if ($duration == -1)
		{
			lg('Could not determine duration');
			return false;
		}
		
    $data = array();

    $data['localfull']  = $local;
    $data['remotefull'] = $remote;
		$data['connected'] = '';
		
    $data['local'] = $local;
    $data['remote'] = $remote;

    $data['local2'] = '';
    $data['remote2'] = '';
		
		$data['direction'] = $direction;
		$remote_is_local = ($direction==2);
		$local_is_remote = ($direction==3);
	
    $data['startdate'] = $startdate;
    $data['duration'] = $duration;

		$data['flags'] = 0;
		$data['isarchived'] = 1;
		
		$rec_uniqifier = $dbintf->GetRecIdsNext();
		
		$data['filename'] = date('Ymd_His',$startdate) . '_x' .$rec_uniqifier . '.' . $storageExtension;
		$data['directory'] = date('Y/m/d/H',$startdate);
		$data['remoteid'] = date('Ymd_His',$startdate) . '_x' . $rec_uniqifier;
		
		// use other recorder
		$recorderId = '3CX_' . $source['id'];
		$recorderName = $source['description'];
		$data['recorderid'] = $dbintf->GetOrCreateRecorderId($recorderId,$recorderName,60);
			
		$data['waittime'] = -1;

		$data['source'] = 0; // unknown
		$data['encoding'] = 51; // unknown, GSM or G.711
		$data['encryption'] = 0; // no (not used for ALE)
		$data['processingstate'] = 4; // audio processing
		$data['lineid'] = 0; // not applicable

		$data['localcontact'] = 0;
		$data['remotecontact'] = 0;
		
		$localname = '';
		$remotename = '';

		$cid = $local;
		$ret = $dbintf->GetOrCreateCallerID($cid,$data['local2' ],$local_is_remote?1:0,$localname);
		$data['localcallerid'] = $ret["id"];
		$data['localcontact'] = $ret["contactid"];
		
		$cid = $remote;
		$ret = $dbintf->GetOrCreateCallerID($cid,$data['remote2'],$remote_is_local?1:0,$remotename);
		$data['remotecallerid'] = $ret["id"];
		$data['remotecontact'] = $ret["contactid"];
		
		$data['data1'] = $DisplayName;
		$data['data2'] = $callId3cx;
		
		$convert = true;
		
		$tenantid = CalcTenant($data);
		$data['tenantid'] = $tenantid;
		
		$ALE = (int) $dbintf->GetConfigValue('ALE');
		$AleDisk = (int) $dbintf->GetConfigValue('AleDisk');
		
		if($tenantid !== null)
		{
			$group = $dbintf->GetGroupInfo( $data['tenantid'] );
			$key = $dbintf->GetCallEncryptionKey($group['accid']);
			if($key && $key['inuse'])
			{
				$ALE = 2;
			}
		}

		if ($ALE==1)
		{ 
			$data['flags'] |= 16;
		}
		else if($ALE==2)
		{
			$data['flags'] |= 2048;
		}
		
		// Because original file could contain spaces,
		// which is not handled well by the handler,
		// we first rename.
		// The prefix busy_ is used to detect if it is a busy file,
		// so we skip it during the next round if it is still there then.
		$tmpfn = $directory . '/busy_' . $data['filename'];
		if (rename($fullfn,$tmpfn)==false)
		{
			lg('Could not rename file');
			return false;
		}
		
		$handler_act = 'copyrec';
		if ($convert) $handler_act = 'copyconvrec';
		if (signal_service($handler_act
			.' '.$tmpfn
			.' '.$data['directory'].'/'.$data['filename']
			.' '.$data['remoteid']
			.' '.$ConfRecEnc
			.' '.$ALE
			.' '.$AleDisk
			)!='')
		{
			lg('Failed to ' . $handler_act);
			DeleteFn($tmpfn);
			return false;
		}
		
		$dbintf->InsertActiveCall($data);		
		// debug: lg('active processing count = ' . $dbintf->GetActiveCallsProcessing() );
		$this->importBusyCount++;
		
		return true;
	}
	
	function ImportWildixDirectory
	( $basedir
	, $directory
	, $source
	) 
	{
		global $dbintf;
		// verbose: lg('scan ' . $directory);
		$files = scandir($directory);
		foreach ($files as $file)
		{
			// skip these directories
			if ($file=='.' || $file=='..') continue;
			if ($file=='rejected') continue;
			if (time() - $this->startImportTime > 30)
			{
				lg('postpone further wildix import (already took 30 s)');
				break;
			}
			
			if ($this->importBusyCount >= $this->importBusyMax)
			{
				// recalibrate with database
				$this->importBusyCount = $dbintf->GetActiveCallsProcessing();
				if ($this->importBusyCount >= $this->importBusyMax)
				{
					lg('wildix sleep...');
					usleep(200*1000);
					$this->importBusyCount = $dbintf->GetActiveCallsProcessing();
					if ($this->importBusyCount >= $this->importBusyMax)
					{
						lg('postpone further wildix import ('.$this->importBusyCount.' busy)');
						break;
					}
				}
			}
			
			$fullfn = $directory . '/' . $file;
			if (is_dir($fullfn))
			{
				// recursive import
				$this->ImportWildixDirectory($basedir,$fullfn,$source);
				continue;
			}
			
			if (substr($file,0,5)=='busy_')
			{
				lg('skip busy file: ' . $file);
				continue;
			}
			
			if ($this->ImportWildixFile($file,$fullfn,$source))
			{
				// imported started ok (delete is done elsewhere),
				// or import is delayed (so do not delete)
			} else
			{
				// rejected, maybe invalid format
				lg('Import rejected / failed');
				$rejectDir = $basedir . '/' . 'rejected';
				if (file_exists($rejectDir)==false)
				{
					if (mkdir( $rejectDir )==false)
					{
						lg('Failed to create ' . $rejectDir . '. Import stopped');
						return;
					}
				}
				if (rename($fullfn, $rejectDir . '/' . $file )==false)
				{
					lg('Failed to move file to rejected directory. Import stopped');
					return;
				}
				// rejected file is moved to rejected directory
				// customer may review it
				
			}
			
		}
		
	}	

	function Import3CXDirectory
	( $basedir
	, $directory
	, $source
	) 
	{
		global $dbintf;
		// verbose: lg('scan ' . $directory);
		$files = scandir($directory);
		foreach ($files as $file)
		{
			// skip these directories
			if ($file=='.' || $file=='..') continue;
			if ($file=='rejected') continue;
			if (time() - $this->startImportTime > 30)
			{
				lg('postpone further 3cx import (already took 30 s)');
				break;
			}
			
			if ($this->importBusyCount >= $this->importBusyMax)
			{
				// recalibrate with database
				$this->importBusyCount = $dbintf->GetActiveCallsProcessing();
				if ($this->importBusyCount >= $this->importBusyMax)
				{
					lg('3cx sleep...');
					usleep(200*1000);
					$this->importBusyCount = $dbintf->GetActiveCallsProcessing();
					if ($this->importBusyCount >= $this->importBusyMax)
					{
						lg('postpone further 3cx import ('.$this->importBusyCount.' busy)');
						break;
					}
				}
			}
			
			$fullfn = $directory . '/' . $file;
			if (is_dir($fullfn))
			{
				// recursive import
				$this->Import3CXDirectory($basedir,$fullfn,$source);
				continue;
			}
			
			if (substr($file,0,5)=='busy_')
			{
				lg('skip busy file: ' . $file);
				continue;
			}
			
			if ($this->Import3CXFile($file,$fullfn,$source))
			{
				// imported started ok (delete is done elsewhere),
				// or import is delayed (so do not delete)
			} else
			{
				// rejected, maybe invalid format
				lg('Import rejected / failed');
				$rejectDir = $basedir . '/' . 'rejected';
				if (file_exists($rejectDir)==false)
				{
					if (mkdir( $rejectDir )==false)
					{
						lg('Failed to create ' . $rejectDir . '. Import stopped');
						return;
					}
				}
				if (rename($fullfn, $rejectDir . '/' . $file )==false)
				{
					lg('Failed to move file to rejected directory. Import stopped');
					return;
				}
				// rejected file is moved to rejected directory
				// customer may review it
				
			}
			
		}
		
	}
	
	function ImportWildix()
	{
		global $dbintf;
		$sources = $dbintf->GetEnabledImportTargetsOfType('Wildix');
		if ($sources===false) return;
		if (count($sources)==0) return;

		$cmdStart = '/var/apresa/cmd/wildix_start';
		if (file_exists($cmdStart))
		{
			lg('Wildix start');
			$exitcode = 0;
			$output = null;
			if (exec($cmdStart,$output,$exitcode)==false)
			{
				lg('Wildix failed to start');
				return;
			}
			lg("Output: " . implode("\n",$output));
			if ($exitcode!=0)
			{
				lg('Wildix start failed with code ' . $exitcode);
				return;
			}
		}
		
		$this->importBusyCount = $dbintf->GetActiveCallsProcessing();
		// debug: lg('3CX import busy count = ' . $this->importBusyCount);
		$this->importBusyMax = 10;
		$this->startImportTime = time();
		foreach ($sources as $source)
		{
			$interval = (int) $source['interval'];
			$interval /= 60;
			if ($this->runcnt % $interval != 0) continue;
			$directory = $source['directory'];
			$recorderName = $source['description'];
			lg('Wildix import ' . $recorderName . ' from ' . $directory . ' (recursive)');
			$this->ImportWildixDirectory($directory,$directory,$source);
			lg('Wildix import ' . $source['description'] . ' finished' );
		}
		
		$cmdStop = '/var/apresa/cmd/wildix_stop';
		if (file_exists($cmdStop))
		{
			lg('Wildix stop');
			$exitcode = 0;
			$output = null;
			if (exec($cmdStop,$output,$exitcode)==false)
			{
				lg('Wildix failed to stop');
				return;
			}
			lg("Output: " . implode("\n",$output));
			if ($exitcode!=0)
			{
				lg('Wildix stop failed with code ' . $exitcode);
				return;
			}
		}

	}

	function Import3CX()
	{
		global $dbintf;
		$sources = $dbintf->GetEnabledImportTargetsOfType('3CX');
		if ($sources===false) return;
		if (count($sources)==0) return;
		
		$cmd3cx = '/var/apresa/cmd/3cx_start';
		if (file_exists($cmd3cx))
		{
			lg('3CX start');
			$exitcode = 0;
			$output = null;
			if (exec($cmd3cx,$output,$exitcode)==false)
			{
				lg('3CX failed to start');
				return;
			}
			lg("Output: " . implode("\n",$output));
			if ($exitcode!=0)
			{
				lg('3CX start failed with code ' . $exitcode);
				return;
			}
		}
		
		$this->importBusyCount = $dbintf->GetActiveCallsProcessing();
		// debug: lg('3CX import busy count = ' . $this->importBusyCount);
		$this->importBusyMax = 10;
		$this->startImportTime = time();
		foreach ($sources as $source)
		{
			$interval = (int) $source['interval'];
			$interval /= 60;
			if ($this->runcnt % $interval != 0) continue;
			$directory = $source['directory'];
			$recorderName = $source['description'];
			lg('3CX import ' . $recorderName . ' from ' . $directory . ' (recursive)');
			$this->Import3CXDirectory($directory,$directory,$source);
			lg('3CX import ' . $source['description'] . ' finished' );
		}
		
		$cmd3cx = '/var/apresa/cmd/3cx_stop';
		if (file_exists($cmd3cx))
		{
			lg('3CX stop');
			$exitcode = 0;
			$output = null;
			if (exec($cmd3cx,$output,$exitcode)==false)
			{
				lg('3CX failed to stop');
				return;
			}
			lg("Output: " . implode("\n",$output));
			if ($exitcode!=0)
			{
				lg('3CX stop failed with code ' . $exitcode);
				return;
			}
		}
		
	}
	
	function ExpireStoreOnDmd()
	{
    global $dbintf;
		$runInterval = 3;
		// Run only once in X minutes:
		if ($this->runcnt % $runInterval != 0)
		{
			return false;
		}
		// skip if setting is disabled
		$delay = (int) $dbintf->GetConfigValue('StoreOnDemandDelay');
		if ( $delay == 0) return false;
		lg('Expire store on demand');
		$basepath = AUDIO_PATH;
		// remove max. 10K in one round
		for ($i=0;$i<100;$i++)
		{
			$batchSize = 100;
			$calls = $dbintf->GetExpiredSodCalls( $delay, $batchSize );
			// remove these calls
			$ids_remove = [];
			foreach($calls as $call)
			{
				// verbose:
				lg('Remove expired store on demand call ' . $call['remoteid']);
				// skip information entry (but still delete them from CallsArchive)
				if ($call['filename']!='')
				{
					$dirfn = $call['directory'] . '/' . $call['filename'];
					if ($call['tenantid']!=NULL && $call['isarchived']!=0)
					{
						UpdateTnDiskUsage($call['tenantid'], -GetCallStorageEx($call,$basepath), -1);
					}
					$delfn = $basepath;
					$delfn .= '/' . $dirfn;
					//echo 'to delete: '.$delfn."\n";
					DeleteFn($delfn);

					if (HasScreenRecording($call))
					{
						$dirfn = $call['directory'] . '/' . GetScreenFileName($call);
						$delfn = $basepath;
						$delfn .= '/' . $dirfn;
						//echo 'to delete: '.$delfn."\n";
						DeleteFn($delfn);
					}
					
					// Do not do: DeleteEmptyDirsAfterDelete($delfn,$dirfn);
          // Because it are relatively recent files, it could interfere
          // with file creation of the recording engines,
          // especially in case of encrypted recording.
          // The empty directories do not really hurt,
          // except taking a bit of space.
				}
				
				$ids_remove[] = $call['id'];
			}
			if (count($ids_remove)>0)
			{
				$dbintf->DeleteActiveCallsForIDs($ids_remove);
			} 
			
			if (count($calls)==$batchSize) ; // try again
			else break;
		}
		lg('Expire store on demand finished');
		return true;
	}
	
	function ReadMiccAgents()
	{
		$obj = new MiccSync();
		$obj->Run($this);
	}
	
	function CheckTeamsBot()
	{
    global $dbintf;
		
		$connectTimeout = 20; // seconds
		$runInterval = 3; // minutes
		$anydone = false;
		
		// Run only once in X minutes:
		if ($this->runcnt % $runInterval != 0)
		{
			return $anydone;
		}
		
		$check = $dbintf->GetConfigValue('TeamsBotCheck');
		if ($check != 1) return $anydone;

		// lets see how the Teams bot is doing
		verbose_lg("TeamsBot!");
		$server = $dbintf->GetConfigValue('TeamsBotServer');
		$pwd = decrypt( $dbintf->GetConfigValue('TeamsBotPwd') );

		$am = '';
		$value = '';
		$health = true;
		$checkTime = time();
		$answer = false;
		$retobj = null;
		
		$host = $this->GetHostFromUrl($server);
		verbose_lg("Check IP address of Teams bot hostname $host ($server)");
		$ipaddr = gethostbyname($host);
		if ($ipaddr=='0.0.0.0')
		{
			$health = false;
			$value = 'Invalid DNS IP address (deployment deleted?)';
			$am = 'Invalid DNS IP address for Teams Bot';
		} else
		{
			verbose_lg("Connect to Teams bot server $server ($ipaddr)");
			
			$cu = curl_init();
			curl_setopt($cu,CURLOPT_URL, $server . "/status");
			curl_setopt($cu,CURLOPT_POST, 1);
			curl_setopt($cu,CURLOPT_RETURNTRANSFER, true);
			curl_setopt($cu,CURLOPT_CONNECTTIMEOUT,$connectTimeout);
			curl_setopt($cu,CURLOPT_POSTFIELDS, http_build_query(array
				( 'auth' => $pwd
				) ) );
			$answer = curl_exec($cu);
			$retobj = null;
			if ($answer===false)
			{
				$health = false;
				$value = 'Could not connect';
				$am = 'No connection to Teams Bot';
			} else if ($answer=='')
			{
				$health = false;
				$value = 'Invalid reply';
				$am = 'Invalid reply Teams Bot';
			} else
			{
				verbose_lg('Answer = ' . $answer);
				$retobj = json_decode( $answer, true );
				if ($retobj === null)
				{
					$health = false;
					$value = 'Invalid reply';
					$am = 'Invalid reply Teams Bot';
				} else if ($retobj['result']=='OK')
				{
					$value = 'Version ' . $retobj['botVersion'];
				} else 
				{
					$health = false;
					$value = $retobj['errorType'];
					if ($retobj['errorDetail']!='') $value .= ' ' . $retobj['errorDetail'];
					$am = 'Error reported by Teams Bot';
				}
			}
		}
		$anydone = true;
		
		verbose_lg('status = ' . $value);
		
		$toSet = array
		( 'TeamsBotHealth' => ($health ? 1 : 0)
		, 'TeamsBotAlarmMsg' => $am
		, 'TeamsBotStatus' => $value
		, 'TeamsBotCheckTime' => $checkTime
		);
		$toSet['TeamsBotAnswer'] = '';
		if ($retobj !== null)
		{
			// store JSON answer, so we can do more things in health.php
			$toSet['TeamsBotAnswer'] = $answer;
		}
		$dbintf->SetConfigsPart($toSet,'s');
		
		return $anydone;
	}
	
  function ExpiryTmpFiles()
  {
    global $dbintf;
    $theTime = time();
    $fileList = $dbintf->GetExpiredTmpFiles( time() );
    foreach ($fileList as $fileEntry)
    {
      lg('To delete expired ' . $fileEntry['filepath'] );
      @unlink( $fileEntry['filepath'] );
    }
    // this must be the same list as previous
    // because new items never have an expiry that is shorter than one second
    $dbintf->DeleteExpiredTmpFiles( $theTime );
  }
  
  function CheckCrec()
  {
    verbose_lg('Check CREC');
    
    if ($this->CrecCheckEnabled()==false)
    {
      verbose_lg('crec check is disabled');
      return false;
    }
    
		$runInterval = 5; // minutes
		// Run only once in X minutes:
		if ($this->runcnt % $runInterval != 0)
		{
			return false;
		}
    
    if (file_exists('/var/apresa/data/busy'))
    {
      verbose_lg('skip check, system action is busy');
      return false;
    }
    
    if ($this->CrecToRun()==false)
    {
      verbose_lg('skip check, CREC is not expected to run');
      return true;
    }
    
    verbose_lg('CREC is expected to run');

    $now = time();
    
		$serviceStart = $this->GetCrecStartTime();
    if ($serviceStart==0)
    {
      // no PID file
      return true;
    }
    $runTime = $now - $serviceStart;
    verbose_lg('CREC started at: ' . date('D M G:i:s Y',$serviceStart) . ' , so has run for ' . (int)($runTime/60) . ' minutes' );
    if ($runTime < 15 * 60)
    {
      // give it some time to start and run
      return true;
    }
    
    $lastEvent = $this->GetCrecEventTime();
    if ($lastEvent==0)
    {
      // no event file
      return true;
    }

    verbose_lg('CREC last event at: ' . date('D M G:i:s Y',$lastEvent) . ' , ' . (int)( ($now-$lastEvent) /60) . ' minutes ago');
    
    // no events for 5 minutes
    if ($lastEvent > $now - 5 * 60)
    {
      verbose_lg('An event in the last 5 minutes');
      return true;
    }
    
    $kernlogfn = '/var/log/kern.log';
    if ($this->KernLogShowsCardFail($kernlogfn)==false)
    {
      verbose_lg('No card failure deduced from kern.log');
      return true;
    }
    
    lg('CREC error detected (kernel log err and no events)');
    // Alarm
    file_put_contents('/var/apresa/data/err_crec.dat','Driver error condition detected');
    // Automatic restart (as root)
    exec('service apresacrec restart');
    lg('End of CREC check');
		return true;
  }

  function SyncAzureGroups()
  {
		global $dbintf;
		if ($dbintf->nodePassive) return;
    $anydone = false;

    // using LDAP interval setting
		$counter = $dbintf->GetConfigValue('LdapSyncInt');
    // set a default
    if ($counter===false) $counter = 15;
		if ($counter == 0) return false;
		// Run only once in X minutes:
		if ($this->runcnt % $counter != 0)
		{
			return false;
		}
	
		return AzureSyncGroups();    
  }
  
	function SyncSamwinGroups()
	{
		global $dbintf;
		if ($dbintf->nodePassive) return;

		$configs_s = $dbintf->LoadConfigsPart('s');

		if (int_ensure_defined($configs_s['SamwinInt'])!=1)
		{
			return false;
		}
	
		$counter = 15;		//Default 15 minutes.
		if (isset($configs_s['SamwinInterval']))
		{		
			$counter = (int)$configs_s['SamwinInterval'];
		}
		if ($counter == 0)
		{
			return false;	//No synchronization!
		}
		// debug: lg("Samwin sync counter: $this->runcnt $counter");
		
		// Run only once in X minutes:
		if ($this->runcnt % $counter != 0)
		{
			return false;
		}
	
		include('samwin.php');
		return DoSyncSamwinGroups();
	}
	
	function SyncLdapGroups()
	{
		global $dbintf;
		if ($dbintf->nodePassive) return;
		
		$anydone = false;
		$dbintf->SetAdmin();
		
		$configs2 = $dbintf->LoadConfigsPart('n');
		if (isset($configs2['LdapSyncInt']))
		{		
			$counter = (int)$configs2['LdapSyncInt'];		//If $configs2['LdapSyncInt']=='' => $counter = 0!
		}
		else
		{
			$counter = 15;		//Default 15 minutes.
		}
		if ($counter == 0)
		{
			return $anydone;	//No synchronization!
		}
		
		// Run only once in X minutes:
		if ($this->runcnt % $counter != 0)
		{
			return $anydone;
		}
		
		$result = $dbintf->getADGroupLinks();
		if (count($result)==0)
		{
			// nothing to sync.
			return $anydone;
		}
		
		$start_time = microtime(true);
		lg('Start synchronizing Ldap groups.');
		$configs_o = $dbintf->LoadConfigsPart('o');
		$dbintf->StartTransaction();
		$result = $dbintf->getADGroupLinks();
		$ldapImport = new LdapImport();
		$ldapImport->Init();
		foreach($result as $a)
		{
			try
			{
				$ldapImport->ResetFailure();
				$ldapImport->LdapImportGroup($a['id']);
			} catch (Exception $e)
			{
				lg('Exception: ' . $e->getMessage());
				$ldapImport->AddError($a['name'],$e->getMessage());
			}
			if ($ldapImport->failure)
			{
				$ldapImport->AddFailure($a['name']);
			}
		}
		$ldapImport->DbFinish();
		$dbintf->CommitTransaction();
		$ldapImport->Finalize();
		$end_time = microtime(true);
		lg('Finished synchronizing Ldap groups. Duration: '.round($end_time - $start_time,3).' seconds.');
		return true;
	}
	
  function ExportTenantRecs()
  {
		global $dbintf, $configs;
		if ($dbintf->nodePassive) return;
    // tenancy.php
    ExportTenantRecordings($this);
  }
  
	function ExportRecs()
	{
		global $dbintf, $configs;
		if ($dbintf->nodePassive) return;
		$anydone = false;
    
    $interval = $dbintf->GetConfigValue('ExportInterval');
    if ($interval===false) $interval = 3;
    $interval = (int) $interval;
    if ($interval <= 0) $interval = 3;

		// Run only once in X minutes:
		if ($this->runcnt % $interval != 0)
		{
			return $anydone;
		}
		
		$begintime = time();

		LoadBasicSettings();
		
		$configs = array_merge( $configs, $dbintf->LoadConfigsPart('exp') );
		// a setting if this is to be done at all or not
		if (int_ensure_defined($configs['ExpRecs'])==0)
		{
			return $anydone;
		}
		// from now on, in this function 'return' is forbidden
		
		// we do first an umount, to recover from previous failure/crash
		$this->DoUnmount();

    $cfg = array();
    if ($configs['ExpRecs']==1)
    {
      $cfg['Flag'] = EXPORT_FLAG;
    }
    if ( str_ensure_defined( $configs['ExpAccFilter'] )!="" )
    {
      $cfg['accidfilter'] = (int) $configs['ExpAccFilter'];
    }

    $IdHandled = 0;
    if (isset($configs['EBckIdHandled']))
    {
      // new style
      $IdHandled = $configs['EBckIdHandled'];
    } else
    {
      // upgrade old style
      if (isset($configs['EBckOffset']))
      {
        lg('Upgrade to new style offset');
        $ExportOffset = (int) $configs['EBckOffset'];
        $idConvCfg = array
        ( 'AndFlag' => 64
        , 'EqFlag' => 64
        );
        $IdHandled = $dbintf->GetIdHandled($ExportOffset,$idConvCfg);
        lg("IdHandled: " . $IdHandled);
        lg("ExportOffset: " . $ExportOffset);
      }
    }
    
    verbose_lg('Check export recordings');
    
		$BackupDest = '/mnt/drive_E';
    if ($configs['EBckTargetType']==1)
    {
      // backup to internal or USB disk
      $BackupDest = $configs['EBckPath'];
      $options['Mounting'] = false;
      // check if a mount point
      exec('mountpoint '.escapeshellarg($BackupDest),$dummy,$retval);
      if ($retval != 0)
      {
        $this->AddErrMsg("Export destination $BackupDest is not a mount point");
        return false;
      }
    }
    if ($configs['EBckTargetType']==2)
    {
      // backup to any directory
      $BackupDest = $configs['EBckPath'];
      $options['Mounting'] = false;
    }    
    
		$mounted = false;
		$to_stop = false;
		try
		{
		
			$chunck_size = 30;
			// Doing in chunck, not all at once
			// because otherwise maybe too large a list for memory
			while ($this->errflag==false && $to_stop==false)
			{
				// Get list of recordings to be exported...
				$testtime1 = microtime(true);
        $IdLatest = $dbintf->GetIdOfLatestCall();
        $calls_pre = $dbintf->GetToHandleList($IdHandled,$chunck_size,$cfg);
				$testtime2 = microtime(true);
				$db_time = (int) (($testtime2-$testtime1)*1000);
				if ($db_time>500)
				{
					lg( 'Get export list query took '.$db_time. ' ms' );
				}
        if (count($calls_pre)==0)
        {
          // we found nothing to export
          if ($IdLatest > $IdHandled)
          {
            // but when filtering, it might still be possible to update our pointer,
            // and move it forward a bit, which improves speed of next query
            $IdHandled = $IdLatest;
            $dbintf->SetConfigsPart( array('EBckIdHandled'=>$IdHandled) , 'exp' );
          }
          break;
        }
        $calls = array();
        $delay_screen_cnt = 0;
        $delay_any_cnt = 0;
				foreach ($calls_pre as $call)
				{
					$age = time() - (int)$call['startdate'] - (int)$call['duration'];
					//lg('Age: ' . $age . ' s');
          if (int_ensure_defined($configs['ExpWait'])!=0)
          {
            $minage = int_ensure_defined($configs['ExpWait']) * 60;
            if ($age<$minage)
            {
              $delay_any_cnt ++;
							$to_stop = true;
							continue;
            }
          }
					if (int_ensure_defined($configs['ExpWaitScreen'])!=0)
					{
						if (HasScreenRecording($call)==false && $age<60)
						{
							// wait on screen recording
              $delay_screen_cnt ++;
              // we dont stop immediately,
              // because otherwise we might never export,
              // if a new recording without screen recording arrives
              // periodically within each minute
							$to_stop = true;
							continue;
						}
					}
          $calls[] = $call;
				}
        if ($delay_any_cnt>0)
        {
          // verbose... lg('Delaying export for ' . $delay_any_cnt . ' recordings');
        }
        if ($delay_screen_cnt>0)
        {
          lg('Delaying export for screen recording for ' . $delay_screen_cnt . ' recordings');
        }
        
				$cnt_calls = count($calls);
				if ($cnt_calls==0)
				{
					// we are ready, or nothing to do
					break;			
				}
				lg('Calls to export: ' . $cnt_calls);

				// we need mounting
				if ($mounted==false)
				{
					// but have not mounted yet
					if ($this->DoMount($configs))
					{
						$mounted = true;

            if ($configs['EBckTargetType']==3)
            {
              if ($configs['EBckSubdir']!='')
              {
                // we will backup into a subdirectory
                // create it if not already existing
                $BackupDest .= '/' . $configs['EBckSubdir'];
                $backup_perm = fileperms($BackupDest);
                mkdir($BackupDest,$backup_perm,true);
              }
            }            
            
					} else
					{
						// we already have an error message created by DoMount when it returns false
						// so superfluous: $this->AddErrMsg('Failed to mount export directory');
						break;
					}
				}
				
				foreach ($calls as $call)
				{
					$dirfn = $call['directory'] . '/' . $call['filename'];
					$src = AUDIO_PATH . '/' . $dirfn;
					$einfo = array();
					$dstfn = GetCustomFileName($configs["ExpCustomFN"],$call['filename'],$call,true);
					$einfo['dstfn'] = $dstfn;
					$dst = $BackupDest . '/' . $dstfn;
					if (file_exists($src)==false)
					{
						lg('Recording file ' . $src . ' does not exist!');
						continue;
					}
					if (copy($src,$dst)===false)
					{
						// already checked that source exists,
						// so this should have worked.
						$this->AddErrMsg('Copy failure of '.$src.' to '.$dst);
						// do not continue
						break;
					} 
					lg('Copied '.$src);
					$anydone = true;
          $ExpScrRec = true;
          if (int_ensure_defined($configs['ExpNotScrRec'])==1) $ExpScrRec = false;
					if ($ExpScrRec && HasScreenRecording($call))
					{
						$dirfn = $call['directory'] . '/' . GetScreenFileName($call);
						$src = AUDIO_PATH   . '/' . $dirfn;
						$screen_dstfn = GetScreenFileName($call);
						if ($configs["ExpCustomFN"]!='')
						{
							// the IF is needed.
							$screen_dstfn = GetCustomFileName('screen'.$configs["ExpCustomFN"],$screen_dstfn,$call,true);
						}
						$einfo['scrdstfn'] = $screen_dstfn;
						$dst = $BackupDest . '/' . $screen_dstfn;
						if (file_exists($src)==false)
						{
							lg('Screen recording file ' . $src . ' is missing');
						} else
						{
							if (copy($src,$dst)===false)
							{
								lg('Copy failure (screen) of '.$src);
							} else // copy was OK
							{
								lg('Copied '.$src);
							}
						}
					} // if has screen recording
					if ($configs['ExpXml'])
					{
						// now create XML
						if ($this->CreateRecInfoXml($BackupDest,$call,$configs,$einfo)==false)
						{
							$this->AddErrMsg('Failed to write XML');
							// do not continue
							break;
						} else
						{
							lg('Written XML');
						}
					}
					$IdHandled = $call['id'];
				}
				
        $dbintf->SetConfigsPart( array('EBckIdHandled'=>$IdHandled) , 'exp' );
			} // end of while loop
		} catch(Exception $e)
		{
			$this->AddErrMsg("Exception: ".$e->getMessage());
		}
		// the TRY-CAST ensure, unmount (umount) is always done
		if ($mounted)
		{
			lg('Unmount');
			$this->DoUnmount();
		}
		// Inform health system about the current state
		$toSet = null;
		if ($this->errflag)
		{
			// Something went wrong
			$toSet = array( 'ExpHealth' => 0
			, 'ExpHStatus' => $this->errmsg
			);
		} else if ($anydone)
		{
			// OK and something done.
			$elapsed = time() - $begintime;
			// If nothing is done, this does not negate previous errors.
			$toSet = array( 'ExpHealth' => 1
			, 'ExpHStatus' => 'Succeeded at ' . date('Y-m-d H:i',time())
			);
		}
		if ($toSet!==null)
		{
			$dbintf->SetConfigsPart($toSet,'exp');
		}
		return $anydone;
	}

	function CreateRecInfoXml($BackupDest,$call, $configs, $einfo)
	{
		global $dbintf;
		
		$dst = $BackupDest . '/' . change_filename_extension($einfo['dstfn'],'.xml');
		
		$info = $dbintf->GetCallInfo($call['id']);
		$callextra = StringToArray($info['callextra']);
		
		$x = new SimpleXMLElement('<callinfo />');
		// using this syntax, it is quoted automatically, amazingly
		$x->startdate = date('Y-m-d H:i:s',$info['startdate']);
    if (isset($info['startms']) && $info['startms']!==null) $x->start_ms = $info['startms'];
		$x->duration = $info['duration'];
    if (isset($info['durms']) && $info['durms']!==null) $x->duration_ms = $info['durms'];
		$x->local = $info['local'];
		$x->remote = $info['remote'];
		$local_name  = $dbintf->GetContactName($info['localcontact' ]);
		$remote_name = $dbintf->GetContactName($info['remotecontact']);
    $connected_name = $dbintf->GetContactName($info['connectedcontact']);

		$x->local_name  = $local_name;
		$x->remote_name = $remote_name;
		if ($info['direction']==0) // incoming
		{
			$x->caller   = $info['remote'];
			$x->receiver = $info['local'];
			$x->caller_name   = $remote_name;
			$x->receiver_name = $local_name;
		} else
		{
			$x->caller   = $info['local'];
			$x->receiver = $info['remote'];
			$x->caller_name   = $local_name;
			$x->receiver_name = $remote_name;
		}

    $x->connected = $info['connected'];
    $x->connected_name = $connected_name;

		// tenant?
		if ($info['tenantid']!=NULL)
		{
			$group = $dbintf->GetGroupInfo( $info['tenantid'] );
			$x->tenant = $group['name'];
		}
		// user
		if ($info['agent']!=NULL)
		{
			$user = $dbintf->GetUserInfoOnAccId($info['agent']);
			$x->user = $user['name'];
      $x->user_email = $user['email'];
		}
		// group list
    $groupMatch = array();
		if ($info['agent']!=NULL)
		{
			$groups = $dbintf->GetGroupsOfWhichMember( $info['agent'] );
			foreach ($groups as $groupAccId)
			{
				$groupMatch[ $groupAccId ] = true;
			}
		} else
		{
			// determine which users match the call
			// second param is whether to exclude not-enabled users
			$users = $dbintf->GetUsersOfCall($info,false);
			// determine which the groups of which these users are member
			foreach ($users as $userAccId)
			{
				// not needed: $user = $dbintf->GetUserInfoOnAccId($userAccId);
				$groups = $dbintf->GetGroupsOfWhichMember( $userAccId );
				foreach ($groups as $groupAccId)
				{
					$groupMatch[ $groupAccId ] = true;
				}
			}
		}
    // add the name of the groups to the list
    $groupList = array();
    foreach ($groupMatch as $groupAccId=>$dummy)
    {
      $group = $dbintf->GetGroupInfoOnAccid( $groupAccId );
			if ($info['tenantid']!=NULL)
			{
				// if the recording belongs to a tenant,
				// then the group must belong to the same tenant
				if ($info['tenantid']!=$group['tenantid']) continue;
			}
      $groupList[] = $group['name'];
    }
    // add to XML comma separated
    $x->groups = implode(',',$groupList);
    // end of group list code block
		$x->direction = $info['direction'];
		$x->category = $info['category'];
		$x->notes = $info['notes'];
		$x->notes2 = $info['notes2'];
		$x->notes3 = $info['notes3'];
		$x->notes4 = $info['notes4'];
		$x->notes5 = $info['notes5'];
		$x->data = str_ensure_defined($callextra['extdata']);
    $x->data1 = $info['data1'];
    $x->data2 = $info['data2'];
    $x->data3 = $info['data3'];
    $x->data4 = $info['data4'];
    $x->data5 = $info['data5'];
		$x->id = $info['remoteid'];
		if ($info['isarchived'])
		{
			$x->recording = $einfo['dstfn'];
			$x->location = str_ensure_defined($configs['ExpXmlDirPrefix']) . $x->recording;
			if (HasScreenRecording($info))
			{
				$x->screenrec = $einfo['scrdstfn'];
				$x->screenloc = str_ensure_defined($configs['ExpXmlDirPrefix']) . $x->screenrec;
			}
		}
		$x->errormsg = '';
		return $x->asXML($dst);
	}
	
	function AddErrMsg($t)
	{
		$this->errflag = true;
		if ($this->errmsg!='') $this->errmsg .= "\n";
		$this->errmsg .= $t;
	}
	
	function reportToAdmin($subj,$msg)
	{
		global $dbintf;
		$dat = array();
		$configs_n = $dbintf->LoadConfigsPart('n');

		if ($configs_n['SMTPhost']=='')
		{
			lg('E-mail is not configured');
			return false;
		}
		if ($configs_n['EmAdmin']=='')
		{
			lg('No admin email address is specified');
			return false;
		}
		$dat['To'] = array( $configs_n['EmAdmin'] , $configs_n['EmAdmin2'] );
		$dat['Subject'] = GetEmailSubjPrefix($dbintf) . $subj;
		$body = "";
		$body .= "\n";
		$body .= $msg . "\n";
		$dat['Body'] = $body;
		return SendEmailData($configs_n,$dat);
	}

  function signalHandler($where,$msg)
  {
    $reply = 1;
    $cmd_timeout = 20;
    $err = signal_service($msg,$reply,$cmd_timeout);
    if ($err!='')
    {
			// logging might be duplicated
      lg($where . ' failed. Socket error: '.$err);
			$this->AddErrMsg($where . ' failed. Socket error: '.$err); 
      return false;
    }
    // more user friendly
    if ($reply=='OK')
    {
      // OK
      return true;
    } else if ($reply[0]=='E')
    {
      lg($where . ' failed: '.$reply);
			$this->AddErrMsg($where . ' failed: '.$reply); 
      return false;
    } else
    {
      lg($where . ' failed, unexpected reply: '.$reply.' LEN: '.strlen($reply));
			$this->AddErrMsg($where . ' failed, unexpected reply: '.$reply.' LEN: '.strlen($reply)); 
      return false;
    }
  }
  
  // for export
	function DoUnmount()
	{
		return $this->signalHandler('Export unmount','umnt E');
	}
	
  // for export
	function DoMount($configs)
	{
		return $this->signalHandler('Export mount','mnt E exp');
	}

	
}


function AzureSyncGroups()
{
  global $dbintf;
  $doneSomething = false;
  $groups = $dbintf->GetGroupList();
	$groupErrors = array();
	$any = false;
  foreach ($groups as $group)
  {
    if ($group['azuregroupid']!='')
    {
			$any = true;
      // only load on demand
      // otherwise there is an error on earlier Debian
      require_once(CMD_WEB_RELPATH . 'azure.php');
			try
			{
				if (AzureSyncGroupE($group)) $doneSomething = true;
			} catch (Exception $e)
			{
				// there was some error
				$groupErrors[] = $e->getMessage() . ' for group ' . $group['name'];
			}
    }
  }
  if ($doneSomething)
  {
    EnginesSettingsCalcApply();
  }
	if ($any)
	{
		$newcfg = array();
		$newcfg['AzGroupSyncHealth'] = (count($groupErrors)==0)?'1':'0';
		$newcfg['AzGroupSyncHStatus'] = implode(',',$groupErrors);
		$dbintf->SetConfigsPart($newcfg,'n');
	}
  return $doneSomething;
}



?>