<?php
error_reporting(E_ALL ^ E_NOTICE);
define('CMD',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "health.php";
require_once CMD_WEB_RELPATH . "emailing.php";
require_once CMD_WEB_RELPATH . "signaling.php";

define('CMD_DATA_PATH',CMD_WEB_RELPATH . DATA_PATH);

$debug_mode = false;
$simulate = false;
$forcemail = false;
for ($i=1;$i<$argc;$i++)
{
	if ($argv[$i]=='debug')
	{
		$debug_mode = true;
	}
	if ($argv[$i]=='sim')
	{
		$simulate = true;
	}
	if ($argv[$i]=='mail')
	{
		$forcemail = true;
	}
}

dbglog('Starting');

$dbintf = null;
$configs = array();

$dbfailure = false;
try
{
  // Interface to database
  $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH, 'apresa' , true);
	$dbintf->SetAdmin();
} catch(Exception $e)
{
  $dbfailure = true;
  $dbError = $e;
}

if ($dbfailure)
{
  HealthDbFailure($dbError);
} else
{
  // Load configuration
  $configs = $dbintf->LoadConfigsPart('n');

  HealthMonitor();
}

dbglog('End');

function SNMPDatabaseError()
{
  return "Database|Connection error|40|Cannot connect to the database|dbcon";
}

function HealthDbFailure()
{
  dbglog("Health check database failure");
  
  if ( @file_exists('/var/apresa/conf/opt_cluster') )
  {
    // if database cannot be reached, it is *bad*
    setNodeScore( -10 );
  }

  $SNMPInfo = SNMPDatabaseError();
	file_put_contents(CMD_DATA_PATH . '/SNMPHealth.new',$SNMPInfo);
  rename(CMD_DATA_PATH . '/SNMPHealth.new' , CMD_DATA_PATH . '/SNMPHealth');
  NotifySNMPAgent();

  
  // TODO: use database fail count
  // so not report immediately (during start-up)
  // also report restore?
  
  /*
  if (isset($health['database']))
  {
    // (int) $health['database']
  }

  if ($dbWasOk)
  {
    dbglog("New database error");
		// report via email
		$body = '';
		$body = '<p>Report date and time: ' . date('Y-m-d H:i:s');
		$body .= '<p>Could not connect to database';
		$serial = (int) file_get_contents( '/var/apresa/conf/serial' );
		$body .= '<p>Software serial number: '.$serial;
		
    $emaildata = array();
		//$emaildata['To'] = array( $configs['EmAdmin'] , $configs['EmAdmin2'] );
		$prefix = 'APRESA ';
    $prefix .= '[SN '.$serial.'] ';
		$emaildata['Subject'] = $prefix . 'Database failure';
		$emaildata['HtmlBody'] = $body;

		//SendEmailDataUnconfig($emaildata);
  }
  
  $health['database'] = '0';
  
  */
}

function NotifySNMPAgent()
{
  $socket = @socket_create(AF_UNIX, SOCK_DGRAM, 0);
  if ($socket===FALSE)
  {
    lg("Error in socket create");
    return;
  } 

  if (@socket_connect($socket, "/tmp/apresa_snmp") === FALSE)
  {
    lg("Could not connect to snmp socket");
    return;
  }

  $message = "act:check\x1e";
  
  if(@socket_write($socket,$message,strlen($message)) === FALSE)
  {
    lg("Could not write to snmp socket");
    return;
  }
}

function HealthMonitor()
{
	global $configs,$debug_mode,$simulate,$forcemail,$dbintf;

	$prev_failure = file_get_contents(CMD_DATA_PATH . '/health_all.dat');
	$prev_failure = (int) substr($prev_failure,6);
	
    $data = UpdateHealthState(CMD_DATA_PATH,$simulate);

	$infotab = GetInfoTable($data->newinfo);
	dbglog('Saving to ' . CMD_DATA_PATH . '/health.htm');
	file_put_contents(CMD_DATA_PATH . '/health.htm.new',$infotab);
	rename(CMD_DATA_PATH . '/health.htm.new' , CMD_DATA_PATH . '/health.htm');

  $alldat = 'Alarm=' . ((int) $data->any_failure);
  file_put_contents(CMD_DATA_PATH . '/health_all.dat.new',$alldat);
	rename(CMD_DATA_PATH . '/health_all.dat.new' , CMD_DATA_PATH . '/health_all.dat');
  
  $SNMPInfo = GetSNMPInfo($data->newinfo, $data->messages, $data->any_failure);
	file_put_contents(CMD_DATA_PATH . '/SNMPHealth.new',$SNMPInfo);
  rename(CMD_DATA_PATH . '/SNMPHealth.new' , CMD_DATA_PATH . '/SNMPHealth');
  NotifySNMPAgent();
  

	if ($prev_failure && !$data->any_failure)
	{
		$data->restore = true;
	}
	
	if ($debug_mode)
	{
		dbglog($infotab);
		dbglog('Messages:');
		foreach ($data->messages as $msg)
		{
			dbglog($msg);
		}
	}

  if($data->lowmem)
  {
    $meminfo = array();
    exec('ps aux --sort -rss | head -n 20',$meminfo);
    $procmem = implode("\n", $meminfo);

    lg("Low on memory: ", LOG_ERR);
    lg($procmem, LOG_ERR);
  }

	if ($data->new_failure || ($data->restore && !$data->any_failure) || $forcemail)
	{
		dbglog('Sending email');
		if ($data->new_failure)
		{
			dbglog("New failure!");
			// also write details to log
			lg('New failure!',LOG_ERR);
			foreach ($data->messages as $msg)
			{
				lg($msg,LOG_ERR);
			}
			foreach ($data->newinfo as $info)
			{
        // 2 == ignored
				if(isset($info['h']) && $info['h']!==true && $info['h']!==2)
				{
					$t = $info['part'] . ': ' . $info['value'] . ' ALARM';
          $err = int_ensure_defined($info['err']);
					if ($err==1) $t .= ', TOO HIGH';
					if ($err==2) $t .= ', TOO LOW';
					lg($t,LOG_ERR);
				}
			}
		}
		if (!$data->any_failure) dbglog("All OK!");

		// report via email
		$body = '';
		$body = '<p>Report date and time: ' . date('Y-m-d H:i:s');
		foreach ($data->messages as $msg)
		{
			$body .= '<p class="alarmmsg">' . $msg;
		}
		$body .= '<style>';
		$body .= 'p.alarmmsg{';
		$body .= 'font-weight:bold;';
		$body .= '}';
		$body .= '</style>';
		$serial = (int) file_get_contents( '/var/apresa/conf/serial' );
		$body .= '<p><p>Software serial number: '.$serial;
		$body .= '<p>' . $infotab;

		$emaildata = array();
		$emaildata['To'] = array( ensure_defined($configs['EmAdmin'], null) , ensure_defined($configs['EmAdmin2'], null) );
		$prefix = 'APRESA ';
    $sysname = $dbintf->GetConfigValue('SystemName');
    if ($sysname!='')
    {
      $prefix .= $sysname . ' ';
    }
    $prefix .= '[SN '.$serial.'] ';
    $prefix .= 'System';
		if ($data->new_failure)
		{
			$emaildata['Subject'] = $prefix . ' Error';
		} else if (!$data->any_failure)
		{
			$emaildata['Subject'] = $prefix . ' All OK';
		} else
		{
			$emaildata['Subject'] = $prefix . ' Status Update';
		}
		$emaildata['HtmlBody'] = $body;

		SendEmailData($configs,$emaildata);
	}
}
?>