<?php

// connection with MICC
class MiccSync
{
	
	function Run($frqtask)
	{
		global $dbintf;
		
		if ((int)$dbintf->GetConfigValue('MiccSync')==0)
		{
			return false;
		}
		
		$runInterval = $dbintf->GetConfigValue('MiccInterval');
		if ($runInterval=='') $runInterval = 2;
		else $runInterval = (int) $runInterval;
		
		if ($frqtask->runcnt % $runInterval != 0)
		{
			return false;
		}
		
		$configs_o = $dbintf->LoadConfigsPart('o');
		$Server = '' . $configs_o['MiccServer'];
		$Db     = '' . $configs_o['MiccDb'];
		$Usr    = '' . $configs_o['MiccUsr'];
		$Pwd    = '' . decrypt($configs_o['MiccPwd']);
		
		$users = $dbintf->GetUserList();
		$agents = [];
		foreach ($users as $user)
		{
			$agent = $user['usr'];
			if ($agent=='') continue;
			$agents[ $agent ] = true;
		}
		unset($users);
		
		lg('MICC Server:'.$Server.' Database:'.$Db.' User:'.$Usr);
		
		$pdo_dsn = "dblib:host=".$Server.";dbname=".$Db;
		
		$maxts = int_ensure_defined( $configs_o['MiccTs'] );
		/*
		if ($maxts==0)
		{
			$begin2022 = strtotime('2022-01-01');
			$now = time();
			// go two days back
			$maxts = ($now - $begin2022) - 2 * 24*60*60;
		}
		*/
		
		$logonmap = [];
		// load logon map from database agent map
		$items = explode(',',$configs_o['CSTAAgentMap']);
		foreach ($items as $item)
		{
			$parts = explode(':',$item);
			if (count($parts)==2)
			{
				$ext = $parts[0];
				$agent = $parts[1];
				if (isset($agents[$agent])==false)
				{
					// not known as Apresa user, so skip
					continue;
				}
				$logonmap[ $agent ] = $ext;
			}
		}
		
		lg('Trying to connect to MICC database...');
		$msdb = new PDO($pdo_dsn, $Usr, $Pwd);
		$msdb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		lg('Connected');
		// get users that are logged off
		$sql = "SELECT * FROM cc_user "
		. "WHERE delete_date < GetDate() "
		. ";";
		$stmt = $msdb->prepare($sql);
		if ($stmt->execute()==false) throw new exception("Error in query 1");
		while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$agent = $row['external_login_id'];
			unset( $logonmap[ $agent ] );
		}
		// get latest logon actions
		$sql = "SELECT cc_user.external_login_id"
		. ", agt_logon_act.time_from "
		. ", datediff(ss,'2022-01-01',agt_logon_act.time_from) AS ts "
		. ", agt_logon_act.agt_logon_ext "
		. "FROM agt_logon_act "
		. "INNER JOIN cc_user on agt_logon_act.agent_id = cc_user.id "
		. "WHERE agt_logon_act.time_from >= DATEADD(ss,$maxts,cast('2022-01-01' AS DATETIME)) "
		. "ORDER BY time_from "
		. ";";
		$stmt = $msdb->prepare($sql);
		if ($stmt->execute()==false)
		{
			throw new exception("Error in query 2");
		}
		$rowcnt = 0;
		while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			//var_dump($row);
			$ts = (int) $row['ts'];
			if ($ts>$maxts) $maxts = $ts;
			verbose_lg( $row['time_from'] . ' (' . $row['ts'] . ') ' . $row['agt_logon_ext'] . ' = ' . $row['external_login_id']);
			$phone = $row['agt_logon_ext'];
			$agent = $row['external_login_id'];
			if (strlen($phone)>5) $phone = '';
			if (isset($agents[$agent]))
			{
				// it is an Apresa user
				if ($phone=='') unset( $logonmap[ $agent ] );
				else $logonmap[ $agent ] = $phone;
			}
			$rowcnt ++;
		}
		lg('Read ' . $rowcnt . ' rows');
		// clear reference (close connection)
		$msdb = null;
		
		$monitorList = $logonmap;
		// add Apresa telephones to monitor list
		$ApresaNr = $configs_o['CSTAApresaNr'];
		$ApresaParts = explode(',',$ApresaNr);
		foreach ($ApresaParts as $ApresaPart)
		{
			$dashPos = strpos($ApresaPart,'-');
			if ($dashPos!==false)
			{
				// range
				$first = (int) substr($ApresaPart,0,$dashPos);
				$last = (int) substr($ApresaPart,$dashPos+1);
				$limit = 1000;
				$cnt = 0;
				if ($last-$first>$limit) continue; // too many
				for ($val=$first;$val<=$last;$val++)
				{
					$monitorList[] = $val;
				}
			} else
			{
				// single thing
				$item = trim($ApresaPart);
				if ($item!='') $monitorList[] = $item;
			}
		}
		$CstaList = implode(',',$monitorList);
		verbose_lg( 'CSTA list: ' . $CstaList );
		$agentarr = [];
		foreach ($logonmap as $user => $phone)
		{
			if ($phone=='') continue;
			if ($user=='') continue;
			$agentarr[] = $phone . ':' . $user;
		}
		$agentMap = implode(',',$agentarr);
		verbose_lg( 'Agent map: ' . $agentMap);
		
		$needReconf = false;
		
		$newconf = [];
		// store timestamp of latest row
		$newconf['MiccTs'] = $maxts;
		if ($configs_o['CSTATelIPFilter']!=$CstaList)
		{
			lg('New CSTA list: ' . $CstaList);
			$newconf['CSTATelIPFilter'] = $CstaList;
			$needReconf = true;
		} else verbose_lg('CSTA list already up to date');
		if ($configs_o['CSTAAgentMap']!=$agentMap)
		{
			lg('New CSTA agent map: ' . $agentMap);
			$newconf['CSTAAgentMap'] = $agentMap;
			$needReconf = true;
		} else verbose_lg('CSTA agent map already up to date');
		$dbintf->SetConfigsPart($newconf,'o');
		
		if ($needReconf)
		{
			lg('Signal engine');
			// save to XML
			$frqtask->signalHandler('Micc','os');
			// let engine reread XML
			$frqtask->signalHandler('Micc','sgnork reconfig');
		}
		
		lg('End of MICC handling');
		return true;
		
	}
	
}