<?php
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";

function DoSyncSamwinGroups()
{
	global $dbintf;
	
	lg('Sync Samwin groups starts');
	$anydone = false;
	
	// get general Samwin configuration
	$configs_s = $dbintf->LoadConfigsPart('s');
	$SamwinServer = '' . $configs_s['SamwinServer'];
	$SamwinDb     = '' . $configs_s['SamwinDb'];
	$SamwinUsr    = '' . $configs_s['SamwinUsr'];
	$SamwinPwd    = '' . decrypt($configs_s['SamwinPwd']);
	
	lg('Server:'.$SamwinServer.' Database:'.$SamwinDb.' User:'.$SamwinUsr);
	
	$pdo_dsn = "dblib:host=".$SamwinServer.";dbname=".$SamwinDb;

	// connection will be initiated when needed
	$msdb = null;

	$allgroups = $dbintf->GetGroupList();
	
	foreach ($allgroups as $group)
	{
		if ($group['samwingroup']=='') continue;
		$SamwinGroup = $group['samwingroup'];
		lg('Apresa group:'.$group['name'].' Samwin group:'.$SamwinGroup);
		
		if ($msdb==null)
		{
			$anydone = true;
			lg('Trying to connect to Samwin database...');
			$msdb = new PDO($pdo_dsn, $SamwinUsr, $SamwinPwd);
			lg('Connected');
		}
		
		// get members of the group
		$stmt = $msdb->prepare('SELECT '
		. ' tbl_usr.id as uid, '
		. ' tbl_usr.fld_usrnam as usr, '
		. ' tbl_usr.fld_firstname as firstname, '
		. ' tbl_usr.fld_lastname as lastname, '
		. ' tbl_usr.fld_email as email '
		. ' FROM tbl_groups '
		. ' JOIN tbl_group_members ON tbl_groups.id = tbl_group_members.fld_groupid '
		. ' JOIN tbl_usr ON tbl_usr.id = tbl_group_members.fld_userid '
		. ' WHERE tbl_groups.fld_name = ? '
		);
		$stmt->bindValue(1,$SamwinGroup);
		if ($stmt->execute()==false)
		{
			throw new exception("Error in query");
		}
		
		// read in the users of this group from the query
		$users = array();
		while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			//lg('Found user:'.$row["usr"].' '.$row['email']);
			
			$usrdata = array();
			$usrdata['usr'] = $row["usr"];
			$usrdata['name'] = trim($row["firstname"] . ' ' . $row['lastname']);
			$usrdata['email'] = $row["email"];
			
			$uinfo = array();
			$uinfo['dat'] = $usrdata;
			$uinfo['uid'] = $row['uid'];
			
			$users[] = $uinfo;
		}
		
		$members = array();
		
		// import each user we have found
		foreach ($users as $uinfo)
		{

			$usrdata = $uinfo['dat'];
			$userid = null;
			$accid = null;
			
			if ($dbintf->UsrExists($usrdata['usr'])==false)
			{
				// this is a new user
				// set defaults
				$usrdata['mphones'] = '';
				$usrdata['enabled'] = 1;
				$usrdata['rod'] = 0;
				// for Samwin, LDAP login
				$usrdata['logonmethod'] = 1;
				$usrdata['playtimelim'] = 0;
				$usrdata['ldapuser'] = 0;
				$usrdata['ldaphost'] = '';
				$usrdata['samwinuser'] = 1;	 //User added because of samwin sync
				lg('Adding user: '.$usrdata['usr'].' ('.$usrdata['name'].')');
				$isok = $dbintf->AddUserAccount($userid,$usrdata,0);
				$accid = $dbintf->GetUserInfo($userid)['accid'];
			} else
			{
				// the user already exists
				// get existing info
				$existing = $dbintf->GetUserInfoOnUsername($usrdata['usr']);
				// update user?
				if ($existing['email'] != $usrdata['email']
				|| $existing['name'] != $usrdata['name'])
				{
					lg('Editing user: '.$usrdata['usr'].' ('.$usrdata['name'].')');
					$dbintf->EditUserAccountLC($existing['id'],$usrdata);
				}
				
				$accid = $existing['accid'];
			}
			$members[] = $accid;
			
			// the phone numbers
			$phone_nums = array();
			// the User ID is used as phone number for Samwin
			$phone_nums[] = $uinfo['uid'];
			// convert the phone numbers to contacts
			$phones = array();
			$newphones = array();
			foreach ($phone_nums as $num)
			{
				// fill in name
				$res = $dbintf->GetOrCreateCallerID($num,'',1,$usrdata['name']);
				$phones[] = array('contactid'=>$res['contactid']);
				$newphones[] = $res['contactid'];
			}				
			$oldphones = $dbintf->GetUserPhones($accid);
			if (count(array_diff($newphones,$oldphones))!=0
			|| count(array_diff($oldphones,$newphones))!=0)
			{
				lg('Phones list changed for user: '.$usrdata['usr']);
				// the list of phones: implode(',',$phone_nums));	
				$dbintf->SetUserPhones($accid,$phones);
			}
			
		} // end of user loop

		$oldmembers = $dbintf->GetGroupMembers($group['accid']);
		//Are there any useraccounts made by samwin sync and not part of any group anymore?
		$delusers = array_diff($oldmembers, $members);
		//SetGroupMembers is quite slow!!!
		if (count($delusers) != 0 || (count($oldmembers) != count($members)))
		{
			lg('The group now has '.count($members).' members');
			$dbintf->SetGroupMembers($group['accid'],$members);
		} else
		{
			lg('Group membership unchanged');
		}

		foreach ($delusers as $deluser)
		{
			$user = $dbintf->GetUserInfoOnAccId($deluser);
			lg('Removing '.$user['usr'].' from group');
			if ($user['samwinuser']==1)
			{
				$dbgroups = $dbintf->GetGroupsOfWhichMember($deluser);
				lg('It is still member of '.count($dbgroups).' groups');
				if (count($dbgroups)==0)
				{
					lg("Removing user: ".$user['usr']);
					//The user account was created by Samwin sync,
					// and is no longer member of any group => delete
					$dbintf->DeleteUserAccount($user['id']);
				}
			}
		}
		
		// force recalc
		$dbintf->userphones = array();
		unset($dbintf->allgroups);
		
	}
	
  EnginesSettingsCalcApply();
		
	$msdb = null;
	lg('Sync Samwin group ended');
	
	return $anydone;
}








?>