<?php
error_reporting(E_ALL ^ E_NOTICE);
define('CMD',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "actions.php";
require_once CMD_WEB_RELPATH . "utility.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "tenancy.php";
require_once CMD_WEB_RELPATH . "mitelimport.php";
require_once CMD_WEB_RELPATH . "reportscheduling.php";
require_once CMD_WEB_RELPATH . "certificates_general.php";
require_once CMD_WEB_RELPATH . "signaling.php";

define('MSG_INF',0);
define('MSG_ERR',1);
define('MSG_WRN',2);

/*
	This file is run each day during the night.
	It implements infrequent tasks.
*/

// Interface to database
$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);

// load all
$configs = $dbintf->LoadConfigs();

if (str_ensure_defined($argv[1])=='debug')
{
	// logging to console instead of apresa.log
	define('DEBUG_LOGING',1);
}

AutoDelete();
TenantsAutoDelete();
ClusterCleanup();
MitelImport();
CustomLocalIdCleanup();
ReportSchedules();
CleanupActive();
CleanupTmp();
LetsEncryptRenew();

function ClusterCleanup()
{
	// check if clustering is active
	if (@file_exists(dirname(__FILE__) . '/../conf/opt_cluster')==false)
	{
		return;
	}
	$now = time();
	// check if this pg archiving directory is there
	$pgArchiveDir = '/database/db/pg_archive';
	$oneDay = 24 * 60 * 60;
	if (file_exists($pgArchiveDir))
	{
		lg("Scanning $pgArchiveDir");
		$files = scandir($pgArchiveDir);
		foreach ($files as $relfn)
		{
			// skip any files with dot in it ?
			if (strpos($relfn,'.')!==false)
			{
				// no: continue;
			}
			$fullfn = $pgArchiveDir . '/' . $relfn;
			if (is_file($fullfn)==false)
			{
				// skip non-files (directories)
				continue;
			}
			$modtime = filemtime($fullfn);
			$age = $now - $modtime;
			$act = 'KEEP';
			if ($age > $oneDay)
			{
				$act = 'DELETE';
			}
			lg($relfn . ' is not modified for ' . $age . ' seconds : ' . $act);
			if ($act=='DELETE')
			{
				// actually delete
				DeleteFn($fullfn);
			}
		}
		lg('Scanning completed');
	}	
}

function CustomLocalIdCleanup()
{
  global $dbintf, $configs;
  // delete expired custom local id mappings
  $time = time() - 30 * 60;
  $dbintf->ExpireLocalIdMapping($time);
}

function MitelImport()
{
	global $dbintf, $configs;
	if(int_ensure_defined($configs['EnableMitel']))
	{
		$oldpm = $dbintf->actuinfo['permissions'];
		$dbintf->SetAdmin();
		DoMitelImport();
		$dbintf->actuinfo['permissions'] = $oldpm;
	}
}

function CleanupActive()
{
  global $dbintf;
  $now = time();
  // spin call might otherwise linger forever if never uploaded
  $dbintf->DeleteActiveCallSpin( $now - 60 * 60 * 24 );
}

function CleanupTmp()
{
	$now = time();
	$tmpDir = '/tmp';
	$oneDay = 24 * 60 * 60;
	if (file_exists($tmpDir)==false) return;
  lg("Scanning $tmpDir");
  $files = scandir($tmpDir);
  foreach ($files as $relfn)
  {
    // skip any files with dot in it ?
    if (strpos($relfn,'.')!==false)
    {
      // no: continue;
    }
    $fullfn = $tmpDir . '/' . $relfn;
    if (is_file($fullfn)==false)
    {
      // skip non-files (directories, sockets)
      continue;
    }
    // cleaning up files starting with apresa
    //    these are used for MP3 conversion
    if ( substr($relfn,0,6)=='apresa') ;
		else if ( startsWith($relfn,'php') && endsWith($relfn,'.upl') ) ;
		else continue;

    $accesstime = fileatime($fullfn);
    $age = $now - $accesstime;
    $act = 'KEEP';
    if ($age > $oneDay)
    {
      $act = 'DELETE';
    }
    if ($act=='DELETE')
    {
      lg($relfn . ' is not accessed for ' . $age . ' seconds : ' . $act);
      // actually delete
      DeleteFn($fullfn);
    }
  }
  lg('Scanning completed');
}

function ReportSchedules()
{
  global $dbintf;
  $dbintf->SetAdmin();
  LoadBasicSettings();
  $reports = $dbintf->GetReportSchedules();
  foreach ($reports as $report)
  {
    // not force, and act is run
    PerformReportSchedule($report,false,'run' );
  }
}

function LetsEncryptRenew()
{
	global $dbintf;
	$dbintf->SetAdmin();

	$AutoRenewal = $dbintf->GetConfigValue("LetsEncryptAutoRenewal");

	if(!$AutoRenewal)
	{
		lg("Let's Encrypt auto renewal is disabled");
		return;
	}

	$RegLetsEncrypt = $dbintf->GetConfigValue('RegLetsEncrypt');

	if(!$RegLetsEncrypt)
	{
		lg("Trying to renew Let's Encrypt certificates, but no account has been registered");
	}

	$certs = $dbintf->GetExpiringLetsEncryptCerts(30);

	$report = [];

	$err = false;

	foreach($certs as $cert)
	{
		lg("Trying to renew the soon expiring cert: $cert[name] ($cert[id])");
		try 
		{
			RenewLetsEncrypt($cert['id']);
			$certinfo = $dbintf->GetCertificate($cert['id']);

			chmod(CERTDIR . $certinfo['cert_location'], 0666);
			chmod(CERTDIR . $certinfo['key_location'], 0400);

			if(isset($certinfo['intermediate_location']))
			{
				chmod(CERTDIR. $certinfo['intermediate_location'], 0666);
			}

			if($certinfo['webkey'])
			{
				chown(CERTDIR . $certinfo['key_location'], 'www-data');
				chgrp(CERTDIR . $certinfo['key_location'], 'www-data');
			}
			else 
			{
				chown(CERTDIR . $certinfo['key_location'], 'root');
				chgrp(CERTDIR . $certinfo['key_location'], 'root');
			}

			if(file_exists(CERTDIR . "lets_encrypt_old/" . $certinfo['key_location']))
			{
				chown(CERTDIR . "lets_encrypt_old/" . $certinfo['key_location'], 'root');
				chmod(CERTDIR . "lets_encrypt_old/" . $certinfo['key_location'], 0400);
			}
			
			$report[] = "The Let's Encrypt certificate '$certinfo[name]' has been renewed";
			
			addToSay(MSG_INF,end($report));
			lg(end($report));

			$uses = $dbintf->GetCertificateUses($certinfo['id']);
  		foreach($uses as $use)
  		{
        $msg = [];
    		if($use['usedfor'] == "Https")
    		{
          if(!msg_signal_service($msg, "xcertmake"))
          {
            throw new Exception ("Failed to apply renewed Let's Encrypt HTTPS certificate '$certinfo[name]':" . end($msg)->text);
          }

          if(GetDebianVersion() >= 10)
          {
            $report[] = "The HTTPS server certificate '$certinfo[name]' has been renewed.";
            msg_signal_service($msg, "nl");
          }
          else
          {
            $report[] = "The HTTPS server certificate '$certinfo[name]' has been renewed. The network component must be restarted before the new certificate is used.";
          }
					
      		addToSay(MSG_INF, end($report));
    		}
    		if($use['usedfor'] == "Telsys" || $use['usedfor'] == "TelsysClient")
    		{
          if(!msg_signal_service($msg, "telsys apply"))
          {
            throw new Exception ("Failed to apply renewed Let's Encrypt VoIP Service certificate '$certinfo[name]':" . end($msg)->text);
          }

					$report[] = "The VoIP service TLS certificate '$certinfo[name]' has been renewed.";
      		addToSay(MSG_INF, end($report));
    		}
        if($use['usedfor'] == "TunnelTLS")
        {
          if(!msg_signal_service($msg, "tunneltlscert"))
          {
            throw new Exception ("Failed to apply renewed Let's Encrypt for V-Tap over TLS '$certinfo[name]':" . end($msg)->text);
          }

          $report[] = "The V-Tap over TLS certificate '$certinfo[name]' has been renewed. Restart the recording component to apply it";
      		addToSay(MSG_INF, end($report));
        }
			}
			
		} 
		catch (Exception $ex) 
		{
			$report[] = "Failed to renew certificate '$cert[name]': " . $ex->getMessage();
			addToSay(MSG_ERR, end($report));
			lg(end($report));
			$err = true;
		}
	}

	if(sizeof($report) == 0) return;
		
	$configs_n = $dbintf->LoadConfigsPart('n');
  $to = array();
	if(str_ensure_defined($configs_n['EmAdmin']) != "")
	{
		$to[] = $configs_n['EmAdmin'];
	}

	if(str_ensure_defined($configs_n['EmAdmin2']) != "")
	{
		$to[] = $configs_n['EmAdmin2'];
	}

	if(sizeof($to) == 0)
	{
		lg("No administrator email set, sending no email report of Let's Encrypt certificate renewal");
    return;
	}

	$serial = (int) file_get_contents('../conf/serial');

	$sysname = $dbintf->GetConfigValue('SystemName');
	if ($sysname=='')
	{
		$sysname = $dbintf->GetConfigValue('IPname');
	}
	if ($sysname=='')
	{
		$sysname = 'Apresa';
	}

	$subject = $sysname . ' [SN ' . $serial .'] ' . "Let's Encrypt certificates have been renewed";

	$html = '';

	if($err)
	{
		$html .= "<p> Some certificates could not be renewed </p>";
	}

	foreach($report as $line)
	{
		$html .= "<p> " . $line . " </p>";
	}

	$dat = array();
	$dat['To'] = $to;
	$dat['Subject'] = $subject;
	$dat['HtmlBody'] = $html;
	
	EmailingWebOrCmd($dat);
}


?>