<?php

require_once dirname(__FILE__)."/../conf/voip_config.php";
require_once dirname(__FILE__)."/". CMD_WEB_RELPATH . "db.php";
require_once dirname(__FILE__)."/". CMD_WEB_RELPATH . "encryption.php";
require_once dirname(__FILE__)."/". CMD_WEB_RELPATH . "signaling.php";
require_once dirname(__FILE__)."/". CMD_WEB_RELPATH . "utility.php";

MainLoop();

function slog($t)
{
  global $shore;
  if ($shore->logging)
  {
    lg($t);
  }
}

function MainLoop()
{
  global $shore, $argv;
  $shore = new Shore();
  slog('ShoreTel php start');
	$shore->shoredb = null;
	$shore->shoredb2 = null;
  $shore->connect();
  
	if (isset($argv[1]) && $argv[1]=='simulate')
	{
		define('DEBUG_LOGING',1);
		while (true)
		{
			$reply = '';
			try
			{
				$reply = $shore->handle('');
			} catch(Exception $e)
			{
				lg("ShoreTel exc: " . $e->getMessage() );
				$shore->setStatus(false,"Error: " . $e->getMessage());
				echo "ERROR\n";
				break;
			}
			// our answer
			slog('ShoreTel answer: ' . $reply);
			sleep(1);
		}
	}
	
  // endless loop
  // calling process has to terminate us in some way (exit command)
  // or when we reach end of stream
  while (true)
  {
    $line = fgets(STDIN);
    if ($line===FALSE)
    {
      // we reached end of stream/file
      break;
    }
    $query = str_replace("\n",'',$line);
    if ($query=='exit' || $query=='x')
    {
      echo "Exiting\n";
      break;
    }
    $reply = '';
    try
    {
      $reply = $shore->handle($query);
    } catch(Exception $e)
    {
      lg("ShoreTel exc: " . $e->getMessage() );
      $shore->setStatus(false,"Error: " . $e->getMessage());
      echo "ERROR\n";
      break;
    }
    // our answer
    slog('ShoreTel answer: ' . $reply);
    echo $reply . "\n";
    // and on to the next round / question
  }
  $shore->shutdown();
  slog('ShoreTel end');
}

class ShoreCall
{
  var $local;
  var $remote;
  // var $localip;
  // var $direction;
}

class Shore
{
  var $dbintf = null;
  var $cfg = null;
  var $ipmap = array();
  var $calls = array();
  var $lastStatus = '';
  var $lastStatusMsg = '';
  
  function __construct ()
  {
    $this->dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
    $this->cfg = $this->dbintf->GetConfigsWithPrefix('ShoreTelDb_');
    $this->logging = int_ensure_defined( $this->cfg['Logging'] )!=0;
  }

  function setStatus($ok,$msg)
  {
    if ($this->lastStatus === $ok
    && $this->lastStatusMsg === $msg)
    {
      // it is all the same, nothing to do
      // this reduces load on database
      return;
    }
    $this->lastStatus = $ok;
    $this->lastStatusMsg = $msg;
    $this->dbintf->SetConfigsPart(
      array 
      ( 'ShoreTelDb_Status'    => ($ok?'1':'0')
      , 'ShoreTelDb_StatusMsg' => $msg
      ) , 'o'
    );
    slog('Status:'.($ok?'1':'0').' Msg:'.$msg);
  }
  
  function connect()
  {
    $host = $this->cfg['Host'];
    $port = $this->cfg['Port'];
    $dbname = $this->cfg['DbName'];
		$dbname2 = $this->cfg['DbName2'];
    $username = $this->cfg['Usr'];
    $password = decrypt($this->cfg['Pwd']);
		$this->mode = $this->cfg['Mode'];
    // apply defaults same as in systemopt.php
    if ($port=='') $port = '4308';
    if ($dbname=='') $dbname = 'Shorewarestatus';
		if ($this->mode=='') $this->mode = 'internal';
		// dbname2 is normally Shoreware
		$this->secondDb = ($dbname2!='');
    
    // without host, dont even try
    if ($host=='') return;
    try
    {
			if ($this->shoredb == null)
			{
				$connectStr = "mysql:host=$host;port=$port;dbname=$dbname";
				slog($connectStr);
				$this->shoredb = new PDO($connectStr,$username,$password
					, array( PDO::ATTR_TIMEOUT => "5" , PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION ) );
			}
			if ($this->secondDb && $this->shoredb2 == null)
			{
				$connectStr2 = "mysql:host=$host;port=$port;dbname=$dbname2";
				slog($connectStr2);
				$this->shoredb2 = new PDO($connectStr2,$username,$password
					, array( PDO::ATTR_TIMEOUT => "5" , PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION ) );
			}
    } catch (Exception $e)
    {
      // cannot connect for whatever reason
      lg("ShoreTel db connect error: " . $e->getMessage());
      $this->setStatus(false,"Connect error: " . $e->getMessage());
    }
  }
  
  function shutdown()
  {
    $this->dbintf->close();
    $this->shoredb = null;
    $this->shoredb2 = null;
  }
	
	function CleanupRemote($tel)
	{
		$plusPos = strpos($tel,'+');
		if ($plusPos===1)
		{
			$tel = substr($tel,1);
		}
		return $tel;
	}
  
  function determineCalls()
  {
    // reconnect if we dont have connection
    if ($this->shoredb==null) $this->connect();
    if ($this->shoredb==null) return;
    // Determine if connected
    $stmt = $this->shoredb->prepare('SELECT 1');
    $stmt->execute();
    if ($stmt->fetch(PDO::FETCH_ASSOC)===false)
    {
      $this->shoredb = null;
      $this->connect();
    }
    if ($this->shoredb==null) return;
		
		// second database
		if ($this->secondDb)
		{
			if ($this->shoredb2==null) $this->connect();
			if ($this->shoredb2!=null)
			{
				// Determine if connected
				$stmt = $this->shoredb2->prepare('SELECT 1');
				$stmt->execute();
				if ($stmt->fetch(PDO::FETCH_ASSOC)===false)
				{
					$this->shoredb2 = null;
					$this->connect();
				}
			}
			// no connection is not fatal - use only main then
		}
		
		if ($this->mode=='internal')
		{
			// Determine mapping from local phone number to an IP address
			$dnCount = 0;
			$dnFilled = 0;
			$this->ipmap = array();
			$softphones = array();
			$stmt = $this->shoredb->prepare('SELECT DN,InternetAddress,PhoneType FROM heapportstatus');
			$stmt->execute();
			while ( ($row = $stmt->fetch(PDO::FETCH_ASSOC)) !== false )
			{
				$dnCount ++;
				if ($row['DN']!='')
				{
					$dnFilled ++;
					if ($row['PhoneType']==20)
					{
						// in case of a soft phone,
						// use other table
						$softphones[ $row['DN'] ] = true;
					}
					$this->ipmap[ $row['DN'] ] = $row['InternetAddress'];
				}
			}
			// Determine IP address of softphones
			$portsCount = 0;
			$portsFilled = 0;
			if ($this->secondDb && $this->shoredb2 != null)
			{
				$stmt = $this->shoredb2->prepare('SELECT CurrentDN,PublicIPAddress FROM ports');
				$stmt->execute();
				while ( ($row = $stmt->fetch(PDO::FETCH_ASSOC)) !== false )
				{
					$portsCount ++;
					if ($row['CurrentDN']!=''
					&& $row['PublicIPAddress']!=NULL )
					{
						$portsFilled ++;
						$dn = $row['CurrentDN'];
						if (isset( $softphones[$dn] ))
						{
							$this->ipmap[ $dn ] = $row['PublicIPAddress'];
						}
					}
				}
			}
		}
		// debug: slog( var_export( $this->ipmap , true ) );
    // Determine active calls
		$trunkCount = 0;
		$trunkFilled = 0;
    $this->calls = array();
    $stmt = $this->shoredb->prepare('SELECT InsideID,OutsideID,CallState FROM heaptrunkstatus');
    $stmt->execute();
		$localInList = array();
    while ( ($row = $stmt->fetch(PDO::FETCH_ASSOC)) !== false )
    {
			$trunkCount++;
			$caller = $row['InsideID'];
			$receiver = $row['OutsideID'];
			$callState = $row['CallState'];
			if ($caller=='' && $receiver=='') continue;
			$trunkFilled++;
			if (strpos($callState,'Connected')!==false)
			{
				$call = new ShoreCall();
				$direction = -1;
				if ($this->mode=='trunk')
				{
					$callerLocal   = strlen($caller)>=1 && strlen($caller)<=5;
					$receiverLocal = strlen($receiver)>=1 && strlen($receiver)<=5;
				} else if ($this->mode=='internal')
				{
					$callerLocal   = isset( $this->ipmap[ $caller   ] );
					$receiverLocal = isset( $this->ipmap[ $receiver ] );
				}
				if ($callerLocal)
				{
					if ($receiverLocal) $direction = 2; // internal
					else $direction = 1; // outgoing
				} else if ($receiverLocal)
				{
					$direction = 0; // incoming
				}
				// neither party is local - weird
				if ($direction == -1) continue;
				
				if ($direction==1 || $direction==2)
				{
					$call->local  = $caller;
					$call->remote = $receiver;
				} else
				{
					$call->local  = $receiver;
					$call->remote = $caller;
				}
				$call->remote = $this->CleanupRemote($call->remote);
				if ($this->mode=='internal')
				{
					if (isset( $this->ipmap[ $call->local ] )==false) continue;
				}
				// debug: slog( 'L=' . $call->local . ' R=' . $call->remote . ' cs=' . $callState );
				// no double entries for the same local caller
				if (isset( $localInList[ $call->local ] )==true)
				{
					$bestCallState = 'Connected/NearEndAns';
					if ($callState==$bestCallState
					&& $localInList[ $call->local ]!=$bestCallState )
					{
						// debug: slog( 'Call state ' . $callState . ' better than ' . $localInList[ $call->local ] . ' for ' . $call->local );
						// new call state is better
						// remove previous entry
						$todel = array();
						foreach ($this->calls as $ix => $otherCall)
						{
							if ($otherCall->local == $call->local)
							{
								// debug: slog('to del ix = ' . $ix);
								$todel[] = $ix;
							}
						}
						foreach ($todel as $ix)
						{
							unset( $this->calls[ $ix ] );
						}
						// better entry will then be added below
						// no continue here
					} else
					{
						// debug: slog( 'Prevent dup entry for ' . $call->local );
						continue;
					}
				}
				if ($this->mode=='internal')
				{
					$call->localip = $this->ipmap[ $call->local ];
				}
				$call->direction = $direction;
				$this->calls[] = $call;
				$localInList[ $call->local ] = $callState;
			}
    }
		$this->calls = array_values( $this->calls );
    $this->setStatus(true,"OK");
		$logEntry = "[v5] $this->mode heapportstatus $dnFilled / $dnCount , heaptrunkstatus $trunkFilled / $trunkCount";
		if ($this->secondDb && $this->shoredb2 != null)
		{
			$logEntry .= " , ports $portsFilled / $portsCount";
		}
		slog($logEntry);
  }
  
  function handle($query)
  {
    if ($query=='status' || $query=='')
    {
      $this->determineCalls();
      if ($this->shoredb==null)
      {
        return 'Error NoConnection';
      }
      return json_encode( $this->calls );
    } else
    {
      return "Error UnknownCommand";
    }
  }
  
}


