<?php
/*
Spin. For multi-Apresa setups. Enterprise.
Client mode:
  It listens for data from trackunit,
  and sends it along to a central server (the server).
It is started and managed by apresa-remote.
*/

// without CMD it cannot load language
define('CMD',1);
// to get logging in other remote log file
define('CMD_REMOTE',1);

use phpseclib\Crypt\RSA;

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "utility.php";
require_once CMD_WEB_RELPATH . "datafile.php";
require_once CMD_WEB_RELPATH . "actions.php";
require_once CMD_WEB_RELPATH . "language.php";
require_once "stream-socket-app.php";

require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Crypt/Random.php' );
require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Crypt/RSA.php' );
require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Crypt/Hash.php' );
require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Math/BigInteger.php' );

if (str_ensure_defined($argv[1])=='debug')
{
	// logging to console instead of apresa.log
	define('DEBUG_LOGING',1);
}
$logging_prefix = 'spin-client';

$exit = false;

#region Signal handling

function sig_handler($signo)
{
  global $exit;
  lg("sc Signal $signo");
  if ($signo==SIGINT
  || $signo==SIGTERM)
  {
    $exit = true;
  }
}

pcntl_signal(SIGINT, 'sig_handler');
pcntl_signal(SIGTERM, 'sig_handler');

#endregion

// connection with trackunit
class TrackerConn extends Connection
{
  function __construct()
  {
    $this->logShutdown = false;
  }
}

// connection with a remote spin-server
class RemoteConn extends Connection
{
}

class TrackerListener extends Listener
{
  function createConnection($newSock)
  {
    $conn = new TrackerConn();
    $conn->sock = $newSock;
    return $conn;
  }
}

// Client will connect to Server,
// and send active call data to it.
// It listens for info from trackunit.
class SpinClient extends StreamSocketApp
{
  var $maindb;
  var $server = NULL;
  var $lastConnectTime = 0;
  var $expacalls = 0;
  var $acceptact = 0;
  var $importrecconf = 0;
  var $lastImportConfTime = 0;
  var $serverIp;
  var $serialnum;
  
  function Init()
  {
    global $dbintf;
		$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
    $this->SetUnknownStatus('Starting');
    $this->maindb = $dbintf;
    $transferlist = $dbintf->GetTransferTargets();
    $foundOne = false;
    foreach ($transferlist as $transfer)
    {
      if ($transfer['enabled']==0) continue;
      if ($transfer['expacalls']!=0
      || $transfer['acceptact']!=0
      || $transfer['importrecconf']!=0)
      {
        // options indicate we need spin handle this one
        if ($foundOne)
        {
          lg('ERROR: supporting one target, more than one found');
          continue;
        }
        $foundOne = true;
        $this->serverIp = $transfer['ip'];
        $this->expacalls = $transfer['expacalls'];
        $this->acceptact = $transfer['acceptact'];
        $this->importrecconf = $transfer['importrecconf'];
      }
    }
    $this->serialnum = GlobGetSerialNumber();
    
    $ipc_path = '/tmp/apresa_spin';
    unlink($ipc_path);
    $newSock = stream_socket_server("unix://" . $ipc_path);
    if ($newSock===false)
    {
      lg("Error starting unix domain socket server");
      $this->SetErrorStatus('unix domain socket');
      exit;
    }
    // otherwise clients cannot write to it
    chmod($ipc_path,0766);
    $listener = new TrackerListener();
    $listener->sock = $newSock;
    $this->listeners[] = $listener;    
    
    $this->ConnectToServer();
  }
  
  function SetOkStatus()
  {
    $this->SetStatus('OK');
  }
  
  function SetUnknownStatus($msg)
  {
    $this->SetStatus('UNKNOWN ' . $msg);
  }

  function SetErrorStatus($msg)
  {
    $this->SetStatus('ERROR ' . $msg);
  }
  
  function SetStatus($msg)
  {
    global $dbintf;
    $dbintf->SetConfigsPart( array('SpinClientStatus'=>$msg) , 's');
  }
  
  function GetRemoteConnection()
  {
    foreach ($this->connections as $sc)
    {
      if ($sc instanceof RemoteConn)
      {
        return $sc;
      }
    }
    return null;
  }
  
  // this is called about once per 10 seconds or faster
  function InLoop()
  {
    if ($this->server===null
    && time() - $this->lastConnectTime >= 10)
    {
      $this->ConnectToServer();
    }

    // import configuration
    if ($this->server!=null
    // each 5 min
    && time() - $this->lastImportConfTime >= 5 * 60
    // and some time after connect
    && time() - $this->lastConnectTime > 10
    )
    {
      $sc = $this->GetRemoteConnection();
      if ($sc!=null)
      {
        $this->AskConfig($sc);
      }
    }
    
  }

  function ConnectToServer()
  {
    if ($this->serverIp=='')
    {
      lg('No destination configured to connect to');
      $this->SetUnknownStatus('no destination');
      return;
    }
    lg('Connect to server ' . $this->serverIp );
    $this->lastConnectTime = time();
    
    $cert_dir = '/var/apresa/certificates/';
    $context = stream_context_create();
    stream_context_set_option($context,'ssl','verify_peer',true);
    stream_context_set_option($context,'ssl','capath','/etc/ssl/certs');
    $newSock = @stream_socket_client("tls://" . $this->serverIp . ':9005'
      ,$errno,$errstr
      ,10 // timeout
      ,STREAM_CLIENT_CONNECT
      ,$context);
    if ($newSock===false)
    {
      lg("Error stream_socket_client: " . $errno . ' ' . $errstr);
      $this->SetErrorStatus('Error connecting to ' . $this->serverIp . ' , error ' . $errno . ' ' . $errstr);
      return false;
    }
    lg('Connected to server ' . $this->serverIp );
    $conn = new RemoteConn();
    $conn->sock = $newSock;
    $this->connections[] = $conn;
    // store a copy for easy access
    $this->server = $conn;
    // send something immediately
    $pubkey = trim( file_get_contents('/var/apresa/data/connector.pub') );
    $intro_request = array('action'=>'hello'
      ,'sn'=>$this->serialnum
      ,'pubkey'=>$pubkey
      );
    $this->server->sendObjMsg($intro_request);
    
    return true;
  }

  function EndConn( $conn )
  {
    StreamSocketApp::EndConn($conn);
    if ($conn instanceof RemoteConn)
    {
      lg("Connection to remote server ended");
      $this->SetErrorStatus('lost connection');
      $this->server = null;
      // this will trigger retry connect later
    }
  }
  
  function HandleMsg( $sc , $msg )
  {
    global $configs;
    if ($sc instanceof TrackerConn)
    {
      lg('Received message: ' . $msg );
      
      $reply = array();
      $reply['status'] = 'OK';
      $sc->sendObjMsg($reply);
      //lg('To send back reply with length: ' . strlen($client->toSend) );
      if ($this->server != NULL
      && $this->expacalls!=0)
      {
        $this->server->sendMsg( $msg );
      }
    } else if ($sc instanceof RemoteConn)
    {
      lg('From server: ' . $msg );
      $data = json_decode($msg,true);
      if (isset($data['errmsg']))
      {
        $this->SetErrorStatus($data['errmsg']);
      }
      if (str_ensure_defined($data['status'])=='authenticated')
      {
        $this->SetOkStatus();
        // not immediately: $this->AskConfig($sc);
        // see InLoop
      }
      if (str_ensure_defined($data['action'])=='challenge')
      {
        $encryptedData = $data['data'];
        $rsa = new RSA();
        $privateKey = file_get_contents('/var/apresa/data/connector');
        $rsa->loadKey($privateKey);
        $randomData = $rsa->decrypt( hex2bin($encryptedData) );
        if ($this->server != NULL)
        {
          $this->server->sendObjMsg( array('action'=>'response' , 'data'=>bin2hex($randomData)) );
        }
      } 
      else if (str_ensure_defined($data['action'])=='set-config')
      {
        $this->HandleSetConfig($sc,$data);
      }
      else if (isset($data['act']))
      {
        $this->HandleAct($data);
      }
    }
  }

  function HandleAct($data)
  {
    global $dbintf;
    if ($this->acceptact==0)
    {
      if ($this->server != NULL)
      {
        $this->server->sendObjMsg( array('action'=>'reply' , 'errmsg'=>'call actions not accepted' ) );
      }
      return;
    }
    $act = $data['act'];
    if(!isset($data['id'])) return;

    if ($act=='keep')
    {
      // handle this here
      $recid = $data['id'];
      $call = $dbintf->GetActiveCallInfoRM($recid);
      if ($call===false) return;
      // unpack
	    $callextra = StringToArray($call['callextra']);
	    // set option
			$callextra['to-keep'] = '1';
			// pack again
	    $newdata = array('callextra' => ArrayToString($callextra) );
	    // store in database
	    $dbintf->SetActiveCallInfoRM($recid,$newdata);
      return;
    }
    // we have a call action to forward
    // we can assume it is VoIP for now
    $signal_reply = true;
    $err = signal_service('sgnork '.$data['act'].' '.$data['id'],$signal_reply);
  }
  
  function AskConfig($sc)
  {
    if ($this->server==null) return;
    if ($this->importrecconf==0) return;
    $this->lastImportConfTime = time();
    $this->server->sendObjMsg( array('action'=>'get-config') );
  }
    
  function HandleSetConfig($sc,$data)
  {
    global $dbintf;
    if ($this->importrecconf==0) return;
    $keysToImport = array
    ( 'SipFilterType', 'SipFilters'
    , 'H323FilterType', 'H323Filters'
    , 'UnimegaFilterType', 'UnimegaFilters'
    , 'RecOnDmdType', 'RecOnDmdList'
    , 'DelOnDmdType', 'DelOnDmdList'
    , 'SilOnDemand', 'SilOnDmdList'
    );
    // 3 means define at user level
    // Since we do not import users,
    // this would cause the configuration to change if triggered to recalculate,
    // which is not what we want.
    // So we convert 3 to the value that signifies a positive filter.
    $filterType31 = array
    ( 'SipFilterType' // 3 => 1
    , 'H323FilterType' // 3 => 1
    , 'UnimegaFilterType' // 3 => 1
    );
    $filterType32 = array
    ( 'RecOnDmdType' // 3 => 2
    , 'DelOnDmdType' // 3 => 2
    , 'SilOnDemand' // 3 => 2
    );
    $toSet = array();
    $conf = $data['conf'];
    foreach ($keysToImport as $key)
    {
      if (isset($conf[$key]))
      {
        $value = $conf[$key];
        // define at user level needs some converting
        if (in_array($key,$filterType31))
        {
          if ($value=='3') $value = '1';
        }
        if (in_array($key,$filterType32))
        {
          if ($value=='3') $value = '2';
        }
        $oldValue = $dbintf->GetConfigValue($key);
        if ($value != $oldValue)
        {
          $toSet[ $key ] = $value;
        }
      }
    }
    if (count($toSet)>0)
    {
      lg('Applying remote configuration');
      $dbintf->SetConfigsPart( $toSet , 'o' );
      $msg = array();
      EnginesReconfig($msg, $dbintf);
    }
  }
  
}

lg("Begin");

$spinClient = new SpinClient();
$spinClient->Init();
$spinClient->Run();

lg("End");

