<?php
/*
Spin. For multi-Apresa setups. Enterprise.
Server mode:
  It listens for data from the network from other Apresa machines (spin-client),
  and uses this to update the ActiveCalls table in the database.
It is started and managed by apresa-remote.
*/

// without CMD it cannot load language
define('CMD',1);
// to get logging in other remote log file
define('CMD_REMOTE',1);

use phpseclib\Crypt\Random;
use phpseclib\Crypt\RSA;

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";
require_once CMD_WEB_RELPATH . "utility.php";
require_once CMD_WEB_RELPATH . "actions.php";
require_once CMD_WEB_RELPATH . "language.php";
require_once CMD_WEB_RELPATH . "tenancy.php";
require_once "stream-socket-app.php";

require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Crypt/Random.php' );
require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Crypt/RSA.php' );
require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Crypt/Hash.php' );
require_once( CMD_WEB_RELPATH . '/p/lib/phpseclib/Math/BigInteger.php' );


if (str_ensure_defined($argv[1])=='debug')
{
	// logging to console instead of apresa.log
	define('DEBUG_LOGING',1);
}
$logging_prefix = 'spin-server';

$exit = false;

#region Signal handling

function sig_handler($signo)
{
  global $exit;
  lg("sc Signal $signo");
  if ($signo==SIGINT
  || $signo==SIGTERM)
  {
    $exit = true;
  }
}

pcntl_signal(SIGINT, 'sig_handler');
pcntl_signal(SIGTERM, 'sig_handler');

#endregion

function SnAppend( $text , $sn )
{
	$pos = strpos($text, '.');
	if ($pos===false)
	{
		$pos = strlen($text);
	}
	$before = substr($text,0,$pos);
	$toadd = '_SN' . $sn;
	// already has this postfix?
	if (substr($before,-strlen($toadd))==$toadd) return $text;
	return $before . $toadd . substr($text,$pos);
}

// for connection swith a local webclient/client.php
class ClientConn extends Connection
{
  function __construct()
  {
    $this->logShutdown = false;
  }
}

// connection with a remote spin-client,
class RemoteConn extends Connection
{
  var $peerAddr;
  var $activeCalls = array();
  var $auth = false;
  var $sn;
  var $impacalls;
  var $exprecconf;
  var $randomData;

  function shutdown()
  {
    lg('Connection to ' . $this->peerAddr . ' closed');
    fclose($this->sock);
  }
}

class RemoteListener extends Listener
{
  function createConnection($newSock)
  {
    $conn = new RemoteConn();
    $conn->sock = $newSock;
    $conn->peerAddr = stream_socket_get_name($newSock,true);
    lg('New incoming connection from ' . $conn->peerAddr);
    return $conn;
  }
}

class ClientListener extends Listener
{
  function createConnection($newSock)
  {
    $conn = new ClientConn();
    $conn->sock = $newSock;
    return $conn;
  }
}

// Client will connect to Server,
// and send active call data to it.
// Server needs to store in ActiveCalls.
class SpinServer extends StreamSocketApp
{
  var $maindb;
  var $activeCalls;
  
  function Init()
  {
    global $dbintf;

    StreamSocketApp::Init();

		$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
    $this->maindb = $dbintf;
    
    $certId = $dbintf->GetCertificateFor('spin-server');
    if ($certId==false)
    {
      lg('No certificate selected for spin-server');
      return false;
    }
    $cert = $dbintf->GetCertificate($certId);
    if ($certId==false)
    {
      lg('Invalid certificate reference in database');
      return false;
    }

    // for remote connections from spin-client
    $cert_dir = '/var/apresa/certificates/';
    $context = stream_context_create
    ( array
      ( 'ssl' => array
          ( 'local_cert' => $cert_dir . $cert['cert_location']
          , 'local_pk'   => $cert_dir . $cert['key_location']
          )
      )
    );
    $newSock = stream_socket_server("tls://0.0.0.0:9005",$errno,$errstr
      ,STREAM_SERVER_BIND | STREAM_SERVER_LISTEN,$context);
    if ($newSock===false)
    {
      lg("Error starting TLS socket server: " . $errstr);
      exit;
    }
    $listener = new RemoteListener();
    $listener->sock = $newSock;
    $this->listeners[] = $listener;
    
    // for local connections from client.php
    $ipc_path = '/tmp/apresa_spin-server';
    unlink($ipc_path);
    $newSock = stream_socket_server("unix://" . $ipc_path);
    if ($newSock===false)
    {
      lg("Error starting unix domain socket server");
      exit;
    }
    // otherwise clients cannot write to it
    chmod($ipc_path,0766);
    $listener = new ClientListener();
    $listener->sock = $newSock;
    $this->listeners[] = $listener;
    // OK
    return true;
  }

  function HandleMsg( $sc , $msg)
  {
    global $configs;
    if ($sc instanceof RemoteConn)
    {
      lg('Received message: ' . $msg);
      $data = json_decode( $msg , true );
      if ($data==NULL)
      {
        $sc->errReply('json_decode');
        return;
      }
      if (isset($data['action'])==false)
      {
        $sc->errReply('no-action');
        return;
      }
      if ($data['action']=='hello')
      {
        $this->HandleHello($sc,$data);
      }
      // reponse to challenge during logon
      else if ($data['action']=='response')
      {
        $this->HandleChallengeResponse($sc,$data);
      }
      else if ($data['action']=='reply')
      {
        // nothing to do with it
      }
      else
      {
        if ($sc->auth==false)
        {
          $sc->errReply('access-denied');
        } else
        {
          if ($data['action']=='start')
          {
            $this->HandleStart($sc,$data);
            $sc->sendObjMsg( array('r'=>0) );
          }
          else if ($data['action']=='stop')
          {
            $this->HandleStop($sc,$data);
            $sc->sendObjMsg( array('r'=>0) );
          }
          else if ($data['action']=='delete')
          {
            $this->HandleDelete($sc,$data);
            $sc->sendObjMsg( array('r'=>0) );
          } 
          else if ($data['action']=='set')
          {
            $this->HandleSet($sc,$data);
            $sc->sendObjMsg( array('r'=>0) );
          }
          else if ($data['action']=='get-config')
          {
            $this->HandleGetConfig($sc,$data);
          }
          else
          {
            $sc->errReply('unknown-action');
          }
        }
      }
    } else if ($sc instanceof ClientConn)
    {
      // local message (from client.php)
      lg('Local message: ' . $msg);
      $data = json_decode( $msg , true );
      $sc->sendObjMsg( array('r'=>0) );
      $id = $data['id'];
      // find the right client to send this to
      if (isset($this->activeCalls[ $id ])==false)
      {
        lg('Could not find matching client to send command to');
      } else
      {
        $obj = $this->activeCalls[ $id ];
        lg('Forwarding message to ' . $obj['sc']->peerAddr );
        $msgToSend = array();
        $msgToSend['act'] = $data['act'];
        $msgToSend['id'] = $obj['oriId'];
        $obj['sc']->sendObjMsg( $msgToSend );
      }
    } else {
      lg('unknown connection type'); // should never happen
    }
  }  
  
  function HandleChallengeResponse($sc,$data)
  {
    lg('random data = ' . bin2hex($sc->randomData) );
    lg('data from client = ' . $data['data'] );
    if ( bin2hex($sc->randomData) != $data['data'] )
    {
      $sc->errReply('challenge-failed');
      return false;
    }
    $sc->auth = true;
    lg('Authenticated OK for SN=' . $sc->sn);
    $sc->sendObjMsg( array('r'=>0,'status'=>'authenticated') );
  }
  
  function HandleHello($sc,$data)
  {
    global $dbintf;
    $pubkey = $data['pubkey'];
    $authkeys = $dbintf->GetAuthKeys();
    $sc->sn = $data['sn'];
    $sc->auth = false;
    $known = false;
    foreach ($authkeys as $authobj)
    {
      if ( trim($pubkey) == trim($authobj['pubkey']) )
      {
        // a match
        $sc->impacalls = $authobj['impacalls'];
        $sc->exprecconf = $authobj['exprecconf'];
        $known = true;
      }
    }
    if ($known==false)
    {
      $sc->errReply('unknown-client');
      return false;
    }
    if ($sc->impacalls==0
    && $sc->exprecconf==0)
    {
      $sc->errReply('action-not-enabled');
      return false;
    }
    
    // send challenge of random bytes
    $randomData = Random::string(32);
    // store copy, so we can verify
    $sc->randomData = $randomData;
    $rsa = new RSA();
    $rsa->loadKey($pubkey,RSA::PUBLIC_FORMAT_OPENSSH);
    $encryptedRandom = $rsa->encrypt( $randomData );
    
    $sc->sendObjMsg( array( 'action'=>'challenge', 'data' => bin2hex($encryptedRandom) ) );
    return true;
  }
  
  function HandleStart($sc,$data)
  {
    global $dbintf;
    
    if ($sc->impacalls==0)
    {
      $sc->errReply('action-not-enabled');
      return false;
    }
    
    // We have to convert the data we receive, to data we could store in the database,
    // in the ActiveCalls table.
    // We need to convert things to IDs, instead of text.
    
    // before prefixing with SN
    // this is for sending commands back later
    $oriId = $data['remoteid'];
    
    $direction = $data['direction'];
    $remote_is_local = ($direction==2);
    $local_is_remote = ($direction==3);
    
    $cidLocal = $data['local_cid'];
    $ret = $dbintf->GetOrCreateCallerID($cidLocal,$data['local2' ],$local_is_remote?0:1);
    $data['localcallerid'] = $ret["id"];
    $data['localcontact'] = $ret["contactid"];

    $cidRemote = $data['remote_cid'];
    $ret = $dbintf->GetOrCreateCallerID($cidRemote,$data['remote2'],$remote_is_local?1:0);
    $data['remotecallerid'] = $ret["id"];
    $data['remotecontact'] = $ret["contactid"];
    
    if ($data['connected']!='')
    {
      $ret = $dbintf->GetOrCreateCallerID($data['connected'],'',2);
      $data['connectedcallerid'] = $ret["id"];
      $data['connectedcontact']  = $ret["contactid"];
    }
    
    // recorder_sn could be different from the SN sending this
    $data['recorderid'] = $dbintf->GetOrCreateRecorderId($data['recorder_sn'],$data['recorder_name']);
    $data['remoteid'] = SnAppend($data['remoteid'], $data['recorder_sn']);
    $data['processingstate'] = 0;
    $data['filename'] = '';
    $data['directory'] = '';
    
    $tenantid = null;
    if (isset($data['tenant_name']))
    {
      $matchingTenants = $dbintf->GetGroupInfoOnName( $data['tenant_name'] );
      if (count($matchingTenants) >= 1)
      {
        $tenant = $matchingTenants[0];
        if ($tenant['is_tenant'])
        {
          $tenantid = $tenant['id'];
        }
      }
    }
    if ($tenantid==null)
    {
      // apply based on local tenant configuration (its user phones)
      $tenantid = CalcTenant($data);
    }
    $data['tenantid'] = $tenantid;
    
    // the agent that we receive from other server is faulty
    unset($data['agent']);
    // maybe determine agent ourselves here?
    // (reuse DetermineAgent in trackunit)
    // it seems not really needed,
    // because it is done elsewhere it seems for finished call
    
    $to_unset = array( 'action'
    , 'sn'
    , 'orktenant'
    , 'local_cid'
    , 'remote_cid'
    , 'connected_cid'
    , 'recorder_sn'
    , 'recorder_name'
    , 'local_name'
    , 'remote_name'
    , 'connected_name'
    , 'tenant_name'
    , 'agent'
    );
    foreach ($to_unset as $key)
    {
      unset( $data[$key] );
    }    
    // next is rather verbose
    // verbose: lg( json_encode($data) );
    // set flag to tell this was added by spin-server
    $data['flags'] |= 1024;
    $dbintf->InsertActiveCall( $data );
    
    // data remoteid is the SN appended, oriId is the unappended
    $this->activeCalls[ $data['remoteid'] ] = array( 'sc' => $sc , 'oriId' => $oriId );
  }
  
  function HandleStop($sc,$data)
  {
    global $dbintf;
    if ($sc->impacalls==0)
    {
      $sc->errReply('action-not-enabled');
      return false;
    }
    $id = SnAppend($data['id'],$data['recorder_sn']);
    // same flag as when call is being converted (codec)
    $dbintf->SetActiveProcessingFlag($id,4);
    unset( $this->activeCalls[ $id ] );
  }
  
  function HandleDelete($sc,$data)
  {
    global $dbintf;
    if ($sc->impacalls==0)
    {
      $sc->errReply('action-not-enabled');
      return false;
    }
    $sn = $data['sn'];
    $id = SnAppend($data['id'],$data['recorder_sn']);
    $dbintf->RemoveActiveCall($id);
    unset( $this->activeCalls[ $id ] );
  }  
  
  function HandleGetConfig($sc,$data)
  {
    global $dbintf;
    if ($sc->exprecconf==0)
    {
      $sc->errReply('exprecconf-not-enabled');
      return false;
    }
    $keysToExport = array
    ( 'SipFilterType', 'SipFilters'
    , 'H323FilterType', 'H323Filters'
    , 'UnimegaFilterType', 'UnimegaFilters'
    // maybe not: , 'NumberFilterType', 'NumberFilters'
    , 'RecOnDmdType', 'RecOnDmdList'
    , 'DelOnDmdType', 'DelOnDmdList'
    , 'SilOnDemand', 'SilOnDmdList'
    );
    $conf = array();
    foreach ($keysToExport as $key)
    {
      $conf[$key] = $dbintf->GetConfigValue( $key );
    }
    $reply = array();
    $reply['action'] = 'set-config';
    $reply['conf'] = $conf;
    $sc->sendObjMsg( $reply );
  }
  
  function HandleSet($sc,$data)
  {
    global $dbintf;
    if ($sc->impacalls==0)
    {
      $sc->errReply('action-not-enabled');
      return false;
    }
    $sn = $data['sn'];
    $id = SnAppend($data['id'],$data['recorder_sn']);
    $callobj = array();
    foreach ($data as $key=>$value)
    {
      if ($key=='id') continue;
      if ($key=='action') continue;
      if ($key=='recorder_sn') continue;
      $callobj[ $key ] = $value;
    }
    $dbintf->UpdateActiveCall( $id , $callobj );
  }
  
}

lg("Begin");

$spinServer = new SpinServer();
if ($spinServer->Init())
{
  $spinServer->Run();
}

lg("End");

