<?php
error_reporting(E_ALL ^ E_NOTICE);
define('CMD',1);

require_once "../conf/voip_config.php";
require_once CMD_WEB_RELPATH . "db.php";
require_once CMD_WEB_RELPATH . "signaling.php";

/*
  We are starting up.
  Do clean-up of previous sessions.
*/

if (isset($argv[1])==false)
{
  // No file name specified
  exit(2);
}

DBConnect();

StartUp($argv[1]);

function IsSpinCall($call) {
  return ($call['flags']&1024)!=0;
}

function IsED137Call($call) {
  return ($call['source'] == 7);
}

function IsCrecCall($call) {
  return ($call['source']==1 || $call['source']==5 || $call['source']==6);
}

function IsVoipCall($call) {
  return !(IsED137Call($call) && !IsCrecCall($call));
}

function IsCrecLiveCall($lcall) {
  return false;
}

function IsVoipLiveCall($lcall) {
  return $lcall['protocol'] != 22;
}

function IsED137LiveCall($lcall) {
  return $lcall['protocol'] == 22;
}

function StartUp($param)
{
	global $dbintf;

  // Set full permission
  $dbintf->SetAdmin();

  if ($param=='-o' || $param=='-c' || $param=='-v' || $param=='-e')
  {
    // -o : Call this when all recording components parts will be restarted
    //        - On reboot
    //        - Restart recording component
    //        - After channel license change
    //        - After restore from backup
    // -c : crec
    //      Used as ExecStart systemd service apresacrec
    // -v : voip (ork)
    //      Used as ExecStart systemd serivce orkaudio
    // -e : ed137
    //      Used as ExecStart systemd service apresa_ed137

    if($param == '-o')
    {
      system('rm -R /dev/shm/apresa/2???');
      system('rm -R /database/standbyrecs/2???');
      system('rm -R /recordings/rectmp/*');
      $filter = "IsSpinCall";
    }
    
    if ($param=='-v')
    {
      $voipRestarts = true;
      $filter = "IsVoipCall";
      $liveFilter = "IsVoipLiveCall";
    }
    
    if ($param=='-c')
    {
      $crecRestarts = true;
      $filter = "IsCrecCall";
      $liveFilter = "IsCrecLiveCall";
    }

    if ($param=='-e')
    {
      $ed137Restarts = true;
      $filter = "IsED137Call";
      $liveFilter = "IsED137LiveCall";
    }
    

    // Live Calls list
    $lcalls = $dbintf->GetLiveCalls();
    foreach ($lcalls as $lcall)
    {
			$del = $liveFilter($lcall);
      
			if($del)
			{
				$remoteid = $lcall['remoteid'];
      	$dbintf->DelLiveCall( $remoteid );
			}
    }
    
		// Active Call list must be moved to normal list
    // Get Active Call list
    // We need the full information that is in the table
    $acalls = $dbintf->GetFullActiveCallList();

    foreach($acalls as $acall)
    {
      $consider = $filter($acall);
      if(!$consider) continue;

      // keep delayed store on demand calls
      if (($acall['processingstate']&128)!=0) continue;

      $id = $acall['id'];
			// A new ID is automatically assigned
      unset($acall['id']);
      // flag 1 indicates a call not properly finished
      $acall['flags'] |= 1;
			// do not add RecOnDmd calls to main call list
      // also do not add spin calls
			if ($acall['isarchived']==1 && IsSpinCall($acall)==false)
			{
				// call was being recording
				// but the recording is not available,
				// because it was not properly finished
				$acall['isarchived'] = 0;
				// add call to normal list
				$dbintf->InsertCall($acall);
			}
      // remove call from active list
      $dbintf->RemoveActiveCallOnId($id);
    }
  }
}

/*
	If we cannot connect at once, we wait, and retry a few times.
	This is needed at reboot under Debian.
*/
function DBConnect()
{
	global $dbintf;
	$retry = 0;
	while(true)
	{
        $ok = false;
        try
        {
                $dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH, 'apresa', true);
                $ok = true;
                //echo "Connected OK\n";
        } catch(Exception $e)
        {
                //echo "Failed with exception.\n";
        }
        if ($ok) break;
        else
        {
                $retry ++;
                if ($retry>10)
                {
                        //throw new Exception('Cannot connect to database.');
                        throw $e;
                        exit;
                }
                // sleep for one second
                sleep(1);
        }
	}
}


?>