<?php

/*
A network application that responds to stream socket events,
using select.
*/

class Connection
{
  // stream resource:
  var $sock;
  // data to send (buffered)
  var $toSend;
  // receive buffer
  var $rcv;
  // should log connection shutdown
  var $logShutdown;
  var $createdAt;
  
  function __construct()
  {
    $this->rcv = '';
    $this->toSend = '';
    $this->createdAt = time();
    $this->logShutdown = true;
  }
  
  function sendMsg($msg)
  {
    $this->toSend .= $msg . "\0";
  }
  
  function errReply($errmsg)
  {
    $this->sendObjMsg( array('r'=>1,'errmsg'=>$errmsg) );
  }
  
  function okReply()
  {
    $this->sendObjMsg( array('r'=>0) );
  }
  
  function sendObjMsg($obj)
  {
    $jsonStr = json_encode($obj);
    lg('Sending msg: ' . $jsonStr );
    $this->toSend .= $jsonStr . "\0";
  }
  
  function shutdown()
  {
    if ($this->logShutdown)
    {
      lg('Connection closed');
    }
    fclose($this->sock);
  }
}

class Listener
{
  // master socket
  var $sock;
  
  function createConnection($newSock)
  {
    lg('New connection');
    $conn = new Connection();
    $conn->sock = $newSock;
    return $conn;
  }

  function shutdown()
  {
    fclose($this->sock);
  }  
}

class StreamSocketApp
{
  // all connections to local or remote
  var $connections = array();
  // listening for incoming connections
  var $listeners = array();
  
  function Init()
  {
  }
  
  function Run()
  {
    global $exit;
    lg("SSA Run");
    
    while ($exit==false)
    {
      $readSocks = array();
      foreach ($this->listeners as $listener)
      {
        $readSocks[] = $listener->sock;
      }
      foreach ($this->connections as $sc)
      {
        $readSocks[] = $sc->sock;
      }
      if (count($readSocks)==0)
      {
        lg('No listeners or connections, so closing');
        break;
      }

      $writeSocks = array();
      foreach ($this->connections as $sc)
      {
        if ($sc->toSend != '')
        {
          $writeSocks[] = $sc->sock;
        }
      }        
      
      $excSocks = $readSocks;
      
      // verbose: lg('select (wait)');
      $result = @stream_select($readSocks,$writeSocks,$excSocks,10);
      if ($result === false)
      {
        // interrupted probably
      } else if ($result>0)
      {
        // loop through all sockets that we can read from
        foreach ($readSocks as $readSock)
        {
          // find out what kind of socket this is
          // check if it is a listener socket
          foreach ($this->listeners as $listener)
          {
            if ($listener->sock === $readSock)
            {
              $this->AcceptConn($listener);
            }
          }
          foreach ($this->connections as $conn)
          {
            if ($conn->sock === $readSock)
            {
              $this->ReadFromConn($conn);
            }
          }
        }
        // loop through sockets we can write to
        foreach ($writeSocks as $writeSock)
        {
          foreach ($this->connections as $conn)
          {
            if ($conn->sock === $writeSock)
            {
              $this->WriteToConn($conn);
            }
          }
        }
        foreach ($excSocks as $excSock)
        {
          lg('exception on socket');
        } 
      }
      
      // no sleep needed anymore! 
      //usleep(500 * 1000);
      pcntl_signal_dispatch();

      // opportunity for subclass to anything extra
      $this->InLoop();
    }
    
    lg('SSA Cleanup');
    foreach ($this->connections as $conn)
    {
      $conn->shutdown();
    }
    foreach ($this->listeners as $listener)
    {
      $listener->shutdown();
    }
  }
  
  function InLoop()
  {
  }

  function AcceptConn($listener)
  {
    $newSock = @stream_socket_accept($listener->sock);
    if ($newSock===false); // problem
    else
    {
      // createConnection can be overloaded
      $this->connections[] = $listener->createConnection($newSock);
    }
  }

  function ReadFromConn($conn)
  {
    // verbose lg('ReadFromConn');
    $buf = fread($conn->sock,1024);
    if ($buf === false)
    {
      lg('error reading from socket');
    } else if ($buf === '')
    {
      //lg('end of socket (empty read)');
      $this->EndConn($conn);
    } else
    {
      // received data
      $conn->rcv .= $buf;
      // there can multiple messages in the buffer
      while (true)
      {
        $nulpos = strpos($conn->rcv,"\0");
        if ($nulpos===false) break;
        $msg = substr($conn->rcv,0,$nulpos);
        $this->HandleMsg($conn,$msg);
        $conn->rcv = substr($conn->rcv,$nulpos+1);
      }
    }
  }
  
  function WriteToConn($conn)
  {
    // verbose lg('WriteToConn');
    $result = fwrite($conn->sock,$conn->toSend);
    if ($result===false)
    {
      lg('error writing to socket');
    } else if ($result>0)
    {
      $conn->toSend = substr($conn->toSend,$result);
      // $toSendCnt = strlen($conn->toSend);
    }            
  }
  
  function HandleMsg($conn,$msg)
  {
    lg('HandleMsg');
  }
  
  function EndConn($conn)
  {
    $conn->shutdown();
    foreach ($this->connections as $key => $connection)
    {
      if ($connection == $conn)
      {
        unset($this->connections[$key]);
      }
    }
  }
  
}

