-- Database definition (Postgresql)
-- for Call Recorder Apresa
-- Version 115
-- must match with apresa-upgrade.c

CREATE TABLE Calls
( id SERIAL PRIMARY KEY
, startdate BIGINT
, startms INT
, duration INT
, durms INT
, isactive INT
, direction INT
, remotefull TEXT
, remote TEXT
, localfull TEXT
, local TEXT
, remote2 TEXT
, local2 TEXT
, isconnected INT
, linenr INT
, waittime INT
, source INT
, isarchived INT
, filename TEXT
, screenfn TEXT
, directory TEXT
, encoding INT
, encryption INT
, cardno TEXT
, flags INT
, callextra TEXT
, remoteid TEXT
, processingstate INT
, recorderid INT
, lineid INT
, localcallerid INT
, remotecallerid INT
, localcontact INT
, remotecontact INT
, notes TEXT
, notes2 TEXT
, notes3 TEXT
, notes4 TEXT
, notes5 TEXT
, data1 TEXT
, data2 TEXT
, data3 TEXT
, data4 TEXT
, data5 TEXT
, connected TEXT
, category INT DEFAULT(0)
, connectedcallerid INT DEFAULT(0)
, connectedcontact INT DEFAULT(0)
, recyclebin INT
, tenantid INTEGER
, agent INTEGER
, retention INTEGER
);

CREATE INDEX calls_date_ix ON Calls(startdate DESC, startms DESC);
CREATE INDEX calls_remoteid_ix ON Calls(remoteid ASC);
CREATE INDEX calls_tenant_ix ON Calls(tenantid);
CREATE INDEX calls_date_arch ON Calls(startdate DESC) where isarchived=1;
CREATE INDEX calls_retain_ix ON Calls(retention, startdate);

-- Same structure as table Calls
CREATE TABLE ActiveCalls
( id SERIAL PRIMARY KEY
, startdate BIGINT
, startms INT
, duration INT
, durms INT
, isactive INT
, direction INT
, remotefull TEXT
, remote TEXT
, localfull TEXT
, local TEXT
, remote2 TEXT
, local2 TEXT
, isconnected INT
, linenr INT
, waittime INT
, source INT
, isarchived INT
, filename TEXT
, screenfn TEXT
, directory TEXT
, encoding INT
, encryption INT
, cardno TEXT
, flags INT
, callextra TEXT
, remoteid TEXT
, processingstate INT
, recorderid INT
, lineid INT
, localcallerid INT
, remotecallerid INT
, localcontact INT
, remotecontact INT
, notes TEXT
, notes2 TEXT
, notes3 TEXT
, notes4 TEXT
, notes5 TEXT
, data1 TEXT
, data2 TEXT
, data3 TEXT
, data4 TEXT
, data5 TEXT
, connected TEXT
, category INT DEFAULT(0)
, connectedcallerid INT DEFAULT(0)
, connectedcontact INT DEFAULT(0)
, recyclebin INT
, tenantid INTEGER
, agent INTEGER
, retention INTEGER
);

-- Same structure as table Calls
CREATE TABLE CallsArchive
( id SERIAL PRIMARY KEY
, startdate BIGINT
, startms INT
, duration INT
, durms INT
, isactive INT
, direction INT
, remotefull TEXT
, remote TEXT
, localfull TEXT
, local TEXT
, remote2 TEXT
, local2 TEXT
, isconnected INT
, linenr INT
, waittime INT
, source INT
, isarchived INT
, filename TEXT
, screenfn TEXT
, directory TEXT
, encoding INT
, encryption INT
, cardno TEXT
, flags INT
, callextra TEXT
, remoteid TEXT
, processingstate INT
, recorderid INT
, lineid INT
, localcallerid INT
, remotecallerid INT
, localcontact INT
, remotecontact INT
, notes TEXT
, notes2 TEXT
, notes3 TEXT
, notes4 TEXT
, notes5 TEXT
, data1 TEXT
, data2 TEXT
, data3 TEXT
, data4 TEXT
, data5 TEXT
, connected TEXT
, category INT DEFAULT(0)
, connectedcallerid INT DEFAULT(0)
, connectedcontact INT DEFAULT(0)
, recyclebin INT
, tenantid INTEGER
, agent INTEGER
, retention INTEGER
);

-- Same indexes as Calls
CREATE INDEX callsarchive_date_ix ON CallsArchive(startdate DESC, startms DESC);
CREATE INDEX callsarchive_remoteid_ix ON CallsArchive(remoteid ASC);
CREATE INDEX callsarchive_tenant_ix ON CallsArchive(tenantid);
CREATE INDEX callsarchive_date_arch ON CallsArchive(startdate DESC) where isarchived=1;
CREATE INDEX callsarchive_retain_ix ON CallsArchive(retention, startdate);

CREATE TABLE LiveCalls
( id SERIAL PRIMARY KEY
, remoteid TEXT
, inittime BIGINT
, ringtime BIGINT
, connecttime BIGINT
, callstatus INT
, recreason INT
, protocol INT
, direction INT
, remote TEXT
, local TEXT
);

CREATE TABLE ActionCache
( id SERIAL PRIMARY KEY
, action TEXT
, addtime BIGINT
, expires INT
);

CREATE TABLE LicNotRec
( id SERIAL PRIMARY KEY
, startdate BIGINT
);

CREATE TABLE Contacts
( id SERIAL PRIMARY KEY
, name TEXT
, extra TEXT
, parentcontact INT
, deleted INT DEFAULT(0)
, ctype INT
, value TEXT
);

CREATE INDEX contacts_tpval_ix ON Contacts(ctype,value);

CREATE TABLE CallerIDs
( id SERIAL PRIMARY KEY
, callerid TEXT
, callerid2 TEXT
, simplified TEXT
, islocal INT
, contactid INT
, deleted INT DEFAULT(0)
);

CREATE INDEX callerids_ct_ix ON CallerIDs(contactid ASC);
CREATE INDEX callerids_cid_ix ON CallerIDs(callerid);

CREATE TABLE Lines
( id SERIAL PRIMARY KEY
, contactid INT
, recorderid INT
, linenr INT
);

CREATE INDEX lines_ct_ix ON Lines(contactid ASC);

CREATE TABLE Cards
( id SERIAL PRIMARY KEY
, cardno TEXT
, name TEXT
);

CREATE TABLE Recorders
( id SERIAL PRIMARY KEY
, recid TEXT
, name TEXT
, rectype INT
, options TEXT
);

CREATE TABLE Accessors
( id SERIAL PRIMARY KEY
, type INT
);

CREATE TABLE Users
( id SERIAL PRIMARY KEY
, accid INT
, usr TEXT
, pwd TEXT
, enabled INT
, permissions INT
, name TEXT
, email TEXT
, extra TEXT
, rod INT
, manager INT DEFAULT(0)
, evaluated INT DEFAULT(0)
, mphones TEXT
, logonmethod INT DEFAULT(0)
, lastnewlogin INT DEFAULT(0)
, playtimelim INT DEFAULT(0)
, ldapvalidated INT
, ldapcacheuse INT
, ldappwd TEXT
, customlocalid INT DEFAULT(0)
, freeseat INT DEFAULT(0)
, changepwd INT
, numwrongpwd INT DEFAULT(0)
, pwdchangedate INT
, tempdisabled INT DEFAULT(0)
, disabledtime BIGINT
, pwdhistory TEXT
, lng TEXT
, ldapuser INT DEFAULT(0)
, tenantid INTEGER
, recondmd INT DEFAULT(0)
, delondmd INT DEFAULT(0)
, silondmd INT DEFAULT(0)
, ldaphost TEXT
, samwinuser INT DEFAULT(0)
, webclientsettings INT
, azureapp INTEGER
, colcfg TEXT
, maxcallage INTEGER DEFAULT(-1)
, iaperiod INTEGER DEFAULT(-1)
);

CREATE INDEX users_acc_ix ON Users(accid);

CREATE TABLE UserPhones
( id SERIAL PRIMARY KEY
, accid INT
, contactid INT
);
CREATE INDEX userphones_ct_ix ON UserPhones(contactid ASC);

CREATE TABLE Groups
( id SERIAL PRIMARY KEY
, accid INT
, typegrp INT
, name TEXT
, permissions INT
, super INT
, extra TEXT
, adgroup TEXT
, is_tenant INTEGER
, tenantid INTEGER
, ldaphost TEXT
, samwingroup TEXT
, idp INT
, cologons INTEGER DEFAULT(0)
, azureapp INTEGER
, azuregroupname TEXT
, azuregroupid TEXT
, scimenabled INTEGER DEFAULT(0)
, scimtoken TEXT
);

CREATE TABLE GroupMembers
( id SERIAL PRIMARY KEY
, grpid INT
, accid INT
);

CREATE TABLE AccAcc
( id SERIAL PRIMARY KEY
, accid INT
, callaccid INT
, epermissions INT
, applytype INT
);

CREATE TABLE AccRecorders
( id SERIAL PRIMARY KEY
, accid INT
, recorderid INT
, epermissions INT
);

CREATE TABLE TenantContacts
( id SERIAL PRIMARY KEY
, contactid INTEGER NOT NULL
, tenantid INTEGER NOT NULL
, name TEXT
, deleted INTEGER DEFAULT(0)
);
CREATE INDEX tenantcontacts_ix ON TenantContacts(tenantid,contactid);
ALTER TABLE TenantContacts
ADD CONSTRAINT tenantcontacts_contactid_fkey
FOREIGN KEY (contactid)
REFERENCES Contacts(id)
ON DELETE CASCADE;
ALTER TABLE TenantContacts
ADD CONSTRAINT tenantcontacts_tenantid_fkey
FOREIGN KEY (tenantid)
REFERENCES Groups(id)
ON DELETE CASCADE;

CREATE TABLE Configs
( param TEXT PRIMARY KEY
, value TEXT
, part TEXT
);

CREATE TABLE Sessions
( id SERIAL PRIMARY KEY
, sessionid TEXT
, userid INT
, firstacc INT
, lastacc INT
);

CREATE TABLE LicKeys
( id SERIAL PRIMARY KEY
, usedkey TEXT
, sn TEXT
);


CREATE TABLE CEAnswerTypes (
id SERIAL PRIMARY KEY,
description TEXT  ,
type INTEGER  ,
minval INTEGER  ,
maxval INTEGER
);

CREATE TABLE CECalls (
id SERIAL PRIMARY KEY,
recordingid INT,
projectid INT,
status INT,
answercnt INT,
useraccid INT,
score REAL,
datescored INT,
supervisor INT
);

ALTER TABLE CECalls
ADD CONSTRAINT cecalls_recid_fkey
FOREIGN KEY (recordingid)
REFERENCES Calls(id)
ON DELETE CASCADE;

ALTER TABLE CECalls
ADD CONSTRAINT cecalls_user_fkey
FOREIGN KEY (useraccid)
REFERENCES Accessors(id)
ON DELETE SET NULL;

CREATE TABLE CEAnswers (
id SERIAL PRIMARY KEY,
callid INTEGER   ,
qid INTEGER   ,
answer INTEGER ,
notes TEXT
);

ALTER TABLE CEAnswers
ADD CONSTRAINT ceanswers_callid_fkey
FOREIGN KEY (callid)
REFERENCES CECalls(id)
ON DELETE CASCADE;

CREATE TABLE CEForms (
id SERIAL PRIMARY KEY,
name TEXT
);

CREATE TABLE CEProjects (
id SERIAL PRIMARY KEY,
formid INTEGER  ,
name TEXT
);

CREATE TABLE CEQuestions (
id SERIAL PRIMARY KEY ,
qtext TEXT  ,
atypeid INTEGER  ,
formid INTEGER  ,
ordering INTEGER ,
weight REAL DEFAULT(10),
canna INT DEFAULT(0)
);

CREATE TABLE CESchedules
( id SERIAL PRIMARY KEY
, project INTEGER
, supervisor INTEGER
, name TEXT
, qspec TEXT
, callcnt INTEGER
, add_per_id INTEGER
, type INTEGER
, nexttrigger INTEGER
, lasttrigger INTEGER
, day INTEGER
, moving TEXT
);

ALTER TABLE CESchedules
ADD CONSTRAINT ceschedules_project_fkey
FOREIGN KEY (project)
REFERENCES CEProjects(id)
ON DELETE CASCADE;

CREATE INDEX idx_cecalls_projectid ON CECalls(projectid ASC);
CREATE INDEX idx_cecalls_recordingid ON CECalls(recordingid ASC);

CREATE TABLE Queries
( id SERIAL PRIMARY KEY
, name TEXT
, qspec TEXT
);

CREATE SEQUENCE recids_sq;

CREATE TABLE MobileDevices
( id SERIAL PRIMARY KEY
, imei TEXT
, phonenum TEXT
);

CREATE TABLE TransferTargets
( id SERIAL PRIMARY KEY
, description TEXT
, enabled INTEGER
, ip TEXT
, accid INTEGER
, backlog TEXT
, lastid INTEGER
, triedid INTEGER
, lasttry INTEGER
, status INTEGER
, errmsg TEXT
, minage INT DEFAULT(0)
, expacalls INT DEFAULT(0)
, acceptact INT DEFAULT(0)
, importrecconf INT DEFAULT(0)
, sendafter TEXT
, sendbefore TEXT
);

CREATE TABLE AuthKeys
( id SERIAL PRIMARY KEY
, name TEXT
, pubkey TEXT
, impacalls INT DEFAULT(0)
, exprecconf INT DEFAULT(0)
, undouble INT DEFAULT(0)
);

CREATE TABLE IdMap
( id SERIAL PRIMARY KEY
, theid TEXT
, value TEXT
, target TEXT
);
CREATE INDEX idmap_theid_ix ON IdMap(theid ASC);

CREATE TABLE Annotations
( id SERIAL PRIMARY KEY
, call_id INTEGER
, label TEXT
, timepos REAL
, descr TEXT
, createtime BIGINT
, createdby INTEGER
, edittime BIGINT
, editedby INTEGER
);
CREATE INDEX annotations_callid_ix ON Annotations(call_id);

CREATE TABLE Transcriptions
( call_id INTEGER PRIMARY KEY
, transcript TEXT
);

ALTER TABLE Transcriptions
ADD CONSTRAINT transcriptions_callid_fkey
FOREIGN KEY (call_id)
REFERENCES Calls(id)
ON DELETE CASCADE;

CREATE INDEX transcript_ix ON Transcriptions
USING gin(to_tsvector('simple',transcript));

ALTER TABLE Annotations
ADD CONSTRAINT annotations_callid_fkey
FOREIGN KEY (call_id)
REFERENCES Calls(id)
ON DELETE CASCADE;

CREATE TABLE LocalIdMapping
( srcid INT PRIMARY KEY
, dstid INT
, updatetime BIGINT
);

CREATE TABLE FreeSeats
( id SERIAL PRIMARY KEY
, pcname TEXT
, contactid INT
);
CREATE INDEX freeseats_pcname_ix ON FreeSeats(pcname);

CREATE TABLE MaxCalls
( id SERIAL PRIMARY KEY
, numcalls INT
, hour INT
);
CREATE INDEX maxcalls_hours ON MaxCalls(hour);

CREATE TABLE ActiveAnnotations
( id SERIAL PRIMARY KEY
, call_id INTEGER REFERENCES ActiveCalls(id) ON DELETE CASCADE
, timepos REAL
, descr TEXT
, createtime BIGINT
);
CREATE INDEX activeannotations_callid_ix ON ActiveAnnotations(call_id);


CREATE TABLE ImportTargets
( id SERIAL PRIMARY KEY
, description TEXT
, enabled INTEGER
, ip TEXT
, username TEXT
, password TEXT
, interval INTEGER
, restart INTEGER
, importfromdate INTEGER
, lasttry INTEGER
, status INTEGER
, lastfile TEXT
, errmsg INTEGER
, inactivitycheck INT DEFAULT(1)
, recorderid INT DEFAULT(-1)
, rectype TEXT DEFAULT('VidiFTP')
, directory TEXT
, delay INT DEFAULT(0)
);

CREATE TABLE AuditTrail
( id SERIAL PRIMARY KEY
, moment INT
, ipaddr TEXT
, userid INT
, descr TEXT
);
ALTER TABLE AuditTrail
ADD CONSTRAINT audittrail_userid_fkey
FOREIGN KEY (userid)
REFERENCES Users(id)
ON DELETE SET NULL;
CREATE INDEX audittrail_moment_ix ON AuditTrail(moment ASC);

CREATE TABLE VTaps
( id SERIAL PRIMARY KEY
, vtap_id TEXT UNIQUE
, password TEXT
, tenantid INTEGER
, name TEXT
);

CREATE TABLE TScriptPlans
( id SERIAL PRIMARY KEY
, name TEXT
, enabled INT DEFAULT (0)
, qstatus INT DEFAULT(0)
, tstatus INT DEFAULT(0)
, filter TEXT
, maxdur REAL DEFAULT(-1)
, durqueued REAL
, durdone REAL
, cdone INT DEFAULT (0)
, idqueued INT
, errmsg TEXT
);

CREATE TABLE TScriptQ
( id SERIAL PRIMARY KEY
, planid INTEGER
, callid INTEGER
, duration FLOAT
, idext TEXT
, added INTEGER
);

ALTER TABLE TScriptQ
ADD CONSTRAINT tscript_plan_fkey
FOREIGN KEY (planid)
REFERENCES TScriptPlans(id)
ON DELETE CASCADE;

CREATE TABLE Certificates
(
	id SERIAL PRIMARY KEY
, name TEXT
, subject TEXT
, valid_to BIGINT
, privatekey INTEGER
, trusted INTEGER
, cert_location TEXT
, key_location TEXT
, intermediate_location TEXT
, webkey INT DEFAULT(0)
, lets_encrypt INT DEFAULT(0)
);

CREATE TABLE CertificateUse
(
	id SERIAL PRIMARY KEY
,	certificateid INTEGER
,	usedfor TEXT 
);

ALTER TABLE CertificateUse
ADD CONSTRAINT certificate_use_fkey
FOREIGN KEY (certificateid)
REFERENCES Certificates(id);


CREATE TABLE dashboard_order
( id SERIAL PRIMARY KEY
, location INT
, usr INT
, tile INT
);
		
CREATE TABLE dashboard_settings
( id SERIAL PRIMARY KEY
, value INT
, clm INT
, usr INT
);

CREATE TABLE dashboard_week_average
( usr INT PRIMARY KEY
, date_updated INT
, average_array TEXT
);
    
CREATE TABLE CjAccounts
( id SERIAL PRIMARY KEY
, server TEXT
, usr TEXT
, pwd TEXT
);

CREATE TABLE ExtIdP
( id SERIAL PRIMARY KEY
, name TEXT
, tech TEXT
, cert INTEGER
, entityid TEXT
, signonurl TEXT
, signoffurl TEXT
, extlogout INTEGER
, azureapp INTEGER
);

ALTER TABLE ExtIdP
ADD CONSTRAINT extidp_cert_fkey
FOREIGN KEY (cert)
REFERENCES Certificates(id);

ALTER TABLE Groups
ADD CONSTRAINT groups_idp_fkey
FOREIGN KEY (idp)
REFERENCES ExtIdP(id)
ON DELETE SET NULL;

CREATE TABLE ReportSchedules
( id SERIAL PRIMARY KEY
, name TEXT
, type TEXT
, period TEXT
, filter TEXT
, email TEXT
, lastrun INTEGER
, laststart INTEGER
, lastperiod TEXT
, output TEXT
, directory TEXT
);

CREATE TABLE CertificateRequests
(
id SERIAL PRIMARY KEY
, name TEXT
, subject TEXT
, request_location TEXT
, key_location TEXT
);

CREATE TABLE WebclientSettings
( id SERIAL PRIMARY KEY
, accid INT
, settings INT
); 

CREATE TABLE AzureApps
( id SERIAL PRIMARY KEY
, name TEXT
, appid TEXT
, appsecret TEXT
, mstenant TEXT
, importphones_bphones INT
, importphones_upn INT
, importphones_mobile INT
, importphones_name INT
);

ALTER TABLE ExtIdP
ADD CONSTRAINT extidp_azureapp_fkey
FOREIGN KEY (azureapp)
REFERENCES AzureApps(id)
ON DELETE SET NULL;

ALTER TABLE Groups
ADD CONSTRAINT groups_azureapp_fkey
FOREIGN KEY (azureapp)
REFERENCES AzureApps(id)
ON DELETE SET NULL;

ALTER TABLE Users
ADD CONSTRAINT users_azureapp_fkey
FOREIGN KEY (azureapp)
REFERENCES AzureApps(id)
ON DELETE SET NULL;

CREATE TABLE TmpFiles
( filepath TEXT PRIMARY KEY
, filesize BIGINT
, added BIGINT
, expiry BIGINT
);

CREATE TABLE CallEncryptionKeys
( id SERIAL PRIMARY KEY
, accid INT
, inuse INT
, pubkey TEXT
, privkey TEXT
, iv TEXT
, salt TEXT
, deleted INT DEFAULT(0)
);

Create TABLE CallKeys 
(
  id SERIAL PRIMARY KEY
, callid INT
, keyid INT
, callkey TEXT
, iv TEXT
);

ALTER TABLE CallKeys
ADD CONSTRAINT callskeys_call_fkey
FOREIGN KEY (callid)
REFERENCES calls(id)
ON DELETE CASCADE;

ALTER TABLE CallKeys
ADD CONSTRAINT callskeys_key_fkey
FOREIGN KEY (keyid)
REFERENCES CallEncryptionKeys(id)
ON DELETE CASCADE;

CREATE INDEX call_keys_call_ix ON CallKeys(callid);
CREATE INDEX call_key_key_ix ON CallKeys(keyid);

Create TABLE ActiveCallKeys 
(
  id SERIAL PRIMARY KEY
, callid INT
, keyid INT
, callkey TEXT
, iv TEXT
);

ALTER TABLE ActiveCallKeys
ADD CONSTRAINT acallskeys_call_fkey
FOREIGN KEY (callid)
REFERENCES activecalls(id)
ON DELETE CASCADE;

ALTER TABLE ActiveCallKeys
ADD CONSTRAINT acallskeys_key_fkey
FOREIGN KEY (keyid)
REFERENCES CallEncryptionKeys(id)
ON DELETE CASCADE;

CREATE TABLE Chat
( id SERIAL PRIMARY KEY
, tenantid INTEGER
, eid TEXT
, type INTEGER
, topic TEXT
, chatid TEXT
, teamid TEXT
, channelid TEXT
, teamname TEXT
, channelname TEXT
, lastupdated BIGINT
);
CREATE UNIQUE INDEX chat_eid ON Chat(eid);

CREATE TABLE ChatMembers
( id SERIAL PRIMARY KEY
, chat INTEGER REFERENCES Chat(id) ON DELETE CASCADE
, member TEXT
, membercallerid INTEGER
, membercontact INTEGER
);
ALTER TABLE ChatMembers
ADD CONSTRAINT chatmembers_uniq
UNIQUE(chat,membercontact);

CREATE TABLE ChatMessages
( id SERIAL PRIMARY KEY
, chat INTEGER REFERENCES Chat(id) ON DELETE CASCADE
, messageid TEXT
, sender TEXT
, sendercallerid INTEGER
, sendercontact INTEGER
, content TEXT
, created BIGINT
, modified BIGINT
, deleted BIGINT
);

CREATE TABLE ScimData
(
	id SERIAL PRIMARY KEY
, created TIMESTAMPTZ NOT NULL DEFAULT NOW()
, updated TIMESTAMPTZ NOT NULL DEFAULT NOW()
, externalid TEXT
, accid INTEGER
, scimparent INTEGER
, FOREIGN KEY (accid) REFERENCES Accessors(id) ON DELETE CASCADE
);

CREATE TABLE ScimUserPhones
(
	id SERIAL PRIMARY KEY
, is_primary BOOLEAN
, type TEXT
, userphoneid INTEGER
, FOREIGN KEY (userphoneid) REFERENCES UserPhones(id) ON DELETE CASCADE
);

-- needs to be done after table Accessors is created
ALTER TABLE Calls
ADD CONSTRAINT calls_agent_fkey
FOREIGN KEY (agent)
REFERENCES Accessors(id)
ON DELETE SET NULL;

ALTER TABLE ActiveCalls
ADD CONSTRAINT activecalls_agent_fkey
FOREIGN KEY (agent)
REFERENCES Accessors(id)
ON DELETE SET NULL;

ALTER TABLE CallsArchive
ADD CONSTRAINT callsarchive_agent_fkey
FOREIGN KEY (agent)
REFERENCES Accessors(id)
ON DELETE SET NULL;

CREATE TABLE RetentionPolicies
( nr INTEGER PRIMARY KEY
, name TEXT
, recdays INTEGER
, screenrecdays INTEGER
, tradays INTEGER
, bckrecdays INTEGER
, bckscreenrecdays INTEGER
);

ALTER TABLE Calls
ADD CONSTRAINT calls_reten_fkey FOREIGN KEY(retention)
REFERENCES RetentionPolicies(nr) ON DELETE SET NULL;

ALTER TABLE ActiveCalls
ADD CONSTRAINT activecalls_reten_fkey FOREIGN KEY(retention)
REFERENCES RetentionPolicies(nr) ON DELETE SET NULL;

ALTER TABLE CallsArchive
ADD CONSTRAINT callsarchive_reten_fkey FOREIGN KEY(retention)
REFERENCES RetentionPolicies(nr) ON DELETE SET NULL;


