<?php

require_once "../apresa/conf/voip_config.php";
require_once "db.php";
require_once "actions.php";
require_once "tenancy.php";
require_once "callhandler.php";

try{
	echo HandleAPI();
} catch(Exception $e)
{
	echo '399 Exception '.$e;
}
exit;

function GetSerialNumber()
{
	return trim(file_get_contents('../apresa/conf/serial'));
}

function HandleAPI()
{
	global $dbintf;
	InitDB();
  BasicConstants();

	if ($dbintf->GetConfigValue('Https')==2)
	{
		if (isset($_SERVER['HTTPS'])==false)
		{
			return "500 HTTP is used while HTTPS is required";
		}
	}

  // allow XML content type for OpenScape phones
  if ($_SERVER['CONTENT_TYPE']=="text/xml")
  {
    $rawInput = file_get_contents('php://input');
    parse_str($rawInput, $_POST);
  }
  
  // handling non UTF-8 characters
  FixInvalidChars($_POST);
	
	if (isset($_POST['auth'])==false
	&& (isset($_POST['username'])==false || isset($_POST['username'])==false) )
	{
		return '101 Missing authorization credentials';
	}
	if (isset($_POST['act'])==false)
	{
		return '101 Missing parameter act';
	}

	$auth = str_ensure_defined($_POST['auth']);
	$accessAllowed = false;
	$requiredAuth = $dbintf->GetConfigValue('ApiAuthE');
	if ($requiredAuth=='')
	{
		$requiredAuth = $dbintf->GetConfigValue('ApiAuth');
	} else
	{
		$requiredAuth = decrypt( $requiredAuth );
	}
	if ($requiredAuth!='' && $auth==$requiredAuth)
	{
		$accessAllowed = true;
	}
	
	$userLoggedIn = false;
	if (isset($_POST['username']))
	{
		LoadBasicSettings();
		
		$username = $_POST['username'];
		$password = $_POST['password'];
		if ($username!='')
		{
			$uid = -1;
			if (GenTryLogin($dbintf, $username, $password, '', $uid))
			{
				$userLoggedIn = true;
			} else
			{
				$err = "501 Login failed";
				lg('API. '.$err.'. IP:'.$_SERVER['REMOTE_ADDR']);
				return $err;
			}
			// login was OK.
			$dbintf->SetCurrentUserId($uid);
		}
	}
	
	$act = $_POST['act'];
	
	if ($accessAllowed) // global auth based access
	{
    if ($act=='ping') return Ping();
    if ($act=='startrec') return StartRec();
    if ($act=='stoprec') return StopRec();
    if ($act=='delete') return DeleteRec();
    if ($act=='delrestart') return DelRestartRec();
		if ($act=='keep') return KeepRec();
    if ($act=='isactive') return IsActive();
    if ($act=='isready') return IsReady(false);
		if ($act=='dataimport') return DataImport();
		if ($act=='dataimportconv') return DataImportConv();
		if ($act=='importnotes') return ImportNotes();
		if ($act=='silence') return SilenceRec();
		if ($act=='sound') return SoundRec();
		if ($act=='addrec') return AddRec(false);
    if ($act=='addvideo') return AddVideo(false);
		if ($act=='chat') return Chat();
		if ($act=='idmap') return IdMap();
    if ($act=='phonemap') return PhoneMap();
		if ($act=='addentry') return AddEntry();
		if ($act=='sipfilteradd') return SipFilterAdd();
		if ($act=='sipfilterdel') return SipFilterDel();
		if ($act=='sipfilterget') return SipFilterGet();
		if ($act=='dataset') return DataSet();
    if ($act=='editcall') return EditCall();
    if ($act=='annotatecall') return AnnotateCall();
		if ($act=='userconfget') return UserConfGet();
    if ($act=='reconfig') return Reconfig();
    // user management
    if ($act=='useradd') return UserAdd();
    if ($act=='useredit') return UserEdit();
    if ($act=='userdel') return UserDel();
    if ($act=='groupadd') return GroupAdd();
    if ($act=='groupdel') return GroupDel();
    if ($act=='groupedit') return GroupEdit();
    if ($act=='groupmembers') return GroupMembers();
    if ($act=='userlist') return UserList(false);
    if ($act=='grouplist') return GroupList();
		// SNMP
    if ($act=='sendtrap') return SendSNMPTrap();
    if ($act=='sendokay') return SendSNMPOkay();
	} else if ($userLoggedIn)
	{
    if ($act=='calllistget') return CallListGet();
    if ($act=='audioget') return AudioGet();
    if ($act=='isready') return IsReady(true);
    if ($act=='addvideo') return AddVideo(true);
    if ($act=='userlist') return UserList(true);
		if ($act=='addrec')
		{
			if ( ($dbintf->actuinfo['permissions'] & PM_ADDREC) != 0)
			{
				return AddRec(true);
			} else
			{
				$err = '502 The user does not have permission to add recordings';
				lg('API. '.$err);
				return $err;
			}
		} 
	}
	
	$err = '100 Unknown action for given authorization';
	lg('API. '.$err.'. IP:'.$_SERVER['REMOTE_ADDR']);
	return $err;
}

function Ping()
{
  return '0 OK (pong)';
}

function StartRec()
{
	global $dbintf;

	if (isset($_POST['localnum'])==false
	&& isset($_POST['linenr'])==false
	)
	{
		return '101 Missing parameter localnum or linenr';
	}
	$lgtext = 'API.StartRec';
	$localnum = null;
	if (isset($_POST['localnum']))
	{
		$localnum = $_POST['localnum'];
		$lgtext .= ' localnum: '.$localnum;
	}
	$linenr = null;
	if (isset($_POST['linenr']))
	{
		$linenr = $_POST['linenr'];
		$lgtext .= ' linenr: '.$linenr;
	}

	$data = '';
	if (isset($_POST['data']))
	{
		$data = $_POST['data'];
		$lgtext .= ' data: '.$data;
	}
	$data2 = '';
	if (isset($_POST['data2']))
	{
		$data2 = $_POST['data2'];
		$lgtext .= ' data2: '.$data2;
	}
	$exp = false;
	if (isset($_POST['exp']) && $_POST['exp']==1)
	{
		$exp = true;
		$lgtext .= ' exp: '.$exp;
	}
	$datadest = 'extdata';
	if (isset($_POST['datadest']))
	{
		$datadest = $_POST['datadest'];
		$lgtext .= ' datadest: '.$datadest;
	}
	$datadest2 = 'notes2';
	if (isset($_POST['datadest2']))
	{
		$datadest2 = $_POST['datadest2'];
		$lgtext .= ' datadest2: '.$datadest2;
	}
	
	// Find if there is call active on this number
	$calls = $dbintf->GetFullActiveCallList();
	$thecall = null;
	foreach ($calls as $call)
	{
    // skip finished calls (processing or spin-waiting)
    if (($call['processingstate']&4)!=0) continue;
		$match = false;
		if ($localnum!==null)
		{
			if ($call['local']==$localnum)
			{
				$match = true;
			}
		}
		if ($linenr!==null)
		{
			if ($call['linenr']==$linenr)
			{
				$match = true;
			}
		}
		if ($match)
		{
			$thecall = $call;
		}
	}
	if ($thecall==null)
	{
		if ($localnum!==null)
		{
			$lgtext .= ' ret: 1 not found';
			lg($lgtext);
			return '1 Call with local number not found';
		} else
		{
			$lgtext .= ' ret: 1 not found';
			lg($lgtext);
			return '1 Call with line number not found';
		}
	}
	if ($thecall['isarchived']==1)
	{
		$lgtext .= ' ret: 3 already started';
		lg($lgtext);
		return '3 Recording already started';
	}

	// Verification of phone number
	//   if that is enabled
	if ($dbintf->GetConfigValue('ApiVerifyTel')==1)
	{
		$callextra = StringToArray($thecall['callextra']);
		if ($localnum!==null)
		{
			$testlocal = $localnum;
		} else
		{
			$testlocal = $linenr;
		}
		// debug: $testlocal = '678';
		$dtmf = $callextra['dtmf'];
		if (strstr($dtmf,$testlocal)===FALSE)
		{
			$lgtext .= ' ret: 2 not input';
			lg($lgtext);
			return '2 Local number not found in telephone input';
		}
	}
	
	// start recording (on demand)
	$errmsg = CallAction('start',$thecall['remoteid'],$thecall['source'],$thecall['flags']);
	if ($errmsg!='')
	{
		$lgtext .= ' ret: 102 FAIL: '.$errmsg;
		lg($lgtext);
		return '102 FAIL: '.$errmsg;
	}

	// Wait until the recorded call is in the active call list
	// But with a time-out.
	// Usually within 200 ms.
	for ($i=0;$i<10;$i++)
	{
		$calls = $dbintf->GetFullActiveCallList();
		$theid = null;
		$remoteid = null;
		$callextra = null;
		foreach ($calls as $call)
		{
			$match = false;
			if ($localnum!==null)
			{
				if ($call['local']==$localnum)
				{
					$match = true;
				}
			}
			if ($linenr!==null)
			{
				if ($call['linenr']==$linenr)
				{
					$match = true;
				}
			}
			if ($match
			&& $call['isarchived']==1)
			{
				// Recording has started
				$remoteid = $call['remoteid'];
				$theid = $call['id'];
				$callextra = StringToArray($thecall['callextra']);
			}
		}
		if ($theid!=null)
		{
			$newdata = array();
			// Store custom external data in database
			if ($data != '')
			{
				if ($datadest=='notes')
				{
					$newdata['notes'] = $data;
				} else if ($datadest=='notes2')
				{
					$newdata['notes2'] = $data;
				} else if ($datadest=='extdata')
				{
					$callextra['extdata'] = $data;
				}
			}
			if ($data2 != '')
			{
				if ($datadest2=='notes')
				{
					$newdata['notes'] = $data2;
				} else if ($datadest2=='notes2')
				{
					$newdata['notes2'] = $data2;
				} else if ($datadest2=='extdata')
				{
					$callextra['extdata'] = $data2;
				}
			}
			// Also store setting
			$callextra['spexp'] = $exp ? '1' : '0';
      $newdata['callextra'] = ArrayToString($callextra);

			// Write it to DB
      $dbintf->SetActiveCallInfoRM($remoteid,$newdata);
			if ($exp)
			{
				$dbintf->SetActiveProcessingFlag($remoteid,64);
			}
			// v2. returns also remoteid
			$lgtext .= ' ret: 0 started '.$remoteid;
			lg($lgtext);
			return '0 Recording started' . "\n" . $remoteid;
		}
		usleep(200*1000);
	}

	$lgtext .= ' ret: 103 timeout';
	lg($lgtext);
	return '103 Unknown failure (timeout)';
}

function CallAction($act,$remoteid,$source,$flags)
{
  // spin?
  // check if this call was added by spin-server
  if ( ($flags&1024) != 0)
  {
    // tell spin-server
    $msg = array( 'act' => $act , 'id' => $remoteid );
    $reply = SendJsonToDomainSocket( $msg , '/tmp/apresa_spin-server');
  }
	// send msg
	// 1: analog, 5: digital TDM, 6: PRI
	$signal_reply = true;
	$err = '';
	if ($source==1 || $source==5 || $source==6)
	{
		$err = signal_service('sgncrc ' . $act . ' ' . $remoteid,$signal_reply);
	} else
	{
		$err = signal_service('sgnork ' . $act . ' ' . $remoteid,$signal_reply);
	}
	if ($err != '')
	{
		return 'Signaling: ' . $err;
	}
	if ($signal_reply != 'OK')
	{
		return 'Reply: "' . $signal_reply . '"';
	}
	return '';
}

// a/b: phone numbers to see if equivalent
// difmax: maximum number of unmatched characters
function IsNumMatch($a,$b,$difmax)
{
  // '0001' == '1' because numerical the same
	if ($a==$b) return true;
	if (endsWith($a,$b)!==false)
	{
		// b is found in a
		$dif = strlen($a) - strlen($b);
    return ($dif <= $difmax);
	}
	if (endsWith($b,$a)!==false)
	{
		// a is found in b
		$dif = strlen($b) - strlen($a);
    return ($dif <= $difmax);
	}
  return false;
}

/* This generic could be used to some degree,
	also by StartRec and StopRec,
	but it is too dangerous, or requires too much testing.
*/
function GetSpecifiedActiveCall($lgtext,$allowMissing=false)
{
	global $dbintf;
	
	$ret = array();
	
  $remotenumdifmax = 0;
  $localnumdifmax = 0;
  $tomatch = 0;
	$localnum = null;
	if (isset($_POST['localnum']))
	{
		$localnum = $_POST['localnum'];
		$lgtext .= ' localnum: '.$localnum;
    $tomatch++;
	}
	$remotenum = null;
	if (isset($_POST['remotenum']))
	{
		$remotenum = $_POST['remotenum'];
		$lgtext .= ' remotenum: '.$remotenum;
    $tomatch++;
	}
  if (isset($_POST['remotenumdifmax']))
  {
    $remotenumdifmax = (int) $_POST['remotenumdifmax'];
    $lgtext .= ' remotenumdifmax: '.$remotenumdifmax;
  }
  if (isset($_POST['localnumdifmax']))
  {
    $localnumdifmax = (int) $_POST['localnumdifmax'];
    $lgtext .= ' localnumdifmax: '.$localnumdifmax;
  }
	$linenr = null;
	if (isset($_POST['linenr']))
	{
		$linenr = $_POST['linenr'];
		$lgtext .= ' linenr: '.$linenr;
    $tomatch++;
	}
  $id = null;
	if (isset($_POST['id']))
	{
		$id = $_POST['id'];
		$lgtext .= ' id: '.$id;
    $tomatch++;
	}
  $nci = null;
  if (isset($_POST['nci']))
  {
    $nci = $_POST['nci'];
    $lgtext .= ' nci: ' . $nci;
    $tomatch++;
  }
  $contacts = null;
  if (isset($_POST['usr']))
  {
    $usr = $_POST['usr'];
    $lgtext .= ' usr: ' . $usr;
    $tomatch++;
    $userinfo = $dbintf->GetUserInfoOnUsername( $usr );
    if ($userinfo===false)
    {
      $ret['ret'] = '102 User account not found';
      return $ret;
    }
    $contacts = $dbintf->GetUserPhones($userinfo['accid']);
  } else if (isset($_POST['email']))
  {
    $email = $_POST['email'];
    $lgtext .= ' email: ' . $email;
    $tomatch++;
    $userinfo = $dbintf->GetUserInfoOnEmail( $email );
    if ($userinfo===false)
    {
      $ret['ret'] = '102 User account not found';
      return $ret;
    }
    $contacts = $dbintf->GetUserPhones($userinfo['accid']);
  } 

	if ($tomatch==0)
	{
		$ret['ret'] = '101 Missing parameter to select call';
		return $ret;
	}
  
	// Find if there is call active on this number
	$calls = $dbintf->GetFullActiveCallList();
	$thecall = null;
	foreach ($calls as $call)
	{
    // skip finished calls (processing or spin-waiting)
    if (($call['processingstate']&4)!=0) continue;
    $callextra = StringToArray($call['callextra']);
		$match = 0;
		if ($localnum!==null)
		{
			if (IsNumMatch($call['local'],$localnum,$localnumdifmax)) $match ++;
		}
		if ($remotenum!==null)
		{
			if (IsNumMatch($call['remote'],$remotenum,$remotenumdifmax)) $match ++;
		}
    if ($contacts!==null)
    {
      foreach ($contacts as $contactid)
      {
        if ($call['localcontact']==$contactid
        || $call['remotecontact']==$contactid)
        {
          // additional tenant check if assigned to other tenant
          if ($userinfo['tenantid']!==null
          // we allow matching with a call without tenant,
          // because the tenant is sometimes assigned later
          && $call['tenantid']!==null
          // the tenant should be the same
          && $call['tenantid']!=$userinfo['tenantid'] )
          {
            continue;
          }
          $match ++;
          // contacts match
          // break to prevent double matching
          break;
        }
      }
    }
		if ($linenr!==null)
		{
			if ($call['linenr']==$linenr) $match ++;
		}
    if ($id!==null)
		{
			if ($call['remoteid']==$id) $match ++;
		}
    if ($nci!='')
    {
      if (isset($callextra['NativeCallID']))
      {
        if ($callextra['NativeCallID']==$nci) $match ++;
      }
    }
		if ($match >= $tomatch)
		{
      // debug 
      // echo '<p>' . $call['id'] . ' ' . 'Match L=' . $call['local'] . ' R=' . $call['remote'] . "\n";
      // we found a matching call
			$thecall = $call;
		}
	}
	if ($thecall==null)
	{
    if ($allowMissing==false)
    {
      $lgtext .= ' ret: 1 not found';
      lg($lgtext);
      $ret['ret'] = '1 No active call with this property';
      return $ret;
    }
	}
	// we found a call
	
	$ret['call'] = $thecall;
	$ret['lg'] = $lgtext;
	return $ret;
}

function KeepRec()
{
	global $dbintf;
  $cache = (int_ensure_defined($_POST['cache'])==1);
	$inf = GetSpecifiedActiveCall('API.Keep',$cache);
	$lgtext = $inf['lg'];

	if (!isset($inf['call']))
	{
    if ($cache)
    {
      $tocopy = array( 'act', 'localnum', 'linenr', 'id', 'nci'
        , 'remotenum' , 'localnumdifmax', 'remotenumdifmax'
        , 'email' , 'usr' );
      $action = array();
      foreach ($tocopy as $key)
      {
        if (isset($_POST[$key]))
        {
          $action[ $key ] = $_POST[$key];
        }
      }

      $dbexpires = NULL;
      if (isset($_POST['expires']))
      {
        $expires = (int) $_POST['expires'];
        $lgtext .= ' expires: ' . $expires . ' s';
        $dbexpires = time() + (int) $_POST['expires'];
      }

      $dbintf->AddToActionCache(ArrayToString($action),$dbexpires);
      $lgtext .= ' ret: 0 OK (cached)';
      lg($lgtext);
      return '0 OK (cached)';
    }
		return $inf['ret'];
	}

	$thecall = $inf['call'];

  if (($thecall['processingstate']&128)!=0)
  {
    KeepPendingStoreOnDemand($thecall);
    lg($inf["lg"] . ' ret: 0 OK');
    return '0 OK';
  }
  if ($thecall['source']==0) // VoIP call
  {
    $errmsg = CallAction('keep',$thecall['remoteid'],$thecall['source'],$thecall['flags']);
    if ($errmsg!='')
    {
      $lgtext .= ' ret: 102 FAIL: '.$errmsg;
      lg($lgtext);
      return '102 FAIL: '.$errmsg;
    }
  }
	// unpack
	$callextra = StringToArray($thecall['callextra']);
	// set option
	$callextra['to-keep'] = '1';
	// pack again
	$newdata = array('callextra' => ArrayToString($callextra) );
	// store in database
	$dbintf->SetActiveCallInfoRM($thecall['remoteid'],$newdata);
	lg($lgtext . ' ret: 0 OK');
	return '0 OK';
}

function SilenceRec()
{
	$inf = GetSpecifiedActiveCall('API.Silence');
	if (isset($inf['ret']))
	{
		return $inf['ret'];
	}
	$lgtext = $inf['lg'];
	$thecall = $inf['call'];
	$errmsg = CallAction('silence',$thecall['remoteid'],$thecall['source'],$thecall['flags']);
	if ($errmsg!='')
	{
		$lgtext .= ' ret: 102 FAIL: '.$errmsg;
		lg($lgtext);
		return '102 FAIL: '.$errmsg;
	}
	$lgtext .= ' ret: 0 OK';
	lg($lgtext);
	return '0 OK';
}

function SoundRec()
{
	$inf = GetSpecifiedActiveCall('API.Sound');
	if (isset($inf['ret']))
	{
		return $inf['ret'];
	}
	$lgtext = $inf['lg'];
	$thecall = $inf['call'];
	$errmsg = CallAction('sound',$thecall['remoteid'],$thecall['source'],$thecall['flags']);
	if ($errmsg!='')
	{
		$lgtext .= ' ret: 102 FAIL: '.$errmsg;
		lg($lgtext);
		return '102 FAIL: '.$errmsg;
	}
	$lgtext .= ' ret: 0 OK';
	lg($lgtext);
	return '0 OK';
}

function DelRestartRec()
{
  $inf = GetSpecifiedActiveCall('API.DelRestart');
	if (isset($inf['ret']))
	{
		return $inf['ret'];
	}
	$lgtext = $inf['lg'];
	$thecall = $inf['call'];
	
	// delete on demand
	$errmsg = CallAction('delre',$thecall['remoteid'],$thecall['source'],$thecall['flags']);
	if ($errmsg!='')
	{
		$lgtext .= ' ret: 102 FAIL: '.$errmsg;
		lg($lgtext);
		return '102 FAIL: '.$errmsg;
	}
	
	$lgtext .= ' ret: 0 OK';
	lg($lgtext);
	return '0 OK';
}

function DeleteRec()
{
  global $dbintf;
  $cache = (int_ensure_defined($_POST['cache'])==1);
  $inf = GetSpecifiedActiveCall('API.Delete',$cache);
	$lgtext = $inf['lg'];

	if (!isset($inf['call']))
	{
    if ($cache)
    {
      $tocopy = array( 'act', 'localnum', 'linenr', 'id', 'nci'
        , 'remotenum' , 'localnumdifmax', 'remotenumdifmax'
        , 'email' , 'usr' );
      $action = array();
      foreach ($tocopy as $key)
      {
        if (isset($_POST[$key]))
        {
          $action[ $key ] = $_POST[$key];
        }
      }
      $dbexpires = NULL;
      if (isset($_POST['expires']))
      {
        $expires = (int) $_POST['expires'];
        $lgtext .= ' expires: ' . $expires . ' s';
        $dbexpires = time() + (int) $_POST['expires'];
      }
      $dbintf->AddToActionCache(ArrayToString($action),$dbexpires);
      $lgtext .= ' ret: 0 OK (cached)';
      lg($lgtext);
      return '0 OK (cached)';
    }
		return $inf['ret'];
	}
	
	// delete on demand
  $thecall = $inf['call'];
	$errmsg = CallAction('delete',$thecall['remoteid'],$thecall['source'],$thecall['flags']);
	if ($errmsg!='')
	{
		$lgtext .= ' ret: 102 FAIL: '.$errmsg;
		lg($lgtext);
		return '102 FAIL: '.$errmsg;
	}
	
	$lgtext .= ' ret: 0 OK';
	lg($lgtext);
	return '0 OK';
}

function StopRec()
{
	$inf = GetSpecifiedActiveCall('API.StopRec');
	if (isset($inf['ret']))
	{
		return $inf['ret'];
	}
	$lgtext = $inf['lg'];
	$thecall = $inf['call'];
	if ($thecall['isarchived']==0)
	{
		$lgtext .= ' ret: 3 rec not started';
		lg($lgtext);
		return '3 Recording is not started';
	}
	
	// stop recording (on demand)
	$errmsg = CallAction('stop',$thecall['remoteid'],$thecall['source'],$thecall['flags']);
	if ($errmsg!='')
	{
		$lgtext .= ' ret: 102 FAIL: '.$errmsg;
		lg($lgtext);
		return '102 FAIL: '.$errmsg;
	}
	
	$lgtext .= ' ret: 0 OK';
	lg($lgtext);
	return '0 OK';
}

function IsActive()
{
	global $dbintf;

	if (isset($_POST['id'])==false
	)
	{
		return '101 Missing parameter id';
	}
  $id = null;
	if (isset($_POST['id']))
	{
		$id = $_POST['id'];
	}

	// Find if there is call active on this number
	$calls = $dbintf->GetFullActiveCallList();
	$thecall = null;
	foreach ($calls as $call)
	{
		$match = false;
		{
			if ($call['remoteid']==$id)
			{
				$match = true;
			}
		}
		if ($match)
		{
			$thecall = $call;
		}
	}
	if ($thecall==null)
	{
		return '1 No call with this property';
	}
	if ($thecall['isarchived']==0)
	{
		return '3 Recording is not started';
	}
  $duration = (time() - $thecall['startdate']) / 1;
	return '0 Call recording active' . "\n" . $duration;
}

function IsReady($byUser)
{
	global $dbintf;

	if (isset($_POST['id'])==false
	)
	{
		return '101 Missing parameter id';
	}
  $id = null;
	if (isset($_POST['id']))
	{
		$id = $_POST['id'];
	}
  
  $CNF = '1 Call not found or no permission';

	// Find if there is finished call
  $call = $dbintf->GetCallInfoRM( $id );
  if ($call!==false)
  {
    if ($byUser)
    {
      $call['permissions'] = $dbintf->GetCallPermissions($call);
      if ( ($call['permissions'] & PM_LIST) == 0 )
      {
        return $CNF;
      }
    }
    return '0 OK';
  }
  $call = $dbintf->GetActiveCallInfoRM( $id );
  if ($call!==false)
  {
    if ($byUser)
    {
      $call['permissions'] = $dbintf->GetCallPermissions($call);
      if ( ($call['permissions'] & PM_LIST) == 0 )
      {
        return $CNF;
      }
    }
  }
  if ($call===false)
  {
    return $CNF;
  }
  if ( ($call['processingstate'] & 4) != 0 )
  {
    return '3 Recording is converting';
  } else
  {
    return '2 Call is active';
  }
}

function DateStrToTs($t)
{
	if (strlen($t)<16) return 0;
	$year		= (int) substr($t,0,4);
	$month	= (int) substr($t,5,2);
	$day		= (int) substr($t,8,2);
	$hour 	= (int) substr($t,11,2);
	$minute = (int) substr($t,14,2);
	$second = (int) substr($t,17,2);
	if (strlen($t)>19)
	{
		$milli = (int) substr($t,20,3);
		// mktime can handle '60' seconds,
		// it will automatically increment minutes (and hours)
		if ($milli>500) $second ++;
	}
	return mktime($hour,$minute,$second,$month,$day,$year);
}

function DataImportConv()
{
	global $dbintf;
	//var_dump( $_FILES );
	if (isset($_FILES['csvdata'])==false)
	{
		return '1 No csvdata';
	}
	$upl = $_FILES['csvdata'];
	if ($upl['error']!=0)
	{
		// error in upload
		return '2 Error in file upload';
	} else if (is_uploaded_file($upl['tmp_name'])==false)
	{
		return '2 Error in file upload';
	}
	
	$newconf = array();
	
	// there is a good upload
	// let's read it
	//echo "<pre>";
	$h = fopen($upl['tmp_name'],'r');
	if (! $h ) return '1 Error reading file';
	while (feof($h)==false)
	{
		$line = rtrim( fgets($h) );
		//echo 'LINE: '.$line."\n";
		$dat = ParseCsvLine($line);
		if (count($dat)!=2) continue;
		$newconf[ $dat[0] ] = $dat[1];
	}
	fclose($h);

	$dbintf->SetConfigsPart( $newconf , 'dataimportconv' );
	
	return '0 Imported '.count($newconf).' items';
}

function AddEntry()
{
	global $dbintf;
	if (isset($_FILES['csvdata'])==false)
	{
		return '1 No csvdata';
	}
	$upl = $_FILES['csvdata'];
	if ($upl['error']!=0)
	{
		// error in upload
		return '2 Error in file upload';
	} else if (is_uploaded_file($upl['tmp_name'])==false)
	{
		return '2 Error in file upload';
	} 
	$conf = array();
	
	$h = fopen($upl['tmp_name'],'r');
	if (! $h ) return '1 Error reading file';
	$dbintf->StartTransaction();
	$info = AddEntryAction($h,$conf);
  $dbintf->CommitTransaction();
	fclose($h);
	return $info['ret'];
}

function DataImport()
{
	global $dbintf;
	//var_dump( $_FILES );
	if (isset($_FILES['csvdata'])==false)
	{
		return '1 No csvdata';
	}
	$upl = $_FILES['csvdata'];
	if ($upl['error']!=0)
	{
		// error in upload
		return '2 Error in file upload';
	} else if (is_uploaded_file($upl['tmp_name'])==false)
	{
		return '2 Error in file upload';
	} 
	// there is a good upload
	// let's read it
	// but first read configs
	$conf = array();
	$possible_confs = array
	(	'timedifmax' => 5
	,	'numdifmax' => 2
	, 'datadest' => 'notes2'
	, 'format' => 1
	, 'verbose' => 0
	);
	foreach ($possible_confs as $key=>$def_value)
	{
		if (isset($_POST[$key]))
		{
			$conf[ $key ] = $_POST[$key];
		} else
		{
			$conf[ $key ] = $def_value;
		}
	}
  
  $valid_dest = array( 'notes','notes2','notes3','notes4','notes5'
  , 'data1', 'data2', 'data3', 'data4', 'data5'
    ,'extdata' );
  if ( in_array($conf['datadest'],$valid_dest)==false )
  {
    return '3 Invalid data destination';
  }
	
	$h = fopen($upl['tmp_name'],'r');
	if (! $h ) return '1 Error reading file';
	$dbintf->StartTransaction();
	$info = DataImportAction($h,$conf);
  $dbintf->CommitTransaction();
	fclose($h);
	
	$t = '0 Imported OK:'.$info['ok'].' fail:'.$info['fail'];
	if ($conf['verbose'])
	{
		$t .= "\r\n" . $info['log'];
	}
	return $t;
}

function TextDiff( $a , $b )
{
	// only check last 7 digits at the end
	// to get more matches
	$a = substr($a,-7);
	$b = substr($b,-7);

	if (strlen($a)==0) return 1000;
	if (strlen($b)==0) return 1000;
	//echo $a . ' ' . $b;
	if ($a==$b) return 0;
	if (strpos($a,$b)!==false)
	{
		// b is found in a
		return strlen($a) - strlen($b);
	}
	if (strpos($b,$a)!==false)
	{
		// a is found in b
		return strlen($b) - strlen($a);
	}
	return 1000;
}

function ParseCustomDate($datestr,$timestr)
{
	// parse date
	$sep = '-';
	if (strpos($datestr,'/')!==false) $sep = '/';
	if (strpos($datestr,'.')!==false) $sep = '.';
	$darr = explode($sep,$datestr);
	if ( count($darr) != 3 ) return 0;
	// parse time
	$sep = ':';
	$tarr = explode($sep,$timestr);
	if ( count($tarr) != 3 ) return 0;
	// create time stamp
	$ret = mktime( $tarr[0],$tarr[1],$tarr[2], $darr[1], $darr[0], $darr[2] );
	return $ret;
}

function ParseDuration($durstr)
{
	$sep = ':';
	$tarr = explode($sep,$durstr);
	if ( count($tarr) != 3 ) return 0;
	$seconds = (int) $tarr[2] + 60 * (int) $tarr[1] + 60*60 * (int) $tarr[0] ;
	return $seconds;
}

function ParseStrangeTimestamp($dt)
{
	// dd/mm/yyyy hh:mm:ss
	$year = (int) substr($dt,6,4);
	$month = (int) substr($dt,3,2);
	$day = (int) substr($dt,0,2);
	$hour = (int) substr($dt,11,2);
	$minu = (int) substr($dt,14,2);
	$sec = (int) substr($dt,17,2);
	return mktime($hour,$minu,$sec,$month,$day,$year);
}

function AddEntryAction($h,$conf)
{
	global $dbintf;
	$dbintf->SetAdmin();

	$FullSipId = (int) $dbintf->GetConfigValue('FullSipId');
	
	$info = array();
	$info['ret'] = '1 Not implemented';
	
	//echo '<pre>';
	while (feof($h)==false)
	{
		$line = rtrim( fgets($h) );
		if (int_ensure_defined($conf['verbose']))
		{
			$info['log'] .= 'Line: ' . $line . "\r\n";
		}
		$dat = ParseCsvLine($line);
		if (count($dat)<5) continue;
		$id = $dat[0];
		if (is_numeric($id[0])==false) continue;

		$data = array();
		$data['localfull'] = $dat[9];
		$data['remotefull'] = $dat[6];
		$data['direction'] = $dat[3];
		$data['cardno'] = 'ADDENTRY';
		
		$startdate = ParseStrangeTimestamp($dat[1]);
		$data['startdate'] = $startdate;
		//echo date('D M G:i:s Y',$data['startdate']);
		$data['duration'] = $dat[2];
		
		$data['recorderid'] = 0;
		$data['local'] = IdSimplify($data['localfull']);
		$data['remote'] = IdSimplify($data['remotefull']);
		$data['local2'] = '';
		$data['remote2'] = '';

		$data['flags'] = 0; // initial state
		$data['isarchived'] = 0;
		
		$callextra = array();
		//$callextra['XXX'] = $_POST['XXX'];
		$data['callextra'] = ArrayToString($callextra);

		$rec_uniqifier = $dbintf->GetRecIdsNext();
		
    $file_extension = '';
		$data['filename'] = date('Ymd_His',$startdate) . '_x' . $rec_uniqifier . '.' . $file_extension;
		$data['directory'] = date('Y/m/d/H',$startdate);
		$data['remoteid'] = date('Ymd_His',$startdate) . '_x' . $rec_uniqifier;
		
		$data['waittime'] = -1;
		$data['source'] = 0; // unknown
		$data['encoding'] = 51; // unknown, GSM or G.711
		$data['encryption'] = 0; // no
    $convert = 0;
		$data['processingstate'] = ($convert)?4:0; // audio processing
		$data['lineid'] = 0; // not applicable
		$data['localcontact'] = 0;
		$data['remotecontact'] = 0;
		$cid = GetSipIdToUse($data['localfull'],$FullSipId);
    $local_is_remote = 0;
		$ret = $dbintf->GetOrCreateCallerID($cid,$data['local2' ],$local_is_remote?0:1);
		$data['localcallerid'] = $ret["id"];
		$data['localcontact'] = $ret["contactid"];
		$cid = GetSipIdToUse($data['remotefull'],$FullSipId);
    $remote_is_local = 0;
		$ret = $dbintf->GetOrCreateCallerID($cid,$data['remote2'],$remote_is_local?1:0);
		$data['remotecallerid'] = $ret["id"];
		$data['remotecontact'] = $ret["contactid"];
		$dbintf->InsertCall($data);
	}
	
	$info['ret'] = '0 OK';
	return $info;
}

function DataImportActionLine($dat,$conf,&$info)
{
    global $dbintf;
    
		if ($conf['format']==1)
		{
			if (count($dat)!=5) return;
			//var_dump($dat);
			$start = DateStrToTs($dat[0]);
			$stop  = DateStrToTs($dat[1]);
			if ($conf['verbose'])
			{
				//$info['log'] .= 'Timestamp Start: ' . $start . ' Stop: ' . $stop . "\r\n";
			}
			if ($stop==0) $duration = -1;
			else $duration = $stop - $start;
			$remote = trim($dat[2]);
			$local = trim($dat[3]);
			$custom_data = trim($dat[4]);
		}
		if ($conf['format']==2)
		{
			if (count($dat)<12) return;
			$start = ParseCustomDate($dat[0],$dat[1]);
			if ($start==0) return;
			$duration = ParseDuration($dat[11]);
			if ($duration==0) return;
			$stop = $start + $duration;
			// the var $remote is used for matching
			$local = '';
			$remote = '';
			// do not use for matching
			$know_remote = trim($dat[5]);
			//echo 'Start: '.$start." Len:".$dur."\r\n";
			
			$custom_data = $dat[7];
			if (isset($dataconf[$custom_data]))
			{
				$newdata = $dataconf[$custom_data];
				if ($newdata!='')
				{
					$custom_data = $newdata;
				}
			}
		}
		//continue;
		//echo date('c',$start) . " " . date('c',$stop) . "\r\n";
		//echo 'Data: ' . $custom_data . "\r\n";
		//echo 'KnowRemote: ' . $know_remote . "\r\n";
		//echo 'Remote: ' . $remote . "\r\n";
		//echo 'Local: ' . $local . "\r\n";
		//continue;
		
		// For each row in import
		// get the calls in a time range around the mentioned start date.
		// Evaluate each and choose best match, or fail.
		
		// Settings:
		// max seconds difference in start time and duration
		$max_timepnt = $conf['timedifmax']; 
		// extra unmatched characters in local and remote
		$max_numpnt  = $conf['numdifmax'];
		// can be: notes, notes2, extdata
		$data_dest   = $conf['datadest'];
		

		$searchrange_seconds = $max_timepnt * 2;
		$sk = 'startdate';
		$search = array();
		$search['fd'] = $start - $searchrange_seconds;
		$search['ld'] = $start + $searchrange_seconds;
		$listconfig = array();
		$listconfig['allrecords'] = true;
		$calls = $dbintf->GetCallList($sk,'',$search,$listconfig);
		$cnt = $calls['cnt'];
		unset($calls['cnt']);
		$best_call = null;
		$best_pnt = 9999999;
		for ($i=0;$i<$cnt;$i++)
		{
			$call = $calls[$i];
			// Points are negative
			$numpnt = 0;
			$timepnt = 0;
			
			// How much is start date/time different (in seconds)
			$timepnt += abs( $start - $call['startdate'] );
			// How much is duration different (in seconds)
			if ($duration != -1)
			{
				$timepnt += abs( $duration - $call['duration'] );
			}

			// match local/remote
			if ($local!='')
			{
				$numpnt += TextDiff( $local , $call['local'] );
			}
			if ($remote!='')
			{
				$numpnt += TextDiff( $remote , $call['remote'] );
			}

			$pnt = $timepnt + $numpnt * 10;
			
			
			if ($conf['verbose'])
			{
				//$info['log'] .= 'Call ' . $call['id'] . " time:$timepnt num:$numpnt tot: $pnt\r\n";
			}
			
			if ($timepnt > $max_timepnt)
			{
				if ($conf['verbose']) $info['log'] .= 'Call ' . $call['id']
			  . ' ' . date('D M G:i:s Y',$call['startdate'])
				 . " Time difference\r\n";
				continue;
			}
			if ($numpnt > $max_numpnt)
			{
				if ($conf['verbose']) $info['log'] .= 'Call ' . $call['id']
			  . ' ' . date('D M G:i:s Y',$call['startdate'])
				. " Phonenum. difference R: " . $remote . '-' . $call['remote']
				. " L: " . $local . '-' . $call['local'] . "\r\n";
				continue;
			}
			
			if ($conf['verbose']) $info['log'] .= 'Call ' . $call['id']
			  . ' ' . date('D M G:i:s Y',$call['startdate'])
				. " R: " . $remote . '-' . $call['remote']
				. " L: " . $local . '-' . $call['local'] . " Pnts: " . $pnt . "\r\n";
				
			if ($pnt<$best_pnt)
			{
				$best_call = $call;
				$best_pnt = $pnt;
			}
			
		}
		if ($cnt==0)
		{
			if ($conf['verbose']) $info['log'] .= "No call within $searchrange_seconds s.\r\n";
		}
		if ($best_call !== null)
		{
			// We have found a matching
			// Store custom data
			if ($data_dest=='notes'
      || $data_dest=='notes2'
      || $data_dest=='notes3'
      || $data_dest=='notes4'
      || $data_dest=='notes5' 
      || $data_dest=='data1'
      || $data_dest=='data2'
      || $data_dest=='data3'
      || $data_dest=='data4'
      || $data_dest=='data5'
      )
			{
        $newdata = array( $data_dest => $custom_data );
        $dbintf->SetCallInfo($best_call['id'],$newdata);
			} else if ($data_dest=='notes')
			{
				$dbintf->EditRecordingNotes($best_call['id'],$custom_data);
			} else if ($data_dest=='extdata')
			{
				$callextra = StringToArray( $best_call['callextra'] );
				$callextra['extdata'] = $custom_data;
				$newdata = array('callextra' => ArrayToString($callextra) );
				$dbintf->SetCallInfo($best_call['id'],$newdata);
			}
			$info['ok'] ++;
			
		} else
		{
			$info['fail'] ++;
			if ($conf['verbose']) $info['log'] .= "ERR: No match.\r\n";
		}
  
}

function DataImportAction($h,$conf)
{
	global $dbintf;
	
	$dbintf->SetAdmin();
	
	$dataconf = $dbintf->LoadConfigsPart('dataimportconv');
	//var_dump( $dataconf );
	
	$info = array();
	$info['ok'] = 0;
	$info['fail'] = 0;
	$info['log'] = '';
	
	//echo '<pre>';
  $csvParser = new CsvParser();
  
	while (feof($h)==false)
	{
		$line = fgets($h);
    $csvParser->addLine( $line );
    
		if ($conf['verbose'])
		{
			$info['log'] .= '* * * Line: ' . $line;
      if (strlen($line)>1 && $line[strlen($line)-1]!="\n")
      {
        $info['log'] .= "\r\n";
      }
		}
    
    $data = $csvParser->getData();
    if ($data !== null)
    {
      DataImportActionLine( $data, $conf, $info );
    }
    
	}
  $csvParser->flush();
  $data = $csvParser->getData();
  if ($data !== null)
  {
    DataImportActionLine( $data, $conf, $info );
  }
	
	
	return $info;
}

function ParseToTimestamp($dt)
{
  if (substr($dt,4,1)=='-')
  {
    // yyyy-mm-dd hh:mm:ss
    $year = (int) substr($dt,0,4);
    $month = (int) substr($dt,5,2);
    $day = (int) substr($dt,8,2);
    $hour = (int) substr($dt,11,2);
    $minu = (int) substr($dt,14,2);
    $sec = (int) substr($dt,17,2);
    return mktime($hour,$minu,$sec,$month,$day,$year);
  } else
  {
    // take as literal timestamp
    return (int) $dt;
  }
}

// for adding a screen or video recording to an existing record
function AddVideo($byUser)
{
  global $dbintf;
  
	$lgtext = 'API.AddVideo'
	. ' posterip='.$_SERVER['REMOTE_ADDR'];

  if ($byUser)
  {
    if ( ($dbintf->actuinfo['permissions'] & PM_ADDREC) == 0)
    {
      $lgtext .= ' ret: 502 The user does not have permission to add recordings';
      lg($lgtext);
      return '502 The user does not have permission to add recordings';
    }
  }

  // check recording file
  if (isset($_FILES['file'])==false)
  {
    $lgtext .= ' ret: 1 No file';
    lg($lgtext);
    return '1 No file';
  }

  $upl = $_FILES['file'];

  if ($upl['error']!=0)
  {
    $lgtext .= ' ret: 2 Error in file upload';
    lg($lgtext);
    // error in upload
    return '2 Error in file upload';
  } 
  if (is_uploaded_file($upl['tmp_name'])==false)
  {
    $lgtext .= ' ret: 2 Error in file upload';
    lg($lgtext);
    return '2 Error in file upload';
  } 
    
  if (file_exists($upl['tmp_name'])==false)
  {
    $lgtext .= ' ret: 2 Uploaded file does not exist';
    lg($lgtext);
    return '2 Uploaded file does not exist';
  } 
  
  $filesize = filesize($upl['tmp_name']);
  if ($filesize==0)
  {
    $lgtext .= ' ret: 2 Uploaded file is empty';
    lg($lgtext);
    return '2 Uploaded file is empty';
  }
  
  $CNF = '3 Call not found or no permission';
  
  $id = str_ensure_defined($_POST['id']); // is remoteid
	$lgtext .= ' id=' . $id;
  $info = $dbintf->GetCallInfoRM($id);
  if ($info===false)
  {
    $lgtext .= ' ret: ' . $CNF;
    lg($lgtext);
    return $CNF;
  } else
  {
    if ($byUser)
    {
      $info['permissions'] = $dbintf->GetCallPermissions($info);
      if ( ($info['permissions'] & PM_LIST) == 0 )
      {
        $lgtext .= ' ret: ' . $CNF;
        lg($lgtext);
        return $CNF;
      }
    }
  }
  
  // check if already has screen recording
  if ( ($info['flags']&2) != 0 )
  {
    $lgtext .= ' ret: 5 Call already has video or screen recording';
    lg($lgtext);
    return '5 Call already has video or screen recording';
  }
  
  $fn_extension = '.' . get_filename_extension($upl['name']);
  
  $filename = 'video' . change_filename_extension($info['filename'],$fn_extension);
	$dst = AUDIO_PATH.'/'.$info['directory'].'/'.$filename;
  $finalDst = $dst;
  
  // encryption (see client.php)
  $ALE = (int) $dbintf->GetConfigValue('ALE');
  if ($ALE!=0)
  {
    // encryption
		if ($dbintf->GetConfigValue('AleDisk')=='1')
		{
			// dont use dev shm here
			$dst = $dst . '.tmp';
		} else
		{
			if (IsEnoughSpaceAvailOnDevShm( $upl['size'] )==false)
			{
				return '9 Not enough space on /dev/shm';
			}
			$parts = pathinfo($dst);
			$dir = '/dev/shm/apresa/'.substr($parts['dirname'],12);
			@mkdir($dir,0777,TRUE);
			$dst = $dir . '/upl_' . $parts['basename'];
		}
  }
  
  $tmpfn = $upl['tmp_name'] . '.upl';
  if (move_uploaded_file($upl['tmp_name'],$dst)==false)
  {
    $lgtext .= ' ret: 4 Failed to move uploaded file';
    lg($lgtext);
    return '4 Failed to move uploaded file';
  }
  
  // again for encryption
  if ($ALE!=0)
  {
    // encryption
    $cmdToExec = 'apresa_crypt -e -i '.$dst.' -o '.$finalDst;
    $retValue = 999;
    $dummyOutput = array();
    exec($cmdToExec,$dummyOutput,$retValue);
    // remove source unencrypted file
    if (file_exists($dst))
    {
      DeleteFn($dst);
    }
    if ($retValue!=0)
    {
      $lgtext .= ' ret: 8 Encryption failed';
      lg($lgtext);
      return '8 Encryption failed';
    }				
  }  

  $dbintf->SetCallInfo($info['id']
  , array( 'screenfn' => $filename )
  );
  // set flag to indicate we have a screen recording (or video)
  $dbintf->SetFlagOnForIDs(array($info['id']),2);
	
	$fileSizeVideo = filesize($finalDst);
	$lgtext .= ' fz=' . showkb($fileSizeVideo);
	
  // tenancy disk usage
  if ($info['tenantid']!=NULL)
  {
    require_once 'tenancy.php';
    UpdateTnDiskUsage( $info['tenantid'], filesize($finalDst), 0);
		// the above could trigger auto-deletion of this recording itself
		// so it is important that we have registered already the screen recording,
		// so that is taken into account while deleting the recording.
  }
  
	$lgtext .= ' OK';
	lg($lgtext);
	return '0 OK';
}

function Chat()
{
	global $dbintf;
  
	$lgtext = 'API.Chat' . ' posterip='.$_SERVER['REMOTE_ADDR'];
	
	// check data file
	if (isset($_FILES['data'])==false)
	{
		$lgtext .= ' ret: 1 No data';
		lg($lgtext);
		return '1 No data';
	}
	$upl = $_FILES['data'];
	if ($upl['error']!=0)
	{
		$lgtext .= ' ret: 2 Error in file upload';
		lg($lgtext);
		// error in upload
		return '2 Error in file upload';
	} else if (is_uploaded_file($upl['tmp_name'])==false)
	{
		$lgtext .= ' ret: 2 Error in file upload';
		lg($lgtext);
		return '2 Error in file upload';
	} 
	
	if (file_exists($upl['tmp_name'])==false)
	{
		$lgtext .= ' ret: 2 Uploaded file does not exist';
		lg($lgtext);
		return '2 Uploaded file does not exist';
	} 
	
	$filesize = filesize($upl['tmp_name']);
	if ($filesize==0)
	{
		$lgtext .= ' ret: 2 Uploaded file is empty';
		lg($lgtext);
		return '2 Uploaded file is empty';
	}
	
	$tmpfn = $upl['tmp_name'] . '.upl';
	if (move_uploaded_file($upl['tmp_name'],$tmpfn)==false)
	{
		$lgtext .= ' ret: 2 Failed to move uploaded file';
		lg($lgtext);
		return '2 Failed to move uploaded file';
	}

	$root = simplexml_load_file($tmpfn);
	if ($root===false
	|| $root->getName()!='conversations')
	{
		// failed to parse
		$lgtext .= ' ret: 3 Failed to parse';
		lg($lgtext);
		return '3 Failed to parse';
	}
	
	define('CONVTYPE_CHAT',1);
	define('CONVTYPE_CHANNEL',2);
	
	foreach ($root->chat as $chat)
	{
		$chatdata = array();
		$chatdata['chatid'] = (string) $chat->id;
		if (isset($chat->topic ))
		{
			$chatdata['topic'] = (string) $chat->topic;
		}
		if (isset($chat->tenantName))
		{
			$tenantName = (string) $chat->tenantName;
			$matchingTenants = $dbintf->GetGroupInfoOnName( $tenantName );
			if (count($matchingTenants) >= 1)
			{
				$tenant = $matchingTenants[0];
				if ($tenant['is_tenant'])
				{
					$chatdata['tenantid'] = $tenant['id'];
				}
			}
		}
		// determine eid
		$eid = 'chats(' . $chatdata['chatid'] . ')';
		$chatdata['eid'] = $eid;
		$chatdata['type'] = CONVTYPE_CHAT;
		
		$memberList = GetChatMemberListFromXml($chat->members);
		$msgs = GetChatMessagesFromXml($chat->messages);
		
		$dbintf->SetChat($chatdata, $memberList, $msgs);
	}
	
	foreach ($root->channel as $channel)
	{
		$chatdata = array();
		$chatdata['teamid'] = (string) $channel->teamId;
		$chatdata['channelid'] = (string) $channel->channelId;
		$chatdata['teamname'] = (string) $channel->teamName;
		$chatdata['channelname'] = (string) $channel->channelName;
		if (isset($chat->tenantName))
		{
			$tenantName = (string) $chat->tenantName;
			$matchingTenants = $dbintf->GetGroupInfoOnName( $tenantName );
			if (count($matchingTenants) >= 1)
			{
				$tenant = $matchingTenants[0];
				if ($tenant['is_tenant'])
				{
					$chatdata['tenantid'] = $tenant['id'];
				}
			}
		}
		// determine eid
		$eid = "teams(" . $chatdata['teamid'] . ").channels(" . $chatdata['channelid'] . ")";
		$chatdata['eid'] = $eid;
		$chatdata['type'] = CONVTYPE_CHANNEL;
		
		$memberList = GetChatMemberListFromXml($channel->members);
		$msgs = GetChatMessagesFromXml($channel->messages);
		
		$dbintf->SetChat($chatdata, $memberList, $msgs);
	}
	
	DeleteFn($tmpfn);	
	
	$lgtext .= ' OK';
	lg($lgtext);
	
	return '0 OK';
}

function GetChatMessagesFromXml($elem)
{
	global $dbintf;
	$msgs = array();
	foreach ($elem->message as $message)
	{
		$msg = array();

		$msg['messageid'] = (string) $message->id;
		
		if (isset($message->from))
		{
			$msg['sender'] = (string) $message->from;
			$ret = $dbintf->GetOrCreateCallerID($msg['sender'],'',2);
			$msg['sendercallerid'] = $ret["id"];
			$msg['sendercontact'] = $ret["contactid"];
		}
		
		if (isset($message->content))
		{
			$msg['content'] = (string) $message->content;
		}
		$msg['action'] = (string) $message->action;
		if ($msg['action']=='deleted')
		{
			$msg['content'] = NULL;
		}
		
		if (isset($message->created))
		{
			$msg['created'] = GetTimestampFromUtcXml( (string) $message->created);
		}
		if (isset($message->modified))
		{
			$msg['modified'] = GetTimestampFromUtcXml( (string) $message->modified);
		}
		if (isset($message->deleted))
		{
			$msg['deleted'] = GetTimestampFromUtcXml( (string) $message->deleted);
		}
		
		$msgs[] = $msg;
	}
	return $msgs;
}

function GetChatMemberListFromXml($elem)
{
	global $dbintf;
	$memberList = array();
	foreach ($elem->member as $member)
	{
		$mem = array();
		$mem['member'] = (string) $member;
		$ret = $dbintf->GetOrCreateCallerID($mem['member'],'',2);
		$mem['membercallerid'] = $ret["id"];
		$mem['membercontact'] = $ret["contactid"];
		$memberList[] = $mem;
	}
	return $memberList;
}

function GetTimestampFromUtcXml($elem)
{
	if (isset($elem)==false) return NULL;
	$dt = (string) $elem;

	if (substr($dt,20,3)!='UTC') return NULL;
	
	// yyyy-mm-dd hh:mm:ss
	$year = (int) substr($dt,0,4);
	$month = (int) substr($dt,5,2);
	$day = (int) substr($dt,8,2);
	$hour = (int) substr($dt,11,2);
	$minu = (int) substr($dt,14,2);
	$sec = (int) substr($dt,17,2);
	// gmmktime assumes UTC
	return gmmktime($hour,$minu,$sec,$month,$day,$year);
	
}

function AddRec($byUser)
{
	global $dbintf;
  $dbintf->AllowEditNames();

	$lgtext = 'API.AddRec'
	. ' posterip='.$_SERVER['REMOTE_ADDR'];

  $missedCall = (int_ensure_defined($_POST['missed'])==1);
	
  if ($missedCall)
  {
    $tmpfn = 'X';
    $filesize = 0;
  } else
  {
    // check recording file
    if (isset($_FILES['recording'])==false)
    {
      $lgtext .= ' ret: 1 No recording';
      lg($lgtext);
      return '1 No recording';
    }
    $upl = $_FILES['recording'];
    if ($upl['error']!=0)
    {
      $lgtext .= ' ret: 2 Error in file upload';
      lg($lgtext);
      // error in upload
      return '2 Error in file upload';
    } else if (is_uploaded_file($upl['tmp_name'])==false)
    {
      $lgtext .= ' ret: 2 Error in file upload';
      lg($lgtext);
      return '2 Error in file upload';
    } 
    
    if (file_exists($upl['tmp_name'])==false)
    {
      $lgtext .= ' ret: 2 Uploaded file does not exist';
      lg($lgtext);
      return '2 Uploaded file does not exist';
    } 
    
    $filesize = filesize($upl['tmp_name']);
    if ($filesize==0)
    {
      $lgtext .= ' ret: 2 Uploaded file is empty';
      lg($lgtext);
      return '2 Uploaded file is empty';
    }
    
    $tmpfn = $upl['tmp_name'] . '.upl';
    if (move_uploaded_file($upl['tmp_name'],$tmpfn)==false)
    {
      $lgtext .= ' ret: 2 Failed to move uploaded file';
      lg($lgtext);
      return '2 Failed to move uploaded file';
    }
  }

  $SN = GetSerialNumber();
  
	$ConfRecEnc = $dbintf->GetConfigValue('RecEnc');
	
	// convert?
	$convert = false;
  if ($missedCall==false)
  {
    $has_avconv = file_exists("/usr/bin/avconv") || file_exists("/usr/bin/ffmpeg");
    $pi = pathinfo($upl['name']);
    $file_extension = $pi['extension'];
    if ($file_extension=='3gpp'
    || ($file_extension=='amr' && $has_avconv) 
    || ($file_extension=='m4a' && $has_avconv) 
    || ($file_extension=='aac' && $has_avconv) 
    || ($file_extension=='3gp' && $has_avconv) )
    {
      $convert = true;
      if ($ConfRecEnc=='mp3') $file_extension = 'mp3';
      else $file_extension = 'wav';
    }
    if ($file_extension=='wav' && $ConfRecEnc=='mp3')
    {
      $convert = true;
      $file_extension = 'mp3';
    }
  }

	$localnum = '';
	if (isset($_POST['local']))
	{
		$localnum = $_POST['local'];
		$lgtext	.= ' local='.$localnum;
	}
	
	$mobEntry = null;
	if (isset($_POST['appauth']))
	{
		$dt = str_ensure_defined($_POST['startdate']);
		$hour = (int) substr($dt,11,2);
		$minu = (int) substr($dt,14,2);
		$sec = (int) substr($dt,17,2);
		$appauth = $_POST['appauth'];
		$expected = (int) ( ($hour + $minu*60 + $sec*60*60) * 13.4);
		$expected += 10309;
		if ($appauth != $expected)
		{
			echo $expected . "\r\n";
			DeleteFn($tmpfn);
			return '3 Application had invalid authorization code';
		} 
	} else
	{
		$imeiInput = trim( str_ensure_defined($_POST['imei'] ));
		$lgtext	.= ' imei='.$imeiInput;
		$imei = extractDigits( $imeiInput );
		$imeiLen = strlen( $imei );
		if ($imeiLen==0 // empty
		|| $imeiLen==14 // basic
		|| $imeiLen==15 // + check digit
		|| $imeiLen==16) // + software rev.
		{
		} else
		{
			$lgtext .= ' ret: 3 IMEI invalid';
			lg($lgtext);
			DeleteFn($tmpfn);
			return '3 IMEI invalid';
		}
		$mobs = $dbintf->GetMobileDevices();
		$mobcnt = count($mobs);
		$moblic = (int) $dbintf->GetConfigValue('MobileChCnt');
		//var_dump( $mobs );
		$mobEntry = null;
		$linkedByImei = false;
		// first try match on IMEI
		for ($i=0;$i<$moblic && $i<$mobcnt;$i++)
		{
			if ($imeiLen>0
			&& substr($imei,0,14)== substr($mobs[$i]['imei'],0,14) )
			{
				$mobEntry = $mobs[$i];
				$linkedByImei = true;
			}
			//echo '<br>TST'.$mobs[$i]['imei'];
		}
		if ($mobEntry===null && $imeiLen==0)
		{
			// match on local phone number
			for ($i=0;$i<$moblic && $i<$mobcnt;$i++)
			{
				if ($imei==''
				&& $localnum!=''
				&& $localnum==$mobs[$i]['phonenum'] )
				{
					$mobEntry = $mobs[$i];
				}
				//echo '<br>TST'.$mobs[$i]['imei'];
			}
		}
		if ($mobEntry===NULL)
		{
			if ($imeiLen==0)
			{
				$lgtext .= ' ret: 4 Local ID unlicensed';
				lg($lgtext);
				DeleteFn($tmpfn);
				return '4 Local ID unlicensed';
			} else
			{
				$lgtext .= ' ret: 4 IMEI unlicensed';
				lg($lgtext);
				DeleteFn($tmpfn);
				return '4 IMEI unlicensed';
			}
		}
		if ($mobEntry['phonenum']!='')
		{
			$localnum = $mobEntry['phonenum'];
		}
		if ($byUser)
		{
			$phones = $dbintf->GetUserPhones($dbintf->actuinfo['accid']);
			$inPhoneList = false;
			foreach ($phones as $phone)
			{
				$contact = $dbintf->GetContactInfo($phone);
				if ($contact['ctype']==0)
				{
					if ($contact['callerid']==$localnum)
					{
						$inPhoneList = true;
					}
				}
			}
			if ($inPhoneList==false)
			{
				if ($linkedByImei)
				{
					$lgtext .= ' ret: 7 Local ID does not belong to user';
					lg($lgtext);
					DeleteFn($tmpfn);
					return '7 Local ID of this IMEI does not belong to user';
				} else
				{
					$lgtext .= ' ret: 6 Local ID does not belong to user';
					lg($lgtext);
					DeleteFn($tmpfn);
					return '6 Local ID does not belong to user';
				}
			}
		}
	}

	$data = array();
	
	$FullSipId = (int) $dbintf->GetConfigValue('FullSipId');
  $SimplifyId = true;
  if (isset($_POST['simplifyid']) && $_POST['simplifyid']=='0')
  {
    $SimplifyId = false;
  }
	
	$serialnum = GetSerialNumber();
	if ( ((int)$serialnum) == 0 )
	{
		$lgtext .= ' ret: 12 Apresa has no serial number';
		lg($lgtext);
		DeleteFn($tmpfn);
		return '12 Apresa has no serial number';
	}

	$data['localfull'] = '';
	if (isset($_POST['local']))
	{
		$data['localfull'] = $_POST['local'];
	}
	if ($mobEntry!==null && $mobEntry['phonenum']!='')
	{
		$data['localfull'] = $mobEntry['phonenum'];
	}
	$data['remotefull'] = '';
	if (isset($_POST['remote']))
	{
		$data['remotefull'] = $_POST['remote'];
	}
  if ($SimplifyId)
  {
    $data['local'] = IdSimplify($data['localfull']);
    $data['remote'] = IdSimplify($data['remotefull']);
  } else
  {
    $data['local'] = $data['localfull'];
    $data['remote'] = $data['remotefull'];
  }
	$connectedfull = '';
	if (isset($_POST['connected']))
	{
		$connectedfull = $_POST['connected'];
		if ($SimplifyId)
		{
			$data['connected'] = IdSimplify($connectedfull);
		} else
		{
			$data['connected'] = $connectedfull;
		}
	}
  $localname = '';
  if (isset($_POST['localname']))
  {
    $localname = $_POST['localname'];
  }
  $remotename = '';
  if (isset($_POST['remotename']))
  {
    $remotename = $_POST['remotename'];
  }
	
	$data['local2'] = '';
	$data['remote2'] = '';
	
	$data['direction'] = int_ensure_defined($_POST['direction']);
	$direction = $data['direction'];
	
	$remote_is_local = ($direction==2);
	$local_is_remote = ($direction==3);
	
  for ($i=1;$i<=5;$i++)
  {
    $key = 'notes';
    if ($i!=1) $key .= $i;
    if (isset($_POST[$key]))
    {
      $data[$key] = $_POST[$key];
    }
    $key = 'data' . $i;
    if (isset($_POST[$key]))
    {
      $data[$key] = $_POST[$key];
    }
  }
	if (isset($_POST['category']))
	{
		$data['category'] = (int) $_POST['category'];
	}
	
	$recorderid = null;
	if ($byUser==false)
	{
		// a custom recorder?
		if (isset($_POST['recorderid']) && $_POST['recorderid']!='')
		{
			$recordername = $_POST['recorderid'];
			if (isset($_POST['recordername']) && $_POST['recordername']!='')
			{
				$recordername = $_POST['recordername'];
			}
			$recorderid = $dbintf->GetOrCreateRecorderId
					( $_POST['recorderid'], $recordername, 70 );
		}
	}
	if ($recorderid===null)
	{
		$recorderid = $dbintf->GetOrCreateRecorderId($serialnum,"Apresa$serialnum");
	}
	$data['recorderid'] = $recorderid;
	
	$startdate = ParseToTimestamp($_POST['startdate']);
	$data['startdate'] = $startdate;
	//echo date('D M G:i:s Y',$data['startdate']);
	$data['duration'] = (int) $_POST['duration'];
  
	$data['flags'] = 0;
	$data['isarchived'] = 1;
	
  if ($missedCall)
  {
    $data['isarchived'] = 0;
    $data['flags'] |= 64;
    $data['duration'] = -1;
  }
  
	$callextra = array();
	//$callextra['XXX'] = $_POST['XXX'];
	$data['callextra'] = ArrayToString($callextra);

	$rec_uniqifier = $dbintf->GetRecIdsNext();
	
	$data['filename'] = date('Ymd_His', $startdate) . '_x' . $rec_uniqifier . '.' . $file_extension;
  if ($missedCall) $data['filename'] = date('Ymd_His',$startdate) . '_x' . $rec_uniqifier;
	$data['directory'] = date('Y/m/d/H',$startdate);
	$data['remoteid'] = date('Ymd_His',$startdate) . '_x' . $rec_uniqifier;
  
  $uplname = $upl['name'];
  if ($missedCall) $uplname = '';
  
	$lgtext	.= ' local='.$data['localfull']
	. ' remote='.$data['remotefull']
	. ' direction='.$data['direction']
	. ' reportedstartdate='.$_POST['startdate']
	. ' dur='.$data['duration']
	. ' uplnm='.$uplname
	. ' tmpfn='.$tmpfn
	. ' storefn='.$data['filename']
	. ' fz='.$filesize
	;

	$extrasize = filesize($tmpfn);
	
	$data['waittime'] = -1;

	$data['source'] = 0; // unknown
	$data['encoding'] = 51; // unknown, GSM or G.711
	$data['encryption'] = 0; // no (not used for ALE)
	$data['processingstate'] = 4; // audio processing
	$data['lineid'] = 0; // not applicable

	$data['localcontact'] = 0;
	$data['remotecontact'] = 0;

	$cid = GetSipIdToUse($data['localfull'],$FullSipId);
  if ($SimplifyId==false) $cid = $data['localfull'];
	$ret = $dbintf->GetOrCreateCallerID($cid,$data['local2' ],$local_is_remote?0:1,$localname);
  if ($localname!='' && $dbintf->GetContactName($ret['contactid']) == '')
  {
    $dbintf->EditContact($ret['contactid'],$localname);
  } 
	$data['localcallerid'] = $ret["id"];
	$data['localcontact'] = $ret["contactid"];
  
	$cid = GetSipIdToUse($data['remotefull'],$FullSipId);
  if ($SimplifyId==false) $cid = $data['remotefull'];
	$ret = $dbintf->GetOrCreateCallerID($cid,$data['remote2'],$remote_is_local?1:0,$remotename);
  if ($remotename!='' && $dbintf->GetContactName($ret['contactid']) == '')
  {
    $dbintf->EditContact($ret['contactid'],$remotename);
  } 
	$data['remotecallerid'] = $ret["id"];
	$data['remotecontact'] = $ret["contactid"];
	
	if (isset($data['connected']))
	{
		$cid = GetSipIdToUse($connectedfull,$FullSipId);
		if ($SimplifyId==false) $cid = $connectedfull;
		$ret = $dbintf->GetOrCreateCallerID($cid,'',2);
		$data['connectedcallerid'] = $ret["id"];
		$data['connectedcontact'] = $ret["contactid"];
	}
	
  // prefer to use specified tenant
  $tenantid = null;
  if (isset($_POST['tenant']))
  {
    $tenantName = $_POST['tenant'];
    $matchingTenants = $dbintf->GetGroupInfoOnName( $tenantName );
    if (count($matchingTenants) >= 1)
    {
      $tenant = $matchingTenants[0];
      if ($tenant['is_tenant'])
      {
        $tenantid = $tenant['id'];
      }
    }
  }
  if ($tenantid==null)
  {
    // apply based on local tenant configuration (its user phones)
    $tenantid = CalcTenant($data);
  }

  // if uploaded by a tenant user,
  // then always assign to this tenant
  if ($byUser)
  {
    if ($dbintf->actuinfo['tenantid']!==null)
    {
      $tenantid = $dbintf->actuinfo['tenantid'];
    }
  }
  $data['tenantid'] = $tenantid;
  
  $ALE = (int) $dbintf->GetConfigValue('ALE');
	$AleDisk = (int) $dbintf->GetConfigValue('AleDisk');
  
  if($tenantid !== null)
  {
    $group = $dbintf->GetGroupInfo( $data['tenantid'] );
    $key = $dbintf->GetCallEncryptionKey($group['accid']);
    if($key && $key['inuse'])
    {
      $ALE = 2;
    }
  }
  
  if ($ALE==1)
  { 
    $data['flags'] |= 16;
  }
  else if($ALE==2)
  {
    $data['flags'] |= 2048;
  }
  
  // Note: both copyrec and copyconvrec will update tenant usage,
  // through specialtask.php ActiveDone
  // It will also wait some time for active call below to have been added, if needed.
	$handler_act = 'copyrec';
	if ($convert) $handler_act = 'copyconvrec';
	if (signal_service($handler_act
		.' '.$tmpfn
		.' '.$data['directory'].'/'.$data['filename']
		.' '.$data['remoteid']
		.' '.$ConfRecEnc
    .' '.$ALE
		.' '.$AleDisk
		)!='')
	{
		$lgtext .= ' ret: 5 Failed to copy recording';
		lg($lgtext);
		DeleteFn($tmpfn);
		return '5 Failed to copy recording';
	}
	
	$dbintf->InsertActiveCall($data);

	$lgtext .= ' OK';
	lg($lgtext);
	
	return '0 OK' . "\n" . $data['remoteid'];
}

function extractDigits($t)
{
	$len = strlen( $t );
	$num = '';
	for ($i=0;$i<$len;$i++)
	{
		if (ctype_digit($t[$i]))
		{
			$num .= $t[$i];
		}
	}
	return $num;
}

function PhoneMap()
{
	global $dbintf;
  $lgtext = 'API.PhoneMap';
  $map = ensure_defined($_POST['map'], null);
  $dbintf->SetConfigsPart( array( 'PhoneMap' => $map ) , 'o' );
  lg($lgtext);
  $msg = array();
  EnginesReconfig($msg,$dbintf);
  return '0 OK';
}

function IdMap()
{
	global $dbintf;
	//var_dump( $_FILES );
	if (isset($_FILES['csvdata'])==false)
	{
		return '1 No csvdata';
	}
	$upl = $_FILES['csvdata'];
	if ($upl['error']!=0)
	{
		// error in upload
		return '2 Error in file upload';
	} else if (is_uploaded_file($upl['tmp_name'])==false)
	{
		return '2 Error in file upload';
	}
	
	$target = 'notes2';
	if (isset($_POST['target']) && $_POST['target']!='')
	{
		$target = $_POST['target'];
	}
	if ($target=='notes'
	|| $target=='notes2')
	{
		// OK
	} else
	{
		return '3 Invalid target (only notes and notes2 are supported)';
	}
	// this should become a setting
	$replace = int_ensure_defined($_POST['replace'])!=0;
	
	// there is a good upload
	// let's read it
	//echo "<pre>";
	$h = fopen($upl['tmp_name'],'r');
	if (! $h ) return '1 Error reading file';
	$linenr = 0;
	$importcnt = 0;
	$dbintf->StartTransaction();
	if ($replace)
	{
		$dbintf->ClearIdMap();
	}
	while (feof($h)==false)
	{
		$linenr ++;
		$line = rtrim( fgets($h) );
		//echo 'LINE: '.$line."\n";
		$dat = ParseCsvLine($line);
		if (count($dat)!=2) continue;
		if ($linenr==1) continue;
		$value = $dat[0];
		$theid = $dat[1];
		//echo 'OK' . "\n";
		if ($replace)
		{
			$dbintf->AddToIdMap( $theid, $value, $target );
		} else
		{
			$dbintf->EditIdMap( $theid, $value, $target );
		}
		$importcnt ++;
	}
	$dbintf->CommitTransaction();
	fclose($h);
	
	return '0 OK. Imported: '.$importcnt;
}

function ImportNotes()
{
	global $dbintf;
	$dbintf->SetAdmin();

	//var_dump( $_FILES );
	if (isset($_FILES['csvdata'])==false)
	{
		return '1 No csvdata';
	}
	$upl = $_FILES['csvdata'];
	if ($upl['error']!=0)
	{
		// error in upload
		return '2 Error in file upload';
	} else if (is_uploaded_file($upl['tmp_name'])==false)
	{
		return '2 Error in file upload';
	}
	
	$target = 'notes2';
	if (isset($_POST['target']) && $_POST['target']!='')
	{
		$target = $_POST['target'];
	}
	if ($target=='notes'
	|| $target=='notes2')
	{
		// OK
	} else
	{
		return '3 Invalid target (only notes and notes2 are supported)';
	}
	
	if (isset($_POST['date'])==false)
	{
		return '4 No date specified.';
	}
	
	// determine the day (date)
	// and its time range
	$date = $_POST['date'];
	$year		= (int) substr($date,0,4);
	$month	= (int) substr($date,5,2);
	$day		= (int) substr($date,8,2);
	$start = mktime(  0,  0,  0, $month, $day, $year );
	$end   = mktime( 23, 59, 59, $month, $day, $year );
	//echo $start . ' - ' . $end . ' - ' . ($end-$start);
	//echo date( "Y-m-d", $start );

	// there is a good upload
	// let's read it
	//echo "<pre>";
	$h = fopen($upl['tmp_name'],'r');
	if (! $h ) return '1 Error reading file';
	$names = array();
	$linenr = 0;
	$importcnt = 0;
	while (feof($h)==false)
	{
		$linenr ++;
		$line = rtrim( fgets($h) );
		//echo 'LINE: '.$line."\n";
		$dat = ParseCsvLine($line);
		if (count($dat)!=2) continue;
		if ($linenr==1) continue;
		$value = $dat[0];
		$theid = $dat[1];
		//echo 'OK' . "\n";
		$names[ $theid ] = $value;
		$importcnt ++;
	}
	fclose($h);
	
	
	$sk = 'startdate';
	$search = array();
	$search['fd'] = $start;
	$search['ld'] = $end;
	$listconfig = array();
	$listconfig['allrecords'] = true;
	$calls = $dbintf->GetCallList($sk,'',$search,$listconfig);
	$cnt = $calls['cnt'];
	unset($calls['cnt']);
	
	//echo 'There were ' . $cnt . ' recorded calls on that day';

	//echo '<pre>';
	$matchcnt = 0;
	$dbintf->StartTransaction();
	foreach ($calls as $call)
	{
		//echo 'Call '.$call['remoteid'];
		$direction = $call['direction'];
		//echo ' ' . $direction;
		$match = false;
		$local = $call['local'];
		if (isset($names[ $local ]))
		{
			$match = $names[ $local ];
		}
		if ($direction==2)
		{
			$local = $call['remote'];
			if (isset($names[ $local ]))
			{
				$match = $names[ $local ];
			}
		}
		if ($match !== false)
		{
			//echo ' toset '.$match;
			if ($target=='notes' ) $dbintf->EditRecordingNotes ($call['id'],$match);
			if ($target=='notes2') $dbintf->EditRecordingNotes2($call['id'],$match);
			$matchcnt++;
		}
		//echo "\n";
	}
  $dbintf->CommitTransaction();

	
	return '0 OK. Set ' . $matchcnt . ' of ' . $cnt ;
}

// use Reconfig to apply to engine
function UserDel()
{
	global $dbintf;
	$dbintf->SetAdmin();
  
  $usr = $_POST['usr'];
  $userinfo = $dbintf->GetUserInfoOnUsername($usr);
  if ($userinfo==false)
  {
    return '1 User account does not exist';
  }

  LogAction('API Delete user account');
  
  if ($dbintf->DeleteUserAccount( $userinfo['id'] )==false)
  {
    return '2 Failed to delete user account';
  }

  return '0 OK';  
}

// use Reconfig to apply to engine
function UserEdit()
{
	global $dbintf;
	$dbintf->SetAdmin();

  // look up user that we want to find
  $usr = str_ensure_defined($_POST['usr']);
  $theuser = $dbintf->GetUserInfoOnUsername( $usr );  
  if ($theuser==false)
  {
    return '10 User with this username not found';
  }

  if (isset($_POST['groups']))
  {
    $groups = $_POST['groups'];
    $groupNames = explode(',',$groups);
    $groupAccIds = array();
    foreach ($groupNames as $groupName)
    {
      $groupName = trim($groupName);
      if ($groupName=='') continue;
      
      $group = $dbintf->GetSingleGroupInfoByName($groupName);
      if ($group===false)
      {
        return '1 Group not found';
      }
      $groupAccIds[] = $group['accid'];
    }
  }
  
  $usrdata = array();

	$name = '';
  if (isset($_POST['name']))
  {
    $name = $usrdata['name'] = $_POST['name'];
  }
  if (isset($_POST['email']))
  {
    $usrdata['email'] = $_POST['email'];
  }
  if (isset($_POST['enabled']))
  {
    $usrdata['enabled'] = ($_POST['enabled']!='0')?1:0;
  }
  if (isset($_POST['logonmethod']))
  {
    $usrdata['logonmethod'] = $_POST['logonmethod'];
  }
  if (isset($_POST['pwd']))
  {
    $usrdata['rawpwd'] = $_POST['pwd'];
  }

  if ( isset($usrdata['name'])
  && (int)$dbintf->GetConfigValue('DupNamesAllowed') == 0
  && $dbintf->NameInUseExcept($usrdata['name'],$theuser['accid']))
  {
    return '3 Name is already in use';
  }
  if ( isset($usrdata['enabled'])
  && $usrdata['enabled']!=0
  && $dbintf->UsrExistsExcept($theuser['usr'],$theuser['accid']))
  {
    return '4 Username is already in use';
  }

  LogAction('API Edit user account');
  
  if ($dbintf->EditUserAccount($theuser['id'],$usrdata)==false)
  {
    return '5 Failed to add user account';
  }

  // edit phones
  if (isset($_POST['phones']))
  {
    $phones = array();
    $phonenums = explode(',',$_POST['phones']);
    foreach ($phonenums as $phonenum)
    {
      $phonenum = trim($phonenum);
      if (strlen($phonenum)==0) continue;
      $res = $dbintf->GetOrCreateCallerID($phonenum,'',1,$name);
      $obj = array();
      $obj['contactid'] = $res['contactid'];
      $phones[] = $obj;
    }
    if ($dbintf->SetUserPhones($theuser['accid'],$phones)==false)
    {
      return '7 Failed to set user phones';
    }
  }

  // set membership
  if (isset($_POST['groups']))
  {
    if ($dbintf->SetGroupsOfWhichMemberLC( $theuser['accid'] , $groupAccIds )==false)
    {
      return '8 Failed to set group membership';
    }
  }
    
  return "0 OK";
}

// use Reconfig to apply to engine
function UserAdd()
{
	global $dbintf;
	$dbintf->SetAdmin();
  
  $rod_def = 0;
  $groups = str_ensure_defined($_POST['groups']);
  $groupNames = explode(',',$groups);
  $groupAccIds = array();
  foreach ($groupNames as $groupName)
  {
    $groupName = trim($groupName);
    if ($groupName=='') continue;
    
    $group = $dbintf->GetSingleGroupInfoByName($groupName);
    if ($group===false)
    {
      return '1 Group not found';
    }
    $groupAccIds[] = $group['accid'];
    if ($group['tenantid']!=0)
    {
      $tenantid = (int) $group['tenantid'];
      $tn_prefix = 'TN' . $tenantid . '_';
      $rod_def = (int) $dbintf->GetConfigValue( $tn_prefix . 'StoreOnDmdDef' );
    }
  }
  
  $usrdata = array();
  $usrdata['usr'] = str_ensure_defined($_POST['usr']);
  
  $name = str_ensure_defined($_POST['name']);
  if ($name=='') $name = $usrdata['usr'];
  $usrdata['name'] = str_ensure_defined($_POST['name']);
  $usrdata['email'] = str_ensure_defined($_POST['email']);
  $usrdata['mphones'] = '';
  $usrdata['enabled'] = (str_ensure_defined($_POST['enabled'])!='0')?1:0;
  $usrdata['rod'] = $rod_def;
  $usrdata['logonmethod'] = int_ensure_defined($_POST['logonmethod']);
  if (isset($_POST['pwd']))
  {
    $usrdata['rawpwd'] = $_POST['pwd'];
  }
  $usrdata['playtimelim'] = 0;

  if ( (int)$dbintf->GetConfigValue('DupNamesAllowed') == 0
  && $dbintf->NameInUse($usrdata['name']))
  {
    return '3 Name is already in use';
    $isok = false;
  }
  if ($usrdata['enabled']!=0 && $dbintf->UsrExists($usrdata['usr']))
  {
    return '4 Username is already in use';
  }
  
  LogAction('API Add user account');

  $userid = false;
  if ($dbintf->AddUserAccount($userid,$usrdata,0)==false)
  {
    return '5 Failed to add user account';
  }

  // determine accid
  $userinfo = $dbintf->GetUserInfo($userid);
  if ($userinfo==false)
  {
    return '6 Could not get info of added account';
  }
  
  // add phones
  $phones = array();
  $phonenums = explode(',',str_ensure_defined($_POST['phones']));
  foreach ($phonenums as $phonenum)
  {
    $phonenum = trim($phonenum);
    if (strlen($phonenum)==0) continue;
    $res = $dbintf->GetOrCreateCallerID($phonenum,'',1,$name);
    $obj = array();
    $obj['contactid'] = $res['contactid'];
    $phones[] = $obj;
  }
  if ($dbintf->SetUserPhones($userinfo['accid'],$phones)==false)
  {
    return '7 Failed to set user phones';
  }
  
  // set membership
  if ($dbintf->SetGroupsOfWhichMemberLC( $userinfo['accid'] , $groupAccIds )==false)
  {
    return '8 Failed to set group membership';
  }
    
  // for testing, use C# test program
  // apresa\toolsrc\tests\apitest-net\Program.cs

  return "0 OK";
  //return "0 OK\r\n" . $userid;
}

function UserList($byUser)
{
  global $dbintf;

  if ($byUser)
  {
    ;
  } else
  {
    $dbintf->SetAdmin();
  }
  
  $ret = array();
  
  $list = array();
  
  $users = $dbintf->GetUserListForApi();
  foreach ($users as $user)
  {
    $uinfo = array('name'=>$user['name'],'usr'=>$user['usr']);
    if ($user['tenantid']!=0)
    {
      $uinfo['tenantid'] = (int) $user['tenantid'];
      $tenant = $dbintf->GetGroupInfo( $user['tenantid'] );
      $uinfo['tenantname'] = $tenant['name'];
    }

		$userphones = $dbintf->GetUserPhones($user['accid']);
		$phones = array();
		foreach ($userphones as $userphone)
		{
			$ct = $dbintf->GetContactInfo( $userphone );
			if (isset($ct['callerid']))
			{
				$phones[] = $ct['callerid'];
			}
		}
		$uinfo['phones'] = $phones;
		
    $usergroups = $dbintf->GetGroupsOfWhichMember( $user['accid'] );
		$groups = array();
    foreach ($usergroups as $usergroup)
    {
      $groupInfo = $dbintf->GetGroupInfoOnAccid( $usergroup );
      $groups[] = $groupInfo['name'];
    }
		$uinfo['groups'] = $groups;
		
    $list[] = $uinfo;
  }

  $ret['users'] = $list;

  return json_encode($ret);
}

function GroupMembers()
{
  global $dbintf;
  
  $ret = array();
  
  // find existing group
  $group = $dbintf->GetSingleGroupInfoByName($_POST['name']);
  if ($group===false)
  {
    return json_encode( array('error'=>'Group does not exist') );
  }
  
  LogAction('API List group members',array('level'=>'view') );
  
  $ret['group'] = $group['name'];
  
  $list = array();
  
  $members = $dbintf->GetGroupMembers($group['accid']);
  foreach ($members as $member)
  {
    $user = $dbintf->GetUserInfoOnAccId($member);
    $list[] = array('name'=>$user['name'],'usr'=>$user['usr']);
  }

  $ret['members'] = $list;

  return json_encode($ret);
}

function GroupList()
{
  global $dbintf;

  LogAction('API List groups',array('level'=>'view') );
  
  $ret = array();
  
  $onlyTenants = (int_ensure_defined($_POST['tenant']) == 1);
  
  $list = array();
  
  $groups = $dbintf->GetGroupList();
  foreach ($groups as $group)
  {
    if ($onlyTenants && $group['is_tenant']==0) continue;
    $list[] = $group['name'];
  }
  
  $ret['groups'] = $list;
  
  return json_encode($ret);
}

function GroupAdd()
{
	global $dbintf;
	$dbintf->SetAdmin();
  
  $superGroupInfo = $dbintf->GetSingleGroupInfoByName(str_ensure_defined($_POST['supergroup']));
  if ($superGroupInfo===false)
  {
    return '1 Super group not found';
  }
  $name = str_ensure_defined($_POST['name']);
  $oldGroups = $dbintf->GetGroupInfoOnName($name);
  if (count($oldGroups)>0)
  {
    return '2 Group with this name already exists';
  }
  
  $grpdata = array();
  $grpdata['name'] = $name;
  $grpdata['super'] = $superGroupInfo['accid'];
  $grpdata['typegrp'] = 0;
  $grpdata['is_tenant'] = (int_ensure_defined($_POST['tenant'])!=0) ? 1 : 0;

  LogAction('API Add group');
  
  $groupid = -1;
  if ($dbintf->AddGroup($groupid,$grpdata)==false)
  {
    return '3 Failed to add group';
  }
  
  if ($grpdata['is_tenant']!=0)
  {
    // tenant settings?
    $tenant_cfg_keys = array( 'MaxDiskUsage' );
    $configs_tn = $dbintf->LoadConfigsPart('tn');
    $configs_tn_new = array();
    $tn_prefix = 'TN' . $groupid . '_';
    foreach ($tenant_cfg_keys as $param)
    {
      if (isset($_POST[$param]))
      {
        $new_value = $_POST[$param];
        $old_value = '';
        $key = $tn_prefix . $param;
        if (isset($configs_tn[$key]))
        {
          $old_value = $configs_tn[$key];
        }
        if ($old_value !== $new_value)
        {
          $configs_tn_new[$key] = $new_value;
        }
      }
    }
    if (count($configs_tn_new)>0)
    {
      $dbintf->SetConfigsPart($configs_tn_new, 'tn');
    }
  }
  
  return '0 OK';
}

function GroupEdit()
{
	global $dbintf;
	$dbintf->SetAdmin();

  // find existing group
  $group = $dbintf->GetSingleGroupInfoByName($_POST['name']);
  if ($group===false)
  {
    return '1 Group does not exist';
  }
  $groupid = $group['id'];

  $grpdata = array();
  if (isset($_POST['newname']))
  {
    $grpdata['name'] = $_POST['newname'];
  }
  if (isset($_POST['supergroup']))
  {
    $supergroup = $dbintf->GetSingleGroupInfoByName($_POST['supergroup']);
    if ($supergroup===false)
    {
      return '2 Supergroup does not exist';
    }
    $grpdata['super'] = $supergroup['accid'];
  }
  if (isset($_POST['tenant']))
  {
    $grpdata['is_tenant'] = ($_POST['tenant']!=0)?1:0;
  }
  
  $isok = true;
  $errmsg = '';

  LogAction('API Edit group');
  
  $dbintf->StartTransaction();

  if (count($grpdata)>0)
  {
    $isok = $dbintf->EditGroup($group['id'], $grpdata );
  }

  if ($isok)
  {
    $supert = 0; $subt = 0;
    if ($dbintf->TenantInsideTenant($supert,$subt))
    {
      $superg = $dbintf->GetGroupInfo($supert);
      $subg = $dbintf->GetGroupInfo($subt);
      $errmsg = "A tenant cannot be a subgroup of another tenant";
      $isok = false;
    }
  }

  if ($isok)
  {
    $dbintf->CommitTransaction();
  } else
  {
    $dbintf->CancelTransaction();
    return '100 ' . $errmsg;
  }
  
  $groupid = $group['id'];
  
  if ($group['is_tenant']==1
  && isset($grpdata['is_tenant']) && $grpdata['is_tenant']==0)
  {
    // no longer a tenant
    lg('Disabling tenancy for group '.$group['name']); 
    // unassign any calls of this tenant
    // also those who are active
    $dbintf->UnassignTenantCalls($groupid);
    // remove settings and stats
    $tn_prefix = 'TN' . $groupid . '_';
    $dbintf->DeleteConfigsWithPrefix($tn_prefix);
    // recalculate user and group tenant membership
    $dbintf->CalcTenantUsers();
    $dbintf->UnassignVTapTenant($groupid);
  }
  
  // reload group info
  $group = $dbintf->GetGroupInfo($group['id']);
  
  if ($group['is_tenant']!=0)
  {
    // tenant settings?
    $tenant_cfg_keys = array( 'MaxDiskUsage' );
    $configs_tn = $dbintf->LoadConfigsPart('tn');
    $configs_tn_new = array();
    $tn_prefix = 'TN' . $groupid . '_';
    foreach ($tenant_cfg_keys as $param)
    {
      if (isset($_POST[$param]))
      {
        $new_value = $_POST[$param];
        $old_value = '';
        $key = $tn_prefix . $param;
        if (isset($configs_tn[$key]))
        {
          $old_value = $configs_tn[$key];
        }
        if ($old_value !== $new_value)
        {
          $configs_tn_new[$key] = $new_value;
        }
      }
    }

    // apply settings
    $dbintf->ApplyTenantSettings( $groupid );
    
    if (count($configs_tn_new)>0)
    {
      $dbintf->SetConfigsPart($configs_tn_new, 'tn');
    }
    
  }
  
  return '0 OK';
}

function GroupDel()
{
	global $dbintf;
	$dbintf->SetAdmin();

  $name = $_POST['name'];
  $groupinfo = $dbintf->GetSingleGroupInfoByName($name);
  if ($groupinfo===false)
  {
    return '1 Group does not exist';
  }
  
  $subgroups = array();
  $dbintf->DetermineAllGroups();
  $dbintf->FindSubGroups($subgroups,$groupinfo['accid']);
  if (count($subgroups)>0)
  {
    return '2 The group contains subgroups';
  }

  if ($groupinfo['is_tenant']==1)
  {
    if (isset($_POST['DeleteData'])==false || $_POST['DeleteData']!=1 )
    {
      return '4 Cannot delete tenant group without deleting associated data';
    }
    LogAction('API Delete tenant data');
    
    $DelFromBackup = (int) $_POST['DelFromBackup'];
    $msg = array();
    $err = signal_service('TnDelete '.$DelFromBackup.' '.$groupinfo['id']);
    if ($err != '') return '4 Signaling: ' . $err;
    // TnDelete also deletes through DeleteTenant in specialtask
    // and a lot of other things
    // Lets wait until it is ready.
    $startWait = time();
    $maxWait = 5;
    if (isset($_POST['MaxWait']))
    {
      $maxWait = (int) $_POST['MaxWait'];
    }
    while (true)
    {
      if ($dbintf->GetGroupInfo($groupinfo['id'])===false)
      {
        // group is gone
        if (IsTenantActionBusy()==false) break;
      }
      if (time() - $startWait > $maxWait)
      {
        return '5 Group not yet deleted when timeout reached';
      }
      usleep(300 * 1000);
    }
    
  } else
  {
    LogAction('API Delete group');
    if ($dbintf->DeleteGroup($groupinfo['id'])==false)
    {
      return '3 Failed to delete group';
    }
  }
  
  return '0 OK';
}

function SipFilterDel()
{
	global $dbintf;
	$dbintf->SetAdmin();
	
	$idParam = 'sipid';
	$filterParam = 'SipFilters';
	$filterTypeParam = 'SipFilterType';
	$filterName = 'SIP filter';

	$lgtext = 'API.SipFilterDel';

	if (isset($_POST[$idParam])==false)
	{
		return '101 Missing parameter '.$idParam;
	}
	$theid = $_POST[$idParam];
	$lgtext .= ' ' . $idParam . '=' . $theid;

	$filterType = (int) $dbintf->GetConfigValue($filterTypeParam);
	if ($filterType==0)
	{
		$lgtext .= ' filter no enabled';
		lg($lgtext);
		return '2 Filter not enabled';
	}
	
	$filters = explode(',',$dbintf->GetConfigValue($filterParam));
	if (in_array( $theid, $filters )==false)
	{
		$lgtext .= ' ret 1 not in list';
		lg($lgtext);
		return '1 Not in '.$filterName;
	} 
	$keys = array_keys($filters,$theid,true);
	foreach ($keys as $key)
	{
		unset($filters[$key]);
	}
	
	$toset = implode(',',$filters);
	$dbintf->SetConfigsPart(array($filterParam=>$toset),'o');
	
  $msg = array();
  EnginesReconfig($msg,$dbintf);
  
	$lgtext .= ' ret 0 OK';
	lg($lgtext);
	return '0 OK.';
}

function SipFilterAdd()
{
	global $dbintf;
	$dbintf->SetAdmin();
	
	$idParam = 'sipid';
	$filterParam = 'SipFilters';
	$filterTypeParam = 'SipFilterType';
	$filterName = 'SIP filter';

	$lgtext = 'API.SipFilterAdd';

	if (isset($_POST[$idParam])==false)
	{
		return '101 Missing parameter '.$idParam;
	}
	$theid = $_POST[$idParam];
	$lgtext .= ' ' . $idParam . '=' . $theid;
	
	$filterType = (int) $dbintf->GetConfigValue($filterTypeParam);
	if ($filterType==0)
	{
		$lgtext .= ' filter no enabled';
		lg($lgtext);
		return '2 Filter not enabled';
	}
	
	$filters = explode(',',$dbintf->GetConfigValue($filterParam));
	if (in_array( $theid, $filters ))
	{
		$lgtext .= ' ret 1 already in list';
		lg($lgtext);
		return '1 Already in '.$filterName;
	} 
	$filters[] = $theid;
	
	$toset = implode(',',$filters);
	$dbintf->SetConfigsPart(array($filterParam=>$toset),'o');
  
  $msg = array();
  EnginesReconfig($msg,$dbintf);
	
	$lgtext .= ' ret 0 OK';
	lg($lgtext);
	return '0 OK.';
}

function SipFilterGet()
{
	global $dbintf;
	$dbintf->SetAdmin();
	
	$filterParam = 'SipFilters';
	$filterTypeParam = 'SipFilterType';
	$filterName = 'SIP filter';

	lg('API.SipFilterGet');

	$filters = explode(',',$dbintf->GetConfigValue($filterParam));
	$t = "0 OK\r\n";
	$filterType = (int) $dbintf->GetConfigValue($filterTypeParam);
	$descr = '';
	if ($filterType==0) $descr = ' no filter';
	if ($filterType==1) $descr = ' positive filter';
	if ($filterType==2) $descr = ' negative filter';
	$t .= 'type ' . $dbintf->GetConfigValue($filterTypeParam) . $descr . "\r\n";
	if ($filterType!=0)
	{
		foreach ($filters as $sipid)
		{
			$t .= $sipid . "\r\n";
		}
	}
	return $t;
}

function AnnotateCall()
{
	global $dbintf;
  
  $lgtext = 'API.AnnotateCall';
	if (isset($_POST['id'])==false)
	{
    $lgtext .= ' ret: 101 Parameter id missing';
    lg($lgtext);
    return '101 Parameter id missing';
	}
  $id = $_POST['id'];
  $lgtext .= ' id: '.$id;
  
  $call = $dbintf->GetCallInfoRM($id);
  if ($call === false)
  {
    $lgtext .= ' ret: 1 No call with specified ID';
    lg($lgtext);
    return "1 No call with specified ID";
  }
  
  if (isset($_POST['count'])==false)
  {
    $lgtext .= ' ret: 101 Parameter count missing';
    lg($lgtext);
    return '101 Parameter count missing';
  }
  
  $count = (int) $_POST['count'];
  if ($count < 0)
  {
    return '9 Invalid value for count';
  }
  
  $dbintf->SetAdmin();
  
  // replace existing
  $dbintf->DelAnnotationsOfCall( $call['id'] );

  $i = 0;
  for ($i=1;$i<=$count;$i++)
  {
    if (isset($_POST['text' . $i])==false
    || isset($_POST['time' . $i])==false)
    {
      continue;
    }
    $marker = array();
    $marker['call_id'] = $call['id'];
    $marker['descr'] = $_POST['text' . $i];
    $marker['timepos'] = (float) $_POST['time' . $i];
    $dbintf->AddAnnotation( $marker );
  }
  
  return '0 OK';
}

function RedetermineAgent($call, & $ref_data, $is_active)
{
	global $dbintf;
  
  $DetermineAgent = $dbintf->GetConfigValue('DetermineAgent');
	if ( $DetermineAgent===false) $DetermineAgent = 1;
	if ( (int) $DetermineAgent == 0 ) return;
  
  $MultiTenancy = (int) $dbintf->GetConfigValue('MultiTenancy');
	// determine if tenant assignment is enabled and slow
	// (slow = assigned at end of call)
	$SlowTenancy = false;
  if ($MultiTenancy==1)
	{
    $UsrFilterLocal = (int) $dbintf->GetConfigValue('UsrFilterLocal');
    $TenantQuickAssign = (int) $dbintf->GetConfigValue('TenantQuickAssign');
		if ($UsrFilterLocal!=0
		|| $TenantQuickAssign!=0) ;
		else $SlowTenancy = true;
	}
	if ($SlowTenancy)
	{
		// determine agent when call has finished
		if ($is_active) return; // do at another stage
	}
	
	// determine agent
  $OnlyEnabled = false;
	$accids = $dbintf->GetUsersOfCall($call,$OnlyEnabled);
  
	// is the recording assigned to a tenant?
	// then the user should match
  foreach($accids as $accid)
  {
    $userinfo = $dbintf->GetUserInfoOnAccId($accid);
    // must be the same tenant
    if ($userinfo['tenantid'] == $call['tenantid'])
    {
      $ref_data['agent'] = $accid;
      return;
    }
  }

  $ref_data['agent'] = NULL;
  return;
}

function EditCall()
{
	global $dbintf;
  
  $call = false;
  $is_active = false;

  $lgtext = 'API.EditCall';
	if (isset($_POST['id']))
	{
		$id = $_POST['id'];
		$lgtext .= ' id: '.$id;
    
    $call = $dbintf->GetActiveCallInfoRM($id);
    if ($call !== false)
    {
      $is_active = true;
    } else
    {
      $call = $dbintf->GetCallInfoRM($id);
    }
	} else
  {
    // params not filled in are null
    // this construct is to avoid PHP warning
    $keys = [ 'local', 'remote', 'time', 'time-margin'];
    $search = [];
    foreach ($keys as $key)
    {
      if (isset($_POST['s-' . $key]))
      {
        $search[$key] = $_POST['s-' . $key];
      }
    }
    $lgtext .= " search";
    $call = $dbintf->SearchCall($search);
  }
  
  $data = array();
  $fields = array( 'notes','notes2','notes3','notes4','notes5' 
  ,'data1','data2','data3','data4','data5'
  ,'local','remote','connected'
  ,'retention'
  );
  $int_fields = array('retention'=>1);
  foreach ($fields as $field)
  {
    if (isset($_POST[$field]))
    {
      $data[ $field ] = $_POST[$field];
      if (isset($int_fields[$field]))
      {
        $data[ $field ] = (int) $data[ $field ];
        if ($field=='retention')
        {
          if ($data[$field]==0) $data[$field] = null;
          else if ($dbintf->IsRetentionPolicy($data[$field])==false)
          {
            $lgtext .= ' ret: 4 Unknown retention policy';
            lg($lgtext);
            return "4 Unknown retention policy";
          }
        }
      }
    }
  }

  $dbintf->SetAdmin();
  
  if ($call===false)
  {
    $lgtext .= ' ret: 1 Call not found';
    lg($lgtext);
    return "1 Call not found";
  }

	$FullSipId = (int) $dbintf->GetConfigValue('FullSipId');
  
  // Local and Remote field requires special care
  $telnum_changed = false;
  if (isset($data['local']))
  {
    $data['localfull'] = $data['local'];
    $data['local'] = IdSimplify($data['local']);
		$cid = GetSipIdToUse($data['localfull'],$FullSipId);
		$ret = $dbintf->GetOrCreateCallerID($cid,'',1);
		$data['localcallerid'] = $ret["id"];
		$data['localcontact'] = $ret["contactid"];
    $telnum_changed = true;
  }
  if (isset($data['remote']))
  {
    $data['remotefull'] = $data['remote'];
    $data['remote'] = IdSimplify($data['remote']);
		$cid = GetSipIdToUse($data['remotefull'],$FullSipId);
		$ret = $dbintf->GetOrCreateCallerID($cid,'',0);
		$data['remotecallerid'] = $ret["id"];
		$data['remotecontact'] = $ret["contactid"];
    $telnum_changed = true;
  }
	if (isset($data['connected']))
	{
    // connectedfull does not exist in database
    $connectedfull = $data['connected'];
    $data['connected'] = IdSimplify($data['connected']);
		$cid = GetSipIdToUse($connectedfull,$FullSipId);
		$ret = $dbintf->GetOrCreateCallerID($cid,'',0);
		$data['connectedcallerid'] = $ret["id"];
		$data['connectedcontact'] = $ret["contactid"];
    $telnum_changed = true;
	}
  if ($telnum_changed)
  {
    // copy all from data to call
    foreach ($data as $k=>$v)
    {
      $call[$k] = $v;
    }
    // redetermine agent
    RedetermineAgent($call, $data, $is_active);
  }  
  
  if ($is_active)
  {
    $dbintf->SetActiveCallInfoRM($call['remoteid'],$data);
  } else
  {
    if ($dbintf->SetCallInfo($call['id'],$data)==false)
    {
      // it failed
      $lgtext .= ' ret: 3 Set call data failed';
      lg($lgtext);
      return "3 Set call data failed";
    }
    // Copy retention policy number to CallsArchive,
    // so it can be applied during backup retention.
    $UseCallsArchive = (int) $dbintf->GetConfigValue('UseCallsArchive');
    if ($UseCallsArchive!=0 && isset($data['retention']))
    {
      // id is the same
      $dbintf->SetCallArchiveInfo($call['id'],array('retention'=>$data['retention']));
    }
    
    if (isset($_POST['transcription']))
    {
      if ($_POST['transcription']=='')
      {
        $dbintf->DeleteTranscript($call['id']);
      } else
      {
        if ($dbintf->SetTranscript($call['id'],$_POST['transcription'])==false)
        {
          $toret = '2 ' . $dbintf->lasterr;
          $lgtext .= ' ret: ' . $toret;
          return $toret;
        }
      }
    }
  }
  
  $lgtext .= ' ret: 0 OK';
  lg($lgtext);
  return "0 OK";
}

function DataSet()
{
	global $dbintf;
  $cache = (int_ensure_defined($_POST['cache'])==1);
	$inf = GetSpecifiedActiveCall('API.DataSet',$cache);
	if (isset($inf['ret']))
	{
		return $inf['ret'];
	}
  $lgtext = $inf['lg'];

  if (!isset($inf['call']))
  {
    if ($cache)
    {
      $tocopy = array( 'act'
      , 'localnum', 'localnumdifmax'
      , 'remotenum', 'remotenumdifmax'
      , 'linenr', 'id', 'nci' 
      , 'target', 'value'
      , 'email', 'usr'
      );
      $action = array();
      foreach ($tocopy as $key)
      {
        if (isset($_POST[$key]))
        {
          $action[ $key ] = $_POST[$key];
        }
      }
      $dbexpires = NULL;
      if (isset($_POST['expires']))
      {
        $expires = (int) $_POST['expires'];
        $lgtext .= ' expires: ' . $expires . ' s';
        $dbexpires = time() + (int) $_POST['expires'];
      }
      $dbintf->AddToActionCache(ArrayToString($action),$dbexpires);
      $lgtext .= ' ret: 0 OK (cached)';
      lg($lgtext);
      return '0 OK (cached)';
    } else
    {
      // should never happen
      return '103 Could not find call';
    }
  }

	$thecall = $inf['call'];

	$target = '';
	$ptarget = $_POST['target'];
	$FullSipId = (int) $dbintf->GetConfigValue('FullSipId');
	if ($ptarget=='local')
	{
		$data = array();
		$data['localfull'] = $_POST['value'];
    $data['local'] = IdSimplify($data['localfull']);
		$cidLocal = GetSipIdToUse($data['localfull'],$FullSipId);
		$ret = $dbintf->GetOrCreateCallerID($cidLocal,'',2);
		$data['localcallerid'] = $ret["id"];
		$data['localcontact'] = $ret["contactid"];
		$dbintf->SetActiveCallInfoRM($thecall['remoteid'],$data);
		$lgtext .= ' result: OK';
		lg($lgtext);
		return "0 OK";
	} else if (in_array($ptarget,array('notes','notes2','notes3','notes4','notes5'
    , 'data1', 'data2', 'data3', 'data4', 'data5' )))
  {
    $dbintf->SetActiveCallInfoRM( $thecall['remoteid'], array($ptarget => $_POST['value']) );
		$lgtext .= ' result: OK';
		lg($lgtext);
    return "0 OK";
  } else if ($ptarget=='tenant')
  {
    $map = GetTenantMap();
    $tenantid = false;
    $tenantName = $_POST['value'];
    foreach ($map as $id => $name)
    {
      if ($name == $tenantName) $tenantid = $id;
    }
    if ($tenantid === false)
    {
      $lgtext .= ' Tenant not found';
      lg($lgtext);
      return "104 Tenant not found";
    } else
    {
      $dbintf->SetActiveCallInfoRM( $thecall['remoteid'], array('tenantid' => $tenantid) );
      $lgtext .= ' result: OK';
      lg($lgtext);
      return "0 OK";
    }
  }
	$lgtext .= ' Error, no valid target';
	lg($lgtext);
	return "102 No valid target specified";
}

function set_http_header($code,$txt)
{
  header($_SERVER['SERVER_PROTOCOL'] . ' ' . $code . ' ' . $txt);
}

function AudioGet()
{
	global $dbintf;
	$lgtext = 'API.AudioGet';
  $lgtext .= ' usr:' . $dbintf->actuinfo['usr'];

  $id = $_POST['id'];
  $lgtext .= ' ' . $id;
  $info = $dbintf->GetCallInfoRM($id);
  if ($info===false)
  {
    set_http_header(404,'Not found');
    echo '1 Not found';
    $lgtext .= ' Error: Not found';
    lg($lgtext);
    exit;
  }
  
  $handler = new CallHandler();
  $handler->errorEcho = false;
  $handler->ids = array( $info['id'] );
  $handler->download = true;
  $handler->embed = false;
  $handler->Perform();
  if ($handler->good==false)
  {
    set_http_header($handler->errcode,$handler->errmsg);
    echo '2 ' . $handler->errmsg;
    $lgtext .= ' Error: ' . $handler->errmsg;
  } else
  {
    $lgtext .= ' OK';
  }
  lg($lgtext);
  exit;
}

function to_timestamp($datestr)
{
  return strtotime($datestr);
}

function CallOutputJson($item, &$listconfig)
{
  global $configs, $dbintf;
  $isadmin = $listconfig['userdata']['isadmin'];
  $listconfig['userdata']['cnt'] ++;
  
  $call = array();
  $call['id']        = $item['remoteid'];
  $call['start']     = date('Y-m-d H:i:s',$item['startdate']);
  $call['duration']  = $item['duration'];
  $call['local']     = $item['localfull'];
  $call['remote']    = $item['remotefull'];
  $call['direction'] = $item['direction'];
  
  $conditionalCols = array( 'lname', 'rname' );
  foreach ($conditionalCols as $col)
  {
    if (int_ensure_defined( $configs['col_' . $col] )!=0 || $isadmin)
    {
      $call[$col] = $item[$col];
    }
  }
  
  if ($isadmin && $listconfig['userdata']['groupinfo']==true)
  {
    if ($item['tenantid']!=null)
    {
      $call['tenant'] = $listconfig['userdata']['tenantmap'][ $item['tenantid'] ];
    }
    // group list
    // determine which users match the call
    // second param is whether to include not-enabled users
    $users = $dbintf->GetUsersOfCall($item,false);
    // determine the groups of which these users are member
    $groupMatch = array();
    foreach ($users as $userAccId)
    {
      $groups = $dbintf->GetGroupsOfWhichMember( $userAccId );
      foreach ($groups as $groupAccId)
      {
        $groupMatch[ $groupAccId ] = true;
      }
    }
    // add the name of the groups to the list
    $groupList = array();
    foreach ($groupMatch as $groupAccId=>$dummy)
    {
      $group = $dbintf->GetGroupInfoOnAccid( $groupAccId );
      $groupList[] = $group['name'];
    }
    // add comma separated
    $call['groups'] = implode(',',$groupList);
  }
  
  // Be careful with outputing too much data,
  // fields that are hidden could be unwanted information leackage.
  // For that reason we check if the column is visually enabled,
  // or the user is an admin.
  
  if (int_ensure_defined( $configs['col_category'] )!=0 || $isadmin)
  {
    $call['category']  = $item['category'];
    $catname = '';
    if ($call['category'] >= 1)
    {
      $catname = $dbintf->GetCachedConfigValue('CatLabel' . $call['category']);
    }
    $call['catname'] = $catname;
  }
  for ($i=1;$i<=5;$i++)
  {
    $field = 'notes';
    if ($i!=1) $field .= $i;
    if (int_ensure_defined( $configs['col_'.$field] )!=0 || $isadmin)
    {
      $call[$field]  = $item[$field];
    }
  }
  for ($i=1;$i<=5;$i++)
  {
    $field = 'data';
    $field .= $i;
    if (int_ensure_defined( $configs['col_'.$field] )!=0 || $isadmin)
    {
      $call[$field]  = $item[$field];
    }
  }
  // do we have global or any specific view transcript permission?
  // this is only for speed because GetTranscript check permission
  if ( $dbintf->HasPmForAny(PM_VIEW_TRANSCRIPT) )
  {
    $transcript = $dbintf->GetTranscript($item['id']);
    if ($transcript !== false)
    {
      $call['transcript'] = $transcript;
    }
  }
  if ($listconfig['userdata']['cnt'] != 1) echo ",";
  echo json_encode($call) . "\n";
}

function CallListGet()
{
	global $dbintf, $configs;
	$lgtext = 'API.CallListGet';
  $lgtext .= ' usr:' . $dbintf->actuinfo['usr'];

  $isadmin = ($dbintf->actuinfo['permissions']&PM_CONFIG) !=0 ;
  
  $count = 10; // default
  if (isset($_POST['count']))
  {
    $count = $_POST['count'];
  }
  // no max anymore: $maxcount = 1000; // max.
  // if ($count>$maxcount) $count = $maxcount;
  // because we output data in a callback
  
  $sk = '';
  $sd = '';
  $search = array();
  $listconfig = array();
  
  // apply search parameters
  if (isset($_POST['fd']) && $_POST['fd']!='')
  {
    $search['fd'] = to_timestamp( $_POST['fd'] );
  }
  if (isset($_POST['ld']) && $_POST['ld']!='')
  {
    $search['ld'] = to_timestamp( $_POST['ld'] );
  }
  if (isset($_POST['id']) && $_POST['id']!='')
  {
    $search['rem'] = $_POST['id'];
  }
  
  header('Content-Type: application/json');
  
  echo "[";
  //
  $listconfig['startfrom'] = 0;
  $listconfig['checkrange'] = $count;
  $listconfig['loadrange'] = (int) $listconfig['checkrange'];
  $listconfig['callback'] = 'CallOutputJson';
  $listconfig['userdata'] = array();
  $listconfig['userdata']['isadmin'] = $isadmin;
  $listconfig['userdata']['cnt'] = 0;
  $listconfig['userdata']['groupinfo'] = (int_ensure_defined($_POST['groupinfo'])!=0);
  if ($isadmin) $listconfig['userdata']['tenantmap'] = GetTenantMap();
  $list = $dbintf->GetCallList($sk,$sd,$search,$listconfig);
  $cnt = $list['cnt'];
  unset($list['cnt']);
  
  echo "]";
  
  LogAction('Get call listing using API'
    ,array('level'=>'view'
      ,'calllist_search'=>$search
      ,'calllist_config'=>$listconfig
      )
    );

  lg($lgtext);
  
  return "";
}

function GetTenantMap()
{
  global $dbintf;
  $multiTenancy = ($dbintf->GetConfigValue('MultiTenancy')!=0);
  if ($multiTenancy==false) return array();
  $tenantNameMap = array();
  $tenants = $dbintf->GetTenantList();
  foreach ($tenants as $tenant)
  {
    $tenantNameMap[ $tenant['id'] ] = $tenant['name'];
  }
  return $tenantNameMap;
}

function Reconfig()
{
  // true: always do a reconfig of the engine
  // even if not detected that necessary
  EnginesSettingsCalcApply(true);
  return "0 OK";
}

function UserConfGet()
{
	global $dbintf;
	$lgtext = 'API.UserConfGet';
	if (isset($_POST['usr'])==false || $_POST['usr']=='')
	{
		return "101 Missing or empty parameter usr";
	}
	$usr = $_POST['usr'];
	$lgtext .= ' usr:' . $usr;
	$uinfo = $dbintf->GetUserInfoOnUsername($usr);
	if ($uinfo===false)
	{
		sleep(2);
		lg($lgtext . ' NotFound');
		return "1 User not found";
	}
	// store on demand
	$GlobalStoreOnDmd = (int) $dbintf->GetConfigValue("RecOnDemand");
	$UserStoreOnDmd = $uinfo['rod'];
	$StoreOnDmd = -1;
	if ($UserStoreOnDmd==0)
	{
		$StoreOnDmd = ($GlobalStoreOnDmd==0)?0:1;
	} else if ($UserStoreOnDmd==1)
	{
		$StoreOnDmd = 1;
	} else if ($UserStoreOnDmd==2)
	{
		$StoreOnDmd = 0;
	}
	// recording on demand
	$GlobalRecOnDmdType = (int) $dbintf->GetConfigValue("RecOnDmdType");
	$UserRecOnDmd = $uinfo['recondmd'];
	$RecordingOnDemand = -1;
	if ($GlobalRecOnDmdType==0)
	{
		$RecordingOnDemand = 0;
	} else if ($GlobalRecOnDmdType==1)
	{
		$RecordingOnDemand = 1;
	} else if ($GlobalRecOnDmdType==3)
	{
		$RecordingOnDemand = $UserRecOnDmd;
	}
	// silence on demand
	$GlobalSilOnDmd = (int) $dbintf->GetConfigValue("SilOnDemand");
	$UserSilOnDmd = $uinfo['silondmd'];
	$SilenceOnDemand = -1;
	if ($GlobalSilOnDmd==0)
	{
		$SilenceOnDemand = 0;
	} else if ($GlobalSilOnDmd==1)
	{
		$SilenceOnDemand = 1;
	} else if ($GlobalSilOnDmd==3)
	{
		$SilenceOnDemand = $UserSilOnDmd;
	}
	// delete on demand
	$GlobalDelOnDmdType = (int) $dbintf->GetConfigValue("DelOnDmdType");
	$UserDelOnDmd = $uinfo['delondmd'];
	$DeleteOnDemand = -1;
	if ($GlobalDelOnDmdType==0)
	{
		$DeleteOnDemand = 0;
	} else if ($GlobalDelOnDmdType==1)
	{
		$DeleteOnDemand = 1;
	} else if ($GlobalDelOnDmdType==3)
	{
		$DeleteOnDemand = $UserDelOnDmd;
	}
	
	$t = "0 OK\r\n";
	$t .= "StoreOnDemand: " . $StoreOnDmd . "\r\n";
	$t .= "RecordingOnDemand: " . $RecordingOnDemand . "\r\n";
	$t .= "SilenceOnDemand: " . $SilenceOnDemand . "\r\n";
	$t .= "DeleteOnDemand: " . $DeleteOnDemand . "\r\n";
	$t .= "User: " . $usr . "\r\n";
	return $t;
}

function SendSNMPTrap()
{
  $msg = [
    "act:sendtrap"
  ];

  $keys = [
    "quantity",
    "value",
    "health",
    "message"
  ];

  foreach($keys as $key)
  {
    if(isset($_POST[$key]) && strlen($_POST[$key]) > 0)
    {
      $msg[] = $key . ":" . $_POST[$key];
    }
  }

  return SendSNMPMsg($msg);
}

function SendSNMPOkay()
{
  $msg = [
    "act:sendokay"
  ];

  return SendSNMPMsg($msg);

}

function SendSNMPMsg($msg)
{
  $socket = @socket_create(AF_UNIX, SOCK_DGRAM, 0);
  if ($socket===FALSE)
  {
    return "500 Could not contact snmp agent: socket create failed\r\n";
  } 

  if (@socket_connect($socket, "/tmp/apresa_snmp") === FALSE)
  {
    return "500 Could not contact snmp agent: socket connect failed\r\n";
  }

  $messagestr = implode("\x1e", $msg) . "\x1e";

  if(@socket_write($socket,$messagestr,strlen($messagestr)) === FALSE)
  {
    return "500 Could not contact snmp agent: socket semd failed\r\n";
    return;
  }

  return "0 OK\r\n";
}

?>