<?php

require_once 'signaling.php';
require_once 'datafile.php';
require_once 'datetime.php';
require_once 'emailing.php';
require_once 'language.php';
require_once 'utility.php';

function BasicConstants()
{
  define('APRESA_VERSION','14.3.2.1');
  define('MSG_INF',0);
  define('MSG_ERR',1);
  define('MSG_WRN',2);
}

function InitDB()
{
  global $dbintf;
  // Interface to database
  $dbintf = new DBInterface(DATA_PATH);
}

function LoadBasicSettings()
{
  global $configs, $lng, $dbintf;

  // load configuration
  $configs = $dbintf->LoadConfigsPart('web');
  $configs['has_ED137'] = (int) $dbintf->GetConfigValue('ED137ChCnt') > 0;

  /*
  // Make defaults
  if (isset($configs['col_direction'])==false)
  {
    // missing, so make defaults
    $configs['lng'] = 'UK';
    $configs['dateformat'] = 0; // USA
    $configs['col_playlink'] = 1;
    $configs['col_startdate'] = 1;
    $configs['col_length'] = 1;
    $configs['col_direction'] = 1;
    $configs['col_notes'] = 1;
    $configs['col_rname'] = 1;
    $configs['col_remote'] = 1;
    $configs['col_remote2'] = 0;
    $configs['col_lname'] = 1;
    $configs['col_local'] = 1;
    $configs['col_local2'] = 0;
  }
  */

  $browserlng = '';
  if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE']))
  {
    $lng = explode(';',$_SERVER['HTTP_ACCEPT_LANGUAGE']);
    if (isset($lng[0]))
    {
      $lng = explode(',',$lng[0]);
      if (isset($lng[0]))
      {
        $browserlng = $lng[0].substr(1,2);
      }
    }
  }

  $lng = new TLanguage();

  $lngchoice = $configs['lng'];
  if ($lngchoice=='auto' || $lngchoice=='')
  {
    // try auto-detect
		$browserlng_map = array
		(	'da' => 'DA'
		,	'de' => 'DE'
		, 'es' => 'ES'
		, 'fi' => 'FI'
		, 'fr' => 'FR'
		, 'it' => 'IT'
		, 'nl' => 'NL'
		, 'no' => 'NO'
		, 'pl' => 'PL'
		, 'sv' => 'SV'
		);
		if (isset($browserlng_map[$browserlng]))
		{
			$lngchoice = $browserlng_map[$browserlng];
		} else
		{
			$lngchoice = 'UK';
		}
  }

  SetDateTimeF($configs['dateformat'],0);

  // Loads from Control Panel on Windows
  // Use server default
  // setlocale(LC_ALL,'en_EN');
  //echo date_default_timezone_get();

  $lng->Load($lngchoice);

}

class CsvParser
{
  var $part = '';
  var $inside = false;
  var $beenInside = false;
  var $data = array();
  var $readyData = null;
  // detect unquote NULL,
  // and convert to PHP NULL instead of string NULL
  var $detectPureNull = false;
  
  function __construct()
  {
    //
  }
  
  function getData()
  {
    $toret = $this->readyData;
    $this->readyData = null;
    return $toret;
  }
  
  function flush()
  {
		if ($this->detectPureNull && $this->beenInside==false && $this->part==='NULL')
		{
		  $this->part = NULL;
		}
    $this->data[] = $this->part;
    $this->part = '';
    $this->beenInside = false;
    $this->readyData = $this->data;
    $this->data = array();
  }
  
  function addLine($t)
  {
    //echo '<<' . $t . '>>';
    $this->beenInside = false;
    $len = strlen($t);
    for($i=0;$i<$len;$i++)
    {
      $ch = $t[$i];
      if ($this->inside)
      {
        if ($ch=='"')
        {
          if ($i+1<$len && $t[$i+1]=='"')
          {
            $this->part .= '"';
            $i ++;
            continue;
          }
          $this->inside = false;
          continue;
        }
      } else
      {
        if ($ch==',')
        {
          if ($this->detectPureNull && $this->beenInside==false && $this->part==='NULL')
          {
            $this->part = NULL;
          }
          $this->data[] = $this->part;
          $this->part = '';
          $this->beenInside = false;
          continue;
        }
        if ($ch=='"')
        {
          if ($this->part=='')
          {
            $this->inside = true;
            $this->beenInside = true;
            continue;
          }
        }
        $newline = false;
        if ($ch=="\n")
        {
          $newline = true;
        }
        if ($ch=="\r")
        {
          if ($i+1<$len && $t[$i+1]=="\n")
          {
            // Windows CR-LF
            $newline = true;
            $i ++;
          }
        }
        if ($newline)
        {
          if ($this->detectPureNull && $this->beenInside==false && $this->part==='NULL')
          {
            $this->part = NULL;
          }
          $this->data[] = $this->part;
          $this->part = '';
          $this->beenInside = false;
          $this->readyData = $this->data;
          $this->data = array();
        }
      }
      $this->part .= $ch;
      if ($this->inside==false)
      {
        $this->part = ltrim($this->part);
      }
    }
  }
}

function ParseCsvLine($t)
{
  $arr = array();
  $len = strlen($t);
  $part = '';
  $inside = false;
  for($i=0;$i<$len;$i++)
  {
    $ch = $t[$i];
    if ($inside)
    {
      if ($ch=='"')
      {
        if ($i+1<$len && $t[$i+1]=='"')
        {
          $part .= '"';
          $i ++;
          continue;
        }
        $inside = false;
        continue;
      }
    } else
    {
      if ($ch==',')
      {
        $arr[] = $part;
        $part = '';
        continue;
      }
      if ($ch=='"')
      {
        if ($part=='')
        {
          $inside = true;
          continue;
        }
      }
    }
    $part .= $ch;
    if ($inside==false)
    {
      $part = ltrim($part);
    }
  }
  $arr[] = $part;
  return $arr;
}

function CsvQuote($text)
{
	return '"' . str_replace('"','""',$text) . '"';
}

function ApplyReplacements($rpl , $text )
{
  foreach ($rpl as $from => $to)
  {
    $text = str_replace( $from, $to , $text );
  }
  return $text;
}

function SetReplacementTags(&$ref_rpl, $info)
{
  global $dbintf, $configs;
  
  $dir = $info['direction'];
  
  $L = $dbintf->GetContactName($info['localcontact']);
  $R = $dbintf->GetContactName($info['remotecontact']);
  $C = $dbintf->GetContactName($info['connectedcontact']);
  
	if ($dir==0)
	{
		// for incoming calls,
		// the local side is the receiver.
		$callerNum    = $info['remote'];
		$receiverNum  = $info['local'];
		$callerName   = $R;
		$receiverName = $L;
	} else
	{
		// for outgoing/internal/external calls,
		// the local side is the caller
		$callerNum    = $info['local'];
		$receiverNum  = $info['remote'];
		$callerName   = $L;
		$receiverName = $R;
	}

  $ref_rpl['<ID>']        = $info['remoteid'];

  $ref_rpl['<CALLER>']    = $callerNum;
  $ref_rpl['<RECEIVER>']  = $receiverNum;
  $ref_rpl['<CALLERNAME>']    = $callerName;
  $ref_rpl['<RECEIVERNAME>']  = $receiverName;
  
  $ref_rpl['<LOCAL>']     = $info['local'];
  $ref_rpl['<REMOTE>']    = $info['remote'];
  $ref_rpl['<LOCALNAME>']     = $L;
  $ref_rpl['<REMOTENAME>']    = $R;
  
  $ref_rpl['<LINENR>'] = $info['linenr'];

  $ref_rpl['<DIRECTION>'] = formatDirection($info['direction']);
    
  $ref_rpl['<DATETIME>']  = date($configs['datetimef'],$info['startdate']);
  $ref_rpl['<DURATION>'] = formatDuration($info['duration']);
  
  for ($i=1;$i<5;$i++)
  {
    $postfix = $i;
    $ref_rpl['<DATA'.$postfix.'>'] = $info['data'.$postfix];
    if ($i==1) $postfix = '';
    $ref_rpl['<NOTES'.$postfix.'>'] = $info['notes'.$postfix];
  }
  $ref_rpl['<CATEGORY>'] = formatCategory($info['category']);
  
}


function EmailRecordings($recording_ids,$addressee)
{
  global $msg, $lng, $dbintf,$configs;
  lg('EmailRecordings');

  if ($addressee=='')
  {
    $msg[] = new Msg(MSG_ERR,'No addressee specified for the e-mail');
		return false;
	}

  $dat = array();
  $configs_n = $dbintf->LoadConfigsPart('n');
  $EmailAttach = $dbintf->GetConfigValue('EmailAttach');
  if ($EmailAttach=='' || $EmailAttach===false) $EmailAttach = 'attach';
  $RecAttach = ($EmailAttach=='attach');

  $connectType = ensure_defined($configs_n['EmailConnectType'], 'relay');

	if ( ($connectType=='relay')
    && $configs_n['SMTPhost']=='')
	{
  	lg('E-mail is not configured (relay)');
    $msg[] = new Msg(MSG_ERR,'E-mail SMTP server address is not configured');
		return false;
	}
	if ($connectType=='none')
	{
  	lg('E-mail is not configured (none)');
    $msg[] = new Msg(MSG_ERR,'E-mail is not configured');
		return false;
	}

  $dat['To'] = $addressee;
  
  // use template if available
  $key = 'EmailTemplate' . 'EmailRec';
  $subjectTemplate = $dbintf->GetConfigValue($key . 'Subject');
  $bodyTemplate    = $dbintf->GetConfigValue($key . 'Body');
  
  if ($RecAttach) $neededPm = PM_DOWNLOAD;
  else $neededPm = PM_LIST;
  
  $rpl = array();
  if ($subjectTemplate!='')
  {
    foreach($recording_ids as $recording_id)
    {
      $info = $dbintf->GetCallInfo($recording_id);
      $pm = (int) $info['permissions'];
      if ($pm & $neededPm)
      {
        SetReplacementTags($rpl,$info);
        break;
      }
    }
  }

  if ($subjectTemplate=='') $subject = $lng->Get('Email.CallSubject');
  else
  {
    $subject = $subjectTemplate;
    $subject = ApplyReplacements( $rpl, $subject );
  }
  $dat['Subject'] = $subject;
  $body = '';
  if ($bodyTemplate=='')
  {
    // Set information about the call in the email body
    $body = "";
    $body .= "\n";
    if ($RecAttach)
    {
    $body .= $lng->Get('Email.CallAttached') . "\n";
  }
  }
  $att_cnt = 0;
	$toDelete = array();
	
  foreach($recording_ids as $recording_id)
  {
    LogAction('Email call', array('call_id'=>$recording_id) );
    $info = $dbintf->GetCallInfo($recording_id);
    $pm = (int) $info['permissions'];
    $body .= "\n";
		if ($info['isarchived']==0)
		{
			$body .= 'Recording is not available';
			continue;
		}
    if ($pm & $neededPm)
    {
      if ($bodyTemplate=='')
      {
        $body .= $lng->Get("Cols.startdate") . ": ".date($configs['datetimef'],$info['startdate'])."\n";
        $body .= $lng->Get("Cols.length") . ": ".formatDuration($info['duration'])."\n";
        $body .= $lng->Get("Cols.category") . ": ".formatCategory($info['category'])."\n";
        $body .= $lng->Get("Cols.notes") . ": ".$info['notes'] . "\n";
        $body .= $lng->Get("Cols.direction") . ": ".formatDirection($info['direction']) . "\n";
        $body .= $lng->Get("Cols.rname") . ": " . $dbintf->GetContactName($info['remotecontact']) . "\n";
        $body .= $lng->Get("Cols.remoteID") . ": ".$info['remote'] . "\n";
        $body .= $lng->Get("Cols.lname") . ": " . $dbintf->GetContactName($info['localcontact']) . "\n";
        $body .= $lng->Get("Cols.localID") . ": " .$info['local'] . "\n";
        if ($info['linenr']!=0)
        {
          $body .= $lng->Get("Cols.linenr") . ": " .$info['linenr'] . "\n";
        }
      } else
      {
        $rpl = array();
        SetReplacementTags($rpl,$info);
        $body .= ApplyReplacements( $rpl, $bodyTemplate );
      }
      if ($RecAttach)
      {
	    $attachfn = AUDIO_PATH.'/'.$info['directory'].'/'.$info['filename'];
			
			// Loading from backup
			$autoDeleted = ($info['flags']&4)!=0;
			if ($autoDeleted)
			{
				// check setting
				if((int)$dbintf->GetConfigValue('LoadDelBackup') == 0 )
				{
					echo 'The recording has expired and was auto-deleted.';
				}
				else
				{
					// Can we load it from the backup?
          $errmsg = '';
					$newfn = LoadFromBackup($info,$info['filename'],$errmsg);		//LoadFromBackup, moved to utility.php!
					if ($newfn!='')
					{
						//We have restored the recording from backup. Rename and mail!
						$path = pathinfo($newfn);
						$attachfn = $path['dirname']."/".$info['filename'];
						rename($newfn, $attachfn);
						//We need to delete this file afterwards
						$toDelete[] = $attachfn;
					}
				}
			}
			
			if (file_exists($attachfn)==false)
			{
				$body .= 'File does not exist';
				continue;
			}
			
	    $att_cnt ++;
	    $dat['Attach_'.$att_cnt] = $attachfn;
      } else
      {
        // maybe generate play link here
        if ($EmailAttach=='link')
        {
          $playUrl = $dbintf->GetConfigValue('AccessUrl') . "/play.php?id=" . $info['id'];
          //$bodyHtml .= "\n<a href=\"" . $playUrl . "\">" . $lng->Get('Web.Play') . "</a>";
          $body .= "\n" . $lng->Get('Web.Play') . ": " . $playUrl . "\n";
	  }
      }
	  }
		else
	  {
			$body .= 'Permission denied';
		}
  }
  $dat['Body'] = $body;

	if (defined('CMD'))
	{
		//echo "Directly emailing\n";
	  return SendEmailData($configs_n,$dat);
	} else
	{
    return EmailingFromWebIntf($dat);
	}
	
	foreach ($toDelete as $delFn)
	{
		if (file_exists($delFn))
		{
			DeleteFn($delFn);
		}
	}
}

function EmailingFromWebIntf($dat)
{
    global $lng , $msg;
    
	  $fn = WriteRandomDataFile($dat);
	  lg('Written data file to '.$fn);
	  if ($fn===false)
	  {
	    $msg[] = new Msg(MSG_ERR,'Failed to write data file.');
		  return false;
	  } else
	  {
	    lg('Signaling...');
	    if (msg_signal_service($msg,'email '.$fn))
	    {
	      if (defined('EMAILQ'))
	      {
					lg('Email added to queue');
		      $msg[] = new Msg(MSG_INF,$lng->Get('Email.Queued'));
		      return true;
			  } else {
					lg('Email sent');
		      $msg[] = new Msg(MSG_INF,$lng->Get('Email.Sent'));
		      return true;
				}
	    } else
	    {
	      lg('Failed to send email.');
	      $msg[] = new Msg(MSG_ERR,'Failed to send e-mail');
		    return false;
	    }
	  }
		return false;
}

function formatDirection($dir)
{
  global $lng;
  if ($dir==0) return $lng->Get('Web.Incoming');
  else return $lng->Get('Web.Outgoing');
}

function formatCategory($cat)
{
	global $configs,$lng;
	
	if($cat==0) return $lng->Get('Main.None');
	$key = 'CatLabel' . $cat;
	if (isset($configs[$key]) && $configs[$key]!='') return $configs[$key];
	return $cat;
}

class Msg
{
  var $type;
  var $text;
  function __construct($type,$text)
  {
    $this->type = $type;
    $this->text = $text;
  }
};

function GetNotesColumnHeader($ix)
{
  global $configs, $lng;
  if ($ix==1) return $lng->Get('Cols.notes');
  $key = 'NotesLabel' . $ix;
  if (isset($configs[$key]) && $configs[$key]!='') return $configs[$key];
  return $lng->Get('Cols.notes') . ' ('.$ix.')';
}

function GetDataColumnHeader($ix)
{
  global $configs, $lng;
  $key = 'DataLabel' . $ix;
  if (isset($configs[$key]) && $configs[$key]!='') return $configs[$key];
  return $lng->Get('Web.Data') . ' ('.$ix.')';
}

function ExportGetCols(&$options)
{
  global $configs, $dbintf;
  $cols = array();
  $cols[] = 'id';
  $cols[] = 'startdate';

  if (isset($options['ceproject'])) $cols[] = 'ceproject';
	if (isset($options['ceproject'])) $cols[] = 'ceuser';
	if (isset($options['score']))     $cols[] = 'cescore';
  if (int_ensure_defined($configs['col_length']))    $cols[] = 'length';
  if (int_ensure_defined($configs['col_waittime']))  $cols[] = 'waittime';
  if (int_ensure_defined($configs['col_direction'])) $cols[] = 'dir1';
  if (int_ensure_defined($configs['col_direction'])) $cols[] = 'dir2';
  if (int_ensure_defined($configs['col_category']))  $cols[] = 'category';
  for ($i=1;$i<=5;$i++)
  {
    $postfix = ($i==1)?'':$i;
    if (int_ensure_defined($configs['col_notes' . $postfix])) $cols[] = 'notes' . $postfix;
  }
  for($i=1;$i<=5;$i++)
  {
	  if (int_ensure_defined($configs['col_data' . $i])) $cols[] = 'data' . $i;
  }
  if (int_ensure_defined($configs['col_remote']))    $cols[] = 'remoteID';
  if (int_ensure_defined($configs['col_rname']))     $cols[] = 'rname';
  if (int_ensure_defined($configs['col_connected'])) $cols[] = 'connected';
  if (int_ensure_defined($configs['col_cname']))     $cols[] = 'cname';
  // direction 2 (do not export)
  if (int_ensure_defined($configs['col_local']))     $cols[] = 'localID';
  if (int_ensure_defined($configs['col_lname']))     $cols[] = 'lname';
  if (int_ensure_defined($configs['col_linenr']))    $cols[] = 'linenr';
  if (int_ensure_defined($configs['col_remote2']))   $cols[] = 'remoteIP';
  if (int_ensure_defined($configs['col_local2']))    $cols[] = 'localIP';
  if (int_ensure_defined($configs['col_checksum']))  $cols[] = 'checksum';
  if (int_ensure_defined($configs['col_rem']))       $cols[] = 'rem';
  if (int_ensure_defined($configs['col_recorder']))  $cols[] = 'recorder';
	if ($dbintf->AmIAdminT123())
	{
		if (int_ensure_defined($configs['col_tenant']))  $cols[] = 'tenant';
	}
	if (int_ensure_defined($configs['col_agent']))     $cols[] = 'agent';

  return $cols;
}

function ExportCreateColsHeader(&$options)
{
  global $lng;

  $cols = $options['cols'];
  $file = $options['file'];
  $row = $file->CreateRow();

  foreach ($cols as $col)
  {
    $item = '';
    $lt = '';
    if ($col=='id') $item = 'ID';
    else if ($col=='startdate') $lt = 'Cols.startdate';
    else if ($col=='ceproject') $lt = 'CE.Project';
    else if ($col=='ceuser'   ) $lt = 'PM.User';
    else if ($col=='cescore'  ) $lt = 'CE.Score';
    else if ($col=='length'   ) $item = $lng->Get('Cols.length')   . ' [' . $lng->Get('DateTime.s') . ']';
    else if ($col=='waittime' ) $item = $lng->Get('Cols.waittime') . ' [' . $lng->Get('DateTime.s') . ']';
    else if ($col=='dir1'     ) $lt = 'Cols.direction';
    else if ($col=='dir2'     ) $lt = 'Cols.direction';
    else if ($col=='category' ) $lt = 'Cols.category';
    else if ($col=='rname'    ) $lt = 'Cols.rname';
    else if ($col=='remoteID' ) $lt = 'Cols.remoteID';
    else if ($col=='remoteIP' ) $lt = 'Cols.remoteIP';
    else if ($col=='cname'    ) $lt = 'Cols.cname';
    else if ($col=='connected') $lt = 'Cols.connected';
    else if ($col=='lname'    ) $lt = 'Cols.lname';
    else if ($col=='localID'  ) $lt = 'Cols.localID';
    else if ($col=='localIP'  ) $lt = 'Cols.localIP';
    else if ($col=='linenr'   ) $lt = 'Cols.linenr';
    else if ($col=='checksum' ) $lt = 'Cols.checksum';
    else if ($col=='rem'      ) $lt = 'Web.Identifier';
    else if ($col=='recorder' ) $lt = 'Cols.recname';
		else if ($col=='tenant'   ) $lt = 'Web.Tenant';
		else if ($col=='agent'    ) $lt = 'PM.User';
    for ($i=1;$i<=5;$i++)
    {
      $postfix = ($i==1)?'':$i;
      if ($col=='notes'.$postfix) $lt = GetNotesColumnHeader($i);
	    else if ($col=='data'.$i) $lt = GetDataColumnHeader($i);
    }
    
    if ($lt!='') $item = $lng->Get($lt);

    $row->add($item);
  }

  return $row->output();
}

function Export_RowHandler($row, &$options)
{
  global $configs,$lng,$dbintf;
  $cols = $options['cols'];

  $file = $options['file'];
  $filerow = $file->CreateRow();

  $callextra = StringToArray($row['callextra']);
  $chk = '';
  if (isset($callextra['chk']))
  {
    $chk = $callextra['chk'];
  }
  $strDirection = '';
  if ($row['direction']==0) $strDirection = $lng->Get('Web.Incoming');
  if ($row['direction']==1) $strDirection = $lng->Get('Web.Outgoing');
  if ($row['direction']==2) $strDirection = $lng->Get('Web.Internal');
  //if ($row['direction']==3) $strDirection = $lng->Get('Web.External'); // no transla

  $strCategory = '';
  if ($row['category']>0)
  {
    $strCategory = $configs['CatLabel' . $row['category']];
  }

  $items = array();
  foreach ($cols as $col)
  {
    $item = '';
    if ($col=='id') $item =  $row['id'];
    if ($col=='startdate') $item = date($configs['datetimef'],$row['startdate']);
    if ($col=='ceproject') $item = $options['projects'][ $row['projectid'] ]['name'];
    if ($col=='ceuser'   ) $item = $row['username'];
    if ($col=='cescore'  )
    {
      if (isset($row['score']))
      {
        $item = round($row['score']);
      }
    }
		if ($col=='length')   $item = $row['duration'];
    if ($col=='waittime') $item = ($row['waittime'] == -1) ? '' : $row['waittime'];
		if ($col=='dir1')     $item = $row['direction'];
		if ($col=='dir2')     $item = $strDirection;
    if ($col=='category') $item = $strCategory;
		if ($col=='rname')    $item = $row['rname'];
		if ($col=='remoteID') $item = $row['remote'];
		if ($col=='remoteIP') $item = $row['remote2'];
    if ($col=='cname'    )$item = $row['cname'];
    if ($col=='connected')$item = $row['connected'];
		if ($col=='lname')    $item = $row['lname'];
		if ($col=='localID')  $item = $row['local'];
		if ($col=='localIP')  $item = $row['local2'];
		if ($col=='linenr')   $item = ($row['linenr']==0) ? '' : $row['linenr'];
		if ($col=='checksum') $item = $chk;
		if ($col=='notes')    $item = $row['notes'];
		if ($col=='notes2')   $item = $row['notes2'];
		if ($col=='notes3')   $item = $row['notes3'];
		if ($col=='notes4')   $item = $row['notes4'];
		if ($col=='notes5')   $item = $row['notes5'];
		if ($col=='data1')    $item = $row['data1'];
		if ($col=='data2')    $item = $row['data2'];
		if ($col=='data3')    $item = $row['data3'];
		if ($col=='data4')    $item = $row['data4'];
		if ($col=='data5')    $item = $row['data5'];
    if ($col=='rem')      $item = $row['remoteid'];
    if ($col=='recorder') $item = $row['recordername'];
		if ($col=='tenant')
		{
			// load tenant name from cache
			if (isset($options['tenantNameCache']))
			{
				if (isset($options['tenantNameCache'][$row['tenantid']]))
				{
					$item = $options['tenantNameCache'][$row['tenantid']];
				}
			} else
			{
				$options['tenantNameCache'] = [];
			}
			if ($item=='') // not found in cache
			{
				$group = $dbintf->GetGroupInfo($row['tenantid']);
				$item = $group ? $group['name'] : null;
				// store in cache
				$options['tenantNameCache'][$row['tenantid']] = $item;
			}
		}
		if ($col=='agent')
		{
			// load agent name from cache
			if (isset($options['agentNameCache']))
			{
				if (isset($options['agentNameCache'][$row['agent']]))
				{
					$item = $options['agentNameCache'][$row['agent']];
				}
			} else
			{
				$options['agentNameCache'] = [];
			}
			if ($item=='') // not found in cache
			{
				$user = $dbintf->GetUserInfoOnAccId($row['agent']);
				$item = $user ? $user['name'] : null;
				// store in cache
				$options['agentNameCache'][$row['agent']] = $item;
			}
		}

    $filerow->add($item);
  }
  $filerow->output();
}

function Export_Prepare($format, &$options)
{
	global $configs,$lng;
  $filename = 'calls_' . date('Ymd_His');
  
  if($format == 'xlsx')
  {
    $options['file'] = new XLSXFile();
  }
  else
  {
    $options['file'] = new CsvFile();
  }

  $options['file']->Init($filename);
  $options['cols'] = ExportGetCols($options);
  
  ExportCreateColsHeader($options);
}

function ExportCSV($call_list,$options=array())
{
	Export_Prepare('csv', $options);
  foreach ($call_list as $ix=>$row)
  {
  	Export_RowHandler($row,$options);
	}
	exit;
}

function Export_Done(&$options)
{
  $options['file']->Finish();
}

// not used anymore
function GlobGetMachineId()
{
	return trim(file_get_contents( dirname(__FILE__) . '/../apresa/conf/machineid'));
}

function GlobGetSerialNumber()
{
	return trim(file_get_contents( dirname(__FILE__) . '/../apresa/conf/serial'));
}

function UpdateSystemInfoCfg()
{
	global $dbintf;
	if ($dbintf->readonly) return;

	$newconfigs = array();

	$newconfigs['ApresaVersion'] = APRESA_VERSION;
	$newconfigs['ApresaSerial'] = GlobGetSerialNumber();

	$dbintf->SetConfigsPart($newconfigs,'info');

}

function MakeValidFnWithoutExt($t)
{
	$t = preg_replace('/[^a-zA-Z0-9\-_]+/','-',$t);
	return $t;
}

function GetAuditActionDescr($act)
{
  $descr = '';
  if ($act=='start'  ) $descr = 'Start recording on demand';
  if ($act=='stop'   ) $descr = 'Stop recording on demand' ;
  if ($act=='silence') $descr = 'Start silence on demand'  ;
  if ($act=='sound'  ) $descr = 'Stop silence on demand'   ;
  if ($act=='delete' ) $descr = 'Delete on demand';
  if ($act=='email'  ) $descr = 'Email recording';
  if ($act=='email2' ) $descr = 'Email previous recording';
  if ($act=='annotation') $descr = 'Add annotation marker';
  if ($act=='keep'   ) $descr = 'Store on demand';
	if ($act=='remrec' ) $descr = 'Start recording remote side';
	if ($act=='remsil' ) $descr = 'Stop recording remote side';
  if ($descr!='') return $descr;
  return $act;
}

function LogAction($descr, $einfo = array(), $usr = '')
{
	global $dbintf , $configs, $msg;
  if (isset($configs)==false)
  {
    LoadBasicSettings();
  }
  if (int_ensure_defined($configs['Audit'])==0
  && int_ensure_defined($configs['AuditDb'])==0)
  {
    return;
  }
  if ( isset($einfo['level']) && $einfo['level']=='view' )
  {
    if ( int_ensure_defined($configs['AuditView'])==0 )
    {
      // detail level is view, but this is not configured to be included
      // in the audit trail, so skip it
      return;
    }
    unset($einfo['level']);
  }
  $to_add_call_info = false;
  if (isset($einfo['call_id']))
  {
    $to_add_call_info = $dbintf->GetCallInfo($einfo['call_id']);
    unset($einfo['call_id']);
  }
  if (isset($einfo['annotate_id']))
  {
    $ann = $dbintf->GetAnnotation($einfo['annotate_id']);
    $to_add_call_info = $dbintf->GetCallInfo($ann['call_id']);
    unset($einfo['annotate_id']);
  }
  if ( isset($einfo['calllist_search']) && isset($einfo['calllist_config']) )
  {
    $descr .= GetAuditCallListDescription($einfo['calllist_search'],$einfo['calllist_config']);
    unset($einfo['calllist_search']);
    unset($einfo['calllist_config']);
  }
  if ($to_add_call_info!==false)
  {
    $descr .= ', ' . GetAuditCallDescr($to_add_call_info);
  }
  if ( array_key_exists('tenantid',$einfo) )
  {
    if ($einfo['tenantid']!=0)
    {
      $tgrp = $dbintf->GetGroupInfo($einfo['tenantid']);
      $einfo['tenant'] = $tgrp ? $tgrp['name'] : null;
    }
    unset( $einfo['tenantid'] );
  }
  
  foreach($einfo as $key => $value)
  {
    $descr .= ", ".$key.":".$value;
  }
  
  if (isset($_SESSION['Cologon']))
  {
    $descr .= ' (cologin: ' . $_SESSION['CologonUsers'] . ')';
  }

  
  $ipaddr = str_ensure_defined($_SERVER['REMOTE_ADDR']);
  $line = 'IP:' . $ipaddr . ' ' . $descr;
  $oriusr = $usr;
  $isAdmin = false;
  if ($usr=='')
  {
    $actuinfo = $dbintf->GetActiveUserInfo();
    $user = $actuinfo;
    $usr = str_ensure_defined($actuinfo['usr']);
    $isAdmin = $dbintf->AmIAdminT123();
  } else
  {
    $user = $dbintf->GetUserInfoOnUsername($usr);
    if ($user !== false)
    {
      if ($dbintf->IsUserAdminT123($user['id']))
      {
        $isAdmin = true;
      }
    }
  }
  
  if ($user===false || isset($user['id'])===false)
  {
    $userid = NULL;
  } else
  {
    $userid = $user['id'];
  }
  
  // audit trail in the database
  if (int_ensure_defined($configs['AuditDb'])!=0)
  {
    if ($dbintf->db==null)
    {
      // database is closed
      $msg[] = new Msg(MSG_ERR,'Audit trail database failure');
    } else
    {
      $dbintf->AddAuditTrailEntry
      ( time() // current unix timestamp
      , $ipaddr
      , $userid
      , $descr
      );
    }
  }

  // audit trail in files
  if (int_ensure_defined($configs['Audit'])!=0)
  {
    $auditPath="/audit";
    if ($isAdmin) 
    {
      $auditPath .= "/admins";
    }
    
    // need absolute path when called from tracker
    if (COM_TYPE=='Win') $datapath = DATA_PATH;
    else $datapath = '/recordings';
    $fn = $datapath . $auditPath . '/log_' . MakeValidFnWithoutExt($usr) . '.txt';
    if (file_exists($datapath . $auditPath)==false)
    {
      mkdir($datapath . $auditPath,0770,true);
    }
    $existingFile = file_exists($fn);
    error_log(date('c ') . $line . "\n", 3, $fn);
    if ($existingFile==false)
    {
      if (COM_TYPE!='Win')
      {
        // set permissions if this is a new file
        chown($fn,'www-data');
        chgrp($fn,'www-data');
        chmod($fn,0644);
      }
    }
  }
}

function GetAuditCallListDescription($search,$cfg)
{
  global $dbintf;
  $cedb = null;
  $descr = '';
  foreach ($search as $key => $value)
  {
    if ($key=='fd' || $key=='ld')
    {
      $value = date('c',$value);
    }
    if ($key=='dir')
    {
      if ($value==0) $value .= '(Incoming)';
      if ($value==1) $value .= '(Outgoing)';
      if ($value==2) $value .= '(In/Out)';
      if ($value==3) $value .= '(Internal)';
    }
    if ($key=='rec')
    {
      $recorder = $dbintf->GetRecorderById($value);
      if ($recorder!==false)
      {
        $value .= '(' . $recorder['name'] . ')';
      }
    }
    if ($key=='prjid')
    {
      if ($cedb===null) $cedb = new CEDB($dbintf);
      $key = 'project';
      $value = $cedb->GetProjectName($value);
    }
    if ($key=='projects')
    {
      if ($cedb===null) $cedb = new CEDB($dbintf);
      $items = array();
      foreach($value as $prjid)
      {
        $items[] = $cedb->GetProjectName($prjid);
      }
      $value = implode('|',$items);
    }
    if ($key=='ceusr' || $key=='usr')
    {
      $userinfo = $dbintf->GetUserInfoOnAccId((int)$value);
      if ($userinfo!==false)
      {
        $value .= '(' . $userinfo['name'];
        if (isset($userinfo['tenantid']))
        {
          $tgrp = $dbintf->GetGroupInfo($userinfo['tenantid']);
          $value .= '[' . $tgrp['name'] . ']';
        }
        $value .= ')';
      }
    }
    $descr .= ', ' . $key . ':' . $value;
  }
  foreach ($cfg as $key => $value)
  {
    if ($key=='startfrom')
    {
      $descr .= ', ' . $key . ':' . $value;
    }
  }
  return $descr;
}

function GetAuditCallDescr($info)
{
	global $dbintf, $configs;
	$t = 'date: '.date($configs['datetimef'],$info['startdate'])
	.', remote: '.$info['remote']
	.', local: '.$info['local']
  .', id: ' . $info['remoteid'];
	if (isset($info['linenr']) && $info['linenr']>0)
	{
		$t .= ', linenr: ' . $info['linenr'];
	}
  if (isset($info['tenantid']))
  {
    $tgrp = $dbintf->GetGroupInfo($info['tenantid']);
    $t .= ', tenant: ' . $tgrp['name'];
  }
	return $t;
}

function SendTestMail($configs)
{
  global $dbintf;
  $dat = array();
	$ok = false;
	for ($i=1;$i<=2;$i++)
	{
		$key = 'EmAdmin';
		if ($i!=1) $key .= $i;
		// NOTE: we cannot use an array here,
		// because this is the web interface, not cmd.
		if (str_ensure_defined($configs[$key])=='') continue;
		$dat['To'] = $configs[$key];
		$dat['Subject'] = GetEmailSubjPrefix($dbintf) . 'Test email';
		$dat['Body'] = 'This is a test email from Apresa';
		if (!EmailingFromWebIntf($dat)) $ok = false;
	}
	return $ok;
}


function GetUploadErrDescr($code)
{
  switch ($code)
  {
    case UPLOAD_ERR_OK : return 'No error';
    case UPLOAD_ERR_INI_SIZE : return 'Max. upload size exceeded';
    case UPLOAD_ERR_FORM_SIZE : return 'HTML form size limit exceeded.';
    case UPLOAD_ERR_PARTIAL : return 'Only partially uploaded';
    case UPLOAD_ERR_NO_FILE : return 'No file was uploaded';
    case UPLOAD_ERR_NO_TMP_DIR : return 'No temp dir';
    case UPLOAD_ERR_CANT_WRITE : return 'Cannot write to disk';
    case UPLOAD_ERR_EXTENSION : return 'An extension stopped the download';
    default:
      return 'code: '.$code;
  }
}

function GetDefaultRetentionPolicy($cfg)
{
  global $dbintf;
  return [ "nr"=>null
  , "name" => "Default"
  , "recdays" => int_ensure_defined( $cfg['HdDaysKeep'] ) 
  , "screenrecdays" => int_ensure_defined( $cfg['ScreenRecDaysKeep'] )
  , "tradays" => int_ensure_defined( $cfg['TraDaysKeep'] )
  , "bckrecdays" => int_ensure_defined( $cfg['BckRecDays'] )
  , "bckscreenrecdays" => int_ensure_defined( $cfg['BckScreenRecDays'] )
  ];
}

function AutoDelete()
{
  global $configs,$dbintf;
  $configs = $dbintf->LoadConfigs(); // all

  $pols = $dbintf->GetRetentionPolicies();
  array_unshift($pols, GetDefaultRetentionPolicy($configs));

  foreach ($pols as $pol)
  {
    $PolNr = $pol['nr'];
    // disable filtering for speed if only one policy
    if (count($pols)==1) $PolNr = false;
  
    DeleteOlder($pol["recdays"],false,false,false,$PolNr);
    DeleteOlderDataType( $pol['screenrecdays'] , 'screenrec', false, false, $PolNr );
    DeleteOlderDataType( $pol['tradays'] , 'tra', false, false, $PolNr );
  
  }

  $TdmLogDaysKeep = int_ensure_defined( $configs['TdmLogDaysKeep'] );
  DeleteOlderTdmLogs( $TdmLogDaysKeep );
  
  $AuditDaysKeep = int_ensure_defined( $configs['AuditDaysKeep'] );
  DeleteOlderAuditTrail( $AuditDaysKeep );
  
}

function DeleteOlderAuditTrail($days)
{
  global $dbintf;
  $days = (int) $days;
  if ($days<=0) return;
  
  lg('Apply retention period for audit trail (' . $days . ' days)');
  $before = time() - $days * 24*60*60 ;
  $count = $dbintf->AutoTrailAutoDel($before);

  if($count > 0)
  {
    LogAction("Deleted " . $count . " Audit trail entries older than " . $days . " days");
  }
  
}

function DeleteOlderTdmLogs($days)
{
  $days = (int) $days;
  if ($days<=0) return;

  $cmd = 'find /recordings/log/ShCtiLog/ -type f ! -mtime -' . $days . ' -print0 | xargs -0 -r rm';
  exec($cmd);
}

// delfn : deleted file
// dirfn : directory and filename that could be deleted,
//         but leave the rest of the path untouched
function DeleteEmptyDirsAfterDelete($delfn,$dirfn)
{
  $prefixSize = strlen($delfn) - strlen($dirfn);
  // preserve = substr($delfn,0,$prefixSize);
  $path = dirname($delfn);
  while ( true )
  {
    if (strlen($path) <= $prefixSize ) break;
    if ($path == '.' || $path=='/') break;
    // rmdir only deletes if empty, so should be safe
    if ( ! @rmdir($path) ) break;
    $path = dirname($path);
  }
}

define('DELMODE_SCREENREC',4);
define('DELMODE_TRA',5);

function DeleteOlderDataType( $DaysKeep , $DataType , $tenant=FALSE, $backup=false, $retention=false )
{
	global $dbintf, $configs;
	
	if ($DaysKeep<=0) return;

  $logextra = '';
  if ($DataType == 'screenrec')
  {
    $logextra .= ' for screen recordings'; 
    $DeleteMode = DELMODE_SCREENREC;
  } else if ($DataType == 'tra')
  {
    $logextra .= ' for transcriptions';
    $DeleteMode = DELMODE_TRA;
  } else
  {
    return;
  }
  
  // calculate threshold time
  $olderthan = time() - $DaysKeep*60*60*24;
  
	$tenantid = FALSE;
	if ($tenant!==FALSE)
	{
		$tenantid = $tenant['id'];
		$logextra .= ' of tenant ' . $tenant['name'];
	}
  // minus one means there is no maximum
  $MaxId = -1;
  if ($tenant!==false)
  {
    $key = 'TN' . $tenantid . '_KeepUnexported';
    $KeepUnexported = (int) $dbintf->GetCachedConfigValue($key);
    if ($KeepUnexported)
    {
      $key = 'TN' . $tenantid . '_ExpIdHandled';
      $MaxId = (int) $dbintf->GetCachedConfigValue($key);
    }
  }
  if ($retention!==false)
  {
    if ($retention===null) $logextra .= ' (default policy)';
    else $logextra .= " (policy $retention)";
  }

  $basepath = AUDIO_PATH;
  if ($backup)
  {
    $logextra .= ' in the backup';
    $configs_b = $dbintf->LoadConfigsPart('b');
    $options = InitBackupOptions($configs_b);
    $basepath = $options['BackupDest'];
  }
  
  $DebugSimDelete = (int) $dbintf->GetConfigValue('DebugSimDelete');
  
  lg("Apply retention period" . $logextra . ' (' . $DaysKeep . ' days)');
  
  // do in chunks, otherwise might use too much memory
  $expiredCount = 0;
  $iteration = 0;
  while (true)
  {
    $iteration++;
    $chunksize = 100;
    $calls = $dbintf->GetOlderRecordings($olderthan, $chunksize, $DeleteMode, $tenantid, $backup, $MaxId, $retention );
    if (count($calls)==0) break;
    $ids_handled = array();
    $expiredCount += count($calls);
    if ($iteration==1 && count($calls)>0)
    {
      lg('Retention period expired' . $logextra);
    }
		// variable to turn debug on easily
		$onlyPrintDontDelete = ($DebugSimDelete!=0);
		// BEGIN DEBUG BLOCK
		if ($onlyPrintDontDelete)
		{
			foreach($calls as $call)
			{
				$callinfo = $dbintf->GetCallInfo( $call['id'] );
				$age = time() - $callinfo['startdate'];
				$days = $age / (60*60*24);
        $screenfn = GetScreenFileName($callinfo);
        $delfn = $basepath;
        $delfn .= '/' . $callinfo['directory'] . '/' . $screenfn;
        $exists = file_exists($delfn);
				lg( date('D M G:i:s Y', $callinfo['startdate']) . ' age ' . round($days,2) . ' days '
				. $callinfo['tenantid'] . ' flags: ' . $callinfo['flags'] . ' ' . $delfn . ' ' . ($exists?'y':'n') );
			}
			lg('not really doing (debug!!!!)');
			return;
		}
		// END DEBUG BLOCK
    //echo 'No. calls: '.count($calls)."\n"; 
    foreach($calls as $call)
    {
      if ($DeleteMode==DELMODE_SCREENREC)
      {
        if (HasScreenRecording($call))
        {
          $dirfn = $call['directory'] . '/' . GetScreenFileName($call);
          $delfn = $basepath;
          $delfn .= '/' . $dirfn;
          //echo 'to delete: '.$delfn."\n";
          DeleteFn($delfn);
        }
      }
      if ($DeleteMode==DELMODE_TRA)
      {
        // not here for speed: $dbintf->DeleteTranscript( $call['id'] );
      }
      $ids_handled[] = $call['id'];
    }
    if (count($ids_handled)>0)
    {
      if ($DeleteMode==DELMODE_SCREENREC)
      {
        if ($backup==false) $dbintf->SetFlagOffForIDs($ids_handled,2);
				else $dbintf->SetFlagOffForIDsArchive($ids_handled,2);
      }
      if ($DeleteMode==DELMODE_TRA)
      {
        if ($backup==false) $dbintf->DeleteTranscripts($ids_handled);
				// deleting transcripts from the backup is not supported,
				// because it is in the database
				else ;
      }
		}    
    
  }
  if ($expiredCount>0)
  {
    lg('Retention period expired for ' . $expiredCount . ' calls' . $logextra);

    $extra=[];
    if($tenantid)
    {
      $extra['tenantid'] = $tenantid;
    }

    if ($DeleteMode==DELMODE_SCREENREC)
    {
      LogAction("Auto-delete: Deleted " . $expiredCount . " screen recordings of calls older than " . $DaysKeep . " days" 
      . ($backup ? " in backup" : ""), $extra);
    }
    if ($DeleteMode==DELMODE_TRA && $backup == false)
    {
      LogAction("Auto-delete: Deleted " . $expiredCount . " transcripts of calls older than " . $DaysKeep . " days", $extra);
    }
  }
  
}

function DeleteOlder($HdDaysKeep, $tenant=FALSE, $backup=false, $forceDelMeta=false, $retention=false)
{
	global $dbintf, $configs;

  if (isset($configs)==false)
  {
    LoadBasicSettings();
  }
	
	if ($HdDaysKeep<=0) return;
  // calculate threshold time
  $olderthan = time() - $HdDaysKeep*60*60*24;
  //echo 'Older than: '.date('r',$olderthan)."\n";
  $KeepDelListed = (int) int_ensure_defined($configs['KeepDelListed']);
  $KeepAgEvListed = (int) int_ensure_defined($configs['KeepAgEvListed']);
  $DebugSimDelete = (int) $dbintf->GetConfigValue('DebugSimDelete');
  // forceDelMeta is true when deleting from backup,
  // or when deleting the same from the main storage
  if ($forceDelMeta) $KeepDelListed = 0;
  
  $cedb = new CEDB($dbintf);

	$tenantid = FALSE;
	$logextra = '';
	if ($tenant!==FALSE)
	{
		$tenantid = $tenant['id'];
		$logextra = ' of tenant ' . $tenant['name'];
	}
  $basepath = AUDIO_PATH;
  if ($backup)
  {
    $logextra .= ' in the backup';
    $configs_b = $dbintf->LoadConfigsPart('b');
    $options = InitBackupOptions($configs_b);
    $basepath = $options['BackupDest'];
  }
  $DeleteMode = 0;
  if ($KeepDelListed!=0)
  {
    $DeleteMode = 1;
    $logextra .= ' (keep meta data)';
  } else
  {
    $DeleteMode = 2;
    $logextra .= ', delete meta data';
    if ($KeepAgEvListed)
    {
      $DeleteMode = 3;
      $logextra .= ', except for AgEv';
    }
  }

  // minus one means there is no maximum
  $MaxId = -1;
  if ($tenant!==false)
  {
    $key = 'TN' . $tenantid . '_KeepUnexported';
    $KeepUnexported = (int) $dbintf->GetCachedConfigValue($key);
    if ($KeepUnexported)
    {
      $key = 'TN' . $tenantid . '_ExpIdHandled';
      $MaxId = (int) $dbintf->GetCachedConfigValue($key);
    }
  }
  if ($retention!==false)
  {
    if ($retention===null) $logextra .= ' (default policy)';
    else $logextra .= " (policy $retention)";
  }
  
  lg("Apply retention period" . $logextra . ' (' . $HdDaysKeep . ' days)');
  
  // do in chunks, otherwise might use too much memory
  $expiredCount = 0;
  $iteration = 0;
  while (true)
  {
    $iteration++;
    $chunksize = 100;
    $calls = $dbintf->GetOlderRecordings($olderthan, $chunksize, $DeleteMode, $tenantid, $backup, $MaxId, $retention );
    if (count($calls)==0) break;
    $ids_keeplisted = array();
    $ids_remove = array();
    $expiredCount += count($calls);
    if ($iteration==1 && count($calls)>0)
    {
      lg('Retention period expired for calls' . $logextra);
    }
		// variable to turn debug on easily
		$onlyPrintDontDelete = ($DebugSimDelete!=0);
		// BEGIN DEBUG BLOCK
		if ($onlyPrintDontDelete)
		{
			foreach($calls as $call)
			{
				$callinfo = $dbintf->GetCallInfo( $call['id'] );
				$age = time() - $callinfo['startdate'];
				$days = $age / (60*60*24);
        $IsInAgEval = $cedb->IsCallInAgEval( $call['id'] );
				lg( date('D M G:i:s Y', $callinfo['startdate']) . ' age ' . round($days,2) . ' days '
				. $callinfo['tenantid'] . ' ' . ($IsInAgEval?'AgEv':'NotAgEv') );
			}
			lg('not really doing (debug!!!!)');
			return;
		}
		// END DEBUG BLOCK
    //echo 'No. calls: '.count($calls)."\n"; 
    foreach($calls as $call)
    {
      // skip information entry (but still delete them from CallsArchive)
      if ($call['filename']!='')
      {
        $dirfn = $call['directory'] . '/' . $call['filename'];
        if ($call['tenantid']!=NULL && $call['isarchived']!=0)
        {
          if ($backup) UpdateTnBackupDiskUsage($call['tenantid'], -GetCallStorageEx($call,$basepath), -1);
          else UpdateTnDiskUsage($call['tenantid'], -GetCallStorageEx($call,$basepath), -1);
        }
        $delfn = $basepath;
        $delfn .= '/' . $dirfn;
        //echo 'to delete: '.$delfn."\n";
        DeleteFn($delfn);

        if (HasScreenRecording($call))
        {
          $dirfn = $call['directory'] . '/' . GetScreenFileName($call);
          $delfn = $basepath;
          $delfn .= '/' . $dirfn;
          //echo 'to delete: '.$delfn."\n";
          DeleteFn($delfn);
        }
        
        DeleteEmptyDirsAfterDelete($delfn,$dirfn);
      }

      $KeepThisOneListed = ($KeepDelListed!=0);
      if ($KeepThisOneListed==false && $KeepAgEvListed!=0)
      {
        // check if in CECalls, then we need to keep it listed
        if ($cedb->IsCallInAgEval( $call['id'] ))
        {
          $KeepThisOneListed = true;
        }
      }
      if ($KeepThisOneListed) $ids_keeplisted[] = $call['id'];
      else $ids_remove[] = $call['id'];
    }
    if (count($ids_remove)>0)
    {
      if ($backup) $dbintf->DeleteCallsArchiveForIDs($ids_remove);
    	else $dbintf->DeleteCallsForIDs($ids_remove);
    } 
    if (count($ids_keeplisted)>0)
    {
      $dbintf->SetFlagOnForIDs($ids_keeplisted,4);
			$dbintf->SetIsArchivedForIDs($ids_keeplisted,0);
		}
  }
  if ($expiredCount>0)
  {
    lg('Retention period expired for ' . $expiredCount . ' calls' . $logextra);

    $extra=[];
    if($tenantid)
    {
      $extra['tenantid'] = $tenantid;
    }

    LogAction("Auto-delete: Deleted " . $expiredCount . " calls older than " . $HdDaysKeep . " days" 
      . ($forceDelMeta ? " width metadata" : "") 
      . ($backup ? " in backup" : ""), $extra);
  }
}

function TenantsAutoDelete()
{
	TenantsRetentionPeriod();
	// max. storage is done dynamically
}

function TenantsRetentionPeriod()
{
	global $dbintf,$configs;
	// TenantsRetentionPeriod
  $configs = $dbintf->LoadConfigs();
	$tenants = $dbintf->GetTenantList();
	foreach ($tenants as $tenant)
	{
		$prefix = 'TN' . $tenant['id'] . '_';
		$HdDaysKeep = int_ensure_defined($configs[$prefix . 'HdDaysKeep']);
		// lg("Tenant: " . $tenant['name'] . ' Days:' . $HdDaysKeep);
		DeleteOlder($HdDaysKeep, $tenant);
    
    $ScreenRecDaysKeep = int_ensure_defined( $configs[$prefix . 'ScreenRecDaysKeep'] );
    DeleteOlderDataType( $ScreenRecDaysKeep , 'screenrec', $tenant );
    
    $TraDaysKeep = int_ensure_defined( $configs[$prefix . 'TraDaysKeep'] );
    DeleteOlderDataType( $TraDaysKeep , 'tra', $tenant );
    
	}
}

// $configs might not know about KeepDelListed
// so we take it as param
function AutoDeleteCall($call,$KeepDelListed,$KeepAgEvListed)
{
	global $dbintf;
  $cedb = new CEDB($dbintf);
  
	$dirfn = $call['directory'] . '/' . $call['filename'];
	$delfn = AUDIO_PATH . '/' . $dirfn;
	lg('auto-delete: '.$delfn.' '.showkb(GetFileSize($delfn)));
	DeleteFn($delfn);

	if (HasScreenRecording($call))
	{
		$dirfn = $call['directory'] . '/' . GetScreenFileName($call);
		$delfn = AUDIO_PATH . '/' . $dirfn;
		lg('auto-delete: '.$delfn.' '.showkb(GetFileSize($delfn)));
		DeleteFn($delfn);
	}

	$idlist = array();
	$idlist[] = $call['id'];
  
  $KeepThisOneListed = ($KeepDelListed!=0);
  if ($KeepThisOneListed==false && $KeepAgEvListed!=0)
  {
    // check if in CECalls, then we need to keep it listed
    if ($cedb->IsCallInAgEval( $call['id'] ))
    {
      $KeepThisOneListed = true;
    }
  }
  
	if ($KeepThisOneListed==false)
	{
		$dbintf->DeleteCallsForIDs($idlist);
	} else
	{
		$dbintf->SetFlagOnForIDs($idlist,4);
		$dbintf->SetIsArchivedForIDs($idlist,0);
	}
}

function DeleteGroup($dbintf,$groupid)
{
  global $msg;
  $groupinfo = $dbintf->GetGroupInfo($groupid);
  $subgroups = array();
  $dbintf->FindSubGroups($subgroups,$groupinfo['accid']);
  if (count($subgroups)>0)
  {
    $msg[] = new Msg(MSG_ERR,'The group contains subgroups.');
    return false;
  }
	//User accounts created by ldap-synchronization and no longer part of any group, must be deleted.
	$oldmembers = $dbintf->GetGroupMembers($groupinfo['accid']);
  if ($dbintf->DeleteGroup($groupid)==false)
  {
    $msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
    return false;
  }
	else
	{
		foreach ($oldmembers as $oldmember)
		{
			if ($dbintf->IsUserCreatedByAD($oldmember))
			{
				$dbgroups = $dbintf->GetGroupsOfWhichMember($oldmember);
				if (count($dbgroups)==0)
				{
					//The user account was created by LDAP synchronization and is no longer member of any group => delete!
					$user = $dbintf->GetUserInfoOnAccId($oldmember);
					$dbintf->DeleteUserAccount($user['id']);
				}
			}
		}
	}
	return true;
}

function CdConfigApply(&$ref_reconfig, $configs, &$ref_newconfigs, $cfgkey, $cfgvalue)
{
  if (isset($configs[$cfgkey])==true && $configs[$cfgkey]==$cfgvalue)
  {
    ; // already set to the same value
  } else
  {
    $ref_newconfigs[$cfgkey] = $cfgvalue;
    $ref_reconfig = true;
  }
}

function MobilePhonesSync()
{
  global $dbintf;
  $groupid = (int) $dbintf->GetConfigValue('MobileDevicesLinkGroup');
  if ($groupid==0 || $groupid==-1) return;
  $prop = $dbintf->GetConfigValue('MobileDevicesLinkProp');
  $mobs = array();
  $group = $dbintf->GetGroupInfo( $groupid );
  if ($group===false) return;
  $phones = array();
  $OnlyEnabled = false;
  if ( $prop == 'phones')
  {
    $phones = $dbintf->GetGroupAllNumbers( $group['accid'] , $OnlyEnabled );
  }
  if ( $prop == 'name')
  {
    $phones = $dbintf->GetGroupAllProp( $group['accid'] , $OnlyEnabled , $prop );
  }
  foreach ($phones as $phone)
  {
    $obj = array();
    $obj['imei'] = '';
    $obj['phonenum'] = $phone;
    $mobs[] = $obj;
  }
  $dbintf->SetMobileDevices($mobs);
}

// use this function to apply the calculate and apply the automatic settings
// $reconfig = true if we already know we need to reconfig
// $doReconfigIfNeeded = if false, we do not apply settings here
function EnginesSettingsCalcApply( $reconfig=false, $doReconfigIfNeeded=true )
{
  global $dbintf, $msg;
  if (isset($msg)==false)
  {
    $msg = array();
  }
  $tenants = $dbintf->GetTenantList();
  foreach($tenants as $tenant)
  {
    if($dbintf->UpdateTenantPhones($tenant['id'])) $reconfig = true;
  }
  
  MobilePhonesSync();
  
  $newconfigs = array();
  $configs = $dbintf->LoadConfigsPart('o');
  if(int_ensure_defined($configs['RecOnDmdType']) == 3)
  {
    $numbers = $dbintf->GetRecOnDmdUserNumbers();
    CdConfigApply($reconfig, $configs, $newconfigs, 'RecOnDmdList', implode(",",$numbers) );
  }
  if(int_ensure_defined($configs['DelOnDmdType']) == 3)
  {
    $numbers = $dbintf->GetDelOnDmdUserNumbers();
    CdConfigApply($reconfig, $configs, $newconfigs, 'DelOnDmdList', implode(",",$numbers) );
  }
  if(int_ensure_defined($configs['SilOnDemand']) == 3)
  {
    $numbers = $dbintf->GetSilOnDmdUserNumbers();
    CdConfigApply($reconfig, $configs, $newconfigs, 'SilOnDmdList', implode(",",$numbers) );
  }

  $numbers = $dbintf->GetUserPhoneNumbers();
  if(int_ensure_defined($configs['SipFilterType']) == 3)
  {
    CdConfigApply($reconfig, $configs, $newconfigs, 'SipFilters', implode(",",$numbers) );
  }
  if(int_ensure_defined($configs['SipFilterTypeIn']) == 3)
  {
    CdConfigApply($reconfig, $configs, $newconfigs, 'SipFiltersIn', implode(",",$numbers) );
  }
  if(int_ensure_defined($configs['SipFilterTypeOut']) == 3)
  {
    CdConfigApply($reconfig, $configs, $newconfigs, 'SipFiltersOut', implode(",",$numbers) );
  }
 if(int_ensure_defined($configs['CSTATelIPType']) == 2)
  {
    CdConfigApply($reconfig, $configs, $newconfigs, 'CSTATelIPFilter', implode(",",$numbers) );
  }
  if(int_ensure_defined($configs['H323FilterType']) == 3)
  {
    CdConfigApply($reconfig, $configs, $newconfigs, 'H323Filters', implode(",",$numbers) );
  }
  if(int_ensure_defined($configs['UnimegaFilterType']) == 3)
  {
    CdConfigApply($reconfig, $configs, $newconfigs, 'UnimegaFilters', implode(",",$numbers) );
  }  
  if ($reconfig)
  {
    $dbintf->SetConfigsPart($newconfigs,'o');
    if ($doReconfigIfNeeded)
    {
      EnginesReconfig($msg,$dbintf);
    }
  }
  return $reconfig;
}

function LogDeleteFn($fn)
{
  if (file_exists($fn))
  {
    lg("Deleting: " . $fn );
    DeleteFn($fn);
  }
}

function DeleteTheCallFiles($info,$cfg)
{
	$filename = $info['filename'];
  $basepath = AUDIO_PATH;
  if (isset($cfg['basepath']))
  {
    $basepath = $cfg['basepath'];
  }
  $path = $basepath.'/'.$info['directory'].'/';
	LogDeleteFn($path . $info['filename']);
	if (HasScreenRecording($info))
	{
		LogDeleteFn($path . GetScreenFileName($info));
	}
}

function InitBackupOptions($configs_b)
{
  $options = array();
  $options['Mounting'] = true;
  $options['BackupDest'] = BACKUP_MOUNT;
	if ($configs_b['BckTargetType']==1)
	{
		// backup to internal or USB disk
		$options['BackupDest'] = $configs_b['BckPath'];
		$options['Mounting'] = false;
		exec('mountpoint '.escapeshellarg($options['BackupDest']),$dummy,$retval);
		if ($retval != 0)
		{
      $options['error'] = $options['BackupDest'] . ' is not a mount point';
		}
	}
	if ($configs_b['BckTargetType']==2)
	{
		// backup to any directory
		$options['BackupDest'] = $configs_b['BckPath'];
		$options['Mounting'] = false;
	}
  return $options;
}

function GlobGetAudioDuration($fn)
{
	$cmd = 'soxi -d ' . escapeshellarg($fn);
	$output = array();
	exec($cmd,$output);
	if (count($output)<1) return -1;
	$line = $output[0];
	$colonPos1 = strpos($line,':');
	if ($colonPos1===false) return -1;
	$colonPos2 = strpos($line,':',$colonPos1+1);
	if ($colonPos2===false) return -1;
	$hours = (int) substr($line,0,$colonPos1);
	$minu = (int) substr($line,$colonPos1+1,$colonPos2-$colonPos1);
	$seconds = round( floatval( substr($line,$colonPos2+1) ) );
	$duration = $hours*60*60 + $minu*60 + $seconds;
	return $duration;
}


function EmailRecording($callinfo,$back,$side)
{
  global $dbintf , $configs;
  EnsureActionsLoaded();

  lg('Email recording: '.$callinfo['filename'].' back: '.$back . ' side: ' . $side);
  // find out e-mail address
  $users = $dbintf->GetUsersOfCall($callinfo,true,$side);
  if (count($users)==0)
  {
    lg("No associated users.");
    return;
  }
  $useraccid = $users[0];
  $userinfo = $dbintf->GetUserInfoOnAccId($useraccid);
  $email = $userinfo['email'];
  lg('User acc: '.$useraccid.' em: '.$email.' usr: '.$userinfo['usr']);
  if ($email=='')
  {
    lg('No email address defined');
    return;
  }

  // set access rights
  $dbintf->SetCurrentUserId($userinfo['id']);

  if ($back==0) // use the ID as specified
  {
		// find ID
		$data = $dbintf->GetCallInfoRM($callinfo['remoteid']);
		if ($data==null)
		{
			lg('Call not found '.$callinfo['remoteid']);
			return;
		}
    lg('before EmailRecordings call t='.strftime($configs['datetimef'],$data['startdate']) );
    EmailRecordings(array($data['id']),$email);
    lg('after EmailRecordings');
    return;
  }

  $data = $dbintf->GetLastOwnCall($back-1);
  if ($data==null)
  {
    lg('Call not found');
    return;
  }
  lg('before EmailRecordings call t='.strftime($configs['datetimef'],$data['startdate']) );
  EmailRecordings(array($data['id']),$email);
  lg('after EmailRecordings');

}

function EnsureActionsLoaded()
{
  global $actions_loaded;
  if (isset($actions_loaded)==false)
  {
    BasicConstants();
    LoadBasicSettings();
    $actions_loaded = true;
  }
}

function KeepPendingStoreOnDemand($data)
{
  global $dbintf, $configs_o, $configs_s;
  // Special case. A call that has finished,
  // but we were waiting on keep command.
  // Move from ActiveCalls to Calls.
  $oldid = $data['id'];
  unset($data['id']);
  $data['processingstate'] = $data['processingstate'] & ~128;
  $newid = $dbintf->InsertCall($data);
  $dbintf->MoveActiveAnnotations($newid, $oldid);
  $dbintf->MoveActiveCallKey($newid, $oldid);
  $dbintf->RemoveActiveCall($data['remoteid']);
  $data['id'] = $newid;
  if (isset($configs_o)==false)
  {
    $configs_o = $dbintf->LoadConfigsPart('o');
  }
  if (isset($configs_s)==false)
  {
    $configs_s = $dbintf->LoadConfigsPart('s');
  }
  CallPostProcessing($data);
}

function CallPostProcessing(&$ref_data)
{
  global $dbintf, $configs_s, $configs_o;
  
  $callextra = StringToArray($ref_data['callextra']);
  
  // check if anything must be done at the finish of this call
  if ($ref_data['isarchived']==1)
  {
    if ( isset($callextra['to-email']) )
    {
      $side = $callextra['to-email'];
      // 1 was the old way of setting this option
      // in that case dont check side
      if ($side=='1') $side = null;
      EnsureActionsLoaded();
      EmailRecording($ref_data,0,$side);
      // .... unset($callextra['to-email']);
      // not nec. to update DB, because it will happen once anyway
    }

    $EmailRecs = str_ensure_defined($configs_s['EmailRecs']);
    if ($EmailRecs=='fixed')
    {
      EnsureActionsLoaded();
      $dbintf->SetAdmin();
      EmailRecordings(array($ref_data['id']),$configs_s['EmailRecsDst']);
    } else if ($EmailRecs=='user')
    {
      EnsureActionsLoaded();
      EmailRecording($ref_data,0,null);
    }
    
    if (int_ensure_defined($configs_o['SforceLink'])==1
    && int_ensure_defined($configs_o['SforceLink_TaskAdd'])==1)
    {
      $cmd = '/usr/share/apresa/sforcelink.sh';
      $cmd .= ' --act add';
      $cmd .= ' --id ' . $ref_data['remoteid'];
      $cmd .= ' --tel ' . $ref_data['remote'];
      $cmd .= ' --start ' . $ref_data['startdate'];
      $cmd .= ' --dur ' . $ref_data['duration'];
      $cmd .= ' >/dev/null 2>&1 &';
      lg($cmd);
      exec($cmd);
    }
  } // isarchived
}

?>