<?php
require_once('utility.php');
require_once('callhandler.php');
require_once('actions.php');


// this class handles the serving of recordings
// (not active, but finished calls)
class CallHandler
{
	var $ids = null; // array, IDs of calls to download/playback
	var $download; // int, download or playback (influences disposition)
	var $screenrec; // int, download the screen recording ?
  var $monitorix; // which monitor (multi-screen recording)
  var $countMonitors = false; // only return the number (no output)
	var $embed; // int, this is a request from embedded player (not verifiable)
	var $toDelete; // array, which files to delete
	var $multiple; // boolean, if more than one file requested
	var $anyEncrypted; // boolean, are any of the call encrypted?
	var $zipfilename; // string, filename of zip
	var $zip; // object, zip object
	var $meta; // map, for .vidi download
	var $good; // boolean, if all requested recordings can be retrieved
	var $errorEcho = true; // what to do on error
  var $errcode; // HTTP status code if used
	var $errmsg; // error message
  var $mp3 = false;

	function Init()
	{
		global $dbintf, $configs;

		if (isset($this->ids))
		{
			// settings are set from outside
			if (isset($this->download)==false) $this->download = 1; 
			if (isset($this->screenrec)==false) $this->screenrec = false; 
			if (isset($this->embed)==false) $this->embed = false; 
		} else
		{
			// Which IDs is the user playing / downloading?
			$this->ids = explode(",",$_GET['id']);
			// does the user request download?
			$this->download = 0;
			if (isset($_GET['d']))
			{
				$this->download = (int) $_GET['d'];
			}
			// Does the user want the screen recording?
			if (isset($_GET['scr']))
			{
				$this->screenrec = (int) $_GET['scr'];
			}
			if (isset($_GET['mx']))
      {
				$this->monitorix = (int) $_GET['mx'];
        $this->videoFileExtension = 'wmv';
        if (isset($_GET['ext']))
        {
          $this->videoFileExtension = $_GET['ext'];
        }
      }
			// is this embedded playback?
			$this->embed = 0;
			if (isset($_GET['embed']))
			{
				$this->embed = (int) $_GET['embed'];
			}
		}
		
		$this->multiple = count($this->ids)>1;
		// Is any of the calls (call content) encrypted?
		$this->anyEncrypted = false;
		foreach ($this->ids as $id)
		{
			$info = $dbintf->GetCallInfo($id);
			$thisOneEncrypted = ($info['flags']&16)!=0;
			if ($thisOneEncrypted)
			{
				$this->anyEncrypted = true;
			}
		}

		if ( ($this->download==false && int_ensure_defined($configs['ToMp3Play'])==1)
		||  ($this->download==true && int_ensure_defined($configs['ToMp3Down'])==1) )
		{
			$this->mp3 = true;
		}
		
		$this->toDelete = array();
		
		$this->InitZip();
	}

	function InitZip()
	{
		// If we are downloading multiple,
		// then compress them in a .zip file.
		$this->zip = null;
		$this->meta = array();
		if ($this->multiple)
		{
			$this->zip = new ZipArchive();
			if ($this->anyEncrypted)
			{
				// even the zip file is stored in a memory location
				$this->zipfilename = tempnam('/dev/shm/apresa/web', 'apresa');
			} else
			{
				$this->zipfilename = tempnam(sys_get_temp_dir(), 'apresa');
			}
			$this->zip->open($this->zipfilename,ZIPARCHIVE::CREATE);
			// before any exit, we should delete the .zip
		}
	}
  
  // check if the file is a G.723 wave file
  function isG723($fn)
  {
    $f = fopen($fn,'r');
    if ($f===false) return false;
    $ret = $this->isG723File($f);
    fclose($f);
    return $ret;
  }
  function isG723File($f)
  {
    $header = fread($f,32);
    if (substr($header,0,4)!='RIFF') return false;
    if (substr($header,8,8)!='WAVEfmt ') return false;
    $binfmt = substr($header,20,2);
    $fmtarr = unpack("v",$binfmt);
    if (count($fmtarr)==0) return false;
    $fmt = $fmtarr[1];
    if ($fmt==0x111) return true;
    if ($fmt==0x42) return true;
    return false;
  }
	
	function HandleSingleCall($id)
	{
		global $dbintf, $configs;

		// get information about this call from the database
	  $info = $dbintf->GetCallInfo($id);
		if ($info==null)
		{
      $this->errcode = 404;
			$this->errmsg = 'Recording with ID not found';
			return false;
		}
	  
		// Check permission
		if ($this->download)
	  {
		  if (($info['permissions']&PM_DOWNLOAD)==0)
		  {
        $this->errcode = 403;
		    $this->errmsg = 'Permission denied (A)';
		    return false;
		  }
		} else
		{
			// it is playback
		  if ($this->screenrec)
			{
				if (($info['permissions']&PM_PLAYSCR)==0 && ($info['permissions']&PM_DOWNLOAD)==0)
				{
          $this->errcode = 403;
					$this->errmsg = 'Permission denied (B)';
					return false;
				}
			} else
			{
				if (($info['permissions']&PM_PLAY)==0 && ($info['permissions']&PM_DOWNLOAD)==0)
				{
          $this->errcode = 403;
					$this->errmsg = 'Permission denied (C)';
					return false;
				}
			}
		}
		// download permission might be needed if we do not embed
		$NoEmbedRequiresDownloadPM = int_ensure_defined($configs['PlayPmEmbedded']);
		if ($this->embed==0 && $NoEmbedRequiresDownloadPM)
		{
		  if (($info['permissions']&PM_DOWNLOAD)==0)
		  {
        $this->errcode = 403;
		    $this->errmsg = 'Permission denied (D)';
		    return false;
		  }
		}
		// checking playback time limit now is already handled
		// in GetCallPermissions which is called by GetCallInfo

		// screen recording has special filename
		if ($this->screenrec)
		{
			if ( ($info['flags']&2) != 0 )
			{
				$filename = GetScreenFileName($info);
			} else
			{
				return true;
			}
		} else
		{
		  $filename = $info['filename'];
		}
		$fn = AUDIO_PATH.'/'.$info['directory'].'/'.$filename;
		$downloadfn = GetDownloadFileName($filename,$info,true);

		// auditing
		LogAction('Playback/download call', array('call_id'=>$info['id']) );

		// Loading from backup
		$autoDeleted = ($info['flags']&4)!=0;
		if ($autoDeleted)
		{
			// check setting
			if ( (int) $dbintf->GetConfigValue('LoadDelBackup') == 0 )
			{
        $this->errcode = 404;
				$this->errmsg = 'The recording has expired and was auto-deleted.';
				return false;
			} else
			{
				// Can we load it from the backup?
        $errmsg = '';
				$newfn = LoadFromBackup($info,$filename,$errmsg);		//LoadFromBackup, moved to utility.php!
				if ($newfn=='')
				{
					return false;
				} else
				{
					// we have restored a recording from backup,
					// and have it ready for playback
					$fn = $newfn;
					// we need to delete this file afterwards
					$this->toDelete[] = $fn;
				}
			}
		}
		
		// Application Level Encryption
		$thisOneEncrypted = ($info['flags']&16)!=0;
		if ($thisOneEncrypted)
		{
			// ALE files are decrypted to a memory file location
			// to avoid ever storing them unencrypted
			$oldfn = $fn;
			$parts = pathinfo($oldfn);
			$dir = '/dev/shm/apresa/'.substr($parts['dirname'],12);
			@mkdir($dir,0777,TRUE);
			$fn = $dir . '/' . session_id() . '_' . $parts['basename'];
			if (file_exists($oldfn)==false)
			{
				echo 'The file is missing: '.$oldfn;
				return false;
			}
			
			$GlobalKey = file_exists('/dev/shm/apresa/play.ekey');
			
			// perform the decryption
			$cmdToExec = 'apresa_crypt -d' . ($GlobalKey?' ':' -u '.$_SESSION['PlayKeyId']) . ' -i '.$oldfn.' -o '.$fn;
			$retValue = 999;
			$dummyOutput = array();
			exec($cmdToExec,$dummyOutput,$retValue);
			if ($retValue!=0)
			{
				lg('Playback decryption error '.$retValue);
				
				if (file_exists($fn))
				{
					DeleteFn($fn);
				}
        $this->errcode = 500;
				if ($retValue=='41') $this->errmsg = 'Decryption error: Playback of encrypted recordings is currently not possible';
		    else $this->errmsg = 'Decryption error (code: '.$retValue.')';
		    return false;
			}
			// schedule delete tmp. decrypted file, not immediately
			$this->toDelete[] = $fn;
			$GeneratedFile = true;
		}
	
		if ( $this->mp3 )
		{
			$parts = pathinfo($fn);
			if ($parts['extension']!='mp3'
			&& $parts['extension']!='wmv'
      && $parts['extension']!='mp4'
      && $parts['extension']!='zip'
      && !($info['flags'] & 2048)
			)
			{
				$downloadfn = change_filename_extension($downloadfn,'.mp3');
				$tmp_fn = tempnam(sys_get_temp_dir(), 'apresa_call_');
				$this->toDelete[] = $tmp_fn;
				$tmp_fn .= '.mp3';
				$this->toDelete[] = $tmp_fn;
				$GeneratedFile = true;

        // qual. should depend on mono/stereo
        $cmdToExec = 'soxi -c ' . escapeshellarg($fn);
        $output = array();
        exec($cmdToExec, $output);
        $channels = (int) $output[0];
        $qual = '16';
        if ($channels==2) $qual = '32';
				
        if ($this->isG723($fn))
        {
          $program = 'avconf';
          if (file_exists('/usr/bin/ffmpeg'))
          {
            $program = 'ffmpeg';
          }
          $cmdToExec = $program . ' -f g723_1 -i ' . escapeshellarg($fn)
            . ' -b:a '.$qual.'k ' . escapeshellarg($tmp_fn);
        } else
        {
          $cmdToExec = 'sox ' . escapeshellarg($fn) . ' -C '.$qual.'.4 ' . escapeshellarg($tmp_fn);
        }
        
				$dummyOutput = array();
				exec($cmdToExec,$dummyOutput,$retValue);
				if ($retValue!=0)
				{
					lg('Error encoding to MP3 (code: '.$retValue.')');
          $this->errcode = 500;
					$this->errmsg = 'Error encoding to MP3 (code: '.$retValue.')';
					return false;
				}
				$fn = $tmp_fn;
			}
		}		

	  if ($this->multiple==false)
	  {
			// we are not zipping
			if (file_exists($fn))
	    {
				// we find to get latest path info
				clearstatcache();

				// don't change file name
				// it should already be set through the URL
				// otherwise: GetDownloadFileName($fn,$info);

				// decide on a content type
	      $pif = pathinfo($fn);
				$ext = $pif['extension'];
        
        // requesting a monitor from a zipped multi-screen recording ?
        if ($ext=='zip' && $this->screenrec
        && (isset($this->monitorix) || $this->countMonitors) )
        {
          if ($this->countMonitors)
          {
            $this->zip = new ZipArchive();
            $this->zip->open($fn);
            if ($this->countMonitors)
            {
              $this->monitorCount = $this->zip->numFiles;
              $zipEntry = $this->zip->statIndex(0);
              if (isset($zipEntry))
              {
                $zipEntryName = $zipEntry['name'];
                $pifEntry = pathinfo($zipEntryName);
                $this->videoFileExtension = $pifEntry['extension'];
              }
              $this->zip->close();
              return true;
            }            
          } else
          {
            $expiryTime = 10 * 60;
            // test if in TmpFiles
            $cachefn = $pif['dirname'] . '/' . $pif['filename'] . '-mx' . $this->monitorix . '.' . $this->videoFileExtension;
            $cacheEntry = $dbintf->GetTmpFile($cachefn);
            $cacheFound = false;
            if ($cacheEntry!==false)
            {
              // try to use it
              // it should exist, but maybe it does not
              if (file_exists($cachefn))
              {
                // then stream this file
                $fn = $cachefn;
                $pif = pathinfo($fn);
                $ext = $pif['extension'];                
                $cacheFound = true;
                $newExpiry = time() + $expiryTime;
                // check if significantly different (one minute)
                // to prevent flooding the database with small changes
                if ($newExpiry - $cacheEntry['expiry'] >= 60)
                {
                  $dbintf->EditTmpFile($cachefn, array('expiry'=>time() + $expiryTime) );
                }
              }
            }
            if ($cacheFound==false)
            {
              $this->zip = new ZipArchive();
              $this->zip->open($fn);
              $zipix = $this->monitorix;
              $zipEntry = $this->zip->statIndex($zipix);
              if (isset($zipEntry))
              {
                $zipEntryName = $zipEntry['name'];
                $pifEntry = pathinfo($zipEntryName);
                $destfn = $pif['dirname'] . '/' . $pif['filename'] . '-mx' . $this->monitorix . '.' . $pifEntry['extension'];
                // debug: echo $destfn; exit;
                copy("zip://".$fn.'#'.$zipEntryName,$destfn);
                $dbTmpFile = array
                ( 'filepath' => $destfn
                , 'filesize' => filesize($destfn)
                , 'added' => time()
                , 'expiry' => time() + $expiryTime
                );
                $dbintf->AddTmpFile( $dbTmpFile );
                // then stream this file
                $fn = $destfn;
                $pif = pathinfo($fn);
                $ext = $pif['extension'];
              } else
              {
                echo 'zip entry not found';
                exit;
              }
            }
            
          }

        }

        if ($this->countMonitors)
        {
          $this->monitorCount = 1;
          return true;
        }

				if ($ext=='wmv') $ct = 'video/x-ms-wmv';
				else if ($ext=='zip') $ct = 'application/zip';
				else if ($ext=='mp3') $ct = 'audio/mpeg';
        else if ($ext=='mp4') $ct = 'video/mp4';
				else if ($ext=='wav') $ct = 'audio/vnd.wave';
				else $ct = 'application/octet-stream'; // default
				header('Content-Type: '.$ct);
	      header('Pragma: '); // needed for IE (HTTPS)
				if ($this->download)
	      {
					header('Content-Disposition: attachment; filename='.$downloadfn);
	      }
        // This also serves partial file request,
        // which is needed for seeking in WMP.
        ServerFilePart($fn);
        // instead of readfile($fn);
	    } else {
			  if ($info['isarchived']==0)
			  {
          $this->errcode = 404;
	      	$this->errmsg = 'The archive does not contain this recording.';
				} else
				{
          $this->errcode = 404;
	      	$this->errmsg = 'The recording file does not exist or cannot be accessed. '.$fn;
	   		}
				return false;
	    }
	  } else
	  {
			// we are ZIPing
	    if (file_exists($fn))
	    {
        // add this recording to the ZIP object collection
				
				// charset below might not work on other PCs
        // don't use: $zip_charset = "CP852";
        // but ASCII is guaranteed to work
        $zip_charset = "ASCII";
        $exp_fn = GetDownloadFileName($filename,$info,true);
				$exp_fn = change_filename_extension($exp_fn,'.' . get_filename_extension($fn));
        $inzip_fn = iconv("UTF-8","$zip_charset//IGNORE",$exp_fn);

        // Collect meta data
        // For use with VidiPlayer
        $metaobj = array();
        // load some extra from DB
        $L = $dbintf->GetContactName($info['localcontact']);
        $R = $dbintf->GetContactName($info['remotecontact']);
        $C = $dbintf->GetContactName($info['connectedcontact']);
        // no need to UTF8 encode, because ASCII
        // other things are already UTF-8
        $metaobj["fn"] = $inzip_fn;
        $metaobj["start"] = date('Y-m-d H:i:s',$info['startdate']);
        // channelid defines a unique number for
        // defining what is one channel
        // For the Apresa, this is the localcontact (id)
        $metaobj["channelid"] = $info['localcontact'];
        $metaobj["channelname"] = ($L!='')?$L:$info['local'];
        // extra data:
        $metaobj["local"] = $info['local'];
        $metaobj["lname"] = $L;
        $metaobj["linenr"] = $info['linenr'];
        $metaobj["remote"] = $info['remote'];
        $metaobj["rname"] = $R;
        $this->meta[] = $metaobj;

				$this->zip->addFile($fn,$inzip_fn);
			} else
			{
				// not there, skip
			}
	  }
		
		return true;
	} // end of HandleSingleCall

	function FinalizeZip()
	{
		// if multiple files were requested, this means ZIP it.
		if ($this->multiple && $this->good)
		{
			// this metadata is used by VidiPlayer
			$this->zip->addFromString("METADATA.dat",json_encode($this->meta));
		}
		if ($this->multiple)
		{
			$this->zip->close();
		}
	}
	
	function ServeCloseZip()
	{
		if ($this->multiple && $this->good)
		{
			// debugging code:
			//echo json_encode($this->meta);
			//DeleteFn($this->zipfilename);
			//exit;
			
			if ($this->download)
			{
				$download_fn = 'calls.zip';
				$download_ct = 'application/zip';
			} else
			{
				$download_fn = 'calls.vidi';
				$download_ct = 'application/vnd.vidicode.audio';
			}
			
			
			header('Content-Type: '.$download_ct);
			header('Pragma: '); // needed for IE (HTTPS)
			if ($this->download)
			{
				header('Content-Disposition: attachment; filename='.$download_fn);
			}
		
			readfile($this->zipfilename);
		}
		if ($this->multiple)
		{
			DeleteFn($this->zipfilename);
		}
	}
	
	// this is the main function,
	// and only entry point of this class
	function Perform()
	{
		global $dbintf, $configs;

		// first parse URL params,
		// and process these,
		// and prepare.
		$this->Init();

		$this->good = true;
		// We are now going to loop through all the requested calls
		foreach ($this->ids as $id)
		{
			$ok = $this->HandleSingleCall($id);
			if ($this->errorEcho && isset($this->errmsg))
			{
				echo $this->errmsg;
			}
			if ($ok==false)
			{
				$this->good = false;
				break;
			}
		} // end of calls loop

		// single file request are already served at this point

		$this->FinalizeZip();

		// delete temporal files
		// these can be decryption files
		foreach ($this->toDelete as $delFn)
		{
			if (file_exists($delFn))
			{
				DeleteFn($delFn);
			}
		}

		$this->ServeCloseZip();
    
    return $this->good;
	}
}


?>