<?php
require_once('general.php');
require_once('certificates_general.php');
require_once('acmeclient.php');

HandleCertificate($dbintf);

function RestoreKeyPermissions($id)
{
  global $dbintf, $msg;
  $cert = $dbintf->GetCertificate($id);
  $owner = $cert['webkey'] ? "www-data" : "root";
  msg_signal_service($msg, "pkeypermissions $owner $cert[key_location]");
}

function HandleCertificate($dbintf)
{
  global $rpl,$rooturl,$msg,$lng,$configs;
	
	if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0)
  {
    echo 'Permission denied';
    exit;
  }
	
	$id = str_ensure_defined($_GET['id']);
	$cert = $dbintf->GetCertificate($id);
	
	if($cert === false)
	{
		$msg[] = new Msg(MSG_ERR, 'Certificate does not exit');
		$_SESSION['msg'] = $msg;
    header('Location: '.$rooturl.'certificates.php');
		exit;
	}
	
	$act = str_ensure_defined($_POST['act']);
	
	if($act != '')
	{
		if($act == 'updateCert')
		{
			LogAction("Edit Certificate",array('level'=>'view', "id"=>$id, "name"=>$cert['name']));
			
			$certupdate = array();
			
			if(isset($_POST['name']))
			{
				$certupdate['name'] = $_POST['name'];
			}
			if(isset($_POST['trusted']))
			{
				$certupdate['trusted'] = intval($_POST['trusted']);
			}
			if(isset($_POST['webkey']))
			{
				$certupdate['webkey'] = intval($_POST['webkey']);
        if ($certupdate['webkey']==0)
        {
					msg_signal_service($msg, 'pkeypermissions root ' . $cert['key_location']);
        } else
        {
					msg_signal_service($msg, 'pkeypermissions web ' . $cert['key_location']);
        }
			}
			
			$dbintf->UpdateCertificate($id, $certupdate);
			
			if(isset($certupdate['trusted']))
			{
				if($certupdate['trusted'] == 0)
				{
					msg_signal_service($msg, 'xcertnottrust ' . $cert['cert_location']);
				}
				else
				{
					msg_signal_service($msg, 'xcerttrust ' . $cert['cert_location']);
				}
			}
			
			$msg[] = new Msg(MSG_INF, "Updated certificate");
			$_SESSION['msg'] = $msg;
			header('Location: '.$rooturl.'certificates.php');
			exit;
		}
		else if($act == 'download')
		{
			$what = str_ensure_defined($_POST['what']);
			
			switch($what)
			{
				case "certificate":
					DownloadCertificate($cert['cert_location'], $cert['name'] . '.crt');
					break;
				case "intermediate":
					DownloadCertificate($cert['intermediate_location'], $cert['name'] . '-intermediate.crt');
					break;
				default:
					exit;
			}
    }
    else if($act == 'forcerenewletsencrypt')
    {
			try 
			{
        msg_signal_service($msg, 'pkeypermissions web ' . $cert['key_location']);
        RenewLetsEncrypt($id);
        $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.CertRenewed"));

        $uses = $dbintf->GetCertificateUses($cert['id']);
        $applyMsg = [];

        foreach($uses as $use)
        {
          if($use['usedfor'] == "Https")
          {
            if(!msg_signal_service($applyMsg, "xcertmake"))
            {
              $msg[] = new Msg(MSG_ERR, "Failed to apply HTTPS certificate: " . end($applyMsg)->text);
            }

            $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.HTTPSRenewed"));

          }
          if($use['usedfor'] == "Telsys" || $use['usedfor'] == "TelsysClient")
          {
            if(!msg_signal_service($applyMsg, "telsys apply"))
            {
              $msg[] = new Msg(MSG_ERR, "Failed to apply VoIP Service certificate: " . end($applyMsg)->text);
            }

            $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.VoipServiceRenewed"));
          }
          if($use['usedfor'] == "TunnelTLS")
          {
            if(!msg_signal_service($msg, "tunneltlscert"))
            {
              throw new Exception ("Failed to apply renewed Let's Encrypt for V-Tap over TLS:" . end($msg)->text);
            }

            $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.TunnelTLSRenewed"));
          }
        }
			}
			catch(Exception $ex)
			{
				lg("Failed forced renewal of certificate $cert[name] " . $ex->getMessage());
        $msg[] = new Msg(MSG_ERR, $ex->getMessage());
			}

      RestoreKeyPermissions($id);
			if(file_exists(CERTDIR . "lets_encrypt_old/" . $cert['key_location']))
			{
				msg_signal_service($msg, "pkeypermissions root " . "lets_encrypt_old/" . $cert['key_location']);
			}

			$_SESSION['msg'] = $msg;
			header('Location: '.$rooturl.'certificate.php?id=' . $id);
			exit;
    }
    else if($act == 'revokeletsencrypt')
    {
			try
			{
        msg_signal_service($msg, 'pkeypermissions web ' . $cert['key_location']);
				$reason = int_ensure_defined($_POST['reason']);
				RevokeLetsEncrypt($id, $reason);
			}
			catch(Exception $ex)
			{
				lg("Failed revocation of certificate $cert[name] " . $ex->getMessage());
        $msg[] = new Msg(MSG_ERR, $ex->getMessage());
      }
      
      RestoreKeyPermissions($id);
      
			$_SESSION['msg'] = $msg;
			header('Location: '.$rooturl.'certificate.php?id=' . $id);
			exit;
		}
	}
	
	$cert_string = GetCertificateInfoString($cert['cert_location']);
	
	if(strlen(str_ensure_defined($cert['intermediate_location'])) > 0)
	{
		$intermediate_string = GetCertificateInfoString($cert['intermediate_location']);
  }
  
  if(int_ensure_defined($cert['lets_encrypt']) == 2)
  {
    $msg[] = new Msg(MSG_ERR, $lng->Get("Crypt.RevokedCert"));
  }
	
	$data = "var name = \"" . JSQuote($cert['name']) . "\";\r\n";
	$data .= "var trusted = " . int_ensure_defined($cert['trusted']) . ";\r\n";
	$data .= "var webkey = " . int_ensure_defined($cert['webkey']) . ";\r\n";
	$data .= "var privatekey = " . int_ensure_defined($cert['privatekey']) . ";\r\n";
	$data .= "var intermediate = " . intval(strlen(str_ensure_defined($cert['intermediate_location'])) > 0) . ";\r\n";
  $data .= "var id = " . $id . ";\r\n";
  $data .= "var lets_encrypt = " . int_ensure_defined($cert['lets_encrypt']) . ";\r\n";
	
	$certificate_uses = $dbintf->GetCertificateUses($id);
	
	$data .= "var uses =[\r\n";
	$first = true;
	foreach($certificate_uses as $use)
	{
		if(!$first) $data .= ",\r\n";
		$data .= "{ usedfor: " . '"' . JSQuote($use['usedfor']) . '"' . " }";
		$first = false;
	}
	$data .= "\r\n];\r\n";
	
	$data .= GeneralJSData();
	
	LogAction("View Certificate",array('level'=>'view', "id"=>$id, "name"=>$cert['name']));
	
	$dbintf->close();
	
  $template_content = file_get_contents('../apresa/templ/certificate.htm');

  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
	$rpl['cert'] = $cert_string;
	$rpl['intermediate'] = str_ensure_defined($intermediate_string);
	$rpl['CommonName'] = str_ensure_defined($cert['subject']);
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}

?>