<?php
require_once('general.php');
require_once('certificates_general.php');

HandleCertificateRequest($dbintf);

function GetCertificateFiles()
{	
  $out = [];

	$certfile = $_FILES['certfile'];
	
	if($certfile['error'] !== UPLOAD_ERR_OK)
	{
		throw new Exception("The upload of the certificate did not succeed ($pkeyfile[error])");
	}
	
	$cert = file_get_contents($certfile['tmp_name']);
	
	if($cert === false)
	{
    throw new Exception("Could not read the uploaded certificate");
  }
  
  $out["cert"] = $cert;
	
	$interfile = $_FILES['interfile'];
	
	if($interfile['error'] === UPLOAD_ERR_OK)
	{
		$intermediate = file_get_contents($interfile['tmp_name']);
		
		if($intermediate === false)
		{
			throw new Exception("Could not read the uploaded intermediate certificate(s) file");
    }
    
    $out["intermediate"] = $intermediate;
	}
	else if($interfile['error'] !== UPLOAD_ERR_NO_FILE )
	{
		throw new Exception("The upload of the intermediate certificate(s) did not succeed ($pkeyfile[error])");
  }
  
  return $out;
}

function UploadSignedCertificate($req)
{
	global $msg, $dbintf;
		
	if($_POST['type'] == 'file')
	{
		$files = GetCertificateFiles();
	}
	else
	{
		$cert = str_ensure_defined($_POST['certtext']);
		$intermediate = str_ensure_defined($_POST['intertext']);
    
    $files = [];
    $files["cert"] = $cert;
    if($intermediate != "") $files["intermediate"] = $intermediate;
  }
  
	msg_signal_service($msg, "pkeypermissions web " . $req['key_location']);
	$pkey = file_get_contents(CERTDIR . $req['key_location']);
	msg_signal_service($msg, "pkeypermissions root " . $req['key_location']);
	
	if($pkey === false)
	{
		throw new Exception("Could not read private key from disk");
  }

  $files["key"] = $pkey;
  $id = UploadCertificate($req['name'], $files);
  
	return $id;
}

function HandleCertificateRequest($dbintf)
{
  global $rpl,$rooturl,$msg,$lng,$configs;
	
	if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0)
  {
    echo 'Permission denied';
    exit;
  }
	
	$id = str_ensure_defined($_GET['id']);
	$req = $dbintf->GetCertificateRequest($id);
	
	if($req === false)
	{
		$msg[] = new Msg(MSG_ERR, 'Certificate signing request does not exit');
		$_SESSION['msg'] = $msg;
    header('Location: '.$rooturl.'certificates.php');
	}

	$act = str_ensure_defined($_POST['act']);
	
	if($act != '')
	{
		if($act == 'updateRequest')
		{
			LogAction("Edit certificate signing request",array("id"=>$id, "name"=>$req['name']));
			
			$requpdate = array();
			
			if(isset($_POST['name']))
			{
				$requpdate['name'] = $_POST['name'];
			}
			
			$dbintf->UpdateCertificateRequest($id, $requpdate);
			
			$msg[] = new Msg(MSG_INF, "Updated certificate signing request");
			$_SESSION['msg'] = $msg;
			header('Location: '.$rooturl.'certificates.php');
			exit;
		}
		else if($act == 'download')
		{
			LogAction("Download certificate signing request",array("id"=>$id, "name"=>$req['name']));
			$what = str_ensure_defined($_POST['what']);
			
			switch($what)
			{
				case "certificate":
					DownloadCertificate($req['request_location'], $req['name'] . '.csr');
					break;
				default:
					exit;
			}
		}
		else if($act == 'upload')
		{
      try 
      {
        $id = UploadSignedCertificate($req);
        $location = $rooturl.'certificate.php?id=' . $id;
        LogAction('Upload signed certificate for certificate request', array('id'=>$id, 'name'=>$_POST['name']));
      } 
      catch (Exception $ex) 
      {
        $msg[] = new Msg(MSG_ERR, $ex->getMessage());
        $location = $location = $rooturl.'certificaterequest.php?id=' . $id;
      }
			
			if($_POST['delReq'] === '1')
			{
				if(DeleteCertificateRequest($req['id']))
				{
					$msg[] = new Msg(MSG_INF, "Certificate signing request has been deleted");
					LogAction("Delete certificate signing request",array('id'=>$id,'name'=>$cert['name']));
				}
			}
			
			$_SESSION['msg'] = $msg;
			header('Location: '.$location);
			exit;
		}
	}
	
	$req_string = GetCertificateRequestInfoString($req['request_location']);
	
	$data = "var name = '" . JSQuote($req['name']) . "';\r\n";
	$data .= "var id = " . $id . ";\r\n";
	
	$data .= GeneralJSData();
	
	LogAction("View Certificate",array('level'=>'view', "id"=>$id, "name"=>$req['name']));
	
	$dbintf->close();
	
  $template_content = file_get_contents('../apresa/templ/certificaterequest.htm');

  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
	$rpl['req'] = $req_string;
	$rpl['CommonName'] = str_ensure_defined($req['subject']);
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}

?>