<?php
require_once('general.php');
require_once('certificates_general.php');
require_once("acmeclient.php");

HandleCertificate($dbintf);

function RegisterLetsEncrypt()
{
	global $dbintf, $msg, $lng;
	
	LogAction("Register account at Let's Encrypt");
	lg("Register account $_POST[email] at Let's Encrypt");

  $client = new AcmeClient(ACME_ENDPOINT, ACME_VERIFY_SSL);
  $account = $client->RegisterAccount($_POST["email"]);
  $keyFilename = CERTDIR . 'leacc.key';

  ExportKeyToFile($account["key"], $keyFilename);

  $toDB =
  [
    'RegLetsEncrypt' => 1,
    'RegLetsEncryptEmail' => $_POST['email'],
    'RegLetsEncryptAccount' => $account["url"],
    'RegLetsEncryptKey' => $keyFilename,
  ];

  $dbintf->SetConfigsPart($toDB, 'web');
  $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.RegisteredAtLe"));
}

function UpdateLetsEncrypt()
{
	global $dbintf, $msg, $lng;
	
	LogAction("Update account at Let's Encrypt");
	lg("Update account email $_POST[email] at Let's Encrypt");

  $client = new AcmeClient(ACME_ENDPOINT, ACME_VERIFY_SSL);
  $client->SetAccount(GetLetsEncryptAccount());
  $client->UpdateAccount($_POST["email"]);

  $toDB = array
  (
    'RegLetsEncryptEmail' => $_POST['email']
  );

  $dbintf->SetConfigsPart($toDB, 'web');

  $msg[] = new Msg(MSG_INF,  $lng->Get("Crypt.UpdatedLeEmail"));
}

function DeactivateLetsEncrypt()
{
	global $dbintf, $msg, $lng;
	
	LogAction("Deactivate account at Let's Encrypt");
	lg("Deactivate account at Let's Encrypt");

  $client = new AcmeClient(ACME_ENDPOINT, ACME_VERIFY_SSL);
  $client->SetAccount(GetLetsEncryptAccount());
  $client->DeactivateAccount();

  $dbintf->DeleteConfigsWithPrefix("RegLetsEncrypt");

  $msg[] = new Msg(MSG_INF,  $lng->Get("Crypt.DeactivatedLeAccount"));
}

function GetLetsEncryptCertificate()
{
	global $msg, $dbintf, $lng;
	
	LogAction("Request Let's Encrypt certificate");
	lg("Get Let's Encrypt certificates: $_POST[domains]");

  $webDomains = explode(",", $_POST["domains"]);
  $domains = [];
  
  foreach($webDomains as $domain)
  {
    $domains[] = [
      "type" => "dns",
      "value" => $domain
    ];
  }
  
  $keyConfig = TransfromWebPrivKey($_POST);

  $client = new AcmeClient(ACME_ENDPOINT, ACME_VERIFY_SSL);
  $client->SetAccount(GetLetsEncryptAccount());
  $certs = $client->RequestCert($domains, $keyConfig);

  $baseName = uniqid($dbintf->actuinfo['id'], true);
  $certFile = $baseName . '.crt';
  $certKey = $baseName . '.key';

  $cert = array_shift($certs["certs"]);

  ExportKeyToFile($certs["key"], CERTDIR . $certKey);
  ExportCertToFile($cert, CERTDIR . $certFile);

  if(sizeof($certs["certs"]) > 0)
  {
    $intermediateFile = $baseName . '-CA.crt';
    ExportCertBundleToFile($certs["certs"], CERTDIR . $intermediateFile);
  }
  else 
  {
    $intermediateFile = null;
  }

  $certInfo = openssl_x509_parse($cert);

  $toDB = [
    "name" => $_POST["name"],
    "subject" => str_ensure_defined($certInfo['subject']['CN']),
    "valid_to" => int_ensure_defined($certInfo['validTo_time_t']),
    "trusted" => 0,
    "privatekey" => 1,
    "cert_location" => $certFile,
    "key_location" => $certKey,
    "intermediate_location" => $intermediateFile,
    "lets_encrypt" => 1
  ];

  $id = $dbintf->InsertCertificate($toDB);

  msg_signal_service($msg, 'pkeypermissions root ' . $certKey);

  $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.GotLeCert"));

  $_SESSION['msg'] = $msg;
  header('Location: certificate.php?id=' . $id);
  exit;
}

function LetsEncryptAct()
{
  global $msg, $dbintf, $lng;

  $cmd = str_ensure_defined($_POST['cmd']);

  if($cmd == 'register-account')
  {
    if(int_ensure_defined($_POST['delold']))
    {
      DeactivateLetsEncrypt();
    }

    RegisterLetsEncrypt();
  }
  else if($cmd == 'update-email')
  {
    UpdateLetsEncrypt();
  }
  else if($cmd == 'get-cert')
  {
    GetLetsEncryptCertificate();
  }
  else if($cmd == 'deactivate-account')
  {
    DeactivateLetsEncrypt();
  }
  else if($cmd == 'autorenewal-on')
  {
		LogAction("Enable Let's Encrypt automatic renewal");

		lg("Enable Let's Encrypt automatic renewal");

    $dbintf->SetConfigsPart(
      ["LetsEncryptAutoRenewal" => 1],
      "web"
		);

    $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.AutomaticLeRenewalEnabled"));
  }
  else if($cmd == 'autorenewal-off')
  {
		LogAction("Disable Let's Encrypt automatic renewal");

		lg("Disable Let's Encrypt automatic renewal");

    $dbintf->SetConfigsPart(
      ["LetsEncryptAutoRenewal" => 0],
      "web"
    );

    $msg[] = new Msg(MSG_INF, $lng->Get("Crypt.AutomaticLeRenewalDisabled"));
  }
}

function HandleCertificate($dbintf)
{
  global $rpl,$msg,$lng,$configs;
  
  if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0)
  {
    echo 'Permission denied';
    exit;
  }
  
  $act = str_ensure_defined($_POST['act']);
  
  if($act != '')
  {
    if ($act=='del-certificates')
    {
      $ids = explode(',',$_POST['del-ids']);
      foreach ($ids as $id)
      {
        DeleteCertificate($id);
      }
    }
    if($act == 'del-certificaterequests')
    {
      $ids = explode(',',$_POST['del-ids']);
      foreach ($ids as $id)
      {
        DeleteCertificateRequest($id);
      }
    }
    if($act == 'lets-encrypt')
    {
      try 
      {
        LetsEncryptAct();
      }
      catch(Exception $ex)
      {
				lg("Failed Let's Encrypt: $_POST[cmd]: " . $ex->getMessage());
        $msg[] = new Msg(MSG_ERR, $ex->getMessage());
      }     
    }
    if($act == 'cert-expiry-settings')
    {
      $dbintf->SetConfigsPart(
        [
          "PrivKeyExpiryAlarm" => int_ensure_defined($_POST["PrivKeyExpiryAlarm"]),
          "NoPrivKeyExpiryAlarm" => int_ensure_defined($_POST["NoPrivKeyExpiryAlarm"]),
          "CertExpiryTimeAlarm" => int_ensure_defined($_POST["CertExpiryTimeAlarm"]),
        ],
        "web"
      );

      $msg[] = new Msg(MSG_INF, $lng->Get("Options.ConfigSaved"));
    }

    $_SESSION['msg'] = $msg;
    header('Location: certificates.php');
    exit;
  }

	if(isset($_GET['lesubscriberagreement']))
	{
		echo GetLetsEncryptSubscriberAgreement();
		exit;
	}
  
  
  $certificates = $dbintf->GetCertificates();
  $certificatedata = "var certificates = " . json_encode($certificates) . ";\r\n";
  
  $certificatesrequests = $dbintf->GetCertificateRequests();
  $certificatereqdata = "var certificaterequests = " . json_encode($certificatesrequests) . ";\r\n";
  
  $data = '';
  $data .= $certificatedata;
  $data .= $certificatereqdata;
  $data .= "var RegLetsEncrypt=" . int_ensure_defined($configs['RegLetsEncrypt']) . ";\r\n";
  $data .= "var RegLetsEncryptEmail=\"" . JSQuote(str_ensure_defined($configs['RegLetsEncryptEmail'])) . "\";\r\n";
  $data .= "var LetsEncryptAutoRenewal=" . int_ensure_defined($configs['LetsEncryptAutoRenewal']). ";\r\n";

  $configs_web = $dbintf->LoadConfigsPart("web");

  $data .= "var PrivKeyExpiryAlarm=" . JSQuote(int_ensure_defined($configs_web['PrivKeyExpiryAlarm'])) . ";\r\n";
  $data .= "var NoPrivKeyExpiryAlarm=" . JSQuote(int_ensure_defined($configs_web['NoPrivKeyExpiryAlarm'])) . ";\r\n";
  $data .= "var CertExpiryTimeAlarm=" . (isset($configs_web['CertExpiryTimeAlarm']) ? JSQuote(int_ensure_defined($configs_web['CertExpiryTimeAlarm'])) : "7") . ";\r\n";

  if(isset($_SESSION['leCertName']))
  {
    $data .= "var leCertName=\"" . JSQuote($_SESSION['leCertName']) . "\";\r\n";
    unset($_SESSION['leCertName']);
  }
  else 
  {
    $data .= "var leCertName=\"\";\r\n";
  }

  if(isset($_SESSION['leDomains']))
  {
    $domains = explode(",", $_SESSION['leDomains']);
    $domains_tab = array();
    foreach($domains as $domain)
    {
      $domains_tab[] = array("name" => $domain);
    }
    $data .= "var leDomains = " . json_encode($domains_tab) . ";\r\n";
    unset($_SESSION['leDomains']);
  }
  else 
  {
    $data .= "var leDomains = [];\r\n"; 
  }
	
	$data .= "var eckey=" . (PHP_VERSION_ID > 70100 ? 1 : 0) . ";\r\n";
  $data .= GeneralJSData();
  
  LogAction("View certificates",array('level'=>'view'));
  
  $dbintf->close();
  
  $template_content = file_get_contents('../apresa/templ/certificates.htm');
  
  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}
?>