<?php

require_once('general.php');
require_once('actions.php');

HandlePage($dbintf);

function HandlePage($dbintf)
{
  global $rpl,$msg,$lng,$linemap;

  if (($dbintf->actuinfo['permissions']&PM_VIEWPHONEBOOK)==0
  ||  ($dbintf->actuinfo['permissions']&PM_EDITNAMES)==0 )
  {
    echo 'Permission denied';
    exit;
  }
  if ($dbintf->UseTenantContactList())
  {
    echo 'Not available for tenant contact list';
    exit;
  }
  
  
  $step = 'init';
  $data = '';
  
  if ($_POST['act']=='import')
  {
    LogAction('Import contacts from CSV');
    $csv = $_POST['csv'];
    $importer = new ContactsImporter();
    $dbintf->StartTransaction();
    $importer->Init();
    $importer->ParseCsv($csv);
    $importer->PerformImport();
    $dbintf->CommitTransaction();
    
		$msg[] = new Msg(MSG_INF,'<%Web.ImportDone%>');
		$_SESSION['msg'] = $msg;
    header('Location: '.'contacts.php');
    exit;
  }

  if (isset($_FILES['thelist']))
  {
    $upl = $_FILES['thelist'];
    if ($upl['error']!=0)
    {
      // error in upload
      $msg[] = new Msg(MSG_ERR,'Error in file upload. '.GetUploadErrDescr($upl['error']));
    } else
    {
      // there is a good upload
      $csv = file_get_contents($upl['tmp_name']);
      if (IsValidUtf8($csv)==false)
      {
        $data .= "var contacts = [];\n";
        $data .= "var csv = \"\";\n";
      } else
      {
        $importer = new ContactsImporter();
        $dbintf->StartTransaction();
        $importer->Init();
        $importer->ParseCsv($csv);
        $dbintf->CancelTransaction();
        
        $data .= 'var contacts = ' . json_encode($importer->contacts) . ";\n";
        $data .= 'var csv = "' . JSQuote($csv) . "\";\n";
      }
      
      $step = 'verify';
    }
  }

	LogAction("Viewed contacts import.",array("level"=>"view"));

  // ready with database
  $dbintf->close();

  if ($step=='init')
  {
    $template_content = file_get_contents('../apresa/templ/contacts_import.htm');
  }
  if ($step=='verify')
  {
    $template_content = file_get_contents('../apresa/templ/contacts_import_verify.htm');
  }
  
  $data .= GeneralJSData();
	
  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}

class ContactsImporter
{
  var $linemap;
  var $contacts;
  var $lineIx;

  function Init()
  {
    global $dbintf;
    $ctlines = $dbintf->GetLineList();
    $this->linemap = array();
    foreach ($ctlines as $ctline)
    {
      $this->linemap[$ctline['linenr']] = $ctline['contactid'];
    }
  }
  
  function ParsePhoneItem($phone_item)
  {
    global $dbintf;
    $obj = array();
    if (substr($phone_item,0,3)=='IP#')
    {
      $ipaddr =  substr($phone_item,3);
      // $obj['contactid'] = $dbintf->GetIpAddr($ipaddr);
      // $obj['ctype'] = 2;
      // not supported for now,
      // because also not shown in contact list
      $obj['err'] = 'Import of IP address contact is not supported';
    } else if (substr($phone_item,0,5)=='LINE#')
    {
      $telline = substr($phone_item,5);
      if (isset($this->linemap[$telline])==false)
      {
        $obj['err'] = 'Line '.$telline.' not found';
        return $obj;
      }
      $obj['contactid'] = $this->linemap[$telline];
      $obj['ctype'] = 1;
    } else
    {
      $ret = $dbintf->GetCallerIDonCallerID($phone_item,false);
      if($ret !== false)
      {
        $obj['contactid'] = $ret['contactid'];
      }
      $obj['ctype'] = 0;
    }
    return $obj;
  }
  
  function AddLineArray($data)
  {
    global $dbintf, $msg;
    
    $this->lineIx ++;
    if ($this->lineIx==1) return;
    
    if(count($data) < 2) return;

    $phone_item = $data[0];
    $name = $data[1];
    $islocal = false;
    if (count($data) > 2) $islocal = (bool) $data[2];
    if ($phone_item=='') return;
    if ($name=='') return;
    if (IsValidUtf8($phone_item)==false) return;
    if (IsValidUtf8($name)==false) return;

    $oldname = '';
    $contactid = -1;
    $action = 'add';
    $iden = $phone_item;
    $obj = $this->ParsePhoneItem($phone_item);
    if ( isset($obj['err'])==true )
    {
      $msg[] = new Msg(MSG_ERR,$obj['err']);
      return;
    }
    if (isset($obj['contactid'])) $contactid = $obj['contactid'];
    else $contactid = -1;
    $ctinfo = $dbintf->GetContactInfo( $contactid );
    // ctinfo can be false, then it is a new item!
    if ($ctinfo !== false)
    {
      $oldname = $ctinfo['name'];
      $action = 'edit';
    }
    
    $this->contacts[] = array
    ( 'iden' => $iden // displayed as identifier
    , 'oldname' => $oldname
    , 'name'   => $name
    , 'contact' => $contactid
    , 'action' => $action
    , 'islocal' => $islocal
    , 'ctype' => $obj['ctype']
    , 'cid' => $ctinfo ? $ctinfo['cidid'] : null // ID of caller IDs (or NULL)
    );
  }
  
  function ParseCsv($csv)
  {
    $this->contacts = array();
    $this->lineIx = 0;
    $csvParser = new CsvParser();
    $line = strtok($csv,"\n");
    while ($line !== false)
    {
      $csvParser->addLine( $line . "\n" );
      $data = $csvParser->getData();
      if ($data !== null)
      {
        $this->AddLineArray( $data );
      }
      $line = strtok("\n");
    }
    $csvParser->flush();
    $data = $csvParser->getData();
    if ($data !== null)
    {
      $this->AddLineArray( $data );
    }      
    // debug: var_dump($this->contacts);
  }
  
  function PerformImport()
  {
    global $dbintf;
    foreach ($this->contacts as $contact)
    {
      $ctype = $contact['ctype'];
      $ctid = $contact['contact'];
      if ($ctype==0) // caller ID
      {
        if ($ctid == -1)
        {
          // it is a new contact
          $cid2 = '';
          $islocal = $contact['islocal']?1:0;
          $dbintf->GetOrCreateCallerID($contact['iden'],$cid2,$islocal,$contact['name']);
        } else
        {
          // it is an existing contact
          $dbintf->EditContact2($ctid,$contact['cid'],$contact['islocal']?1:0,$contact['name']);
        }
      } else if ($ctype==1) // line
      {
        if ($ctid == -1) continue; // adding lines not support
        $dbintf->EditContact($ctid,$contact['name']);
      }
    }
    
  }
  
}



?>
