<?php
  require_once('general.php');
  require_once('utility.php');
  require_once('signaling.php');

	HandleED137($dbintf);


  function setting($default = null, $check = null)
  {
    return [
      "default" => $default,
      "check" => $check
    ];
  }

  function CheckPort($port)
  {
    $port = int_ensure_defined($port);
    return $port > 1 && $port < 65536;
  }

  function CheckNum($val)
  {
    return is_numeric($val);
  }

	function HandleED137($dbintf)
	{
  	global $rpl,$rooturl,$msg,$lng,$configs;

    $config_keys = [
      "RtspTcp" => setting("1"), 
      "RtspTcpPort" => setting("554", "CheckPort"), 
      "RtspUdp" => setting("1"), 
      "RtspUdpPort" => setting("554", "CheckPort"),
      "ED137Service" => setting("0"),
      "RtpUdp" => setting("1"),
      "RtpUdpMinPort" => setting("32768", "CheckPort"),
      "RtpUdpMaxPort" => setting("49152", "CheckPort"),
      "RtpTcp" => setting("1"),
      "RtpTcpMinPort" => setting("32768", "CheckPort"),
      "RtpTcpMaxPort" => setting("49152", "CheckPort"),
      "RtpInterleaved" => setting("1"),
      "Rtcp" => setting("1"),
      "PausedRecordingTimeout" => setting("5", "CheckNum"),
      "MultiUrl" => setting("0"),
      "SplitDurationImmediate" => setting("0"),
      "DeriveRtpPortRemote" => setting("0"),
      "OpAsAnnotations" => setting("1"),
			"AlertingNr" => setting("none"),
			"BSSMethod" => setting("none"),
			"CallRef" => setting("none"),
			"CallType" => setting("none"),
			"ClientID" => setting("none"),
			"ClientType" => setting("none"),
			"DisconnectCause" => setting("none"),
			"DisconnectReason" => setting("none"),
			"DisconnectSource" => setting("none"),
			"Priority" => setting("none"),
      "Path" => setting("none")
    ];

    if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0)
    {
      echo 'Permission denied';
      exit;
    }
 
	  if (!$configs['has_ED137'])
	  {
      // redirect
      $_SESSION['msg'] = $msg;
      header('Location: '.$rooturl);
      exit;
	  }

    if(isset($_POST['act']))
    {
      $act = $_POST['act'];
      if($act == "save-settings")
      {
        $new_configs = [];
        $okay = true;

        foreach($config_keys as $key => $setting)
        {
          if(isset($_POST[$key]))
          {
            if(isset($setting["check"]))
            {
              $okay = $setting["check"]($_POST[$key]);
              if(!$okay) break;
            }

            $new_configs[$key] = $_POST[$key];
          }
        }

        if($okay)
        {
          $dbintf->SetConfigsPart($new_configs, 'ed137');
          if(msg_signal_service($msg, "as"))
          {
            $msg[] = new Msg(MSG_INF, "Settings stored, restart the ED137 to apply them");
          }
        }
        else
        {
          $msg[] = new Msg(MSG_ERR, "Some setting was invalid");
        }
      }
      else if($act = "restart")
      {
        msg_signal_service($msg, "ar");
      }

      $_SESSION['msg'] = $msg;
      header('Location: '.$rooturl . "/ed137.php");
			exit;
    }

    $configs_ed137 = $dbintf->LoadConfigsPart('ed137');
    $checks = [];
    
    // Load defaults
    foreach($config_keys as $key => $setting)
    {
      if(isset($setting['default']) && !isset($configs_ed137[$key]))
      {
        $configs_ed137[$key] = $setting['default'];
      }

      if(isset($setting["check"]))
      {
        $checks[$key] = $setting["check"];
      }
    }

    $data = '';
    $data .= 'var configs_ed137 = ' . json_encode($configs_ed137) . ";\r\n\t";
    $data .= 'var checks_ed137 = ' . json_encode($checks) . ";\r\n";

		$data .= GeneralJSData();

		$template_content = file_get_contents('../apresa/templ/ed137.htm');

		$rpl['msg'] = message($msg);
		$rpl['nav'] = '';
		$rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
		$rpl['FOOTER'] = '';
		$rpl['data'] = $data;
		$rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  	$rpl['header'] = GetHeader();
		$template_content = do_replace_tags($template_content);
		echo $template_content;
	}
?>