<?php
require_once "p/lib/Swift/swift_required.php";
require_once "encryption.php";

// replacing one function
class Vidi_Swift_Message extends Swift_Message
{
  protected function getRandomId()
  {
    $l = time() . '.' . uniqid();
    if (empty($_SERVER['SERVER_NAME']))
    {
      $r = 'apresa.generated';
    } else
    {
      // replace port sign (:) with underscore
      // Otherwise fails RFC test
      $r = str_replace(':','_',$_SERVER['SERVER_NAME']);
    }
    //return $l . '@' . $r;
    return $l . '@' . 'apresa.generated';
  }
}

function GetEmailSubjPrefix($dbintf)
{
  $prefix = 'APRESA ';
  $sysname = $dbintf->GetConfigValue('SystemName');
  if ($sysname!='')
  {
    $prefix .= $sysname . ' ';
  }
  $serial = (int) file_get_contents( '/var/apresa/conf/serial' );
  $prefix .= '[SN '.$serial.'] ';
  return $prefix;
}

// - From-address is taken from configuration
// - To-address is an array,
//       like array('b@cccc.com','john@a.com'=>'John',...)
// - Subject: text
// - Body: text
// Returns the number of emails sent
function SendEmail($configs,$to,$subject,$body,$attachfns)
{
	if ($to=='')
	{
		lg('Not sending email: Addressee is not specified');
		return false;
	}
	if ($configs['EmailConnectType']=='none')
  {
		lg('Email is disabled');
		return false;
  }
  if ($configs['SMTPhost']=='')
	{
		lg('Email is not configured.');
		return false;
	}
  /*
  $body = 'subject';
  $attachfns = array();
  echo '<pre>';
  echo 'PWD: '.decrypt($configs['SMTPpwd'])."\n";
  echo 'USR: '.$configs['SMTPusr']."\n";
  echo $body."\n";
  echo 'To: '.$to."\n";
  var_dump($attachfns);
  echo 'Subject: '.$subject."\n";
  echo $configs['SMTPfrom']."\n";
  echo $configs['SMTPhost']."\n";
  echo $configs['SMTPport']."\n";
  */
  //$msg = Swift_Message::newInstance();
  $msg = new Vidi_Swift_Message();
  //echo $msg->getId() . "\n";
  //exit;
  $msg->setSubject($subject);
  $msg->setFrom(array($configs['SMTPfrom']));
  $msg->setTo($to);
	// Because of Swift bug,
	// we must make sure there are no invalid UTF-8 character,
	// otherwise strange duplicated content.
	mb_substitute_character(0xFFFD);
	$body = mb_convert_encoding($body,'UTF-8','UTF-8');
  $msg->setBody($body);
  foreach ($attachfns as $attachfilename)
  {
    $msg->attach(Swift_Attachment::fromPath($attachfilename));
  }

  $transport = Swift_SmtpTransport::newInstance
  ( $configs['SMTPhost']
  , $configs['SMTPport']
  , $configs['SMTPenc']
  );
  $transport->setUsername($configs['SMTPusr']);
  $transport->setPassword(decrypt($configs['SMTPpwd']));

  $mailer = Swift_Mailer::newInstance($transport);

	return TrySendEmail($mailer,$msg);
}


function SendEmailData($configs,$dat)
{
	// remove empty addressees
	// and convert to an array
	$newto = array();
  if (is_string($dat['To']))
  {
    $dat['To'] = explode(',',$dat['To']);
  }
  foreach ($dat['To'] as $item)
  {
    if ($item!='') $newto[] = $item;
  }
	$dat['To'] = $newto;
	if (count($dat['To'])==0)
	{
		lg('Not sending email: Addressee is not specified');
		return false;
	}
  $connectType = ensure_defined($configs['EmailConnectType'], 'relay');

	if ($connectType=='none')
  {
		lg('Email is disabled');
		return false;
  }

  if ( $connectType == 'relay'
  && $configs['SMTPhost']=='')
	{
		lg('Email is not configured.');
		return false;
	}

  if(PHP_VERSION_ID > 80100)
  {
    require_once "emailingsymfony.php";
    return SendEmailDataSymfony($configs, $dat);
  }

  $msg = new Vidi_Swift_Message();
  $msg->setSubject($dat['Subject']);
  $msg->setFrom(array($configs['SMTPfrom']));
  $msg->setTo($dat['To']);

  // images
  $toreplace = array();
  for ($i=1;$i<=100;$i++)
  {
    $key = 'Embed_' . $i;
    if (isset($dat[$key])==false) break;
    $id = $msg->embed(Swift_Image::fromPath($dat[$key]));
    $key = $key . 'tag';
    $toreplace[ $dat[$key] ] = $id;
  }

  if (isset($dat['HtmlBody']))
  {
    $htmlbody = $dat['HtmlBody'];
    foreach($toreplace as $from=>$to)
    {
      $htmlbody = str_replace($from,$to,$htmlbody);
    }
		// Because of Swift bug,
		// we must make sure there are no invalid UTF-8 character,
		// otherwise strange duplicated content.
		mb_substitute_character(0xFFFD);
		$htmlbody = mb_convert_encoding($htmlbody,'UTF-8','UTF-8');
    $msg->setBody($htmlbody,'text/html');
    $msg->addPart($dat['Body'],'text/plain');
  } else {
    $msg->setBody($dat['Body'],'text/plain');
  }
  for ($i=1;$i<=100;$i++)
  {
    $key = 'Attach' . '_' . $i;
    if (isset($dat[$key])==false) break;
    $msg->attach(Swift_Attachment::fromPath($dat[$key]));
  }

  if (defined('EMAILQ'))
  {
		lg('EmailQueue type '.EMAILQ);
		$transport = Swift_SendmailTransport::newInstance('/usr/sbin/sendmail -t');
	} else
	{
	  $transport = Swift_SmtpTransport::newInstance
	  ( $configs['SMTPhost']
	  , $configs['SMTPport']
	  , $configs['SMTPenc']
	  );
  	$transport->setUsername($configs['SMTPusr']);
  	$transport->setPassword(decrypt($configs['SMTPpwd']));
	}

  $mailer = Swift_Mailer::newInstance($transport);

  return TrySendEmail($mailer,$msg);
}

function TrySendEmail(&$mailer,&$msg)
{
	$retry = 0;
	while(true)
	{
    $ok = false;
    try
    {
			$ret = $mailer->send($msg);
			$ok = true;
    } catch(Exception $e)
    {
			lg('Email failed with exception: '.$e);
    }
    if ($ok) break;
    else
    {
      $retry ++;
      if ($retry>5)
      {
        $ret = false;
        break;
      }
      // sleep for one second
      sleep(1);
    }
	}
  return $ret;
}

function EmailingWebOrCmd($dat)
{
  global $dbintf;
  $configs_n = $dbintf->LoadConfigsPart('n');
  if (defined('CMD'))
  {
    //echo "Directly emailing\n";
    return SendEmailData($configs_n,$dat);
  } else
  {
    // we dont support arrays in $dat[X]
    if (is_array($dat['To']))
    {
      $dat['To'] = implode(',',$dat['To']);
    }
    return EmailingFromWebIntf($dat);
  }
}
