<?php

require_once '../apresa/conf/voip_config.php';
require_once 'datetime.php';
require_once 'db.php';
require_once 'signaling.php';
require_once 'encryption.php';
require_once 'actions.php';
require_once 'utility.php';
require_once 'web-utils.php';

// general initialisation
ini_set( 'session.cookie_httponly', 1 );
if (isset($_SERVER['HTTPS']))
{
  ini_set( 'session.cookie_secure', 1 );
}
// protect against clickjacking
header("Content-Security-Policy: frame-ancestors 'self'");

if (defined('DEMO'))
{
  if (function_exists('get_magic_quotes_gpc'))
  {
    // for DEMO system, deal with this setting
    // it should have been off
    if (get_magic_quotes_gpc()) 
    {
        function stripslashes_gpc(&$value)
        {
            $value = stripslashes($value);
        }
        array_walk_recursive($_GET, 'stripslashes_gpc');
        array_walk_recursive($_POST, 'stripslashes_gpc');
        array_walk_recursive($_COOKIE, 'stripslashes_gpc');
        array_walk_recursive($_REQUEST, 'stripslashes_gpc');
    }
  }
}

if (file_exists(DATA_PATH . '/busy'))
{
	$busy_act = trim(file_get_contents(DATA_PATH . '/busy'));
	$rooturl = (isset($_SERVER['HTTPS'])?'https':'http').'://'.$_SERVER['HTTP_HOST'].rtrim(dirname($_SERVER['PHP_SELF']),'/\\').'/';
	header('Refresh: 2; URL=' . $rooturl);
	if ($busy_act=='restore')
	{
		$text = 'Please wait while the database is restored from the backup...';
	} else if ($busy_act=='update')
	{
		$text = 'Please wait while the software update is installed...';
	} else
	{
		$text = 'Please wait while the system action ('.$busy_act.') is performed...';
	}
	echo '<br><br><br><table cellpadding="10" align="center"><tr><td><img src="p/yui/assets/skins/sam/ajax-loader.gif"><td>' . $text . '</tr></table>';
	exit;
}

if (file_exists(dirname(__FILE__).'/p/general.css')==false)
{
	// generate from current theme
	lg('need to generate css');
	signal_service('makecss');
}

BasicConstants();
InitDB();
LoadBasicSettings();

$readonly = $dbintf->readonly;

function IsUrlAvailable($url)
{
	$context = stream_context_create(
		array
		( 'http' => array
			(	'method' => 'HEAD'
			, 'timeout' => 3.0
			)  
		, 'ssl'  => array
			( 'verify_peer' => false
			, 'verify_peer_name' => false
			)
		)
	);
	$ct = @file_get_contents($url,null,$context);
	$isok = false;
	if (isset($http_response_header))
	{
		foreach ($http_response_header as $header)
		{
			if (substr($header,0,5)=='HTTP/')
			{
				list(,$code) = explode(' ', $header);
				$first_digit = substr($code,0,1);
				// 2xx and 3xx are OK
				if ($first_digit=='2' || $first_digit=='3')
				{
					$isok = true;
				}
			}
		}
	}
	return $isok;
}

// messages
$msg = array();

$configs_s = $dbintf->LoadConfigsPart('s');
if (int_ensure_defined($configs_s['Https'])==2)
{
  if (isset($_SERVER['HTTPS'])==false)
  {
		if (IsUrlAvailable('https://127.0.0.1'))
		{
	    // with fixed HTTPS
	    $rooturl = 'https://'.$_SERVER['HTTP_HOST'].rtrim(dirname($_SERVER['PHP_SELF']),'/\\').'/';
	    header('Location: '.$rooturl);
	    exit;
		} else
		{
			$msg[] = new Msg(MSG_ERR,"HTTPS required but not available.");
		}
  }
}

// template replacer
$rpl = array();

$rpl['prodname'] = 'Call Recorder Apresa';
if (isset($configs['ProductName']))
{
	$rpl['prodname'] = $configs['ProductName'];
}

if ($dbintf->nodePassive)
{
	$msg[] = new Msg(MSG_INF,"This node is in stand-by mode (read-only)");
}

// Find root
$rooturl = (isset($_SERVER['HTTPS'])?'https':'http').'://'.$_SERVER['HTTP_HOST'].rtrim(dirname($_SERVER['PHP_SELF']),'/\\').'/';
if (isset($CLIENT))
{
	$rooturl .= 'client.php';
}

// login system
$UseSessions = true;
if (isset($_GET['password']) || ensure_defined($NOSESSIONS,false) )
{
	$UseSessions = false;
}

if ($UseSessions)
{
	session_start();
  if (isset($_SESSION['token'])==false)
  {
    // generate a token
    if (function_exists('random_bytes'))
    {
      $SessionToken = bin2hex( random_bytes(22) );
    } else
    {
      require_once( __DIR__ . '/p/lib/phpseclib/Crypt/Random.php' );
      $SessionToken = bin2hex(phpseclib\Crypt\Random::string(20));
    }
    $_SESSION['token'] = $SessionToken;
  }
  if ( isset($_SESSION['uid'])
    && (empty($_POST)==false || empty($_FILES)==false) )
  {
    // for every POST, when logged in, there must be a token supplied
    if ( $_POST['SessionToken'] != $_SESSION['token'] )
    {
      // except when trying to relogin when already logged on
      if (isset($_SESSION['uid'])
        && empty($_FILES)
        && count($_POST)==2
        && isset($_POST['username']) && isset($_POST['password']) )
      {
        // this is to allow a fresh login through an app
        // that doesnt know if logged in already or not,
        // to go through and yield a workable page
      }
			elseif(isset($_SESSION['uid'])
        && empty($_FILES)
        && count($_POST)==2 
				&& isset($_POST['verificationCode'])
				&& (isset($_POST['LoginBtn']) || isset($_POST['CancelBtn'])))
      {
				//Verification code is necessary!
			}
			else
      {
        echo 'ERROR: Session token invalid or missing';
        exit;
      }
    }
  }
	if (str_ensure_defined($_POST['act'])=='logout')
	{
		if (isset($_SESSION['uid']))
		{
			$dbintf->SetCurrentUserId($_SESSION['uid']);
			LogAction('Log out');
		}
		
		// delete ALE session playback key if present
		if (isset($_SESSION['PlayKeyId']))
		{
			$playKeyFn = '/dev/shm/apresa/web/play_'.$_SESSION['PlayKeyId'].'.ekey';
			DeleteFn($playKeyFn);
			$dbintf->RemoveTmpFile($playKeyFn);
		}
    
    if (SsoHasLoggedIn())
    {
      include('sso.php');
      SsoLogout();
      // this line is not reached because of redirect to ADFS
      exit;
    }

	  $dbintf->DeleteSession(session_id());
	  end_of_session();
    
	  // redirect to remove post action
	  if (!isset($CLIENT))
		{
      $target = $rooturl;
      if (SsoIsEnabled() && GetSsoOneAction()=='redirect')
      {
        // to avoid immediate redirect to ADFS
        $target .= '?nosso';
      }
	  	header('Location: '.$target);
	 	}
	  exit;
	}
}

// cnt: number of pages visited in this session
/*
if (!isset($_SESSION['cnt']))
{
  $_SESSION['cnt'] = 0;
} else {
  $_SESSION['cnt'] ++;
}
*/

// A delay, because otherwise a DB update is made for each page visit, which is not nice
if ($UseSessions)
{
	$SES_UPD_DELAY = 5*60;

	if (isset($_SESSION['uid']))
	{
	  // we are logged in
	  // check expiry
	  // total session must not be too long
	  // also not too much between accesses (leaving the browser open)
	  if ( (time() - $_SESSION['first_access'] > $configs['WebMaxTime'] * 60)
	  || (time() - $_SESSION['last_access' ] > $configs['WebMaxIdle'] * 60)  )
	  {
	    $dbintf->DeleteSession(session_id());
	    end_of_session();
	  } else if (isset($NO_LAST_ACCESS_UPDATE)==false) {
      $_SESSION['last_access' ] = time();
	    if ( int_ensure_defined($_SESSION['last_access_db' ])+$SES_UPD_DELAY < time() )
	    {
	      $_SESSION['last_access_db' ] = time();
	      $dbintf->UpdateSession(session_id(),$_SESSION['last_access_db' ]);
	    }
	  }
	}
}

if (isset($_GET['password']))
{
	$username = str_ensure_defined( $_GET['username'] );
	$password = str_ensure_defined( $_GET['password'] );
  $pcname   = str_ensure_defined( $_GET['pcname'  ] );
} else
{
	$username = str_ensure_defined( $_POST['username'] );
	$password = str_ensure_defined( $_POST['password'] );
  $pcname   = str_ensure_defined( $_POST['pcname'  ] );
}

$act = str_ensure_defined($_POST['act']);
if ($act!='')
{
	if ($act=='forgot-pw' && int_ensure_defined($configs['PwdRecover'])!=0)
	{
		//The user needs a new password!
		if (isset($_POST['usrname']))
		{
			$minpwdlength = int_ensure_defined($configs['MinPwdLength']);
			$mustchangepwd = int_ensure_defined($configs['MustChangePwd']);
			$usrpwd = generateStrongPassword($minpwdlength, 'lud');
			if ($dbintf->NewPassword($_POST['usrname'], DBInterface::CreatePwdHash($usrpwd), $mustchangepwd, $emailaddress, $friendlyname))
			{
				if ($emailaddress == '') $msg[] = new Msg(MSG_ERR, $lng->Get('Profile.UserHasNoEmail'));
				else
				{
					//Send the password to the user!
					$dat = array();
					$dat['To'] = $emailaddress;
					$dat['Subject'] = $lng->Get('Profile.NewPwd');
					if ($friendlyname == '')
					{
						$dat['Body'] = $_POST['usrname'] . "\r\n\r\n" . $lng->Get('Profile.NewPwd') . ': ' . $usrpwd;
					}
					else
					{
						$dat['Body'] = $friendlyname . "\r\n\r\n" . $lng->Get('Profile.NewPwd') . ': ' . $usrpwd;
					}
					EmailingFromWebIntf($dat);
					$msg[] = new Msg(MSG_INF, $lng->Get('Profile.NewPwdSent'));
				}
			}
			else
			{
				$msg[] = new Msg(MSG_ERR, $lng->Get('PM.UsrMissing'));
			}
		}
	}
}

if ($UseSessions)
{
	if (!isset($_SESSION['uid']))
	{
	  if ($readonly==false && $dbintf->GetSessionCount()>=$configs['WebMaxCnt'])
	  {
	    // check if any expired
	    $sessions = $dbintf->GetSessions();
	    foreach ($sessions as $session)
	    {
	      if ( (time() - $session['firstacc'] - $SES_UPD_DELAY > $configs['WebMaxTime'] * 60)
	      ||   (time() - $session['lastacc' ] - $SES_UPD_DELAY > $configs['WebMaxIdle'] * 60)  )
	      {
	        $dbintf->DeleteSession($session['sessionid']);
	      }
	    }
	  }
	  if ($readonly==false && $dbintf->GetSessionCount()>=$configs['WebMaxCnt'])
	  {
	    $msg[] = new Msg(MSG_ERR,"Maximum web users reached.");
	  } else
	  {
			$uid = -1;
      if ($username!='' && str_ensure_defined($_POST['stage'])=='usrsel')
      {
        $userInfo = $dbintf->GetUserInfoOnUsername( $username );
        if ($userInfo == false)
        {
          $msg[] = new Msg(MSG_ERR, 'The user was not found in Apresa' );
          sleep(1);
          $_SESSION['msg'] = $msg;
          header('Location: ' . $rooturl);
          exit;
        }
        if ( (int) $userInfo['logonmethod'] != 2)
        {
          // go to normal user login page with username already filled in
          $_SESSION['UserName'] = $username;
          // fall through
        } else
        {
          // logon using SSO identity provider for username
          $ssoId = $dbintf->GetUserIdp($userInfo);
          if ($ssoId === false)
          {
            // maybe there is only one
            $idps = $dbintf->GetExtIdPs();
            if (count($idps)==0)
            {
              $msg[] = new Msg(MSG_ERR, 'No external identity provider is configured' );
              $_SESSION['msg'] = $msg;
              header('Location: ' . $rooturl);
              exit;
            }
            if (count($idps)==1)
            {
              $ssoId = $idps[0]['id'];
            }
          }
          
          if ($ssoId === false)
          {
            $msg[] = new Msg(MSG_ERR, 'Could not determine which external identity provider to use' );
            $_SESSION['msg'] = $msg;
            header('Location: ' . $rooturl);
            exit;
          }
          include('sso.php');
          SsoLogin( $ssoId );
        }
			} else if ($username!='')
			{
				if (GenTryLogin($dbintf, $username, $password, $pcname, $uid))
				{
          LogInUidUsr($uid,$username);
				} else
				{
          // log failure in audit trail
          $uinfo = $dbintf->GetUserInfoOnUsername($username);
          // we need to sanitize the username
          if ($uinfo!==false)
          {
            $logusr = $uinfo['usr'];
            LogAction('Failed login',array(),$logusr);
          } else
          {
            LogAction('Failed login for '.$username,array(),'_system_');
          }
					sleep(1);
          
					// redirect to prevent F5 to submit again the usr/pwd.
					$_SESSION['msg'] = $msg;
          $targeturl = $rooturl;
          // this is after failed local login
          header('Location: '.$targeturl . '?nosso');
					exit;
				}
			} else
			{
				// no username specified
			}
	  } // end block: session max not reached
	} else
	{
		// already logged in
		$uid = $_SESSION['uid'];
	}
} else
{
	$uid = -1;
	if ($username!='' && GenTryLogin($dbintf, $username, $password, $pcname, $uid))
	{
		// nothing to do
	} else
	{
		if (isset($CLIENT)==false)
		{
			echo 'Login failed';
			exit;
		}
	}
}

if ($UseSessions)
{
	// because of redirect,
	// some messages might be stored in the session obj
	if (isset($_SESSION['msg']))
	{
	  $msg = $_SESSION['msg'];
	  unset($_SESSION['msg']);
	}
}

if ($UseSessions)
{
	if (!isset($_SESSION['uid']))
	{
		if (@file_exists('/var/apresa/install/inithw.php'))
		{
			echo 'First remove apresa-install package!';
			exit;
		}
	  if (defined('NFR') && NFR!=0)
	  {
	    $msg[] = new Msg(MSG_WRN,'Not For Resale Version');
	  }
		if (isset($CLIENT)==false && isset($NO_LOGIN_PAGE)==false)
		{
			if (strstr($_SERVER['REQUEST_URI'],'/getcall/')!==FALSE)
			{
				// this is seen as subdirectory, and otherwise cause template error
        header('Location: '.$rooturl);
				exit;
			}

			// store original request,
			// so we may redirect to it later after login
			// but only for pages, not for async of image phps
			if ( IsPageRequested() )
			{
				$_SESSION['OriRequest'] = $_SERVER['REQUEST_URI'];
			} else
			{
				header('HTTP/1.1 403 Forbidden');
				exit;
			}
      
      $do_sso = SsoIsEnabled();
      if ($do_sso && str_ensure_defined($_POST['stage'])=='initlogin')
      {
        // select the first SSO
        $idps = $dbintf->GetExtIdPs();
        $ssoId = $idps[0]['id'];
        include('sso.php');
        SsoLogin( $ssoId );
        exit;
      }
      
      if ($do_sso && $username=='' && isset($_GET['nosso'])==false)
      {
        include('sso.php');
        $idps = $dbintf->GetExtIdPs();
        $ssoAction = 'askusr';
        if (count($idps)==1)
        {
          $ssoAction = GetSsoOneAction();
          if (isset($_GET['loggedout']))
          {
            if ($ssoAction=='redirect')
            {
              // to avoid immediate login after logout
              // to avoid loop on failure
              $ssoAction = 'confirm';
            }
          }
        }
        if ($ssoAction=='confirm')
        {
          SsoLoginPage(false);
        } else if ($ssoAction=='askusr')
        {
          SsoLoginPage(true);
        } else if ($ssoAction=='redirect')
        {
          // select the first SSO
          $ssoId = $idps[0]['id'];
          SsoLogin( $ssoId );
        }
        exit;
      }
      
			$extrainfo = '';
			if (defined('DEMO'))
			{
				if (file_exists('../apresa/templ/custom/demologin_custom.htm'))
				{
					$extrainfo = file_get_contents('../apresa/templ/custom/demologin_custom.htm');
				} else
				{
					$extrainfo = file_get_contents('../apresa/templ/demologin.htm');
				}
			}
			if (file_exists('../apresa/templ/custom/login_custom.htm'))
			{
		  	$template_content = file_get_contents('../apresa/templ/custom/login_custom.htm');
			} else
			{
		  	$template_content = file_get_contents('../apresa/templ/login.htm');
		  }
			
			if (int_ensure_defined($configs['NoLoginPage'])!=0)
			{
				$template_content = file_get_contents('../apresa/templ/nologin.htm');
				$msg[] = new Msg(MSG_WRN, 'Session ended. To continue, login again.');
			}
			
			//Forgot password link!
			$data = '';
      $usr = '';
      if (isset($_SESSION['UserName'])) $usr = $_SESSION['UserName'];
      
			if (isset($_SESSION['UserName']) && int_ensure_defined($configs['PwdRecover'])!=0)
			{
				$numberofwrongpwd = $dbintf->WrongPassword($usr, -1, 0);	//-1 is just the number, don't change anything. 0=$MaxPwdFailures (not important here)
				if ($numberofwrongpwd == 0 ) $data .= "var showlink=0;\r\n";
				else
				{
					if ($dbintf->IsAccountTempDisabled($usr))
						$data .= "var showlink=0;\r\n";
					else
						$data .= "var showlink=1;\r\n";
				}
				//The username was ok!
			}
			else
			{
				$data .= "var showlink=0;\r\n";
			}
			if ($usr=='' && isset($_GET['usr']))
			{
				$usr = $_GET['usr'];
			}
      $data .= "var username=\"" . JsQuote($usr) . "\";\r\n";
			$data .= "var loginPageMessage=\"" . JsQuote(str_ensure_defined($configs['LoginPageMsg'])) . "\";\r\n";
			$data .= GeneralJSData();

      CologonLoginPageMsg();
      
		  // template replacements:
		  $rpl['msg'] = message($msg);
		  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
		  $rpl['FOOTER'] = '';
		  $rpl['INFO'] = $extrainfo;
			$rpl['data'] = $data;
		  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
		  $template_content = do_replace_tags($template_content);
		  echo $template_content;
	  	//echo $_SERVER['PHP_SELF'];
	  	exit;
		}
	} else
	{
		if(isset($_SESSION['LoginVerificationEnabled']))					//Logon verification just enabled.
		{
			$uinfo = $dbintf->GetUserInfo($_SESSION['uid']);				//Check for an email address!
			$emailaddress="";				
			if ($uinfo!==false) $emailaddress = $uinfo['email'];		//Get the email address.
			if($emailaddress == "")																	//No email address. 
			{
				$msg[] = new Msg(MSG_WRN, "You don't have a valid e-mail address. Please take care of this first!");
				unset($_SESSION['LoginVerificationEnabled']);
				$dbintf->SetConfigsPart(array('LoginVerification'=>0),'web');
				$configs['LoginVerification']=0;
			}
		}
		if(isset($_POST['CancelBtn']) && isset($_SESSION['LoginVerificationEnabled']))			//Cancel clicked after enabling Login Verification.
		{
			$dbintf->SetCurrentUserId($_SESSION['uid']);
			if(($dbintf->actuinfo['permissions']&PM_CONFIG) !=0 )		//Just to be sure.
			{
				if(isset($_SESSION['veriCode'])) unset($_SESSION['veriCode']);
				if(isset($_SESSION['LoginVerificationEnabled'])) unset($_SESSION['LoginVerificationEnabled']);
				if(isset($_SESSION['VeriEmailOk'])) unset($_SESSION['VeriEmailOk']);
				$dbintf->SetConfigsPart(array('LoginVerification'=>0),'web');
				$configs['LoginVerification']=0;
			}
		}
		// we are logged in:
		if(int_ensure_defined($configs['LoginVerification'])!=0 && !isset($_SESSION['VerificationChk']))	//$_SESSION['VerificationChk']: only check once.
		{
			if(!isset($_POST['verificationCode']))
			{
				$template_content = file_get_contents('../apresa/templ/login_verify.htm');
				//The username was ok!
				$uinfo = $dbintf->GetUserInfo($_SESSION['uid']);
				$emailaddress="";
				if ($uinfo!==false)
				{
					$usr = $uinfo['usr'];
					$friendlyname = $uinfo['name'];
					$emailaddress = $uinfo['email'];
				}

				if(!isset($_SESSION['VeriEmailOk']))	//Do not email again after a refresh.
				{
					//Generate the verification code
					$vericode = generateVerification(6);	//6 digits
					//Save the generated code
					$_SESSION['veriCode']=$vericode;
					//Send the verification code to the user! Use the template.
					$key = 'EmailTemplate' . 'VeriCode';
					$subject = $dbintf->GetConfigValue($key . 'Subject');
					$body    = $dbintf->GetConfigValue($key . 'Body');
					if ($subject=='') $subject = $lng->Get('Web.VerificationCode');
					if ($body=='')
					{
						$body = "\r\n";
						$body .= $lng->Get('Main.Name') . ': <NAME>' . "\r\n";
						$body .= $lng->Get('Main.Username') . ': <USR>' . "\r\n";
						$body .= $lng->Get('Web.VerificationCode') . ': <CODE>' . "\r\n";
						$body .= "\r\n";
					}
					//Replace template tags
					if ($friendlyname == '') $body = str_replace('<NAME>',$usr,$body);
					else $body = str_replace('<NAME>',$friendlyname,$body);
					$body = str_replace('<USR>' ,$usr,$body);
					$body = str_replace('<CODE>' ,$vericode,$body);
					$dat = array();
					$dat['To'] = $emailaddress;
					$dat['Subject'] = $subject;
					$dat['Body'] = $body;
					if($emailaddress != "")
					{
						EmailingFromWebIntf($dat);
						$_SESSION['VeriEmailOk'] = 1;
					}
				}
				
				$data = '';
				if(isset($_SESSION['LoginVerificationEnabled']))
				{
					$data .= "var showcancel=1;\r\n";
				}
				else $data .= "var showcancel=0;\r\n";
				$data .= "var username='" . $usr . "'".";\r\n";
				$data .= GeneralJSData();
				
				// template replacements:
				if($emailaddress == "") $msg[] = new Msg(MSG_WRN, "You don't have a valid e-mail address. Please contact your administrator.");
				else $msg[] = new Msg(MSG_INF, $lng->Get('Web.EnterVeriCode'));
				$rpl['msg'] = message($msg);
				$rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
				$rpl['FOOTER'] = '';
				$rpl['INFO'] = '';
				$rpl['data'] = $data;
        $rpl['headerblock'] = GetHeaderBlock();
				$rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
				$template_content = do_replace_tags($template_content);
				echo $template_content;
				exit;
			}
			else
			{
				$veriCode = $_POST['verificationCode'];
				$uinfo = $dbintf->GetUserInfo($_SESSION['uid']);
				if($uinfo!==false)
				{
					$usr = $uinfo['usr'];
				}
				if(strcmp($veriCode,$_SESSION['veriCode']))
				{
					//Wrong verification code.
          sleep(1);
					$MaxPwdFailures = 5;
					$ret = $dbintf->WrongPassword($usr, 1, $MaxPwdFailures);			//1->increase the number of wrong passwords/verificationcodes!
					if($ret >= $MaxPwdFailures)							//Too much wrong verificationcodes. Start all over with or without temp disabled!
					{
						if(isset($_SESSION['LoginVerificationEnabled'])) unset($_SESSION['LoginVerificationEnabled']);
						if(isset($_SESSION['veriCode'])) unset($_SESSION['veriCode']);
						if(isset($_SESSION['VeriEmailOk'])) unset($_SESSION['VeriEmailOk']);
						if (isset($_SESSION['uid']))
						{
							$dbintf->SetCurrentUserId($_SESSION['uid']);
							LogAction('Too many failed logins');
						}
						$dbintf->DeleteSession(session_id());
						end_of_session();
						if (file_exists('../apresa/templ/custom/login_custom.htm'))
						{
							$template_content = file_get_contents('../apresa/templ/custom/login_custom.htm');
						} else
						{
							$template_content = file_get_contents('../apresa/templ/login.htm');
						}
						$msg[] = new Msg(MSG_ERR, $lng->Get('Profile.AccountDisabled'));
						$data .= "var showlink=0;\r\n";
						$usr = '';
						$extrainfo = '';
						$data .= "var username='';\r\n";
						$data .= GeneralJSData();
						//template replacements:
						$rpl['msg'] = message($msg);
						$rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
						$rpl['FOOTER'] = '';
						$rpl['INFO'] = $extrainfo;
						$rpl['data'] = $data;
						$rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
						$template_content = do_replace_tags($template_content);
						echo $template_content;
						exit;
					}
					else
					{
						$template_content = file_get_contents('../apresa/templ/login_verify.htm');
						$data = '';
						if(isset($_SESSION['LoginVerificationEnabled']))
						{
							$data .= "var showcancel=1;\r\n";
						}
						else $data .= "var showcancel=0;\r\n";
						$data .= "var username='" . $usr . "'".";\r\n";
						$data .= GeneralJSData();
						
						// template replacements:
						$msg[] = new Msg(MSG_ERR,$lng->Get('Web.VeriCodeNOK'));
						$rpl['msg'] = message($msg);
						$rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
						$rpl['FOOTER'] = '';
						$rpl['INFO'] = '';
						$rpl['data'] = $data;
            $rpl['headerblock'] = GetHeaderBlock();
						$rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
						$template_content = do_replace_tags($template_content);
						echo $template_content;
						exit;
					}
				}
				$dbintf->WrongPassword($usr, 0, $MaxPwdFailures);		//0->reset number of wrong passwords!
				$_SESSION['VerificationChk'] = 1;
				if(isset($_SESSION['LoginVerificationEnabled'])) unset($_SESSION['LoginVerificationEnabled']);
				if(isset($_SESSION['veriCode'])) unset($_SESSION['veriCode']);
				if(isset($_SESSION['VeriEmailOk'])) unset($_SESSION['VeriEmailOk']);
			}
		}

		//$dbintf->SetCurrentUserId(1);
		$dbintf->SetCurrentUserId($_SESSION['uid']);
		$actuinfo = $dbintf->GetActiveUserInfo();
    // next block could happen after a system restore
    if (isset($actuinfo['id'])==false)
    {
      $dbintf->DeleteSession(session_id());
      end_of_session();
      header('Location: '.$rooturl);
      exit;
    }
    $shownUsr = $actuinfo['usr'];
    if (isset($_SESSION['CologonUsers']))
    {
      $shownUsr = $_SESSION['CologonUsers'];
    }
		$rpl['username'] = $shownUsr;

		//Load the correct language for the user!
		if ($actuinfo['lng'] != '' && $actuinfo['lng'] != 'def')
		{
			$lng->Load($actuinfo['lng']);
		}
		else
		{
			if (isset($_SESSION['lng']))
			{
				$lng->Load($_SESSION['lng']);
			}
		}

		$act = $_POST['act'];
		if ($act!='')
		{
			if ($act=='sysChangeReported')
			{
				require_once('update-gen.php');
				StoreCurrentSysDescr();
			}
			if ($act=='change-pwd' && $actuinfo['changepwd'])
			{
				if (isset($_POST['password2']))
				{
					//The user had to change his/her password!
					$password2 = str_ensure_defined( $_POST['password2'] );	//New password set!
					if ($dbintf->ChangePassword($actuinfo['id'],$password2))
					{
						//Remove the changepwd flag!
						$dbintf->ResetChangePwdFlag($actuinfo['id']);
						$actuinfo['changepwd'] = 0;
						$actuinfo['pwdchangedate'] = $dbintf->GetPwdChangeDate($actuinfo['id']);
					}
					else
					{
						if ($dbintf->lasterr == 'pwd-used-before')
						{
							$msg[] = new Msg(MSG_ERR, $lng->Get('Profile.PwdReuseErr'));
						}
						else
						{
							$msg[] = new Msg(MSG_ERR, $lng->Get('Profile.NewPwdRejected'));
						}
					}
				}
			}
		}
		
		//Is it time for a password renewal?
		$timediff = time() - $actuinfo['pwdchangedate'];
		
		$renewpwddays = int_ensure_defined($configs['RenewPwdDays']);
		if ($renewpwddays > 0)											//0 = no password change!
		{
			$renewpwdsec = $renewpwddays*24*60*60;		//In seconds!
			if ($timediff > $renewpwdsec) 						//New password.
			{
				$actuinfo['changepwd'] = 1;
        $dbintf->EditUserAccountLC( $actuinfo['id'], array( 'changepwd' => 1 ) );
			}
		}
		
		//Check for new password!
		if ($actuinfo['changepwd'])
		{
			$minpwdlength = int_ensure_defined($configs['MinPwdLength']);
			$strongpwdauth	= int_ensure_defined($configs['StrongPwdAuth']);
			
			$data = '';
			$data .= "var minpwdlength=" . $minpwdlength . ";\r\n";
			$data .= "var strongpwdauth=" . $strongpwdauth . ";\r\n";
			$data .= "var friendlyname=\"" . JsQuote($actuinfo['name']) . "\"".";\r\n";
			$data .= GeneralJSData();
			
			$template_content = file_get_contents('../apresa/templ/changepwd.htm');
		  //template replacements:
		  $rpl['msg'] = message($msg);
		  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
		  $rpl['FOOTER'] = '';
		  $rpl['INFO'] = '';
			$rpl['data'] = $data;
		  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
		  $template_content = do_replace_tags($template_content);
		  echo $template_content;
	  	//echo $_SERVER['PHP_SELF'];
	  	exit;
		}

		/*if ($actuinfo['name']=='')
		{
		  $rpl['loggedinas'] = $actuinfo['usr'];
		} else {
		  $rpl['loggedinas'] = $actuinfo['name'];
		} */

		// one-time check
		if (GetSessionVar('OneTimeCheck')==NULL)
		{
			OneTimeCheck();
		}
		
		if (isset($configs['AEDashOnLogin'])==false) $AEDashOnLogin = 1;
		else $AEDashOnLogin = (int) $configs['AEDashOnLogin'];
    if ($dbintf->actuinfo['permissions']&PM_CE_SUPERVISOR && ($AEDashOnLogin!=0) )
    {
			if (GetSessionVar('CeSupervisorChk')==NULL)
			{
				SetSessionVar('CeSupervisorChk',true);
        if ($_SERVER['PHP_SELF']!='/dashboard.php')
        {
          $_SESSION['msg'] = $msg;
          header('Location: '.$rooturl.'dashboard.php');
          exit;
        }
			}
    }
		
		// Redirect to original request URL.
		// This allow the use of deep links,
		// prompt for usr/pwd, and then go to the specified page.
		if ( isset($_SESSION['OriRequest']) && $_SESSION['OriRequest']!='/' && substr($_SESSION['OriRequest'],0,2)!='/?' )
		{
			// substr is to remove the / at the front,
			// because rooturl already contains that
			$url = $rooturl . substr($_SESSION['OriRequest'],1);
			unset($_SESSION['OriRequest']);
			$_SESSION['msg'] = $msg;
			header('Location: '.$url);
			exit;
		}
    
		if ($dbintf->actuinfo['permissions']&PM_CONFIG)
		{
			// check system changed while service is active
			if (GetSessionVar('ServiceSysChangeChk')==NULL)
			{
				require_once('update-gen.php');
				ServiceSysChangeChk();
				SetSessionVar('ServiceSysChangeChk',true);
			}
			// check cluster setup
			if (GetSessionVar('ClusterSetupChk')==false)
			{
				include 'clustersetup.php';
				CheckClusterSetup();
			}
			// check licensing
			if (isset($_SESSION['LicChk']))
			{
				// Already sufficiently checked
			} else
			{
				// check if licensed
				CheckLicensing();
      }
      
      // show messages to administrator
      ShowMessages('/var/apresa/data/to_say');
    }
    
    ShowMessages('/var/apresa/data/to_say_' . $dbintf->actuinfo['accid']);
	}
} else
{
	$dbintf->SetCurrentUserId($uid);
	$actuinfo = $dbintf->GetActiveUserInfo();
	$rpl['username'] = $actuinfo['usr'];
}
// end of main code, functions follow.....

function ShowMessages($file_path)
{
  global $msg, $lng;
  
  if (@file_exists($file_path))
	{
		$lines = file($file_path);
		foreach($lines as $line)
		{
			$text = trim(substr($line,1));
			if (strlen($text)>1 && $text[0]=='~')
			{
				$text = $lng->Get(substr($text,1));
			}
      // new: also allow regular language tags, so we can have more than one in a single to-say text
      $text = do_replace_tags($text);
			$msg[] = new Msg((int)$line[0],$text);
		}
		if (unlink($file_path)==false)
		{
			$msg[] = new Msg(2,"Failed to delete message file");
		}
  }
}

function LogOutEveryone()
{
	if (COM_TYPE=='Win')
	{
		// probably not possible on windows
		// so remove only own session
		end_of_session();
	} else
	{
		signal_service('logout');
	}
}

function DetermineWebclientOnly()
{
	global $dbintf;
	if(($dbintf->actuinfo['permissions'] === PM_WEBCLIENT) && ($dbintf->anyaccpms === PM_NOTES || $dbintf->anyaccpms === 0))
	{
		return 1;
	}
	return 0;
}

function GeneralJSData()
{
	global $dbintf,$configs,$configs_s;
	$subadmin = false;
  if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0
	&& ($dbintf->actuinfo['permissions']&PM_SUBADMIN)!=0
	)
	{
		$subadmin = true;
	}
	$pm_recyclebin = (($dbintf->actuinfo['permissions']&PM_DELETE)!=0)
		|| (($dbintf->anyaccpms&PM_DELETE)!=0);
	$pm_users = ( ($dbintf->actuinfo['permissions']&PM_CONFIG)==0
	           && ($dbintf->actuinfo['permissions']&PM_USERS)!=0 );
	$pm_anyusr = ($dbintf->anyaccpms&PM_USERS)!=0;
	$pm_filter = ( ($dbintf->actuinfo['permissions']&PM_CONFIG)==0
	           && ($dbintf->actuinfo['permissions']&PM_FILTER)!=0 );
	$pm_sysalarm = ($dbintf->actuinfo['permissions']&PM_SYSALARM)!=0 ;
	$ce = 0;
	if (CheckCePm(PM_CE_VIEW)) $ce = 1;
	if (CheckCePm(PM_CE_EVALUATE)) $ce = 2;
	if (CheckCePm(PM_CE_SUPERVISOR)) $ce = 3;
	if (CheckCePm(PM_CE_MANAGE)) $ce = 3;
	
	$pm_stats = (((($dbintf->actuinfo['permissions']&PM_CONFIG)!=0)
						|| int_ensure_defined($configs['StatsAccess'])==0));
						
	$canlogoff = true;
	$toolsmenu = true;
	if (str_ensure_defined($configs['Theme'])=='ts'
	&& ($dbintf->actuinfo['permissions']&(PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN|PM_TENANT_ADMIN_AUDIT) )==0)
	{
    if (int_ensure_defined($configs['LogOffButton'])==0)
    {
      $canlogoff = false;
    }
		$toolsmenu = false;
	}
	
	$pms = $dbintf->actuinfo['permissions'];

	if ($pms&PM_CONFIG){ $pms |= PM_CE_MANAGE;};
	if ($pms&PM_CE_MANAGE){ $pms |= PM_CE_EVALUATE; $pms |= PM_CE_SCHEDULING;};
	if ($pms&PM_CE_EVALUATE){ $pms |= PM_CE_VIEW;};

  $to_active = true;
	if((str_ensure_defined($configs['Theme'])=='ts' && ($dbintf->actuinfo['permissions']&(PM_CONFIG|PM_T2_ADMIN) )==0)
  || (str_ensure_defined($configs['ActiveCallsAccess'])=='admins'     && ($dbintf->actuinfo['permissions']&(PM_CONFIG|PM_T2_ADMIN) )==0)
  ){
    $to_active = false;
  }
  
  $to_live = false;
  if ( ($dbintf->actuinfo['permissions']&PM_CONFIG)!=0
    && int_ensure_defined($configs['LiveCalls'])!=0 )
  {
    $to_live = true;
  }
	
	return '
  var SessionToken = "' . JSQuote(str_ensure_defined($_SESSION['token'])) . '";
  var to_phonebook = '. (($dbintf->actuinfo['permissions']&(PM_VIEWPHONEBOOK|PM_T2_ADMIN))?1:0) . ';
  var to_config = ' . (($dbintf->actuinfo['permissions']&(PM_CONFIG))?1:0) . ';
  var is_subadmin = ' . ($subadmin?1:0) . ';
	var exportimport = ' . (($dbintf->actuinfo['permissions']&(PM_CONFIG|PM_T2_ADMIN))?1:0) . ';
	var tenantadmin_audit = ' . (($dbintf->actuinfo['permissions']&(PM_TENANT_ADMIN_AUDIT))?1:0) . ';
  var tenantadmin_exp = ' . (($dbintf->actuinfo['permissions']&(PM_TENANT_ADMIN_EXP))?1:0) . ';
  var tenantadmin_encrypt = ' . (($dbintf->actuinfo['permissions']&(PM_TENANT_ADMIN_ENCRYPT))?1:0) . ';
	var to_active = ' . (int) $to_active . ';
  var to_live = ' . (int) $to_live . ';
  var pm_users = ' . (($pm_users||($dbintf->actuinfo['permissions']&(PM_T2_ADMIN|PM_T3_ADMIN)))?1:0) . ';
  var pm_anyusr = ' . ($pm_anyusr?1:0) . ';
  var pm_filter = ' . ($pm_filter?1:0) . ';
	var pm_recyclebin = ' . ((int_ensure_defined($configs['RecycleBin'])&$pm_recyclebin)?1:0) . ';
  var pm_sysalarm = ' . ($pm_sysalarm?1:0) . ';
  var datef = "' . ToJSDateTimeFormat($configs['datef']) . '";
  var timeformatix = ' . int_ensure_defined($configs['timeformat']) . ';
  var personal_config = ' . int_ensure_defined($configs['EditProfile']) . ';
  var crec_on = ' . int_ensure_defined($configs_s['crec']) . ';
  var calleval_on = ' . int_ensure_defined($configs_s['calleval']) . ';
  var pms_extended = ' . int_ensure_defined($pms) . ';
  var mobiledevconf = ' . int_ensure_defined($configs['MobileDevConf']) . ';
  var pm_stats = ' . (($pm_stats)?1:0) . ';
	var T2_Admin = ' . (($dbintf->actuinfo['permissions']&(PM_T2_ADMIN))?1:0) . ';
	var T3_Admin = ' . (($dbintf->actuinfo['permissions']&(PM_T3_ADMIN))?1:0) . ';
	var MultiTenancy = ' . int_ensure_defined($configs_s['MultiTenancy']) . ';
	var clustering = ' . ($dbintf->clustering?1:0) . ';
	var nodePassive = ' . ($dbintf->nodePassive?1:0) . ';
	var canlogoff = ' . ($canlogoff?1:0) . ';
	var showtoolsmenu = ' . ($toolsmenu?1:0) . ';
	var pm_webclient = ' . (($dbintf->actuinfo['permissions']&PM_WEBCLIENT)?1:0) . ';
	var pm_livedash = ' . (($dbintf->actuinfo['permissions']&PM_CE_VIEW_LIVEDASH)?1:0) . ';
	var only_webclient = ' . DetermineWebclientOnly() . ';
	var LegacyAudio = ' . int_ensure_defined($configs['LegacyAudio']) . ';
  var SystemName = "' . JSQuote(str_ensure_defined($configs['SystemName'])) . '";
  var XlsxAvailable = ' . (HasExcelExport() ? 1 : 0) . ';
  var has_ED137 = ' . (int_ensure_defined($configs_s['ED137ChCnt']) > 0 ? 1 : 0) . ';
  ';
}

function HasExcelExport()
{
  return PHP_VERSION_ID >= 70200;
}

function SetUrlParam($newurl,$param,$prjid)
{
	$pos = strpos($newurl,$param.'=');
	if ($pos===FALSE)
	{
		if (strpos($newurl,'?')===FALSE) $newurl .= '?';
   			else $newurl .= '&';
		$newurl .= $param . '=' . $prjid;
	} else
	{
		// need to replace!
		$testpos = $pos + strlen($param) + 1;
		while (ord($newurl[$testpos])>=ord('0')
		&& ord($newurl[$testpos])<=ord('9'))
		{
			$testpos++;
		}
		$newurl = substr($newurl,0,$pos+6) . $prjid . substr($newurl,$testpos);
	}
	return $newurl;
}

function CheckCePm($RequiredAccess)
{
	global $dbintf;
	$pms = $dbintf->actuinfo['permissions'];

	if ($pms&PM_CONFIG){ $pms |= PM_CE_MANAGE;};
	if ($pms&PM_CE_MANAGE){ $pms |= PM_CE_EVALUATE; $pms |= PM_CE_SCHEDULING;};
	if ($pms&PM_CE_EVALUATE){ $pms |= PM_CE_VIEW;};
	
	return (($pms & $RequiredAccess)!=0);
}

function GetDefaultHeaderBlock()
{
    // inline HTML to save time and disk access
    $blk = '<img class="header-logo" style="margin-left:auto;"'
    . ' src="p/img/LogoVidicodeHeader.png" '
    . ' alt="<%prodname%> by Vidicode"/>';
    return $blk;
}

function GetHeaderBlock()
{
	if (file_exists('../apresa/templ/custom/header-block.htm'))
	{
    $blk = file_get_contents('../apresa/templ/custom/header-block.htm');
	} else
  {
    $blk = GetDefaultHeaderBlock();
  }
  return $blk;
}

function GetHeader()
{
  /*
	if (file_exists('../apresa/templ/custom/header_custom.htm'))
	{
    // Say something maybe
	}
  */
  $header = file_get_contents('../apresa/templ/header.htm');
  $blk = GetHeaderBlock();
  $header = str_replace("<%headerblock%>",$blk,$header);
  return $header;
}

function OneTimeCheck()
{
	SetSessionVar('OneTimeCheck', true);
	// nothing to do here at the moment
}

function GetDefaultCategoryColors()
{
  return array( 'ff0000','6688ff','00dd00'
    ,'ffff44','dc53c1','fb990d'
    ,'ffffff','36873e','aef9f6','bbbbbb' );
}

function CheckLicensing()
{
	global $lng, $msg, $configs_s;

	// if busy with activation, stay quiet
	if (strstr($_SERVER['PHP_SELF'],'activation.php'))
	{
		// stay quiet
		return;
	}
	
	// before software-only (old style), then no system license needed
	if (@file_exists('../apresa/conf/opt_so')==false) 
	{
		$_SESSION['LicChk'] = 1;
		$_SESSION['LicState'] = 1;
		return;
	}

	// view-only satelite system
	if (int_ensure_defined($configs_s['LicViewer'])!=0)
	{
		// dont check system license
		// and assume it has none
		$_SESSION['LicChk'] = 1;
		$_SESSION['LicState'] = 0;
		return;
	}

	// renting
	if ( int_ensure_defined($configs_s['LicRent'])!=0 )
	{
		$LicSysRentDaysLeft = int_ensure_defined($configs_s['LicSysRentDaysLeft']);
		if ($LicSysRentDaysLeft<=0)
		{
			$msgtext = "The renting period of the system has expired! Recording of calls is disabled.";
			$msg[] = new Msg(MSG_ERR, $msgtext);
		} else if (int_ensure_defined($configs_s['LicSystem'])==0
		&& int_ensure_defined($configs_s['LicGrace'])==0
		&& int_ensure_defined($configs_s['LicTrial'])==0)
		{
			$msgtext = "The renting period of the system has expired!";
			$msg[] = new Msg(MSG_ERR, $msgtext);
			// repeat this message
		} else if ($LicSysRentDaysLeft<=5)
		{
			$msgtext = "The renting period of the system will expire in {0} days.";
			$msgtext = str_replace( "{0}", $LicSysRentDaysLeft, $msgtext );
			$msg[] = new Msg(MSG_WRN, $msgtext);
			// repeat this message
		} else
		{
			$msgtext = "The renting period of the system will expire in {0} days.";
			$msgtext = str_replace( "{0}", $LicSysRentDaysLeft, $msgtext );
			$msg[] = new Msg(MSG_INF, $msgtext);
			// do not repeat this message
			$_SESSION['LicChk'] = 1;
		}
		return;
	} 
	
	// system has a full and correct license?
	if (int_ensure_defined($configs_s['LicSystem'])!=0) 
	{
		$_SESSION['LicChk'] = 1;
		$_SESSION['LicState'] = 1;
		return;
	} 

	// without system license,
	// it can still be that we are allowed to record
	// for these reasons:
	// - grace period
	// - trial period
	
	if (int_ensure_defined($configs_s['LicGrace'])==1)
	{
		$_SESSION['LicState'] = 1;
		$days = int_ensure_defined( $configs_s["LicFailDaysLeft"] );
		$msgtext = $lng->Get('Licensing.Fail').' '.$lng->Get('Licensing.StopAfter');
		$msgtext = str_replace('%1',$days,$msgtext);
		$msg[] = new Msg(2,$msgtext.' (<a href="activation.php">'.$lng->Get('Licensing.SoftwareActivation').'</a>)');
	} else if (int_ensure_defined($configs_s['LicTrial'])==1)
	{
		$_SESSION['LicChk'] = 1;
		$_SESSION['LicState'] = 1;
		$days = int_ensure_defined( $configs_s["LicTrialDaysLeft"] );
		$msgtext = $lng->Get('Licensing.Trial').'. '.$lng->Get('Licensing.StopAfter');
		$msgtext = str_replace('%1',$days,$msgtext);
		$msg[] = new Msg(2,$msgtext.' (<a href="activation.php">'.$lng->Get('Licensing.SoftwareActivation').'</a>)');
	} else
	{
		$_SESSION['LicState'] = 0;
		if (int_ensure_defined($configs_s["LicOnce"])==0)
		{
			$msg[] = new Msg(2,$lng->Get('Licensing.NotYet').' (<a href="activation.php">'.$lng->Get('Licensing.SoftwareActivation').'</a>)');
		} else
		{
			$msg[] = new Msg(2,$lng->Get('Licensing.FailStopped').' (<a href="activation.php">'.$lng->Get('Licensing.SoftwareActivation').'</a>)');
		}
	}
	
}

// session variables that is reset during update

function GetSessionVar($key)
{
	if (isset($_SESSION['svar'])==false)
	{
		$_SESSION['svar'] = array();
	}
	if (isset($_SESSION['svar'][$key])==false) return NULL;
	return $_SESSION['svar'][$key];
}
function SetSessionVar($key, $value)
{
	if (isset($_SESSION['svar'])==false)
	{
		$_SESSION['svar'] = array();
	}
	$_SESSION['svar'][$key] = $value;
}
function ClearSessionVars()
{
	$_SESSION['svar'] = array();
}

function generateStrongPassword($length = 9, $available_sets = 'luds')
{
	// at least, generate a password of length:
	if ($length<10) $length = 10;
	$sets = array();
	if(strpos($available_sets, 'l') !== false)
		$sets[] = 'abcdefghjkmnpqrstuvwxyz';
	if(strpos($available_sets, 'u') !== false)
		$sets[] = 'ABCDEFGHJKMNPQRSTUVWXYZ';
	if(strpos($available_sets, 'd') !== false)
		$sets[] = '23456789';
	if(strpos($available_sets, 's') !== false)
		$sets[] = '!@#$%&*?';
	$all = '';
	$password = '';
	foreach($sets as $set)
	{
		$password .= $set[array_rand(str_split($set))];
		$all .= $set;
	}
	$all = str_split($all);
	for($i = 0; $i < $length - count($sets); $i++)
		$password .= $all[array_rand($all)];
	$password = str_shuffle($password);

	return $password;
}

function generateVerification($digits)
{
	return rand(pow(10, $digits - 1) - 1, pow(10, $digits) - 1);
}

function IsPageRequested()
{
	$page = substr($_SERVER['SCRIPT_NAME'],1);
	$notPages = array( 'getactive.php' => true
	, 'getdata.php' => true
	, 'getdirs.php' => true
	, 'getshares.php' => true
	, 'getservers.php' => true
	, 'cecallsimglegacy.php' => true
	, 'cecallsimg.php' => true
	, 'livedashboarddata.php' => true
	);
	if (isset($notPages[$page])) return false;
	return true;
}

function GetSsoOneAction()
{
  global $configs;
  $value = str_ensure_defined($configs['SsoOneAction']);
  if ($value=='') return 'askusr'; // the default
  return $value;
  
}

function SsoIsEnabled()
{
  global $configs;
  if (int_ensure_defined($configs['ExtLogon'])==1) return true;
  return false;
}

function SsoHasLoggedIn()
{
  $do_sso = SsoIsEnabled();
  if ($do_sso)
  {
    if (isset($_SESSION['sso']))
    {
      return true;
    }
  }
  return false;  
}


?>