<?php
/*
	This PHP is used to serve recordings.
	
	It checks permission.
	
	This .php is not referenced directly, but through the virtual dir. /getcall/
	The purpose of this is to get the filename right in the URL.	
	
*/

require_once('general.php');
require_once('utility.php');
require_once('callhandler.php');

// finished call
// these are referenced from the main page,
// as well as from the agent evaluation pages
if (isset($_GET['id']))
{
	HandleCall();
	exit;
}
// active calls
// these are referenced from the active calls page
if (isset($_GET['aid']))
{
	HandleActiveCall();
	exit;
}

exit('no ID specified');

// the rest are functions (or classes)
// ==================================================

function HandleActiveCall()
{
	global $dbintf, $configs;

	// get information about this active call
	$id = (int) $_GET['aid'];
  $info = $dbintf->GetActiveCallInfo($id);
	if ($info==NULL)
	{
		exit('call not found');
	}

	// active calls are never downloaded
  $download = false;
	
	// Check if this is an embedded playback
	// currently this is simply given in the URL.
	// An alternative would be to check user agent.
	// the problem is, for HTML5 inline playback,
	// the user agent is the same as for download.
	$embed = 0;
	if (isset($_GET['embed']))
	{
		$embed = (int) $_GET['embed'];
	}
	
	// Check permission
	// currently, for active calls there is download option, but for completeness,
	// the code to handle it is here.
  if ($download)
  {
		// need both permissions
		if ( ($info['permissions']&PM_ACTIVEPLAY)==0
		||   ($info['permissions']&PM_DOWNLOAD  )==0   )
	  {
	    echo 'Permission denied';
	    exit;
	  }
	} else
	{
		// it is an active call so
	  if (($info['permissions']&PM_ACTIVEPLAY)==0)
	  {
	    echo 'Permission denied';
	    exit;
	  }
	}
	// If we do not embed,
	// then maybe we need download permission
	$NoEmbedRequiresDownloadPM = int_ensure_defined($configs['PlayPmEmbedded']);
	if ($embed==0 && $NoEmbedRequiresDownloadPM)
	{
		if (($info['permissions']&PM_DOWNLOAD)==0)
		{
			echo 'Permission denied (only embedded playback allowed)';
			return false;
		}
	}

	// get file details, and whether the file is encrypted
	// screen recording is not available during the call
	// so it not a consideration here
  $filename = $info['filename'];
	$fn = AUDIO_PATH.'/'.$info['directory'].'/'.$filename;
	$thisOneEncrypted = ($info['flags']&16)!=0;
	if ($thisOneEncrypted)
	{
		// encrypted calls are recorded to memory
		$fn = '/dev/shm/apresa'.'/'.$info['directory'].'/'.$filename;
	}

	// auditing
	LogAction('Playback/download active call', array('call_id'=>$info['id']) );

	// Recording might be first done to a .g711 file,
	// which is later converted to the file storage format.
  if (!file_exists($fn))
  {
		$fn = $fn . '.g711';
	}

	if (file_exists($fn))
  {
    // this format audio/vnd.wave is what is officially the format,
		// but actually more common would be audio/wave.
		// however, having a separate format has the advantage
		// that you can set a specific handle for the audio (vidiplayer)
		header('Content-Type: audio/vnd.wave');
    header('Pragma: '); // needed for IE (HTTPS)
		if ($download)
    {
      header('Content-Disposition: attachment');
    } 
		ServerFilePart($fn);
    // old: readfile($fn);
  } else {
	  echo 'The audio file does not exist or cannot be accessed. '.$fn;
	}
	exit;
}

function HandleCall()
{
	// lets everything be done by the class
	$handler = new CallHandler();
	$handler->Perform();
}
