<?php
require_once('general.php');
require_once('smbgen.php');
require_once('signaling.php');

$configs = array_merge( $configs, $dbintf->LoadConfigsPart('b') );
$configs = array_merge( $configs, $dbintf->LoadConfigsPart('n') );

HandlePage($dbintf);

function EnsureEndingSlash($path)
{
  if (strlen($path)>0 && $path[strlen($path)-1]!='/') $path .= '/';
  return $path;
}

function GetDirList($path,$dir,$type)
{
  $ret = '';
  if ($type=='F')
  {
    // do not allow random file listing
    if ($dir != '*.sql.gz') return $ret;
  }
  if ($dh = @opendir($path))
  {
    $first = true;
    while (($finfo = readdir($dh)) !== false)
    {
      if ($finfo=='.' || $finfo=='..') continue;
      if ($dir!='' && $dir!='*')
      {
        if (substr($finfo,-strlen($dir)+1)!=substr($dir,1))
        {
          continue;
        }
      }

      $fullfilename = EnsureEndingSlash($path) . $finfo;
      if ($type=='F')
      {
        if (is_file($fullfilename)==false) continue;
      } else if ($type=='D')
      {
        if (is_dir($fullfilename)==false) continue;
      }
      
      if (!$first) $ret .= ',';
      $obj = array ('name'=>$finfo);
      $ret .= json_encode($obj);
      $first = false;
    }
    closedir($dh);
  }
  return $ret;
}


function HandlePage($dbintf)
{
  global $configs;
  if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0)
  {
    echo 'Permission denied';
    exit;
  }
  
  // also allow $_GET for debugging
  foreach ($_GET as $a=>$b)
  {
    $_POST[$a] = $b;
  }

	$srctype = str_ensure_defined($_POST['srctype']);
  $domain = '';
	if (isset($_POST['domain']))
	{
		$domain = $_POST['domain'];
	}
  $svr = str_ensure_defined($_POST['svr']);
  $share = str_ensure_defined($_POST['share']);
  $usr = str_ensure_defined($_POST['usr']);
  $pwd = '';
  if (isset($configs['WinPwd']))
  {
    $pwd = decrypt($configs['WinPwd']);
  }
	if (isset($_POST['cfgpwd']))
	{
    $pwd = decrypt($dbintf->GetConfigValue($_POST['cfgpwd']));
	}
  if (isset($_POST['pwd']))
  {
    $pwd = $_POST['pwd'];
  }
  $subdir = '';
	if (isset($_POST['subdir']))
	{
		$subdir = $_POST['subdir'];
	}
  $dir = '';
	if (isset($_POST['dir']))
	{
		$dir = $_POST['dir'];
	}
	$type = 'D';
	if (isset($_POST['type']))
	{
		$type = $_POST['type'];
	}
	$path = '';
	if (isset($_POST['path']))
	{
		$path = $_POST['path'];
	}
	$disk = '';
	if (isset($_POST['disk']))
	{
		$disk = $_POST['disk'];
	}
  if (isset($_POST['cfg']) && $_POST['cfg']=='Bck')
  {
    // look up from configs
    $domain = str_ensure_defined($configs['WinDomain']);
    $usr = str_ensure_defined($configs['WinUsr']);
    $pwd = decrypt($configs['WinPwd']);
    
    $srctype = int_ensure_defined($configs['BckTargetType']);
    $svr     = str_ensure_defined($configs['BckServerIP']);
    $share   = str_ensure_defined($configs['BckShare']);
    $subdir  = str_ensure_defined($configs['BckSubdir']);
    $path    = str_ensure_defined($configs['BckPath']);
    $disk    = str_ensure_defined($configs['BckDisk']);
  }

  $subnetmask = $configs['IPmask'];
  if (IsIP4($svr)==false
  && MayBeIP6($svr)==false
  && $subnetmask!=''
  )
  {
    $res = NmbLookupIPAddr($svr,$subnetmask);
    if ($res!==false)
    {
      $svr = $res;
    }
  }

  $ret = '{"data":[';

  $cmdline = "smbclient ".GetSmbVersioningParam().' '.escapeshellarg("//$svr/$share")." -U ".escapeshellarg("$usr%$pwd");
	if ($domain!='')
	{
		$cmdline .= " -W " . escapeshellarg($domain);
	}
	$cmdline .= " -c " . escapeshellarg("dir ".$dir);
  if ($subdir!='')
  {
		$cmdline .= " -D " . escapeshellarg($subdir );
	}
	// known problem dir listing does not work from Windows 10
  // cli_list: Error: unable to parse name from info level 260
  // it is an external bug
  
	// debug:
	//echo '{"data":[{"name":"'.$cmdline.'"}]}'; exit;

  if (COM_TYPE=='Win')
  {
		if ($type=='D')
		{
    	echo '{"data":[{"name":"backup0912"},{"name":"Backup sales documents"}]}';
    } else
		{
    	echo '{"data":[{"name":"apresa20100109.sql.gz"},{"name":"apresa20100203.sql.gz"}]}';
		}
    exit;
  }


  $errtext = '';
	if ($srctype==1 || $srctype==2)
	{
    $ret .= GetDirList($path,$dir,'F');
	} else if ($srctype==3 || $srctype==4)
  {
    
    // for this, we need to temporarily mount
    // this must be done in the handler
    // to communicate with the handler we store data somewhere
    $cfg = array();
    $cfg['RtmpBckTargetType'] = $srctype;
    $cfg['RtmpBckDisk'      ] = $disk;
    $cfg['RtmpBckServer'    ] = $svr;
    $cfg['RtmpWinUsr'       ] = $usr;
    $cfg['RtmpWinPwd'       ] = encrypt($pwd);
    $cfg['RtmpBckPath'      ] = $path;
    $dbintf->SetConfigsPart($cfg,'rb');
    
    $reply = true;
    signal_service('mnt Rtmp rb',$reply);
    
    $path = '/mnt/drive_Rtmp';
    if ($srctype==3)
    {
      $path .= '/' . $subdir;
    }
    
    if ($type=='F')
    {
      $ret .= GetDirList($path,$dir,'F');
    } else
    {
      $ret .= GetDirList($path,'','D');
    }
    
    $reply = true;
    signal_service('umnt Rtmp',$reply);
    
    foreach ($cfg as $param=>$value)
    {
      $dbintf->DeleteFromConfigs($param);
    }
  } else // srctype=0
	{
	  $smbret = shell_exec($cmdline);
	  $smblines = explode("\n",$smbret);

	  $first = true;
	  foreach ($smblines as $line)
	  {
			if ($first && strpos($line,'failed')!=0 && count($smblines)<=2)
			{
				$errtext = $line;
			}
	    if (strlen($line)<30) continue;
	    if ($line[0]!=' ') continue;
	    if ($line[2]==' ') continue;
	    $linelen = strlen($line);
	    $name = trim(substr($line,2,$linelen-42));
	    if ($name=='.' || $name=='..') continue;
	    $attr = trim(substr($line,$linelen-38,3));
	    if ($type=='D')
	    {
				// it must be directory
	    	if (strpos($attr,'D')!==0) continue;
			}
	    if ($type=='F')
	    {
				// it must be a file, not a directory
	    	if (strpos($attr,'D')===0) continue;
			}

	    if (!$first) $ret .= ',';
	    $obj = array ('name'=>$name);
	    $ret .= json_encode($obj);
	    $first = false;
	  }
  }

  $ret .= ']';
  $ret .= ',"errtext":"'.$errtext.'"';

  $ret .= '}';
  echo $ret;
}

?>
