<?php
require_once('general.php');
require_once('actions.php');

HandlePage($dbintf);

function HandlePage($dbintf)
{
  global $rpl,$msg,$lng,$linemap;

  if ( ($dbintf->actuinfo['permissions']&(PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN))==0 )
  {
    echo 'Permission denied';
    exit;
  }

  $step = 'init';

  if ($_POST['act']=='import')
  {
    $step = 'import';
    $csv = $_POST['csv'];

		LogAction('Import user groups from CSV');
    $dbintf->StartTransaction();

    $file = new CsvFile();
    $file->ReadFromString($csv);

    $err = false;
    $groups = ParseGroupList($dbintf,$file,$err);
		if ($err)
		{
			$msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
			$msg[] = new Msg(MSG_ERR,'Import cancelled');
			$dbintf->CancelTransaction();
			$_SESSION['msg'] = $msg;
			header('Location: '.$rooturl.'users.php');
			exit;
		}

    $group_list = $dbintf->GetGroupList();
    $GroupNameToAccId = array();
    $GroupDBIdToAccId = array();
    $GroupDBIdToTenant = array();
    foreach($group_list as $group)
    {
      $GroupNameToAccId[$group['name']] = $group['accid'];
      $GroupDBIdToAccId[$group['id']] = $group['accid'];
      $GroupDBIdToTenant[$group['id']] = int_ensure_defined($group['is_tenant']);
    }
    
    foreach($groups as $group)
    {
      if($group['id'] != '')
      {
        $GroupNameToAccId[$group['name']] = $GroupDBIdToAccId[$group['id']];
      }
    }
    
    $reconfig = false;
    while(!empty($groups) && !$err)
    {
      $DefferedGroups = array();
      foreach($groups as $group)
      {
        if($group['super'] != '' && isset($GroupNameToAccId[$group['super']]) == false)
        {
          // Not able to associate an accid with the super group name yet,
          // so try again when other entries have been processed.
          $DefferedGroups[] = $group;
          continue;
        }
        $grpdata = array();
        $grpdata['name'] = $group['name'];
        $grpdata['adgroup'] = $group['adgroup'];
        $grpdata['is_tenant'] = $group['tenant'];
        $grpdata['typegrp'] = 0;
        if($group['super'] == '')
        {
          $grpdata['super'] = 0;
        }
        else
        {
          $grpdata['super'] = $GroupNameToAccId[$group['super']];
        }
        
        if($group['action'] =='add')
        {
          if($dbintf->AddGroup($accid, $grpdata, true) == false)
          {
            $msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
            $err = true;
            break;
          } else
          {
            $ginfo = $dbintf->GetGroupInfoOnAccid($accid);
            LogAction('Add user group',array('id'=>$ginfo['id'],'name'=>$ginfo['name'],'tenantid'=>$ginfo['tenantid']) );
          }
          $GroupNameToAccId[$group['name']] = $accid;
          if($group['tenant'])
          {
            $groupInfoDB = $dbintf->GetGroupInfoOnAccid($accid);
            $grpid = $groupInfoDB['id'];
            $prefix = 'TN' . $grpid . '_';
            $tenant_configs = array();
            foreach( array('RecMinDur', 'HdDaysKeep', 'MaxDiskUsage', 'RecOnDmdAvail', 'SilOnDemandAvail', 'MaxChannels') as $key)
            {
              if($group[$key] !== '')
              { 
                $tenant_configs[$prefix . $key] = $group[$key];
              }
            }
            $dbintf->SetConfigsPart($tenant_configs, 'tn');
            $dbintf->ApplyTenantSettings( $grpid );
            $reconfig = true;
          }
        }
        else if($group['action'] == 'edit')
        {
          $grpid = $group['id'];
          if($dbintf->EditGroup($grpid, $grpdata) == false)
          {
            $msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
            $err = true;
            break;
          } else
          {
            $ginfo = $dbintf->GetGroupInfo($grpid);
            LogAction('Edit user group',array('id'=>$ginfo['id'],'name'=>$ginfo['name'],'tenantid'=>$ginfo['tenantid']) );
          }
          
          if($group['tenant'])
          {
            $prefix = 'TN' . $group['id'] . '_';
            $tenant_configs = array();
            $tenant_configs_old = $dbintf->LoadConfigsPart('tn');
            foreach( array('RecMinDur', 'HdDaysKeep', 'MaxDiskUsage', 'RecOnDmdAvail', 'SilOnDemandAvail', 'MaxChannels') as $key)
            {
              $tkey = $prefix . $key;
              if($group[$key] !== '' || isset($tenant_configs_old[$tkey]))
              { 
                $tenant_configs[$prefix . $key] = $group[$key];
              }
            }
            $dbintf->SetConfigsPart($tenant_configs, 'tn');
            $dbintf->ApplyTenantSettings( $group['id'] );
            $reconfig = true;
          }
          else
          {
            if($GroupDBIdToTenant[$group['id']] == 1)
            {
              $prefix = 'TN' . $group['id'] . '_';
              $dbintf->DeleteConfigsWithPrefix($prefix);
              $dbintf->ApplyTenantSettings( $group['id'] );
              $reconfig = true;
            }
          }
        }
      }
      $groups = $DefferedGroups;
    }
    if ($err)
		{
			$msg[] = new Msg(MSG_ERR,'Import cancelled');
			$dbintf->CancelTransaction();
			$_SESSION['msg'] = $msg;
			header('Location: '.$rooturl.'groups.php');
			exit;
		}
    $msg[] = new Msg(MSG_INF,'<%Web.ImportDone%>');
    

    EnginesSettingsCalcApply($reconfig);
    $dbintf->CommitTransaction();
    
    $_SESSION['msg'] = $msg;
    header('Location: groups.php');
    exit;
  }

  $grpdata = '';

  if (isset($_FILES['grouplist']))
  {
    $upd = $_FILES['grouplist'];
    if ($upd['error']!=0)
    {
      // error in upload
      $msg[] = new Msg(MSG_ERR,'Error in file upload. '.GetUploadErrDescr($upd['error']));
    } else
    {
      // there is a good upload
      // file name:
      $fn = $upd['tmp_name'];

      if(HasExcelExport() && (
        get_filename_extension($upd["name"]) == "xlsx" || 
        $upd['type'] == "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"))
      {
        $file = new XLSXFile();
      }
      else
      {
        $file = new CsvFile();
      }

      $file->Read($fn);
			
			$err = false;
      $groups = ParseGroupList($dbintf,$file,$err);
      $grpdata .= 'var groups = [';
      $first = true;
      foreach ($groups as $row)
      {
        if ($first==false) $grpdata .= ',';				
        $grpdata .= '{name:"'.JSQuote($row['name']).'"'
				. ',oldname:"'.JSQuote($row['oldname']).'"'
				. ',super:"'.JSQuote($row['super']).'"'
        . ',adgroup:"'.JSQuote($row['adgroup']).'"'
        . ',tenant:'.$row['tenant']
        . ',recmindur:"'.JSQuote($row['RecMinDur']).'"'
        . ',maxdiskusage:"'.JSQuote($row['MaxDiskUsage']).'"'
        . ',hddayskeep:"'.JSQuote($row['HdDaysKeep']).'"'
        . ',maxchannels:"'.JSQuote($row['MaxChannels']).'"'
        . ',recondmdavail:'.$row['RecOnDmdAvail']
        . ',silondemandavail:'.$row['SilOnDemandAvail']
        . ',action:"'.JSQuote($row['action']).'"'
        . "}\r\n";
        $first = false;
      }
      
      $grpdata .= "];\r\n";

      $grpdata .= 'var has_err=' . ($err?1:0) . ';' . "\r\n";

      $grpdata .= 'var csv="'.JSQuote($file->asCSV()).'";'."\r\n";
			
      $step = 'verify';
    }
  }
	
	LogAction('Viewed groups import page.',array('level'=>'view'));


  $grpdata .= GeneralJSData();

  // ready with database
  $dbintf->close();

  if ($step=='init')
  {
    $template_content = file_get_contents('../apresa/templ/groups_import.htm');
  }
  if ($step=='verify')
  {
    $template_content = file_get_contents('../apresa/templ/groups_import_verify.htm');
  }

  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $grpdata;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}



/* This is the function that parses the CSV data.
	It is used both during preview and actual import.
	The CSV data is passed as a string that contains
	the complete CSV file as uploaded.
*/
function ParseGroupList($dbintf,$file,&$err)
{
  global $msg, $lng, $linemap;
  $group_list = $dbintf->GetGroupList();
  $existing_groups = array();
  $existingGroupsOnAccId = array();
  $existingGroupsSupers = array();
  foreach ($group_list as $group)
  {
    $existing_groups[ $group['id'] ] = $group;
    $existingGroupsOnAccId[ $group['accid'] ] = $group['name'];
    $existingGroupsSupers[ $group['name'] ] = $group['super'];
  }

  $line_nr = 0;
  $groups = array();
  $editids = array();
  
	foreach($file as $parts)
  {
    $line_nr++;	
		if ($line_nr==1)
    {
      // the header
      continue;
    }

    if (count($parts)<=1 && $parts[1]=='') continue;

    if ($parts[1]=='')
    {
      $err = true;
      $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': No name specified');
    }
		
    // If an ID is defined, then edit
    // If no ID is defined, then add
    // That is the most safe procedure
		$editid = trim($parts[0]);
    $oldname = '';
		$action = '';
    if ($editid!='')
    {
      if (isset($editids[$editid]))
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': Duplicate DB-ID');
        continue;
			}
      if (isset($existing_groups[$editid])==false)
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': DB-ID of group not found');
        continue;
      } 
      $action = 'edit';
      $oldname = $existing_groups[$editid]['name'];
    } else
    {
      $action = 'add';
    }	
		
		$name = trim($parts[1]);
    
		// Check if super group exists, is done after the entire file has been read
		// As it may be defined somewhere later in the file
		$super = trim($parts[2]);
    $ldap = trim($parts[3]);
    $is_tenant = ($parts[4]=='1'?1:0);
		$RecMinDur = trim($parts[5]);
    $MaxDiskUsage = trim($parts[6]);
    $HdDaysKeep = trim($parts[7]);
    $MaxChannels = trim($parts[8]);
    $RecOnDmdAvail = ($parts[9]=='1'?1:0);
    $SilOnDemandAvail = ($parts[10]=='1'?1:0);
    
    if ($action=='edit' && $is_tenant==0 && $existing_groups[$editid]['is_tenant']==1)
    {
      $err = true;
      $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': Tenancy for a group cannot be disabled through group import');
      continue;
    }
    
    if(!ValidTenantSetting($RecMinDur) || !ValidTenantSetting($MaxDiskUsage) || !ValidTenantSetting($HdDaysKeep) || !ValidTenantSetting($MaxChannels))
    {
      $err = true;
      $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': Tenant setting not valid');
      continue;
    }
    
    if ($editid!='')
    {
      // add *after* error checks
      $editids[$editid] = true;
    }
    
    $groups[] = array
    ( 'id'      => $editid
    , 'name'    => $name
    , 'oldname' => $oldname
    , 'super' 	=> $super
    , 'adgroup' => $ldap
    , 'tenant'	=> $is_tenant
    , 'action'  => $action
		, 'linenr' 	=> $line_nr
    , 'RecMinDur' => $RecMinDur
    , 'MaxDiskUsage' => $MaxDiskUsage
    , 'HdDaysKeep' => $HdDaysKeep
    , 'MaxChannels' => $MaxChannels
    , 'RecOnDmdAvail' => $RecOnDmdAvail
    , 'SilOnDemandAvail' => $SilOnDemandAvail
    );
  }
	// Check if all super groups exist 
	// If a group exists in the database, but the import would change
	// the name of that group, the import fails
	
	$GroupExists = array(); // Groups in database that are not editted + imported groups
	$GroupsInDB = array();  // Groups in database that are not editted during import
	
  foreach( $group_list as $group )
	{
		$GroupsInDB[$group['name']] = true;
		if( isset($editids[$group['id']]) == false)
		{
			$GroupExists[$group['name']] = true;
		}
	}
	
	foreach($groups as $group)
	{
		$GroupExists[$group['name']] = true;
	}
	foreach($groups as $key => $group)
	{
		if( $group['super'] != '' && isset( $GroupExists[$group['super']] ) == false)
		{
			$err = true;
			if(isset( $GroupsInDB[$group['super']] ) == true)
			{
				$msg[] = new Msg(MSG_ERR,'Line '.$group['linenr'].': Higher level group exists in database, but the name will be changed during import');
			}
			else
			{
				$msg[] = new Msg(MSG_ERR,'Line '.$group['linenr'].': Higher level group does not exist');
			}
			unset($groups[$key]);
			continue;
		}
	}
  
  $editGroupsSupers = array();
  foreach($groups as $group)
  {
    $editGroupsSupers[$group['name']] = $group['super'];
  }
  
  // Check if group does not have itself as supergroup
  foreach($groups as $key => $group)
  {
    $super = $group['super'];
    $supers = array();
    while($super != '')
    {
      if($super == $group['name'])
      {
        $msg[] = new Msg(MSG_ERR,'Line '.$group['linenr'].': Group is supergroup of itself');
        unset($groups[$key]);
        $err = true;
        break;
      }
      
      if(isset($supers[$super]))
      {
        // Somewhere a group is a supergroup of itself, causing an endless loop
        // but it is not the result of this entry
        break;
      }
      $supers[$super] = true;
      $super = GetSuperGroup($editGroupsSupers, $existingGroupsSupers, $existingGroupsOnAccId, $super);
    }
  }
	return $groups;
}

function GetSuperGroup($editGroupsSupers, $existingGroupsSupers, $existingGroupsOnAccId, $group)
{
  if($group === '')
  {
    return '';
  }
  else if(isset($editGroupsSupers[$group]))
  {
    $r = $editGroupsSupers[$group];
    return $r;
  }
  else
  {
    $accid = $existingGroupsSupers[$group];
    $r = $existingGroupsOnAccId[$accid];
    return $r;
  }
}

// Returns true is $setting is either empty string
// or numeric otherwise
function ValidTenantSetting($setting)
{
  if($setting === '')
  {
    return true;
  }
  else if(is_numeric($setting))
  {
    return true;
  }
  else
  {
    return false;
  }
}

?>
