<?php
require_once('utility.php');
require_once('actions.php');

class HealthData
{
  var $health;
  var $newhealth;
  var $healthconf;
  var $newinfo;
  var $messages;
  var $new_failure;
  var $datapath;
  var $simulate;
  var $ignoreList;
	var $handledPowerSupply;
  var $main_configs_s;
	var $configs_n;
  var $lowmem;
  var $restore;
  var $any_failure;
  var $node_failures;
  var $node_db;
  var $clustering;
  var $VoIPnetwChk;
  var $addedPostProcess;


  function init($datapath)
  {
    $this->datapath = $datapath;
    $this->newhealth = array();
    $this->newinfo = array();
    $this->messages = array();
    $this->new_failure = false;
    $this->restore = false;
    $this->any_failure = false;
    $this->node_failures = 0;
    $this->node_db = false;
    $this->simulate = false;
    $this->ignoreList = array();
		$this->handledPowerSupply = false;
    $this->lowmem = false;
    $this->addedPostProcess = false;
  }

  function load()
  {
    global $debug_mode, $dbintf;
    // read current(previous) health state:
    $this->health = getConfArr($this->datapath . '/health.dat');
    if ($debug_mode)
    {
      var_dump($this->health);
    }
    // read health threshold configuration
    $config_filename = 'healthconf.dat'; // default Apresa system layout
    if (defined('HEALTH_CONFIG_FILE'))
    {
      $config_filename = HEALTH_CONFIG_FILE;
    }
    dbglog('Config.File: '.$config_filename);
    $this->healthconf = getConfArr($this->datapath . '/../conf/' . $config_filename);
    dbglog('Config.File: '.$this->datapath . '/../conf/' . $config_filename);
    if ($debug_mode)
    {
      var_dump($this->healthconf);
    }
    
    $list = $dbintf->GetConfigValue('HealthIgnoreList');
    $parts = explode(',',$list);
    foreach ($parts as $part)
    {
      $this->ignoreList[$part] = true;
    }
    
    $this->main_configs_s = $dbintf->LoadConfigsPart('s');
		SetHealthConfigDefaults($this->main_configs_s);
		$this->configs_n = $dbintf->LoadConfigsPart('n');
  }

  // major version number
  function GetDebianVersion()
  {
    if ($this->simulate)
    {
      return 8;
    }
    $issuenet = @file_get_contents('/etc/issue.net');
    if (strpos($issuenet,'Debian')===false) return 0;
    $parts = explode(' ',$issuenet);
    if (count($parts)==0) return $parts[0];
    $lastword = array_pop( $parts );
    return (int) $lastword;
  }
  
  function GetHealth($key)
  {
    if (isset($this->health[$key])) return $this->health[$key]!=0;
    else return true;
  }
  
  function Add($info)
  {
    if (isset($this->ignoreList[ $info['id'] ]))
    {
      $info['h'] = 2;
    }
    $this->newinfo[] = $info;
    $isNodeHealth = ! isset( $info['notnode'] );
    $this->setItem($info['id'],$info['h'],$info['am'],$isNodeHealth);
  }

	function InitCluster()
	{
		$this->clustering = false;
		if (@file_exists(dirname(__FILE__) . '/../apresa/conf/opt_cluster'))
		{
			$this->clustering = true;
		}
	}
	
	function GetNodeListInCrmLine($line)
	{
		$arr = array();
		$pos = strpos($line,'[');
		if ($pos===false) return $arr;
		$t = substr($line,$pos);
		$parts = explode(' ',$t);
		foreach ($parts as $part)
		{
			$part = trim($part);
			if ($part=='[' || $part==']') continue;
			$arr[] = $part;
		}
		return $arr;
	}
	
	function GetNodeAttrInCrmLine($line)
	{
		$pos = strpos($line,':');
		if ($pos===false) return '';
		$t = substr($line,$pos+1);
		return trim($t);
	}
	
	function Clustering()
	{
    global $dbintf;
		if ($this->clustering==false) return;
    
    $node_name = $dbintf->GetConfigValue('IPname');
		
		$this->AddCaption('H.A. Cluster');
		// get cluster status
		$crmLines = array();
		$drbdLines = array();
    if (!$this->simulate)
    {
			//msg_signal_service($msg,'cluster info');
			exec('crm_mon -A -1',$crmLines);
			exec('cat /proc/drbd',$drbdLines);
		} else
		{
			$crmLines = <<<SIMDATA
Last updated: Fri Nov 20 09:27:17 2015
Last change: Fri Nov 20 09:21:30 2015
Stack: corosync
Current DC: node2 (3232249622) - partition with quorum
Version: 1.1.12-561c4cf
2 Nodes configured
8 Resources configured


Online: [ node1 node2 ]

 Master/Slave Set: ms_drbd_r0 [drbd_r0]
     Masters: [ node2 ]
     Slaves: [ node1 ]
 Resource Group: apgrp
     VIP	(ocf::heartbeat:IPaddr2):	Started node2 
     recs_fs	(ocf::heartbeat:Filesystem):	Started node2 
     mapupl	(ocf::vidicode:mapupload):	Started node2 
     anode	(ocf::vidicode:apresanode):	Started node2 
 Master/Slave Set: ms_postgresql [postgresql]
     Masters: [ node2 ]
     Slaves: [ node1 ]

Node Attributes:
* Node node1:
    + master-drbd_r0                  	: 10000     
    + master-postgresql               	: 100       
    + postgresql-data-status          	: STREAMING|SYNC
    + postgresql-status               	: HS:sync   
* Node node2:
    + master-drbd_r0                  	: 10000     
    + master-postgresql               	: 1000      
    + postgresql-data-status          	: LATEST    
    + postgresql-master-baseline      	: 0000000000000000
    + postgresql-status               	: PRI       

Failed actions:
    VIP_monitor_10000 on node2 'not running' (7): call=122, status=complete, last-rc-change='Fri Nov 20 09:21:21 2015', queued=0ms, exec=0ms

SIMDATA;
			$drbdLines = <<<SIMDATA

version: 8.3.11 (api:88/proto:86-96)
srcversion: F937DCB2E5D83C6CCE4A6C9 

 1: cs:Connected ro:Primary/Secondary ds:UpToDate/UpToDate A r-----
    ns:42096 nr:140 dw:42236 dr:858 al:12 bm:1 lo:0 pe:0 ua:0 ap:0 ep:1 wo:f oos:0

SIMDATA;
			$crmLines = explode("\n",$crmLines);
			$drbdLines = explode("\n",$drbdLines);
		}
		
		dbglog("Clustering");
		$online = array();
		$drbdMasters = array();
		$drbdSlaves  = array();
		$dbMasters = array();
		$dbSlaves  = array();
		$section = '';
		$nodeatt = '';
		foreach ($crmLines as $line)
		{
			if (startsWith($line,'Online:'))
			{
				$online = $this->GetNodeListInCrmLine($line);
			}
			if (strstr($line,': ms_drbd_r0')!==false)
			{
				$section = 'drbd';
			}
			if (strstr($line,': apgrp')!==false)
			{
				$section = 'grp';
			}
			if (strstr($line,': ms_postgresql')!==false)
			{
				$section = 'db';
			}
			if (startsWith($line,'* Node '))
			{
				$section = 'nodeatt';
				$tline = trim($line);
				$nodenm = substr($tline,7,strlen($tline)-8);
				$nodeatt[$nodenm] = array();
			}
			if ($section=='nodeatt')
			{
				if (strstr($line,'+ postgresql-data-status ')!==false)
				{
					$nodeatt[$nodenm]['pgdat'] = $this->GetNodeAttrInCrmLine($line);
				}
				if (strstr($line,'+ postgresql-status ')!==false)
				{
					$nodeatt[$nodenm]['pg'] = $this->GetNodeAttrInCrmLine($line);
				}
			}
			if ($section=='drbd')
			{
				if (strstr($line,'Masters: [')!==false)
				{
					$drbdMasters = $this->GetNodeListInCrmLine($line);
				}
				if (strstr($line,'Slaves: [')!==false)
				{
					$drbdSlaves = $this->GetNodeListInCrmLine($line);
				}
			}
			if ($section=='db')
			{
				if (strstr($line,'Masters: [')!==false)
				{
					$dbMasters = $this->GetNodeListInCrmLine($line);
				}
				if (strstr($line,'Slaves: [')!==false)
				{
					$dbSlaves = $this->GetNodeListInCrmLine($line);
				}
			}
		}
		// now parse DRBD status
		$drbd_info = array();
		$drbd_info['cs'] = '?';
		$drbd_info['ro'] = '?';
		$drbd_info['ds'] = '?';
		foreach ($drbdLines as $line)
		{
			if (strstr($line,' cs:')!==false)
			{
				$kvs = explode(' ',$line);
				foreach ($kvs as $kvt)
				{
					$kv = explode(':',$kvt);
					if (count($kv)!=2) continue;
					$key = $kv[0];
					$value = $kv[1];
					//$sls = explode('/',$value);
					$drbd_info[$key] = $value;
				}
			}
		}
		// crm and drbd parsed,
		// now tell:
		$this->Add( array
			('part'=>'Nodes online'
			,'qua'=>'txt'
			,'value'=>implode(', ',$online)
			,'h'=>(count($online)>=2)
			,'id'=>'cluster_nodesonline'
			,'am'=>'Less than two nodes are online'
      ,'notnode'=>true // this does not effect node health (it is overal cluster health)
			) 
		);
		$this->Add( array
			('part'=>'DRBD Master'
			,'qua'=>'txt'
			,'value'=>implode(', ',$drbdMasters)
			,'h'=>(count($drbdMasters)==1)
			,'id'=>'cluster_drbdmaster'
			,'am'=>'The number of DRBD Masters is not 1'
      ,'notnode'=>true
			) 
		);
		$this->Add( array
			('part'=>'DRBD Slave'
			,'qua'=>'txt'
			,'value'=>implode(', ',$drbdSlaves)
			,'h'=>(count($drbdSlaves)==1)
			,'id'=>'cluster_drbdslave'
			,'am'=>'The number of DRBD Slaves is not 1'
      ,'notnode'=>true
			) 
		);
		$this->Add( array
			('part'=>'Database Master'
			,'qua'=>'txt'
			,'value'=>implode(', ',$dbMasters)
			,'h'=>(count($dbMasters)==1)
			,'id'=>'cluster_dbmaster'
			,'am'=>'The number of Database Masters is not 1'
      ,'notnode'=>true
			) 
		);
		$this->Add( array
			('part'=>'Database Slave'
			,'qua'=>'txt'
			,'value'=>implode(', ',$dbSlaves)
			,'h'=>(count($dbSlaves)==1)
			,'id'=>'cluster_dbslave'
			,'am'=>'The number of Database Slaves is not 1'
      ,'notnode'=>true
			) 
		);
		foreach ($nodeatt as $nodenm => $attmap)
		{
      if ($nodenm == $node_name)
      {
        // this is our node
        if ($attmap['pgdat']=='LATEST') $this->node_db = true;
        if ($attmap['pgdat']=='STREAMING|SYNC') $this->node_db = true;
        if ($attmap['pgdat']=='STREAMING|ASYNC') $this->node_db = true;
      }
			if (in_array($nodenm,$dbMasters))
			{
				// this is the Database Master
				$status = $attmap['pg'] . ', ' . $attmap['pgdat'];
				$good = false;
				if ($attmap['pg']=='PRI'
				&& $attmap['pgdat']=='LATEST')
				{
					$good = true;
				}
				$this->Add( array
					('part'=>'Database Master status'
					,'qua'=>'txt'
					,'value'=>$status
					,'h'=>$good
					,'id'=>'cluster_dbslavestatus'
					,'am'=>'The Database Master is not working'
          ,'notnode'=>true
					) 
				);
			}
			if (in_array($nodenm,$dbSlaves))
			{
				// this is the Database Slave
				$status = $attmap['pg'] . ', ' . $attmap['pgdat'];
				$good = false;
				if ($attmap['pg']=='HS:sync'
				&& $attmap['pgdat']=='STREAMING|SYNC')
				{
					$good = true;
				}
				if ($attmap['pg']=='HS:async'
				&& $attmap['pgdat']=='STREAMING|ASYNC')
				{
					$good = true;
				}
				$this->Add( array
					('part'=>'Database Slave status'
					,'qua'=>'txt'
					,'value'=>$status
					,'h'=>$good
					,'id'=>'cluster_dbslavestatus'
					,'am'=>'The Database Slave is not working or synchronized'
          ,'notnode'=>true
					) 
				);
			}
		}
		$this->Add( array
			('part'=>'DRBD Connection'
			,'qua'=>'txt'
			,'value'=>$drbd_info['cs']
			,'h'=>($drbd_info['cs']=='Connected')
			,'id'=>'cluster_drbd_cs'
			,'am'=>'DRBD not connected'
      ,'notnode'=>true
			) 
		);
		$this->Add( array
			('part'=>'DRBD Roles'
			,'qua'=>'txt'
			,'value'=>$drbd_info['ro']
			,'h'=>( $drbd_info['ro']=='Primary/Secondary' || $drbd_info['ro']=='Secondary/Primary')
			,'id'=>'cluster_drbd_ro'
			,'am'=>'DRBD Roles incorrect or incomplete'
      ,'notnode'=>true
			) 
		);
		$this->Add( array
			('part'=>'DRBD Data Synchronisation'
			,'qua'=>'txt'
			,'value'=>$drbd_info['ds']
			,'h'=>($drbd_info['ds']=='UpToDate/UpToDate')
			,'id'=>'cluster_drbd_ds'
			,'am'=>'DRBD data not synchronized'
      ,'notnode'=>true
			) 
		);
		// end of clustering check
	}

	function AzGroupSyncCheck()
	{
		global $dbintf;
		// get all groups
		// check if Azure AD sync is enabled for any group
		$any = false;
		$groups = $dbintf->GetGroupList();
		foreach ($groups as $group)
		{
			if ($group['azuregroupid']!='')
			{
				$any = true;
			}
		}
		if ($any==false) return;
		
		if ( isset( $this->configs_n['AzGroupSyncHealth'] ) == false)
		{
			// not filled in yet
			return;
		}

		$part = 'Azure group sync';
		$id = 'AzGroupSync';
		$value = $this->configs_n['AzGroupSyncHStatus'];
		$health = ((int) $this->configs_n['AzGroupSyncHealth'])==1;
		$am = 'Azure group sync failed';
		
		$this->Add( array('part'=>$part
			, 'qua'=>'txt'
			, 'value'=>$value
			, 'h'=>$health
			, 'id'=>$id
			, 'am'=>$am ) );		
		
	}
	
	function LdapCheck()
	{
		global $dbintf;
		// get all groups
		// check if LDAP sync is enabled for any group
		$adGroups = $dbintf->getADGroupLinks();
		if ( count($adGroups)==0 ) return;
		
		if ( isset( $this->configs_n['LdapHealth'] ) == false)
		{
			// not filled in yet
			return;
		}

		$part = 'LDAP group sync';
		$id = 'LdapSync';
		$value = $this->configs_n['LdapHStatus'];
		$health = ((int) $this->configs_n['LdapHealth'])==1;
		$am = 'LDAP failed';
		
		$this->Add( array('part'=>$part
			, 'qua'=>'txt'
			, 'value'=>$value
			, 'h'=>$health
			, 'id'=>$id
			, 'am'=>$am ) );		
		
	}
	
	function ExportCheck()
	{
		global $dbintf;
		
		// is export enabled
		
		$part = 'Export status';
		$id = 'ExportStatus';
		$expcfg = $dbintf->LoadConfigsPart('exp');
		if (isset($expcfg['ExpRecs'])==false || $expcfg['ExpRecs']==0)
		{
			// export is not configured or enabled
			$this->newinfo[] = array('part'=>$part,'id'=>$id,'qua'=>'txt','value'=>'not enabled','h'=>null,'err'=>3);
		} else if (isset($expcfg['ExpHStatus'])==false)
		{
			// export is not checked
			$this->newinfo[] = array('part'=>$part,'id'=>$id,'qua'=>'txt','value'=>'not checked','h'=>null,'err'=>3);
		} else
		{

			$value = $expcfg['ExpHStatus'];
			$health = ((int) $expcfg['ExpHealth'])==1;
			$am = 'Export failed';
			// $checkTime = (int) $dbintf->GetConfigValue('ExpTime');
			
			$this->Add( array('part'=>$part
				, 'qua'=>'txt'
				, 'value'=>$value
				, 'h'=>$health
				, 'id'=>$id
				, 'am'=>$am ) );
		}
		
		// check tenant export
		$MultiTenancy = $dbintf->GetCachedConfigValue('MultiTenancy')!=0;
		if ($MultiTenancy)
		{
			$tnConfig = $dbintf->LoadConfigsPart('tn');
			
			$tenants = $dbintf->GetTenantList();
			foreach ($tenants as $tenant)
			{
				$tnId = $tenant['id'];
				$tnName = $tenant['name'];
				$prefix = 'TN' . $tnId . '_';
				if (int_ensure_defined( $tnConfig[ $prefix . 'ExpEnabled'] ) != 1)
				{
					// exporting is not enabled for this tenant
					continue;
				}
				// now check status
				$part = 'Export for ' . $tnName;
				$id = 'ExportStatusTN' . $tnId;
				if (isset($tnConfig[ $prefix . 'ExpHStatus'])==false)
				{
					$this->newinfo[] = array('part'=>$part,'id'=>$id,'qua'=>'txt','value'=>'not checked','h'=>null,'err'=>3);
				} else
				{
					$value = $tnConfig[ $prefix . 'ExpHStatus'];
					$health = ((int) $tnConfig[ $prefix . 'ExpHealth'])==1;
					$am = 'Tenant export failed';
					$this->Add( array('part'=>$part
					, 'qua'=>'txt'
					, 'value'=>$value
					, 'h'=>$health
					, 'id'=>$id
					, 'am'=>$am ) );
				}
				
			}
		}
		
	}
	
	function BackupCheck()
	{
		global $dbintf;
		// is backup enabled
		$bckcfg = $dbintf->LoadConfigsPart('b');
		$part = 'Backup status';
		$id = 'BackupStatus';
		if (isset($bckcfg['BckAct'])==false || $bckcfg['BckAct']==0)
		{
			// backup is not configured
			$this->newinfo[] = array('part'=>$part,'id'=>$id,'qua'=>'txt','value'=>'not enabled','h'=>null,'err'=>3);
			return;
		}
		$status = @file_get_contents($this->datapath . '/backup_status');
		if ($status=='')
		{
			// backup not completed or erred (so we dont know)
			$this->newinfo[] = array('part'=>$part,'id'=>$id,'qua'=>'txt','value'=>'unknown','h'=>null,'err'=>3);
			return;
		}
		$spacepos = strpos($status,' ');
		$good = false;
		if ($spacepos===false)
		{
			// should never happen
			$value = 'invalid';
		} else
		{
			$errcode = substr($status,0,$spacepos);
			if ($errcode=='0') $good = true;
			$value = substr($status,$spacepos+1);
			$this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$good,'id'=>$id,'am'=>'Backup failed: '.$value) );
		}
	}
	
  function ResourceCheck()
  {
    $loadavg = @file('/proc/loadavg');
    // line 1, split in parts
    $parts = explode(' ',$loadavg[0]);
    // 1 min. 5 min. 15 min.
    $value = $parts[1];
    
    $this->newinfo[] = array('part'=>'CPU load','qua'=>'res','value'=>$value,'h'=>null,'err'=>3 ,'id' => 'cpuload');

    dbglog("Debian version: " . $this->GetDebianVersion() );
    
    $foundMemInfo = false;
    if ($this->GetDebianVersion()>=8)
    {
      $meminfo = array();
      if (!$this->simulate)
      {
        exec('cat /proc/meminfo | grep MemAvailable',$meminfo);
      } else
      {
        $memLines = <<<SIMDATA
MemAvailable:    2200300 kB
SIMDATA;
        $meminfo = explode("\n",$memLines);
      }
      if (count($meminfo)>=1)
      {
        $parts = preg_split('/[\s]+/',$meminfo[0],-1,PREG_SPLIT_NO_EMPTY);
        $kbytes = $parts[1];
        $value = round($kbytes/1024);
        $foundMemInfo = true;
      }
    } 
    if ($foundMemInfo==false)
    {
      $freeinfo = array();
      if (!$this->simulate)
      {
        exec('free',$freeinfo);
      } else
      {
        $freeLines = <<<SIMDATA
             total       used       free     shared    buffers     cached
Mem:       4060320    3332328     727992          0     404044    2418908
-/+ buffers/cache:     509376    3550944
Swap:      7811056      66428    7744628
SIMDATA;
        $freeinfo = explode("\n",$freeLines);
      }
      $kbytes = -1;
      if (substr($freeinfo[2],0,11)=='-/+ buffers')
      {
        $parts = preg_split('/[\s]+/',$freeinfo[2],-1,PREG_SPLIT_NO_EMPTY);
        $kbytes = $parts[3];
      } else
      {
        // Debian 9 lacks buffers/cache line
        // so we take the first line which is now
        // already takes this into account
        $parts = preg_split('/[\s]+/',$freeinfo[1],-1,PREG_SPLIT_NO_EMPTY);
        $kbytes = $parts[3];
      }
      $value = round($kbytes / 1024);
    }

		$good = ($value > 300);
		$this->Add( array('part'=>'Free application memory','qua'=>'num','value'=>$value.' MB','h'=>$good,'id'=>'appmemfree', 'am'=>'Low on memory (try restarting)') );
  }

	function MaxPeakCalls()
	{
		global $dbintf;
		
		$configs_s = $this->main_configs_s;
		$PeakCalls = int_ensure_defined($configs_s['iaPeakCalls']);	//Percentage.
		if ($PeakCalls==0)
		{
			return;
		}
		//Get the number of VoIP-channels.
		$NumOfChan = (int)$dbintf->GetConfigValue('ChCnt');
		if ($NumOfChan==0)
		{
			return;
		}
		$CurrentHour = (int)(time() / 3600);
		$SimCallsThisHour = (int)$dbintf->GetSimCallsForHour($CurrentHour);
		
		$good = true;
		$PercentageSimCalls = ($SimCallsThisHour * 100 / $NumOfChan);
		$PercentageSimCalls_rounded = round($PercentageSimCalls);
		$value = "$SimCallsThisHour calls ($PercentageSimCalls_rounded % of $NumOfChan)";
		if($PercentageSimCalls > $PeakCalls) $good = false;
    // notnode: license overflow is probably a general problem
		$this->Add( array('part'=>'Max. simultaneous VoIP calls (this hour)','qua'=>'num','value'=>$value,'h'=>$good,'id'=>'maxsimcalls','am'=>'Too many simultaneous calls.','notnode'=>true));
	}
	
  function RecIntfCheck()
  {
    global $dbintf, $configs_crec;
    $configs_crec = $dbintf->LoadConfigsPart('crec');
    $configs_crec_line = $dbintf->LoadConfigsPart('crec_line');
    $CardList = str_ensure_defined($configs_crec['CardList']);
    if ($CardList=='') $CardList = '';
    $PrevCardList = $configs_crec['PrevCardList'];
    if ($PrevCardList=='') $PrevCardList = '';
    //echo 'CardList: ' . $CardList . "\r\n";
    //echo 'PrevCardList: ' . $PrevCardList . "\r\n";
    // check if changed
    $CardList_Arr = explode(',',$CardList);
    $PrevCardList_Arr = explode(',',$PrevCardList);
    $good = true;
    $value = '';
    $qua = 'txt';
    $cardcnt = 0;
    foreach ($CardList_Arr as $Card)
    {
      $Card = trim($Card);
      if ($Card=='') continue;
      if (isset($configs_crec[$Card . '_Present']))
      {
        if ($configs_crec[$Card . '_Present']==0) continue;
      }
      $cardcnt++;

      $ch = 0;
      if (isset($configs_crec[$Card . '_Channels']))
      {
        $ch = (int) $configs_crec[$Card . '_Channels'];
      }
      $prevch = 0;
      if (isset($configs_crec['Prev_' . $Card . '_Channels']))
      {
        $prevch = (int) $configs_crec['Prev_' . $Card . '_Channels'];
      }
      dbglog($Card . ' ' . $ch . ' ' . $prevch);
      if ($prevch>$ch)
      {
        $good = false;
        $value .= 'Card ' . $Card . ' lost ' . ($prevch-$ch) . ' channels. ';
        //echo $value;
      }
    }
    // check if each card is still there
    foreach ($PrevCardList_Arr as $PrevCard)
    {
      $PrevCard = trim($PrevCard);
      //echo "'" . $PrevCard . "'\r\n";
      if (strlen($PrevCard)==0) continue;

      if (isset($configs_crec['Prev_' . $PrevCard . '_Present']))
      {
        if ($configs_crec['Prev_' . $PrevCard . '_Present']==0)
        {
          continue;
        }
      }

      // check if still there
      $isThere = false;
      foreach ($CardList_Arr as $Card)
      {
        $Card = trim($Card);
        if ($configs_crec[$Card . '_Present']==0)
        {
          continue;
        }
        if ($Card==$PrevCard) $isThere = true;
      }
      if (!$isThere)
      {
        $good = false;
        $value .= 'Failed/missing card ' . $PrevCard . '. ';
      }
    }
    if ($good)
    {
      $newconfigs = array();
      $newconfigs['PrevCardList'] = $CardList;

      foreach ($CardList_Arr as $Card)
      {
        $Card = trim($Card);
        if ($Card=='') continue;
        $ch = 0;
        if (isset($configs_crec[$Card . '_Channels']))
        {
          $ch = (int) $configs_crec[$Card . '_Channels'];
        }
        $newconfigs['Prev_' . $Card . '_Channels'] = '' . $ch;
        $present = 1;
        if (isset($configs_crec[$Card . '_Present']))
        {
          $ch = (int) $configs_crec[$Card . '_Present'];
        }
        $newconfigs['Prev_' . $Card . '_Present'] = '' . $ch;
      }

      $dbintf->SetConfigsPart($newconfigs,'crec');

      $qua = 'num';
      $value = '' . $cardcnt;
    }
    $this->Add( array('part'=>'Recording cards','qua'=>$qua,'value'=>$value,'h'=>$good,'id'=>'reccard','am'=>'Failed/missing recording card') );

    foreach ($CardList_Arr as $Card)
    {
      $Card = trim($Card);
      if ($Card=='') continue;

        if (substr($configs_crec[$Card . '_Model'],0,3)=='DTP')
        {
          dbglog("PRI card ".$Card);
          // PRI-line
          if (isset($configs_crec_line[$Card . '_Line']))
          {
            $goodsub = ((int) $configs_crec_line[$Card . '_Line'])!=0;
            if ($goodsub) $value = '';
            else $value = 'unavailable';
            $this->Add( array('part'=>$Card.' PRI line','qua'=>'txt','value'=>$value,'h'=>$goodsub,'id'=>$Card . '_Line','am'=>'PRI line is down'));
          }
        }
    }

		$lines_info = $dbintf->GetLineList();
		$line_names = array();
		foreach($lines_info as $line_info)
		{
			$line_names[ $line_info['linenr'] ] = $line_info['name'];
		}

		// Check Line Status
		dbglog('Check Line Status start');
		// Read line status file
		// This file is created by card recording engine (crec)
		$LineStatusArr = ReadDataFile($this->datapath . '/linestatus.dat');
		//var_dump( $LineStatusArr );
		// Loop through cards and channels,
		// and see for which channel,
		// check line status is enabled.
    foreach ($CardList_Arr as $Card)
    {
			$Card = trim($Card);
			if ($Card=='') continue;
			if ($configs_crec[$Card . '_Present']==0)
			{
				continue;
			}
			//dbglog($Card);
			// get properties of card
      $startch = (int) $configs_crec[$Card . '_StartCh'];
      $channels = (int) $configs_crec[$Card . '_Channels'];
			// get the setting
      $cls_list = $configs_crec[$Card . '_CheckLineStatus' ];
      //dbglog($cls_list);
      $cls_list = explode(',',$cls_list);
      // loop through the channels of this card
      for ($ch=1;$ch<=$channels;$ch++)
      {
        $linenr = $startch + $ch;
				// cls = Check Line Status (the setting)
        $cls = false;
        if (isset($cls_list[$ch-1]))
        {
          $cls = ( (int) $cls_list[$ch-1] ) != 0;
        }
				if ($cls)
				{
					// Check Line Status is enabled
					$status = 'unknown';
					// Lookup current status
					$key = 'Line_' . $linenr;
					if (isset($LineStatusArr[ $key ]))
					{
						$status = $LineStatusArr[ $key ];
					}
					// Load line name for better reports
					$extra_name = '';
					if (isset($line_names[$linenr]))
					{
						$extra_name = ' (' . $line_names[$linenr] . ')';
					}
					$lineDescr = "channel $ch of card $Card$extra_name";
					$extrastatus = $status;
					if ($status=='ok') $extrastatus = 'ok';
					if ($status=='error') $extrastatus = 'error (signal interpretation or signal quality error)';
					if ($status=='offline') $extrastatus = 'offline (line is disconnected or phone unavailable)';
					dbglog($lineDescr . ' = ' . $status);
					$this->Add( array
						('part'=>"Line status of $lineDescr"
						,'qua'=>'txt'
						,'value'=>$status
						,'h'=>($status == 'ok')
						,'id'=>'linestatus_' . $linenr
						,'am'=>"Line status of $lineDescr is $extrastatus"
						)
					);
				}
			}
		}
		
  }

  function LicensingCheck()
  {
    global $dbintf;
    if (file_exists(dirname(__FILE__) . '/../apresa/conf/opt_so'))
    {
      $configs_s = $this->main_configs_s;
			if (int_ensure_defined($configs_s['LicViewer'])!=0)
			{
				return;
			}
      $good = false;
      if ($configs_s['LicSystem']>=1 || $configs_s['LicTrial']>=1)
      {
        $good = true;
      }
      $part = 'Licensing';
      $value = '';
      $itemname = 'license';
      $errmsg = 'Not licensed correctly';
      $this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$good,'id'=>$itemname,'am'=>$errmsg));
    }
  }

  function JtapiCheck()
  {
    dbglog("JtapiCheck");
		global $dbintf;
    $configs_o = $dbintf->LoadConfigsPart('o');
		if(int_ensure_defined($configs_o['CiscoArZeroPfOut'])!=2)
		{
      dbglog("Jtapi not in use");
      // JTAPI not in use
			return;
		}
    $path = '/var/apresa/data/cj-status.dat';
    if (COM_TYPE=='Win')
    {
      $path = $this->datapath . '/cj-status.dat';
    }
    dbglog($path);
    $good = false; $value = '';
    if(file_exists($path)==false)
    {
      $value = 'Status file not found';
    } else
    {
      dbglog('found');
      $content = file($path);
      if (count($content)<1)
      {
        $value = 'Status file has unexpected format';
      } else
      {
        $line = $content[0];
        echo $line;
        $spacepos = strpos($line,' ');
        if ($spacepos===false)
        {
          $value = 'Status file has unexpected format';
        } else
        {
          $statuscode = substr($line,0,$spacepos);
          if ($statuscode=='0') $good = true;
          $statusmsg = substr($line,$spacepos+1);
          $value = 'Status code ' . $statuscode . '. ' . trim($statusmsg);
          if ($statuscode==0) $value = '';
        }
      }
    }
		$errmsg = 'Cisco JTAPI not OK';
		$this->Add(array('part'=>'Cisco JTAPI','qua'=>'txt','value'=>$value,'h'=>$good,'id'=>'cstalink','am'=>$errmsg));
  }
  
	function CSTALinkCheck()
	{
		global $dbintf;
    $configs_o = $dbintf->LoadConfigsPart('o');
		if(int_ensure_defined($configs_o['CSTAType'])==0)	//CSTA not in use.
		{
			$good = true;
			$value = 'Not in use';
		}
		else
		{
			if(file_exists('/var/apresa/data/csta_status'))
			{
				$lines = file('/var/apresa/data/csta_status');
        $good = true;
        $fvalue = '';
        foreach ($lines as $line)
        {
          $status = explode("=", $line);
          if(count($status) != 2) continue; // invalid
          if($status[1] == 0)
          {
            $value = 'OK';
          }
          else
          {
            $good = false;
            $extError = explode(";", $status[1]);
            if(count($extError) == 2)
            {
              $status[1] = $extError[0];
              $errDesc = $extError[1];
            }
            else $errDesc = "";
            switch($status[1])
            {
              case 1:
                $value = 'Port unknown or incorrect!';
              break;
              case 2:
                $value = 'Server address invalid!';
              break;
              case 3:
                $value = 'Connection failed!';
              break;
              case 4:
                $value = 'Login rejected-permanent';
              break;
              case 5:
                $value = 'Login rejected-transient';
              break;
              case 6:
                $value = 'Restarting connection because lack of response from CSTA server';
              break;
              case 20:
                $value = 'Program closed';
              break;
              case 100:
                $value = 'Initializing';
              break;
              case 101:
                $value = 'Authentication error!';
              break;
              case 102:
                $value = 'Monitor failed!';
              break;
              case 103:
                $value = 'SSC failed! '.$errDesc;
              break;
              case 104:
                $value = 'Recorder login failed! '.$errDesc;
              break;
              case 105:
                $value = 'Recorder registration failed! '.$errDesc;
              break;
              default:
                $value = 'Unknown error!';
              break;
            }
          }
          if ($fvalue!='') $fvalue .= ', ';
          $fvalue .= $value;
				} // lines
        $value = $fvalue;
      }
			else
			{
				$good = false;
				$value = 'Csta_status file not found!';
			}
		}
		$errmsg = 'CSTA link not OK';
		$this->Add(array('part'=>'CSTA Link','qua'=>'txt','value'=>$value,'h'=>$good,'id'=>'cstalink','am'=>$errmsg));
	}
	
  function VoIPCheck()
  {
    if ($this->simulate) return;

    global $dbintf;
    $this->VoIPnetwChk = ((int) $dbintf->GetConfigValue('VoIPnetwChk')) != 0;
    if ($this->VoIPnetwChk)
    {
      $VoIPnetw = (int) $dbintf->GetConfigValue('VoIPnetw');
      dbglog('VoIPnetw: ' . $VoIPnetw);
      
      $intfs = GetNetwInterfaces(); // function from utility.php
      
      dbglog('Primary:   ' . $intfs[0]);
      dbglog('Secondary: ' . $intfs[1]);
      
      $VoIPintf = '';
      
      if ($VoIPnetw==0) $VoIPintf = $intfs[0];
      if ($VoIPnetw==1) $VoIPintf = $intfs[1];
      
      if ($VoIPintf=='') return;
      
      $t = trim(file_get_contents('/sys/class/net/' . $VoIPintf . '/carrier'));
      $carrier = (int) $t;
      $good = ($carrier==1);
      $value = '';
      $qua = 'txt';
      if ($good) {$qua='num';$value = trim(file_get_contents('/sys/class/net/' . $VoIPintf . '/speed')) . ' Mbps';}
      else $value = 'down';
      dbglog("Good: " . ($good?1:0));
      $this->Add( array('part'=>'VoIP interface ' . $VoIPintf, 'qua'=>$qua,'value'=>$value,'h'=>$good,'id'=>'voip_eth','am'=>'VoIP interface ' . $VoIPintf . ' is down'));

      //dbglog(formatBytes(123));
      //dbglog(formatBytes(123123));
      //dbglog(formatBytes(123123123));
      //dbglog(formatBytes(123123123123));

      if (file_exists('/sys/class/net/' . $VoIPintf . '/statistics/rx_packets'))
      {
        $value = (int) trim(file_get_contents('/sys/class/net/' . $VoIPintf . '/statistics/rx_packets'));
        //$value = formatBytes($value);
        $this->newinfo[] = array('part'=>'Packets received on ' . $VoIPintf,'qua'=>'num','value'=>$value,'h'=>null,'err'=>3);
      }

      if (file_exists('/sys/class/net/' . $VoIPintf . '/statistics/rx_dropped'))
      {
        $rx_dropped = (int) trim(file_get_contents('/sys/class/net/' . $VoIPintf . '/statistics/rx_dropped'));
        if (file_exists('/var/apresa/data/eth_drop'))
        {
          $old_rx_dropped = (int) trim(file_get_contents('/var/apresa/data/eth_drop'));
          $value = $rx_dropped - $old_rx_dropped;
          $good = ($value == 0);
          $this->Add( array('part'=>'Network packet drops of '.$VoIPintf,'qua'=>'num','value'=>$value,'h'=>$good, 'id'=>'voip_drop','am'=>'VoIP interface ' . $VoIPintf . ' dropping packets'));
        }
        file_put_contents('/var/apresa/data/eth_drop',$rx_dropped."\n");
      }

    }

  }

  function GetBoardOfCh($linenr)
  {
    global $configs_crec;

    $cardlist = $configs_crec['CardList'];
    $cardlist = explode(',',$cardlist);
    foreach ($cardlist as $card)
    {
      //if ($card[''];
      if (!$configs_crec[$card . '_Present']) return null;
      $startch = (int) $configs_crec[$card . '_StartCh'];
      $channels = (int) $configs_crec[$card . '_Channels'];

      if ($linenr>=$startch+1 && $linenr<$startch+1+$channels)
      {
        //dbglog($card . ' ' . $startch . ' ' . $channels);
        return $card;
      }

    }

    return '???';
  }

  function havePostProcessItems()
  {
    if ($this->addedPostProcess==false)
    {
      $this->addedPostProcess = true;
      $this->AddCaption('Post-processing');
    }
  }
  
	function ScribeCheck()
	{
		global $dbintf;
    
    dbglog('ScribeCheck');
		
    $plans = $dbintf->GetTScriptPlans();
		$any = false;
		foreach ($plans as $plan)
		{
			if ($plan['enabled']==0) continue;
      $this->havePostProcessItems();
			break;
		}
		
		foreach ($plans as $plan)
		{
			if ($plan['enabled']==0) continue;
			$tstatus = (int) $plan['tstatus'];
			dbglog('tplan: '.$plan['name'].' '.$tstatus);
			$good = ($tstatus != 3);
			$value = '';
			if ($good==false) $value = $plan['errmsg'];
			$part = 'Transcription task: '.$plan['name'];
			$itemname = 'TScriptPlan' . $plan['id'];
			$errmsg = 'Error transcribing '.$plan['name'].': '.$plan['errmsg'];
			$this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$good,'id'=>$itemname,'am'=>$errmsg));
		}
	}
  
	function TransferCheck()
	{
		global $dbintf;
		
		$targets = $dbintf->GetTransferTargets();
		$any = false;
		foreach ($targets as $target)
		{
			if ($target['enabled']==0) continue;
			$this->havePostProcessItems();
			break;
		}
		
		foreach ($targets as $target)
		{
			if ($target['enabled']==0) continue;
			$status = (int) $target['status'];
			dbglog('T: '.$target['description'].' '.$status);
			$good = ($status < 100);
			$value = '';
			if ($good==false) $value = $target['errmsg'];
			$part = 'Export to server: '.$target['description'];
			$itemname = 'TransferTarget' . $target['id'];
			$errmsg = 'Error exporting to other Apresa '.$target['description'].': '.$target['errmsg'];
			$this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$good,'id'=>$itemname,'am'=>$errmsg));
		}
    
    // also some spin check
    if ($dbintf->GetConfigValue('SpinClient')==1)
    {
      $status = $dbintf->GetConfigValue('SpinClientStatus');
      $good = startsWith($status,'OK');
      $errmsg = 'Active call connector: ' . $status;
      $this->Add( 
        array('part'=>'Active call connector'
        ,'qua'=>'txt'
        ,'value'=>$status
        ,'h'=>$good
        ,'id'=>'SpinClient'
        ,'am'=>$errmsg
        ) );
    }
    
	}
	
  function InactivityCheck($tenantid, $tenantname, $useDefault)
  {
    global $dbintf, $configs_crec;

		$configs_s = $this->main_configs_s;

    if ( ! int_ensure_defined($configs_s['InactivityAlarm']))
    {
      dbglog('InactivityAlarm not enabled');
      return;
    }

    $settings = array();
		if($tenantid==-1 || $useDefault)
		{
			$settings['weekdaylist'] = $configs_s['iaWeekdays'];
			$InactPeriodSystem  = $configs_s['iaPeriodSystem'];
			$InactPeriodSystemApresa = $configs_s['iaPeriodSystemApresa'];
			$InactPeriodChannel = $configs_s['iaPeriodChannel'];
			$InactPeriodOtherRec = $configs_s['iaPeriodOtherRec'];
			$MaxCallLength = $configs_s['iaMaxCallLength'];
			$start1 = $configs_s['iaStart1'];
			$stop1 = $configs_s['iaStop1'];
			$start2 = $configs_s['iaStart2'];
			$stop2 = $configs_s['iaStop2'];
		}
		else
		{
			$lookupkey = 'TN'.$tenantid.'_iaWeekdays';
			$settings['weekdaylist'] = $dbintf->GetConfigValue($lookupkey);
			$lookupkey = 'TN'.$tenantid.'_iaPeriodSystem';
			$InactPeriodSystem = $dbintf->GetConfigValue($lookupkey);
			$lookupkey = 'TN'.$tenantid.'_iaMaxCallLength';
			$MaxCallLength = $dbintf->GetConfigValue($lookupkey);
			$lookupkey = 'TN'.$tenantid.'_iaStart1';
			$start1 = $dbintf->GetConfigValue($lookupkey);
			$lookupkey = 'TN'.$tenantid.'_iaStop1';
			$stop1 = $dbintf->GetConfigValue($lookupkey);
			$lookupkey = 'TN'.$tenantid.'_iaStart2';
			$start2 = $dbintf->GetConfigValue($lookupkey);
			$lookupkey = 'TN'.$tenantid.'_iaStop2';
			$stop2 = $dbintf->GetConfigValue($lookupkey);
		}

    /*
    // settings (simulated)
    $settings['weekdaylist'] = '23456';
    //$settings['weekdaylist'] = '1';
    //$settings['weekdaylist'] = '';
    $start1 =  '8:00'; $stop1 = '12:30';
    $start2 =   'off'; $stop2 =      '';
    //$settings['start2'] = '13:30'; $settings['stop2'] = '17:30';
    // in minutes
    $InactPeriodSystem  = 15;
		$InactPeriodSystemApresa = 15;
		$InactPeriodOtherRec = 15;
    $InactPeriodChannel = 2 * 60;
    $MaxCallLength = 60;
    */

    // parse settings

    $settings['start1'] = $this->ParseTimeOfDay($start1);
    $settings['stop1' ] = $this->ParseTimeOfDay($stop1 );
    $settings['start2'] = $this->ParseTimeOfDay($start2);
    $settings['stop2' ] = $this->ParseTimeOfDay($stop2 );

    // now it is in seconds
    $MaxCallLength_s = int_ensure_defined($MaxCallLength) * 60;

    $settings['tocheck_seconds'] = int_ensure_defined($InactPeriodSystem) * 60;
		if($settings['tocheck_seconds']>0)
		{
			// do system wide check
			$checkstart = $this->GetInactivityCheckStartPoint($settings);
			if ($checkstart!==null)
			{
				dbglog("checkstart: ".date("Y-m-d H:i:s D",$checkstart).' '.$checkstart);
				$good = $this->ActivitySince($checkstart-$MaxCallLength_s,$checkstart,-1,-1,$tenantid);		//No line numbers and all recorders!
				if($tenantid==-1)
				{
					$this->Add( array('part'=>'Recording activity (system-wide)','qua'=>'num','value'=>'','h'=>$good,'id'=>'recact_sys','am'=>"No recording activity system-wide during $InactPeriodSystem minutes or more within specified start/stop periods"));
					dbglog('Recording activity (system-wide): '.($good?"OK":"ERR"));
				}
				else
				{
					$this->Add( array('part'=>"Recording activity $tenantname",'qua'=>'num','value'=>'','h'=>$good,'id'=>'recact_tn'.$tenantid,'am'=>"No recording activity for $tenantname during $InactPeriodSystem minutes or more within specified start/stop periods"));
					dbglog("Recording activity $tenantname: ".($good?"OK":"ERR"));
				}
			}
		}
		
		if($tenantid!=-1) return;	//For specific tenants only the system-wide check.
		
		$settings['tocheck_seconds'] = int_ensure_defined($InactPeriodSystemApresa) * 60;
		if($settings['tocheck_seconds']>0)
		{
			// do Apresa check
			$checkstart = $this->GetInactivityCheckStartPoint($settings);
			$serialnum = GlobGetSerialNumber();
			if ( ((int)$serialnum) != 0 )
			{
				$recorderid = $dbintf->GetOrCreateRecorderId($serialnum,"Apresa$serialnum");
				if ($checkstart!==null)
				{
					dbglog('RecorderID of Apresa = '.$recorderid);
					dbglog("checkstart: ".date("Y-m-d H:i:s D",$checkstart).' '.$checkstart);
					$good = $this->ActivitySince($checkstart-$MaxCallLength_s,$checkstart,-1,$recorderid,-1);
					$sysname = $dbintf->GetConfigValue('SystemName');
					if($sysname=='') $sysname='Apresa';
					$this->Add( array('part'=>'Recording activity ('.$sysname.')','qua'=>'num','value'=>'','h'=>$good,'id'=>'recact_apresa','am'=>"No recording activity of the Apresa system during $InactPeriodSystemApresa minutes or more within specified start/stop periods"));
					dbglog('Recording activity (Apresa): '.($good?"OK":"ERR"));
				}
			}
		}
		 
		$settings['tocheck_seconds'] = int_ensure_defined($InactPeriodOtherRec) * 60;
		if($settings['tocheck_seconds']>0)
		{
			//Do check on other recorders
			$checkstart = $this->GetInactivityCheckStartPoint($settings);
			if ($checkstart!==null)
			{
				dbglog("checkstart other recorders: ".date("Y-m-d H:i:s D",$checkstart).' '.$checkstart);
				$OtherRecs = $this->ActivityOtherRecs();
				foreach($OtherRecs as $OtherRec)
				{
					if($dbintf->GetCallCountSpecial($checkstart-$MaxCallLength_s,$checkstart,-1,$OtherRec['recorderid'],-1)>0) $good=true; else $good=false;
					$this->Add( array('part'=>'Recording activity ('.$OtherRec["description"].')','qua'=>'num','value'=>'','h'=>$good,'id'=>'otherrecact_'.$OtherRec['recorderid'],'am'=>"No recording activity of the ".$OtherRec["description"]." during $InactPeriodOtherRec minutes or more within specified start/stop periods"));
					dbglog('Recording activity ('.$OtherRec["description"].'): '.($good?"OK":"ERR"));
				}
			}
		}
		
		$global_iaUserPeriod = int_ensure_defined( $configs_s['iaPeriodUser'] );
    $perUserIaPeriod = int_ensure_defined($configs_s['iaConfigPeriodUser']) > 0;

		if ($global_iaUserPeriod > 0 || $perUserIaPeriod)
		{
      dbglog('User inactivity check');

      $users = $dbintf->GetUserList();
			foreach ($users as $user)
			{
        if($user['iaperiod'] !== -1 && $perUserIaPeriod)
        {
          $settings['tocheck_seconds'] = $user['iaperiod'] * 60;
        }
        else if($global_iaUserPeriod > 0)
        {
          $settings['tocheck_seconds'] = $global_iaUserPeriod * 60;
        }
        else
        {
          continue;
        }

        $inactlen = (int) ($settings['tocheck_seconds'] / 60);
        $checkstart = $this->GetInactivityCheckStartPoint($settings);
        if ($checkstart===null) continue;
         
				$phones = $dbintf->GetUserPhones($user['accid']);
				if (count($phones)==0) continue;
					
				$tenantid = $user['tenantid'];
				if ($tenantid===null) $tenantid = -1;
					
				$good = $this->ActivitySince($checkstart-$MaxCallLength_s,$checkstart,-1,-1,$tenantid,$phones);
					
				dbglog('user: ' . $user['name'] . ' with ' . count($phones) . ' phones , activity? ' . ($good?'yes':'NO'));
				$this->Add( array('part'=>'Recording activity ('.$user['name'].')','value'=>'','h'=>$good, 'id'=>'recact_user_'.$user['id'],'am'=>"No recording activity for user " . $user['name'] . " during $inactlen minutes or more within specified start/stop periods") );
			}
    }

    // do per channel check
    // per channel has other check length
    $settings['tocheck_seconds'] = int_ensure_defined($InactPeriodChannel) * 60;
    // but above is overruled below
    $cardlist = str_ensure_defined($configs_crec['CardList']);
    $cardlist = explode(',',$cardlist);
    foreach ($cardlist as $card)
    {
      if (!int_ensure_defined($configs_crec[$card . '_Present'])) continue;
      $startch = (int) $configs_crec[$card . '_StartCh'];
      $channels = (int) $configs_crec[$card . '_Channels'];

      $inact_list = $configs_crec[$card . '_InactAlarm' ];
      //dbglog($inact_list);
      $inact_list = explode(',',$inact_list);

      $inactlen_list = $configs_crec[$card . '_InactLen' ];
      //dbglog('InactLen list: '.$inactlen_list);
      $inactlen_list = explode(',',$inactlen_list);

      $lines_info = $dbintf->GetLineList();
      $line_names = array();
      foreach($lines_info as $line_info)
      {
        $line_names[ $line_info['linenr'] ] = $line_info['name'];
      }
      
      for ($ch=1;$ch<=$channels;$ch++)
      {
        $linenr = $startch + $ch;
        $inact = false;
        if (isset($inact_list[$ch-1]))
        {
          $inact = $inact_list[$ch-1] != 0;
        }
        $inactlen = '';
        if (isset($inactlen_list[$ch-1]))
        {
          $inactlen = $inactlen_list[$ch-1];
        }
        $inactlen = trim($inactlen);
        if ($inactlen=='')
        {
          $inactlen = $InactPeriodChannel;
        } else
        {
          $inactlen = (int) $inactlen;
        }
        if ($inact)
        {
          dbglog('InactLen: '.$inactlen.' minutes');
          $settings['tocheck_seconds'] = $inactlen * 60;
          $checkstart = $this->GetInactivityCheckStartPoint($settings);
          if ($checkstart!==null)
          {
            dbglog("checkstart: ".date("Y-m-d H:i:s D",$checkstart).' '.$checkstart);

            $channel_name = $card.'.'.$ch;
            $extra_name = '';
            if (isset($line_names[$linenr]))
            {
              $channel_name = $line_names[$linenr];
              $extra_name = ' (' . $line_names[$linenr] . ')';
            }
            dbglog('Card: ' . $card . ' Ch: ' . $ch . ' linenr: ' . $linenr);
            $good = $this->ActivitySince($checkstart-$MaxCallLength_s,$checkstart,$linenr,-1,-1);		//All recorders!
            $this->Add( array('part'=>'Channel recording activity ('.$card.'_'.$ch.')'.$extra_name,'qua'=>'num','value'=>'','h'=>$good, 'id'=>'recact_'.$linenr,'am'=>"No channel recording activity at channel $ch of card $card$extra_name during $inactlen minutes or more within specified start/stop periods"));
            dbglog('Channel recording activity ('.$channel_name.'): '.($good?"OK":"ERR"));
          }
        }
      }
    }

    // now check if there are call busy that are too long
    $good = true;
    $value = '';
    $oldest = $dbintf->GetOldestActiveCall(false);
    if ($oldest)
    {
      $age = time() - $oldest;
      // in seconds
      if ($age>$MaxCallLength_s)
      {
        $good = false;
        dbglog("Active call is too old");
        $value = floor($age / 60) . ' minutes';
      }
    }
    $this->Add( array('part'=>'Recording duration','qua'=>'num','value'=>$value,'h'=>$good,'id'=>'recact_max','am'=>"An active recording has exceeded the maximum duration of $MaxCallLength minutes."));
  }

  function ActivitySince($startdate,$enddate,$linenr,$recorderid,$tenantid,$userphones=false)
  {
    global $dbintf;

    dbglog($startdate . ' ' . $enddate);
    // first check active calls
		if ($userphones!==false)
		{
			$cnt = $dbintf->GetActiveCallsCountPhones($tenantid,$userphones);
			dbglog('active cnt = ' . $cnt );
    } else if ($linenr == -1)
    {
			if($recorderid == -1)
			{
				if($tenantid!=-1) $cnt = $dbintf->GetActiveCallsCountPerTenant($tenantid);
				else $cnt = $dbintf->GetActiveCallsCount();
			}
			else
				$cnt = $dbintf->GetActiveCallsCountForRecorder($recorderid);
    } else
    {
      $cnt = $dbintf->GetActiveCallsCountOfLine($linenr);
    }
    if ($cnt>0) return true;

    // now check finished calls
    $cnt = $dbintf->GetCallCountSpecial($startdate,$enddate,$linenr,$recorderid,$tenantid,$userphones);
    if ($cnt>0) return true;

    return false;
  }

	function ActivityOtherRecs()
	{
			global $dbintf;
			$recs = $dbintf->GetActivityOtherRecs();
			return $recs;
	}
		
  function MinutesToTimestamp($minutes,$info)
  {
    return mktime
    ( floor($minutes/60)
    , $minutes % 60
    , 0
    , $info['mon']
    , $info['mday']
    , $info['year']
    );
  }

  function ParseTimeOfDay($t)
  {
    //dbglog('ParseTimeOfDay: '.$t);
    if ($t=='')
    {
      return null;
    }
    if (!ctype_digit($t[0]))
    {
      $this->healthConfigError('Invalid time of day: '.$t);
      return null;
    }
    //dbglog('It is OK');
    $colon = strpos($t,':');
    if ($colon===FALSE)
    {
      $this->healthConfigError('Invalid time of day: '.$t);
      return null;
    }

    $hours = (int) substr($t,0,$colon);
    $minutes = (int) substr($t,$colon+1);
    //dbglog($hours.' '.$minutes);

    return ($hours * 60) + $minutes;
  }

  function TimePeriodParse($start,$stop, & $checkstart,& $tocheck_seconds_ref)
  {
      $checkstart_info = getdate($checkstart);

      // is checkstart_minutes inside this range?
      $start = $this->MinutesToTimestamp($start,$checkstart_info);
      $stop  = $this->MinutesToTimestamp($stop ,$checkstart_info);

      dbglog('Activity period: '.date('Y-m-d H:i:s',$start).' => '.date('Y-m-d H:i:s',$stop));

      if ($checkstart>$stop)
      {
        $checkstart = $stop;
      }
      if ($checkstart>$start)
      {
        // we have found a period to check
        // how large is this period?
        $period_size_seconds = $checkstart - $start;
        if ($period_size_seconds > $tocheck_seconds_ref)
        {
          // there is more available than needed (usual case)
          // calculate from where to start
          $checkstart -= $tocheck_seconds_ref;
          // nothing more to check
          $tocheck_seconds_ref = 0;
        } else
        {
          // we need to check also other periods (corner case)
          $checkstart = $start;
          // we need to check some less, but not ready yet
          $tocheck_seconds_ref -= $period_size_seconds;
        }
      }
  }

  function healthConfigError($msg)
  {
    dbglog($msg);
    $part = 'Alarm configuration';
    $value = $msg;
    $itemname = 'hconfig';
    $good = false;
    $errmsg = $msg;
    $this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$good, 'id'=>$itemname,'am'=>$errmsg));
  }

  function GetInactivityCheckStartPoint($settings)
  {
    dbglog("GetInactivityCheckStartPoint");

     // get time of day, and day of week

    $now = time();
    //$now = mktime(0,0,0,10,22,2012);
    dbglog("Now: ".date("Y-m-d H:i:s",$now));
    //exit;

    $start1 = $settings['start1'];
    $stop1  = $settings['stop1'];
    $start2 = $settings['start2'];
    $stop2  = $settings['stop2'];
    $tocheck_seconds = $settings['tocheck_seconds'];
    $weekdaylist = $settings['weekdaylist'];

    $checkstart = $now;
    $first = true;

    while ($tocheck_seconds>0)
    {
      dbglog("tocheck minutes: ".floor($tocheck_seconds/60));
      $prev_tocheck_seconds = $tocheck_seconds;
      // go back to a time in latest activity period
      // first go back to a date that is in weekday list
      for ($i=0;$i<7;$i++)
      {
        $checkstart_info = getdate($checkstart);
        if (strstr($weekdaylist,''.($checkstart_info['wday']+1))==FALSE)
        {
          dbglog("checkstart: ".date("Y-m-d H:i:s D",$checkstart));
          dbglog('This day no activity required');
          // go back one day
          $checkstart = strtotime('-1 day',$checkstart);
          // then move to end of that day (to last second)
          $checkstart_info = getdate($checkstart);
          $checkstart = mktime
              ( 23
              , 59
              , 59
              , $checkstart_info['mon']
              , $checkstart_info['mday']
              , $checkstart_info['year']
              );
        } else break;
      }

      $checkstart_info = getdate($checkstart);
      if (strstr($weekdaylist,''.($checkstart_info['wday']+1))==FALSE)
      {
        // still outside activity period
        dbglog("No activity period at all");
        return null;
      }

      if (($start1===null) && ($start2===null))
      {
        dbglog('nothing defined');
        // nothing defined
        return null;
      }

      if ($start2!==null)
      {
        dbglog("checkstart: ".date("Y-m-d H:i:s D",$checkstart));
        // there is a second period
        $this->TimePeriodParse($start2,$stop2,$checkstart,$tocheck_seconds);
      }
      if ($start1!==null && $tocheck_seconds>0)
      {
        dbglog("checkstart: ".date("Y-m-d H:i:s D",$checkstart));
        $this->TimePeriodParse($start1,$stop1,$checkstart,$tocheck_seconds);
      }
      if (!$first && $tocheck_seconds==$prev_tocheck_seconds)
      {
        $this->healthConfigError('Invalid activity period configuration');
        return null;
      }
      if ($tocheck_seconds>0)
      {
        // go back one day
        $checkstart = strtotime('-1 day',$checkstart);
        // then move to end of that day (to last second)
        $checkstart_info = getdate($checkstart);
        $checkstart = mktime
            ( 23
            , 59
            , 59
            , $checkstart_info['mon']
            , $checkstart_info['mday']
            , $checkstart_info['year']
            );
      }
      $first = false;
    }
    dbglog("checkstart: ".date("Y-m-d H:i:s D",$checkstart));

    return $checkstart;
  }

  function ProcesssesErrFileCheck()
  {
    $this->MultipleErrFileCheck(
      [
        '/var/apresa/data/err_proc_orkaudio.dat',
        '/var/apresa/data/err_proc_apresaremote.dat',
        '/var/apresa/data/err_proc_apresanet.dat',
        '/var/apresa/data/err_proc_apresa_handler.dat',
        '/var/apresa/data/err_proc_apresafifo.dat',
        '/var/apresa/data/err_proc_apresacrec.dat',
        '/var/apresa/data/err_proc_apresaav32.dat',
        '/var/apresa/data/err_proc_apresaavnet.dat',
        '/var/apresa/data/err_proc_apresacj.dat',
        '/var/apresa/data/err_proc_apresaoig.dat',
        '/var/apresa/data/err_proc_apresaalarm.dat',
        '/var/apresa/data/err_proc_apresasnmp.dat',
        '/var/apresa/data/err_proc_yate.dat',
        '/var/apresa/data/err_proc_eyate.dat'
      ],
      
      'err_proc', 'Recording processes', 'Recording process error'
    );

		$this->ErrFileCheck('/var/apresa/data/err_rectool.dat', 'err_rectool', 'Recording tools', 'Recording tool error');
		$this->ErrFileCheck('/var/apresa/data/err_crec.dat', 'err_crec', 'Card recording', 'Card recording error');
    $this->ErrFileCheck('/var/apresa/data/err_proc_ed137.dat', 'err_ed137', 'ED137 Service', 'ED137 recording error', true);
  }
  
  function ProcesssesErrFileCheckConnectors()
  {
		$this->ErrFileCheck('/var/apresa/data/err_aprem.dat', 'err_aprem', 'Connector to other recorders', 'Error in connector to other recorders');
  }

  function ErrFileCheck($fn, $itemname, $part, $errmsg, $onlyIfErr=false)
  {
    $good = true;
    $value = '';
    if (file_exists($fn))
    {
      $value = file_get_contents($fn);
      $good = false;
    }
		if ($onlyIfErr==false || $good==false)
		{
			$this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$good, 'id'=>$itemname,'am'=>$errmsg));
		}
  }

  function MultipleErrFileCheck($fns, $itemname, $part, $errmsg)
  {
    $good = true;
    $messages = [];

    foreach($fns as $fn)
    {
      if (file_exists($fn))
      {
        $good = false;
        $messages[] = trim(file_get_contents($fn));
      }
    }

    if(!$good)
    {
      $value = implode(". ", $messages);
    }
    else
    {
      $value = '';
    }

    $this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$good, 'id'=>$itemname,'am'=>$errmsg));
  }
  
  function VoipEngineCheck()
  {
    $lines = @file($this->datapath . '/voipstatus.dat');
    if ($lines===false) return;
    $parsepart = 'header';
    foreach ($lines as $line)
    {
      $line = trim($line);
      if ($parsepart=='header')
      {
        if ($line=='')
        {
          $parsepart = 'body';
          continue;
        }
      } else if ($parsepart=='body')
      {
        $items = explode('|',$line);
        if (count($items)<4) continue;
        $id     = 'voipengine_'.$items[0];
        $part   = $items[1];
        $hcode = (int) $items[2];

        if ($hcode==1) $health = true;
        else if ($hcode==0) $health = false;
        else if ($hcode==-1) $health = null;
        else continue; // should not occur
        $value  = $items[3];
        if ($health===null)
        {
          $this->newinfo[] = array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$health,'id'=>$id,'err'=>3);
        } else
        {
          $am = $part . ' error';
          if ($value!='') $am .= ': ' . $value;
          $this->Add( array('part'=>$part,'qua'=>'txt','value'=>$value,'h'=>$health,'id'=>$id,'am'=>$am) );
        }
      }
    }
  }
  
	function TeamsBot()
	{
		global $dbintf;
		$check = $dbintf->GetConfigValue('TeamsBotCheck');
		if ($check != 1) return;

		// lets see how the Teams bot is doing
		dbglog("TeamsBot");
		
		$value = $dbintf->GetConfigValue('TeamsBotStatus');
		$health = ((int) $dbintf->GetConfigValue('TeamsBotHealth'))==1;
		$am = $dbintf->GetConfigValue('TeamsBotAlarmMsg');
		$checkTime = (int) $dbintf->GetConfigValue('TeamsBotCheckTime');
		
		$TeamsBotPartId = 'teamsbot';
		$TeamsBotPartDescr = 'Teams bot';
		
		if ($checkTime < time() - 5 * 60)
		{
			$this->newinfo[] = array('part'=>$TeamsBotPartDescr,'qua'=>'txt','value'=>'Not checked recently','h'=>null,'err'=>3);
			return;
		}

		$this->Add( array('part'=>$TeamsBotPartDescr
			, 'qua'=>'txt'
			, 'value'=>$value
			, 'h'=>$health
			, 'id'=>$TeamsBotPartId
			, 'am'=>$am ) );
			
		// more info available?
		$answer = $dbintf->GetConfigValue('TeamsBotAnswer');
		if ($answer==false || $answer=='') return;
		$retobj = json_decode( $answer, true );
		if ($retobj==null) return;

		if (isset($retobj['storage']))
		{
			// available since Teams bot v. 3.1.0
			$storage = $retobj['storage'];
			$storageUsedB = (int) $storage['inUseB'];
			$storageUsedMB = round( $storageUsedB / 1048576 );
			$storageLimitMB = (int) $storage['limitMB'];
			$fraction = ($storageUsedB / 1024) / ($storageLimitMB * 1024.0);
			$good = $fraction <= 0.90;
			$value = $storageUsedMB . ' of ' . $storageLimitMB . ' MB';
			$am = 'Teams bot available storage is low';
			$id = 'teamsbotStorage';
			$descr = 'Teams bot storage';
			$this->Add( array('part'=>$descr
			, 'qua'=>'txt'
			, 'value'=>$value
			, 'h'=>$good
			, 'id'=>$id
			, 'am'=>$am ) );
		}
		
		// Older version didnt have it
		if (isset($retobj['channels'])==false) return;
		
		$channels = $retobj['channels'];
		$this->TeamsChannels('ch','channels',$channels['main']);
		$anyVideo = ($channels['video']['max']>0);
		if ($anyVideo) $this->TeamsChannels('vch','video channels',$channels['video']);
		$tenants = $channels['tenants'];
		foreach($tenants as $tenant)
		{
			dbglog("Teams tenant " . $tenant['name'] );
			$name = $tenant['name'];
			$this->TeamsChannels($tenant['name'].'_ch','channels ('.$name.')',$tenant['main']);
			if ($anyVideo) $this->TeamsChannels($tenant['name'].'_vch','video channels ('.$name.')',$tenant['video']);
			$this->TeamsUlocals($name,$tenant['ulocals']);
		}
	}
	
	function TeamsUlocals($tenantName, $ulocals)
	{
		$configs_s = $this->main_configs_s;
		
		$value = implode(',',$ulocals);
		$good = ($value=='');
		if ($value=='') $value = 'none';
		$am = 'Local ID unlicensed during upload by Teams bot for ' . $tenantName;
		$id = 'teamsbot' . '_' . $tenantName . '_ulocals';
		$descr = 'Teams bot unlicensed phones (' . $tenantName . ')';
		
		$this->Add( array('part'=>$descr
			, 'qua'=>'txt'
			, 'value'=>$value
			, 'h'=>$good
			, 'id'=>$id
			, 'am'=>$am ) );
	}
	
	function TeamsChannels($id,$descr,$obj)
	{
		$configs_s = $this->main_configs_s;
		
		$TeamsBotPartId = 'teamsbot';
		$TeamsBotPartDescr = 'Teams bot';
		$descr = $TeamsBotPartDescr . ' ' . $descr;
		$id = $TeamsBotPartId . '_' . $id;

		$used = $obj['used'];
		$max = $obj['max'];
		$overflow = $obj['overflow'];
		
		$maxTxt = $max;
		if ($max==-1) $maxTxt = 'unlimited';
		if ($max==0)
		{
			// can never contain anything
			// ?? return , no?
		}
		$value = 'used ' . $used . ' of ' . $maxTxt;
		if ($overflow>0)
		{
			$value .= ' (' . $overflow . ' x overflow)';
		}
		
		$PercUsed = 100.0 * $used / $max;
		
		$PeakCalls = int_ensure_defined($configs_s['iaPeakCalls']);
		if ( ($PeakCalls==0 || $max==-1) && $overflow==0 )
		{
			// only info
			$this->newinfo[] = array('part'=>$descr,'qua'=>'txt','value'=>$value,'h'=>null,'err'=>3);			
			return;
		}
		
		$good = ($PercUsed <= $PeakCalls);
		if ($overflow>0) $good = false;
		
		$am = 'Too many channels used for ' . $descr;
		
		$this->Add( array('part'=>$descr
			, 'qua'=>'txt'
			, 'value'=>$value
			, 'h'=>$good
			, 'id'=>$id
			, 'am'=>$am ) );		
		
	}
  
  function ALPCheck()
  {
    global $dbintf;
    $apnet = (int) $dbintf->GetConfigValue('apresanet');
    if(!$apnet)
    {
      return;
    }
    $conns = explode("\n",file_get_contents('/var/apresa/data/apnetsvrcon.txt'));
    $reqconns = explode(",",$dbintf->GetConfigValue('ALPIps'));
    $health = true;
    $value = "";
    foreach($reqconns as $reqcon)
    {
      if(!in_array($reqcon, $conns)){
        $value .= "No connection from " . $reqcon . ".\n";
        $health = false;
      }
    }
    $this->Add( array('part'=>'ALP Connections','qua'=>'txt','value'=>$value,'h'=>$health, 'id'=>'err_alp','am'=>'Cannot connect to all ALP') );
  }
	
  function GetYateLogonStatus($lineix,$checkFn)
  {
    $grepText = "SIP line 'apresaSip" . $lineix . "' logon";
    $logonStatus = '';
    $handle = fopen($checkFn,"r");
    if ($handle)
    {
      while (!feof($handle))
      {
        $line = fgets($handle);
        $pos = strpos($line,$grepText);
        if ($pos !== false)
        {
          $logonStatus = substr($line,$pos+strlen($grepText)-5);
        }
      }
    }
    return $logonStatus;
  }
  
  // VoIP service.
  function YateCheck()
  {
		global $dbintf;
    $configs_t = $dbintf->LoadConfigsPart('t');
    
    if (int_ensure_defined($configs_t['VoipServer'])==0)
    {
      return;
    }

    $maxSipLines = 2;
    for ($lineix=1;$lineix<=$maxSipLines;$lineix++)
    {
      $cfgKey = 'TelsysSip' . $lineix . 'Reg';
      if ($configs_t[$cfgKey]!=0)
      {
        dbglog("Register for SIP line " . $lineix . " is configured enabled");
        $file = 'apresaSip' . $lineix . '.dat';
        $logonStatus = file_get_contents('/var/apresa/data/' . $file);

        if($logonStatus)
        {
          if($logonStatus == 'OK')
          {
            $msg = 'Logon succesfull';
            $isok = true;
          }
          else
          {
            $msg = $logonStatus;
            $isok = false;
          }

          $this->Add( array('part'=>'VoIP Service SIP Line '.$lineix,'qua'=>'txt','value'=>$msg,'h'=>$isok,'id'=>'TelsysSipReg'.$lineix,'am'=>'SIP logon register failure'));
        }
      }
    }
  }
  
  function ShoreTelDbCheck()
  {
		global $dbintf;
    // check if enabled at all
		if ( (int) $dbintf->GetConfigValue('ShoreTelDb') == 0)
    {
      return;
    }
    // get settings and status
    $cfg = $dbintf->GetConfigsWithPrefix('ShoreTelDb_');
    $value = $cfg['StatusMsg'];
    $health = ($cfg['Status']==1);
    
    $this->Add( array('part'=>'ShoreTel call database link','qua'=>'txt','value'=>$value,'h'=>$health, 'id'=>'ShoreTelDb','am'=>'Error in ShoreTel call database link') );
  }
  
	function ToshibaCheck()
	{
		global $dbintf;
		$toshiba = (int) $dbintf->GetConfigValue('ToshibaSMDR');
		
		if(!$toshiba)
		{
			return;
		}
		
		if(file_exists('/var/apresa/data/toshiba_conn.dat'))
		{
			$file = file_get_contents('/var/apresa/data/toshiba_conn.dat');
			$status = explode("\n", $file);
			
			if(count($status) < 2)
			{
				$health = false;
				$value = "Status unknown";
			}
			else
			{
				$connStat = explode("=", $status[0]);
				$port = $connStat[0];
				if($connStat[1] != "OK")
				{
					$health = false;
					$value = "Port " . $port . ": " . $connStat[1];
				}
				else
				{
					$health = true;
					$ips = array_slice($status, 1);
					$reqIps = explode(",", $dbintf->GetConfigValue('ToshibaIps'));
					
					foreach($reqIps as $reqIp)
					{
						if(!in_array($reqIp, $ips)){
							$value .= "No connection from " . $reqIp . ".\n";
							$health = false;
						}
					}
					
					if($health)
					{
						$value = "Port " . $port . ": " . "OK";
					}
				}
			}		
		}
		else
		{
			$health = false;
			$value = "Status file not found";
		}
		
		$this->Add( array('part'=>'Toshiba SMDR Status','qua'=>'txt','value'=>$value,'h'=>$health, 'id'=>'err_toshiba','am'=>'Toshiba SMDR Connection Error') );
	}

  function RecCheck()
  {
    $reccheckdata = @file('/var/apresa/data/reccheck.dat');
    if ($reccheckdata===false || count($reccheckdata)==0)
    {
      // the file is not there or empty
      return;
    }
    $line1 = $reccheckdata[0];
    $value = '';
    if (substr($line1,0,3)=='ERR')
    {
      $good = false;
      $value = substr($line1,5);
    } else if (substr($line1,0,3)=='OK.')
    {
      $good = true;
    } else
    {
      return;
    }
    $value = trim($value);
    $key = 'reccheck';
    $this->Add( array('part'=>'Recording file','qua'=>'pwr','value'=>$value,'h'=>$good,'id'=>'recfile','am'=>'Recording file failure'));
  }

	function AddCaption( $t )
	{
		$info = array( 'part' => $t
		, 'qua' => 'caption'
		);
		$this->newinfo[] = $info;
	}
	
  function CustomCheck()
  {
    if ($this->handledPowerSupply) return;
		$cmdret = array();
    if (!$this->simulate)
    {
      exec('apresa_check',$cmdret);
    } else
    {
      $cmdret = array
      ( 'Pwr1: 0'
      , 'Pwr2: 0'
      );
    }

    $CheckNames = array
    ( 'Pwr1' => 'Power Supply 1' // not supported
    , 'Pwr2' => 'Power Supply 2' // not supported
    , 'Pwr'  => 'Power Supply'
    );
    foreach($cmdret as $t)
    {
      dbglog($t);
      $parts = explode(':',$t);
      array_walk($parts,'trim');
      if (count($parts)!=2) continue;
      $key = $parts[0];
      if (isset($CheckNames[$key])==false)
      {
        dbglog("Unknown component ".$key);
      }
      $name = $CheckNames[$key];
      $wasgood = $this->GetHealth($key);
      $good = $parts[1]==0;
      $this->Add( array('part'=>$name,'qua'=>'pwr','value'=>'','h'=>$good, 'id'=>$key,'am'=>'Power supply failure'));
    }
  }

  function HardDiskTemp()
  {
    $hdinfo = array();
    if (!$this->simulate)
    {
      if(GetDebianVersion() < 12)
      {
        exec('hddtemp -w /dev/sd?',$hdinfo);
      }
      else
      {
        exec('/usr/share/apresa/smartctl_hddtemp.sh /dev/sd?',$hdinfo);
      }
    } else
    {
      // simulate
      $hdinfo = array
      (	'/dev/sda: ST3320620NS: 26°C'
      ,	'/dev/sdb: ST3320620NS: 30°C'
      , '/dev/sdc: LSI: S.M.A.R.T. not available'
      );
    }

    // format is:
    // dev : type : temp.


    dbglog("Hard disk temperature info:");
    foreach ($hdinfo as $t)
    {
      dbglog($t);
      $parts = explode(':',$t);
      array_walk($parts, function(&$part){
        $part = trim($part);
      });
      if (count($parts)!=3) continue;
      if (strpos($parts[0],'/dev/sd')===false) continue;
      $device = substr($parts[0],5);
      $strtemp = $parts[2];
      if (strlen($strtemp)==0) continue;
      if (ctype_digit($strtemp[0]) == false || ctype_digit($strtemp[1])==false) continue;
      $temp = (double) $strtemp;
      dbglog($device . ' has temperature: ' . $temp);

      $key = 'hddtemp' . $device;
      $wasgood = $this->GetHealth($key);
      $good = true;
      dbglog('max is ' . $this->healthconf['HDDTEMP_max']);
      if ($wasgood && $temp>$this->healthconf['HDDTEMP_max'])
      {
        $good = false;
      }
      if (!$wasgood && $temp>$this->healthconf['HDDTEMP_max_hyst'])
      {
        $good = false;
      }
      $this->Add( array('part'=>'Temperature of hard disk '.$device,'qua'=>'hddtemp','value'=>$temp.' °C','h'=>$good,'err'=>1, 'id'=>$key,'am'=>'Hard disk overheated'));
    }
  }

  function setItem($key,$good,$msg,$isNodeHealth)
  {
    $wasgood = $this->GetHealth($key);
    $this->newhealth[$key] = (int) $good;
    if ($good==false)
    {
      $this->messages[] = $msg;
      $this->any_failure = true;
      if ($isNodeHealth)
      {
        $this->node_failures ++;
      }
    }
    if ($good==false && $wasgood)
    {
      if($key == "appmemfree")
      {
        $this->lowmem = true;
      }

      $this->new_failure = true;
    }
    if ($good && $wasgood==false)
    {
      $this->restore = true;
    }
  }

	function TenantSpaceCheck()
	{
		global $dbintf;
		
		$tnConfig = $dbintf->LoadConfigsPart('tn');
		
		$tenants = $dbintf->GetTenantList();
		foreach ($tenants as $tenant)
		{
			$tnId = $tenant['id'];
			$tnName = $tenant['name'];
			$prefix = 'TN' . $tnId . '_';
			$alarmThreshold = str_ensure_defined( $tnConfig[ $prefix . 'AlarmDiskUsage'] );
			if ($alarmThreshold=='') continue;
			$alarmThreshold = (double) $alarmThreshold;
			$diskUsage = int_ensure_defined( $tnConfig[$prefix . 'DiskUsage'] );
			$diskUsageMb = $diskUsage / ( 1024 * 1024.0 );
			
			dbglog('Tenant ' . $tenant['name']
				. ' usage: ' . round($diskUsageMb,2)  . ' MB'
				. ' th: ' . $alarmThreshold . ' MB'
				);
			
			$perc = 100.0 * ($diskUsageMb / $alarmThreshold);
			
			$valueStr = round( $diskUsageMb) . ' MB (' . round($perc) . ' %)';
			$good = $diskUsageMb <= $alarmThreshold;
			
			$this->Add( array('part'=>'Disk usage by '.$tnName
			,'qua'=>'hd','value'=>$valueStr,'h'=>$good
			,'err'=>0,'id'=>'tndiskusage'.$tnId
			,'am'=>"Disk space usage by tenant $tnName has exceeded the alarm threshold"
			) );
			
		}
	}

  function DiskSpaceCheck()
  {
		$configs_s = $this->main_configs_s;
    $dflines = array();
    if (!$this->simulate)
    {
      exec('df',$dflines);
    } else
    {
      $dflines = <<<SIMDATA
Filesystem           1K-blocks      Used Available Use% Mounted on
/dev/mapper/apresa-root
                       2979376    977876   1850156  35% /
tmpfs                  2026512         0   2026512   0% /lib/init/rw
udev                   2020924       200   2020724   1% /dev
tmpfs                  2026512         0   2026512   0% /dev/shm
/dev/sda1               188403     22078    156597  13% /boot
/dev/mapper/apresa-data
                     467805760    202872 443839740  81% /recordings
SIMDATA;
      $dflines = explode("\n",$dflines);
    }

		$reported = array();
		$hasRecordings = false;
    $lineix=0;
    $prevname = '';
    foreach($dflines as $line)
    {
      $lineix++;
      if ($lineix==1)
      {
        //if (substr($line,0,10)!='Filesystem') break;
      } else
      {
        $parts = preg_split('/[\s]+/',$line,-1,PREG_SPLIT_NO_EMPTY);
        if (count($parts)==1)
        {
          $prevname = $parts[0];
          continue;
        }
        if ($prevname!='')
        {
          array_unshift($parts,$prevname);
          $prevname = '';
        }
        $name = $parts[5];
				if ($name=='/recordings')
				{
					$hasRecordings = true;
				}
			}
		}
		
    $lineix=0;
    $prevname = '';
    foreach($dflines as $line)
    {
      $lineix++;
      if ($lineix==1)
      {
        //if (substr($line,0,10)!='Filesystem') break;
      } else
      {
        $parts = preg_split('/[\s]+/',$line,-1,PREG_SPLIT_NO_EMPTY);
        if (count($parts)==1)
        {
          $prevname = $parts[0];
          continue;
        }
        if ($prevname!='')
        {
          array_unshift($parts,$prevname);
          $prevname = '';
        }
        $useperc = $parts[4];
        $use = (int) $useperc;
        $name = $parts[5];
        $mbused = round($parts[2] / 1024);
        $gbused = round($parts[2] / 1048576);
				
				$mbavail = round($parts[3] / 1024);
        $gbavail = round($parts[3] / 1048576);
				
				if (isset($reported[$name]))
				{
					// don't repeat
					continue;
				}
				if ($name=='/var/upload')
				{
					// excluded, because equal to recordings
					continue;
				}
        if ($name=='/mnt/backup'
        || $name=='mnt/export'
        || startsWith($name,'/mnt/drive_') )
        {
          // exclude automatic temporary mounting
          // because the alarm cannot be disabled
          continue;
        }
				
        dbglog('Partition '.$name.' usage: '.$use.' % , '.$gbused . ' GB');

        $valuestr = $useperc;
        if ($name=='/recordings')
        {
					// moved to separate info field
          //$valuestr = '(' . $mbused . ' MB) ' .$useperc;
        }

				$reported[$name] = true;

        $alarmPerc = 80.0;
        $limit_key = 'HealthDiskSpace_' . $name . '_AlarmPerc';
        if (isset($configs_s[$limit_key]))
        {
          $alarmPerc = (double) $configs_s[$limit_key];
        }
        dbglog("Alarm % for $name = $alarmPerc");
				
        $good = $use < $alarmPerc;
        $this->Add( array('part'=>'HD partition usage '.$name
          ,'qua'=>'hd','value'=>$valuestr,'h'=>$good
          ,'err'=>0, 'id'=>'ptuse'.$name
          ,'am'=>'Partition '.$name.' is '.$useperc.' full.'
          ));
				
				if ( $name=='/recordings'
					|| ($hasRecordings==false && $name=='/') )
        {
					$infostr = $mbused . ' MB';
					$this->newinfo[] = array('part'=>'HD partition usage '.$name,'qua'=>'hd','value'=>$infostr,'h'=>null,'err'=>3);
          
          $fifo_threshold = 2 * 1024;
          $totalmb = $mbused + $mbavail;
          $fifo_threshold_max = $totalmb * 0.01;
					$fifo_threshold_min = 250;
          if ($fifo_threshold > $fifo_threshold_max)
          {
            $fifo_threshold = $fifo_threshold_max;
          }
					if ($fifo_threshold < $fifo_threshold_min)
          {
						$fifo_threshold = $fifo_threshold_min;
					}
          $good = ($mbavail >= $fifo_threshold * 1.5);
					$valuestr = $mbavail . ' MB';
          $this->Add( array('part'=>'HD partition free '.$name
          ,'qua'=>'hd','value'=>$valuestr,'h'=>$good
          ,'err'=>0,'id'=>'ptfree'.$name
          ,'am'=>'The disk space available on the recordings partition is low and near the auto-delete threshold'
          ));
				}
      }
    }

  }

  function HPcheck()
  {
    $lines = array();
    if (!$this->simulate)
    {
      if (file_exists('/usr/sbin/hpacucli')==false) return;
      exec('hpacucli ctrl all show config',$lines);
    } else
    {
      $lines = <<<SIMDATA

Smart Array P410i in Slot 0 (Embedded)    (sn: 5001438023E2C820)

   array A (SAS, Unused Space: 0 MB)


      logicaldrive 1 (136.7 GB, RAID 1, OK)

      physicaldrive 1I:1:1 (port 1I:box 1:bay 1, SAS, 146 GB, OK)
      physicaldrive 1I:1:2 (port 1I:box 1:bay 2, SAS, 146 GB, OK)

   SEP (Vendor ID PMCSIERA, Model  SRC 8x6G) 250 (WWID: 5001438023E2C82F)

   
SIMDATA;
      $lines = explode("\n",$lines);
    }
    $hpraid = false;
    foreach($lines as $line)
    {
      $line = trim($line,"\r\n");
      dbglog($line);
      if (strstr($line,'Array')!==false)
      {
        $hpraid = true;
      }
      if (strstr($line,'logicaldrive')!==false)
      {
        $bracket = strstr($line,'(');
        if ($bracket===false) continue;
        $bracket = trim($bracket,"() \t");
        $parts = explode(',',$bracket);
        if (count($parts)<3) continue;
        $status = trim($parts[2]);
        for ($i=3;$i<count($parts);$i++)
        {
          $status .= ', ' . trim($parts[$i]);
        }
        $good = ($status == 'OK');
        $this->Add( array('part'=>'HP RAID','qua'=>'hd','value'=>$status,'h'=>$good,'err'=>0, 'id'=>'HP RAID','am'=>"HP RAID status: $status"));
        //dbglog($status);
      }
      if (strstr($line,'physicaldrive')!==false)
      {
        $bracket = strstr($line,'(');
        if ($bracket===false) continue;
        $bracket = trim($bracket,"() \t");
        $parts = explode(',',$bracket);
        if (count($parts)<4) continue;
        $id = trim($parts[0]);
        $status = trim($parts[3]);
        $good = ($status == 'OK');
        $this->Add( array('part'=>'HP RAID '.$id,'qua'=>'hd','value'=>$status,'h'=>$good,'err'=>0, 'id'=>'HP RAID '.$id, 'am'=>"HP RAID $id: $status"));
      }
    }
    //exit;
    
  }

	function SimulatedError()
	{
		global $dbintf;
		if ( ((int)$dbintf->GetConfigValue('SimulateError')) == 1 )
		{
			$this->Add( array('part'=>'Simulated error','qua'=>'na','value'=>'-999','h'=>false,'err'=>0, 'id'=>'simerr', 'am'=>"A system error has been simulated (this is not a real error)"));
		}
	}
  
  function RAIDcheck()
  {
    $mdlines = array();
    if (!$this->simulate)
    {
      exec('cat /proc/mdstat',$mdlines);
    } else
    {
      /*$mdlines = <<<SIMDATA
Personalities : [raid1]
md2 : active raid1 sda3[0] sdb3[1]
      961862520 blocks super 1.2 [2/2] [UU]

md1 : active (auto-read-only) raid1 sda2[0] sdb2[1]
      11870136 blocks super 1.2 [2/2] [UU]
        resync=PENDING

md0 : active raid1 sda1[0] sdb1[1]
      3024884 blocks super 1.2 [2/2] [UU]

unused devices: <none>
SIMDATA;*/
      /*$mdlines = <<<SIMDATA
Personalities : [raid1]
md2 : active raid1 sda3[2] sdb3[1]
      961859448 blocks super 1.2 [2/1] [_U]
        resync=DELAYED

md1 : active raid1 sda2[2] sdb2[1]
      11873208 blocks super 1.2 [2/1] [_U]
      [======>..............]  recovery = 33.7% (4003968/11873208) finish=1.7min speed=76605K/sec

md0 : active raid1 sda1[2] sdb1[1]
      3024884 blocks super 1.2 [2/2] [UU]

unused devices: <none>
SIMDATA;*/
      $mdlines = <<<SIMDATA
Personalities : [raid6] [raid5] [raid4]
md1 : active raid5 sda3[0] sdc3[2] sdb3[1]
      1952849920 blocks super 1.2 level 5, 512k chunk, algorithm 2 [3/3] [UUU]
      bitmap: 2/8 pages [8KB], 65536KB chunk

md0 : active raid5 sda2[0] sdc2[2] sdb2[1]
      389120 blocks super 1.2 level 5, 512k chunk, algorithm 2 [3/3] [UUU]

unused devices: <none>
SIMDATA;
      $mdlines = explode("\n",$mdlines);
    }
    $raid = false;
    $lineix = -1;
    $disks = array();
    $pts = array(); // partitions
    $ptname = '';
    $ptix = -1;
    foreach($mdlines as $line)
    {
      if (substr($line,0,2)=='md')
      {
        // this line shows we are on a RAID system
        $ptname = substr($line,0,3);
        $ptix = (int) $line[2];
        $pts[$ptix] = '';
        $raid = true;
        $lineix = 0;

        $ignorepending = false;
        if (strpos($line,'auto-read-only')!=false)
        {
          $ignorepending = true;
        }
        continue;
      }
      if (!$raid) continue;
      $lineix ++;
      if ($lineix==1)
      {
        $parts = preg_split('/[\s]+/',$line,-1,PREG_SPLIT_NO_EMPTY);
        $strHdState = '';
        if (count($parts)>1)
        {
          $strHdState = $parts[ count($parts)-1 ];
        }
        if ($strHdState[0]!='[')
        {
          // should never occur
          dbglog('Format error');
          continue;
        }
        $diskcnt = strlen($strHdState)-2;
        for ($i=0;$i<$diskcnt;$i++)
        {
          $hdstate = $strHdState[$i+1];
          if ($hdstate=='U')
          {
            // OK
            if (isset($disks[$i])==false)
            {
              $disks[$i] = true;
            }
          } else
          {
            $disks[$i] = false;
          }
        }
      }
      if ($lineix==2)
      {
        if ($line[0]!=' ') continue;
        dbglog($line);
        $parts = preg_split('/[\s]+/',$line,-1,PREG_SPLIT_NO_EMPTY);
        dbglog($parts[1]);
        $status = trim($line);
				if ($parts[0]=='bitmap:')
				{
					continue;
				}
        if ($parts[0]=='resync=DELAYED')
        {
          $status = 'Resync. is delayed';
        }
        if ($parts[0]=='resync=PENDING')
        {
          if ($ignorepending) continue;
           $status = 'Resync. is pending';
        }
        if ($parts[1]=='recovery')
        {
          $status = 'Recovering ('.$parts[3].')';
        }
        if ($parts[1]=='resync')
        {
          $status = 'Resync. ('.$parts[3].')';
        }
        //dbglog('STATUS: '.$status.' '.$ptix);
        $pts[$ptix] = $status;
      }
    }

    if(!$this->simulate)		//Hardware RAID.
    {
			if(file_exists('/opt/MegaRAID/storcli/storcli64'))
			{
				$mdlines = array();
				$controller=true;
				exec('/opt/MegaRAID/storcli/storcli64 -adpCount',$mdlines);
				foreach($mdlines as $line)
				{
					$pos=strpos($line,'Controller Count: 0');
					if($pos!==false)
					{
						$controller=false;
						break;
					}
				}
				if($controller) $raid = true;
				$mdlines = array();
				if($controller) exec('/opt/MegaRAID/storcli/storcli64 /c0/vall show',$mdlines);
			}
		}
		else
		{
      $mdlines = <<<SIMDATA
Controller = 0
Status = Success
Description = None


Virtual Drives :
==============

-------------------------------------------------------------
DG/VD TYPE  State Access Consist Cache Cac sCC     Size Name
-------------------------------------------------------------
0/0   RAID1 Dgrd  RW     No      NRWTD -   ON  931.0 GB
-------------------------------------------------------------

Cac=CacheCade|Rec=Recovery|OfLn=OffLine|Pdgd=Partially Degraded|Dgrd=Degraded
Optl=Optimal|RO=Read Only|RW=Read Write|HD=Hidden|TRANS=TransportReady|B=Blocked|
Consist=Consistent|R=Read Ahead Always|NR=No Read Ahead|WB=WriteBack|
AWB=Always WriteBack|WT=WriteThrough|C=Cached IO|D=Direct IO|sCC=Scheduled
Check Consistency
SIMDATA;
			$mdlines = explode("\n",$mdlines);
		}
		
		if(ensure_defined($controller,false) || $this->simulate)
		{
			$found=false;
			$i=count($disks);	//$i = number of disks in software raid (probably 0).
			foreach($mdlines as $line)
			{
				if(substr($line,0,5)=='DG/VD')
				{
					$found=true;
					continue;
				}
				if(!$found) continue;
				if(substr($line,0,5)!='-----')
				{
					$parts = preg_split('/[\s]+/',$line,-1,PREG_SPLIT_NO_EMPTY);
					$status = $parts[2];
					if($status=='Optl') $disks[$i] = true;
					else $disks[$i] = false;
					$i++;
					break;
				}
			}
			//Status of the individual disks!
			if(!$this->simulate)		//Hardware RAID.
			{
				$mdlines = array();
				exec('/opt/MegaRAID/storcli/storcli64 /c0/eall/sall show',$mdlines);
			}
			else
			{
				$mdlines = <<<SIMDATA
Controller = 0
Status = Success
Description = Show Drive Information Succeeded.

Drive Information :
=================

----------------------------------------------------------------------------------
EID:Slt DID State DG     Size Intf Med SED PI SeSz Model                  Sp Type
----------------------------------------------------------------------------------
62:0      4 Onln   0 931.0 GB SATA HDD N   N  512B WDC WD1005FBYZ-01YCBB1 U  -
62:1      5 Rbld   0 931.0 GB SATA HDD N   N  512B WDC WD1005FBYZ-01YCBB1 U  -
----------------------------------------------------------------------------------

EID-Enclosure Device ID|Slt-Slot No.|DID-Device ID|DG-DriveGroup
DHS-Dedicated Hot Spare|UGood-Unconfigured Good|GHS-Global Hotspare
UBad-Unconfigured Bad|Onln-Online|Offln-Offline|Intf-Interface
Med-Media Type|SED-Self Encryptive Drive|PI-Protection Info
SeSz-Sector Size|Sp-Spun|U-Up|D-Down|T-Transition|F-Foreign
UGUnsp-Unsupported|UGShld-UnConfigured shielded|HSPShld-Hotspare shielded
CFShld-Configured shielded|Cpybck-CopyBack|CBShld-Copyback Shielded
SIMDATA;
				$mdlines = explode("\n",$mdlines);
			}
			$found=false;
			$readingdisks=false;
			$i=count($pts);
			foreach($mdlines as $line)
			{
				if(substr($line,0,7)=='EID:Slt')
				{
					$found=true;
					continue;
				}
				if(!$found) continue;
				if(substr($line,0,5)!='-----')
				{
					$readingdisks=true;
					$parts = preg_split('/[\s]+/',$line,-1,PREG_SPLIT_NO_EMPTY);
					$diskstatus = $parts[2];
					if($diskstatus=='Onln') $pts[$i]='Online';
					if($diskstatus=='Rbld') $pts[$i]='Rebuild';
					if($diskstatus=='UGood') $pts[$i]='Unconfigured Good';
					if($diskstatus=='DHS') $pts[$i]='Dedicated Hot Spare';
					if($diskstatus=='GHS') $pts[$i]='Global Hotspare';
					if($diskstatus=='UBad') $pts[$i]='Unconfigured Bad';
					if($diskstatus=='Offln') $pts[$i]='Offline';
					$i++;
				}
				if($readingdisks && substr($line,0,5)=='-----') break;
			}
		}
		
		if($raid=='')	return;

    foreach ($disks as $disknum=>$diskstate)
    {
      $good = $diskstate;
      $this->Add( array('part'=>'Hard disk SATA'.$disknum,'qua'=>'hd','value'=>'','h'=>$good,'err'=>0, 'id'=>'SATA'.$disknum, 'am'=>'Hard disk SATA'.$disknum.' has failed'));
    }
    foreach ($pts as $ptix=>$status)
    {
      if ($status!='')
      {
        $this->newinfo[] = array('part'=>'RAID partition '.$ptix,'qua'=>'hd','value'=>$status,'h'=>null,'err'=>3);
      }
    }

  }

  function Sensors()
  {
    $sinfo = array();
    if (!$this->simulate)
    {
      exec('sensors',$sinfo);
    } else
    {
      // simulate
      $sinfo = <<<SIMDATA
coretemp-isa-0000      
Adapter: ISA adapter
Core 0:      +40.0°C  (high = +76.0°C, crit = +100.0°C)

coretemp-isa-0001
Adapter: ISA adapter
Core 1:      +34.0°C  (high = +76.0°C, crit = +100.0°C)

w83627dhg-isa-0290
Adapter: ISA adapter
Vcore:       +1.08 V  (min =  +0.92 V, max =  +1.48 V)
in1:         +0.76 V  (min =  +0.67 V, max =  +0.83 V)
AVCC:        +3.31 V  (min =  +2.98 V, max =  +3.63 V)
VCC:         +3.31 V  (min =  +2.98 V, max =  +3.63 V)
in4:         +1.85 V  (min =  +1.36 V, max =  +2.04 V)
in5:         +1.28 V  (min =  +1.13 V, max =  +1.38 V)
in6:         +1.46 V  (min =  +1.42 V, max =  +1.52 V)
3VSB:        +3.31 V  (min =  +2.98 V, max =  +3.63 V)
Vbat:        +3.18 V  (min =  +2.70 V, max =  +3.30 V)
fan1:          0 RPM  (min =  753 RPM, div = 128)  ALARM
fan2:       2083 RPM  (min =  712 RPM, div = 8)
fan3:          0 RPM  (min =  753 RPM, div = 128)  ALARM
fan4:       1288 RPM  (min =  712 RPM, div = 8)
fan5:          0 RPM  (min =  753 RPM, div = 128)  ALARM
temp1:       +27.0°C  (high = +60.0°C, hyst = +55.0°C)  sensor = thermistor
temp2:       +40.0°C  (high = +81.0°C, hyst = +76.0°C)  sensor = diode
temp3:       +40.0°C  (high = +81.0°C, hyst = +76.0°C)  sensor = diode
SIMDATA;
      $sinfo = <<<SIMDATA
acpitz-virtual-0
Adapter: Virtual device
temp1:       +27.8°C  (crit = +103.0°C)
temp2:       +29.8°C  (crit = +103.0°C)

coretemp-isa-0000
Adapter: ISA adapter
Physical id 0: +29.0°C  (high = +82.0°C, crit = +102.0°C)
Core 0:        +28.0°C  (high = +82.0°C, crit = +102.0°C)
Core 1:        +26.0°C  (high = +82.0°C, crit = +102.0°C)

nct6776-isa-0a30
Adapter: ISA adapter
in0:          +0.78 V  (min =  +0.60 V, max =  +1.49 V)
in1:          +1.85 V  (min =  +1.62 V, max =  +1.99 V)
in2:          +3.36 V  (min =  +2.96 V, max =  +3.63 V)
in3:          +3.34 V  (min =  +2.96 V, max =  +3.63 V)
in4:          +1.53 V  (min =  +1.35 V, max =  +1.65 V)
in5:          +1.26 V  (min =  +1.13 V, max =  +1.38 V)
in7:          +3.38 V  (min =  +2.96 V, max =  +3.63 V)
in8:          +3.09 V  (min =  +2.96 V, max =  +3.63 V)
fan1:        1309 RPM  (min =  300 RPM)
fan2:           0 RPM  (min =  300 RPM)  ALARM
fan3:           0 RPM  (min =  300 RPM)  ALARM
fan4:        1005 RPM  (min =  300 RPM)
fan5:           0 RPM  (min =  300 RPM)  ALARM
SYSTIN:       +26.0°C  (high = +75.0°C, hyst = +70.0°C)  sensor = thermistor
CPUTIN:       +24.0°C  (high = +80.0°C, hyst = +75.0°C)  sensor = thermistor
AUXTIN:        +0.5°C                                    sensor = thermistor
PECI Agent 0: +27.0°C  (high = +95.0°C, hyst = +92.0°C)
cpu0_vid:    +2.050 V
SIMDATA;
			$sinfo = <<<SIMDATA
coretemp-isa-0000
Adapter: ISA adapter
Core 0:       +41.0°C  (high = +76.0°C, crit = +100.0°C)
Core 1:       +32.0°C  (high = +76.0°C, crit = +100.0°C)

w83627dhg-isa-0290
Adapter: ISA adapter
Vcore:        +1.08 V  (min =  +0.92 V, max =  +1.48 V)
in1:          +0.75 V  (min =  +0.67 V, max =  +0.83 V)
AVCC:         +3.30 V  (min =  +2.98 V, max =  +3.63 V)
+3.3V:        +3.30 V  (min =  +2.98 V, max =  +3.63 V)
in4:          +1.86 V  (min =  +1.36 V, max =  +2.04 V)
in5:          +1.29 V  (min =  +1.13 V, max =  +1.38 V)
in6:          +1.47 V  (min =  +1.42 V, max =  +1.52 V)
3VSB:         +3.26 V  (min =  +2.98 V, max =  +3.63 V)
Vbat:         +3.15 V  (min =  +2.70 V, max =  +3.63 V)
fan1:           0 RPM  (min =  753 RPM, div = 128)  ALARM
fan2:        2033 RPM  (min =  712 RPM, div = 8)
fan3:           0 RPM  (min =  753 RPM, div = 128)  ALARM
fan4:        1360 RPM  (min =  712 RPM, div = 8)
fan5:           0 RPM  (min =  753 RPM, div = 128)  ALARM
temp1:        +28.0°C  (high = +75.0°C, hyst = +70.0°C)  sensor = thermistor
temp2:        +41.0°C  (high = +81.0°C, hyst = +76.0°C)  sensor = CPU diode
temp3:        +41.0°C  (high = +81.0°C, hyst = +76.0°C)  sensor = CPU diode
intrusion0:  ALARM
SIMDATA;

      $sinfo = explode("\n",$sinfo);
    }
    dbglog('Sensor information:');
    //dbglog($sinfo);
    $sensors = explode(',',str_ensure_defined($this->healthconf['SENSORS']));
    //var_dump($sensors);
    $driver = '???';
    $driverfull = '???';
    $nextLineDriver = true;

    foreach ($sinfo as $line)
    {
      $line = trim($line);
      dbglog('Line: '.$line);
      if ($line=='') 
      {
        $nextLineDriver = true;
        continue;
      }
      if ($line[0]=='#') continue;
      if ($nextLineDriver)
      {
        $driver = $driverfull = $line;
        $dashpos = strrpos($driverfull,'-');
        if ($dashpos!==false)
        {
          $driver = substr($driverfull,0,$dashpos);
        }
        $nextLineDriver = false;
        continue;
      }
      $parts = explode(':',$line);
      if (count($parts)!=2) continue;
      $key = trim($parts[0]);
      $value = $parts[1];
      $value = trim($value);
      $isinfo = false;
      $pos = strpos($value,'(');
      $bracketText = '';
      if ($pos!==false)
      {
        $bracketText = substr($value,$pos);
        $posbr = strpos($bracketText,')');
        if ($posbr!==false)
        {
          $bracketText = substr($bracketText,0,$posbr+1);
        }
      } else
      {
        $isinfo = true;
        $pos = strpos($value,'    ');
      }
      if ($pos!==false)
      {
        $value = substr($value,0,$pos);
      }
      $value = trim($value);
      // next line would delete unit (done previously)
      $dvalue = (double) $value;

      $fullkey = $driver . '.' . $key;
      dbglog('Full-Key: '.$fullkey);

      
      if ($key=='Adapter')
      {
        continue;
      }
      
      if (isset($this->healthconf[$fullkey.'_disabled']))
      {
        continue;
      }
      

      // Previously only a defined set of sensors were accepted.
      // if (!in_array($key,$sensors)) continue;
      // Now it is the other way around.
      // Only those that are set to be ignored, are not shown.
      // Secondly, the values are prefixed with the driver name.

      
      $partname = $driver;
      if (isset($this->healthconf[$driver.'_name']))
      {
        $partname = $this->healthconf[$driver.'_name'];
      }
      
      
      dbglog($driver);
      $partname .= ' ' . $key;
      if (isset($this->healthconf[$fullkey . '_name']))
      {
        $partname = $this->healthconf[$fullkey . '_name'];
      }
      dbglog($partname . " = " . $value);
      
      $good = true;
      $errcode = 0;

      if (strpos($parts[1],'ALARM')!==false)
      {
        $good = false;
        $isinfo = false;
      } 
      
			if (isset($this->healthconf[$fullkey.'_isinfo']))
      {
				$isinfo = true;
			}
			
      if ($isinfo) $good = null;
      else
      {
        $wasgood = $this->GetHealth($fullkey);
				$customset = false;
        if (isset($this->healthconf[$fullkey.'_max']))
        {
          // a maximum for this value is defined
					$customset = true;
          if ($wasgood || isset($this->healthconf[$fullkey.'_max_hyst'])==false)
          {
            if ($dvalue > $this->healthconf[$fullkey.'_max']) $errcode = 1;
          } else
          {
            if ($dvalue > $this->healthconf[$fullkey.'_max_hyst']) $errcode = 1;
          }
        }
        if (isset($this->healthconf[$fullkey.'_min']))
        {
          // a minimum for this value is defined
					$customset = true;
          if ($wasgood || isset($this->healthconf[$fullkey.'_min_hyst'])==false)
          {
            if ($dvalue < $this->healthconf[$fullkey.'_min']) $errcode = 2;
          } else
          {
            if ($dvalue < $this->healthconf[$fullkey.'_min_hyst']) $errcode = 2;
          }
        }
				if ($customset)
				{
					$good = ($errcode==0);
				}
      }
			
      $infoObj = array
      (	'part'=>$partname
      ,	'qua'=>'sensors'
      ,	'value'=>$value
      ,	'h'=>$good
      , 'err'=>$errcode
      , 'id'=>$fullkey
      , 'xtr'=>$bracketText
      );
      if ($isinfo)
      {
        $this->newinfo[] = $infoObj;
      } else
      {
        $alarmtext = $partname;
        if ($errcode==1) $alarmtext .= ' too high';
        if ($errcode==2) $alarmtext .= ' too low';
        if (isset($this->healthconf[$key.'_alarm']))
        {
          $alarmtext = $this->healthconf[$key.'_alarm'];
        }
        if (isset($this->healthconf[$key.'_alarm_'.$errcode]))
        {
          $alarmtext = $this->healthconf[$key.'_alarm_'.$errcode];
        }
        $infoObj['am'] = $alarmtext;
        $this->Add( $infoObj );
      }
    }
    
		// debug:
		//exit;
  }

  function write()
  {
    global $dbintf;
    // Write health state
    $to_write = '';
    foreach ($this->newhealth as $key=>$value)
    {
      $to_write .= "$key=".($value?1:0)."\n";
    }
    file_put_contents($this->datapath . '/health.dat',$to_write);
    
    if ($this->clustering)
    {
      $node_health = 100 - $this->node_failures * 10;
      if ( $node_health < 1 ) $node_health = 1;
      // check if suited to become primary,
      // based on current database status
      if ($this->node_db == false) $node_health = 0;
      setNodeScore( $node_health );
    }
    
  }
	
	function IPMI()
	{
		//$this->ipmiSensors = array();
    $sinfo = array();
    if (!$this->simulate)
    {
			if (file_exists('/usr/sbin/ipmi-sensors')==false)
			{
				// IPMI program not installed
				return;
			}
			$sinfo = @file('/var/apresa/data/ipmisensors.dat');
		} else
		{
			// use simulated data
			$sinfo = <<<SIMDATA
ID   | Name            | Type              | State    | Reading    | Units | Event
4    | System Temp     | Temperature       | Nominal  | 28.00      | C     | 'OK'
71   | Peripheral Temp | Temperature       | Nominal  | 36.00      | C     | 'OK'
138  | CPU Temp        | OEM Reserved      | N/A      | N/A        | N/A   | 'OEM Event = 0000h'
205  | FAN 1           | Fan               | Nominal  | 4500.00    | RPM   | 'OK'
272  | FAN 2           | Fan               | Nominal  | 4500.00    | RPM   | 'OK'
339  | FAN 3           | Fan               | Nominal  | 4575.00    | RPM   | 'OK'
406  | FAN 4           | Fan               | N/A      | N/A        | RPM   | N/A
473  | FAN A           | Fan               | Nominal  | 4800.00    | RPM   | 'OK'
540  | Vcore           | Voltage           | Nominal  | 0.68       | V     | 'OK'
607  | 3.3VCC          | Voltage           | Nominal  | 3.39       | V     | 'OK'
674  | 12V             | Voltage           | Nominal  | 12.24      | V     | 'OK'
741  | VDIMM           | Voltage           | Nominal  | 1.53       | V     | 'OK'
808  | 5VCC            | Voltage           | Nominal  | 5.06       | V     | 'OK'
875  | -12V            | Voltage           | Nominal  | -11.90     | V     | 'OK'
942  | VBAT            | Voltage           | Nominal  | 3.10       | V     | 'OK'
1009 | VSB             | Voltage           | Nominal  | 3.36       | V     | 'OK'
1076 | AVCC            | Voltage           | Nominal  | 3.41       | V     | 'OK'
1143 | Chassis Intru   | Physical Security | Nominal  | N/A        | N/A   | 'OK'
1210 | PS1 Status      | Power Supply      | Critical | N/A        | N/A   | 'Presence detected' 'Power Supply Failure detected' 'Power Supply input lost (AC/DC)'
1746 | PS2 Status      | Power Supply      | Nominal  | N/A        | N/A   | 'Presence detected'
SIMDATA;
			$sinfo = explode("\n",$sinfo);
		}
		$first = false;
    foreach ($sinfo as $line)
    {
      $line = trim($line);
      //dbglog('Line: '.$line);
      if ($line=='') continue;
      $parts = explode('|',$line);
			if (count($parts)!=7) continue;
			$state = trim($parts[3]);
			//dbglog( $state );
			if ($state=='Nominal' || $state=='Warning'
			|| $state=='Critical')
			{
				$compupart = trim($parts[1]);
				dbglog($compupart . ' has state ' . $state);
				$good = ($state=='Nominal');
				$value = trim($parts[4]);
				if ($value!='N/A')
				{
					$unit = trim($parts[5]);
					if ($unit!='N/A')
					{
						// add unit
						$value .= ' ' . trim($parts[5]);
					}
				}
				$msg = trim($parts[6]);
				$theid = 'IPMI_'.$compupart;
        if ($this->GetDebianVersion()>=10)
        {
          if ($compupart=='PS1 Status'
          || $compupart=='PS2 Status')
          {
            if ($msg!="'Presence detected'")
            {
              // if the presence is not detected it is missing
              $good = false;
              if ($msg=="'OK'")
              {
                $msg = 'Missing';
              }
            }
          }
        }
				if ($good==false && isset($this->ignoreList[ $theid ])==false )
				{
					if ($value=='N/A') $value = '';
					else $value .= ' , ';
					$value .= $msg;
				} else
				{
					if ($value=='N/A' && strlen($msg)<25)
					{
						$value = $msg;
					}
				}
				if ($first==false)
				{
					$first = true;
					$this->AddCaption('Hardware (IPMI)');
				}
				if ($compupart=='PS1 Status')
				{
					$this->handledPowerSupply = true;
				}
				$alarm_msg = $compupart . ': ' . $state . '. ' . $value;
				$this->Add( array
					(	'part' => $compupart
					,'qua'=>'num'
					,'value'=>$value
					,'h'=>$good
					,'id'=>$theid
					,'am'=>$alarm_msg
					)
				);				
			} else
			{
				// unknown state (N/A)
			}
		}
		//exit;
	}
	
	function ConstructTunnelName($vtapid)
	{
		global $dbintf;
		
		$vtap = $dbintf->GetVTap($vtapid);
		
		if($vtap === false) return $vtapid;
		
		if(strlen(str_ensure_defined($vtap['name'])) > 0)
		{
			$quaname = str_ensure_defined($vtap['name']);
		}
		else
		{			
			$quaname = $vtap['vtap_id'];
		}
			
		if(isset($vtap['tenantid']))
		{
			$tenant = $dbintf->GetGroupInfo($vtap['tenantid']);
			$quaname .= " (" . $tenant['name'] . ")";
		}
		
		return $quaname;
	}
	
	function TunnelConnectionsCheck()
  {
    global $dbintf;
		
		$TunnelOn = $dbintf->GetConfigValue('EnableTunnel');
		
		if( int_ensure_defined($TunnelOn) != 1)
			return;
		
		$this->AddCaption('V-Tap Connections');
	
		if(file_exists('/var/apresa/data/TunnelAcceptor.dat'))
		{
			$file = file_get_contents('/var/apresa/data/TunnelAcceptor.dat');
			$status = explode("=", $file);
			
			if(count($status) == 2)
			{
				if($status[1] === 'false')
				{
					$good = false;
					$value = 'Disconnected';
				}
				else
				{
					$good = true;
					$value = 'Connected';
				}
				$this->Add( array( 'part' => 'Tunnel Status', 'qua' => 'txt', 'value' => $status[0], 'h' => $good, 'id' => 'TunnelStatus', 'am' => "Error when opening tunnel" ));
			}
		}
		else
		{
			$this->newinfo[] = array( 'part' => 'Tunnel Status', 'qua' => 'txt', 'value' => 'unknown', 'h' => null, 'id' => 'TunnelStatus', 'am' => "Error when opening tunnel" );
		}

    $tunneltls = $dbintf->GetConfigValue('TunnelTLS');

    if($tunneltls)
    {
      if(file_exists('/var/apresa/data/TunnelAcceptorTls.dat'))
      {
        $file = file_get_contents('/var/apresa/data/TunnelAcceptorTls.dat');
        $status = explode("=", $file);
        
        if(count($status) == 2)
        {
          if($status[1] === 'false')
          {
            $good = false;
            $value = 'Disconnected';
          }
          else
          {
            $good = true;
            $value = 'Connected';
          }
          $this->Add( array( 'part' => 'Tunnel Status (TLS)', 'qua' => 'txt', 'value' => $status[0], 'h' => $good, 'id' => 'TunnelTlsStatus', 'am' => "Error when opening TLS tunnel" ));
        }
      }
      else
      {
        $this->newinfo[] = array( 'part' => 'Tunnel Status (TLS)', 'qua' => 'txt', 'value' => 'unknown', 'h' => null, 'id' => 'TunnelTlsStatus', 'am' => "Error when opening TLS tunnel" );
      }
    }
		
		$vtaps_connected = array();
		if(file_exists("/var/apresa/data/TunnelConnections.dat"))
		{
			$file = new CsvFile();
			$file->Read("/var/apresa/data/TunnelConnections.dat");

      $DisconErr = $dbintf->GetConfigValue('TunnelDisconErr');
      if($DisconErr === false) $DisconErr = 1;

			foreach($file as $parts)
	  	{
	    	$line_nr++;	
				if ($line_nr==1)
	    	{
	      	// the header
	      	continue;
	    	}

				if (count($parts)<13 && $parts[1]=='') continue;
				$id = $parts[0];
				$resettable = (int) $parts[4];

				$connected = (int) $parts[2];
				if($connected == -1)
				{
					$good = false;
					$value = $parts[3];
				}
				else if($connected == 0)
				{
					$value = $parts[3];
					$good = $DisconErr ? false : null;
				}
				else
				{
          $good = true;
					$value = $parts[1];
					
					$sd = (int) $parts[5];
					$phonebusy = (int) $parts[6];
					$recordingbusy = (int) $parts[8];
					$linedisconnected = (int) $parts[9];
					$clockwrong = (int) $parts[10];
					$rebootcount = (int) $parts[11];
					$error_code = (int) $parts[12];
					
					$extra_status = array();
					if($sd)
					{
						$extra_status[] = "SD Card not inserted.";
						$good = false;
					}

					if($phonebusy && !$recordingbusy)
					{
						$extra_status[] = "Phone Line is busy.";
					}

					if($recordingbusy)
					{
						$extra_status[] = "Recording.";
					}

					if($linedisconnected)
					{
						$extra_status[] = "Line disconnected.";
						$good = false;
					}

					if($clockwrong)
					{
						$extra_status[]= "Clock wrong.";
						$good = false;
					}

					if($rebootcount > 4)
					{
						$extra_status[] = "Many reboots in last 24 hours ($rebootcount).";
						$good = false;
					}

					if($error_code > 0)
					{
						$error_reason = [
							"OK",
							"SD card OS init failed",
							"SD card FS init failed",
							"SD card is write protected",
							"Other error"
						];

						$extra_status[] = "Error {$error_code}: {$error_reason[min($error_code, count($error_reason)-1)]}.";
						$good = false;
					}

					if(count($extra_status) > 0)
					{
						$value .= ": " . implode(" ", $extra_status);
					}
				}

				$quaname = $this->ConstructTunnelName($id);
					
				$am = "An error has occured for V-Tap {$id}";
				if($quaname != $id)
				{
					$am .= " ({$quaname})";
				}
				$entry = array( 'part' => $quaname, 'qua' => 'txt', 'value'=> $value, 'h' => $good, 'id' => 'tip_' . $id, 'am' => $am, 'constat' => $good );
				if($resettable)
				{
					$entry['ct'] = $id;
				}

				if($good === null)
				{
					$this->newinfo[] = $entry;
				}
				else
				{
					$this->Add($entry);
				}
				$vtaps_connected[] = $id;
			}
		}
		$vtaps_table = $dbintf->GetVTaps();
		$alarmtable = $dbintf->GetConfigValue('TunnelAlarmTableDisconnect');
		
		foreach($vtaps_table as $vtap)
		{
			$id = $vtap['vtap_id'];
			
			if(in_array($id, $vtaps_connected)) continue;
			
			$quaname = $this->ConstructTunnelName($id);
			
			$am = "An error has occured for V-Tap {$id}";
			if($quaname != $id)
			{
				$am .= " ({$quaname})";
			}
			
			if($alarmtable)
			{
				$this->Add(array('part' => $quaname, 'qua' => 'txt', 'value'=> 'Not connected', 'h' => false, 'id' => 'tip_' . $id, 'am' => $am));
			}
			else
			{
				$this->newinfo[] = array('part' => $quaname, 'qua' => 'txt', 'value'=> 'Not connected', 'h' => null, 'id' => 'tip_' . $id, 'am' => $am);
			}
		}
	}
	
	function LicOverflowCheck()
	{
		global $dbintf, $configs;
		$licoverflows = $dbintf->GetConfigsWithPrefix("LastLicOverflow");
		
		if(sizeof($licoverflows) > 0)
			$this->AddCaption("Channel limits");
		
		SetDateTimeF($dbintf->GetConfigValue('dateformat'),0);
		
		foreach($licoverflows as $param=>$date)
		{
			if(!$param)
			{
				$id = "licoverflow_voip";
				$name = "VoIP";
				$am = "Maximum number of VoIP channels reached: Call not recorded";
			}
			else if($param === "_RTA")
			{
				$id = "licoverflow_RTA";
				$name = "RTA";
				$am = "Maximum number of RTA channels reached: Call not recorded or not fully recorded";
			}
			else if(substr($param,0,7) === "_V-Tap_")
			{
				$vtapid = substr($param,7);
				$id = "licoverflow_VTAP_" . $vtapid;
				
				$vtapname = $this->ConstructTunnelName($vtapid);
				$name = "V-Tap $vtapname";
				$am = "Maximum number of V-Tap channels reached for $vtapname: Call not recorded";
			}
			else if(substr($param,0,8) === "_tenant_")
			{
				$groupid = intval(substr($param,10));
				$group = $dbintf->GetGroupInfo($groupid);
				
				if($group === false)
					$groupname = "group " . substr($param,10);
				else
					$groupname = $group["name"];
				
				$name = "Tenant $groupname";
				$id = "licoverflow_tenant_$groupid";
				$am = "Maximum number of channels reached for tenant $groupname: Call not recorded";
				
			}
			else
			{
				$name = substr($param, 1);
				$id = "licoverflow_" . $name;
				$am = "Maximum number of $name channels reached: Call not recorded";
			}
			
			$this->Add(array('part' => $name, 'qua' => 'txt', 'value'=> date($configs['datetimef'], $date), 'h' => false, 'am' => $am, 'id' => 'hid_' . $id));			
		}
	}

  function CertificateExpiryCheck()
  {
    global $dbintf, $configs;

    $configs_web = $dbintf->LoadConfigsPart('web');

    $PrivKeyExpiryAlarm = int_ensure_defined($configs_web["PrivKeyExpiryAlarm"]);
    $NoPrivKeyExpiryAlarm = int_ensure_defined($configs_web["NoPrivKeyExpiryAlarm"]);

    if($PrivKeyExpiryAlarm || $NoPrivKeyExpiryAlarm)
    {
      $ExpiryTime = int_ensure_defined($configs_web['CertExpiryTimeAlarm']);
      $certificates = $dbintf->GetExpringCerts($ExpiryTime, $PrivKeyExpiryAlarm, $NoPrivKeyExpiryAlarm);
      
      if(sizeof($certificates) > 0)
      {
        $this->AddCaption("Certificates");
        foreach($certificates as $cert)
        {
          if($cert['valid_to'] > time())
          {
            $am = "The certificate " . $cert['name'] . " will expire within " . $ExpiryTime . " days on " . date($configs['datetimef'], $cert['valid_to']);
            $val = 'Expires on ' . date($configs['datetimef'], $cert['valid_to']);  
          }
          else
          {
            $am = "The certificate " . $cert['name'] . " has expired on " . date($configs['datetimef'], $cert['valid_to']);
            $val = 'Expired on ' . date($configs['datetimef'], $cert['valid_to']);  
          }
          $this->Add(array('part' => $cert['name'], 'qua' => 'txt', 'value'=> $val, 'h' => false, 'am' => $am, 'id' => 'certid_' . $cert['id']));	
        }
      }
    }
  }
}

function setConfArr($fn,$arr)
{
  $to_write = '';
  foreach ($arr as $key=>$value)
  {
    $to_write .= "$key=".($value?1:0)."\n";
  }
  if (file_put_contents($fn . '.new', $to_write)!==false)
  {
    rename($fn . '.new', $fn);
  } else
  {
    unlink($fn . '.new');
  }
}

function getConfArr($fn)
{
  $arr = array();
  $fp = @fopen($fn,'r');
  if ($fp)
  {
    while (!feof($fp))
    {
      $line = fgets($fp);
      $parts = explode('=',$line);
      if (count($parts)==2)
      {
        $key = trim($parts[0]);
        $value = trim($parts[1]);
        $arr[ $key ] = $value;
      }
    }
    fclose($fp);
  }
  return $arr;
}

function UpdateHealthState($datapath,$simulate=false)
{
  global $debug_mode;
	global $dbintf;
  // prepare result object
  $data = new HealthData();

  $data->init($datapath);
  $data->simulate = $simulate;
  $data->load();
	
	$data->InitCluster();

	$data->SimulatedError();

	// move to other place?
	$data->Clustering();
	
	$data->AddCaption('Hardware');
	$data->IPMI();
  $data->CustomCheck(); // power supply

	// hardware: voltage / fans
	$data->AddCaption('Hardware sensors');
  $data->Sensors();

	// hardware: interfaces
	$data->AddCaption('Hardware interfaces');
  $data->RecIntfCheck(); // card and ch. not missing, PRI-line up
  $data->VoIPCheck(); // eth up and no lost packets
	
	// hardware: disks
	$data->AddCaption('Storage');
  $data->HardDiskTemp();
  
  $data->HPcheck();
  $data->RAIDcheck();
  $data->DiskSpaceCheck();
	$data->TenantSpaceCheck();
	$data->BackupCheck();
	$data->ExportCheck();

	// system resource usage
	$data->AddCaption('System resource usage');
  $data->ResourceCheck(); // CPU and memory

	// processes
	$data->AddCaption('Recording software');
  $data->ProcesssesErrFileCheck(); // processes
  $data->VoipEngineCheck();
  
  $data->AddCaption('Connectors');
  $data->ProcesssesErrFileCheckConnectors();
	$data->LdapCheck();
	$data->AzGroupSyncCheck();
	$data->CSTALinkCheck();	//CSTA link.
  $data->JtapiCheck();
  $data->ALPCheck();
	$data->ToshibaCheck();
  $data->YateCheck();
	$data->TunnelConnectionsCheck(); // V-tap
  $data->ShoreTelDbCheck();
	$data->TeamsBot();
	
	// recording functionality
	$data->AddCaption('Recording functionality');
  $data->LicensingCheck();
  $data->RecCheck(); // recording file integrity
  $data->InactivityCheck(-1,"",false); 	//Recording inactivity check. Not for specific tenant.
	$data->MaxPeakCalls();
	
	//Recording functionality per Tenant
	$tenants_info = $dbintf->GetTenantList();
	foreach($tenants_info as $tenant_info)
	{
		$lookupkey = 'TN'.$tenant_info['tenantid'].'_CheckInactivity';
		$value = $dbintf->GetConfigValue($lookupkey);
		if($value==1) $data->InactivityCheck($tenant_info['tenantid'],$tenant_info['name'],true);		//Use default alarm settings.
		if($value==2) $data->InactivityCheck($tenant_info['tenantid'],$tenant_info['name'],false);	//Use specific alarm settings.
	}
	
	// post-processing
	$data->TransferCheck();
  $data->ScribeCheck();

	$data->LicOverflowCheck();

  $data->CertificateExpiryCheck();

  $data->write();

  return $data;
}

function dbglog($text)
{
  global $debug_mode;
  if (isset($debug_mode) && $debug_mode)
  {
    echo $text . "\n";
  }
}

function GetInfoTable($infoarr)
{
  $t = '';
  $t .= "<style>\n";
  $t .= "table.infotab td {padding:0.2em;}\n";
  $t .= "tr.hdr td{background-color:#008;color:#fff;font-weight:bold;}\n";
  $t .= "td.ok{background-color:#0e0;color:#000;}\n";
  $t .= "td.alarm{background-color:#e00;color:#fff;}\n";
	$t .= "td.caption{font-weight:bold;background-color:#eee;}\n";
  $t .= "</style>\n";

  $t .= "<table class=\"infotab\" border=\"1\">\n";
  $t .= '<tr class="hdr">'
  . '<td>Measured quantity'
  . '<td>Value'
  . '<td>Status'
  . '<td>'
  . '</tr>' . "\n";

  //. '<td>Info'
  $last_qua = '';
  foreach ($infoarr as $info)
  {
    $alignment = 'right';
		if ($info['qua']=='caption')
		{
			$t .= "<tr><td colspan=\"4\" class=\"caption\">" . HtmlQuote($info['part']);
			continue;
		}
    if ($info['qua']=='txt') $alignment='left';
    
		$t .= "<tr><td>" . HtmlQuote($info['part']);
				
		$t .= "<td align=\"$alignment\">" . HtmlQuote($info['value']);
		
		if ($info['h']===2)
    {
      $t .= "<td class=\"info\">ignored";
    } else if ($info['h']===true)
    {
      $t .= "<td class=\"ok\">OK";
    } else if ($info['h']===null)
    {
      $t .= "<td class=\"info\">info";
    } else
    {
      $t .= "<td class=\"alarm\">ALARM";
      $err = int_ensure_defined($info['err']);
      if ($err==1) $t .= '<br>TOO HIGH';
      if ($err==2) $t .= '<br>TOO LOW';
    }
    //$t .= '<td>' . $info['xtr'];
    if ($info['h']!==null && isset($info['id']))
    {
      if ($info['h']===2)
      {
        $t .= '<td class="hasign" hid="'.$info['id'].'">';
      } else
      {
        $t .= '<td class="hashid" hid="'.$info['id'].'">';
      }
    } else
    {
      $t .= '<td>';
    }
		if(isset($info['ct']) && isset($info['constat']) && !$info['constat'])
		{
			$t .= '<td class="hasclear" cid="'.$info['ct'].'">';
		}
    $t .= "</tr>\n";
  }
  $t .= "</table>\n";
  return $t;
}

function GetSNMPInfo($infoarr, $messagearr, $any_failure)
{
  global $configs;

  $t = ''; 
  
  if(str_ensure_defined($configs["SNMPGeneralErr"]) == "1")
  {
    if($any_failure)
    {
      $t .= "Errors|There are errors|40||generr\n";
    }
    else
    {
      $t .= "Errors|There are no errors|20||generr\n";
    }
  }

  $t .= "Database|OK|20||dbcon";

  $messagecounter = 0;
  foreach ($infoarr as $info)
  {
		if ($info['qua']=='caption')
		{
			continue;
		}
    $t .= $info['part'];
		$t .= "|" . str_replace("|", "\\", str_replace("\t", " ", str_replace(array("\r", "\n", "°"), '', $info['value']))) . "|";
    if ($info['h']===2)
    {
      $t .= "10|";
    } else if ($info['h']===true)
    {
      $t .= "20|";
    } else if ($info['h']===null)
    {
      $t .= "30|";
    } else
    {
      $t .= "4";
      $err = int_ensure_defined($info['err']);
      if ($err==1) $t .= "1";
      if ($err==2) $t .= "2";
      else $t .= "0";
	  
      $t .= "|" . $messagearr[$messagecounter];
      $messagecounter++;
    }
		$t .= "|" . str_ensure_defined($info['id']); 
    $t .= "\n";
  }
  return $t;
}

function formatBytes($value)
{
  if ($value < 1024 * 5)
  {
    return $value . ' B';
  } else if ($value < 1024*1024 * 5)
  {
    return round($value/1024) . ' KB';
  } else if ($value < 1024*1024*1024 * 5)
  {
    return round($value/(1024*1024)) . ' MB';
  } else
  {
    return round($value/(1024*1024*1024)) . ' GB';
  }
}

function SetHealthConfigDefaults(& $configs)
{
  $keys  = array(
    'iaStart1' => '08:00'
  ,	'iaStop1'  => '12:00'
  , 'iaStart2' => '13:00'
  ,	'iaStop2'  => '17:00'
  , 'iaWeekdays' => '1234567'
  , 'iaPeriodSystem' => '60'
	, 'iaPeriodSystemApresa' => '60'
	, 'iaPeriodOtherRec' => '60'
  , 'iaPeriodChannel' => '180'
  , 'iaMaxCallLength' => '600'
	, 'iaPeakCalls' => '100'
  );
  foreach ($keys as $key => $value)
  {
    if (isset($configs[$key])==false)
    {
      $configs[$key] = $value;
    }
  }
}

function setNodeScore( $node_health )
{
      if ( @file_exists('/var/apresa/conf/opt_cluster') == false )
      {
        return;
      }

      $nodename = system('hostname');
      exec( "crm_attribute -l reboot -N $nodename -n apresa_score -v $node_health" );
      // set location score
      // we use a shadow configure to make it atomic
      // we add hostname to make unique
      $confname = "newconf_$nodename";
      exec( "crm cib new $confname" );
      exec( "crm -c $confname configure delete loc_$nodename" );
      exec( "crm -c $confname configure location loc_$nodename anode $node_health: $nodename" );
      exec( "crm cib commit $confname" );
      exec( "crm cib delete $confname" );
}

function dbglogff($line)
{
        $file = '/var/apresa/data/logje.txt';
        $current = file_get_contents($file);
        $current .= $line;
        file_put_contents($file, $current);
}
?>