<?php

function handleError($txt)
{
	lg($txt);
	throw new Exception($txt);
}

class LdapImport
{
	var $domsg = true;
	var $oldSipFilter = '';
	var $oldH323Filter = '';
	var $oldUnimegaFilter = '';
	var $groups = array();
	var $groupFailed = '';
	var $failure = false;
	var $failureDescr = '';
	var $failureList = array();

	function Init()
	{
		global $dbintf;
		$this->oldSipFilter = $dbintf->GetConfigValue('SipFilters');	//Get the old SIP-filter!
		$this->oldH323Filter = $dbintf->GetConfigValue('H323Filters');		//Get the old H323-filter!
		$this->oldUnimegaFilter = $dbintf->GetConfigValue('UnimegaFilters');		//Get the old Unimega-filter!
	}
	
	function ResetFailure()
	{
		$this->failure = false;
		$this->failureDescr = '';
	}
	
	function AddFailure($groupName)
	{
		if ($this->failure==false) return;
		$this->failureList[$groupName] = $this->failureDescr;
	}
	
	function AddError($groupName,$errorDescr)
	{
		$this->failureList[$groupName] = $errorDescr;
	}
	
	function CheckLdapPhoneItem($phonenum)
	{
		if ($phonenum=='') return false;
		return true;
	}
	
	function LdapImportGroup($groupid)
	{
			global $dbintf, $msg;
			$configs2 = $dbintf->LoadConfigsPart('n');
      $LdapPhonesToLower = int_ensure_defined( $configs2['LdapPhonesToLower'] ) != 0;
			$LdapUsrToLower = int_ensure_defined( $configs2['LdapUsrToLower'] ) != 0;
			$a = $dbintf->GetGroupInfo($groupid);
			if ($a['adgroup']=='') return;
			//Fill the ldaphost field if empty!
			if($a['ldaphost'] == '')
			{
				$grpdat['ldaphost'] = str_ensure_defined($configs2['LdapHost']);
				if($grpdat['ldaphost'] != '')
				{
					$dbintf->EditGroup($groupid,$grpdat);
					$a['ldaphost'] = $grpdat['ldaphost'];
				}
			}
      $rod_def = 0;
      if ($a['tenantid']!=0)
      {
        $tenantid = (int) $a['tenantid'];
        $tn_prefix = 'TN' . $tenantid . '_';
        $rod_def = (int)$dbintf->GetConfigValue( $tn_prefix . 'StoreOnDmdDef' );		//Be sure that GetConfigValue always returns an integer!
      }
      $logonmethod = 1; // LDAP
      if ($a['idp']!=0)
      {
        $logonmethod = 2; // SAML (ADFS)
      }
      
			$nmbretries=0;
			$res = -1;
			do
			{
				$nmbretries++;
				lg('Synchronizing group '.$a['name'].' with '.$a['adgroup'].' on host '.$a['ldaphost'].'. '.$nmbretries.' try.');
				$res = get_members($a['adgroup'], $a['ldaphost']);
			}
			while($nmbretries < 2 && $res == -1);		//Just one retry!
			if($res == -1)
			{
				lg('Group skipped because of error.');
				$this->failure = true;
				return;																//LDAP search error! Skip this group!
			}
			$members = array();
			$UPNNames = false;
			if(isset($configs2['LdapUseUPN']))
			{
				$UPNNames = ($configs2['LdapUseUPN']=="1");
			}
			foreach($res as $b)
			{
				$accid = '';
				$email = $b['mail'];
				$usr = $b['samaccountname'];
				$upnusr = $b['userprincipalname'];
				if ($LdapUsrToLower)
				{
					$usr = strtolower($usr);
					$upnusr = strtolower($upnusr);
				}
				if (!isset($configs2['LdapTel1'])) $tel1='Telephone number (general)';	else $tel1=$configs2['LdapTel1'];	//Default. TelephoneNumber on general page!
				if (!isset($configs2['LdapTel2'])) $tel2='-'; else $tel2=$configs2['LdapTel2'];
				if (!isset($configs2['LdapTel3'])) $tel3='-'; else $tel3=$configs2['LdapTel3'];
				$phone_items = array();
				if($tel1=='Telephone number (general)' || $tel2=='Telephone number (general)' || $tel3=='Telephone number (general)') $phone_items[] = $b['telephonenumber'];	//Telephone number on general page.
				if($tel1=='Home' || $tel2=='Home' || $tel3=='Home') $phone_items[] = $b['homephone'];																						//Home.
				if($tel1=='Mobile' || $tel2=='Mobile' || $tel3=='Mobile') $phone_items[] = $b['mobile'];																				//Mobile.
				if($tel1=='IP phone' || $tel2=='IP phone' || $tel3=='IP phone') $phone_items[] = $b['ipphone'];																	//IP phone.
				if($tel1=='E-mail' || $tel2=='E-mail' || $tel3=='E-mail') $phone_items[] = $b['mail'];																					//E-mail.
				if($tel1=='Pager' || $tel2=='Pager' || $tel3=='Pager') $phone_items[] = $b['pager'];																						//Pager.
				if($tel1=='Phone-Office' || $tel2=='Phone-Office' || $tel3=='Phone-Office') $phone_items[] = $b['physicaldeliveryofficename'];	//Physical-Delivery-Office-Name.
				if($tel1=='Phone-Lync' || $tel2=='Phone-Lync' || $tel3=='Phone-Lync')		//Primary SIP Phone (Lync).
				{
					$temp = $b['msrtcsip-line'];			//Remove the "tel:" and the "ext=".
					if(stripos($temp, "tel:") === 0)
					{
						$temp = substr($temp,4);
						$pos = strpos($temp, ";");
						if($pos !== false) $temp = substr($temp, 0, $pos);
						$temp = trim($temp);
					}
					$phone_items[] = $temp;
				}
				if($tel1=='Address-Lync' || $tel2=='Address-Lync' || $tel3=='Address-Lync')		//Primary SIP Address (Lync).
				{
					$temp = $b['msrtcsip-primaryuseraddress'];			//Remove the "sip:".
					if(stripos($temp, "sip:") === 0)
					{
						$temp = substr($temp,4);
						$temp = trim($temp);						//Just to be sure.
					}
					$phone_items[] = $temp;
				}
				$phone_items = array_filter( $phone_items , "LdapImport::CheckLdapPhoneItem" );
        if ($LdapPhonesToLower)
        {
          // Convert all phones to lower case
          $phone_items_new = array();
          foreach ($phone_items as $phone)
          {
            $phone_items_new[] = strtolower($phone);
          }
          $phone_items = $phone_items_new;
        }
				$name = $b['name'];
        // for some reason name equals display name on our test system
        // still get the display name in case it matters
        $displayName = $b['displayname'];
				$confchanged = false;
				if($UPNNames) $UserExists=$dbintf->UsrExists($upnusr);
				else $UserExists=$dbintf->UsrExists($usr);
				if(!$UserExists)
				{
					//Maybe recently changed!
					if($UPNNames) $UserExists=$dbintf->UsrExists($usr);
					else $UserExists=$dbintf->UsrExists($upnusr);
					if($UserExists) $confchanged=true;
				}
				if(!$UserExists)
				{
					lg('New user: '.$displayName);
					//New user for Apresa!
					if($UPNNames) $usr=$upnusr;
					lg('Adding user: '.$usr.' ('.$name.')');
					$usrdata['usr'] = $usr;
					$usrdata['name'] = $name;
					$usrdata['email'] = $email;
					$usrdata['mphones'] = '';
					$usrdata['enabled'] = 1;
					$usrdata['rod'] = $rod_def;
					$usrdata['logonmethod'] = $logonmethod;	// LDAP or ADFS
					$usrdata['playtimelim'] = 0;
					$usrdata['ldapuser'] = 1;			//User added because of ldap synchronization.
					$usrdata['ldaphost'] = $a['ldaphost'];
					$isok = $dbintf->AddUserAccount($userid,$usrdata,0);
					if (str_ensure_defined($phone_items[0]) != "")
					{
						$phones = array();	//Just to be sure!
						foreach ($phone_items as $phone_item)
						{
							$obj = null;
							$res = $dbintf->GetOrCreateCallerID($phone_item,'',1,$displayName);
              if ($dbintf->GetContactName($res['contactid']) == '')
              {
                lg('set name (3) to ' . $displayName . ' for contact id ' . $res['contactid'] );
                $dbintf->EditContact($res['contactid'],$displayName);
              }                
							$obj['contactid'] = $res['contactid'];
							$phones[] = $obj;
						}
						$accid = $dbintf->GetAccidUsr($usr);
						$isok = $dbintf->SetUserPhones($accid,$phones);
						if ($isok==false)
						{
							if ($this->domsg)
							{
								$msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr).' (phones)');
							}
						}
					}
				}
				else
				{
					//The user exists. Are the credentials still the same (e-mail, name, telephonenumbers)?
					if(!$confchanged)
					{
						if($UPNNames) $accid = $dbintf->GetAccidUsr($upnusr);
						else $accid = $dbintf->GetAccidUsr($usr);
					}
					else
					{
						if($UPNNames) $accid = $dbintf->GetAccidUsr($usr);
						else $accid = $dbintf->GetAccidUsr($upnusr);
					}
					$userinfo = $dbintf->GetUserInfoOnAccId($accid);
					if($userinfo['ldapuser'])		//Only useraccounts created by ldap-synchronization!
					{
						$callerids = $dbintf->GetUserPhones($accid);
						$telephonesold = array();	//Define an array!
						foreach($callerids as $callerid)
						{
							$contactinfo = $dbintf->GetContactInfo($callerid);
              // if contact is still part of this user,
              if (in_array($contactinfo['callerid'],$phone_items)
              // and no name is yet assigned
                && $contactinfo['name'] == '')
              {
                lg('set name (1) to ' . $displayName . ' for contact id ' . $callerid );
                $dbintf->EditContact($callerid,$displayName);
              }                
							$telephonesold[] = $contactinfo['callerid'];
						}
						if (count(array_diff($phone_items, $telephonesold))>0 || count(array_diff($telephonesold, $phone_items))>0)
						{
							lg("Phones of user: ".$upnusr." changed.");
							$phones = array();	//Just to be sure!
							foreach ($phone_items as $phone_item)
							{
								$obj = null;
								$res = $dbintf->GetOrCreateCallerID($phone_item,'',1,$displayName);
                if ($dbintf->GetContactName($res['contactid']) == '')
                {
                  lg('set name (2) to ' . $displayName . ' for contact id ' . $res['contactid'] );
                  $dbintf->EditContact($res['contactid'],$displayName);
                }                
								$obj['contactid'] = $res['contactid'];
								$phones[] = $obj;
							}
							$isok = $dbintf->SetUserPhones($accid,$phones);
							if ($isok==false)
							{
								if ($this->domsg)
								{
									$msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr).' (phones)');
								}
							}
						}
						if ($userinfo['name'] != $name || $userinfo['email'] != $email || $userinfo['ldaphost'] != $a['ldaphost'] || $confchanged)
						{
							lg("Name or email for user: ".$upnusr." changed.");
							//Name or email of the user changed! Make a new array! $userinfo contains 0-indexed column numbers!
							$newinfo = array();
							$newinfo['name'] = $name;
							$newinfo['email'] = $email;
							$newinfo['ldaphost'] = $a['ldaphost'];
							if($confchanged)
							{
								if($UPNNames) $usr=$upnusr;
								$newinfo['usr'] = $usr;
							}
							$dbintf->EditUserAccountLC($userinfo['id'], $newinfo);
						}
					}
				}
				if($accid == '') $accid = $dbintf->GetAccidUsr($usr);	//$accid is never '' for an existing (ldap) user!
				$members[] = $accid;
			}
			$oldmembers = $dbintf->GetGroupMembers($a['accid']);
			//Are there any useraccounts made by ldap and not part of any group anymore?
			$delusers = array_diff($oldmembers, $members);
			//SetGroupMembers is quite slow!!!
			if (count($delusers) != 0 || (count($oldmembers) != count($members)))
			{
				$dbintf->SetGroupMembers($a['accid'],$members);
			}
			foreach ($delusers as $deluser)
			{
				if ($dbintf->IsUserCreatedByAD($deluser))
				{
					$dbgroups = $dbintf->GetGroupsOfWhichMember($deluser);
					if (count($dbgroups)==0)
					{
						lg("Removing user from group, accid: ".$deluser);
						//The user account was created by LDAP synchronization and is no longer member of any group => delete!
						$user = $dbintf->GetUserInfoOnAccId($deluser);
						$dbintf->DeleteUserAccount($user['id']);
					}
				}
			}
			
			// force recalc
			$dbintf->userphones = array();
			unset($dbintf->allgroups);
	}
	
	function DbFinish()
	{
		global $dbintf;
		$dbintf->CalcTenantUsers();
		$newcfg = array();
		$anyFail = count($this->failureList)>0;
		$newcfg['LdapHealth'] = $anyFail?'0':'1';
		$errorTexts = array();
		foreach ($this->failureList as $groupName => $errDescr)
		{
			if ($errDescr=='') $errDescr = 'Error';
			$errorTexts[] = $errDescr . ' for group ' . $groupName;
		}
		$hstatus = implode(', ',$errorTexts);
		if ($anyFail==false) $hstatus = 'OK';
		$newcfg['LdapHStatus'] = $hstatus;
		$dbintf->SetConfigsPart($newcfg,'n');
	}
	
	function Finalize()
	{
    EnginesSettingsCalcApply();
	}
	
}

function get_groups($ldaphost)
{
	global $dbintf;
	
	$configs_n = $dbintf->LoadConfigsPart('n');
	foreach($configs_n as $param=>$value)	//Search for the LDAP servers!
	{
		$pos = strpos($param, "LdapHost");
		if(($pos !== false) && ($value == $ldaphost))
		{
			$key = substr($param,8);		//LdapHost, LdapHost2, LdapHost3, etc.
		}
	}
	//Get the ldaphost, usersdomain and user properties.
	$usersdomain = str_ensure_defined($configs_n['LdapDomain'.$key]);
	$ldaphost = str_ensure_defined($configs_n['LdapHost'.$key]);
	$ldapuser = str_ensure_defined($configs_n['LdapUser'.$key]);
	$ldappwd = str_ensure_defined($configs_n['LdapPwd'.$key]);
	$domain_usr_prefix = str_ensure_defined($configs_n['UsersDomain'.$key]);
	if ($ldaphost=='') return array();
	if (strpos($ldapuser,'\\')===false
	&& strpos($ldapuser,'@')===false
	&& $domain_usr_prefix!='')
	{
		$ldapuser = $domain_usr_prefix . '\\' . $ldapuser;
	}
	
	//Active Directory DN, base path for our querying user
	if (strpos($usersdomain,'=')!==false)
	{
		$ldap_dn = $usersdomain;
	} else
	{
		$dc = explode(".", $usersdomain);
		$ldap_dn = "";
		foreach($dc as $_dc) $ldap_dn .= "dc=".$_dc.",";
		$ldap_dn = substr($ldap_dn, 0, -1);
	}
	
	//Connect to AD
	$ldap = ldap_connect($ldaphost) or handleError("Could not connect to LDAP");
	ldap_set_option($ldap, LDAP_OPT_PROTOCOL_VERSION, 3);
	ldap_set_option($ldap, LDAP_OPT_REFERRALS, 0);
	ldap_bind($ldap, $ldapuser, decrypt($ldappwd)) or handleError("Could not bind to LDAP");
  
  $output = array();
  if(version_compare(PHP_VERSION, '8.0.0', '<'))
	{
    //Enable pagination with a page size of 100.
    $pageSize = 100;
    $cookie = '';
		$entries2 = array();
		do
		{
			ldap_control_paged_result($ldap, $pageSize, true, $cookie);
			$results2 = ldap_search($ldap,$ldap_dn,"(objectcategory=group)",array("distinguishedname","primarygrouptoken"));
			$entriesfound = ldap_get_entries($ldap, $results2);
			$entries2 = array_merge($entries2, $entriesfound);
			ldap_control_paged_result_response($ldap, $results2, $cookie);
		}
		while($cookie !== null && $cookie != '');
    //Remove extraneous first entry
	  array_shift($entries2);
	
	  //Loop through and find group with a matching primary group token
	  foreach($entries2 as $e)
	  {
		  $output[] = $e['distinguishedname'][0];
	  }
	}
  else 
  {
    $cookie = '';
    $pageSize = 100;
    $entries2 = array();
    do
    {
      $results2 = ldap_search($ldap,$ldap_dn,"(objectcategory=group)",array("distinguishedname","primarygrouptoken"), 0, -1, -1, LDAP_DEREF_NEVER, [
        [
          'oid' => LDAP_CONTROL_PAGEDRESULTS,
          'value' => ['size' => $pageSize, 'cookie' => $cookie],
          'iscritical' => true
        ]
      ]);

      if(ldap_parse_result($ldap, $results2, $errcode, $matcheddn, $errmsg, $referrals, $controls) == false)
      {
        handleError("Could not parse ldap result");
      }

      if($errcode != 0) 
      {
        handleError("LDAP error: " . $errcode . " " . $errmsg);
      }

      if(isset($controls[LDAP_CONTROL_PAGEDRESULTS]['value']['cookie'])) {
        $cookie = $controls[LDAP_CONTROL_PAGEDRESULTS]['value']['cookie'];
      }
      else {
        $cookie = '';
      }

      $entriesfound = ldap_get_entries($ldap, $results2);

      $entriesCnt = $entriesfound['count'];
      unset($entriesfound['count']);
      
      //lg("LDAP group entries found: " . $entriesCnt . " - more pages: " . (empty($cookie) ? "No" : "Yes") . " - paging: critical - memorory usage: " . memory_get_usage());

      foreach($entriesfound as $e)
      {
        $output[] = $e['distinguishedname'][0];
      }

      /*if(count($output) > 2500) {
        lg("Too many LDAP groups, bailing out early...");
        break;
      }*/

    }while(strlen($cookie) > 0);
  }

  lg("Total LDAP groups found: " . count($output));
	
	ldap_unbind($ldap);
  $dbintf->close();
	return $output;
}

function get_members($group, $ldaphost)
{
	global $dbintf;
	
	$configs_n = $dbintf->LoadConfigsPart('n');
	foreach($configs_n as $param=>$value)	//Search for the LDAP servers!
	{
		$pos = strpos($param, "LdapHost");
		if(($pos !== false) && ($value == $ldaphost))
		{
			$key = substr($param,8);		//LdapHost, LdapHost2, LdapHost3, etc.
		}
	}
	
	//Get the ldaphost, usersdomain and user properties.
	// Why oh why is this variable called $usersdomain
	// although there is another setting called UsersDomain?
	// So confusing!
	$usersdomain = str_ensure_defined($configs_n['LdapDomain'.$key]);
	$ldaphost = str_ensure_defined($configs_n['LdapHost'.$key]);
	$ldapuser = str_ensure_defined($configs_n['LdapUser'.$key]);
	$ldappwd = str_ensure_defined($configs_n['LdapPwd'.$key]);
	$domain_usr_prefix = str_ensure_defined($configs_n['UsersDomain'.$key]);
	if (strpos($ldapuser,'\\')===false
	&& strpos($ldapuser,'@')===false
	&& $domain_usr_prefix!='')
	{
		$ldapuser = $domain_usr_prefix . '\\' . $ldapuser;
	}
	
  //Active Directory DN
	if (strpos($usersdomain,'=')!==false)
	{
		$ldap_dn = $usersdomain;
	} else
	{
		$dc = explode(".", $usersdomain);
		$ldap_dn = "";
		foreach($dc as $_dc) $ldap_dn .= "dc=".$_dc.",";
		$ldap_dn = substr($ldap_dn, 0, -1);
	}

	$keep = array("samaccountname","name","mail","userprincipalname","displayname"
  ,"telephonenumber","homephone","mobile","ipphone","pager","msrtcsip-line","msrtcsip-primaryuseraddress","physicaldeliveryofficename"
  );
	//Connect to AD
	$ldap = ldap_connect($ldaphost) or handleError("Could not connect to LDAP");
	ldap_set_option($ldap, LDAP_OPT_PROTOCOL_VERSION, 3);
	ldap_set_option($ldap, LDAP_OPT_REFERRALS, 0);
	ldap_bind($ldap, $ldapuser, decrypt($ldappwd)) or handleError("Could not bind to LDAP");
 
 	//Begin building query
 	if($group) $query = "(&"; else $query = "";
 	$query .= "(&(objectClass=user)(objectCategory=person))";
	$query .= "(memberOf:1.2.840.113556.1.4.1941:=$group)";		//LDAP_MATCHING_RULE_IN_CHAIN! Also nested groups!
	//Close query
	$query .= ")";

	//Search AD
	$searcherror = false;
  $output = array(); //Declare the output array
  if(version_compare(PHP_VERSION, '8.0.0', '<'))
	{
    //Enable pagination with a page size of 100.
    $pageSize = 100;
    $cookie = '';
		$entries = array();
		do
		{
			ldap_control_paged_result($ldap, $pageSize, true, $cookie);
			$results = ldap_search($ldap, $ldap_dn, $query, $keep);
			if(ldap_error($ldap) == "Success")
			{
				$entriesfound = ldap_get_entries($ldap, $results);
				lg('Number of entries found: '.$entriesfound['count']);
				$entries = array_merge($entries, $entriesfound);
			}
			else
			{
				lg('Ldap_search error: '.ldap_error($ldap));
				$searcherror=true;
        // stop to avoid endless loop
        break;
			}
			ldap_control_paged_result_response($ldap, $results, $cookie);
		}
		while($cookie !== null && $cookie != '');

    //Remove first entry (it's always blank)
	  array_shift($entries);
    //Build output array
    foreach($entries as $u)
    {
      $item = array();
      foreach($keep as $x)
      {
        //Check for attribute
        if(isset($u[$x][0])) $attrval = $u[$x][0]; else $attrval = NULL;
  
        //Append attribute to output array
        $item[$x] = $attrval;
      }
      $output[] = $item;
    }
	}
  else
  {
    $pageSize = 100;
    $cookie = '';
		$entries = array();

    do
    { 
      $results = ldap_search($ldap,$ldap_dn,$query, $keep, 0, -1, -1, LDAP_DEREF_NEVER, 
        [['oid' => LDAP_CONTROL_PAGEDRESULTS, 'value' => ['size' => $pageSize, 'cookie' => $cookie], 'iscritical' => true]]);

      ldap_parse_result($ldap, $results, $errcode, $matcheddn, $errmsg, $referrals, $controls);

      if($errcode == 0)
      {
        $entriesfound = ldap_get_entries($ldap, $results);
        if(isset($controls[LDAP_CONTROL_PAGEDRESULTS]['value']['cookie'])) {
          $cookie = $controls[LDAP_CONTROL_PAGEDRESULTS]['value']['cookie'];
        }
        else {
          $cookie = '';
        }

        //lg('Number of entries found: '.$entriesfound['count'] . " - more pages: " . (empty($cookie) ? "No" : "Yes") . " - paging critical - memorory usage: " . memory_get_usage());
        unset($entriesfound["count"]);
        foreach($entriesfound as $u)
        {
          $item = array();
          foreach($keep as $x)
          {
            //Check for attribute
            if(isset($u[$x][0])) $attrval = $u[$x][0]; else $attrval = NULL;
      
            //Append attribute to output array
            $item[$x] = $attrval;
          }
          $output[] = $item;
        }

        /*if(count($output) > 2500) {
          lg("Too many LDAP users, bailing out early...");
          break;
        }*/
      }
      else
      {
        lg('Ldap_search error: '.$errmsg);
        $searcherror=true;
        // stop to avoid endless loop
        break;
      }

    }while(strlen($cookie) > 0);
  }
	if($searcherror) return -1;

  lg('Total number of entries found: ' . count($output));

	ldap_unbind($ldap);
  return $output;
}

?>