<?php

require_once 'signaling.php';
require_once 'actions.php';
require_once 'utility.php';
require_once 'encryption.php';

// Called from service.php and specialtask.php
function DoMitelImport()
{
	global $dbintf, $mitelsecret, $miteltoken, $miteladdress;
	
	if(in_array('curl',get_loaded_extensions()) === false)
	{
		addToSay(MSG_ERR, "PHP Curl extension required for mitel import not found");
		return;
	}
	
	libxml_use_internal_errors(true);

	lg("Doing Mitel import");

	// Interface to database
	if(!isset($dbintf))
	{
		$dbintf = new DBInterface(CMD_WEB_RELPATH . DATA_PATH);
		$dbintf->SetAdmin();
	}

	$configs_n = $dbintf->LoadConfigsPart('n');

	$mitelsecret = decrypt(str_ensure_defined($configs_n['MitelSecret']));
	$miteltoken = str_ensure_defined($configs_n['MitelToken']);
	$miteladdress = str_ensure_defined($configs_n['MitelAddress']);

	$ImportSuper = int_ensure_defined($configs_n['MitelSuper']);

	$mitelImporter = new MitelImporter($dbintf);

	$success = $mitelImporter->fetchData();

	if($success === false)
	{
		addToSay(MSG_ERR,"Mitel Import: Failed to retrieve mitel data");
		lg("No data could be fetched for mitel import");
		return;
	}

	//$mitelImporter->dumpData();
	$mitelImporter->importData($ImportSuper);
	EnginesSettingsCalcApply();

	
	addToSay(MSG_INF,"Mitel Import finished");
	libxml_use_internal_errors(false);
}

function logXmlErrors($xml)
{
	$errors = libxml_get_errors();

	foreach($errors as $error)
	{
		lg(display_xml_error($error, $xml));
	}

	libxml_clear_errors();
}

function display_xml_error($error, $xml)
{
    $return  = $xml[$error->line - 1] . "\n";
    $return .= str_repeat('-', $error->column) . "^\n";

    switch ($error->level) {
        case LIBXML_ERR_WARNING:
            $return .= "Warning $error->code: ";
            break;
         case LIBXML_ERR_ERROR:
            $return .= "Error $error->code: ";
            break;
        case LIBXML_ERR_FATAL:
            $return .= "Fatal Error $error->code: ";
            break;
    }

    $return .= trim($error->message) .
               "\n  Line: $error->line" .
               "\n  Column: $error->column";

    if ($error->file) {
        $return .= "\n  File: $error->file";
    }

    return "$return\n\n--------------------------------------------\n\n";
}

function getSignedUrl($method, $url)
{
	global $mitelsecret, $miteltoken, $miteladdress;
	
	$url = 'http://' . $miteladdress . '/' . $url;
	
	$nonce = base64_encode(mt_rand());
	$parameters = array();
	$parameters['noauth_nonce'] = $nonce;
	$parameters['noauth_token'] = $miteltoken;
	ksort($parameters);
	$parameterStr = '';

	foreach($parameters as $key => $value)
	{
		if( $parameterStr !== '')
		{
			$parameterStr .= '&';
		}
		$parameterStr .= $key . '=' . $value;
	}

	$enc_parameters = rawurlencode($parameterStr);
	$enc_url = rawurlencode($url);
	
	$full_encoded = $method . '&' . $enc_url . '&' . $enc_parameters . '&' . $mitelsecret;
	
	$signature = md5($full_encoded);

	$signedUrl = $url . '?';
	
	foreach($parameters as $key => $value)
	{
		$signedUrl .= $key . '=' . $value . '&';
	}

	$signedUrl .= 'noauth_signature=' . $signature;
	return $signedUrl;
}

function makeRequest($url)
{
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_HTTPHEADER,array('Accept: application/xml')); 
	curl_setopt($ch, CURLOPT_FAILONERROR, 1);
	curl_setopt($ch, CURLOPT_TIMEOUT, 5);

	$output = curl_exec($ch);

	if($output === false)
	{
		$urlApiPart = $url;
		$qpos = strpos($url, '?');
		if($qpos !== false)
		{
			$urlApiPart = substr($urlApiPart, 0, $qpos);
		}

		if(curl_errno($ch) === 22)
		{
				$info = curl_getinfo($ch);
				
				if($info !== false)
				{
					$responseCode = $info['http_code'];
					$errMsg = "HTTP error response code " . $responseCode . " returned when making API request " . $urlApiPart . ": ";
					
					switch($responseCode)
					{
						case 400:
							$errMsg .= "Invalid URI, HTTP header or message body";
							break;
						case 401:
							$errMsg .= "Missing authorization credentials or not authorised access the requested resource";
							break;
						case 403:
							$errMsg .= "Current authorization context does not allow the request";
							break;
						case 404:
							$errMsg .= "No resource found that matches the request URI";
							break;
						case 405:
							$errMsg .= "The method specified in the request is not allowed for resource identified in by the request";
							break;
						case 409:
							$errMsg .= "The request could not be completed due to conflict with the current state of the resource";
							break;
						case 500:
							$errMsg .= "Internal error";
							break;
						case 501:
							$errMsg .= "Request not implemented by provider";
							break;
						default:
							$errMsg .= "Other error";
					}

					lg($errMsg);	
				}
				else
				{
					lg("An error occured when making the API request " . $urlApiPart . ": " . curl_error($ch));
				}
		}
		else
		{
			lg("An error occured when making the API request " . $urlApiPart . ": " . curl_error($ch));
		}
		return false;
	}

	curl_close($ch);

	return $output;
}


class MitelImporter
{
	var $dbintf;
	var $organizations = array();

	function __construct($dbintf)
	{
		$this->dbintf = $dbintf;
	}

	function fetchData()
	{

		lg("Fetching the data");

		if(!$this->fetchOrganizations())
		{
			return false;
		}

		foreach($this->organizations as $name => $organization)
		{
			$organization->fetchUsers();
		}

		return true;
		
	}

	function fetchOrganizations()
	{
		$url = 'api/admin/management/organizations';
		$signedUrl = getSignedUrl('GET', $url);
		$organizationsXml = makeRequest($signedUrl);

		if($organizationsXml === false)	return false;

		$parsedOrganizations = simplexml_load_string($organizationsXml);

		if($parsedOrganizations === false)
		{
			logXmlErrors(explode("\n", $organizationsXml));
			return false;
		}

		foreach($parsedOrganizations->organization as $organization)
		{
			$billingUrl = 'api/admin/user/' . $organization->domain . '/billing';
			$signedBillingUrl = getSignedUrl('GET', $billingUrl);
			$billingXml = makeRequest($signedBillingUrl);

			if($billingXml === false) continue;

			$parsedBilling = simplexml_load_string($billingXml);

			if($parsedBilling === false)
			{
				logXmlErrors(explode("\n", $billingXml));
				continue;
			}

			foreach($parsedBilling->billingInfo as $billingInfo)
			{
				$pattern = '/LIC(\s)*=(\s)*REC/i';
				if(preg_match($pattern, (string) $billingInfo) === 1)
				{
					$newOrg = new Organization((string)$organization->name, (string)$organization->domain);
					$this->organizations[(string)$organization->name] = $newOrg;
					break;
				}
				else
				{
					lg('No LIC=REC in billing user for ' . $organization->domain);
				}
			}
		}

		return true;
	}

	function dumpData()
	{
		foreach($this->organizations as $name => $organization)
		{
			echo "Organization name: " . $name . "\n";
			echo "\tDomain name: " . $organization->domain . "\n";

			echo "\tAdmins: " . "\n";
			foreach($organization->admins as $admin)
			{
				echo "\t\tUserId: " . $admin->userId . "\n";
				echo "\t\t\tFull name: " . $admin->firstName . " " . $admin->lastName . "\n";
				if($admin->primaryNumber != "")
				{
					echo "\t\t\tPrimary number: " . $admin->primaryNumber . "\n";
				}
				if($admin->secondaryNumber != "")
				{
					echo "\t\t\tSecondary number: " . $admin->secondaryNumber . "\n";
				}
			}
			echo "\tUsers: " . "\n";
			foreach($organization->users as $user)
			{
				echo "\t\tUserId: " . $user->userId . "\n";
				echo "\t\t\tFull name: " . $user->firstName . " " . $user->lastName . "\n";
				if($user->primaryNumber != "")
				{
					echo "\t\t\tPrimary number: " . $user->primaryNumber . "\n";
				}
				if($user->secondaryNumber != "")
				{
					echo "\t\t\tPrimary number: " . $user->secondaryNumber . "\n";
				}
			}
		}
	}

	function importData($super)
	{
		lg("Data has been fetched, doing the import");
		foreach($this->organizations as $name => $organization)
		{	
			lg("Importing organization " . $organization->domain);
			$this->dbintf->StartTransaction();
			$tenantaccid = $this->createOrFindTenant($organization, $super);

			if($tenantaccid === false)
			{
				$this->dbintf->CancelTransaction();	
				addToSay(MSG_ERR, "Mitel import: there were errors when importing organization " .  $organization->domain);
				continue;
			}

			$adminaccid = $this->createOrFindGroup($organization->domain . ' Admin', $tenantaccid, $setAdminPermissions);

			if($adminaccid === false)
			{
				$this->dbintf->CancelTransaction();
				addToSay(MSG_ERR, "Mitel import: there were errors when importing organization " .  $organization->domain);
				continue;
			}

			$useraccid = $this->createOrFindGroup($organization->domain . ' Users', $adminaccid, $setUserPermissions);

			if($useraccid === false)
			{
				$this->dbintf->CancelTransaction();	
				addToSay(MSG_ERR, "Mitel import: there were errors when importing organization " .  $organization->domain);
				continue;
			}

			if($setAdminPermissions)
			{
				$res = $this->setAdminPermissions($adminaccid, $useraccid);

				if($res === false)
				{
					$this->dbintf->CancelTransaction();	
					addToSay(MSG_ERR, "Mitel import: there were errors when importing organization " .  $organization->domain);
					continue;
				}
			}
			if($setUserPermissions)
			{
				$res = $this->setUserPermissions($useraccid);

				if($res === false)
				{
					$this->dbintf->CancelTransaction();	
					addToSay(MSG_ERR, "Mitel import: there were errors when importing organization " .  $organization->domain);
					continue;
				}
			}

			foreach($organization->admins as $admin)
			{
				$res = $this->createOrUpdateUser($adminaccid, $organization->domain . ' Admin', $admin, $tenantaccid);

				if($res === false)
				{
					$this->dbintf->CancelTransaction();	
					addToSay(MSG_ERR, "Mitel import: there were errors when importing organization " .  $organization->domain);
					break;
				}
			}

			if($res === false) continue;

			foreach($organization->users as $user)
			{
				$res = $this->createOrUpdateUser($useraccid, $organization->domain . ' Users', $user, $tenantaccid);
				if($res === false)
				{
					$this->dbintf->CancelTransaction();	
					addToSay(MSG_ERR, "Mitel import: there were errors when importing organization " .  $organization->domain);
					break;
				}
			}

			if($res === false) continue;

			$this->dbintf->CommitTransaction();
			
		}
	}

	function createOrFindTenant($organization, $super)
	{
		$existingGroups = $this->dbintf->GetGroupInfoOnName($organization->domain);
		if(sizeof($existingGroups) == 0)
		{
			// Create new group
			lg($organization->domain . " does not exist yet, create it ");

			$grp = array();
			$grp['name'] = $organization->domain;
			$grp['is_tenant'] = 1;
			$grp['typegrp'] = 0;
			$grp['super'] = $super;

			$isok = $this->dbintf->AddGroup($accid, $grp, true);
			if($isok === false)
			{
				lg("Error when creating tenant " . $organization->domain . ": " . $this->dbintf->lasterr);
				return false;
			}
		}
		else
		{
			$found = false;
			foreach($existingGroups as $group)
			{
				if($group['is_tenant'] == 1)
				{
					lg("Found existing tenant with name " . $organization->domain);

					if($group['super'] != $super)
					{
						lg("Found tenant ". $organization->domain . " not a subgroup of mitel supergroup, changing this");

						$obj = array();
						$obj['super'] = $super;
						$isok = $this->dbintf->EditGroup($group['id'], $obj);

						if($isok === false)
						{
							lg("Error when updating supergroup of existing tenant " . $organization->domain . ": " . $this->dbintf->lasterr);
							return false;
						}
					}

					$accid = $group['accid'];
					$found = true;
					break;
				}
			}

			if(!$found)
			{
				lg("Only normal group, but no tenant with name " . $organization->domain . ", create it");

				$grp = array();
				$grp['name'] = $organization->domain;
				$grp['is_tenant'] = 1;
				$grp['typegrp'] = 0;
				$grp['super'] = $super;

				$isok = $this->dbintf->AddGroup($groupid, $grp, true);
				if($isok === false)
				{
					lg("Error when creating tenant " . $organization->domain . ": " . $this->dbintf->lasterr);
					return false;
				}
			}
		}

		return $accid;
	}

	function createOrFindGroup($name, $super, &$setPermissions)
	{
		//$existingGroups = $this->dbintf->GetDirectSubgroupsOnName($super, $name);
		$this->dbintf->DetermineAllGroups(true);
		$subgroups = array();
		$this->dbintf->FindSubGroups($subgroups, $super);

		foreach($subgroups as $subgroup)
		{
			$existingGroup = $this->dbintf->GetGroupInfoOnAccid($subgroup);

			if(strtolower($existingGroup['name']) == strtolower($name))
			{
				lg( $name . " already exists");
				$setPermissions = false;
				return $existingGroup['accid'];
			}
		}
		
		lg($name . " does not exist yet. Create with default permissions");
		$setPermissions = true;
		$grp['name'] = $name;
		$grp['is_tenant'] = 0;
		$grp['typegrp'] = 0;
		$grp['super'] = $super;

		$isok = $this->dbintf->AddGroup($accid, $grp, true);
		if($isok === false)
		{
			lg("Error when creating group " . $name . ": " . $this->dbintf->lasterr);
			return false;
		}

		return $accid;	
	}

	function setAdminPermissions($adminaccid, $useraccid)
	{
		$accpms = array();
		$adminpm = 0 | PM_LIST | PM_DOWNLOAD | PM_PLAY | PM_DELETE | PM_NOTES;
		
		$self = array();
		// For itself
		$self['callaccid'] = -1;
		// This group only
		$self['applytype'] = 0;
		$self['pm'] = $adminpm;
		
		$accpms[] = $self;

		$users = array();
		// For users group
		$users['callaccid'] = $useraccid;
		// This group only
		$users['applytype'] = 0;
		$users['pm'] = $adminpm;

		$accpms[] = $users;

		$isok = $this->dbintf->SetAccAccPms($adminaccid,$accpms);
    if ($isok===false)
    {
      lg("Error when setting admin group permissions: " . $this->dbintf->lasterr);
			return false;
    }

		return true;
	}

	function setUserPermissions($useraccid)
	{
		$accpms = array();
		$userpm = 0 | PM_LIST | PM_PLAY | PM_NOTES;
		
		$self = array();
		// For itself
		$self['callaccid'] = -1;
		// This group only
		$self['applytype'] = 0;
		$self['pm'] = $userpm;
		
		$accpms[] = $self;

		$isok = $this->dbintf->SetAccAccPms($useraccid,$accpms);
    if ($isok===false)
    {
			 lg("Error when setting user group permissions: " . $this->dbintf->lasterr);
			 return false;
    }

		return true;
	}

	function createOrUpdateUser($grpaccid, $grpname, $user, $tenantaccid)
	{
		// Find username from primary number
		if($user->primaryNumber != '')
		{
			if(strlen($user->primaryNumber) < 4 || substr($user->primaryNumber, 0, 4) !== '+358')
			{
				lg("Unknown country code for user " . $user->userId . ": " . $user->primaryNumber);
				return false;
			}
			else
			{
				$userName = 'fi0' . substr($user->primaryNumber, 4);
			}
		}
		else
		{
			// Primary number should always be there
			lg("No primary number for user " . $user->userId);
			return false;
		}
		
		$existingUser = $this->dbintf->GetUserInfoOnUsername($userName);
		if($existingUser === false)
		{
			lg("Creating new user " . $userName . " for group " . $grpname);
			$usrData = array();

			$usrData['name'] = $user->firstName . " " . $user->lastName;
			$usrData['usr'] = $userName;
			$usrData['enabled'] = 1;
			$usrData['email'] = $user->email;

			$isok = $this->dbintf->AddUserAccount($userid,$usrData,$grpaccid);
      if ($isok===false)
      {
				lg("Error when creating user account " . $userName  . " for group " . $grpname . " : " . $this->dbintf->lasterr);
			 	return false;
    	}

			$usrData = $this->dbintf->GetUserInfoOnUsername($userName);

			$isok = $this->dbintf->ChangePassword($usrData['id'], substr($userName, 2), false);
			if ($isok===false)
      {
				lg("Error when setting password for new user account " . $userName  . " for group " . $grpname . " : " . $this->dbintf->lasterr);
			 	return false;
    	}
		}
		else
		{
			lg("User " . $userName . " already exists");
			
			$userGroups = $this->dbintf->GetGroupsOfWhichMember($existingUser['accid']);
			$userSuperGroups = $this->dbintf->GetSuperGroups($userGroups);
			$allUserGroups = array_unique(array_merge($userGroups, $userSuperGroups));
			
			$userTenants = array();
			
			foreach($allUserGroups as $userGroup)
			{
				$groupInfo = $this->dbintf->GetGroupInfoOnAccid($userGroup);
				if($groupInfo['is_tenant'] === 1)
				{
					$userTenants[] = $userGroup;
				}
			}
			
			if(in_array($grpaccid, $userGroups) === false && in_array($tenantaccid, $userTenants) == false)
			{
				lg("Adding existing user " . $userName . " to group " . $grpname);
				$userGroups [] = $grpaccid;
				$isok = $this->dbintf->SetGroupsOfWhichMember($existingUser['accid'],$userGroups);
        if ($isok===false)
        {
					lg("Error when adding existing user " . $userName ." to group " . $grpname  . ": " . $this->dbintf->lasterr);
			 		return false;
        }
			}

			$usrData = $existingUser;
		}
	
		lg("Updating phones for user " . $usrData['usr'] . " for group " . $grpname);
		$numbers = array();

		if($user->primaryNumber != '')
		{
			if(strlen($user->primaryNumber) < 4 || substr($user->primaryNumber, 0, 4) !== '+358')
			{
				lg("Unknown country code in primary number for user " . $user->userId . ": " . $user->primaryNumber);
				return false;
			}
			else
			{
				$primaryNumber = substr($user->primaryNumber, 4);

				$numbers[] = '+358' . $primaryNumber;
				$numbers[] = '0' . $primaryNumber;

				if($user->primaryNumberType == "OFFICE_FIXED")
				{
					$numbers[] = '35BB0' . $primaryNumber;
				}
				else
				{
					$numbers[] = '4D0010' . $primaryNumber;
					$numbers[] = '4D1010' . $primaryNumber;
					$numbers[] = '4D2010' . $primaryNumber;
          $numbers[] = '4D3110' . $primaryNumber;
          $numbers[] = '4D3010' . $primaryNumber;
				}
			}
		}

		if($user->secondaryNumber != '')
		{
			if(strlen($user->secondaryNumber) < 4 || substr($user->secondaryNumber, 0, 4) !== '+358')
			{
				lg("Unknown country code in secondary number for user " . $user->userId . ": " . $user->secondaryNumber);
				return false;
			}
			else
			{
				$secondaryNumber = substr($user->secondaryNumber, 4);
				$numbers[] = '+358' . $secondaryNumber;
				$numbers[] = '0' . $secondaryNumber;
				$numbers[] = '35BB0' . $secondaryNumber;	
			}
		}

		$existingPhones = $this->dbintf->GetUserPhones($usrData['accid']);

		$phones = array();
		foreach($numbers as $number)
		{
			$phone = $this->dbintf->GetOrCreateCallerID($number, '', 2);

			// Check if already an existing user phone
			$pos = array_search($phone['contactid'], $existingPhones);
			if($pos !== false)
			{
				unset($existingPhones[$pos]);
			}

			$phones[] = $phone;
		}

		// Now contains all contact ids that no longer exist in imported data
		// These should not be deleted.
		$existingPhones = array_values($existingPhones);

		foreach($existingPhones as $phone)
		{
			$existingPhone = array();
			$existingPhone['contactid'] = $phone;
			$phones[] = $existingPhone;
		}	
	
		$isok = $this->dbintf->SetUserPhones($usrData['accid'],$phones);
		if ($isok===false)
		{
			lg("Error when setting phones for user " . $userName  . " for group " . $grpname . " : " . $this->dbintf->lasterr);
			return false;
		}

		return true;
	}
};

class Organization
{
	var $name = '';
	var $domain = '';
	var $users = array();
	var $admins = array();

	function __construct($name, $domain)
	{
		$this->name = $name;
		$this->domain = $domain;
	}

	function fetchUsers()
	{
		$admUrl = 'api/admin/organization/' . $this->domain . '/userGroups/More-a/users/';
		$admSignedUrl = getSignedUrl('GET', $admUrl);
		$admXml = makeRequest($admSignedUrl);
		if($admXml !== false)
		{
			$admParsedXml = simplexml_load_string($admXml);
			if($admParsedXml !== false)
			{
				foreach($admParsedXml->user as $admin)
				{
					$newAdmin = new User();
					if($newAdmin->fetchUser((string)$admin->userId, $this->domain))
					{
						$this->admins[$newAdmin->userId] = $newAdmin;
					}
				}
			}
		}


		$usrUrl = 'api/admin/organization/' . $this->domain . '/userGroups/More-u/users/';
		$usrSignedUrl = getSignedUrl('GET', $usrUrl);
		$usrXml = makeRequest($usrSignedUrl);
		if($usrXml !== false)
		{
			$usrParsedXml = simplexml_load_string($usrXml);
			if($usrParsedXml !== false)
			{
				foreach($usrParsedXml->user as $user)
				{
					$newUser = new User();
					if($newUser->fetchUser((string)$user->userId,  $this->domain))
					{
						if(!isset($admins[$newUser->usedId]))
						{
							$this->users[$newUser->userId] = $newUser;
						}
					}
				}
			}
		}
	}
};

class User
{
	var $firstName = '';
	var $lastName = '';
	var $userId = '';
	var $primaryNumber = '';
	var $primaryNumberType = '';
	var $secondaryNumer = '';
	var $secondaryNumerType = '';
	var $email = '';

	function fetchUser($userid, $domain)
	{
		$this->userId = $userid;
		$userUrl = 'api/admin/user/' . $domain . '/' . $userid;
		$userUrlSigned = getSignedUrl('GET', $userUrl);
		$userXml = makeRequest($userUrlSigned);

		$userParsedXml = simplexml_load_string($userXml);

		if($userParsedXml !== false)
		{
			$this->firstName = (string) $userParsedXml->name->givenName;
			$this->lastName = (string) $userParsedXml->name->familyName;
			$this->email = (string) $userParsedXml->email;

			foreach($userParsedXml->phoneNumbers->phone as $phone)
			{
				if((string) $phone->type == "PRIMARY")
				{
					$this->primaryNumber = (string)$phone->value;
					$this->primaryNumberType = (string)$phone->numberType;
				}
				else if((string) $phone->type == "SECONDARY")
				{
					$this->secondaryNumber = (string)$phone->value;
					$this->secondaryNumerType = (string)$phone->numberType;
				}
			}

			return true;
		}
		
		logXmlErrors(explode("\n",$userXml));
		return false;
	}
	
};

?>