
class AudioPlayer
{

	constructor()
	{
		this.ac = null;
	}

	init(codec,channelCount)
	{
		if (codec=='') return;
		this.codec = codec;
		this.channelCount = channelCount;
		this.blockSize = 8;
		this.bytesPerSample = this.channelCount;
		if (this.codec = "alaw")
		{
			this.decoder = alawmulaw.alaw;
		}
		else if (this.codec = "ulaw")
		{
			this.decoder = alawmulaw.mulaw;
		}
		else ; // not supported
		this.sampleRate = 8000;
		this.buffer = new Uint8Array();
		this.ac = new AudioContext( {sampleRate:this.sampleRate} );
		this.playpos = 0;
		this.startTime = 0;
	}
	
	add(data)
	{
		// lg('Adding ' + data.byteLength );
		let availSize = this.buffer.byteLength + data.byteLength;
		let remainder = availSize % this.blockSize;
		let codedSize = availSize - remainder;
		
		// combine this.buffer and data into a new sourceBuf
		let sourceBuf = null;
		if ( this.buffer.byteLength > 0 )
		{
			sourceBuf = new Uint8Array( codedSize );
			if ( this.buffer.byteLength > codedSize )
			{
				arr.set( this.buffer.slice(0,codedSize) );
			} else
			{
				arr.set( this.buffer );
			}
			var takeOf2 = codedSize - this.buffer.byteLength;
			if (takeOf2>0)
			{
				sourceBuf.set( data, this.buffer.byteLength );
			}
		} else
		{
			sourceBuf = data;
		}
		
		if (codedSize>0)
		{
			let source = this.ac.createBufferSource();
			let sampleCount = codedSize / this.bytesPerSample;
			let buf = this.ac.createBuffer(this.channelCount, sampleCount, this.sampleRate);
			
			for (let chIx=0; chIx<this.channelCount; chIx++)
			{
				var chan = buf.getChannelData( chIx );
				// chan is an array of floats
				for (var j=0;j<sampleCount;j++)
				{
					//var sval = Math.sin( (this.playpos + j)*0.2 ) * 1;
					let sval = this.decoder.decodeSample( sourceBuf[j*this.channelCount+chIx] );
					chan[j] = sval / 32768.0;
				}
			}
			source.buffer = buf;
			source.connect(this.ac.destination);
			// schedule the playback for a specific time in seconds
			if (this.startTime == 0)
			{
				this.startTime = this.ac.currentTime + 0.5;
			}
			let when = this.startTime + (this.playpos / this.sampleRate);
			let ahead = (when - this.ac.currentTime);
			lg("when = " + when.toFixed(2) + " ahead " + (when - this.ac.currentTime).toFixed(2) );
			if (ahead<0.1)
			{
				let adjust = 0.5 - ahead;
				lg("ERROR adjust with " + adjust.toFixed(2) );
				this.startTime += adjust;
				when += adjust;
			}
			source.start( when );
			this.playpos += sampleCount;
			
		}
	}
	
	stop()
	{
		if (this.ac!=null)
		{
			this.ac.close();
		}
	}
	
}

class RealTimePlayer
{

	constructor()
	{
		this.offset = 0;
		this.restarting = false;
		this.onError = null;
		this.onStopped = null;
		this.init();
	}
	
	init()
	{
		this.params = [];
		this.player = new AudioPlayer();
		this.shutdown = false;
		this.started = false;
		this.error = null;
		this.reader = null;
		// do not set remid or offset
	}

	setId(id)
	{
		this.remid = id;
	}
	
	handleInitBlock()
	{
		var initBlock = new Uint8Array( this.chunksSize );
		var initWritten = 0;
		for (let j=0;j<this.chunks.length;j++)
		{
			var chunk = this.chunks[j];
			const len = chunk.length;
			for (let i=0;i<len;i++)
			{
				initBlock[ initWritten + i ] = chunk[i];
			}
			initWritten += len;
		}
		let initStr = new TextDecoder('utf-8').decode( initBlock );
		return this.handleInitMessage( initStr );
	}
	
	handleInitMessage( initStr )
	{
		lg( 'meta message = ' + initStr );
		var parts = initStr.split("\r\n");
		for (let ix in parts)
		{
			var part = parts[ix];
			var colonPos = part.indexOf(':');
			if (colonPos==-1) continue;
			var key = part.substr(0,colonPos);
			var value = part.substr(colonPos+2);
			this.params[key] = value;
		}
		if ('Error' in this.params)
		{
			this.error = this.params['Error'];
			if (this.onError!=null)
			{
				this.onError( this.error );
			}
			return false;
		}
		this.player.init( this.params['Format'] , this.params['Channels'] );
		return true;
	}
	
	back(seconds)
	{
		this.offset -= seconds;
		this.onNewOffset();
	}
	
	forward(seconds)
	{
		var newOffset = this.offset + seconds;
		if (newOffset>0) newOffset = 0;
		if (this.offset != newOffset)
		{
			this.offset = newOffset
			this.onNewOffset();
		}
	}

	async restart()
	{
		if (this.restarting)
		{
			lg('already restarting');
			return;
		}
		this.restarting = true;
		lg('restart - stop existing');
		try
		{
			if (this.player!=null) this.player.stop();
			if (this.reader!=null) this.reader.cancel();
			this.shutdown = true;
			lg('reader cancelled');
			while (this.inStart)
			{
				lg('inStart');
				await this.sleep(500);
			}
			lg('restart with new init');
			this.init();
			this.start();
		} catch (e)
		{
			lg('Error restarting: ' + e);
		}
		lg('end of restarting');
		this.restarting = false;
	}
	
	async onNewOffset()
	{
		lg('onNewOffset ' + this.offset);
		this.restart();
	}

	sleep(ms) {
		return new Promise(resolve => setTimeout(resolve, ms));
	}
	
	stop()
	{
		this.player.stop();
		this.shutdown = true;
	}
	
	async start()
	{
		this.inStart = true;
		var url = '/client.php'
		+ '?info=RTCall'
		+ '&webinternal=1'
		+ '&remid=' + this.remid
		+ '&offset=' + this.offset
		;
		var response = await fetch( url );
		this.reader = response.body.getReader();
		
		this.chunks = [];
		this.chunksSize = 0;
		var initDone = false;
		
		while (this.shutdown==false)
		{
			let { value, done } = await this.reader.read();
			if (done)
			{
				lg('done');
				break;
			}
			lg('received ' + value.byteLength + ' bytes');
			if (initDone==false)
			{
				// check if we received enough to do init
				// we must have a null byte
				var nulPos = value.indexOf(0);
				if (nulPos != -1)
				{
					// we have a null byte,
					// we add the portion before the null byte
					this.chunks.push( value.slice(0,nulPos) );
					this.chunksSize += nulPos;
					// we should be able to do init now
					if (this.handleInitBlock()==false)
					{
						lg('init failed, error = ' + this.error);
						this.reader.cancel();
						this.inStart = false;
						return;
					}
					initDone = true;
					this.started = true;
					
					this.chunks = [];
					this.chunksSize = 0;
					// we probably have some extra in buffer
					value = value.slice(nulPos+1);
					// or maybe not
					if (value.byteLength==0) continue;
				} else
				{
					this.chunks.push( value );
					this.chunksSize += value.byteLength;
				}
			} 
			
			if (initDone)
			{
				this.player.add( value );
			}
			
		}		
		if (!initDone)
		{
			lg('error - no info block received');
		}
		this.inStart = false;
		if (this.restarting==false && this.onStopped!=null)
		{
			this.onStopped();
		}
	}
}
