<?php

require_once('general.php');

HandleContacts($dbintf);

function HandleContacts($dbintf)
{
  global $rpl,$configs,$msg,$lng,$rooturl;
  if (($dbintf->actuinfo['permissions']&(PM_CONFIG|PM_FILTER))==0)
  {
    echo 'Permission denied';
    exit;
  }

  $active_cnt = $dbintf->GetActiveCallsCount();

  $configs = array_merge( $configs, $dbintf->LoadConfigsPart('o') );

	$cfgkeys = array(
			'RecList','RecIPs', // filter on IP
			'SipFilterType','SipFilters','SipFilterTypeIn','SipFiltersIn','SipFilterTypeOut','SipFiltersOut', // filter on SIP numbers
			'H323FilterType','H323Filters', // H.323 numbers
			'UnimegaFilterType','UnimegaFilters', // other numbers
			'NumberFilterType','NumberFilters', // PRI numbers
			'DuplicateList',
			'CSTATelIPType','CSTATelIPFilter',	//CSTA telephone/IP pairs
			'CSTAVDNTelType','CSTAVDNTelFilter',	//VDN numbers
      'CSTACalledType','CSTACalledFilter',	//VDN numbers
      'RecOnDmdType','RecOnDmdList', // RecOnDmd is sort of filter
			'DelOnDmdType', 'DelOnDmdList',
			'SilOnDemand', 'SilOnDmdList',
      'SideToRecord', 'RemSideOnDmdType', 'RemSideOnDmdList',
      'RecNotInc','RecNotOutg','RecLocal', // also a filter
			'NotRecEmptyNum', // a filter (TDM/Analog)
			'RecMinDur','CardNotInternal','DuplicateType',
      'AudioDetect','AudioDetectThreshold','AudioDetectTimeout','AudioDetectList'
			);
  if (($dbintf->actuinfo['permissions']&(PM_CONFIG))!=0)
  {
		$cfgkeys = array_merge($cfgkeys
			, array('RecEnc','RecStereo'
				,'RecOnDemand' // store on demand
        ,'StoreOnDemandDelay'
				,'Monitoring',
        'Mp3Quality', 'Mp3Mode', 'Mp3Brate', 'Mp3VbrQuality'
			));
	}

  $act = $_POST['act'];
  if ($act!='')
  {
    if ($act=='change-settings')
    {
			if ($dbintf->nodePassive)
			{
        $msg[] = new Msg(MSG_ERR,'Settings can only be changed on the primary node');
				// redirect
				$_SESSION['msg'] = $msg;
				header('Location: '.$rooturl.'recopt.php');
				exit;
			}
			LogAction('Change recording setting');
      $newconfigs = array();
      foreach ($cfgkeys as $cfgkey)
      {
        if (isset($_POST[$cfgkey]))
        {
          $newconfigs[$cfgkey] = $_POST[$cfgkey];
        }
      }

      // first save user settings
      $dbintf->SetConfigsPart($newconfigs,'o');
      // calculate automatic settings (but no engines reconfig)
      EnginesSettingsCalcApply(false,false);
			
      msg_signal_service($msg,'os');
      $msg[] = new Msg(MSG_INF,$lng->Get('Options.ConfigSaved'));

      $needrestart = $_POST['needrestart'];
			$needreconfig = $_POST['needreconfig'];
      $restart = str_ensure_defined($_POST['restart'])=='1';

      if ($restart)
      {
        if ($active_cnt>0 && $_POST['restart_force']==false)
        {
          $msg[] = new Msg(MSG_ERR,$lng->Get('System.CallsActive').' '.$lng->Get('System.RepeatToForceRestart'));
          $restart = false;
        } else
        {
          if (msg_signal_service($msg,'or'))
          {
            $t = $lng->Get('System.RestartComponentBusy');
            $t = str_replace('%1',$lng->Get('System.TheRecComp'),$t);
            $t = ucfirst($t);
            $msg[] = new Msg(MSG_INF,$t);
          } else {
            $msg[] = new Msg(MSG_ERR,'Failed to restart recording service');
            $restart = false;
          }
        }
      }
      if ($restart==false)
      {
				if ($needreconfig)
				{
					EnginesReconfig($msg,$dbintf);
				}
        if ($needrestart)
        {
          // not restarting, but changed
          $msg[] = new Msg(MSG_WRN,$lng->Get('System.ChangesAfterRestart'));
					// When next expression is true, then:
					// something has changed, but not yet restarted,
					// so the settings are different from actual.
					$_SESSION['recopt_needrestart'] = true;
        } 
      }
			if ($needrestart || $needreconfig)
			{
				clusterConfigNotify($needrestart);
			}
      // redirect
      $_SESSION['msg'] = $msg;
      header('Location: '.$rooturl.'recopt.php');
      exit;
    }
  }

  //var_dump($configs);
  //exit;
  
  // set some defaults
  if (isset($configs['SideToRecord'])==false)
  {
    $configs['SideToRecord'] = 'both';
  }

  if (isset($configs['Mp3Quality']) == false)
  {
    $configs['Mp3Quality'] = '5';
  }

  if (isset($configs['Mp3Mode']) == false)
  {
    $configs['Mp3Mode'] = 'CBR';
  }

  if (isset($configs['Mp3Brate']) == false)
  {
    $configs['Mp3Brate'] = '16';
  }

  if (isset($configs['Mp3VbrQuality']) == false)
  {
    $configs['Mp3VbrQuality'] = '20';
  }

  if (isset($configs['Mp3VbrMode']) == false)
  {
    $configs['Mp3VbrMode'] = 'standard';
  }

  $data = "var active_cnt = $active_cnt; \r\n";
	$canmp3 = file_exists('/usr/lib/sox/libsox_fmt_mp3.so')
         || file_exists('/usr/lib/x86_64-linux-gnu/sox/libsox_fmt_mp3.so');
	if (defined('DEMO')) $canmp3 = true;
	$data .= "var canmp3 = ".(int)$canmp3."; \r\n";

  $data .= "var configs = []; \r\n";
  foreach ($cfgkeys as $cfgkey)
  {
    $data .= "configs[\"".$cfgkey."\"] = \"".JSQuote(str_ensure_defined($configs[$cfgkey]))."\";\r\n";
  }
  $data .= 'var ChCnt   =  ' . (int)$configs['ChCnt'  ] .   ";\r\n";
	$data .= 'var CSTAType = ' . int_ensure_defined($configs['CSTAType']) . ";\r\n";

	if ($configs['RecEnc']=='mp3' && $canmp3==false)
	{
		$msg[] = new Msg(MSG_WRN,'MP3 encoding is not installed. G.711 will be used instead.');
	}

	LogAction('Viewed recording setting',array('level'=>'view'));

  // ready with database
  $dbintf->close();

  $needrestart = 0;
  if (isset($_SESSION['recopt_needrestart']))
  {
    $needrestart = (int) $_SESSION['recopt_needrestart'];
    unset($_SESSION['recopt_needrestart']);
  }
	
  $data .= GeneralJSData();
  $data .= 'var still_needrestart = ' . $needrestart . ";\r\n";


  $template_content = file_get_contents('../apresa/templ/recopt.htm');

  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}


// XML helper functions:

function GetSubNode($node,$arr,$ix=0)
{
  foreach ($node->childNodes as $item)
  {
    if ($item->nodeName==$arr[$ix])
    {
      if (sizeof($arr)==$ix+1)
      {
        return $item;
      } else {
        return GetSubNode($item,$arr,$ix+1);
      }
    }
  }
  return null;
}

function GetNodeValue($node,$arr)
{
  return GetSubNode($node,$arr)->nodeValue;
}

function SetNodeValue($node,$arr,$value,&$changed)
{
  $subnode = GetSubNode($node,$arr);
  if ($subnode->nodeValue != $value)
  {
    $subnode->nodeValue = $value;
    $changed = true;
  }
}


?>
