<?php

require_once('emailing.php');
require_once('tenancy.php');

class ReportOutput
{
	var $dest = '';
	var $xout = '';
	var $f = null;
	
	function out($t)
	{
		if ($this->dest=='echo')
		{
			echo $t;
		} elseif ($this->dest=='file')
		{
			fwrite($this->f,$t);
		} elseif ($this->dest=='buffer')
		{
			$this->xout .= $t;
		}
	}
	
	function close()
	{
		if ($this->dest=='file')
		{
			fclose($this->f);
			$this->f = null;
		}
	}
}

function PerformReportSchedule($report,$force,$act,$reqFormat='default')
{
  global $dbintf,$configs,$lng;
  
  lg('Performing report schedule ' . $report['name'] );
  
  $configs_n = $dbintf->LoadConfigsPart('n');
  $period = $report['period'];

  $serial = (int) file_get_contents( dirname(__FILE__) . '/../apresa/conf/serial' );
	
	$output = $report['output'];
	// default output is email
	if ($output=='') $output = 'email';
	
	if ($output == 'email')
	{
		$toTmp = array();
		if ($report['email']=='')
		{
			// this is not reachable through the GUI
			// because currently filling in nothing yields an error
			$toTmp[] = $configs_n['EmAdmin'];
			$toTmp[] = $configs_n['EmAdmin2'];
		} else
		{
			$items = explode(',',$report['email']);
			foreach ($items as $item)
			{
				$item = trim($item);
				$toTmp[] = $item;
			}
		}
		$to = array();
		foreach ($toTmp as $addr)
		{
			if ($addr!='')
			{
				$to[] = $addr;
			}
		}
		if (count($to)==0)
		{
			lg('The email destination address is not defined');
			return;
		}
	}
  
  // we can only report about time periods that are in the past
  
  // the situation where the report has never been run yet,
  // or the previous time, there was another type of period run.
  
  $begin = 0;
  $end = 0;

  $curDay = date( "j");
  $curMonth = date( "n");
  $curYear = date( "Y");
  
  if ($period=='day')
  {
    $begin = strtotime($curYear."-".$curMonth."-".$curDay. " 00:00 -1 days");
    $end   = strtotime($curYear."-".$curMonth."-".$curDay. " 00:00") - 1;
  }
  else if ($period=='week')
  {
    $curWeekday = (int) date( "w", time());
    $begin = strtotime($curYear."-".$curMonth."-".$curDay. " 00:00 -".$curWeekday." days, -1 week");
    $end   = strtotime($curYear."-".$curMonth."-".$curDay. " 00:00 -".$curWeekday." days") - 1;
  }
  else if ($period=='month')
  {
    $curWeekday = (int) date( "w", time());
    $begin = strtotime($curYear."-".$curMonth."-1 ". " 00:00 -1 months");
    $end   = strtotime($curYear."-".$curMonth."-1 ". " 00:00") - 1;
  }
  
  $timePeriodStr = date($configs['datef'],$begin) . ' - ' . date($configs['datef'],$end);
  
  $sysname = $dbintf->GetConfigValue('SystemName');
  if ($sysname=='')
  {
    $sysname = $dbintf->GetConfigValue('IPname');
  }
  if ($sysname=='')
  {
    $sysname = 'Apresa';
  }
  $subject = $sysname . ' [SN '.$serial.'] ' . $report['name'] . ' ' . $timePeriodStr;
  
  lg( $subject );

  if ($force==false && $act=='run')
  {
    if ($report['period'] == $report['lastperiod']
    && $report['laststart'] == $begin)
    {
      // about this time period, we have already reported, so skip
      lg( 'We have already reported about this time period.');
      return;
    }
  }

	$periodStr = date('Y-m-d',$begin) . ' - ' . date('Y-m-d',$end);
	$fn = $report['name'] . ' ' . $periodStr;
	$fn = str_replace('/','_',$fn);
	$fn = str_replace('.','_',$fn);
	$fn = str_replace(':','_',$fn);
	$fn .= '.csv';
	
	if ($output == 'directory')
	{
		if ($act=='run')
		{
			if ($report['directory']=='')
			{
				lg('Output directory is not defined');
				return;
			}
			if (file_exists($report['directory'])==false)
			{
				lg('Output directory does not exist');
				return;
			}
			if (is_dir($report['directory'])==false)
			{
				lg('Output directory is not a directory');
				return;
			}
		}
		$outputFn = $report['directory'] . '/' . $fn;
		lg('Output file = ' . $outputFn);
	}
	
	$tenantid = -1;
	$filter = $report['filter'];
	if ($filter != null && $filter!='')
	{
		$parts = explode(',',$filter);
		foreach ($parts as $part)
		{
			$kv = explode('=',$part);
			if (count($kv)==2 && $kv[0]=='tenant')
			{
				$tenantid = (int) $kv[1];
			}
		}
	}

	$search = array();
	$search['fd'] = $begin;
	$search['ld'] = $end;
	if ($tenantid!=-1)
	{
		$search['tenantid'] = $tenantid;
	}
	$cfg = array();
	$cfg['allrecords'] = true;
	$cfg['nousrflr'] = true;

	$format = '';
	if ($output=='email') $format = 'html';
	else if ($output=='directory') $format = 'csv';

	$ro = new ReportOutput();
	if ($act=='view')
	{
		$ro->dest = 'echo';
		// in case of view allow overruling of format
		if ($reqFormat=='html') $format = 'html';
		if ($reqFormat=='csv') $format = 'csv';
		if ($format=='csv')
		{
			header('Content-type: text/csv');
			header('Content-Disposition: attachment; filename="' . $fn . '"');
		}
	} elseif ($output=='directory')
	{
		$ro->dest = 'file';
		$ro->f = fopen($outputFn,'w');
		if ($ro->f===false)
		{
			lg('Could not open ' . $outputFn);
			return;
		}
	} elseif ($output=='email')
	{
		$ro->dest = 'buffer';
	}

	if ($format=='csv')
	{
		// UTF-8 BOM
		$ro->out( "\xEF\xBB\xBF" );
	}
	
	if ($format=='html')
	{
		// generic style for all reports
		$style = '';
		$style = 'table { border-collapse: collapse; font-family: sans-serif; }' . "\n";
		$style .= 'table th { border: 1px solid #888; ' . "\n"
			. 'background-color: #036; color:#fff; ' . "\n"
			. 'text-align: left; padding: 0.4em; font-weight: normal; ' . "\n"
			. ' }' . "\n";
		$style .= 'table td { border: 1px solid #888; padding: 0.2em; }' . "\n";
		$style .= 'table tr { background-color: #fff; color:#000; }' . "\n";
		$style .= 'table tr.odd { background-color: #eee; color:#000; }' . "\n";
	}

	if ($act=='run')
	{
		// record in database what we do to prevent doing it twice
		$newdata = array();
		$newdata['lastrun'] = time();
		$newdata['lastperiod'] = $report['period'];
		$newdata['laststart'] = $begin;
		$dbintf->EditReportSchedule($report['id'],$newdata);
	}
	
	if ($report['type']=='CallsPerUser')
	{
		
		$cfg['callback'] = 'FoundCallForReport';
		$cfg['data'] = array( 'users' => array() );
		$sk = '';
		$calls = $dbintf->GetCallList($sk,'',$search,$cfg);

		$usersdata = & $cfg['data']['users'];

		$table = array();
		// GetUserList also returns users that have no enabled user account
		// see also GetUsersOfCall call below
		$users = $dbintf->GetUserList();
		foreach ($users as $user)
		{
			if ($tenantid!=-1)
			{
				if ($user['tenantid'] != $tenantid) continue;
			}
			$accid = $user['accid'];
			$row = array();
			$row[] = $user['name'];
			$cnt = int_ensure_defined( $usersdata[ $accid ] );
			$row[] = '' . $cnt;
			$table[] = $row;
		}
		if (isset($cfg['data']['nouser']))
		{
			$row = array();
			$row[] = 'No user';
			$row[] = '' . $cfg['data']['nouser'];
			$table[] = $row;
		}

		$cols = array();
		$cols[] = array( 'title' => $lng->Get('Main.Name') , 'align' => 'left' );
		$cols[] = array( 'title' => $lng->Get('Stats.numcalls') , 'align' => 'right' );

		$xout = '';
		if ($format=='html')
		{
			$htmlTab = RenderHtmlTable($cols,$table);

			$ix = 0;
			// additional style
			foreach ($cols as $col)
			{
				$ix ++;
				$style .= 'table td.c' . $ix . ' { text-align: ' . $col['align'] . '; }' . "\n";
			}
			$xout .= '<style>' . "\n";
			$xout .= $style;
			$xout .= '</style>' . "\n";
			$xout .= '<h2>' . HtmlQuote($report['name']) . ' ' . $timePeriodStr . '</h2>' . "\n";
			$xout .= $htmlTab;
			$ro->out( $xout );
		}
		if ($format=='csv')
		{
			$csvrow = new CsvRow();
			foreach ($cols as $col)
			{
				$csvrow->add($col['title']);
			}
			$ro->out( $csvrow->get() );
			foreach ($table as $row)
			{
				$csvrow->flush();
				foreach ($row as $elem)
				{
					$csvrow->add($elem);
				}
				$ro->out( $csvrow->get() );
			}
		}
	} // end report CallsPerUser
	else if ($report['type']=='CallListing')
	{
		$cfg['callback'] = 'CallListingReportCB';
		
		$cfg['data'] = array();
		$cfg['format'] = $format;
		$cfg['cnt'] = 0;
		$cfg['ro'] = $ro;
		//$cfg['output'] = 'web';
		$sk = '';
		
		$cols = [ $lng->Get('Web.Identifier')
			, $lng->Get('Cols.startdate')
			, $lng->Get('Cols.length')
			, $lng->Get('Cols.localID'), $lng->Get('Cols.lname')
			, $lng->Get('Cols.remoteID'), $lng->Get('Cols.rname')
			, $lng->Get('Cols.callerID')
			, $lng->Get('Cols.receiverID')
			, $lng->Get('Cols.direction') , $lng->Get('Cols.direction')
			, $lng->Get('Web.Tenant') , $lng->Get('PM.User')
			];
		
		$xout = '';
		if ($format=='html')
		{
			$xout .= '<style>' . "\n";
			$xout .= $style;
			$xout .= '</style>' . "\n";
			$xout .= '<h2>' . HtmlQuote($report['name']) . ' ' . $timePeriodStr . '</h2>' . "\n";
			$xout .= '<table>';
			$xout .= '<tr>';
			foreach ($cols as $elem)
			{
				$xout .= '<th>' . HtmlQuote($elem);
			}
			
		}
		if ($format=='csv')
		{
			$xout = '';
			$header = new CsvRow();
			foreach ($cols as $elem)
			{
				$header->add( $elem );
			}
			$xout .= $header->get();
		}
		
		$ro->out( $xout );
		$xout = '';

		$dbintf->GetCallList($sk,'',$search,$cfg);
		
		if ($format=='html')
		{
			$ro->out( '</table>' );
		}
	}

	$ro->close();
  
  if ($act=='view')
  {
    // debug: file_put_contents('tmp.html',$html); return;
		// output is already echoed
		exit;
  } else if ($act=='run')
  {
		if ($output=='email')
		{
			$dat = array();
			$dat['To'] = $to;
			$dat['Subject'] = $subject;
			$dat['HtmlBody'] = $ro->xout;
			
			EmailingWebOrCmd($dat);
		} else if ($output=='directory')
		{
			// file is already written
		} else
		{
			// incorrect output type
		}
  }

  lg('End of report');
  
}

function RenderHtmlTable($cols,$table)
{
  $html = '';
  $html .= '<table>';
  $html .= '<tr>';
  foreach ($cols as $col)
  {
    $html .= '<th>' . HtmlQuote($col['title']);
  }
  $html .= "\n";
  $rowix = 0;
  foreach ($table as $row)
  {
    $rowix ++;
    $rowClass = 'odd';
    if ($rowix % 2 == 0) $rowClass = 'even';
    $html .= '<tr class="' . $rowClass . '">';
    $ix = 0;
    foreach ($row as $cell)
    {
      $ix++;
      $html .= '<td class="c'.$ix.'">' . HtmlQuote($cell);
    }
    $html .= "\n";
  }
  $html .= '</table>' . "\n";
  return $html;
}

function CallListingReportCB($row, &$cfg)
{
	global $dbintf, $lng;
	$data = array();
	$data[] = $row['remoteid'];
	$data[] = date('Y-m-d H:i:s',$row['startdate']);
	$data[] = $row['duration'];
	$data[] = $row['local'];
	$data[] = $row['lname'];
	$data[] = $row['remote'];
	$data[] = $row['rname'];
	$caller = $receiver = '';
	if ($row['direction']==0) // incoming
	{
		$caller   = $row['remote'];
		$receiver = $row['local'];
	} else
	{
		$caller   = $row['local'];
		$receiver = $row['remote'];
	}
	$data[] = $caller;
	$data[] = $receiver;
	$dir = $row['direction'];
	$data[] = $dir;
	$dirt = '';
	if ($dir==0) $dirt = $lng->Get('Web.Incoming');
	elseif ($dir==1) $dirt = $lng->Get('Web.Outgoing');
	elseif ($dir==2) $dirt = $lng->Get('Web.Internal');
	elseif ($dir==3) ; // $dirt = ??
	$data[] = $dirt;
	// tenant?
	if ($row['tenantid']!=NULL)
	{
		$group = $dbintf->GetGroupInfo( $row['tenantid'] );
		$data[] = $group['name'];
	}	else $data[] = '';
	// user
	$agentid = $row['agent'];
	if ($agentid===NULL)
	{
		$agentid = GetAgentOfCall($row);
	}
	if ($agentid===NULL) $data[] .= '';
	else {
		$user = $dbintf->GetUserInfoOnAccId( $agentid );
		$data[] .= $user['name'];
	}
	$format = $cfg['format'];
	$xout = '';
	$cfg['cnt'] ++;
	if ($format=='html')
	{
    $rowClass = 'odd';
    if ($cfg['cnt'] % 2 == 0) $rowClass = 'even';
		$xout .= '<tr class="'.$rowClass.'">';
	}
	if ($format=='csv')
	{
		$row = new CsvRow();
	}
	foreach ($data as $elem)
	{
		if ($format=='html')
		{
			$xout .= '<td>' . HtmlQuote($elem);
		}
		if ($format=='csv')
		{
			$row->add( $elem );
		}
	}
	if ($format=='csv')
	{
		$xout .= $row->get();
	}
	// see ReportOutput class
	// output can be at once (to avoid too much memory usage)
	// or buffered
	$cfg['ro']->out( $xout );
}

function FoundCallForReport($row, &$cfg)
{
  global $dbintf;
  // debug: echo "call " . date('D M G:i:s Y',$row['startdate']) . ' ' . $row['local'] . ' ' . $row['remote'] . "\n";
  // GetUsersOfCall: second param = false, this means also include users without enabled account
  $users = $dbintf->GetUsersOfCall($row,false);
  $usersdata = & $cfg['data']['users'];
  foreach ($users as $accid)
  {
    $usersdata[ $accid ] ++;
  }
  if (count($users)==0)
  {
    $cfg['data']['nouser'] ++;
  }
  
}










