<?php
require_once "../apresa/conf/voip_config.php";
require_once 'scim_general.php';
require_once 'db.php';
require_once 'actions.php';

function GetUserSchema()
{
    $schema = [
        "name" => "User",
        "description" => "User Account",
        "meta" => [
            "resourceType" => "Schema",
            "created" => "2022-10-06T13:13:00",
            "lastModified" => "2022-10-06T13:13:00",
        ],
        "id" => USER_SCHEMA,
        "schemas" => [SCHEMA_SCHEMA],
        "attributes" => [
            [
                'caseExact' => false,
                'type' => 'string',
                'description' => USER_USERNAME_DESCRIPTION,
                'mutability' => 'readWrite',
                'name' => 'userName',
                'multiValued' => false,
                'required' => true,
                'returned' => 'default',
                'uniqueness' => 'server',
            ],
            [
                "caseExact" => false,
                "type" => "string",
                "description" => USER_DISPLAYNAME_DESCRIPTION,
                "mutability" => "readWrite",
                "name" => "displayName",
                "multiValued" => false,
                "required" => false,
                "returned" => "default",
                "uniqueness" => "none",
            ],
            [
                "caseExact" => false,
                "type" => "boolean",
                "description" => USER_ACTIVE_DESCRIPTION,
                "mutability" => "readWrite",
                "name" => "active",
                "multiValued" => false,
                "required" => false,
                "returned" => "default",
                "uniqueness" => "none",
            ],
            [
                "caseExact" => false,
                "type" => "complex",
                "description" => USER_EMAILS_DESCRIPTION,
                "mutability" => "readWrite",
                "name" => "emails",
                "multiValued" => true,
                "required" => false,
                "returned" => "default",
                "uniqueness" => "none",
                "subAttributes" => [
                    [
                        "caseExact" => false,
                        "type" => "string",
                        "description" => VALUE_DESCRIPTION,
                        "mutability" => "readWrite",
                        "name" => "value",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                    ],
                    [
                        "caseExact" => false,
                        "type" => "string",
                        "description" => TYPE_DESCRIPTION,
                        "mutability" => "readWrite",
                        "name" => "type",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                        "canonicalValues" => [
                            "work",
                            "home",
                            "other",
                        ],
                    ],
                    [
                        "caseExact" => false,
                        "type" => "boolean",
                        "description" => PRIMARY_DESCRIPTION,
                        "mutability" => "readWrite",
                        "name" => "primary",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                    ],
                ],
            ],
            [
                "caseExact" => false,
                "type" => "complex",
                "description" => USER_PHONES_DESCRIPTION,
                "mutability" => "readWrite",
                "name" => "phoneNumbers",
                "multiValued" => true,
                "required" => false,
                "returned" => "default",
                "uniqueness" => "none",
                "subAttributes" => [
                    [
                        "caseExact" => false,
                        "type" => "string",
                        "description" => VALUE_DESCRIPTION,
                        "mutability" => "readWrite",
                        "name" => "value",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                    ],
                    [
                        "caseExact" => false,
                        "type" => "string",
                        "description" => TYPE_DESCRIPTION,
                        "mutability" => "readWrite",
                        "name" => "type",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                        "canonicalValues" => [
                            "work",
                            "home",
                            "other",
                            "mobile",
                            "fax",
                            "pager",
                        ],
                    ],
                    [
                        "caseExact" => false,
                        "type" => "boolean",
                        "description" => PRIMARY_DESCRIPTION,
                        "mutability" => "readWrite",
                        "name" => "primary",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                    ],

                ],
            ],
        ],
    ];

    return $schema;
}

function GetGroupSchema()
{
    $schema = [
        "name" => "Group",
        "description" => "Group",
        "meta" => [
            "resourceType" => "Schema",
            "created" => "2022-10-06T13:13:00",
            "lastModified" => "2022-10-06T13:13:00",
        ],
        "id" => GROUP_SCHEMA,
        "schemas" => [
            SCHEMA_SCHEMA,
        ],
        "attributes" => [
            [
                "caseExact" => false,
                "type" => "string",
                "description" => GROUP_DISPLAYNAME_DESCRIPTION,
                "mutability" => "readWrite",
                "name" => "displayName",
                "multiValued" => false,
                "required" => true,
                "returned" => "default",
                "uniqueness" => "server",
            ],
            [
                "caseExact" => false,
                "type" => "complex",
                "description" => GROUP_MEMBERS_DESCRIPTION,
                "mutability" => "readWrite",
                "name" => "members",
                "multiValued" => true,
                "required" => false,
                "returned" => "default",
                "uniqueness" => "none",
                "subAttributes" => [
                    [
                        "caseExact" => false,
                        "type" => "string",
                        "description" => VALUE_DESCRIPTION,
                        "mutability" => "readWrite",
                        "name" => "value",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                    ],
                    [
                        "caseExact" => false,
                        "type" => "string",
                        "description" => TYPE_DESCRIPTION,
                        "mutability" => "immutable",
                        "name" => "type",
                        "multiValued" => false,
                        "required" => false,
                        "returned" => "default",
                        "uniqueness" => "none",
                        "canonicalValues" => [
                            "User",
                        ],
                    ],
                ],
            ],
        ],
    ];

    return $schema;
}

function HandleRequest()
{
	if(!isset($_GET['group']))
	{
		throw new ScimException("Not found", 404);
	}

	global $requestID;
	$requestID = uniqid();
	$parentGroup = $_GET['group'];

	global $dbintf;
	InitDB();

	$parentInfo = $dbintf->GetSingleGroupInfoByName($parentGroup);
	if($parentInfo === false)
	{
		throw new ScimException("Not found", 404);
	}

	if(IsAuthorized($parentInfo))
	{
		$resources = [GetUserSchema(), GetGroupSchema()];
		header('Content-Type: application/scim+json');
		echo json_encode(Paginate($resources, 1, count($resources)));
	}
	else
	{
		http_response_code(401);
		header('Content-Type: application/scim+json');
		echo json_encode([
			"schemas" => [ERROR_SCHEMA],
			"detail" => "Unauthorized",
			"status" => 401
		]);
	}
}

try
{
	HandleRequest();
}
catch(ScimException $exception)
{
  $result = [
    "schemas" => [ERROR_SCHEMA],
    "detail" => $exception->getMessage(),
    "status" => $exception->getCode()
  ];

  if(isset($exception->scimType))
  {
    $result["scimType"] = $exception->scimType;
  }

  $code = $exception->getCode();
  $response = json_encode($result);
  scimlg('Scim exception on schemas: ' . $response);

  header('Content-Type: application/scim+json');
  http_response_code($code);
  echo $response;
}
catch(Exception $exception)
{
	$result = [
    "schemas" => [ERROR_SCHEMA],
    "detail" => "An internal error occured",
    "status" => 500
  ];

	$response = json_encode($result);
  scimlg('Exception on schemas: ' . $exception->getMessage());
	header('Content-Type: application/scim+json');
  http_response_code(500);
  echo $response;
}