<?php


function socket_read_timeout(&$socket,$size,$flags,$seconds,&$reply)
{
	socket_set_nonblock($socket);
	$read = array($socket);
	$write = null;
	$except = null;
	if (socket_select($read,$write,$except,$seconds)==1)
	{
  	$reply = socket_read($socket,$size,$flags);
 		return '';
 	} else
 	{
 		return 'socket timeout while reading reply';
 	}
}

function signal_socket($socket,$msg,&$reply,$timeout=10)
{
  if (@socket_connect($socket, '/tmp/apresa' )===FALSE)
  {
    return "error in socket_connect";
  }
  $msg = $msg . "\r\n";
  if (@socket_write($socket,$msg,strlen($msg))===FALSE)
  {
    return "error in socket_write";
  }
  if ($reply)
  {
    // we want a reply
    $reply = '';
    return socket_read_timeout($socket,1024,PHP_NORMAL_READ,$timeout,$reply);
    //$reply = socket_read($socket,1024,PHP_NORMAL_READ);
  } 
	/*
	/// would give problem with tracker (must be fast and not wait for a reply)
	else
  {
		// always read reply
		$tmpreply = '';
    return socket_read_timeout($socket,1024,PHP_NORMAL_READ,5,$tmpreply);
		//$tmpreply = socket_read($socket,1024,PHP_NORMAL_READ);
	}
	*/
  return '';
}

function signal_service($msg,&$reply=false,$timeout=10)
{
  $ret = real_signal_service($msg,$reply,$timeout);
  if ($ret!='')
  {
    lg('Signaling error: '.$ret);
  }
  return $ret;
}

function real_signal_service($msg,&$reply=false,$timeout=10)
{
  $ret = "";
  if (COM_TYPE=='Win')
  {
    $h = fopen(COM_PATH,'a');
    if ($h==FALSE) return "Could not open ".COM_PATH;
    fwrite($h,$msg."\r\n");
    if ($reply)
    {
      $reply = 'OK';
    }
    fclose($h);
  } else
  {
    $socket = @socket_create(AF_UNIX, SOCK_STREAM, 0);
    if ($socket===FALSE)
    {
      $ret = "error in socket_create";
    } else
    {
      $ret = signal_socket($socket,$msg,$reply,$timeout);
      if ($ret=='')
      {
        // remove endline
        if($reply === false)
        {
          $reply = '';
        }
        else if ($reply[strlen($reply)-1]=="\r")
        {
          $reply = substr($reply,0,strlen($reply)-1);
        }
      }
      @socket_close($socket);
    }
  }
  return $ret;
}

function msg_signal_service(&$msg,$cmd,$timeout=10)
{
	$reply = 1;
	$signal_ret = signal_service($cmd,$reply,$timeout);
	if ($signal_ret!="")
	{
    if (isset($msg)) $msg[] = new Msg(MSG_ERR,'Socket Error ('.$signal_ret.')');
		return false;
	} else
	{
		if ($reply=='OK')
		{
			return true;
		} else
		{
	  	if (isset($msg)) $msg[] = new Msg(MSG_ERR,'Socket Error, Reply: '.$reply.'');
			return false;
		}
	}
}

function LogDomainSocketError($place)
{
  $err = socket_last_error();
  lg('Domain socket error ' . $err . ' : ' . socket_strerror($err) . ' during ' . $place);
  return false;
}

function SendJsonToDomainSocket($request,$path)
{
  return json_decode( SendToDomainSocket( json_encode($request) , $path ) , true );
}

function SendToDomainSocket($request,$path)
{
  $socket = socket_create(AF_UNIX, SOCK_STREAM, 0);
  $ipc_path = $path;
  if (socket_connect($socket,$ipc_path)===FALSE)
  {
    return LogDomainSocketError('connect');
  }
  socket_write($socket,$request . "\0");
  $reply = '';
  while (true)
  {
    $r = array( $socket);
    $w = array();
    $e = array();
    $result = socket_select($r,$w,$e,NULL);
    if ($result === false)
    {
      return LogDomainSocketError('select');
    }
    $buf = socket_read($socket,4*1024);
    if ($buf === false)
    {
      return LogDomainSocketError('read');
    } else if ($buf === '')
    {
      //lg('end of data');
      break;
    } else
    {
      // received data from client
      $reply .= $buf;
      //lg('received data, now have ' . strlen($reply) . ' bytes');
      if (strlen($reply)>0 && $reply[ strlen($reply)-1 ]=="\0")
      {
        // remove trailing nul-byte
        $reply = substr($reply,0,strlen($reply)-1);
        break;
      }
    }
  }
  socket_shutdown($socket);
  socket_close($socket);
  //lg('string reply: ' . $reply);
  return $reply;
}


if (defined('CMD_REMOTE'))
{
	openlog('apresa_remote',0,LOG_USER);
}
else if (defined('CMD'))
{
	openlog('apresa_cmd',0,LOG_USER);
} 
else if (defined('SCIM'))
{
	openlog('apresa_scim',0,LOG_USER);
}
else
{
	openlog('apresa_web',0,LOG_USER);
}

// logging
function lg($t,$level=LOG_NOTICE)
{
  global $logging_prefix;
  if (isset($logging_prefix))
  {
    $t = $logging_prefix . ' ' . $t;
  }
	if (defined('DEBUG_LOGING'))
	{
		echo $t . "\n";
	} else
	{
		if (COM_TYPE=='Win')
		{
			$fn = dirname(COM_PATH).'/apresa_log.txt';
			error_log(date('c ') . $t . "\r\n", 3, $fn);
		} else
		{
			syslog($level,$t);
		}
	}
}


?>