<?php

// Contains things related to network drives and local drives

function IsIP4($id)
{
  return preg_match("/^([0-9]{1,3}\.){3}[0-9]{1,3}$/",$id)>0;
}
function MayBeIP6($id)
{
  return preg_match("/^[0-9,a-f,A-F,:]+$/",$id)>0;
}

function GetMountVersioningParam()
{
  $debianVersion = -1;
  $issuenet = @file_get_contents('/etc/issue.net');
  if (strpos($issuenet,"Debian GNU/Linux 8")!==FALSE)
  {
    $debianVersion = 8;
  }
  if (strpos($issuenet,"Debian GNU/Linux 9")!==FALSE)
  {
    $debianVersion = 9;
  }
  if (strpos($issuenet,"Debian GNU/Linux 10")!==FALSE)
  {
    $debianVersion = 10;
  }
  if (strpos($issuenet,"Debian GNU/Linux 11")!==FALSE)
  {
    $debianVersion = 11;
  }
  if (strpos($issuenet,"Debian GNU/Linux 12")!==FALSE)
  {
    $debianVersion = 12;
  }
  // NOTE: for Debian 10 nothing needs to be done
  // the default is already OK
  // because the kernel chooses highest available

  $smbVersionParam = '';
  if ($debianVersion>=8 && $debianVersion<=9)
  {
    // deb 6 and deb 7 dont support it
    $smbVersionParam = "-o vers=2.0";
  }
  return $smbVersionParam;
}

// maybe todo: make option for SMB3
function GetSmbVersioningParam()
{
  $debianVersion = -1;
  $issuenet = @file_get_contents('/etc/issue.net');
  if (strpos($issuenet,"Debian GNU/Linux 8")!==FALSE)
  {
    $debianVersion = 8;
  }
  if (strpos($issuenet,"Debian GNU/Linux 9")!==FALSE)
  {
    $debianVersion = 9;
  }
  if (strpos($issuenet,"Debian GNU/Linux 10")!==FALSE)
  {
    $debianVersion = 10;
  }
  if (strpos($issuenet,"Debian GNU/Linux 11")!==FALSE)
  {
    $debianVersion = 11;
  }
  if (strpos($issuenet,"Debian GNU/Linux 12")!==FALSE)
  {
    $debianVersion = 12;
  }

  $smbVersionParam = '';
  if ($debianVersion>=8 && $debianVersion<=10)
  {
    // deb 6 and deb 7 dont support it
    // customer reported -mSMB2 working but -mSMB3 not working on Debian 10
    $smbVersionParam = '-mSMB2';
  }
  return $smbVersionParam;
}

function DnsLookup($dnsname)
{
  $ret = shell_exec("host -t a " . escapeshellarg($dnsname) . " 2>&1");
  $lines = explode("\n",$ret);
  if (count($lines)==0) return false;
  $line = $lines[0];
  $pos = strpos($line,"has address");
  if ($pos !== false)
  {
    $rest = substr($line,$pos+12);
    if (IsIP4($rest))
    {
      return $rest;
    }
  }
  return false;
}

function LookupIpAddress($NameOrIP)
{
  if (IsIP4($NameOrIP)) return $NameOrIP;
  $addr = NmbLookupIPAddr2($NameOrIP);
  if ($addr!==false) return $addr;
  $addr = DnsLookup($NameOrIP);
  if ($addr!==false) return $addr;
  return false;
}

function NmbParseLookupIPAddr($svr,$thisip,$subnetmask,$nmblines)
{
    /*
    // what is subnet mask?
    $this_addr = inet_pton($thisip);
    if ($this_addr===false) {
        return false;
    }
    $mask = inet_pton($subnetmask);
    if ($mask_addr===false) {
        return false;
    }
    $subnet = $this_addr & $mask;
    */

    foreach ($nmblines as $line)
    {
      if (strpos($line,'querying')===FALSE)
      {
        $parts = explode(' ',$line);
        if (count($parts)==2)
        {
          $ip = $parts[0];
          // check if lookup returns the same name
          if (strtoupper(GetSmbServerNameOnIP($ip))==strtoupper($svr))
          {
            return $ip;
          }
          /*
          if (IsIP4($parts[0]))
          {
            // test if in the same subnet
            $addr = inet_pton($parts[0]);
            $subnet2 = $addr & $mask;
            if ($subnet2==$subnet)
            {
              return $parts[0];
            }
          } // end if IsIP
          */
        } // end if count
      } // end if querying
    } // end for each
    return FALSE;
}

function NmbLookupIPAddr2($svr)
{
  $nmbret = shell_exec("nmblookup ".escapeshellarg($svr));
  $nmblines = explode("\n",$nmbret);
  foreach ($nmblines as $line)
  {
    $parts = explode(' ',$line);
    if (count($parts)>=2)
    {
      if (IsIP4($parts[0]))
      {
        return $parts[0];
      }
    } // end if count
  } // end for each  
  return false;
}

// in Apresa: $subnetmask = $configs['IPmask'];
// Is not garanteed to return an IP,
// may return FALSE.
function NmbLookupIPAddr($svr,$subnetmask)
{
    $thisip = $_SERVER['SERVER_ADDR'];
    if (IsIP4($svr))
    {
      return false;
    }
    /*
    if (MayBeIP6($svr)) return false;
    if ($subnetmask=='') return false;
    if (MayBeIP6($thisip)) return false;
    */

    // look-up server IP
    $nmbret = shell_exec("nmblookup ".escapeshellarg($svr));
    $nmblines = explode("\n",$nmbret);
    return NmbParseLookupIPAddr($svr,$thisip,$subnetmask,$nmblines);
}

function GetSmbServerNameOnIP($ip)
{
  $ret = array();
  $cmdline = "nmblookup -A ".escapeshellarg($ip);
  $smbret = shell_exec($cmdline);
  $smblines = explode("\n",$smbret);
  foreach ($smblines as $line)
  {
    if (strpos($line,'Looking up')!==FALSE) continue;
    $line = trim($line);
    $parts = explode(' ',$line);
    if (count($parts)>=1)
    {
      return $parts[0];
    }
  }
  return '';
}

function GetSmbServerListNew()
{
  $ret = array();
  $cmdline = "smbtree -S -N";
  $smbret = shell_exec($cmdline);
  $smblines = explode("\n",$smbret);
  $domain = '';
	foreach ($smblines as $line)
  {
  	$line = trim($line);
  	if (substr($line,0,2)=='\\\\')
  	{
			// it is a server
			$line = substr($line,2);
    	$parts = explode(' ',$line);
    	$ret[] = array('name'=>$parts[0],'domain'=>$domain);
		} else
		{
			// it is a domain
			$domain = $line;
		}
  }
  return $ret;
}

function GetSmbServerList()
{
  $ret = array();
  $cmdline = "nmblookup '*'";
  $smbret = shell_exec($cmdline);
  $smblines = explode("\n",$smbret);
  foreach ($smblines as $line)
  {
    if (strpos($line,'querying')!==FALSE) continue;
    $parts = explode(' ',$line);
    if (count($parts)==2)
    {
      if (IsIP4($parts[0]) || MayBeIP6($parts[0]))
      {
        $ret[] = $parts[0];
      }
    }
  }
  return $ret;
}


function FindDisks()
{
  $disks = array();
  if (COM_TYPE=='Win')
  {
    // demo
    $disks[] = 'by-label/DemoDiskLabel';
    $disks[] = 'by-id/DemoID-0123456789';
  } else
  {
    $diskKeys = array( 'by-label', 'by-id' );
    foreach ($diskKeys as $diskKey)
    {
      $dir = '/dev/disk/' . $diskKey . '/';
      if (file_exists($dir)==false) continue;
      $items = scandir( $dir );
      foreach ($items as $item)
      {
        if ($item=='.' || $item=='..') continue;
        if ($diskKey=='by-id')
        {
          if (substr($item,0,4)!='usb-')
          {
            continue;
          }
        }
        //echo '<br>==> ' . $item;
        $disks[] = $diskKey . '/' . $item;
      }
    }
  }
  return $disks;  
}

function CheckBckDisk($BckDisk)
{
  // check if in array
  $found = false;
  $disks = FindDisks();
  foreach ($disks as $disk)
  {
    if ($disk == $BckDisk)
    {
      $found = true;
    }
  }
  return $found;
}

