<?php
// never called directly
// only do actions in functions

require_once 'p/lib/php-saml/_toolkit_loader.php';
require_once 'p/lib/xmlseclibs/xmlseclibs.php';

require_once '../apresa/conf/voip_config.php';
require_once 'db.php';
require_once 'actions.php';
require_once 'web-utils.php';


use OneLogin\Saml2\Response;
use OneLogin\Saml2\LogoutResponse;
use OneLogin\Saml2\Settings;

function IdpErr($msg)
{
  echo 'IdP. Error: ' . $msg;
  exit;
}

function SamlCfgErr($msg)
{
  echo 'Configuration error: ' . $msg;
  exit;
}

function SamlBrLog($msg)
{
  echo '<br>' . $msg;
  lg($msg);
}

function SamlException($msg,$e)
{
  $t = $msg . ": " . $e->getMessage();
  echo '<br>' . $t;
  lg($t); // apresa.log
  // no stack trace
}

function GetSamlSettings($ssoId)
{
  global $dbintf;
  
  $spEntityId = 'http://apresa.vidicode.com/SN' . GlobGetSerialNumber();
  $accessUrl = $dbintf->GetConfigValue('AccessUrl');
  if (strlen($accessUrl)<2)
  {
    SamlCfgErr('No Apresa access URL is not specified');
  }
  if ($accessUrl[ strlen($accessUrl)-1 ]!='/')
  {
    $accessUrl .= '/';
  }

  $idp = $dbintf->GetExtIdP( $ssoId );
  if ($idp===false)
  {
    SamlCfgErr('Unknown Identity Provider');
  }

  $spCert = '';
  $spKey = '';
	$certId = $dbintf->GetCertificateFor('Saml');
	if($certId === false) SamlCfgErr('No certificate is selected for use by Apresa (System settings, Apresa certificate)');
  $cert = $dbintf->GetCertificate( $certId );
  $fn = $cert['cert_location'];
  if ($fn=='') SamlCfgErr('The selected Apresa certificate does not reference a certificate');
  $path = dirname(__FILE__) . '/../apresa/certificates/' . $fn;
  if (file_exists($path)==false) SamlCfgErr('The file containing the selected Apresa certificate does not exist');
  $spCert = file_get_contents( $path );
  if ($spCert=='') SamlCfgErr('The file containing the selected Apresa certificate cannot be read or is empty');
  
  $fn = $cert['key_location'];
  if ($fn=='') SamlCfgErr('The selected Apresa certificate does not reference a private key');
  $path = dirname(__FILE__) . '/../apresa/certificates/' . $fn;
  if (file_exists($path)==false) SamlCfgErr('The file containing the selected Apresa private key does not exist');
  $spKey = file_get_contents( $path );
  if ($spKey=='') SamlCfgErr('The file containing the selected Apresa private key cannot be read or is empty.'
  . ' Enable the option to allow web server access to the key (Certificates screen).');
  
  $idpCert = '';
  if ($idp['cert']===null)
  {
    SamlCfgErr('No certificate is selected to identify the remote Identity Provider');
  }
  
  // we have a certificate of this identity provider
  $cert = $dbintf->GetCertificate( $idp['cert'] );
  $fn = $cert['cert_location'];
  if ($fn=='') SamlCfgErr('The selected IdP certificate does not reference a certificate');
  $path = dirname(__FILE__) . '/../apresa/certificates/' . $fn;
  if (file_exists($path)==false) SamlCfgErr('The file containing the selected IdP certificate does not exist');
  $idpCert = file_get_contents($path);
  if ($idpCert=='') SamlCfgErr('The file containing the selected IdP certificate cannot be read or is empty');
  
  if ($idp['entityid']=='')
  {
    SamlCfgErr('The Entity ID of the external identity provider is not filled in');
  }
  if ($idp['signonurl']=='')
  {
    SamlCfgErr('The Sign-on URL of the external identity provider is not filled in');
  }
  if ($idp['extlogout']==1 && $idp['signoffurl']=='')
  {
    SamlCfgErr('External sign-off is enabled, but the sign-off URL of the external identity provider is not filled in');
  }
            
  $settings = array(
    // If 'strict' is True, then the PHP Toolkit will reject unsigned
    // or unencrypted messages if it expects them signed or encrypted
    // Also will reject the messages if not strictly follow the SAML
    // standard: Destination, NameId, Conditions ... are validated too.
    'strict' => true,

    // Enable debug mode (to print errors)
    'debug' => true,

    // Set a BaseURL to be used instead of try to guess
    // the BaseURL of the view that process the SAML Message.
    // Ex. http://sp.example.com/
    //     http://example.com/sp/
    'baseurl' => $accessUrl,

    // Apresa is the Service Provider
    // Service Provider Data that we are deploying
    'sp' => array(
        // Identifier of the SP entity  (must be a URI)
        'entityId' => $spEntityId,
        // Specifies info about where and how the <AuthnResponse> message MUST be
        // returned to the requester, in this case our SP.
        'assertionConsumerService' => array(
            // URL Location where the <Response> from the IdP will be returned
            'url' => $accessUrl . 'sso-response.php',
            // SAML protocol binding to be used when returning the <Response>
            // message.  Onelogin Toolkit supports for this endpoint the
            // HTTP-POST binding only
            'binding' => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST',
        ),
        // Specifies info about where and how the <Logout Response> message MUST be
        // returned to the requester, in this case our SP.
        'singleLogoutService' => array(
            // URL Location where the <Response> from the IdP will be returned
            // This seems to be ignored for ADFS, we set it anyway
            'url' => $accessUrl . 'sso-logout.php',
            // SAML protocol binding to be used when returning the <Response>
            // message.  Onelogin Toolkit supports for this endpoint the
            // HTTP-Redirect binding only
            'binding' => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect',
        ),
        // Specifies constraints on the name identifier to be used to
        // represent the requested subject.
        // Take a look on lib/Saml2/Constants.php to see the NameIdFormat supported
        'NameIDFormat' => 'urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified',

        // Usually x509cert and privateKey of the SP are provided by files placed at
        // the certs folder. But we can also provide them with the following parameters
        'x509cert' => $spCert,
        'privateKey' => $spKey,

        /*
         * Key rollover
         * If you plan to update the SP x509cert and privateKey
         * you can define here the new x509cert and it will be 
         * published on the SP metadata so Identity Providers can
         * read them and get ready for rollover.
         */
        // 'x509certNew' => '',
    ),

    // Identity Provider Data that we want connect with our SP
    'idp' => array(
        // Identifier of the IdP entity  (must be a URI)
        'entityId' => $idp['entityid'],
        // SSO endpoint info of the IdP. (Authentication Request protocol)
        'singleSignOnService' => array(
            // URL Target of the IdP where the SP will send the Authentication Request Message
            'url' => $idp['signonurl'],
            // SAML protocol binding to be used when returning the <Response>
            // message.  Onelogin Toolkit supports for this endpoint the
            // HTTP-Redirect binding only
            'binding' => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect',
        ),
        // SLO endpoint info of the IdP.
        'singleLogoutService' => array(
            // URL Location of the IdP where the SP will send the SLO Request
            'url' => $idp['signoffurl'],
            // URL location of the IdP where the SP SLO Response will be sent (ResponseLocation)
            // if not set, url for the SLO Request will be used
            'responseUrl' => $accessUrl . 'sso-logout.php',
            // SAML protocol binding to be used when returning the <Response>
            // message.  Onelogin Toolkit supports for this endpoint the
            // HTTP-Redirect binding only
            'binding' => 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect',
        ),
        // Public x509 certificate of the IdP
        'x509cert' => $idpCert,
        /*
         *  Instead of use the whole x509cert you can use a fingerprint in
         *  order to validate the SAMLResponse, but we don't recommend to use
         *  that method on production since is exploitable by a collision
         *  attack.
         *  (openssl x509 -noout -fingerprint -in "idp.crt" to generate it,
         *   or add for example the -sha256 , -sha384 or -sha512 parameter)
         *
         *  If a fingerprint is provided, then the certFingerprintAlgorithm is required in order to
         *  let the toolkit know which Algorithm was used. Possible values: sha1, sha256, sha384 or sha512
         *  'sha1' is the default value.
         */
        // 'certFingerprint' => '',
        // 'certFingerprintAlgorithm' => 'sha1',

        /* In some scenarios the IdP uses different certificates for
         * signing/encryption, or is under key rollover phase and more 
         * than one certificate is published on IdP metadata.
         * In order to handle that the toolkit offers that parameter.
         * (when used, 'x509cert' and 'certFingerprint' values are
         * ignored).
         */
        // 'x509certMulti' => array(
        //      'signing' => array(
        //          0 => '<cert1-string>',
        //      ),
        //      'encryption' => array(
        //          0 => '<cert2-string>',
        //      )
        // ),
    ),
    'security' => array(
      'logoutRequestSigned' => true
    ),
  );
  return $settings;
}

function AzureGetProvider($idp)
{
  global $dbintf;
  $accessUrl = $dbintf->GetConfigValue('AccessUrl');
  if (strlen($accessUrl)<2)
  {
    SamlCfgErr('No Apresa access URL is not specified');
  }
  if ($accessUrl[ strlen($accessUrl)-1 ]!='/')
  {
    $accessUrl .= '/';
  }
  $redirectUri = $accessUrl . 'sso-response.php';
  
  // get the application details
  $app = $dbintf->GetAzureApp($idp['azureapp']);
  if ($app===false)
  {
    IdpErr('Azure application configuration not found');
  }
  
  $provider = new TheNetworg\OAuth2\Client\Provider\Azure([
    'clientId'          => $app['appid'],
    'clientSecret'      => decrypt($app['appsecret']),
    'redirectUri'       => $redirectUri
  ]);
  $provider->scope = ['openid'];
  $provider->urlAPI = "https://graph.microsoft.com/v1.0/";
  $provider->authWithResource = false;
  return $provider;
}

function AzureSsoLogin($ssoId, $idp)
{
  require_once '../apresa/vendor/autoload.php';
  $provider = AzureGetProvider($idp);
  // redirect!
  $_SESSION['ssoId'] = $ssoId;
  $_SESSION['oauth2state'] = $provider->getState();
  $authUrl = $provider->getAuthorizationUrl();
  header('Location: '.$authUrl);
  exit;
}

function AzureSsoHandleResponse($ssoId, $idp)
{
  global $dbintf;
  require_once '../apresa/vendor/autoload.php';
  $provider = AzureGetProvider($idp);
  $token = $provider->getAccessToken('authorization_code',
  [ 'code' => $_GET['code']
  , 'resource' => 'https://graph.microsoft.com'
  ]);
  if ($token==null)
  {
    IdpErr('Could not get Azure access token');
  }
  $claims = $token->getIdTokenClaims();
  if (isset($claims['upn'])==false)
  {
    IdpErr('No UPN claim received');
  }
  $upn = $claims['upn'];
  // lets go
  SsoLoginWithUsr($upn);
}

function SsoLogin($ssoId)
{
  global $dbintf;
  
  $idp = $dbintf->GetExtIdP( $ssoId );
  if ($idp===false)
  {
    IdpErr('Unknown Identity Provider');
  }
  if ($idp['tech']=='Azure')
  {
    AzureSsoLogin($ssoId,$idp);
    return;
  }
  
  $settings = GetSamlSettings($ssoId);
  //echo "<pre>";
  //var_dump($settings);
  //exit;
  $_SESSION['ssoId'] = $ssoId;
  $auth = new OneLogin\Saml2\Auth( $settings );
  $auth->login();
}

function SsoHandleResponse()
{
  global $dbintf, $rooturl;
  
  session_start();

  WebBasicInit();
  
  $ssoId = $_SESSION['ssoId'];
  if ($ssoId=='')
  {
    SamlCfgErr('ID missing in the session - Please check the configured Apresa URL if it matches the original access URL.');
  }
  $idp = $dbintf->GetExtIdP( $ssoId );
  if ($idp===false)
  {
    SamlCfgErr('Unknown Identity Provider');
  }
  if ($idp['tech']=='Azure')
  {
    AzureSsoHandleResponse($ssoId,$idp);
    return;
  }  
  
  if (isset($_POST['SAMLResponse'])==false){
    echo 'No SAMLResponse received';
    exit;
  }
  
  $samlSettings = new Settings( GetSamlSettings($ssoId) );
  try{
    $samlResponse = new Response( $samlSettings , $_POST['SAMLResponse'] );
  } catch(Exception $e){
    echo $e->getMessage()
      . '<br>Hint: If the error is Failure decrypting Data, then the wrong certificate might be configured on the Encryption tab of the Relying Party Trust configuration in ADFS.'
      ;
    exit;
  }
  if ($samlResponse->isValid()==false) {
    echo 'Invalid SAML Response';
    exit;
  }
  
  // logged on in AD FS
  
  // debug:
  // echo $samlResponse->getXMLDocument()->saveXml();
  // exit;
  // end debug block
  
  $foundUserId = false;
  $getNameIdExc = null;
  try
  {
    $ssoUsr = $samlResponse->getNameId();
    $foundUserId = true;
  } catch (OneLogin\Saml2\ValidationError $e)
  {
    $getNameIdExc = $e;
    // try other methods
  }
  
  // no Name ID, then try something else
  if ($foundUserId==false)
  {
    $claimedAttributes = $samlResponse->getAttributes();
    foreach ($claimedAttributes as $key => $items)
    {
      if ($key=="http://schemas.xmlsoap.org/ws/2005/05/identity/claims/upn"
      ||  $key=="http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name"
      )
      {
        // take the first one
        if ( count($items) > 0 )
        {
          $ssoUsr = $items[0];
          $foundUserId = true;
        }
      }
    }
  }
  
  if ($foundUserId==false)
  {
    SamlBrLog('Could not find any supported identity claim.');
    SamlException('Could not get Name ID from SAML response',$getNameIdExc);
    SamlBrLog('No supported claimed attributes: ' . var_export($claimedAttributes,true) );
    exit;
  }
  
  SsoLoginWithUsr($ssoUsr);
}

function SsoLoginWithUsr($ssoUsr)
{
  global $dbintf, $rooturl;

  $userInfo = $dbintf->GetUserInfoOnUsername( $ssoUsr );
  if ($userInfo == false)
  {
    $msg[] = new Msg(MSG_ERR, 'The user ' . $ssoUsr . ' was not found in Apresa' );
    $_SESSION['msg'] = $msg;
    header('Location: ' . $rooturl);
    exit;
  }
  
  if ((int)$userInfo['logonmethod']!=2)
  {
    $msg[] = new Msg(MSG_ERR, 'The user ' . $ssoUsr . ' is not enabled for SSO log on in Apresa' );
    $_SESSION['msg'] = $msg;
    header('Location: ' . $rooturl);
    exit;
  }

  if ((int)$userInfo['enabled']==0)
  {
    $msg[] = new Msg(MSG_ERR, 'The user ' . $ssoUsr . ' is not enabled in Apresa' );
    $_SESSION['msg'] = $msg;
    header('Location: ' . $rooturl);
    exit;
  }
  
  // boolean, 1=yes, to store that we are logged in using SAML
  $_SESSION['sso'] = 1;  
  
  LogInUidUsr($userInfo['id'],$userInfo['usr']);  
}

function SsoOutside()
{
  global $dbintf, $rooturl, $rpl, $msg;
  
  if ($dbintf->GetConfigValue('SsoOneAction')=='redirect')
  {
    header('Location: ' . $rooturl . '?loggedout');
    exit;
  }

  $target = $rooturl;
  header('Location: ' . $rooturl);
  exit;
}

function AzureSsoLogout($idp)
{
  global $dbintf;
  require_once '../apresa/vendor/autoload.php';

  $dbintf->DeleteSession(session_id());
  end_of_session();
  
  if ($idp['extlogout']==1)
  {
    $accessUrl = $dbintf->GetConfigValue('AccessUrl');
    if (strlen($accessUrl)<2)
    {
      IdpErr('No Apresa access URL is not specified');
    }
    if ($accessUrl[ strlen($accessUrl)-1 ]!='/')
    {
      $accessUrl .= '/';
    }
    if ($dbintf->GetConfigValue('SsoOneAction')=='redirect')
    {
      $accessUrl .= '?loggedout';
    }
    $provider = AzureGetProvider($idp);
    $logoutUrl = $provider->getLogoutUrl( $accessUrl );
    header('Location: ' . $logoutUrl);
    exit;
  } else
  {
    SsoOutside();  
  }
}

function SsoLogout()
{
  global $dbintf, $rooturl;
  
  $ssoId = $_SESSION['ssoId'];
  $idp = $dbintf->GetExtIdP( $ssoId );
  if ($idp['tech']=='Azure')
  {
    AzureSsoLogout($idp);
    return;
  }
  if ($idp['extlogout']==1)
  {
    $auth = new OneLogin\Saml2\Auth( GetSamlSettings($ssoId) );
    $auth->logout();
  } else
  {
    $dbintf->DeleteSession(session_id());
    end_of_session();

    SsoOutside();
  }
}

function SsoHandleLogoutResponse()
{
  global $dbintf, $rooturl, $lng;

  session_start();
  
  WebBasicInit();
  $ssoId = $_SESSION['ssoId'];

  $auth = new OneLogin\Saml2\Auth( GetSamlSettings($ssoId) );
  $samlSettings = new Settings( GetSamlSettings($ssoId) );
  $samlResponse = new LogoutResponse( $samlSettings , $_POST['SAMLResponse']);    
  if ($samlResponse->isValid()==false) {
    echo 'Invalid SAML LogoutResponse';
    exit;
  }
  
  $dbintf->DeleteSession(session_id());
  end_of_session();

  SsoOutside();
}

function SsoLoginPage($UsrSel)
{
  global $rpl, $msg, $dbintf;
  if (file_exists('../apresa/templ/custom/login_custom.htm'))
  {
    $template_content = file_get_contents('../apresa/templ/custom/login_custom.htm');
  } else
  {
    $template_content = file_get_contents('../apresa/templ/login.htm');
  }

  $usr = '';
  $data = '';
  $data .= "var showlink=0;\r\n";
  $data .= "var username=\"" . JsQuote($usr) . "\";\r\n";
	$data .= "var loginPageMessage=\"" . JsQuote( (string) $dbintf->GetConfigValue('LoginPageMsg')) . "\";\r\n";
  if ($UsrSel)
  {
    $data .= "window.addEventListener('DOMContentLoaded', InitUsrSelLoginPage, false);\r\n";
  } else
  {
    $data .= "window.addEventListener('DOMContentLoaded', InitOnlyLoginPage, false);\r\n";
  }
  
  CologonLoginPageMsg();
  
  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['data'] = $data;
  $rpl['INFO'] = '';
  $rpl['FOOTER'] = '';
  $rpl['MAINFORM'] = '';
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}

function SsoIdpSelPage()
{
  global $rpl, $msg, $dbintf;
  if (file_exists('../apresa/templ/custom/idpsel_custom.htm'))
  {
    $template_content = file_get_contents('../apresa/templ/custom/idpsel_custom.htm');
  } else
  {
    $template_content = file_get_contents('../apresa/templ/idpsel.htm');
  }
  $ExtIdPs = $dbintf->GetExtIdPs();
  $idpList = array();
  $idpList[] = array( 'id' => 0 , 'name' => 'Local log on' );
  foreach ( $ExtIdPs as $ExtIdP)
  {
    $idpList[] = array( 'id' => $ExtIdP['id'] , 'name' => 'Log on using ' . $ExtIdP['name'] );
  }
  $data = '';
  $data .= 'var idpList = ' . json_encode($idpList) . ";\n";
  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $template_content = do_replace_tags($template_content);
  echo $template_content;
  
}


