<?php

$duration_buckets = array(5,10,20,40,60,2*60,5*60,10*60,15*60,30*60,3600,2*3600);
$waittime_buckets = array(0,1,2,3,4,5,6,7,8,9,11,13,15,17,19,25,30,40,60,2*60,5*60,10*60,15*60,30*60,3600,2*3600);
$waittime_interval_begin = 10;  
/*
	For each data row read from the database, this function updates the result array.
	This function determines the key to use.
	Finally, $result will contain a double with about this:
		array[keys][direction]
	These keys can be dates, but also names, hours-of-the-day, etc.
	The data array is actually filled by StatApplySub.
*/
// cfg: configurations
// results: changed/filled in
// row: data row
// call-back function
function StatApply($cfg,&$result,$row)
{
  global $duration_buckets;
	global $waittime_buckets;
	global $waittime_interval_begin;
  // make timestamp GMT to be able to divide into day buckets
  $timestamp = $row['startdate'];
  $dt = new DateTime('@'.$timestamp);
  // not supported in older PHP: $dt->setTimestamp( $timestamp );
  $gm_timestamp = $timestamp + $dt->getOffset();
	$row['gm'] = $gm_timestamp;

  switch ($cfg['type'])
  {
		case 'ch': // not recorded because of channel limit
			$key = (int) date('H',$timestamp);
			break;
    case 'nc': case 'du': case 'wt': // number of calls / duration / waittime
      switch ($cfg['gran'])
      {
        case 'x':
					if($cfg['type'] == 'du')
					{
						$dur = $row['duration'];
						if($dur < 0)
						{
							$key = -1;
						} else
						{
							$bucket = 0;
							foreach ($duration_buckets as $value)
							{
								if ($dur >= $value)
								{
									$bucket ++;
								}
							}
							$key = $bucket;
						}
					} else
					{
						$dur = $row['waittime'];
						if($dur < 0)
						{
							$key = -1;
						} else if($dur < $waittime_buckets[$waittime_interval_begin]){
							$key = $dur;
						} else
						{
							$bucket = 0;
							foreach ($waittime_buckets as $value)
							{
								if ($dur >= $value)
								{
									$bucket ++;
								}
							}
							$key = $bucket;
						}
					}
					break;
        case 'd':
          // for days
          //$key = (int) ($row['startdate']/(24*60*60));
          $key = (int) ($gm_timestamp/(24*60*60));
          // textual:
          //$key = date('Y-m-d',$row['startdate']);
          break;
        case 'w':
          $day = (int) ($gm_timestamp/(24*60*60));
          $key = (int) ( ($day-3) / 7);
          break;
        case 'm':
          $year = date('Y',$timestamp);
          $month = date('m',$timestamp);
          $key = $year*12+($month-1);
          break;
        case 'y':
          $key = date('Y',$timestamp);
          break;
      }
      break;
    case 'td': // time of day
      $key = (int) date('H',$timestamp);
      break;
    case 'un':
			$key = $row['agent'];
			break;
    case 'ln': case 'ld': // local caller
    case 'rn': // remote caller
			$dir = $row['direction'];
			$loc = $cfg['type']=='ln' || $cfg['type']=='ld';


	    // We must make a choice, either connected or the other
	    //     otherwise total is not correct
	    //     this choice could be based on the following:
	    //     - prefer to use connected number
	    //     - except when connected number is main number

	    // do connected if
	    // - statting local and call is incoming or internal
	    // - statting remote and call is outgoing or external
	    /*
	    if ( ($loc && ($dir==0 || $dir==2)) || (!$loc && ($dir==1 || $dir==3)) )
	    {
				if ($row['connectedcontact']!=0)
				{
					if ($row['cname']=='')
					{
						$key = $row['connected'];
					} else
					{
						$key = $row['cname'];
					}
					StatApplySub($key,$cfg,$result,$row);
				}
			}
			*/

			// do remote if:
			// - statting remote and call is not internal
			// - or if statting local and call is internal
			if ((!$loc && $dir!=2) || ($loc && $dir==2))
			{
	      if ($row['rname']=='')
	      {
	        $key = $row['remote'];
	      } else
	      {
	        $key = $row['rname'];
	      }
				if (!StatApplySub($key,$cfg,$result,$row))
				{
					return false;
				}
			}
			// do local if:
			// - statting local and call is not external
			// - or if statting remote and call is external
			if (($loc && $dir!=3) || (!$loc && $dir==3))
			{
	      if ($row['lname']=='')
	      {
	        $key = $row['local'];
	      } else
	      {
	        $key = $row['lname'];
	      }
				if (!StatApplySub($key,$cfg,$result,$row))
				{
					return false;
				}
	    }
	    return true;
      break;
  }

	return StatApplySub($key,$cfg,$result,$row);
}

function GetDayBucket($timestamp)
{
  // make timestamp GMT to be able to divide into day buckets
  $dt = new DateTime('@'.$timestamp);
  // not supported in older PHP: $dt->setTimestamp( $timestamp );
  $gm_timestamp = $timestamp + $dt->getOffset();
	return (int) ($gm_timestamp/(24*60*60));
}

function StatLocalDetails($key,$cfg,&$result,$row)
{
	if (isset($result['meta'])==false)
	{
		$result['meta'] = array();
	}
	$meta =& $result['meta'];

	// as sub-key, the day (date) is used.
	$timestamp = $row['startdate'];
	$subkey = GetDayBucket( $timestamp );
	
	if (isset($meta['skmax'])==false)
	{
		// this is the first item
		// because we are sorted from latest to earliest,
		// this is the newest entry.
		// determine the range
		if ($cfg['gran']=='m')
		{
			$year = date('Y',$timestamp);
			$month = date('m',$timestamp);
			// first day of month
			$meta['skmin'] = gmmktime(0,0,0,$month,1,$year);
			// first day of next month (mid night)
			$month ++;
			if ($month==13) {$month=1; $year++; }
			$meta['skmax'] = gmmktime(0,0,0,$month,1,$year);
			/*
			echo '<BR>B '.date('c',$meta['skmin']);
			echo '<BR>E '.date('c',$meta['skmax']);
			echo '<BR>B '.GetDayBucket($meta['skmin']);
			echo '<BR>E '.GetDayBucket($meta['skmax']);
			echo '<BR>B '.gmdate('D M G:i:s Y',GetDayBucket($meta['skmin'])*(24*60*60));
			*/
			$meta['skmin'] = GetDayBucket( $meta['skmin'] );
			$meta['skmax'] = GetDayBucket( $meta['skmax'] ) - 1;
		} else if ($cfg['gran']=='w')
		{
			$b = GetDayBucket( $timestamp );
			// 0:Sunday
			$dow = date('w',$timestamp);
			// move to Sunday
			$b -= $dow;
			$meta['skmin'] = $b;
			$meta['skmax'] = $b+6;
			// max. is inclusive
		} else
    {
			$b = GetDayBucket( $timestamp );
			$meta['skmin'] = $b;
			$meta['skmax'] = $b;
    }
	}
	if ($subkey < $meta['skmin'] )
	{
		return false;
	}
	
  $values =& $result['data'][$key];
	
	if (isset($values[$subkey])==false)
	{
		$values[$subkey] = array();
	}
	
	// The day
	StatLocalDetailsSub( $values[$subkey] , $row );
	// Total for this extension
	StatLocalDetailsSub( $values[ $meta['skmax']+1 ] , $row );
	
	$values =& $result['tot'];
	// Total of this day
	StatLocalDetailsSub( $values[$subkey] , $row );
	// Grand total
	StatLocalDetailsSub( $values[ $meta['skmax']+1 ] , $row );
	
	return true;
}

function StatLocalDetailsSub( &$data_ref, $row )
{
	// Calc. time of day
	$tod = $row['gm'] % (24*60*60);
	// No. Calls
	add_to($data_ref[0],1);
	// Total time
	add_to($data_ref[1], $row['duration'] );
	// Av. dur (this is calculated afterwards)
	// First call
	setSmallest($data_ref[3],$tod);
	// Last call
	setLargest($data_ref[4],$tod);
	$data_ref[5] = $row['gm'] - $tod;
}

function setSmallest(&$var,$value)
{
	if (isset($var))
	{
		$var = min( $var , $value );
	} else
	{
		$var = $value;
	}
}
function setLargest(&$var,$value)
{
	if (isset($var))
	{
		$var = max( $var , $value );
	} else
	{
		$var = $value;
	}
}


/*
	Fills data array for a specific key.

	Data arrays have this structure:
		[0] incoming
		[1] outgoing
		[2] internal
		[3] total
		[4] number of data points for [0]
				(used to calculate average in post processing)
		[5] same for [1]
		[6] same for [2]
		[7] same for [3]
*/
function StatApplySub($key,$cfg,&$result,$row)
{
	if ($cfg['type']=='ld')
	{
		return StatLocalDetails($key,$cfg,$result,$row);
	}

  $values = &$result[$key];
	if ($cfg['type']=='ch')
	{
		add_to($values[3],1);
		add_to($values[7],1);
		return;
	}

  $value = 1;
  if ($cfg['type']=='du' && $cfg['gran']!='x')
  {
    $value = $row['duration'];
  }
	else if ($cfg['type']=='wt' && $cfg['gran']!='x')
  {
    $value = $row['waittime'];
  }
	if($value != -1)
	{
		if ($row['direction']==0) // incoming
		{
			add_to($values[0],$value);
			add_to($values[4],1);
		} else if ($row['direction']==2) // local
		{
			add_to($values[2],$value);
			add_to($values[6],1);
		} else 
		{ // otherwise assume outgoing
			add_to($values[1],$value);
			add_to($values[5],1);
		}
		// all:
		add_to($values[3],$value);
		add_to($values[7],1); // later for calculation average
	}
	return true;
}

function FormatValue($cfg, &$value)
{
  global $configs,$lng;
  if (!isset($value))
  {
    // for call duration null means unknown
    // for other it really means zero
    if ($cfg['type']=='du' || $cfg['type']=='wt') return '';
    return 0;
  }
	if (($cfg['type']=='du' || $cfg['type'] == 'wt') && $cfg['gran']!='x')
	{
		return formatDuration( round($value) );
	} else
	{
		// proper rounding
		return sprintf('%.0f',$value);
	}
}

function FormatKey($cfg,$key)
{
  global $configs,$lng,$duration_buckets,$waittime_buckets, $waittime_interval_begin;
	if ($cfg['type']=='ld')
	{
		return $key;
	}
	if ($cfg['type']=='un')
	{
		if ($key==null) return $lng->Get('Main.None');
		return str_ensure_defined($cfg['agents'][$key]);
	}
  $t = '';
  switch ($cfg['gran'])
  {
    case 'd':
      // for days
      // on purpose using gmdate
      $t = gmdate($configs['datef'],$key * (24*60*60));
      break;
    case 'w':
      // +1 to show monday
      $wk = ($key*7 +3+1) * 24*60*60;
      $t = gmdate($configs['datef'],$wk);
      break;
    case 'm':
      $year = (int) ($key / 12);
      $month = ($key % 12) + 1;
      //$t = $year.'-'.sprintf('%02d',$month+1);
      // maybe TODO: get month from $lng per language
      $t = date('M Y',mktime(0,0,0,$month,1,$year));
      break;
    case 'x':
			if ($cfg['type']=='td' || $cfg['type']=='ch')
      {
				if ($cfg['out']=='bc')
				{
        	$t = $key . $lng->Get('DateTime.h');
        } else
        {
					$t = $key;
				}
      } else if ($cfg['type']=='du')
      {
				if($key == -1)
				{
					$t = $lng->Get('Web.unknown');
				} else if (isset($duration_buckets[$key])==false)
        {
          $t = $lng->Get('Stats.Remainder');
        }	else
        {
          $sec = $duration_buckets[$key];
          if ($sec<60)
          {
            $t = '<'.$duration_buckets[$key].$lng->Get('DateTime.s');
          } else if ($sec<60*60)
          {
            $t = '<'.($duration_buckets[$key]/60).$lng->Get('DateTime.m');
          } else if ($key<count($duration_buckets))
          {
            $t = '<'.($duration_buckets[$key]/3600).$lng->Get('DateTime.h');
          }
        }
      } else if($cfg['type']=='wt'){
				if($key == -1)
				{
					$t = $lng->Get('Web.unknown');
				} else if (isset($waittime_buckets[$key])==false)
        {
          $t = $lng->Get('Stats.Remainder');
        }	else
        {
          $sec = $waittime_buckets[$key];
					$interval = ($key < $waittime_interval_begin ? '' : '<');
          if ($sec<60)
          {
            $t = $interval.$waittime_buckets[$key].$lng->Get('DateTime.s');
          } else if ($sec<60*60)
          {
            $t = $interval.($waittime_buckets[$key]/60).$lng->Get('DateTime.m');
          } else if ($key<count($waittime_buckets))
          {
            $t = $interval.($waittime_buckets[$key]/3600).$lng->Get('DateTime.h');
          }
        }
			}	else
      {
        $t = $key;
      }
      break;
    default:
      $t = $key;
      break;

  }
  return $t;
}

function MakeTotalsArr($statres)
{
  $data = array();
	foreach ($statres as $key=>$values)
	{
		$val = int_ensure_defined($values[0]) 
			+ int_ensure_defined($values[1])
			+ int_ensure_defined($values[2])
			;
		$data[$key] = $val;
	}
  return $data;
}

function StatTotalsPostProcess($cfg,&$tots_ref)
{		
		if (($cfg['type']=='du' || $cfg['type'] =='wt')	&& $cfg['gran']!='x')
		{
			// calculation average
			for ($i=0;$i<4;$i++)
			{
				if ($tots_ref[$i+4]==0)
				{
					$tots_ref[$i] = null;
				} else
				{
					$tots_ref[$i] = $tots_ref[$i+8] / $tots_ref[$i+4];
				}
			}
		}
}
function StatPostProcess($cfg, &$statres)
{
  global $lng;
  if ($statres==null) return;
  if (isset($cfg['max_fraction'])==false)
  {
		$cfg['max_fraction'] = 0.95;
	}
  if (isset($cfg['max_entries'])==false)
  {
		$cfg['max_entries'] = 7;
	}

	if ($cfg['type']=='ld')
	{
		foreach ($statres['data'] as $key=>&$values)
		{
			foreach ($values as $subkey=>&$subdata)
			{
				// Calc. Av. duration
				$subdata[2] = $subdata[1] / $subdata[0];
			}
		}
		foreach ($statres['tot'] as $subkey=>&$subdata)
		{
			// Calc. Av. duration
			$subdata[2] = $subdata[1] / $subdata[0];
		}
		
	}
	
  if ($cfg['type']=='rn' || $cfg['type']=='ln')
  {
    // make a remaining category when needed
    $totals = MakeTotalsArr($statres);
    $tottot = 0;
    foreach ($totals as $key=>$value)
    {
			$tottot += $value;
		}
    arsort($totals);
    $i = 0;
    $remain = array();
    $statres_new = array();
    $so_far = 0;
    foreach ($totals as $key=>$value)
    {
			//echo '<p>Totals: ' . $key . ' '. $value;
      $i ++;
      if ($so_far>$tottot*$cfg['max_fraction'] || $i>$cfg['max_entries']) // make higher when more colors are available
      {
        foreach ($statres[$key] as $k=>$v)
        {
          add_to($remain[$k],$v);
        }
        unset($statres[$key]);
      } else
      {
				$so_far += $value;
        $statres_new[$key] = $statres[$key];
      }
    }
    if (count($remain)>0)
    {
      $statres_new[$lng->Get('Stats.Remainder')] = $remain;
    }
    $statres = $statres_new;

  } else
  {
    // for others it is already sorted
    if ($cfg['gran']=='x')
    {
      ksort($statres);
			// Move unknown times to end of array
    }
  }
  if (($cfg['type']=='du' || $cfg['type']=='wt') && $cfg['gran']!='x')
  {
    // calculation average
    foreach ($statres as $key=>$value)
    {
      for ($i=0;$i<4;$i++)
			{
				$statres[$key][$i+8] = int_ensure_defined($statres[$key][$i]);
				CalcAverage($statres[$key],$i);
			}
    }
  }
}

function CalcAverage(&$arr,$ix)
{
  $value =& $arr[$ix];
  $cnt = int_ensure_defined($arr[$ix+4]);
  if ($cnt==0) $value = null;
	// previously unit was in minutes, not good
  //else $value = $value / ($cnt*60);
	else $value = $value / $cnt;
}

function GetStatKeyHeader($cfg)
{
  global $lng;
  switch ($cfg['type'])
  {
    case 'nc': case 'du': case 'wt': case 'pc':
      if ($cfg['gran']!='x')
      {
        return $lng->Get('DateTime.Date');
      } else
      {
        if ($cfg['type']=='du' || $cfg['type']=='wt')
        {
          return $lng->Get('Cols.length');
        }
      }
    case 'td': case 'ch':
      return $lng->Get('DateTime.Hour');
		case 'un':
			return $lng->Get('PM.User');
    case 'ln': case 'ld':
      return $lng->Get('Main.LocalCaller');
    case 'rn':
      return $lng->Get('Main.RemoteCaller');
  }
  return '';
}

function GetStatUnit($cfg)
{
  global $lng;
  if (($cfg['type']=='du' || $cfg['type'] == 'wt') && $cfg['gran']!='x')
	{
		// because H:M:S will be self explanatory
		return ''; 
	}
  else return $lng->Get('Stats.numcalls');
}

function GetStatTitle($cfg)
{
  global $lng;
  if (isset($cfg['out'])==false) return $lng->Get('Web.Statistics');
  switch ($cfg['type'])
  {
    case 'du': case 'wt':
			switch ($cfg['type'])
			{
				case 'du':
					$t = $lng->Get('Stats.calldur');
					break;
				case 'wt':
					$t = $lng->Get('Stats.waittime');
					break;
			}
			switch ($cfg['gran'])
			{
				case 'w': case 'm': case 'y':
				{
					if ($cfg['out']!='t' && $cfg['out']!='csv')
					{
						if ($cfg['part']==2)
						{
							$t .= ', '.$lng->Get('Stats.Total');
						} else
						{
							$t .= ', '.$lng->Get('Stats.averaged');
						}
					}
				}
				break;
			}
      switch ($cfg['gran'])
      {
        case 'w':
          $t .= ', '.$lng->Get('DateTime.perWeek');
          break;
        case 'm':
          $t .= ', '.$lng->Get('DateTime.perMonth');
          break;
        case 'y':
          $t .= ', '.$lng->Get('DateTime.perYear');
          break;
      }
      break;
    case 'nc':
      $t = $lng->Get('Stats.numcalls');
      switch ($cfg['gran'])
      {
        case 'd':
          $t .= ' per day'; // never used
          break;
        case 'w':
          $t .= ', '.$lng->Get('DateTime.perWeek');
          break;
        case 'm':
          $t .= ', '.$lng->Get('DateTime.perMonth');
          break;
        case 'y':
          $t .= ', '.$lng->Get('DateTime.perYear');
          break;
      }
      break;
    case 'td':
      $t = $lng->Get('Stats.timeofday');
      break;
		case 'un':
			$t = $lng->Get('PM.User');
			break;
    case 'ln':
      $t = $lng->Get('Main.LocalCaller');
      break;
    case 'ld':
      $t = $lng->Get('Main.LocalCaller') . ' - ' . $lng->Get('CE.Details');
      switch ($cfg['gran'])
      {
        case 'w':
          $t .= ' - '.$lng->Get('DateTime.Week');
          break;
        case 'm':
          $t .= ' - '.$lng->Get('DateTime.Month');
          break;
      }
      break;
    case 'rn':
      $t = $lng->Get('Main.RemoteCaller');
      break;
    case 'ch':
    	$t = $lng->Get('Stats.NotRecordedCallsFull');
    	break;
		case 'pc':
			$t = $lng->Get('Stats.MaxCalls');
      switch ($cfg['gran'])
      {
        case 'd':
          $t .= ' - '.$lng->Get('DateTime.perDay');;
          break;
      }
			break;
  }
  return $t;
}


// fills up missing rows in results array
// to prevent gaps
// otherwise this has to be checked by each exporting function
// maybe better: just loop from start key to end key,
// and fill up with zeros.
// Or just check it each exporting function
// This function slows things down
function FillStatMissing($cfg,&$result)
{
  $prevkey = null;
  $cnt = 0;
  $sz = sizeof($result);
  foreach ($result as $key=>$values)
  {
    $cnt ++;
    if ($cnt>$sz) continue;
    if ($prevkey!=null)
    {
      for ($k2=$prevkey+1;$k2<$key;$k2++)
      {
        if (isset($result[$k2]))
        {
          // should never occur
        } else {
          $result[$k2] = array();
        }
      }
    }
    $prevkey = $key;
  }
  ksort($result);
}

// fills in parameters based on GET and POST
function StatInit(&$statcfg,&$search,&$search_use)
{
  global $dbintf;
	$statcfg = array();
	$statcfg['gran'] = 'w';

  //ensure these keys exists;
  if(!isset($statcfg['type'])) $statcfg['type'] = '';
  if(!isset($statcfg['part'])) $statcfg['part'] = '';
  if(!isset($statcfg['sortdir'])) $statcfg['sortdir'] = '';
  
	foreach (array('type','gran','out','part') as $param)
	{
    if (isset($_GET[$param]))
    {
		  $statcfg[$param] = $_GET[$param];
		}
  }
  if ($statcfg['type']=='ch' || $statcfg['type']=='pc')
  {
		$statcfg['onlytot'] = true;
	}
  // Searching
	$search_accepted = array
	( 'r1', 'l1', 'c1', 'r2', 'l2' , 'rn' , 'ln' , 'cn'
  , 'nts', 'ents', 'nts2', 'nts3', 'nts4', 'nts5', 'ann'
  , 'dts1', 'dts2', 'dts3', 'dts4', 'dts4', 'dts5'
	, 'fd' , 'ld' , 'id' , 'ip' , 'nm'
	, 'eid' , 'eip' , 'enm' , 'dir', 'ft', 'lt', 'own'
	, 'fdur', 'ldur', 'lnr', 'cat', 'rem', 'rb', 'rec'
  , 'tenant'
	);	
  $search = array();
  foreach ($search_accepted as $searchix)
  {
    if (str_ensure_defined($_POST[$searchix])!='')
    {
      $search[$searchix] = $_POST[$searchix];
    }
    // in GET when browsing pages
    if (str_ensure_defined($_GET[$searchix])!='')
    {
      $search[$searchix] = $_GET[$searchix];
    }
  }
  $queries = $dbintf->GetQueries();
  foreach ($queries as & $q_ref)
  {
    $q_ref['qspec_arr'] = StringToArray($q_ref['qspec']);
    if ($_GET['savq']==$q_ref['id']
    || $_POST['savq']==$q_ref['id']
    )
    {
      foreach ($q_ref['qspec_arr'] as $param=>$value)
      {
        if (in_array($param,$search_accepted))
        {
          $search[$param] = $value;
        }
      }
    }
  }
  
  $search_use = $search;
	if (isset($search_use['rb'])==false)
	{
		$search_use['rb'] = 0;
	}
  if (isset($search['ld']))
  {
    $ld = $search_use['ld'];
    // next day
    $ld = $ld + 60 * 60 * 24;
    $search_use['ld'] = $ld;
  }
  
  $multiTenancy = (int) ($dbintf->GetConfigValue('MultiTenancy')!=0);
  if (isset($search['tenant'])
    && $dbintf->AmIAdminT123()
    && ($multiTenancy!=0) )
  {
    $searchName = $search_use['tenant'];
    unset( $search_use['tenant'] );
    $foundTenant = -1;
    $tenantNameMap = BuildTenantNameMap();
    if (isset($tenantNameMap) && $foundTenant==-1)
    {
      foreach ($tenantNameMap as $id=>$name)
      {
        // exact match
        if (strtolower($name)==strtolower($searchName))
        {
          $search['tenant'] = $name;
          $foundTenant = $id;
          break;
        }
      }
    }
    if (isset($tenantNameMap) && $foundTenant==-1)
    {
      foreach ($tenantNameMap as $id=>$name)
      {
        // beginning match
        if (strtolower(substr($name,0,strlen($searchName)))==strtolower($searchName))
        {
          // show in GUI full and exact name
          $search['tenant'] = $name;
          $foundTenant = $id;
          break;
        }
      }
    }
    $search_use['tenantid'] = $foundTenant;
  } else
  {
    unset($search_use['tenant']);
  }
  
	if ($statcfg['type']=='ld')
	{
		// only this one starts with latest
		$statcfg['sortdir'] = 'down';
	}
	$statcfg['showtotcols'] = false;
	if (($statcfg['type']=='du'  || $statcfg['type']=='wt') && $statcfg['gran']!='x')
	{
		$statcfg['showtotcols'] = true;
	}
	
	if ($statcfg['type']=='un')
	{
		$agents = array();
		$users = $dbintf->GetUserList();
		foreach ($users as $user)
		{
			$agents[ $user['accid'] ] = $user['name'];
		}
		$statcfg['agents'] = $agents;
	}
	
}

function BuildTenantNameMap()
{
  global $dbintf;
  $tenantNameMap = array();
  $tenants = $dbintf->GetTenantList();
  foreach ($tenants as $tenant)
  {
    $tenantNameMap[ $tenant['id'] ] = $tenant['name'];
  }
  return $tenantNameMap;
}

function GetTotalHeader($statcfg)
{
	global $lng;
  $unit = GetStatUnit($statcfg);
	if ($statcfg['type']=='du' || $statcfg['type'] =='wt')
	{
		return $lng->Get('Main.All');
	}
	if ($statcfg['type']=='ch' || $statcfg['type']=='pc')
	{
		return $unit;
	}
	return $lng->Get('Stats.Total').' ['.$unit.']';
}

function GetLocalDetailsSubHeaders()
{
	global $lng;
	return array
	(		$lng->Get('Stats.numcalls')
	,		$lng->Get('Cols.length')
			. ' ('.$lng->Get('Stats.Total').')'
	,		$lng->Get('Cols.length')
				.' ('.$lng->Get('Stats.averaged').')'
	,		$lng->Get('Stats.FirstCall')
	,		$lng->Get('Stats.LastCall')
	);
}

?>