<?php
// reusuable part of track.php

require_once dirname(__FILE__)."/../apresa/conf/voip_config.php";
require_once 'db.php';
require_once "signaling.php";
require_once "datafile.php";
require_once "utility.php";
require_once('actions.php');


function GetSerialNumber()
{
	return trim(file_get_contents(dirname(__FILE__).'/../apresa/conf/serial'));
}

function HandleTracker()
{
	global $dbintf;
  $server = str_ensure_defined($_SERVER['REMOTE_ADDR']);

	if ($server !='127.0.0.1'
	&& $server !='')
	{
		echo 'Access denied';
		exit;
	}

	if (isset($dbintf)==false)
	{
		$dbintf = new DBInterface(dirname(__FILE__) . '/' . DATA_PATH, 'apresa', true);
		$dbintf->SetAdmin();
	}

  $dbintf->StartTransaction();
  HandleTrackerInT();
  $dbintf->CommitTransaction();
}

function CheckCallEncryption(&$data)
{
  global $dbintf;
  
  if($data['encrypt'] == 'true')
  {
    if(!isset($data['tenantid']))
    {
      // Engine made wrong decision.
      call_decrypt($data);
    }
    else
    {
      $group = $dbintf->GetGroupInfo( $data['tenantid'] );
      $key = $dbintf->GetCallEncryptionKey($group['accid']);
      if($key && $key['inuse'])
      {
        call_encrypt($data, $key);
      }
      else
      {
        // Engine made wrong decision or during recording encryption may have been disabled.
        call_decrypt($data);
      }
    }
  }
  else if(!($data['flags'] & 16)) 
  {
    // Can't perform per tenant encryption if old encryption method has already been applied.
    // It would reverse the order of encryption, which makes playback impossible.
    if(isset($data['tenantid']))
    {
      $group = $dbintf->GetGroupInfo( $data['tenantid'] );
      $key = $dbintf->GetCallEncryptionKey($group['accid']);
      if($key && $key['inuse'])
      {
        // Engine made wrong decision or during recording encryption may have been enabled, 
        call_fix_encrypt($data, $key);
      }
    }
  }
}

function call_fix_encrypt(&$data, $key)
{
  lg('Fix call encryption ' . $data['remoteid']);

  $pubKey = openssl_pkey_get_public(FormatPublicKey($key['pubkey']));

  if($pubKey === false)
  {
    lg('Could not load public key ' . $data['remoteid'] . openssl_error_string());
    return;
  }

  $key['importedKey'] = $pubKey;

  $in = GetRecordingPath($data) . $data['filename'];
  $out = GetRecordingPath($data) . $data['filename'] . '.enc' . uniqid();
  
  $ret = 0;
  $output = array();

  exec('apresa_crypt -ec -i ' . escapeshellarg($in) . ' -o ' . escapeshellarg($out), $output, $ret);

  if($ret != 0)
  {
    lg('Failed to perform call encryption ' . $data['remoteid']);
    return;
  }

  foreach($output as $line)
  {
    if(strpos($line, "Key: ") !== false)
    {
      $callkey = substr($line, 5);
    }

    if(strpos($line, "IV: ") !== false)
    {
      $iv = substr($line, 4);
    }
  }

  if(!isset($callkey))
  {
    lg('Failed to get encryption key from apresa_crypt ' . $data['remoteid']);
    return;
  }

  if(!isset($iv))
  {
    lg('Failed to get IV from apresa_crypt ' . $data['remoteid']);
    return;
  }

  $data['key'] = $callkey;
  $data['iv'] = $iv; 

  unlink($in);
  rename($out, $in);
  chmod($in, 0777);

  call_encrypt($data, $key);
}

function call_decrypt(&$data)
{
  lg('Decrypt the call ' . $data['remoteid']);

  $in = AUDIO_PATH . '/' . $data['directory'] . '/' . $data['filename'];
  $out = AUDIO_PATH . '/' . $data['directory'] . '/' . $data['filename'] . '.dec';
  $cmd = 'apresa_crypt -dc -i ' . escapeshellarg($in) . ' -o ' . escapeshellarg($out) . ' -key ' . escapeshellarg($data['key']) . ' -iv ' . escapeshellarg($data['iv']);

  $ret = 0;
  $output = array();

  exec($cmd, $output, $ret);

  if($ret != 0)
  {
    lg('Failed to perform call decryption ' . $data['remoteid']);
    return;
  }

  $data['flags'] &= ~2048;
  
  unlink($in);
  rename($out, $in);
  chmod($in, 0777);
}

function call_encrypt(&$data, $key)
{
  lg('Encrypt the key ' . $data['remoteid']);

  if(isset($key['importedKey']))
  {
    $pubKey = $key['importedKey'];
  }
  else
  {
    $pubKey = openssl_pkey_get_public(FormatPublicKey($key['pubkey']));

    if($pubKey === false)
    {
      lg('Could not load public key ' . $data['remoteid'] . openssl_error_string());
      return;
    }
  }

  $callkey = hex2bin($data['key']);
  $iv = hex2bin($data['iv']);

  if(openssl_public_encrypt($callkey, $encryptedKey, $pubKey, OPENSSL_PKCS1_OAEP_PADDING) === false)
  {
    lg('Failed to encrypt the call key ' . $data['remoteid']);
    return;
  }

  $data['encryption_key'] = [
    "key" => base64_encode($encryptedKey),
    "iv" => base64_encode($iv),
    "priv_id" => $key['id']
  ];

  $data['flags'] |= 2048;
}

function HandleTrackerInT()
{
	global $dbintf;
	global $OutgPrefix;
	global $serialnum;
	//global $SIPgate;
	global $FullSipId;
	global $MaskLastRemoteDigits;
	// to save DB queries from subsequent runs:
	global $lastDbSync;
  global $SpinClient;
	global $configs_o, $configs_s, $configs_tn;
  global $configs;
	
	if (isset($lastDbSync)==false)
	{
		$lastDbSync = 0;
	}
	$now = time();
	if ($now - $lastDbSync >= 1)
	{
		//lg("tracker reloads config ".$now);
		$configs_s = $dbintf->LoadConfigsPart('s');
		$configs_o = $dbintf->LoadConfigsPart('o');
		$configs_tn = $dbintf->LoadConfigsPart('tn');
		$lastDbSync = $now;
	} else
	{
		//lg("tracker uses cached config ".$now);
	}
	$OutgPrefix = str_ensure_defined($configs_s['OutgPrefix']);
	$MaskLastRemoteDigits = int_ensure_defined($configs_s['MaskLastRemoteDigits']);
	$ALE = int_ensure_defined($configs_s['ALE']);
	$RecMinDur  = int_ensure_defined($configs_o['RecMinDur']);
	$RecLocal   = ensure_defined($configs_o['RecLocal'], 1);
	$RecNotOutg = int_ensure_defined($configs_o['RecNotOutg']);
	$RecNotInc  = int_ensure_defined($configs_o['RecNotInc']);
	//$NotRecEmptyNum = int_ensure_defined($configs_o['NotRecEmptyNum']);
	$RecOnDemand = int_ensure_defined($configs_o['RecOnDemand']);
	$CalcChecksum = str_ensure_defined($configs_o['CalcChecksum']);
	$FullSipId = int_ensure_defined($configs_o['FullSipId']);
	//$SIPgate    = $configs_o['SIPgate'];
  $SpinClient = int_ensure_defined($configs_s['SpinClient']);

	$serialnum = GetSerialNumber();

	// type can be:
	// - init
	// - tape
	// - dtmf
	// - info

	// stage can be:
	// - start (call started)
	// - update (new call information)
	// - stop (call finished)
	// - ready (converting to audio format)

	/*
	Is done by startup.php and better
	if ($_GET['type']=='init')
	{
		$dbintf->ClearActiveCalls();
	}
	*/

	if ($_GET['type']=='tape')
	{
		if ($_GET['stage']=='start')
		{
			$data = FillData();
			CheckDetermineThings($data,$_GET['stage']);
			$data['isactive'] = 1;
			$alreadydata = $dbintf->GetActiveCallInfoRM($data['remoteid']);
			if ($alreadydata!=null)
			{
				lg('Active call was already in list');
				// already handled
				echo 'class=taperesponse success=true comment=OK';
				return;
			}
			// Also check if in the normal call listing.
			// The reason is, messages can be delayed and repeated.
			// This is to prevent an active call that would never end.
			$alreadydata = $dbintf->GetCallInfoRM($data['remoteid']);
			if ($alreadydata!=null)
			{
				lg('Active call already ready');
				// already handled
				echo 'class=taperesponse success=true comment=OK';
				return;
			}
      ApplyActionCache($data);
      NotifySpinClient( 'start' , $data );
      UnsetNotForDatabase( $data );
			$dbintf->InsertActiveCall($data);
			echo 'class=taperesponse success=true comment=OK';
		} else if ($_GET['stage']=='update')
		{
			$newdata = FillData();
			unset($newdata['callextra']);
      NotifySpinClient( 'update' , $newdata );
      UnsetNotForDatabase( $newdata );
			$dbintf->UpdateActiveCall($newdata['remoteid'],$newdata);
			echo 'class=taperesponse success=true comment=OK';
		} else if ($_GET['stage']=='stop')
		{
			$remoteid = $_GET['recid'];
			// do not remove yet: $dbintf->RemoveActiveCall($remoteid);
			$dbintf->SetActiveProcessingFlag($remoteid,4);
      NotifySpinClient( 'stop' , array('id'=>$remoteid) );
			echo 'class=taperesponse success=true comment=OK';
		} else if ($_GET['stage']=='ready' || $_GET['stage']=='recover')
		{
			// 'ready' is the normal stage when conversion of audio is completed
			$recover = ($_GET['stage']=='recover');
			// 'recover' is similar to ready.
			// It means the node was standby, but has become active.
			// This means the call is not in the active calls.
			$dur = $_GET['duration'];
			$recid = $_GET['recid'];
			$alreadydata = $dbintf->GetCallInfoRM($recid);
			if ($alreadydata!==false)
			{
				lg('Call already in list '.$recid.' ID: '.$alreadydata['id'].' filename: '.$alreadydata['filename']);
				// already handled
				echo 'class=taperesponse success=true comment=OK';
				return;
			}
			if ($recover)
			{
				$data = FillData();
				//$data['notes'] = 'recovered';
				$newdata = $data;
				lg('Recover mode '.$data['remoteid']);
			} else
			{
				$data = $dbintf->GetActiveCallInfoRM($recid);
				$newdata = FillData();
			}
			if($newdata['flags']&32)
			{
				$data = $newdata;
			}
      
			if ($dur<$RecMinDur && !($newdata['flags']&64))
			{
				//verbose: lg('Call too short: ' . $dur);
        NotifySpinClient( 'delete' , array('id'=>$recid) );
				$dbintf->RemoveActiveCall($recid);
				//verbose: lg('Call too short');
				echo 'class=taperesponse success=true comment=OK deletetape=true';
				return;
			}
			
			
			if ($data)
			{
        // due to race condition, it can be we have not applied action cache during start,
        // so then apply it when call has finished.
        ApplyActionCache($data);
        
				$oldid = $data['id'];
				$callextra = StringToArray($data['callextra']);
				CheckDetermineThings($data,$_GET['stage']);
				if (isset($data['tenantid']))
				{
					$prfx = 'TN' . $data['tenantid'] . '_';
					$key = $prfx . 'RecMinDur';
					if (isset($configs_tn[$key]))
					{
						if ($configs_tn[$key]!='')
						{
							$RecMinDur = (int) $configs_tn[$key];
							// verbose: lg('Using tenant RecMinDur: '.$RecMinDur);
						}
					}
				}
				$to_keep = true;
        $pendingCall = false;
				// Check if store on demand is enabled.
				// It can be set globally.
				$users = GetUsers($data);
				foreach ($users as $user)
				{
					// or for this specific user
					if (int_ensure_defined($user['rod'])==1) $RecOnDemand = true;
				}
				// store-on-demand Off takes precedence,
				// so it is checked later.
				foreach ($users as $user)
				{
					// or for this specific user
					if (int_ensure_defined($user['rod'])==2) $RecOnDemand = false;
				}
				if ($RecOnDemand && !isset($callextra['to-keep']))
				{
					if ( int_ensure_defined($configs_o['StoreOnDemandDelay']) > 0 )
					{
						lg('Delayed store on demand ' . $data['remoteid'] );
            $pendingCall = true;
					} else
					{
					lg('Do not keep this call (only store on demand)' . ' ' . $data['remoteid'] );
					$to_keep = false;
				}
				}
				if ($data['direction']==2 && $RecLocal==0)
				{
					lg('Do not save local call');
					$to_keep = false;
				}
				if ($data['direction']==1 && $RecNotOutg==1)
				{
					lg('Do not save outgoing call');
					$to_keep = false;
				}
				if ($data['direction']==0 && $RecNotInc==1)
				{
					lg('Do not save incoming call');
					$to_keep = false;
				}
        if(int_ensure_defined($callextra['storeondemand']) == 1 && !$callextra['to-keep'])
        {
          lg('Do not save this call that was tagged store on demand' . ' ' .  $data['remoteid']);
          $to_keep = false;
        }
				//if ($data['remotefull']=='' && $NotRecEmptyNum)
				//{
				//  lg('Do not save call with empty remote number');
				//	$to_keep = false;
				//}
				if ($to_keep==false)
				{
          NotifySpinClient( 'delete' , array('id'=>$recid) );
					$dbintf->RemoveActiveCall($recid);
					echo 'class=taperesponse success=true comment=OK deletetape=true';
					return;
				}
				unset($data['id']);
				// everything to copy:
				// fix incorrect duration
				// filename gets extra .wav, so reevaluate
				$colcopylist = array('duration','durms','filename'
					, 'isarchived', 'key', 'iv', 'encrypt');
				foreach ($colcopylist as $colcopy)
				{
					$data[$colcopy] = ensure_defined($newdata[$colcopy], null);
				}

				/*
				// copy also latest remote
				$data['remote'] = $newdata['remote'];
				$data['remote2'] = $newdata['remote2'];
				$data['remotefull'] = $newdata['remotefull'];
				$data['remotecontact'] = $newdata['remotecontact'];
				$data['remotecallerid'] = $newdata['remotecallerid'];
				*/

				if($newdata['flags'] & 64)
				{
					// Missed call, so file does not exist
					$data['flags'] |= 64;
					$data['isarchived'] = 0;
					$data['duration'] = -1;
					$data['isconnected'] = 0;
				}
				else if($data['flags'] & 32)
				{
					// Channel limit reached, remove file
					$data['isarchived'] = 0;
					DeleteFn(AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
					$rawfn = change_filename_extension($data['filename'],'.mcf');
					DeleteFn(AUDIO_PATH.'/'.$data['directory'].'/'.$rawfn);
				}
        // Next block is to solve the problem that after failover,
        // recording on demand calls would be added to the listing (with no audio).
        // The reason is that the type:not msg has not removed the active call,
        // because there is no active call anyway during recover.
        // The condition $recover is added for safety,
        // but not seen a case in which it is needed.
        else if ($data['isarchived']==0 && $recover)
        {
          lg( "Call was not to be recorded " . $recid );
          NotifySpinClient( 'delete' , array('id'=>$recid) );
          EraseRecording( $data );
					$dbintf->RemoveActiveCall($recid);
					echo 'class=taperesponse success=true comment=OK deletetape=true';
					return;
        }
				else
				{
					// split in directory and file name
					$slashpos = strrpos($data['filename'],'/');
					if ($slashpos!==FALSE)
					{
						$data['directory'] = substr($data['filename'],0,$slashpos);
						$data['filename'] = substr($data['filename'],$slashpos+1);
					}
					$exists = file_exists(AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
					if (!$exists)
					{
						// try using raw mcf file
						$rawfn = change_filename_extension($data['filename'],'.mcf');
						if (file_exists(AUDIO_PATH.'/'.$data['directory'].'/'.$rawfn))
						{
							$data['filename'] = $rawfn;
							$data['flags'] |= 8;
						}
					}
					$exists = file_exists(AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
					if ($exists)
					{
						// Duplicate code in specialtask.php ActiveDone
            // Calculate checksum if enabled
						if ($CalcChecksum=="md5")
						{
							$callextra['chk'] = md5_file(AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
							$callextra['chktype'] = 'md5';
						}
						if ($CalcChecksum=="sha1")
						{
							$callextra['chk'] = sha1_file(AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
							$callextra['chktype'] = 'sha1';
						}
						if ($CalcChecksum=="sha256")
            {
              $callextra['chk'] = calcChecksum('sha256sum',AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
              $callextra['chktype'] = 'sha256';
            }
						if ($CalcChecksum=="sha512")
            {
              $callextra['chk'] = calcChecksum('sha512sum',AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
              $callextra['chktype'] = 'sha512';
            }
						if (isset($data['tenantid']))
						{
							require_once 'tenancy.php';
							// update tenant disk space usage
              UpdateTnDiskUsage( $data['tenantid'], filesize(AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']), +1 );
            }

            CheckCallEncryption($data);
					} else
					{
						// file is not there
						$data['isarchived'] = 0;
					}
				}
				
				/*if($newdata['flags']&256)
				{
					$data['local'] = $newdata['local'];
					$data['flags'] |= 256;
				}*/
				
				
				$data['callextra'] = ArrayToString($callextra);
				// no longer active
				$data['isactive'] = 0;
				// remove audio data processing flag
				$data['processingstate'] &= ~ 4;
				// apply idmap
				$idmapentry = $dbintf->GetIdMapEntry($data['local']);
				if ($idmapentry!==false)
				{
					if ($idmapentry['target']=='notes'
					|| $idmapentry['target']=='notes2')
					{
						$data[ $idmapentry['target'] ] = $idmapentry['value'];
					}
        }

        if( $data['flags'] & 2048 )
        {
          $encrytionKey = $data['encryption_key'];
        }

        if ($pendingCall)
        {
          // for delayed store on demand
          // this call would normally be deleted
          // we keep it for a while
        UnsetNotForDatabase( $data );
          $data['processingstate'] |= 128; // pending flag
          // we keep it in ActiveCalls to avoid:
          // - post processing such as export
          // - display in call listing
          $dbintf->UpdateActiveCall( $data['remoteid'], $data );
          if(isset($encrytionKey))
          {
            $dbintf->AddActiveCallKey(
              [
                "callid" => $oldid,
                "keyid" => $encrytionKey['priv_id'],
                "callkey" => $encrytionKey['key'],
                "iv" => $encrytionKey['iv']
              ]
              );
          }
          
        } else {
          // normal flow
          
          //lg('Adding call for filename: '.$data['filename']);

          // do we need to tell? NotifySpinClient( 'ready' , array('id'=>$recid) );

          UnsetNotForDatabase( $data );
        $newid = $dbintf->InsertCall($data);
          $data['id'] = $newid;
        
        if(isset($encrytionKey))
        {
          $dbintf->AddCallKey(
            [
              "callid" => $newid,
              "keyid" => $encrytionKey['priv_id'],
              "callkey" => $encrytionKey['key'],
              "iv" => $encrytionKey['iv']
            ]
            );
        }

				$dbintf->MoveActiveAnnotations($newid, $oldid);
				$dbintf->RemoveActiveCall($recid);
        
          CallPostProcessing( $data );
					}

			} else
			{
        // this is not an error
        // it can happen after type:not
				// lg('Could not find matching active call '.$recid);

				$data = $newdata;
        EraseRecording( $data );
        
			}


			// react in a certain way, otherwise it is repeated,
			// and will get repeated in the database
			echo 'class=taperesponse success=true comment=OK deletetape=false';
		} else
		{
			echo 'class=taperesponse success=true comment=OK';
		}
	} else if ($_GET['type']=='init')
	{
		echo 'class=simpleresponse success=true comment=OK';
  } else if ($_GET['type']=='crec')
  {
    $id = $_GET['id'];
    $data = $dbintf->GetCallInfo($id);
    if ($data===false)
    {
      echo 'result=CallNotFound';
    } else
    {
      CallPostProcessing( $data );
      echo 'result=OK';
    }
  } else if ($_GET['type']=='call')
  {
    $data = FillCallData();
    $id = $_GET['id'];
    $phase = (int) $_GET['phase'];
    if ($phase==2) // end of call
    {
      $dbintf->DelLiveCall($id);
    } else if ($phase==0) // new call
    {
      $dbintf->AddLiveCall($id,$data);
    } else // edit call
    {
      $dbintf->EditLiveCall($id,$data);
    }
    echo 'class=simpleresponse success=true';
	} else if ($_GET['type']=='dtmf')
	{
		$event = $_GET['event'];
		$parts = explode(' ',$event);
		$data = array();
		$data['dtmf'] = (int) $parts[0];
		array_splice($parts,0,1);
		foreach ($parts as $elem)
		{
			$kv = explode(':',$elem);
			$data[ trim($kv[0]) ] = trim($kv[1]);
		}
    $side = $data['init'];
		$new_digit = ConvDtmf($data['dtmf']);

		// lookup call based on recid
		$recid = $_GET['recid'];
		$callinfo = $dbintf->GetActiveCallInfoRM($recid);
		$callextra = array();
		if ($callinfo!=null)
		{
			$callextra = StringToArray($callinfo['callextra']);
			// add digit to dtmf
			$callextra['dtmf_' . $side] = ($callextra['dtmf_' . $side] ?? '') . $new_digit;
      $callextra['dtmf'] = ($callextra['dtmf'] ?? '') . $new_digit;
			//verbose lg('DTMF received: '.$new_digit.' recid: '.$recid.' string: '.$callextra['dtmf']);
			// lookup if there is registered action
			$action = '';
			$input_dtmf = $callextra['dtmf_' . $side];
			//echo $input_dtmf;
			for ($ix=1; $ix<10; $ix++)
			{
				if (isset($configs_s['DTMF'.$ix]))
				{
					if (substr($input_dtmf,-strlen($configs_s['DTMF'.$ix])) === $configs_s['DTMF'.$ix])
					{
						// last part matches, so do it!
						$action = $configs_s['DTMFact'.$ix];
					}
				}
			}
			if ($action!='')
			{
				lg('DTMF Action to perform: '.$action . ' for ' . $recid);
				// perform the action
				// preferably should not be done here
				// because it means it cannot be controled by API
				if ($action=='keep')
				{
					$callextra['to-keep'] = '1';
				}
				if ($action=='start+email')
				{
					// mark that this call must be emailed when finished
					$callextra['to-email'] = $side;
				}
				if ($action=='email')
				{
					// mark that this call must be emailed when finished
					$callextra['to-email'] = $side;
				}
				if ($action=='email2')
				{
					EmailRecording($callinfo,1,$side);
				}
			}
			$newdata = array('callextra' => ArrayToString($callextra) );
			$dbintf->SetActiveCallInfoRM($recid,$newdata);
      NotifySpinClient( 'set' , array_merge( array('id'=>$recid) , $newdata ) );
			echo 'class=simpleresponse success=true comment=OK';
		} else {
			echo 'class=simpleresponse success=false comment=UnknownCall';
		}
	} else if ($_GET['type']=='info')
	{
		$msg = $_GET['msg'];
		$parts = explode(' ',$msg);
		$data = array();
		$startdate = $_GET['timestamp'];
		foreach ($parts as $elem)
		{
			$kv = explode(':',$elem);
			$data[ trim($kv[0]) ] = trim($kv[1]);
		}
    if ($data['type']=='heartbeat')
    {
      echo 'class=simpleresponse success=true';
      // Check if anything in action cache
      TrackerHeartBeat();
    }
    else if ($data['type']=='chlimit')
		{
			echo 'class=simpleresponse success=true';
			$dbintf->AddLicNotRec($startdate);
			// Channel type, empty for "normal".
			$channeltype = str_ensure_defined($data['channel']); 
			// Sometimes channel further specified with ID, like vtap
			$channelid = str_ensure_defined($data['id']); 
			
			
			$LastLicOverflowKey = 'LastLicOverflow' . (strlen($channeltype) > 0 ? '_' . $channeltype : '') . (strlen($channelid) > 0 ? '_' . $channelid : '');
			$LastLicOverflow = (int) $dbintf->GetConfigValue($LastLicOverflowKey);
			$dbintf->SetConfigsPart( array($LastLicOverflowKey=>time()), 'dat' );
			
			$NoNotifyLicLimit = (int) $dbintf->GetConfigValue('NoNotifyLicLimit');
			if ($NoNotifyLicLimit==0)
			{
				$LastLicOverflowNotifKey = "Notif_ " . $LastLicOverflowKey;
				$LastLicOverflowNotif = (int) $dbintf->GetConfigValue($LastLicOverflowNotifKey);
				if (time() - $LastLicOverflowNotif > 24*60*60)
				{
					$cmd = 'licnotrec' . (strlen($channeltype) > 0 ? ' ' . $channeltype : '') . (strlen($channelid) > 0 ? ' ' . $channelid : '');
					signal_service($cmd);
					$dbintf->SetConfigsPart( array($LastLicOverflowNotifKey=>time()), 'dat' );
				}
			}
		} 
		else if ($data['type']=='delete')
		{
			$remoteid = $data['recid'];
			lg('delete on demand, recid: '.$remoteid);
      NotifySpinClient( 'delete' , array('id'=>$remoteid) );
			$dbintf->RemoveActiveCall($remoteid);
			echo 'class=simpleresponse success=true';
		} else if ($data['type']=='totalcalls')
		{
			echo 'class=simpleresponse success=true';
			$number = $data['number'];
			// too verbose: lg('Update Max Calls to: '.$number);
			$dbintf->UpdateMaxCalls($number, $startdate);
		} else if ($data['type']=='not')
		{
			$remoteid = $data['recid'];
			lg('do not record, recid: '.$remoteid);
      NotifySpinClient( 'delete' , array('id'=>$remoteid) );
			$dbintf->RemoveActiveCall($remoteid);
			echo 'class=simpleresponse success=true';
		} else if ($data['type']=='act')
		{
			$act = $data['act'];
      if ($act=='stop' || $act=='start'
      || $act=='silence' || $act=='sound'
      || $act=='delete' || $act=='email' || $act=='email2'
      || $act=='annotation' || $act=='keep'
			|| $act=='remrec' || $act=='remsil'
      )
      {
        // this block is for audit trail logging
        $configs = $dbintf->LoadConfigsPart('web');
        if ( int_ensure_defined($configs['Audit'])==1
        || int_ensure_defined($configs['AuditDb'])==1 )
        {
          $local = $data['l'];
          $remote = $data['r'];
          $initiator = $data['i'];
          $phone = '';
          if ($initiator=='L')
          {
            $initiator = 'local side';
            $phone = $local;
          } else if ($initiator=='R')
          {
            $initiator = 'remote side';
            $phone = $remote;
          }
          // Find out the user of phone
          $useraccid = false;
          if ($phone!='')
          {
            $phoneLookup = GetSipIdToUse($phone,$FullSipId);
            $cid = $dbintf->GetCallerIDonCallerID($phoneLookup,false);
            if ($cid !== false)
            {
              $users = $dbintf->GetUsersOfPhonesFl($cid['contactid'],true);
              foreach ($users as $accid)
              {
                $useraccid = $accid;
                break;
              }            
            }
          }
          $usrnm = '';
          if ($useraccid !== false)
          {
            $userinfo = $dbintf->GetUserInfoOnAccId( $useraccid );
            $usrnm = $userinfo['usr'];
          }
          if ($usrnm=='') $usrnm = '_system_';
          
          $descr = GetAuditActionDescr($act);
          LogAction($descr,array('local'=>$local,'remote'=>$remote,'actor'=>$initiator),$usrnm);
        }
      } 
      if ($act=='silence' || $act=='sound' || $act=='keep')
      {
        $recid = $data['recid'];
        if ($recid!='')
        {
          $callinfo = $dbintf->GetActiveCallInfoRM($recid);
          $callextra = array();
          if ($callinfo!=null)
          {
            $callextra = StringToArray($callinfo['callextra']);
          }
          if ($act=='silence')
          {
            $callextra['silencing'] = '1';
          }
          if ($act=='sound')
          {
            $callextra['silencing'] = '0';
          }
          if ($act=='keep')
          {
            $callextra['to-keep'] = '1';
          }
          $newdata = array('callextra' => ArrayToString($callextra) );
          $dbintf->SetActiveCallInfoRM($recid,$newdata);
          NotifySpinClient( 'set' , array_merge( array('id'=>$recid) , $newdata ) );
        }
        lg('ACT '.$act.' '.$recid);
      }
			echo 'class=simpleresponse success=true';
		}
		else if($data['type']=='annotation')
		{
			$recid = $data['recid'];
			$data['descr'] = urldecode($data['descr']);
			$callinfo = $dbintf->GetActiveCallInfoRM($recid);

			lg('New annotation for ' . $recid . ' at ' . $data['timepos'] . ' seconds');
			if($callinfo != null)
			{
				// Still active call
				$dbintf->AddActiveAnnotation($callinfo['id'], $data, $startdate);
				echo 'class=simpleresponse success=true';
				return;
			}

			$callinfo = $dbintf->GetCallInfoRM($recid);
			if($callinfo != null)
			{
				// Call already finished
				$dbintf->AddFinishedAnnotation($callinfo['id'], $data, $startdate);
				echo 'class=simpleresponse success=true';
				return;
			}
			
			lg("Unknown call for annotation");
			echo 'class=simpleresponse success=true comment=UnknownCall';
		}
		else if($data['type'] == 'toshiba_nr')
		{
			$res = $dbintf->ToshibaSMDRUpdateNr($data['local'], $data['remote'], $data['direction'], $data['node_id']);
			if(!$res)
			{
				lg('Could not update local number with toshiba SMDR');
			}
			else
			{
				lg('Toshiba SMDR - Updated call ' . $res['remoteid'] . ' local number ' . $res['local'] . ' to ' . $data['local']);
			}
      echo 'class=simpleresponse success=true';
		}
		else if($data['type'] == 'toshiba_connected')
		{
			$res = $dbintf->ToshibaSMDRUpdateConnected($data['local'], $data['remote'], $data['direction'], $data['connected'], $data['node_id']);
			if(!$res)
			{
				lg('Could not update connected number with toshiba SMDR');
			}
			else
			{
				lg('Toshiba SMDR - Updated call ' . $res['remoteid'] . ' connected number to ' . $data['connected']);
			}
      echo 'class=simpleresponse success=true';
		}
		else if($data['type'] == 'toshiba_nodeid')
		{
			lg('Recid ' . $data['recid'] . ': Setting toshiba SMDR node id to ' . $data['node_id']);
			$recid = $data['recid'];
			$callinfo = $dbintf->GetActiveCallInfoRM($recid);
			if ($callinfo!=null)
			{
				$callextra = StringToArray($callinfo['callextra']);
				$callextra['toshiba_nodeid'] = $data['node_id'];
				$newdata = array('callextra' => ArrayToString($callextra) );
				$dbintf->SetActiveCallInfoRM($recid,$newdata);
        NotifySpinClient( 'set' , array_merge( array('id'=>$recid) , $newdata ) );
			}
			else
			{
				lg('Could not find ' . $data['recid'] . ' in active calls for settings Toshiba SMDR node id');
			}
      echo 'class=simpleresponse success=true';
		}
		/*else if($data['type']=='notstarted')
		{
			// Add missed call info to database
			foreach($data as $key=>$value)
			{
				$_GET[$key] = $value;
			}
			$call = FillData();
			$call['duration'] = -1;
			$call['flags'] |= 64;
      UnsetNotForDatabase( $data );
			$dbintf->InsertCall($call);
			echo 'class=simpleresponse success=true';
		}*/ 
		else
		{
			echo 'class=simpleresponse success=true comment=Ignored';
		}
	} else {
		echo 'class=simpleresponse success=false comment=TypeNotSupported';
	}
}

function EraseRecording($data)
{        
  global $dbintf;
  // throw away
  // needed after type:not
  DeleteFn(AUDIO_PATH.'/'.$data['directory'].'/'.$data['filename']);
  $rawfn = change_filename_extension($data['filename'],'.mcf');
  DeleteFn(AUDIO_PATH.'/'.$data['directory'].'/'.$rawfn);
  // when clustering, and the node is passive, then it is stored in another directory
  $dbintf->DetermineSystemSetup();
  if ($dbintf->clustering)
  {
    DeleteFn('/database/standbyrecs/'.$data['directory'].'/'.$rawfn);
  }
}

function ClearRemoteDialPrefix(&$tel)
{
  global $OutgPrefix;
  if (IsTelNumber($tel)==false) return false;
  if (strpos($tel,$OutgPrefix)===0) // first character match
  {
    $tel = substr($tel,strlen($OutgPrefix));
    return true;
  }
  return false;
}

function UnquoteTag($value)
{
	$v = $value;
	// UnEscapeCVS
	$v = str_replace( '%c', ',', $v );
	$v = str_replace( '%p', '%', $v );
	// UnEscapePair
	$v = str_replace( '%k', ':', $v );
	$v = str_replace( '%p', '%', $v );
	return $v;
}

function MaskApply($party)
{
	global $MaskLastRemoteDigits;
	// no digits to mask?
	if ($MaskLastRemoteDigits==0) return $party;
	
	// find position of @
	// we change things before the first at-sign
	$atpos = strpos($party,'@');
	if ($atpos===false)
	{
		$len = strlen($party);
	} else
	{
		$len = $atpos;
	}
	// only first part is changed
	$part1 = substr($party,0,$len);	
	$part2 = substr($party,$len);
	// mask X digits of part1
	$unmod_count = $len - $MaskLastRemoteDigits;
	// but not if part1 has less digits
	if ($unmod_count<0) $unmod_count=0;
	$mod_count = $len - $unmod_count;
	// the unmodified part of part 1
	$newpart1 = substr($part1,0,$unmod_count);
	// the modified part
	for ($i=0;$i<$mod_count;$i++)
	{
		$newpart1 .= 'X';
	}
	// stitch together
	return $newpart1 . $part2;
}

function UnsetNotForDatabase(&$ref_data)
{
  $to_unset = array( 'orktenant'
  , 'local_cid'
  , 'remote_cid'
  , 'connected_cid'
  , 'recorder_sn'
  , 'recorder_name'
  , 'key'
  , 'iv'
  , 'encrypt'
  , 'encryption_key'
  );
  foreach ($to_unset as $key)
  {
    unset( $ref_data[$key] );
  }
}

function NotifySpinClient($action,$data)
{
  global $dbintf, $SpinClient, $serialnum;
  if ($SpinClient != 1) return;
  $data['action'] = $action;
  if ($action=='start')
  {
    $data['local_name'] = $dbintf->GetContactName( $data['localcontact'] );
    $data['remote_name'] = $dbintf->GetContactName( $data['remotecontact'] );
    if (isset($data['connectedcontact']))
    {
      $data['connected_name'] = $dbintf->GetContactName( $data['connectedcontact'] );
    }
    if (isset($data['tenantid']))
    {
      $group = $dbintf->GetGroupInfo( $data['tenantid'] );
      $data['tenant_name'] = $group['name'];
    }
    
    $to_unset = array( 'filename','directory','processingstate'
    // pointers to other tables (useless on other side):
		, 'recorderid'
		, 'localcallerid'
		, 'remotecallerid'
		, 'connectedcallerid'
		, 'localcontact'
		, 'remotecontact'
		, 'connectedcontact'
		, 'tenantid'
    , 'key'
    , 'iv'
    , 'encrypt'
    , 'encryption_key'
    , 'agent'
    );
    foreach ($to_unset as $key)
    {
      unset( $data[$key] );
    }
  }
  if ($action=='delete' || $action=='stop' || $action=='set')
  {
    // lookup call
    $callinfo = $dbintf->GetActiveCallInfoRM( $data['id'] );
    // if call already gone, forget about it
    if ($callinfo===null || $callinfo===false) return;
    // lookup recorder
    $recorder = $dbintf->GetRecorderById( $callinfo['recorderid'] );
    $data['recorder_sn'] = $recorder['recid'];
  }
  // not anymore: $data['sn'] = $serialnum;
  $reply = SendJsonToDomainSocket( $data , '/tmp/apresa_spin' );
}

function DetermineTenant(&$ref_data)
{
	global $dbintf, $configs_s;
  $TenantPhonesOnlyEnabled = int_ensure_defined( $configs_s['TenantPhonesOnlyEnabled'] );
	$accids = $dbintf->GetUsersOfCall($ref_data,$TenantPhonesOnlyEnabled!=0);
  // first determine all matching tenants
  $tenants = array();
  $tenantList = array();
	foreach($accids as $accid)
	{
		$userinfo = $dbintf->GetUserInfoOnAccId($accid);
		if ($userinfo['tenantid']!==null)
		{
      $tenantid = $userinfo['tenantid'];
      $tenants[ $tenantid ] = true;
      $tenantList[] = $tenantid;
		}
	}
  // if there are no tenants for this call
  if (count($tenants)==0) return;
  // if there are multiple tenants that match
  if (count($tenants)>1)
  {
    // and voip engine has determined a tenant
    if (isset($ref_data['orktenant']))
    {
      // and this tenant is a valid option for this call
      $orkTenantid = (int) $ref_data['orktenant'];
      if (isset($tenants[ $orkTenantid ]))
      {
        // then assign the call to this tenant
        $ref_data['tenantid'] = $orkTenantid;
        return;
      }
    }
  }
  // otherwise simply use first option (as done in previous versions)
  $tenantid = $tenantList[0];
	$ref_data['tenantid'] = $tenantid;
	return;
}

function DetermineTenantFromVTapID(&$ref_data)
{
	global $dbintf;
  $tmpvar = StringToArray($ref_data['callextra']);
	$vtap_id = $tmpvar['TunnelID'];
	
	$vtap_info = $dbintf->GetVTap($vtap_id);	
	if($vtap_info)
	{
		if(isset($vtap_info['tenantid']))
		{
			$ref_data['tenantid'] = $vtap_info['tenantid'];
			return true;
		}
		else
		{
			return false;
		}
	}
	return false;
}

// a/b: phone numbers to see if equivalent
// difmax: maximum number of unmatched characters
function IsNumMatch($a,$b,$difmax)
{
	if ($a==$b) return true;
	if (endsWith($a,$b)!==false)
	{
		// b is found in a
		$dif = strlen($a) - strlen($b);
    return ($dif <= $difmax);
	}
	if (endsWith($b,$a)!==false)
	{
		// a is found in b
		$dif = strlen($b) - strlen($a);
    return ($dif <= $difmax);
	}
  return false;
}

// check if call description matches call data
function CallMatches( $callData, $row )
{
  global $dbintf;
  
  $callDescr = StringToArray( $row['action'] );
  // debug: lg("Check if new call matches description: " . var_export($callDescr,true) );
  
  $tomatch = 0;
  if (isset($callDescr['localnum'])) $tomatch ++;
  if (isset($callDescr['remotenum'])) $tomatch ++;
  if (isset($callDescr['linenr'])) $tomatch ++;
  if (isset($callDescr['id'])) $tomatch ++;
  if (isset($callDescr['nci'])) $tomatch ++;
  $contacts = null;
  if (isset($callDescr['usr']))
  {
    $usr = $callDescr['usr'];
    $tomatch++;
    $userinfo = $dbintf->GetUserInfoOnUsername( $usr );
    if ($userinfo===false)
    {
      $ret['ret'] = '102 User account not found';
      return $ret;
    }
    $contacts = $dbintf->GetUserPhones($userinfo['accid']);
  } else if (isset($callDescr['email']))
  {
    $email = $callDescr['email'];
    $tomatch++;
    $userinfo = $dbintf->GetUserInfoOnEmail( $email );
    if ($userinfo===false)
    {
      $ret['ret'] = '102 User account not found';
      return $ret;
    }
    $contacts = $dbintf->GetUserPhones($userinfo['accid']);
  } 
  
  if ($tomatch==0)
  {
    // something wrong
    return false;
  }

  $remotenumdifmax = 0;
  $localnumdifmax = 0;
  if (isset($callDescr['remotenumdifmax']))
  {
    $remotenumdifmax = (int) $callDescr['remotenumdifmax'];
  }
  if (isset($callDescr['localnumdifmax']))
  {
    $localnumdifmax = (int) $callDescr['localnumdifmax'];
  }
  
  $match = 0;
	if (isset($callDescr['localnum']))
	{
    if (IsNumMatch($callData['local'],$callDescr['localnum'],$localnumdifmax)) $match ++;
	}
	if (isset($callDescr['remotenum']))
	{
    if (IsNumMatch($callData['remote'],$callDescr['remotenum'],$remotenumdifmax)) $match ++;
	}
	if (isset($callDescr['linenr']))
	{
    if ($callData['linenr']==$callDescr['linenr']) $match++;
	}
	if (isset($callDescr['id']))
	{
    if ($callData['remoteid']==$callDescr['id']) $match++;
	}
  if (isset($callDescr['nci']))
  {
    $callextra = StringToArray($callData['callextra']);
    if (isset($callextra['NativeCallID']))
    {
      if ($callextra['NativeCallID']==$callDescr['nci']) $match++;
    }
  }
  if ($contacts!==null)
  {
    foreach ($contacts as $contactid)
    {
      if ($callData['localcontact']==$contactid
      || $callData['remotecontact']==$contactid)
      {
        // additional tenant check if assigned to other tenant
        if ($userinfo['tenantid']!==null
        // we allow matching with a call without tenant,
        // because the tenant is sometimes assigned later
        && $callData['tenantid']!==null
        // the tenant should be the same
        && $callData['tenantid']!=$userinfo['tenantid'] )
        {
          continue;
        }
        $match ++;
        // contacts match
        // break to prevent double matching
        break;
      }
    }
  }

  return ($match >= $tomatch);
}

function CallAction($act,$remoteid,$source)
{
	// send msg
	// 1: analog, 5: digital TDM, 6: PRI
	$signal_reply = true;
	$err = '';
	if ($source==1 || $source==5 || $source==6)
	{
		$err = signal_service('sgncrc ' . $act . ' ' . $remoteid,$signal_reply);
	} else
	{
		$err = signal_service('sgnork ' . $act . ' ' . $remoteid,$signal_reply);
	}
	if ($err != '')
	{
		return 'Signaling: ' . $err;
	}
	if ($signal_reply != 'OK')
	{
		return 'Reply: "' . $signal_reply . '"';
	}
	return '';
}

function TrackerHeartBeat()
{
  global $dbintf;
  $timeoutMinutes = 2 * 60; // two hours
  $dbintf->ExpireActionCache( 60 * $timeoutMinutes );
  $cacheCount = $dbintf->GetActionCacheCount();
  // lg('Items in action cache ' . $cacheCount );
  if ($cacheCount==0) return;
  // we need to check all active calls,
  // but actually each active call only once
  $calls = $dbintf->GetFullActiveCallList();
  foreach ($calls as $call)
  {
    if (ApplyActionCache($call))
    {
      // something changed
      lg('Update active call ' . $call['remoteid'] );
      unset($call['id']);
      // lg(var_export($call,true));
      $dbintf->SetActiveCallInfoRM($call['remoteid'],$call);
    }
  }
}

function ApplyActionCache(&$ref_data)
{
  global $dbintf;
  $changed = false;
  $timeoutMinutes = 2 * 60; // two hours
  $dbintf->ExpireActionCache( 60 * $timeoutMinutes );
  $cache = $dbintf->GetActionCache('CallMatches',$ref_data);
  // debug: lg("Matching action cache count: " . count($cache) );
  foreach ($cache as $data)
  {
    $action = StringToArray( $data['action'] );
    if ($action['act']=='keep')
    {
      lg('Apply action cache ' . $action['act']
        . ' to call ' . $ref_data['remoteid']);
      $callextra = StringToArray($ref_data['callextra'] ?? null);
      $callextra['to-keep'] = '1';
      $ref_data['callextra'] = ArrayToString($callextra);
			// Notify engine (this is needed for Cisco AR,
			// to let it be applied to next call legs.
      CallAction( $action['act'], $ref_data['remoteid'], $ref_data['source'] );
      // continue checking other actions
      $changed = true;
    }
    if ($action['act']=='dataset')
    {
      lg('Apply action cache ' . $action['act']
        . ' to call ' . $ref_data['remoteid']);
      
      $ptarget = $action['target'];
      $FullSipId = (int) $dbintf->GetConfigValue('FullSipId');
      if ($ptarget=='local')
      {
        $ref_data['localfull'] = $action['value'];
        $ref_data['local'] = IdSimplify($ref_data['localfull']);
        $cidLocal = GetSipIdToUse($ref_data['localfull'],$FullSipId);
        $ret = $dbintf->GetOrCreateCallerID($cidLocal,'',2);
        $ref_data['localcallerid'] = $ret["id"];
        $ref_data['localcontact'] = $ret["contactid"];
        $changed = true;
      } else if (in_array($ptarget
				,array('notes','notes2','notes3','notes4','notes5'
					,'data1','data2','data3','data4','data5'
					)))
      {
        $ref_data[$ptarget] = $action['value'];
        $changed = true;
      }
    }
    if ($action['act']=='delete')
    {
      lg('Apply action cache ' . $action['act']
        . ' to call ' . $ref_data['remoteid']);
      CallAction( $action['act'], $ref_data['remoteid'], $ref_data['source'] );
    }
    $dbintf->RemoveActionCacheItem( $data['id'] );
  } // loop over matching cache items
  return $changed;
}

function CheckDetermineThings(&$ref_data, $stage)
{
	MaybeDetermineTenant($ref_data, $stage);
	DetermineAgent($ref_data, $stage);
}

function DetermineAgent(&$ref_data, $stage)
{
	global $dbintf, $configs_s;
	if ( isset($configs_s['DetermineAgent']) && $configs_s['DetermineAgent']==0 )
	{
		// Previously, no agent was determined.
		// Since agent determination takes some calculation,
		// possibly slows things down,
		// it is behind an option,
		// but it is on by default.
		return;
	}
	// determine if tenant assignment is enabled and slow
	// (slow = assigned at end of call)
	$SlowTenancy = false;
	if (int_ensure_defined($configs_s['MultiTenancy'])==1)
	{
		if (int_ensure_defined($configs_s['UsrFilterLocal'])!=0
		|| int_ensure_defined($configs_s['TenantQuickAssign'])!=0) ;
		else $SlowTenancy = true;
	}
	
	if ($SlowTenancy)
	{
		// then do agent determination also at the end of call
		if ($stage=='ready' || $stage=='recover')
		{
			; // do it now
		} else return; // do at another stage
	} else
	{
		if ($stage=='start' || $stage=='recover')
		{
			; // do it now
		} else return; // do at another stage
	}
	
	// determine agent
  $OnlyEnabled = false;
	$accids = $dbintf->GetUsersOfCall($ref_data,$OnlyEnabled);
	
	// is the recording assigned to a tenant?
	// then the user should match
	if (isset($ref_data['tenantid']))
	{
		foreach($accids as $accid)
		{
			$userinfo = $dbintf->GetUserInfoOnAccId($accid);
			// must be the same tenant
			if ($userinfo['tenantid'] == $ref_data['tenantid'])
			{
				$ref_data['agent'] = $accid;
				break;
			}
		}
		return;
	}
	
	if (count($accids)==0) return;
	// assign first
	$ref_data['agent'] = $accids[0];
}

function MaybeDetermineTenant(&$ref_data, $stage)
{
	global $configs_s;
	if (int_ensure_defined($configs_s['MultiTenancy'])!=1)
	{
		// No multitenancy
		return;
	}

	if($ref_data['flags'] & 128)
	{
		if(DetermineTenantFromVTapID($ref_data))
		{
			return;
		}		
	}

  if(isset($ref_data['tenantid']))
  {
    global $dbintf;
    if(!$dbintf->IsTenant($ref_data['tenantid']))
    {
      unset($ref_data['tenantid']);
    }
    else
    {
      return;
    }
  }
  
	if (int_ensure_defined($configs_s['UsrFilterLocal'])!=0
  || int_ensure_defined($configs_s['TenantQuickAssign'])!=0)
	{
		if ($stage=='start' || $stage=='recover')
		{
			DetermineTenant($ref_data);
		}
	} else
	{
		if ($stage=='ready' || $stage=='recover')
		{
			DetermineTenant($ref_data);
		}
	}
}

function FillCallData()
{
  $data = array();
  $dbmap = array
  ( 'status'   => 'callstatus'
  , 'local'    => 'local'
  , 'remote'   => 'remote'
  , 'initt'    => 'inittime'
  , 'ringt'    => 'ringtime'
  , 'connectt' => 'connecttime'
  , 'rr'       => 'recreason'
  , 'prot'     => 'protocol'
  , 'dir'      => 'direction'
  );
  foreach ($dbmap as $msgkey => $dbkey)
  {
    if (isset($_GET[$msgkey])==false) continue;
    $data[$dbkey] = $_GET[$msgkey];
  }
  return $data;
}

function FillData()
{
    global $dbintf, $OutgPrefix, $serialnum, $FullSipId;
    global $SpinClient;
		//, $SIPgate;

    $data = array();

    $data['localfull']  = $_GET['localparty' ];
    $data['remotefull'] = MaskApply($_GET['remoteparty']);
		$data['connected'] = $_GET['connectedparty'];

    $data['local'] = IdSimplify($data['localfull']);
    $data['remote'] = IdSimplify($data['remotefull']);

    $data['local2'] = $_GET['localip'];
    $data['remote2'] = $_GET['remoteip'];

    if ($_GET['direction']=='out') $direction = 1;
    else if ($_GET['direction']=='in') $direction = 0;
    else if ($_GET['direction']=='loc') $direction = 2; // new
    else if ($_GET['direction']=='ext') $direction = 3; // new
    else $direction = -1; // unkown direction

		// but if outgoing dial prefix is present
		if ($direction==1 || $direction==2 || $direction==3) // local or remote
		{
			// remove dial prefix
	    if ($OutgPrefix!='' && ClearRemoteDialPrefix($data['remote']))
	    {
				// set to outgoing
      	$direction = 1;
   		}
    }

    $remote_is_local = ($direction==2);
    $local_is_remote = ($direction==3);

    $data['direction'] = $direction;

    $data['startdate'] = $_GET['timestamp'];
    $data['startms'] = (int) $_GET['startms'];
    if ($data['startms']<0)
    {
      unset( $data['startms'] );
    }
    $data['duration'] = $_GET['duration'];
    $durms = (int) $_GET['durms'];
    if ($durms>=0)
    {
      $data['durms'] = $durms;
    }

    $data['filename'] = $_GET['filename'];
    // split in directory and file name
    $slashpos = strrpos($data['filename'],'/');
    if ($slashpos!==FALSE)
    {
      $data['directory'] = substr($data['filename'],0,$slashpos);
      $data['filename'] = substr($data['filename'],$slashpos+1);
    }

    $data['flags'] = 0; // initial state
		if (isset($_GET['a']))
		{
			$a = (int) $_GET['a'];
			$data['isarchived'] = ( ($a & 1) != 0) ? 1 : 0;
			if ( ($a & 2) != 0 )
			{
				$data['flags'] |= 16;
			}
		} else
		{
			// compatibility for partial upgrade
			$fn = $data['filename'];
			$fn = change_filename_extension($fn,'.wav');
			$filethere = false;
			if (file_exists(AUDIO_PATH.'/'.$data['directory'].'/'.$fn))
			{
				$filethere = true;
			}
			if (file_exists(AUDIO_PATH.'/'.$data['directory'].'/'.$fn.'.g711'))
			{
				$filethere = true;
			}
			$fn = change_filename_extension($fn,'.mcf');
			if (file_exists(AUDIO_PATH.'/'.$data['directory'].'/'.$fn))
			{
				$filethere = true;
			}
			$data['isarchived'] = $filethere?1:0; // file is there
		}

    $data['remoteid'] = $_GET['recid'];
    $callextra = array();
    $callextra['NativeCallID'] = $_GET['nativecallid'];
    //$data['callextra'] = ArrayToString($callextra);

    $data['isconnected'] = 1; // corrected elsewhere (for missed calls)
    $data['linenr'] = 0; // not applicable
		if (isset($_GET['waittime'])==false)
		{
			$data['waittime'] = -1; // unknown
		} else
		{
			$data['waittime'] = $_GET['waittime'];
		}
    $data['source'] = 0; // unknown
    $data['encoding'] = 51; // unknown, GSM or G.711
    $data['encryption'] = 0; // no
    $data['processingstate'] = 0; // initial state
    $data['lineid'] = 0; // not applicable

    $data['localcontact'] = 0;
    $data['remotecontact'] = 0;
    
    FixInvalidChars( $data );

		$cidLocal = GetSipIdToUse($data['localfull'],$FullSipId);
    $ret = $dbintf->GetOrCreateCallerID($cidLocal,$data['local2' ],$local_is_remote?0:1);
    $data['localcallerid'] = $ret["id"];
    $data['localcontact'] = $ret["contactid"];
		$dbcustom = $dbintf->GetCustomLocalId( $data['localcontact'] );
		if ( ($dbcustom!==false)
		&& ( time() < $dbcustom['updatetime'] + 60 ) )
		{
			$ctinfo = $dbintf->GetContactInfo( (int) $dbcustom['dstid'] );
			$data['local'] = $ctinfo['callerid'];
			$data['localcallerid'] = $ctinfo['cidid'];
			$data['localcontact'] = (int) $dbcustom['dstid'];
		}

		$cidRemote = GetSipIdToUse($data['remotefull'],$FullSipId);
    $ret = $dbintf->GetOrCreateCallerID($cidRemote,$data['remote2'],$remote_is_local?1:0);
    $data['remotecallerid'] = $ret["id"];
    $data['remotecontact'] = $ret["contactid"];
		if ($remote_is_local) // internal calls
		{
			$dbcustom = $dbintf->GetCustomLocalId( $data['remotecontact'] );
			if ( ($dbcustom!==false)
			&& ( time() < $dbcustom['updatetime'] + 60 ) )
			{
				$ctinfo = $dbintf->GetContactInfo( (int) $dbcustom['dstid'] );
				$data['remote'] = $ctinfo['callerid'];
				$data['remotecallerid'] = $ctinfo['cidid'];
				$data['remotecontact'] = (int) $dbcustom['dstid'];
			}
		}

		if ($data['connected']!='')
		{
      $connectedfull = $data['connected'];
      $data['connected'] = IdSimplify($data['connected']);
      $cidConn = GetSipIdToUse($connectedfull, $FullSipId);
			$ret = $dbintf->GetOrCreateCallerID($cidConn,'',2);
			$data['connectedcallerid'] = $ret["id"];
			$data['connectedcontact']  = $ret["contactid"];
      if ($SpinClient!=0)
      {
        $data['connected_cid'] = $cidConn;
      }
		}

    if ($SpinClient!=0)
    {
      $data['local_cid'] = $cidLocal;
      $data['remote_cid'] = $cidRemote;
    }
    
		// Analyse tags
		if (isset($_GET['tags']) && $_GET['tags']!='')
		{
			//lg('Tags: '.$_GET['tags']);
			$parts = explode( ',' , $_GET['tags'] );
			foreach ($parts as $part)
			{
				$colon = strpos($part,':');
				if ($colon===false) continue;
				$key = substr($part,0,$colon);
				$value = FixInvalidCharsInString( UnquoteTag( substr($part,$colon+1) ) );
				//lg("tag: '$key'='$value'");
				if ($key=='localname')
				{
					TrySetContactName( $data['localcontact'], $value, $data['local'] );
				}
				if ($key=='remotename')
				{
					TrySetContactName( $data['remotecontact'], $value, $data['remote'] );
				}
				if ($key=='connectedname' && $data['connected']!='' && isset($data['connectedcontact']) )
				{
					TrySetContactName( $data['connectedcontact'], $value, $data['connected'] );
				}
        if ($key=='notes'
          || $key=='notes2'
          || $key=='notes3'
          || $key=='notes4'
          || $key=='notes5'
					|| $key=='data1'
					|| $key=='data2'
					|| $key=='data3'
					|| $key=='data4'
					|| $key=='data5')
        {
          // copy data in tags to database record
          $data[ $key ] = $value;
        }
        // ...
        if ($key=='Act')
        {
          if ($value=='keep')
          {
            //$callextra = StringToArray($data['callextra']);
            $callextra['to-keep'] = '1';
            //$data['callextra'] = ArrayToString($callextra);
          }
        }
				if($key=='LicFail' && $value=='True')
				{
					$data['flags'] |= 32;
				}
				if($key=='MissedCall' && $value=='True')
				{
					$data['flags'] |= 64;
				}
				if($key=='IsTunnel' && $value=='True')
				{
					$data['flags'] |= 128;
				}
				if($key=='Toshiba' && $value=='True')
				{
					$data['flags'] |= 256;
				}
				if($key=="TunnelID")
				{
					//$callextra = StringToArray($data['callextra']);
					$callextra['TunnelID'] = strtoupper($value);
					//$data['callextra'] = ArrayToString($callextra);
				}
        if ($key=='tenant')
        {
          $data['orktenant'] = $value;
        }
        if ($key=='force_ork_tenant')
        {
          $force_ork_tenant = (int) $value;
        }
        if ($key=='storeondemand')
        {
          $callextra['storeondemand'] = 1;
        }
        if($key=='source')
        {
          $data['source'] = $value;
        }
			}
		}

    if(int_ensure_defined($force_ork_tenant) && isset($data['orktenant']))
    {
      $data['tenantid'] = (int) $data['orktenant'];
    }
		
		if($data['flags'] & 128)
		{
      //$callextra = StringToArray($data['callextra']);
			$TunnelID = $callextra['TunnelID'];
			if($TunnelID !== '')
			{
        if ($SpinClient!=0)
        {
          $data['recorder_sn'] = $TunnelID;
          $data['recorder_name'] = "V-Tap $TunnelID";
        }
				$data['recorderid'] = $dbintf->GetOrCreateRecorderId($TunnelID, "V-Tap $TunnelID", 55);
			}
		}
		
		// $_GET['service'] = orkaudio-$ipname
		// assumes request is coming from local/own machine
		if(!isset($data['recorderid']))
		{
      if ($SpinClient!=0)
      {
        $data['recorder_sn'] = $serialnum;
        $data['recorder_name'] = "Apresa$serialnum";
      }
			$data['recorderid'] = $dbintf->GetOrCreateRecorderId($serialnum,"Apresa$serialnum");
    }
    
    $data['key'] = $_GET['key'];
    $data['iv'] = $_GET['iv'];
    $data['encrypt'] = $_GET['encrypt'];
    if($data['encrypt'] == 'true')
    {
      $data['flags'] |= 2048;
    }

    $data['callextra'] = ArrayToString($callextra);

    //lg('DEBUG: Filename: '.$data['filename'] . ' remoteid: ' .$data['remoteid'] . ' stage:'.$_GET['stage'])
    return $data;
}

function TrySetContactName( $contactid , $newname, $callerid )
{
	global $dbintf;
	
	$newname = trim( $newname );
	// maybe to-do: remove text between ()
	if ($newname=='') return;
	// don't use name if equal to caller id (number)
	if ($callerid==$newname) return;
	// don't use if it contains @
	if (strpos($newname,'@')!==false) return;
	$oldname = $dbintf->GetContactName( $contactid );
	// don't use if name already filled in
	if ($oldname!='') return;
	// don't change if equal
	if ($oldname==$newname) return;
	//lg('Set contact '.$contactid.' name to '.$newname);
	$dbintf->EditContact( $contactid, $newname );
}

function ConvDtmf($num)
{
  if ($num>=0 && $num<=9) return '' . $num;
  if ($num==10) return '*';
  if ($num==11) return '#';
  if ($num>=12 && $num<=15) return chr(ord('A') + ($num-12));
  if ($num>=16 && $num<=20) return chr(ord('S') + ($num-16));
  return '?';
}

function GetUsers($callinfo)
{
  global $dbintf;
	$accids = $dbintf->GetUsersOfCall($callinfo,false);
	$users = array();
	foreach($accids as $accid)
	{
		$users[] = $dbintf->GetUserInfoOnAccId($accid);
	}
	return $users;
}

?>