<?php

require_once('general.php');
require_once('update-gen.php');

HandlePage($dbintf);

function GetDebianCodeName($version)
{
  $codeNames = array
  (  7 => 'wheezy'
  ,  8 => 'jessie'
  ,  9 => 'stretch'
  , 10 => 'buster'
  , 11 => 'bullseye'
  , 12 => 'bookworm'
  );
  if (isset($codeNames[$version]))
  {
    return $codeNames[$version];
  }
  return '';
}

// returns configured mirror for Debian updates if any
function GetAptMirror()
{
  $debv = GetDebianVersion();
  if ($debv=='') return;
  $codeName = GetDebianCodeName($debv);
  if ($codeName=='') return;
  
  $sources = file('/etc/apt/sources.list');
  foreach ($sources as $line)
  {
    if (strpos($line,$codeName)===false) continue;
    $parts = preg_split('/\s+/',$line);
    if (count($parts)<4) continue;
    if ($parts[0]!='deb') continue;
    if ($parts[2]!=$codeName) continue;
    return $parts[1];
  }
  return '';
}

function getInstalledDebVersion($packname)
{
  $version = '';
  $dpkg_output = array();
  exec('dpkg-query -W -f=\'${binary:Package}\t${Version}\t${Status}\n\' ' . escapeshellarg($packname) , $dpkg_output);
  if (count($dpkg_output)==1)
  {
    $parts = explode("\t",$dpkg_output[0]);
    if (count($parts)==3 && $parts[0]==$packname && $parts[2]=='install ok installed')
    {
      $version = $parts[1];
    }
  }
  return $version;
}

function IsAuReqInstalled()
{
  $neededPackages = array( 'unattended-upgrades' , 'apt-listchanges' , 'bsd-mailx' );
  $allPresent = true;
  foreach ($neededPackages as $packname)
  {
    $version = getInstalledDebVersion($packname);
    if ($version=='') {
      $allPresent = false;
    }
  }
  return $allPresent;
}

function HandlePage($dbintf)
{
  global $rpl,$configs,$msg,$lng,$rooturl;

  if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0)
  {
    echo 'Permission denied';
    exit;
  }

  if (COM_TYPE=='Win')
  {
    $debianVersion = 9;
    $aptMirror = 'http://deb.debian.org/debian/';
    $auReqInstalled = false;
    $auFreq = 1;
  } else
  {
    $debianVersion = GetDebianVersion();
    $aptMirror = GetAptMirror();
    // check if required things are installed for automatic update
    $auReqInstalled = IsAuReqInstalled();
    // current update frequency based on database
    // it might be user has set it themselves
    $auFreq = (int) $dbintf->GetConfigValue('AuFreq');
  }
	
  $updatePeriod = GetUpdatePeriod();
  $active_cnt = $dbintf->GetActiveCallsCount();

  if (isset($_FILES['update']))
  {
		if ($updatePeriod->updating==false)
		{
			echo "The update period has expired";
			exit;
		}

		LogAction('Perform software update using file upload');
		ClearSessionVars();

    if ($active_cnt>0 && $_POST['confirmed']!='1')
    {
      $msg[] = new Msg(MSG_ERR,$lng->Get('System.CallsActive').' '.$lng->Get('System.RepeatToForceUpdate'));
      $_SESSION['msg'] = $msg;
      header('Location: '.$rooturl.'update.php');
      exit;
    }

    $upd_time = $_POST['UpdTime'];

    $upd = $_FILES['update'];
    if ($upd['error']!=0)
    {
      // error in upload
      $msg[] = new Msg(MSG_ERR,'Error in file upload. '.GetUploadErrDescr($upd['error']));
    } else
    {
      // there is a good upload
      $upd_dest = dirname($upd['tmp_name']).'/updatefile.dat';
      move_uploaded_file($upd['tmp_name'],$upd_dest);

			if ( ! $dbintf->nodePassive )
			{
				// save in database
				$dbintf->SetConfigsPart(
					array('UpdUpl'=>$upd_dest,'UpdTime'=>$upd_time),'s');
			}
      // signal
      if (msg_signal_service($msg,'su'))
      {
          if ($upd_time=='now')
          {
            //$msg[] = new Msg(MSG_INF,$lng->Get('System.Updating'));
						sleep(1);
          } else
          {
            $msg[] = new Msg(MSG_INF,$lng->Get('Main.OK'));
          }
      } else {
        $msg[] = new Msg(MSG_ERR,'System update signal failed');
      }
    }
    // redirect
    $_SESSION['msg'] = $msg;
    header('Location: '.$rooturl.'systools.php');
    exit;
  }

  if (isset($_POST['act']))
  {
    $act = $_POST['act'];
    if ($act=='editAptMirror')
    {
      $aptMirror = $_POST['mirror'];
      $dbintf->SetConfigsPart(array('AptMirrorUrl'=>$aptMirror),'s');
      msg_signal_service($msg,'syscfg aptmirror');
    }
    if ($act=='au')
    {
      $freq = (int) $_POST['freq'];
      $reqinstall = (int) $_POST['reqinstall'];
      $runnow = (int) $_POST['runnow'];
      
      if ($freq!=0 && $freq!=1) $freq = 0;
      if ($reqinstall!=0 && $reqinstall!=1) $reqinstall = 0;
      if ($runnow!=0 && $runnow!=1) $runnow = 0;
      
      // it must be done in the background because it can take long
      // two background process cannot work if they need to wait on each other
      // so it must be all one background process
      $dbintf->SetConfigsPart(
        array('AuReqInstall' => "$reqinstall"
        , 'AuFreq' => "$freq"
        , 'AuRunNow' => "$runnow"
        ),'s');
      if (msg_signal_service($msg,'syscfg au'))
      {
        $msg[] = new Msg(MSG_INF,'Configuring automatic updates');
      }
    }
    // redirect
    $_SESSION['msg'] = $msg;
    header('Location: '.$rooturl.'update.php');
    exit;
  }
    
	if ($updatePeriod->updating==false)
	{
		$msg[] = new Msg(MSG_ERR,$lng->Get("Licensing.UpdatePeriodExpired"));
	}
	
	LogAction("Viewed software update page.",array("level"=>"view"));
	
  // ready with database
  $dbintf->close();

  $data = "var active_cnt = $active_cnt; \r\n";
	$data .= "var mayUpdate = " . (int) ($updatePeriod->updating?1:0) . ";\r\n";
	$data .= "var updatesUntil = \"" . JSQuote($updatePeriod->updatesUntilStr) . "\";\r\n";
  $data .= "var debianVersion = " . (int) $debianVersion . ";\r\n";
  $data .= "var auReqInstalled = " . (int) $auReqInstalled . ";\r\n";
  $data .= "var auFreq = " . (int) $auFreq . ";\r\n";
	$data .= "var aptMirror = \"" . JSQuote($aptMirror) . "\";\r\n";
  $data .= GeneralJSData();

  $template_content = file_get_contents('../apresa/templ/update.htm');

  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}



?>
