<?php
require_once('general.php');
require_once('certificates_general.php');

HandleUpload($dbintf);

function GetCertFiles()
{	
  $out = [];

	$certfile = $_FILES['certfile'];
	
	if($certfile['error'] !== UPLOAD_ERR_OK)
	{
		throw new Exception("The upload of the certificate did not succeed ($pkeyfile[error])");
	}
	
	$cert = file_get_contents($certfile['tmp_name']);
	
	if($cert === false)
	{
    throw new Exception("Could not read the uploaded certificate");
  }
  
  $out["cert"] = $cert;
	
  $pkeyfile = $_FILES['pkeyfile'];
  
	if($pkeyfile['error'] === UPLOAD_ERR_OK)
	{
		$pkey = file_get_contents($pkeyfile['tmp_name']);
		
		if($pkey === false)
		{
			throw new Exception("Could not read the uploaded private key");
    }
    
    $out["key"] = $pkey;
	}
	else if($pkeyfile['error'] !== UPLOAD_ERR_NO_FILE )
	{
		throw new Exception("The upload of the private key did not succeed ($pkeyfile[error])");
	}
	
	$interfile = $_FILES['interfile'];
	
	if($interfile['error'] === UPLOAD_ERR_OK)
	{
		$intermediate = file_get_contents($interfile['tmp_name']);
		
		if($intermediate === false)
		{
			throw new Exception("Could not read the uploaded intermediate certificate(s) file");
    }
    
    $out["intermediate"] = $intermediate;
	}
	else if($interfile['error'] !== UPLOAD_ERR_NO_FILE )
	{
		throw new Exception("The upload of the intermediate certificate(s) did not succeed ($pkeyfile[error])");
  }
  
  return $out;
}

function DoUpload()
{
	global $dbintf, $msg;
	
	$gotFiles = false;
	if($_POST['type'] === 'file')
	{
		$files = GetCertFiles();
	}
	else if($_POST['type'] === 'text')
	{
		$cert = str_ensure_defined($_POST['certtext']);
    $pkey = str_ensure_defined($_POST['pkeytext']);
    $intermediate = str_ensure_defined($_POST['intertext']);
    
    $files = [];
    $files["cert"] = $cert;

    if($pkey != "") $files["key"] = $pkey;
    if($intermediate != "") $files["intermediate"] = $intermediate;
	}
		
	$id = UploadCertificate($_POST['name'], $files);
	
	if(str_ensure_defined($_POST['trusted']) == 'on')
	{
		$certificate = $dbintf->GetCertificate($id);
		msg_signal_service($msg, 'xcerttrust ' . $certificate['cert_location']);
		$certificate['trusted'] = 1;
		$dbintf->UpdateCertificate($certificate['id'], array( 'trusted' => 1));
	}
	
	return $id;
}

function HandleUpload($dbintf)
{
	global $rpl,$rooturl,$msg,$lng,$configs;
	
	if (($dbintf->actuinfo['permissions']&PM_CONFIG)==0)
  {
    echo 'Permission denied';
    exit;
  }
	
	$act = str_ensure_defined($_POST['act']);
	
	if($act != '')
	{
		if($act == 'upload')
		{		
      try 
      {
        $id = DoUpload();
        $location = $rooturl.'certificate.php?id=' . $id;
        LogAction('Upload certificate', array('id'=>$id, 'name'=>$_POST['name']));
      } 
      catch (Exception $ex) 
      {
        $msg[] = new Msg(MSG_ERR, $ex->getMessage());
        $location = $rooturl.'uploadcert.php';
      }
		
      $_SESSION['msg'] = $msg;
			header('Location: ' . $location);
			exit;
		}
	}
	
	$data = GeneralJSData();
	
	LogAction('View upload certificate page', array('level'=>'view'));
	
	$dbintf->close();
	
  $template_content = file_get_contents('../apresa/templ/uploadcert.htm');

  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $data;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}

?>
