<?php

require_once('general.php');
require_once('actions.php');

HandlePage($dbintf);

function HandlePage($dbintf)
{
  global $rpl,$msg,$lng,$linemap;

  if (($dbintf->actuinfo['permissions']&(PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN))==0)
  {
    echo 'Permission denied';
    exit;
  }

	$ctlines = $dbintf->GetLineList();
	$linemap = array();
	foreach ($ctlines as $ctline)
	{
		$linemap[$ctline['linenr']] = $ctline['contactid'];
	}
	
  $step = 'init';

  if ($_POST['act']=='import')
  {
    $step = 'import';
    $csv = $_POST['csv'];

		LogAction('Import users from CSV');
		
    $dbintf->StartTransaction();

    $err = false;
    $file = new CsvFile();
    $file->ReadFromString($csv);

    $users = ParseUserAccountList($dbintf,$file,$err);
		if ($err)
		{
			$msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
			$msg[] = new Msg(MSG_ERR,'Import cancelled');
			$dbintf->CancelTransaction();
			$_SESSION['msg'] = $msg;
			header('Location: '.$rooturl.'users.php');
			exit;
		}

    foreach ($users as $user)
    {
      $usrdata = array();
      foreach(array('name','email','usr','enabled','mphones','logonmethod','recondmd','delondmd','silondmd','customlocalid','freeseat','rod','ldapuser','webclientsettings') as $key)
      {
        if (isset($user[$key]))
        {
          $usrdata[$key] = $user[$key];
        }
      }
      if (isset($user['pwd']) && $user['pwd']!='')
      {
        $usrdata['pwd'] = DBInterface::CreatePwdHash($user['pwd']);
      }
      $toLogAdd = false;
      if ($user['action']=='edit')
      {
        $userid = $user['id'];
        $theuser = $dbintf->GetUserInfo((int)$userid);
        $merged = array_merge($theuser,$usrdata);
        LogAction('Edit user', array('id'=>$userid,'name'=>$merged['name'],'tenantid'=>$theuser['tenantid']) );
        if ($dbintf->EditUserAccount($userid,$usrdata)==false)
        {
          $msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
					$err = true;
					break;
        }
      } else if ($user['action']=='add')
      {
        if ($dbintf->AddUserAccount($userid,$usrdata)==false)
        {
          $msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
					$err = true;
					break;
        }
        else $toLogAdd = true;
      }
      $userinfo = $dbintf->GetUserInfo($userid);
      if (isset($user['phones']))
      {
        $phones = array();
        foreach ($user['phones'] as $phone_item)
        {
					// 3 param: true: create if needed
					$obj = ParsePhoneItem( $phone_item , $userinfo['accid'], true );
          $phones[] = $obj;
        }
        if ($dbintf->SetUserPhones($userinfo['accid'],$phones)==false)
				{
          $msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
					$err = true;
					break;
				}
      }
      if (isset($user['groups']))
      {
        if ($dbintf->SetGroupsOfWhichMember($userinfo['accid'],$user['groups'])==false)
				{
          $msg[] = new Msg(MSG_ERR,GetErrDescr($dbintf->lasterr));
					$err = true;
					break;
				}
      }
      if ($toLogAdd)
      {
        // we need to do this delayed, after group membership is set,
        // to be able to log the current tenant
        $uinfo = $dbintf->GetUserInfo($userid);
        LogAction('Add user', array('id'=>$userid,'name'=>$uinfo['name'],'tenantid'=>$uinfo['tenantid']) );
      }
    } // end of for loop for all users
		if ($err)
		{
			$msg[] = new Msg(MSG_ERR,'Import cancelled');
			$dbintf->CancelTransaction();
			$_SESSION['msg'] = $msg;
			header('Location: users.php');
			exit;
		}
    
    $dbintf->CommitTransaction();
    EnginesSettingsCalcApply(true);
		
		$msg[] = new Msg(MSG_INF,'<%Web.ImportDone%>');
	
		$_SESSION['msg'] = $msg;
    header('Location: users.php');
    exit;
  }

  $usersdata = '';

  if (isset($_FILES['userlist']))
  {
    $upd = $_FILES['userlist'];
    if ($upd['error']!=0)
    {
      // error in upload
      $msg[] = new Msg(MSG_ERR,'Error in file upload. '.GetUploadErrDescr($upd['error']));
    } else
    {
      // there is a good upload
      // file name:
      $fn = $upd['tmp_name'];

      if(HasExcelExport() && (
        get_filename_extension($upd["name"]) == "xlsx" || 
        $upd['type'] == "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"))
      {
        $file = new XLSXFile();
        $file->Read($fn);
      }
      else
      {
        $file = new CsvFile();
        $file->Read($fn);
      }

			$err = false;
      $users = ParseUserAccountList($dbintf,$file,$err);

      $group_list = $dbintf->GetGroupList(); // full list
      $groupnames_r = array();
      foreach ($group_list as $group)
      {
        $groupnames_r[ $group['accid'] ] = $group['name'];
      }

      $usersdata .= 'var users = [';
      $first = true;
      foreach ($users as $row)
      {
        if ($first==false) $usersdata .= ',';

        $phonest = '';
        $first2 = true;
        foreach($row['phones'] as $phone)
        {
          if ($first2==false)
          {
            $phonest .= ', ';
          }
          $phonest .= $phone;
          $first2 = false;
        }
        $groupst = '';
        $first2 = true;
        foreach($row['groups'] as $group)
        {
          if ($first2==false)
          {
            $groupst .= ', ';
          }
          $groupst .= $groupnames_r[$group];
          $first2 = false;
        }

				// This change is only visual during preview
				$mphones = $row['mphones'];
				if ($mphones===null) $mphones = '- - -';
				
        $usersdata .= '{usr:"'.JSQuote($row['usr']).'"'
        . ',enabled:'.$row['enabled']
        . ',name:"'.JSQuote($row['name']).'"'
        . ',oldname:"'.JSQuote($row['oldname']).'"'
        . ',email:"'.JSQuote($row['email']).'"'
				. ',mphones:"'.JSQuote($mphones).'"'
				. ',logonmethod:"'.JSQuote($row['logonmethod']).'"'
        . ',phones:"'.JSQuote($phonest).'"'
        . ',groups:"'.JSQuote($groupst).'"'
        . ',action:"'.JSQuote($row['action']).'"'
        . "}\r\n";
        $first = false;
      }
      $usersdata .= "];\r\n";

      $usersdata .= 'var has_err=' . ($err?1:0) . ';' . "\r\n";

      $usersdata .= 'var csv="' . JSQuote($file->asCSV()) . '";'."\r\n";

      $step = 'verify';
    }
  }

  $usersdata .= GeneralJSData();

	LogAction("Viewed user import.",array("level"=>"view"));

  // ready with database
  $dbintf->close();

  if ($step=='init')
  {
    $template_content = file_get_contents('../apresa/templ/users_import.htm');
  }
  if ($step=='verify')
  {
    $template_content = file_get_contents('../apresa/templ/users_import_verify.htm');
  }
	
  // template replacements:
  $rpl['msg'] = message($msg);
  $rpl['nav'] = '';
  $rpl['MAINFORM'] = '<form id="mainform" method="POST"></form>';
  $rpl['FOOTER'] = '';
  $rpl['data'] = $usersdata;
  $rpl['JSGEN'] = file_get_contents('../apresa/templ/general.js');
  $rpl['header'] = GetHeader();
  $template_content = do_replace_tags($template_content);
  echo $template_content;
}



/* This is the function that parses the CSV data.
	It is used both during preview and actual import.
	The CSV data is passed as a string that contains
	the complete CSV file as uploaded.
	Extra fields that were added in recent versions,
	but might be missing, are returned as NULL when missing,
	and this prevent incorrectly overwriting old values.
*/
function ParseUserAccountList($dbintf,$file,&$err)
{
  global $msg, $lng, $linemap;

	$DupNamesAllowed = ((int)$dbintf->GetConfigValue('DupNamesAllowed'))!=0;
	
  $user_list = $dbintf->GetEditableUserList();
  $existing_users = array();
	$UsersOnAccid = array();
  foreach ($user_list as $user)
  {
    $existing_users[ $user['id'] ] = $user;
		$UsersOnAccid[ $user['accid'] ] = $user;
  }

  $group_list = $dbintf->GetGroupList(); // full list
  $groupnames = array();
  foreach ($group_list as $group)
  {
    $groupnames[ $group['name'] ] = $group['accid'];
  }

  $line_nr = 0;
  $users = array();
  $editids = array();
  foreach($file as $parts)
  {
    $line_nr ++;
    if ($line_nr==1)
    {
      // the header
      continue;
    }

    if (count($parts)<=1 && $parts[1]=='') continue;

    if ($parts[1]=='')
    {
      $err = true;
      $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': No name specified');
    }

    // If an ID is defined, then edit
    // If no ID is defined, then add
    // That is the most safe procedure
    $editid = trim($parts[0]);
    $oldname = '';
    if ($editid!='')
    {
      if (isset($editids[$editid]))
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': Duplicate DB-ID');
        continue;
			}
      if (isset($existing_users[$editid])==false)
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': DB-ID of user not found');
        continue;
      } else
      {
        $action = 'edit';
      }
      $oldname = $existing_users[$editid]['name'];
      $editids[$editid] = true;
    } else
    {
      $action = 'add';
    }

    $enabled = ($parts[2]=='1'?1:0);
    $name = $parts[1];
    $usr = $parts[3];

		$mphones = null;
		if (isset($parts[8]))
		{
			$mphones = $parts[8];
		} 
		$logonmethod = null;
		if (isset($parts[9])) $logonmethod = (int) $parts[9];
		$recondmd = null;
		if(isset($parts[10])) $recondmd = ($parts[10]=='1'?1:0);
		$delondmd = null;
		if(isset($parts[11])) $delondmd = ($parts[11]=='1'?1:0);
		$silondmd = null;
		if(isset($parts[12])) $silondmd = ($parts[12]=='1'?1:0);
		$customlocalid = null;
		if(isset($parts[13])) $customlocalid = ($parts[13]=='1'?1:0);
		$freeseat = null;
		if(isset($parts[14])) $freeseat = ($parts[14]=='1'?1:0);
		$rod = null;
		if(isset($parts[15])) $rod = (int)$parts[15];		//0, 1 or 2!
		$ldapuser = null;
		if(isset($parts[16])) $ldapuser = ($parts[16]=='1'?1:0);
    $webclientsettings = 0;
    if(isset($parts[17])) $webclientsettings = (int) $parts[17];
		
		$accid = -1;
    if ($action=='add')
    {
      if ($DupNamesAllowed==false
			&& $dbintf->NameInUse($name))
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.' Name is already in use: '.$name);
      }
      if ($enabled==1 && $usr=='')
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.' Username not specified');
      }
      if ($usr!='' && $dbintf->UsrExists($usr))
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.' Username is already in use: '.$usr);
      }
    }
		if ($action=='edit')
    {
			$accid = $existing_users[$editid]['accid'];
      if ($DupNamesAllowed==false
			&& $dbintf->NameInUseExcept($name,$accid))
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.' Name is already in use: '.$name);
      }
      if ($enabled && $dbintf->UsrExistsExcept($usr,$accid))
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.' Username is already in use: '.$usr);
      }
		}

    // make array
    $phones = explode(',',$parts[5]);
    $phones = array_map('trim',$phones);
    $phones = array_filter($phones);
    // maybe todo: determine how many are new, and say it
    $newphones = 0;
    foreach($phones as $phone)
    {
			// do not create contacts at this point
			$obj = ParsePhoneItem( $phone , $accid, false );
			if (isset($obj['err']))
			{
				$err = true;
				$msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': '.$obj['err']);
			}
			if ( (int) $dbintf->GetCachedConfigValue('UniquePhones') != 0 )
			{
				$other_accid = $dbintf->PhoneInUseExcept( $obj['contactid'] ,  $accid );
				if ($other_accid !== false)
				{
					$err = true;
					$other_usr = '???';
					if (isset($UsersOnAccid[$other_accid]))
					{
						$other_usr = $UsersOnAccid[$other_accid]['usr'];
					}
					$msg[] = new Msg(MSG_ERR,'Line '.$line_nr
					. ': The phone ' . $phone . ' is already attached to user ' . $other_usr );
				}
			}
			
    }

    // check if groups exist, otherwise fail
    // fill with acc-id's of groups
    $groups = explode(',',$parts[6]);
    // remove surrounding white space
    $groups = array_map('trim',$groups);
    // remove empty
    $groups = array_filter($groups);
    $groupaccs = array();
    foreach($groups as $group)
    {
      if ($group=='') continue;
      if (isset($groupnames[$group]))
      {
        $groupaccs[] = $groupnames[$group];
      } else
      {
        $err = true;
        $msg[] = new Msg(MSG_ERR,'Line '.$line_nr.': Unknown group: '.$group);
      }
    }

    $users[] = array
    ( 'id'      => $editid
    , 'name'    => $name
    , 'oldname' => $oldname
    , 'enabled' => $enabled
    , 'usr'     => $usr
    , 'pwd'     => $parts[4]
    , 'phones'  => $phones
    , 'groups'  => $groupaccs
    , 'email'   => $parts[7]
		, 'mphones' => $mphones
		, 'logonmethod' => $logonmethod
    , 'action'  => $action
    , 'recondmd'  => $recondmd
    , 'delondmd'  => $delondmd
    , 'silondmd'  => $silondmd
    , 'customlocalid'  => $customlocalid
    , 'freeseat'  => $freeseat
    , 'rod'  => $rod
    , 'ldapuser'  => $ldapuser
    , 'webclientsettings' => $webclientsettings
    );
  }
  return $users;
}

function ParsePhoneItem($phone_item, $useraccid, $docreate=true)
{
	global $dbintf, $linemap;
	$obj = array();
	if (substr($phone_item,0,3)=='IP#')
	{
		$ipaddr =  substr($phone_item,3);
		if ($docreate)
		{
			$obj['contactid'] = $dbintf->GetOrCreateIpAddr($ipaddr);
		} else
		{
			$obj['contactid'] = $dbintf->GetIpAddr($ipaddr);
		}
	} else if (substr($phone_item,0,5)=='LINE#')
	{
		$telline = substr($phone_item,5);
		if (isset($linemap[$telline])==false)
		{
			$obj['err'] = 'Line: '.$phone_item.' not found';
			return $obj;
		}
		$obj['contactid'] = $linemap[$telline];
	} else
	{
		$res = $dbintf->GetCalleridForUser($phone_item,$useraccid,$docreate);
		$obj['contactid'] = $res ? $res["contactid"] : null;
	}
	return $obj;
}

?>
