<?php

require_once "utility.php";

define('PM_LIST',				    1); 			//1
define('PM_DOWNLOAD',				1<<1); 		//2. previously PM_PLAY
define('PM_DELETE',					1<<2); 		//4
define('PM_VIEWPHONEBOOK',	1<<3); 		//8
define('PM_NOTES',					1<<4); 		//16
define('PM_CONFIG',					1<<5);		//32
define('PM_EDITNAMES',			1<<6); 		//64
define('PM_SUBADMIN',				1<<7);		//128
define('PM_CE_MANAGE',			1<<8);		//256
define('PM_CE_EVALUATE',		1<<9);		//512
define('PM_CE_VIEW',				1<<10);		//1024
define('PM_PLAY',						1<<11);		//2048
define('PM_USERS',					1<<12);		//4096
define('PM_FILTER',					1<<13);		//8192
define('PM_SYSALARM',				1<<14);		//16384
define('PM_PLAYSCR',				1<<15);		//32768
define('PM_ADDREC',     		1<<16);		//65536
define('PM_WEBCLIENT',			1<<17);		//131072
define('PM_CE_SUPERVISOR',	1<<18);		//262144
define('PM_CE_SCHEDULING',	1<<19);		//524288
define('PM_T2_ADMIN',				1<<20);
define('PM_T3_ADMIN',				1<<21);
define('PM_TENANT_ADMIN_AUDIT',1<<22);
define('PM_VIEW_TRANSCRIPT',   1<<23);
define('PM_TENANT_ADMIN_EXP',  1<<24);
define('PM_CE_VIEWOWNREPORTS',	1<<25);		
define('PM_CE_SELFEVALUATE',	1<<26);		
define('PM_CE_VIEW_LIVEDASH',	1<<27);		
define('PM_MASS_DELETE', 1<<28);
define('PM_TENANT_ADMIN_ENCRYPT', 1<<29);
define('PM_ACTIVEPLAY', 1<<30);

// helper class
class DbError extends Exception
{
	public function __construct($msg)
	{
		parent::__construct($msg);
	}
}


// connects to SQLite3 db
// reads call listing
// and does other DB actions

class DBInterface
{
  var $db;
	var $machinedb;
  var $lasterr;
	var $transactionLevel = 0;
	
  //var $actuid;
  //var $actaccid;
  var $accidlist = null; // show replace actaccid
  var $actuinfo = array();
	var $pcname; // for free-seating

  // permissions to access calls of contacts (array)
  var $ctpms = array();
  // permission to access user/group accounts (array)
  var $accpms = array();
	// the same, but aggregated single value
  var $anyaccpms = 0;
	// permission to access TDM lines
	var $linepms = array();
	// permission to access IP addresses
	var $ippms = array();
	// permission to access recorders
	var $recpms = array();
	
  var $allgroups = null;
  var $mysubgroups = array();

  // userphones
  // not garanteed completely filled
  // accid => array of contact-ids
  var $userphones = array();
	var $userphones_lines = array();
	var $userphones_ips = array();

  // boolean, has the user any permission other than to his own calls?
  var $otherpm = false;
	var $otherlistpm = false;
  var $myphones = null;
  var $hasphones = false;
	var $cachedsettings = array();
	var $readonly;
	var $nodePassive;
	var $clustering;
	// if sharedb is true, then we need a local machine.db
	var $sharedb;
	var $errorAsException;
  var $mygroups = array();

  function __construct($path, $dbName='apresa', $setErrorAsException=false)
  {
		$this->DetermineSystemSetup();
		$this->errorAsException = $setErrorAsException;
    if ($path!='')
    {
      if (DB_TYPE=='postgresql')
      {
        //$this->db = new PDO('pgsql:user=www-data;dbname=apresa');
        // it can be user www-data or also user root
				try
				{
					$this->db = new PDO('pgsql:dbname='.$dbName);

          // This was the default pre PHP 8 for which original development was done
          // so for safety keep it that way for now
          $this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
				} catch (Exception $e)
				{
					if ($this->errorAsException)
					{
						throw new Exception("ERROR-DB-Connect");
					}
					// inform user
					echo "Could not connect to database";
					// to apresa.log
					lg($e);
					// do not proceed further
					exit;
				}
      }
      if (DB_TYPE=='sqlite3')
      {
        $this->db = new PDO('sqlite:'.$path.'/vicalls.db');
        $this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
      }
			if ($this->sharedb)
			{
				$this->machinedb = new PDO('sqlite:'.$path.'/machine.db');
        $this->machinedb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
			}
    }
    
    $this->ctpms = array();
		$this->linepms = array();
		$this->ippms = array();
    $this->accpms = array();
		$this->recpms = array();
    $this->anyaccpms = 0;
    $this->actuinfo['permissions'] = 0;
    $this->actuinfo['id'] = null;
    $this->actuinfo['tenantid'] = null;
    $this->actuinfo['usr'] = null;
    $this->actuinfo['accid'] = null;
    $this->actuinfo['playtimelim'] = null;
  }
	
	function DetermineSystemSetup()
	{
		$this->readonly = false;
		$this->nodePassive = false;
		$this->clustering = false;
		$this->sharedb = false;
		if (file_exists(dirname(__FILE__) . '/../apresa/conf/opt_cluster'))
		{
			$this->clustering = true;
			if (file_exists(dirname(__FILE__) . '/' . DATA_PATH . '/node_active')==false)
			{
				$this->nodePassive = true;
				$this->readonly = true;
			}
		}
		if (file_exists(dirname(__FILE__) . '/../apresa/conf/opt_sharedb'))
		{
			$this->sharedb = true;
		}
		
	}

  function cloneInit(& $ref_db)
  {
    $this->db = $ref_db->db;
  }

  function close()
  {
    $this->db = null;
  }

  function GetContactListNormal()
  {
    $result = array();
    $sql = 'SELECT id, name, deleted FROM Contacts ORDER BY name';
    foreach($this->db->query($sql) as $row)
    {
      $result[] = $row;
    }
    return $result;
  }

  function getADGroupLinks()
  {
    $result = array();
    $sql = "SELECT id, accid, name, adgroup, ldaphost FROM Groups WHERE adgroup IS NOT NULL AND adgroup <> ''";
    foreach($this->db->query($sql) as $row)
    {
      $result[] = $row;
    }
    return $result;
  }

  function GetContactInfo($id)
  {
    $stmt = $this->db->prepare('SELECT * FROM Contacts WHERE id=:1');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result === false)
    {
      return $result;
    }
    $result['t'] = $result['value'];

    if ($this->UseTenantContactList())
    {
      $result['name'] = '';
      // reached if a tenant user has edit user accounts permission
      $tenantid = (int) $this->actuinfo['tenantid'];
      $stmt = $this->db->prepare('SELECT * FROM TenantContacts WHERE contactid=:1 AND tenantid=:2');
      $stmt->bindValue(1,$id);
      $stmt->bindValue(2,$tenantid);
      $stmt->execute();
      $res2 = $stmt->fetch();
      if ($res2 !== false)
      {
        $result['name'] = $res2['name'];
      }
    }
    
    $ctid = (int) $id;

    $stmt = $this->db->query('SELECT * FROM CallerIDs WHERE contactid='.$ctid);
    if ($res2 = $stmt->fetch())
    {
      $result['callerid'] = $res2['callerid'];
      $result['islocal'] = $res2['islocal'];
			$result['cidid'] = $res2['id'];
      $result['ctype'] = 0;
			$result['t'] = $result['callerid'];
    }

    $stmt = $this->db->query('SELECT * FROM Lines WHERE contactid='.$ctid);
    if ($res2 = $stmt->fetch())
    {
      $result['linenr'] = $res2['linenr'];
      $result['islocal'] = 1;
      $result['ctype'] = 1;
			$result['t'] = '#' . $result['linenr'];
    }

    return $result;
  }

  function UseTenantContactList()
  {
    if (isset($this->actuinfo['tenantid']))
    {
      $TenantContactList = $this->GetCachedConfigValue('TenantContactList')!=0;
      $MultiTenancy = $this->GetCachedConfigValue('MultiTenancy')!=0;
      if ($MultiTenancy && $TenantContactList)
      {
        return true;
      }
    }
    return false;
  }
  
  function GetContactName($id)
  {
    if ($this->UseTenantContactList())
    {
      $tenantid = (int) $this->actuinfo['tenantid'];
      $stmt = $this->db->prepare('SELECT name FROM TenantContacts WHERE contactid=:1 AND tenantid=:2');
      $stmt->bindValue(1,$id);
      $stmt->bindValue(2,$tenantid);
      $stmt->execute();
      $result = $stmt->fetch();
      if ($result===false) return false;
      return $result['name'];
    }
    $stmt = $this->db->prepare('SELECT name FROM Contacts WHERE id=:1');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    $result = $stmt->fetch();
    if($result===false) return false;
    return $result['name'];
  }

	function GetAccidUsr($usr)
	{
    $stmt = $this->db->prepare('SELECT accid FROM Users WHERE usr=:1');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result['accid'];
	}
	
	function GetCalleridForUser($callerid,$useraccid,$create)
  {
    // prefer existing entry
		$stmt = $this->db->prepare('SELECT * FROM CallerIDs WHERE callerid=:1 '
		. ' AND contactid IN (SELECT contactid FROM UserPhones WHERE accid=:2)'
		);
    $stmt->bindValue(1,$callerid);
    $stmt->bindValue(2,$useraccid);
    $stmt->execute();
		$res = $stmt->fetch(PDO::FETCH_ASSOC);
    if ($res===false)
		{
			if ($create)
			{
				return $this->GetOrCreateCallerID($callerid,'',2);
			} else
			{
				return $this->GetCallerIDonCallerID($callerid,false);
			}
		} else
		{
			return $res;
		}
	}
	
  function GetCallerIDonCallerID($callerid,$alsodeleted=true)
  {
		$sql = 'SELECT * FROM CallerIDs WHERE callerid=:1';
		if ( ! $alsodeleted )
		{
			$sql .= ' AND deleted=0';
		}
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$callerid);
    $stmt->execute();
    return $stmt->fetch(PDO::FETCH_ASSOC);
  }

  function GetCallerID($id)
  {
    $stmt = $this->db->prepare('SELECT * FROM CallerIDs WHERE id=:1');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    return $stmt->fetch(PDO::FETCH_ASSOC);
  }

  function GetLineName($id)
  {
    $stmt = $this->db->prepare('SELECT name FROM Contacts '
		.' JOIN Lines ON Contacts.id=Lines.contactid '
		.' WHERE Lines.id=:1 ');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    return $stmt->fetchColumn(0);
  }
	
  // This function would be expanded when other types of
  // caller ID get supported
  function GetContactList($onlylocal=false)
  {
    $result = array();
    $sql = 'SELECT Contacts.id, name, '
    . ' CallerIDs.callerid, CallerIDs.callerid2, Contacts.deleted '
    . ' FROM Contacts JOIN CallerIDs ON Contacts.id=CallerIDs.contactid ';
    if ($onlylocal)
    {
      $sql .= ' WHERE CallerIDs.islocal=1 ';
    }
    $sql .= ' ORDER BY islocal DESC,callerid,callerid2 ';
    foreach($this->db->query($sql) as $row)
    {
      $result[] = $row;
    }
    return $result;
  }

  function GetLineList()
  {
    $result = array();
    if ($this->UseTenantContactList())
    {
      $tenantid = (int) $this->actuinfo['tenantid'];
      $sql = 'SELECT Lines.id, linenr, tc.name, tc.deleted, tc.contactid, recorderid '
      . ' FROM TenantContacts as tc JOIN Lines ON tc.contactid=Lines.contactid AND tc.tenantid='.$tenantid;
    } else
    {
      $sql = 'SELECT Lines.id, linenr, Contacts.name, Contacts.deleted, contactid, recorderid '
      . ' FROM Contacts JOIN Lines ON Contacts.id=Lines.contactid ';
    }
    $sql .= ' ORDER BY linenr ';
    // line contacts cannot be deleted (currently)
    foreach($this->db->query($sql) as $row)
    {
      $result[] = $row;
    }
    return $result;
  }

  function GetCallerIDCount()
  {
    return $this->db->query('SELECT COUNT(id) FROM CallerIDs')->fetchColumn(0);
  }

  function GetCallerIDSearchCount($alsodeleted,$search)
  {
    $sql  = 'SELECT COUNT(CallerIDs.id) FROM CallerIDs ';
    if ($this->UseTenantContactList())
    {
      $tenantid = (int) $this->actuinfo['tenantid'];
      $sql .= 'JOIN TenantContacts as ct ON CallerIDs.contactid = ct.contactid AND ct.tenantid=' . $tenantid . ' ';
    } else
    {
      $sql .= 'JOIN contacts as ct ON contactid = ct.id ';
    }
    $sql .= 'WHERE '.$this->GetTrue().' ';
    if ($alsodeleted==false)
    {
      $sql .= ' AND ct.deleted=0 ';
    }
    if (isset($search['name']))
    {
      $sql .= ' AND ( name '.$this->GetILike().' :1 OR callerid '.$this->GetILike().' :1 )';
    }
    $stmt = $this->db->prepare($sql);
    if ($stmt==null)
    {
      $this->OnDbErr();
    }
    if (isset($search['name']))
    {
      $stmt->bindValue(1,'%'.$search['name'].'%');
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return $stmt->fetchColumn(0);
  }

  function GetCallerIDPartialList($alsodeleted,$offset,$cnt,$search)
  {
		$cnt = (int) $cnt;
		$offset = (int) $offset;
    $result = array();
    if ($this->UseTenantContactList())
    {
      $tenantid = (int) $this->actuinfo['tenantid'];
      $sql = 'SELECT CallerIDs.id,callerid,callerid2,islocal,CallerIDs.contactid '
      . ' ,ct.name,ct.deleted '
      . ' FROM CallerIDs JOIN TenantContacts as ct ON CallerIDs.contactid = ct.contactid AND tenantid=' . $tenantid;
      $sql .= ' WHERE '.$this->GetTrue().' ';
    } else
    {
      $sql = 'SELECT CallerIDs.id,callerid,callerid2,islocal,contactid,contacts.name,contacts.deleted '
      . ' FROM CallerIDs JOIN contacts ON contactid = contacts.id';
      $sql .= ' WHERE '.$this->GetTrue().' ';
      if ($alsodeleted==false)
      {
        $sql .= ' AND contacts.deleted=0 ';
      }
    }
    if (isset($search['name']))
    {
      $sql .= ' AND ( name '.$this->GetILike().' :1 OR callerid '.$this->GetILike().' :1 )';
    }
    // no sorting for speed
    //$sql .= ' ORDER BY islocal DESC,callerid,callerid2 ';
    $sql .= ' ORDER BY length(callerid),callerid ';
    $sql .= ' LIMIT '.$cnt.' OFFSET '.$offset;

    $stmt = $this->db->prepare($sql);
    if (isset($search['name']))
    {
      $stmt->bindValue(1,'%'.$search['name'].'%');
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $result = array();
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
  }

  function GetCallerIDList($alsodeleted=false)
  {
    $result = array();
    $sql = 'SELECT CallerIDs.id,callerid,callerid2,islocal,contactid,contacts.name,contacts.deleted '
    . ' FROM CallerIDs JOIN contacts ON contactid = contacts.id';
    if ($alsodeleted==false)
    {
      $sql .= ' WHERE contacts.deleted=0 ';
    }
    $sql .= ' ORDER BY islocal DESC,callerid,callerid2 ';
    foreach($this->db->query($sql) as $row)
    {
      $result[] = $row;
    }
    return $result;
  }

  function DelContact($ctids,$cids,$del)
  {
    if (($this->actuinfo['permissions']&(PM_CONFIG|PM_T2_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    $stmt = $this->db->prepare('UPDATE Contacts SET deleted=:1 WHERE id=:2');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    foreach($ctids as $id)
    {
      $stmt->bindValue(1,$del);
      $stmt->bindValue(2,$id);
      $res = $stmt->execute();
    }
    $stmt = $this->db->prepare('UPDATE CallerIDs SET deleted=:1 WHERE id=:2');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    foreach($cids as $id)
    {
      $stmt->bindValue(1,$del);
      $stmt->bindValue(2,$id);
      $res = $stmt->execute();
    }
    return true;
  }

  // contact id, callerid-id, local (boolean), contact name
  function EditContact2($ctid,$cid,$local,$name)
  {
    if (($this->actuinfo['permissions']&(PM_EDITNAMES|PM_T2_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    // update name
    $stmt = $this->db->prepare('UPDATE Contacts SET name=:1 WHERE id=:2');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$name);
    $stmt->bindValue(2,$ctid);
    $res = $stmt->execute();
    // update caller id info
    $stmt = $this->db->prepare('UPDATE CallerIDs SET islocal=:1 WHERE id=:2');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$local);
    $stmt->bindValue(2,$cid);
    $res = $stmt->execute();
    return true;
  }

  function EditContactForCall($ctid,$callid,$newname)
  {
    $allowed = false;
    $pm = $this->actuinfo['permissions'];
    if ( ($pm & PM_T2_ADMIN)!=0 ) $allowed = true;
    else if ( ($pm & PM_EDITNAMES)!=0 
     &&  ($pm & PM_VIEWPHONEBOOK)!=0 ) $allowed = true;
    else if ( ($pm & PM_EDITNAMES)!=0 )
    {
      // The user has Edit names permission,
      // but not access to the whole phone book.
      // Then restrict to calls it can see.
      $ci = $this->GetCallInfo($callid);
      if ($ci!==false)
      {
        // If the contact is actually part of this call,
        $partOfCall = false;
        if ( $ci['localcontact'] == $ctid ) $partOfCall = true;
        if ( $ci['remotecontact'] == $ctid ) $partOfCall = true;
        if ( $ci['connectedcontact'] == $ctid ) $partOfCall = true;
        // and listing permission for this call, then it is allowed
        if ( $partOfCall && ( $ci['permissions'] & PM_LIST ) != 0 ) $allowed = true;
        // Note that this does not prevent leackage if contacts are shared,
        // that is, if the same contact is used for calls of different tenants for example.
      }
    }
    if ($allowed==false)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    // update name
    if ($this->UseTenantContactList())
    {
      $tenantid = (int)$this->actuinfo['tenantid'];
      if ($this->GetContactName($ctid)===false)
      {
        $stmt = $this->db->prepare('INSERT INTO TenantContacts(name,contactid,tenantid) VALUES(:1,:2,:3)');
      } else
      {
        $stmt = $this->db->prepare('UPDATE TenantContacts SET name=:1 WHERE contactid=:2 AND tenantid=:3');
      }
      if (!$stmt)
      {
        $this->OnDbErr();
      }
      $stmt->bindValue(1,$newname);
      $stmt->bindValue(2,$ctid);
      $stmt->bindValue(3,$tenantid);
      $res = $stmt->execute();
      return true;
    }
    
    $stmt = $this->db->prepare('UPDATE Contacts SET name=:1 WHERE id=:2');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$newname);
    $stmt->bindValue(2,$ctid);
    $res = $stmt->execute();
    return true;
  }
  
  function AddTenantContact($ctid,$name)
  {
    $tenantid = (int)$this->actuinfo['tenantid'];
    if ($this->GetContactName($ctid)!==false)
    {
      $this->lasterr = 'already-exists';
      return false;
    } 
    $stmt = $this->db->prepare('INSERT INTO TenantContacts(name,contactid,tenantid) VALUES(:1,:2,:3)');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$name);
    $stmt->bindValue(2,$ctid);
    $stmt->bindValue(3,$tenantid);
    $res = $stmt->execute();
    return true;
  }
  
  function EditContact($ctid,$newname,$extra='')
  {
    if (($this->actuinfo['permissions']&(PM_EDITNAMES|PM_T2_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    // update name
    if ($this->UseTenantContactList())
    {
      $tenantid = (int) $this->actuinfo['tenantid'];
      $stmt = $this->db->prepare('UPDATE TenantContacts SET name=:1 WHERE contactid=:2 AND tenantid=:3');
      if (!$stmt)
      {
        $this->OnDbErr();
      }
      $stmt->bindValue(1,$newname);
      $stmt->bindValue(2,$ctid);
      $stmt->bindValue(3,$tenantid);
      $res = $stmt->execute();
      return true;
    }
    $stmt = $this->db->prepare('UPDATE Contacts SET name=:1,extra=:2 WHERE id=:3');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$newname);
    $stmt->bindValue(2,$extra);
    $stmt->bindValue(3,$ctid);
    $res = $stmt->execute();
    return true;
  }

  function GetCallsBetween($ts1,$ts2,$not_recorderid)
  {
    $stmt = $this->db->prepare('SELECT * FROM calls '
    . ' WHERE startdate >= :1 AND startdate <= :2 '
    . ' AND recorderid <> :3 '
    . ' ORDER BY id DESC');
    $stmt->bindValue(1,$ts1);
    $stmt->bindValue(2,$ts2);
    $stmt->bindValue(3,$not_recorderid);
    $stmt->execute();
    $result = [];
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
		return $result;
  }
  
  function GetCallBefore($timestamp)
  {
    $stmt = $this->db->prepare('SELECT id FROM calls WHERE startdate<:1 ORDER BY id DESC LIMIT 1');
    $stmt->bindValue(1,$timestamp);
    $stmt->execute();
		$res = $stmt->fetchColumn(0);
		return $res;
    
  }
  
	function GetIdOfLatestCall()
	{
    $stmt = $this->db->prepare('SELECT id FROM calls ORDER BY id DESC LIMIT 1');
    $stmt->execute();
		$res = $stmt->fetchColumn(0);
		if ($res===false) return 0;
		return $res;
	}
	
  // Get last call of the current user
  // ix==0, last call (not busy)
  // ix==1, one before last
  // etc....
  function GetLastOwnCall($ix)
  {
		if ($ix<0) return NULL;
		$listconfig = array();
	  $listconfig['startfrom'] = 0;
	  $listconfig['checkrange'] = $ix+1;
	  $listconfig['loadrange'] = $ix+1;
		$search = array();
		$search['own'] = 1;
		$sk = 'startdate';
		$sd = '';
		$list = $this->GetCallList($sk,$sd,$search,$listconfig);
		if ($ix>=$list['cnt']) return NULL;
		unset($list['cnt']);
		return $list[$ix];
  }

	// returns an array of all phone contacts in this group recursively
	function GetAccidPhoneContacts($accid,$OnlyEnabled)
	{
		$ret = array();
		$this->DetermineAllGroups();
		if (isset($this->allgroups[$accid]))
    {
			// the accid refers to a group
	    // examine this group
			$groups = array($accid);
			// but also subgroups
			$this->FindSubGroups($groups,$accid);
			// find group members...
			foreach ($groups as $groupaccid)
			{
				$members = $this->GetGroupMembers($groupaccid);
				foreach ($members as $member)
				{
					$ret = array_merge( $ret , $this->GetAccidPhoneContacts($member,$OnlyEnabled) );
				}
			}
		} else
		{
			// the accid refers to a user
			// get phones if not cached already
			$this->EnsureUserPhones($accid);
      if ($OnlyEnabled)
      {
        $uinfo = $this->GetUserInfoOnAccId($accid);
        if ($uinfo['enabled']==0) return $ret;
      }
			$ret = $this->userphones[$accid];
		}
		return $ret;
	}
  
  function HasNewerCall($localcontact, $startdate, $active)
  {
    $tabname = ($active) ? 'ActiveCalls' : 'Calls';
    $sql = 'select id from ' . $tabname . ' where startdate>? and localcontact=?';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$startdate);
    $stmt->bindValue(2,$localcontact);
    $stmt->execute();
    if ($stmt->fetch(PDO::FETCH_ASSOC)===false) return false;
    return true;
  }
	
	function GetExpiredSodCalls($delay, $batchSize)
	{
		$secondsDelay = ((int) $delay) * 60;
		$tsDel = time() - $secondsDelay;
    
		$sql = 'select * from activecalls where '
				. ' (processingstate&128)<>0 '
				. ' order by startdate desc ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    // loop through records
		$localsSeen = [];
    $cnt = 0;
    $rows = [];
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $toDelete = ($row['startdate']+$row['duration'] < $tsDel);
      if ($toDelete==false)
      {
        $local = $row['localcontact'];
        if (isset($localsSeen[ $local ])) $toDelete = true;
      }
      if ($toDelete==false)
      {
        // maybe still delete if a more recent call
        // look in ActiveCalls and in normal Calls
        if ($this->HasNewerCall($local,$row['startdate'],false)
          || $this->HasNewerCall($local,$row['startdate'],true) )
        {
          // we loop from new to old,
          // so for the older calls, there is also a newer,
          // unless that one gets deleted,
          // but if that is so, the older one would get deleted
          // for the same reason, so cache:
          $localsSeen[ $local ] = true;
          $toDelete = true;
        }
      }
      if ($toDelete)
      {
        $rows[] = $row;
        $cnt ++;
        if ($cnt == $batchSize) break;
      }
		}
    return $rows;
	}
	
	function GetCallsForExport($listconfig)
	{
		$contacts_filter = array();
		$filter = ' WHERE '.$this->GetTrue().' ';
		if ( isset($listconfig['afterid']) )
		{
			$filter .= ' AND id > ' . (int)$listconfig['afterid'];
		}
		if (isset($listconfig['accidfilter'])
		&& $listconfig['accidfilter']!=0)
		{
      $accid = (int) $listconfig['accidfilter'];
      $tenantid = null;
      $tenantFilter = false;
			$accIdFilter = true;
      $group = $this->GetGroupInfoOnAccid( $accid );
      if ($group !== false)
      {
        if ($group['is_tenant']==1)
        {
          $tenantFilter = true;
					$accIdFilter = false;
          $tenantid = $group['id'];
        }
      }
			$user = $this->GetUserInfoOnAccId( $accid );
			if ($user !== false)
			{
				if ($user['tenantid']!=null)
				{
					$tenantFilter = true;
					// also apply accIdFilter
					$tenantid = $user['tenantid'];
				}
			}
      if ($tenantFilter)
      {
        $filter .= " AND tenantid = $tenantid ";
      }
			if ($accIdFilter)
      {
        // accid filter
        $contacts_filter = $this->GetAccidPhoneContacts( $accid , false );
        $filter .= $this->GetEnhContactsFilter($contacts_filter);
      }
		}
    if ( isset($listconfig['minage'])
      && $listconfig['minage']!=0 )
    {
      $maxend = (int) ( time() - $listconfig['minage'] );
      $filter .= " AND startdate + duration <= $maxend ";
    }
		//echo $filter . "\n";
    $sqlstmt = 'SELECT * FROM Calls'
		. $filter
    . ' ORDER BY ID asc';
    if ($listconfig['allrecords']==false)
    {
      $sqlstmt .= ' LIMIT '.(int)$listconfig['loadrange'].' OFFSET '.(int)$listconfig['startfrom'];
  	}
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    // Now fetch the records
    $cnt = 0;
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $cnt ++;
      if (($listconfig['allrecords']==true) or ($cnt<=$listconfig['loadrange']))
      {
        if (isset($listconfig['callback']))
        {
          $listconfig['callback']($row,$listconfig);
        } else
        {
          $result[] = $row;
        }
      }
    }
    $result['cnt'] = $cnt;
    return $result;
	}
	
  function TextLikeSearch($t,$exact)
  {
    if ($exact) // fixed length
    {
      $t = str_replace("%","\\%",$t);
      $t = str_replace("_","\\_",$t);
      // still replace
      $t = str_replace("?","_",$t);
      $t = str_replace("*","%",$t);
    } else
    {
      $t = trim($t);
      $t = str_replace("%","\\%",$t);
      $t = str_replace("_","\\_",$t);
      $ender = (substr($t,-1)=='*');
      $starter = (substr($t,0,1)=='*');
      $t = str_replace("*","%",$t);
      $t = str_replace("?","_",$t);
      if (!$ender && !$starter) $t = '%' . $t . '%';
    }
    return $t;
  }

  function AddPmFilter($pmneed)
  {
    $UsrFilterLocal = $this->GetCachedConfigValue('UsrFilterLocal')!=0;
    $filter = '';
    $ctlist = '';
    $first = true;
    foreach ($this->ctpms as $contactid=>$pm)
    {
      if (($pm & $pmneed)==0) continue;
      
      if ($first==false) $ctlist .= ',';
      $ctlist .= (int) $contactid;
      $first = false;
    }

    $linelist = '';
    $first = true;
    foreach ($this->linepms as $linenr=>$pm)
    {
      if (($pm & $pmneed)==0) continue;
      if ($first==false) $linelist .= ',';
      $linelist .= (int) $linenr;
      $first = false;
    }

    $iplist = '';
    $first = true;
    foreach ($this->ippms as $ipaddr=>$pm)
    {
      if (($pm & $pmneed)==0) continue;
      if ($first==false) $iplist .= ',';
      $iplist .= $this->db->quote($ipaddr);
      $first = false;
    }
		
		$reclist = '';
		$first = true;
		foreach ($this->recpms as $recorderid=>$pm)
		{
      if (($pm & $pmneed)==0) continue;
      if ($first==false) $reclist .= ',';
      $reclist .= $recorderid;
      $first = false;
		}

    $filter .= ' AND ('.$this->GetIn('localcontact',$ctlist);
    if ($UsrFilterLocal==false)
    {
      $filter .= ' OR '.$this->GetIn('remotecontact',$ctlist);
      $filter .= ' OR '.$this->GetIn('connectedcontact',$ctlist);
    }
    $filter .= ' OR '.$this->GetIn('linenr',$linelist);
    $filter .= ' OR '.$this->GetIn('local2',$iplist);
    if ($UsrFilterLocal==false)
    {
      $filter .= ' OR '.$this->GetIn('remote2',$iplist);
    }
		if ($reclist!='')
		{
			$filter .= ' OR ' . $this->GetIn('recorderid',$reclist);
		}
    $filter .= ') ';
    return $filter;
  }

  function IdSearchFilterDirect($searchitems, $cols, $exact)
  {
    $clauses = [];
		$tobind = [];
    foreach ($searchitems as $searchitem)
    {
      $searchitem = $this->TextLikeSearch($searchitem, $exact);
      $parts = [];
      foreach($cols as $col)
      {
        $parts[] = "{$col} {$this->GetILike()} ? {$this->GetEscape()}";
        $tobind[] = $searchitem;
      }
      $clauses[] = implode(" OR ", $parts);
    }

    return [
      "tobind" => $tobind,
      "filter" => implode(" OR ", $clauses)
    ];
  }

  function IdSearchFilterCallerIDs($searchitems, $cols, $exact)
  {
    $clauses = [];
    $idbind = [];

    foreach ($searchitems as $searchitem)
    {
      $searchitem = $this->TextLikeSearch($searchitem, $exact);
      $clauses[] = "simplified {$this->GetILike()} ? {$this->GetEscape()}";
      $idbind[] = $searchitem;
    }

    $filter = " " . implode(" OR ", $clauses);
    $ids = $this->GetCallerIDSearchIDs($filter, $idbind);

    if(count($ids) <= 100)
    {
      $parts = [];
      foreach($cols as $col)
      {
        $parts[] = "{$col}callerid " . (count($ids) > 0 ? "IN (" . implode(',', $ids) . ")" : "IS NULL");
      }

      return [
        "tobind" => [],
        "filter" => implode(" OR ", $parts)
      ];
    }
    else
    {
      return $this->IdSearchFilterDirect($searchitems, $cols, $exact);
    }
  }

  function IdSearchFilter($searchitems, $cols, $exact)
  {
    $searchitems = explode(',',$searchitems);
    $first = true;

    $positives = [];
    $negatives = [];

    foreach($searchitems as $searchitem)
    {
      if (substr($searchitem,0,1)=='!')
      {
        $negatives[] = substr($searchitem,1);
      }
      else
      {
        $positives[] = $searchitem;
      }
    }

    $OptimizedPath = $this->GetCachedConfigValue('OptimizedDb')!=0;    
    
    $filterparts = [];
    $tobind = [];
    
    if(count($positives) > 0)
    {
      $ret = $OptimizedPath ? $this->IdSearchFilterCallerIDs($positives, $cols, $exact) : $this->IdSearchFilterDirect($positives, $cols, $exact);
      $filterparts[] = "({$ret['filter']})";
      $tobind = array_merge($tobind, $ret['tobind']);
    }

    if(count($negatives) > 0)
    {
      $ret = $OptimizedPath ? $this->IdSearchFilterCallerIDs($negatives, $cols, $exact) : $this->IdSearchFilterDirect($negatives, $cols, $exact);
      $filterparts[] = "NOT ({$ret['filter']})";
      $tobind = array_merge($tobind, $ret['tobind']);
    }

    return [
      "filter" => implode(" AND ", $filterparts),
      "tobind" => $tobind
    ];
  } 
  
  function SearchFilter($search,$listconfig)
  {
			$UsrFilterLocal = $this->GetCachedConfigValue('UsrFilterLocal')!=0;
			$MultiTenancy = $this->GetCachedConfigValue('MultiTenancy')!=0;
			$exactmap = array(
      'l1'=>'eid',
      'r1'=>'eid',
      'c1'=>'eid',
      'l2'=>'eip',
      'r2'=>'eip',
      'ln'=>'enm',
      'rn'=>'enm',
      'cn'=>'enm',
      'nts'=>'ents',
      'nts2'=>'ents',
      'nts3'=>'ents',
      'nts3'=>'ents',
      'nts4'=>'ents',
      'nts5'=>'ents',
			'dts1'=>'ents',
			'dts2'=>'ents',
			'dts3'=>'ents',
			'dts4'=>'ents',
			'dts5'=>'ents',
      );
      $colmap = array(
      'l1'=>'local',
      'r1'=>'remote',
      'c1'=>'connected',
      'l2'=>'local2',
      'r2'=>'remote2',
      'ln'=>'cloc.name',
      'rn'=>'crem.name',
      'cn'=>'ccon.name',
      'nts'=>'notes',
			'nts2'=>'notes2',
			'nts3'=>'notes3',
			'nts4'=>'notes4',
			'nts5'=>'notes5',
			'dts1'=>'data1',
			'dts2'=>'data2',
			'dts3'=>'data3',
			'dts4'=>'data4',
			'dts5'=>'data5',
      'dir'=>'direction',
      'prjid'=>'projectid',
      'lnr'=>'linenr',
      'cat'=>'category',
      'rem'=>'remoteid',
			'rb'=>'recyclebin',
      'rec'=>'recorderid',
      'tenantid'=>'tenantid',
			'agent'=>'agent'
      );
      $always_exact = array( 'dir'=>1, 'prjid'=>1, 'lnr'=>1, 'cat'=>1
      , 'rem'=>1, 'rb'=>1 , 'rec'=>1, 'tenantid'=>1, 'agent'=>1 );

      // Check if and which filtering we need to apply
      // Check global permission
      $filter = ' WHERE '.$this->GetTrue().' ';
      if (isset($listconfig['nousrflr'])==false)
      {
        if (($this->actuinfo['permissions']&PM_LIST)==0)
        {
          $filter .= $this->AddPmFilter(PM_LIST);
          }
        if (isset($search['tra']) && $search['tra']!='')
          {
          if (($this->actuinfo['permissions']&PM_VIEW_TRANSCRIPT)==0)
          {
            $filter .= $this->AddPmFilter(PM_VIEW_TRANSCRIPT);
          }
					}
				// apply multi-tenancy filter, even when full list permission
				if ($MultiTenancy && $this->actuinfo['tenantid']!==null)
				{
					$filter .= ' AND Calls.tenantid='.(int)$this->actuinfo['tenantid'];
				}
      }

      $tobind = array();
      // searching
			if (isset($search['rb'])==false)
			{
				$search['rb'] = 0;
			}
      foreach ($search as $searchcol=>$searchvalue)
      {
        switch($searchcol)
        {
          case 'fd':
            $filter .= ' AND startdate >= ? ';
            $tobind[] = $searchvalue;
            break;
          case 'ld':
            $filter .= ' AND startdate <= ? ';
            $tobind[] = $searchvalue;
            break;
          case 'ft':
            if (DB_TYPE=='sqlite3')
            {
              $filter .= ' AND time(startdate,\'unixepoch\',\'localtime\') >= ? ';
            } else
            {
              $filter .= ' AND to_timestamp(startdate)::time >= ? ';
            }
            $tobind[] = $searchvalue;
            break;
          case 'lt':
            if (DB_TYPE=='sqlite3')
            {
              $filter .= ' AND time(startdate,\'unixepoch\',\'localtime\') <= ? ';
            } else
            {
              $filter .= ' AND to_timestamp(startdate)::time <= ? ';
            }
            $tobind[] = $searchvalue;
            break;
          case 'fdur':
            $filter .= ' AND duration >= ? ';
            $tobind[] = $searchvalue;
            break;
          case 'ldur':
            $filter .= ' AND duration <= ? ';
            $tobind[] = $searchvalue;
            break;
          case 'id':
            $idf = $this->IdSearchFilter($searchvalue, ['local', 'remote', 'connected'], isset($search['eid']));
            $filter .= ' AND ' . $idf['filter'];
            $tobind = array_merge($tobind, $idf['tobind']);
            break;
          case 'ip':
            $searchitem = $this->TextLikeSearch($searchvalue,isset($search['eip']));
            $filter .= ' AND (local2 '.$this->GetILike()." ? ".$this->GetEscape()." OR remote2 ".$this->GetILike()." ?  ".$this->GetEscape().") ";
            $tobind[] = $searchitem;
            $tobind[] = $searchitem;
            break;
          case 'nm':
            $OptimizedPath = $this->GetCachedConfigValue('OptimizedDb')!=0;   
            $searchitem = $this->TextLikeSearch($searchvalue,isset($search['enm']));

            if($OptimizedPath)
            {
              $ids = $this->GetContactIDSearchIDs($searchitem);

              if(sizeof($ids) < 100)
              {
                $idsf = sizeof($ids) > 0 ? implode(',', $ids) : "NULL";
                $filter .= ' AND (localcontact IN (' . $idsf . ') OR remotecontact IN (' . $idsf . ') OR connectedcontact IN (' . $idsf . ')) ';
              }
              else
              {
                $filter .= ' AND (cloc.name '.$this->GetILike()." ? ".$this->GetEscape()." OR crem.name ".$this->GetILike()." ? ".$this->GetEscape()." OR ccon.name ".$this->GetILike()." ? ".$this->GetEscape().")";
                $tobind[] = $searchitem;
                $tobind[] = $searchitem;
                $tobind[] = $searchitem;
              }
            }
            else
            {
              $filter .= ' AND (cloc.name '.$this->GetILike()." ? ".$this->GetEscape()." OR crem.name ".$this->GetILike()." ? ".$this->GetEscape()." OR ccon.name ".$this->GetILike()." ? ".$this->GetEscape().")";
              $tobind[] = $searchitem;
              $tobind[] = $searchitem;
              $tobind[] = $searchitem;
            }
            break;
          case 'eid': case 'eip': case 'enm': case 'ents':
            // to be ignored
            break;
          case 'own':
            if ($searchvalue==1)
            {
              $phone_list = $this->GetUserPhones($this->actuinfo['accid']);
							$filter .= $this->GetEnhContactsFilter($phone_list);
            }
            break;
          case 'dir':
            if ($searchvalue==2)
            {
              $filter .= " AND (direction=0 OR direction=1) ";
            } else {
              $filter .= " AND direction=? ";
              if ($searchvalue==3)
              {
                $tobind[] = 2;
              } else
              {
                $tobind[] = $searchvalue;
              }
            }
            break;
					case 'ceusr':
					$filter .= ' AND useraccid='.(int)$searchvalue;
					break;
          case 'usr':
          // similar to 'own'
          $phone_list = $this->GetUserPhones($searchvalue);
					$filter .= $this->GetEnhContactsFilter($phone_list);
          break;
          case 'projects';
          $filter .= ' AND ' . $this->GetIn('projectid',implode(',',$searchvalue)).' ';
          break;
          case 'r1': case 'l1': case 'c1':
            $exactid = $exactmap[$searchcol];
            $idf = $this->IdSearchFilter($searchvalue, [$colmap[$searchcol]], isset($search[$exactid]));
            $filter .= ' AND ' . $idf['filter'];
            $tobind = array_merge($tobind, $idf['tobind']);
            break;
					case 'lnr':
            $searchitems = explode(',',$searchvalue);
            $first = true;
            $filter .= ' AND (';
            foreach ($searchitems as $searchitem)
            {
							if (!$first) $filter .= ' OR ';
              $first = false;
							$filter .= " linenr = ? ";
              $tobind[] = $searchitem;
            }
            $filter .= ') ';
						break;
					case 'tra':
            $parts = explode(' ',$searchvalue);
            $q = '';
            if ($searchvalue!='' && count($parts)>0)
            {
              $first = true;
              foreach ($parts as $part)
              {
                $tp = trim($part);
                if ($tp=='') continue;
                if ($first==false) $q .= ' & ';
                $q .= $tp;
                $first = false;
              }
            }
            if ($searchvalue!='')
            {
              if (DB_TYPE=='postgresql')
              {
                $filter .= " AND Calls.id IN "
                . "( SELECT call_id FROM Transcriptions WHERE "
                . " to_tsvector('simple',transcript) @@ to_tsquery('simple',?) ) "
                ;
                $tobind[] = $q;
              } else
              {
                // not supported yet for sqlite
                $filter .= " AND " . $this->GetFalse();
              }
            }
            break;
					case 'ann':
            $parts = explode(',',$searchvalue);
            if ($searchvalue!='' && count($parts)>0)
            {
              $filter .= " AND Calls.id IN "
              .        "( SELECT call_id FROM Annotations WHERE ";
              $first = true;
              foreach ($parts as $part)
              {
                if ($first==false) $filter .= ' OR ';
                $filter .= " descr ".$this->GetILike()." ? ".$this->GetEscape();
                $first = false;
                $tobind[] = $this->TextLikeSearch($part,false);
              }
              $filter .= ")";
            }
            break;
					case 'rb';
						if ($searchvalue==1)
						{
							$filter .= ' AND recyclebin = 1';
						} else
						{
							$filter .= ' AND (recyclebin = 0 OR recyclebin IS NULL)';
						}
						break;
          default:
            // always_exact : if true, this check for a literal match
            // exact id is set: if true, this checks for a full match
            // It is still that 14? matches 148.
            // And for that reason the LIKE syntax is used.
            $col = $colmap[ $searchcol ];
            if (!isset($always_exact[$searchcol]) || $always_exact[$searchcol] == 0)
            {
              $exactid = $exactmap[$searchcol];
              $searchitems = explode(',',$searchvalue);
              $first = true;
              $filter .= ' AND (';
              $col = $colmap[ $searchcol ];
              foreach ($searchitems as $searchitem)
              {
                if (!$first) $filter .= ' OR ';
                $first = false;
                $filter .= " $col ".$this->GetILike()." ? ".$this->GetEscape()." ";
                $searchitem = $this->TextLikeSearch($searchitem,isset($search[$exactid]));
                $tobind[] = $searchitem;
              }
              $filter .= ') ';
            } else {
              $filter .= " AND $col = ? ";
              $tobind[] = $searchvalue;
            }
        }
      }
			
			if ($this->actuinfo['playtimelim']==1)
			{
				// Only calls that are not 30m. older than the latest call.
				$newest = $this->GetNewestCall();
				if ($newest!==false)
				{
					$filter .= ' AND startdate >= ?';
					$dt = (int) $newest['startdate'];
					$timelim = 30;
					$timelimcfg = $this->GetConfigValue('PlayTimeLimLen');
					if ($timelimcfg!='') $timelim = (int) $timelimcfg;
					$tobind[] = $dt - 60 * $timelim;
				}
			
				// Only calls after the last login.
				$filter .= ' AND startdate >= ?';
				$tobind[] = $this->actuinfo['lastnewlogin'];
			}
			if (isset($this->actuinfo['maxcallage']) && $this->actuinfo['maxcallage']!=-1)
			{
				$filter .= ' AND startdate >= ?';
				$tobind[] = time() - $this->actuinfo['maxcallage'] * 60;
			}

      $ret = array();
      $ret['filter'] = $filter;
      $ret['tobind'] = $tobind;
      return $ret;
  }
	
	// This function is similar to code block in SearchFilter.
	// The difference is that this function is used in getcall (download),
	// while SearchFilter during call listing.
	function CheckCallAccessSpecial($info)
	{
		// code is similar to code block in SearchFilter
		if (int_ensure_defined($this->actuinfo['playtimelim'])==1)
		{
			// Only calls that are not 30m. older than the latest call.
			$newest = $this->GetNewestCall();
			if ($newest!==false)
			{
				$dt = (int) $newest['startdate'];
				$timelim = 30;
				$timelimcfg = $this->GetConfigValue('PlayTimeLimLen');
				if ($timelimcfg!='') $timelim = (int) $timelimcfg;
				$dtlim = $dt - 60 * $timelim;
				if ($info['startdate'] < $dtlim) return false;
			}
		
			// Only calls after the last login.
			$dtlim = $this->actuinfo['lastnewlogin'];
			if ($info['startdate'] < $dtlim) return false;
		}
		if (isset($this->actuinfo['maxcallage']) && $this->actuinfo['maxcallage']!=-1)
		{
			$dtlim = time() - $this->actuinfo['maxcallage'] * 60;
			if ($info['startdate'] < $dtlim) return false;
		}
		return true;
	}
	
	// without current filtering
	function GetNewestCall()
	{
		$result = array();
    $stmt = $this->db->prepare('SELECT * FROM Calls ORDER BY startdate DESC LIMIT 1 ');
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
	}

  function GetCallArchiveCount()
  {
    $stmt = $this->db->prepare('SELECT COUNT(id) FROM CallsArchive');
    $stmt->execute();
		$res = $stmt->fetchColumn(0);
		if ($res===false) return 0;
		return (int) $res;
  }
  
	function GetCallCount()
	{
		$result = array();
    $stmt = $this->db->prepare('SELECT COUNT(id) FROM Calls');
    $stmt->execute();
		$res = $stmt->fetchColumn(0);
		if ($res===false) return 0;
		return (int) $res;
	}
	
	function GetNoTenantCallCount()
	{
		$result = array();
    $stmt = $this->db->prepare('SELECT COUNT(id) FROM Calls WHERE tenantid IS NULL');
    $stmt->execute();
		$res = $stmt->fetchColumn(0);
		if ($res===false) return 0;
		return (int) $res;
	}

	function GetNoAgentCallCount()
	{
		$result = array();
    $stmt = $this->db->prepare('SELECT COUNT(id) FROM Calls WHERE agent IS NULL');
    $stmt->execute();
		$res = $stmt->fetchColumn(0);
		if ($res===false) return 0;
		return (int) $res;
	}
  
  // copy calls (entries in Calls table) of tenant to another database
  function copyTnCallInfo($tenantid, $targetdb, $useArchive)
  {
    $cnt = 0;
    // which columns has the target database?
    $accepted_cols = $targetdb->GetTableColumns( 'Calls' );

    if ($useArchive) $tabname = 'CallsArchive';
    else $tabname = 'Calls';
    
    
		$result = array();
    $stmt = $this->db->prepare('SELECT * FROM '.$tabname.' WHERE tenantid=?');
    $stmt->bindValue(1,$tenantid);
    $stmt->execute();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      // is call is already in target db ?
      if ($targetdb->GetCallInfoRM($row['remoteid'])!==false)
      {
        continue;
      }
      // id column is automatically set
      // and should not be copied
      unset($row['id']);
      // remove cols not known to target database
      foreach ($row as $key=>$value)
      {
        if (isset($accepted_cols[$key])==false)
        {
          unset($key[$value]);
        }
      }
      // add the call
      $targetdb->InsertCall($row);
      $cnt ++;
    }
    return $cnt;
  }
	
  function GetCallList(&$sk,$sd,$search,&$listconfig)
  {
    // Sort key and sort direction
    if ($sk=='length') $sortkey = 'duration';
    else if ($sk=='lname') $sortkey = 'cloc.name';
    else if ($sk=='rname') $sortkey = 'crem.name';
    else if ($sk=='usr') $sortkey = 'useraccid';
    else if ($sk=='startdate'
    || $sk=='remote'
    || $sk=='remote2'
    || $sk=='local'
    || $sk=='local2'
    || $sk=='direction'
    || $sk=='score'
    || $sk=='category'
    || $sk=='waittime'
    ) $sortkey = $sk;
    else if ($sk=='recorder' && isset($listconfig['recorders'])) $sortkey='recorders.name';
    else $sk = $sortkey = 'startdate';

    if ($sd=='asc') $sortdir = 'ASC';
    else $sortdir = 'DESC';

    $sortextra = '';
    // extra sorting on milliseconds
    // this is still fast because upgrade adds startms to index
    if ($sortkey=='startdate') $sortextra .= ' , startms ' . $sortdir;
		else 
		{
			// Stable sort needed, because many duplicates expected.
			// Otherwise the same record might appear twice on different pages.
			$sortextra .= ' , Calls.id DESC';
			// Stable sort can be slow for large databases.
			// But it is not applied for the default sorting (startdate).
		}

    // Create WHERE filter clause,
    // and list of items to bind.
    $sf = $this->SearchFilter($search,$listconfig);
    $tobind = $sf['tobind'];
    $filter = $sf['filter'];

    // For Agent Evaluation (ce)
    $extracols = '';
    $extrajoins = '';
    if (isset($listconfig['ceproject']))
    {
      if ($listconfig['ceproject']==1)
      {
        $extracols .= ' , projectid, score, useraccid, supervisor, datescored, CECalls.id as ceid ';
        $extrajoins .= ' LEFT JOIN CECalls ON calls.id=recordingid ';

      } else if ($listconfig['ceproject']==2)
      {
        $extracols .= ' , projectid, score, useraccid, supervisor, datescored, CECalls.id as ceid ';
        $extrajoins .= ' LEFT JOIN CECalls ON calls.id=recordingid ';
        $filter .= ' AND score IS NOT NULL '.' AND supervisor='.$this->actuinfo['accid']." ";
      } else if ($listconfig['ceproject']==3)
      {
        $extracols .= ' , projectid, score, useraccid, supervisor, cecalls.id AS ceid ';
        $extrajoins .= ' LEFT JOIN CECalls ON calls.id=recordingid ';
        $filter .= ' AND supervisor='.$this->actuinfo['accid']." ";
      } else if ($listconfig['ceproject']==4)
      {
        $extracols .= ' , projectid, score, useraccid, supervisor, datescored, cecalls.id AS ceid ';
        $extrajoins .= ' LEFT JOIN CECalls ON calls.id=recordingid ';
        $filter .= ' AND score IS NOT NULL ';
      } else if ($listconfig['ceproject']==5)
      {
        $extracols .= ' , projectid, score, useraccid, supervisor, datescored, cecalls.id AS ceid ';
        $extrajoins .= ' LEFT JOIN CECalls ON calls.id=recordingid ';
        $filter .= ' AND useraccid='.$this->actuinfo['accid'].' AND supervisor='.$this->actuinfo['accid'].' ';
      } else if ($listconfig['ceproject']==6)
      {
        $extracols .= ' , projectid, score, useraccid, supervisor, datescored, cecalls.id AS ceid ';
        $extrajoins .= ' LEFT JOIN CECalls ON calls.id=recordingid ';
        $filter .= ' AND (useraccid = '.$this->actuinfo['accid']." OR useraccid IS NULL) ";
      }
			
			
    }
    if (isset($listconfig['extracols']))
    {
      $extracols .= $listconfig['extracols'];
    }

    if(isset($listconfig['recorders']))
    {
      $extrajoins .= ' JOIN recorders on calls.recorderid = recorders.id ';
      $extracols .= ' , recorders.name AS recordername, recorders.rectype as recordertype, recorders.recid as recid';
    }

    if ($this->UseTenantContactList())
    {
      $tenantid = (int)$this->actuinfo['tenantid'];
      // tenant contact list
      $contactJoins = ''
      . ' LEFT JOIN TenantContacts AS cloc ON calls.localcontact = cloc.contactid AND cloc.tenantid = ' . $tenantid . ' '
      . ' LEFT JOIN TenantContacts AS crem ON calls.remotecontact = crem.contactid AND crem.tenantid = ' . $tenantid . ' '
      . ' LEFT JOIN TenantContacts AS ccon ON calls.connectedcontact = ccon.contactid AND ccon.tenantid = ' . $tenantid . ' '
      ;
    } else
    {
      // system contact list
      $contactJoins = ''
      . ' JOIN contacts AS cloc ON calls.localcontact = cloc.id'
      . ' JOIN contacts AS crem ON calls.remotecontact = crem.id'
      . ' LEFT JOIN contacts AS ccon ON calls.connectedcontact = ccon.id'
      ;
    }

    $calleridjoins = ' JOIN callerids AS idloc ON calls.localcallerid = idloc.id'
                . ' JOIN callerids AS idrem ON calls.remotecallerid = idrem.id'
                . ' LEFT JOIN callerids AS idcon ON calls.connectedcallerid = idcon.id';
    
    $result = array();
    $sqlstmt = 'SELECT calls.id,remoteid,startdate,startms,duration,durms,'
    . ' remote2,local2,remotefull,localfull, '
    . ' local, remote, connected,'
    . ' isconnected, '
    . ' direction,notes,notes2,notes3,notes4,notes5, '
		. ' data1,data2,data3,data4,data5, '
    . ' localcontact,remotecontact,connectedcontact, '
    . ' cloc.name as lname,crem.name as rname, '
    . ' ccon.name as cname, '
    . ' callextra, flags, isarchived, linenr, filename, directory '
    . ' , category, agent, retention '
    . ' , waittime , Calls.tenantid, recorderid '
    . $extracols
    . ' FROM calls '
    . $contactJoins
    . $extrajoins
    . $filter
    . ' ORDER BY ' . $sortkey . ' ' . $sortdir
    . $sortextra
    ;
		
    if (ensure_defined($listconfig['allrecords'], false)==false)
    {
      $sqlstmt .= ' LIMIT '.(int)$listconfig['checkrange'].' OFFSET '.(int)$listconfig['startfrom'];
    }
    
    //echo $sqlstmt;exit;
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    // searching params binding
    $ix = 0;
    foreach ($tobind as $bind)
    {
      $ix++; $stmt->bindValue($ix,$bind);
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    // Now fetch the records
    $cnt = 0;
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $cnt ++;
      if (($listconfig['allrecords']==true) or ($cnt<=$listconfig['loadrange']))
      {
        // fill in permission for the call
        $row['pm'] = $this->GetCallPermissions($row);
        if (isset($listconfig['addcb']))
        {
          $listconfig['addcb']($row,$listconfig);
        }
        if (isset($listconfig['callback']))
        {
          $listconfig['callback']($row,$listconfig);
        } else
        {
          $result[] = $row;
        }
      }
    }
    $result['cnt'] = $cnt;
    return $result;
  }

  function GetActiveCallInfo($id)
  {
    $stmt = $this->db->prepare('SELECT * FROM ActiveCalls WHERE ActiveCalls.id=:1 ');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result===false) return $result;
    // get permissions on call
    $result['permissions'] = $this->GetCallPermissions($result);
    return $result;
  }

  function GetCallInfo($id)
  {
    $stmt = $this->db->prepare('SELECT * FROM calls WHERE calls.id=:1 ');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result===false) return $result;
    // also get permissions on call
    $result['permissions'] = $this->GetCallPermissions($result);
    return $result;
  }

  function GetCallArchiveInfo($id)
  {
    $stmt = $this->db->prepare('SELECT * FROM CallsArchive WHERE id=:1 ');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result===false) return $result;
    return $result;
  }
  
  function GetMyPhones()
  {
    $accid = $this->actuinfo['accid'];
    $this->EnsureUserPhones($accid);
    return $this->userphones[$accid];
  }
	
  // returns:
  // 0: not with me
  // 1: local
  // 2: remote
  // 3: connected
  function IsCallWithMe($calldata)
  {
		$UsrFilterLocal = $this->GetCachedConfigValue('UsrFilterLocal')!=0;
		$MultiTenancy = $this->GetCachedConfigValue('MultiTenancy')!=0;
		if ($MultiTenancy)
		{
			if ($this->actuinfo['tenantid'] !== null)
			{
				// this is tenant user
				// apply extra filtering
				if ($calldata['tenantid'] != $this->actuinfo['tenantid'])
				{
					// call belongs to another tenant
					return 0;
				}
			}
		}

    $accid = $this->actuinfo['accid'];
    foreach ($this->userphones[$accid] as $contactid)
    {
      if ($contactid==$calldata['localcontact']) return 1;
      if ($UsrFilterLocal==false && $contactid==$calldata['remotecontact']) return 2;
      if ($UsrFilterLocal==false && $contactid==$calldata['connectedcontact']) return 3;
    }
    foreach ($this->userphones_lines[$accid] as $linenr)
    {
      if ($linenr==$calldata['linenr']) return 1; // local
    }
    foreach ($this->userphones_ips[$accid] as $ipaddr)
    {
      if ($ipaddr==$calldata['local2']) return 1;
      if ($UsrFilterLocal==false && $ipaddr==$calldata['remote2']) return 2;
    }    
    return 0;
  }
  
  function GetCallPermissions($calldata)
  {
		if ($calldata==null) return 0; // phan
		$UsrFilterLocal = $this->GetCachedConfigValue('UsrFilterLocal')!=0;
		$MultiTenancy = $this->GetCachedConfigValue('MultiTenancy')!=0;
		if ($MultiTenancy)
		{
			if (isset($this->actuinfo['tenantid']))
			{
				// this is tenant user
				// apply extra filtering
				if (int_ensure_defined($calldata['tenantid']) != $this->actuinfo['tenantid'])
				{
					// call belongs to no or another tenant
					return 0;
				}
			}
		}
    $pm = $this->actuinfo['permissions'];
    if (isset($this->ctpms))
    {
      foreach ($this->ctpms as $contactid=>$dbpm)
      {
        if ($contactid==$calldata['localcontact']
        || ($UsrFilterLocal==false && $contactid==$calldata['remotecontact'])
				|| ($UsrFilterLocal==false && $contactid==$calldata['connectedcontact'])
        ) {
          $pm |= $dbpm;
        }
      }
      foreach ($this->linepms as $linenr=>$dbpm)
      {
        if ($linenr==$calldata['linenr'])
				{
          $pm |= $dbpm;
        }
      }
      foreach ($this->ippms as $ipaddr=>$dbpm)
      {
        if ($ipaddr==$calldata['local2']
				|| ($UsrFilterLocal==false && $ipaddr==$calldata['remote2'])
				) {
          $pm |= $dbpm;
        }
      }
      foreach ($this->recpms as $recorderid=>$dbpm)
      {
        if ($recorderid==$calldata['recorderid']) 
				{
          $pm |= $dbpm;
        }
      }
    }
		if ($this->CheckCallAccessSpecial($calldata)==false)
		{
			return 0;
		}
    return $pm;
  }

  // used by API EditCall
  function SearchCall($search)
  {
    $sql = 'SELECT * FROM Calls WHERE 1=1';
    $to_bind = array();
    if (isset($search['local']))
    {
      $sql .=  ' AND local=?';
      $to_bind[] = $search['local'];
    }
    if (isset($search['remote']))
    {
      $sql .=  ' AND remote=?';
      $to_bind[] = $search['remote'];
    }
    if (isset($search['time']))
    {
      $margin = 0;
      if (isset($search['time-margin'])) $margin = (int) $search['time-margin'];
      $sql .=  ' AND startdate>=? AND startdate<=?';
      $time = (int) $search['time'];
      $to_bind[] = $time - $margin;
      $to_bind[] = $time + $margin;
      $sql .= ' ORDER BY ABS(startdate-?) ASC';
      $to_bind[] = $time;
    } else
    {
      // if no time is specified, take latest
      $sql .= ' ORDER BY startdate DESC';
    }
    $sql .= ' LIMIT 1';
    $stmt = $this->db->prepare($sql);
    if ($stmt==FALSE) $this->OnDbErr();
    $ix = 0;
    foreach ($to_bind as $value)
    {
      $ix++;
      if ($stmt==FALSE) $this->OnDbErr();
      $stmt->bindValue($ix, $value);
    }
    $stmt->execute();
    $result = $stmt->fetch(PDO::FETCH_ASSOC);
    return $result;
  }

  function GetCallInfoRM($remoteid)
  {
    $stmt = $this->db->prepare('SELECT * FROM Calls WHERE remoteid=:1 ');
    $stmt->bindValue(1,$remoteid);
    $stmt->execute();
    $result = $stmt->fetch(PDO::FETCH_ASSOC);
    return $result;
  }

  function GetActiveCallInfoRM($remoteid)
  {
    $stmt = $this->db->prepare('SELECT * FROM ActiveCalls WHERE remoteid=:1 ');
    $stmt->bindValue(1,$remoteid);
    $stmt->execute();
    $result = $stmt->fetch(PDO::FETCH_ASSOC);
    if ($result==null) return $result;
    $pm = $this->GetCallPermissions($result);
    if ($pm&PM_LIST==0) return false;
    return $result;
  }

  function SetCallInfo($id,$data)
  {
    // if no data to set, we are ready quickly
    if (count($data)==0) return true;
    $sql = 'UPDATE Calls SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$id);
    return $stmt->execute();
  }

  function SetCallArchiveInfo($id,$data)
  {
    $sql = 'UPDATE CallsArchive SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$id);
    $res = $stmt->execute();
    return true;
  }
	
  function SetActiveCallInfoRM($remoteid,$data)
  {
    $sql = 'UPDATE ActiveCalls SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE remoteid=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$remoteid);
    $res = $stmt->execute();
    return true;
  }

  function EditActiveRecordingNotes($id,$notes)
  {
    $stmt = $this->db->prepare('UPDATE ActiveCalls SET notes=:1 WHERE id=:2 ');
    $stmt->bindValue(1,$notes);
    $stmt->bindValue(2,$id);
    $res = $stmt->execute();
  }
  function EditActiveRecordingNotes2($id,$notes)
  {
    $stmt = $this->db->prepare('UPDATE ActiveCalls SET notes2=:1 WHERE id=:2 ');
    $stmt->bindValue(1,$notes);
    $stmt->bindValue(2,$id);
    $res = $stmt->execute();
  }

  function EditActiveRecordingCategory($id,$value)
  {
    $stmt = $this->db->prepare('UPDATE ActiveCalls SET category=:1 WHERE id=:2 ');
    $stmt->bindValue(1,(int) $value);
    $stmt->bindValue(2,$id);
    $res = $stmt->execute();
  }
	
  function EditRecordingNotes($id,$notes)
  {
    $stmt = $this->db->prepare('UPDATE Calls SET notes=:1 WHERE id=:2 ');
    $stmt->bindValue(1,$notes);
    $stmt->bindValue(2,$id);
    $res = $stmt->execute();
  }
  function EditRecordingNotes2($id,$notes)
  {
    $stmt = $this->db->prepare('UPDATE Calls SET notes2=:1 WHERE id=:2 ');
    $stmt->bindValue(1,$notes);
    $stmt->bindValue(2,$id);
    $res = $stmt->execute();
  }

  function EditRecordingCategory($id,$value)
  {
    $stmt = $this->db->prepare('UPDATE Calls SET category=:1 WHERE id=:2 ');
    $stmt->bindValue(1,(int) $value);
    $stmt->bindValue(2,$id);
    $res = $stmt->execute();
  }

  function DeleteRecording($recording_id)
  {
    $sql = 'DELETE FROM calls WHERE id=:1 ';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$recording_id);
    $stmt->execute();
  }

	function DeleteRecordingRB($recording_id)
  {
    $stmt = $this->db->prepare('UPDATE Calls SET recyclebin=1 WHERE id=:1');
    $stmt->bindValue(1,$recording_id);
    $res = $stmt->execute();
  }

	function RecoverRecordingRB($recording_id)
  {
    $stmt = $this->db->prepare('UPDATE Calls SET recyclebin=0 WHERE id=:1');
    $stmt->bindValue(1,$recording_id);
    $res = $stmt->execute();
  }

  function InsertCall($data)
  {
    $sql = 'INSERT INTO calls ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		
		return $this->db->lastInsertId('Calls_id_seq');
  }
	
  function CreateCallerID($callerid,$callerid2,$islocal,$name='')
  {
    if (($this->actuinfo['permissions']&(PM_EDITNAMES|PM_T2_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    $result = array();
    $stmt = $this->db->prepare('select id,contactid,islocal from callerids where '
    . ' callerid=:1 AND deleted=0');
    $stmt->bindValue(1,$callerid);
    // not:  and callerid2=:2
    // not: $stmt->bindValue(2,$callerid2);
    // because IP addr can vary
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result === FALSE)
    {
      // caller ID not known,
      // add it now
      // first add contact
      $stmt = $this->db->prepare('insert into Contacts '
      . ' (name    ,extra  ,parentcontact) '
      . ' VALUES '
      . ' (:1      ,:2     ,:3           ) '
      );
      $stmt->bindValue(1,$name);
      $stmt->bindValue(2,'');
      $stmt->bindValue(3,0);
      $stmt->execute();
      $result = array();
      $result["contactid"] = $this->db->lastInsertId('Contacts_id_seq');
      // then add Caller ID
      $stmt = $this->db->prepare('insert into CallerIDs '
      . ' (callerid,callerid2,islocal,contactid,simplified) '
      . ' VALUES '
      . ' (:1       ,:2      ,:3     ,:4,       :5) '
      );
      $stmt->bindValue(1,$callerid);
      $stmt->bindValue(2,$callerid2);
      $stmt->bindValue(3,$islocal);
      $stmt->bindValue(4,$result["contactid"]);
      $stmt->bindValue(5, IdSimplify($callerid));
      $stmt->execute();
      $result["id"] = $this->db->lastInsertId('CallerIDs_id_seq');
    } else {
      $this->lasterr = 'already-exists';
      return false;
    }
  }

	function GetInternalContact($ctype,$value,$name)
	{
		$result = array();
		// first check if already exists
		$stmt = $this->db->prepare('SELECT id FROM Contacts '
		.' WHERE ctype=:1 AND value=:2 '
		);
		$stmt->bindValue(1,$ctype);
		$stmt->bindValue(2,$value);
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row!==false)
		{
			// found
			return $row['id'];
		}
		return NULL;
	}
	
	function GetOrCreateInternalContact($ctype,$value,$name)
	{
		$result = array();
		// first check if already exists
		$stmt = $this->db->prepare('SELECT id FROM Contacts '
		.' WHERE ctype=:1 AND value=:2 '
		);
		$stmt->bindValue(1,$ctype);
		$stmt->bindValue(2,$value);
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row!==false)
		{
			// found
			return $row['id'];
		}
		// it does not exist
		// create it
		$stmt = $this->db->prepare('INSERT INTO Contacts '
		.'(name,extra,parentcontact,ctype,value) '
		.' VALUES '
		."(:1,'',0,:2,:3)"
    );
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		$stmt->bindValue(1,$name);
		$stmt->bindValue(2,$ctype);
		$stmt->bindValue(3,$value);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return $this->db->lastInsertId('Contacts_id_seq');
	}
	
	function GetIpAddr($ipaddr,$name='')
	{
		return $this->GetInternalContact( 2, $ipaddr, $name);
	}

	function GetOrCreateIpAddr($ipaddr,$name='')
	{
		return $this->GetOrCreateInternalContact( 2, $ipaddr, $name);
	}

	function GetLineContact($linenr, $recorderid)
	{
		$result = array();
		// first check if already exists
		$stmt = $this->db->prepare('SELECT id,contactid FROM Lines '
		.' WHERE linenr=:1 AND recorderid=:2 '
		);
		$stmt->bindValue(1,$linenr);
		$stmt->bindValue(2,$recorderid);
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row!==false)
		{
			// found
			$result['id'] = $row['id'];
			$result['contactid'] = $row['contactid'];
			return $result;
		}
		// it does not exist
		return NULL;
	}
	
	function GetOrCreateLineContact($linenr, $recorderid, $name)
	{
		$result = array();
		// first check if already exists
		$stmt = $this->db->prepare('SELECT id,contactid FROM Lines '
		.' WHERE linenr=:1 AND recorderid=:2 '
		);
		$stmt->bindValue(1,$linenr);
		$stmt->bindValue(2,$recorderid);
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row!==false)
		{
			// found
			$result['id'] = $row['id'];
			$result['contactid'] = $row['contactid'];
			return $result;
		}
		// it does not exist
		// create it
		$stmt = $this->db->prepare('INSERT INTO Contacts '
		.'(name,extra,parentcontact) '
		.' VALUES '
		."(:1,'',0)"
    );
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		$stmt->bindValue(1,$name);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $result["contactid"] = $this->db->lastInsertId('Contacts_id_seq');
		
		$stmt = $this->db->prepare('insert into Lines '
		. ' (contactid,recorderid,linenr) '
		. ' VALUES '
		. ' (:1       ,:2        ,:3    ) '
		);
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		$stmt->bindValue(1,$result["contactid"]);
		$stmt->bindValue(2,$recorderid);
		$stmt->bindValue(3,$linenr);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$result["id"] = $this->db->lastInsertId('Lines_id_seq');
		
		return $result;
	}

  // islocal : integer 0 or 1, (2=unknown)
  // returns database ID
  // changes contactid when available
  function GetOrCreateCallerID($callerid,$callerid2,$islocal,$name='')
  {
    $result = array();
    $stmt = $this->db->prepare('select id,contactid,islocal from callerids where '
    . ' callerid=:1 AND deleted=0');
    $stmt->bindValue(1,$callerid);
    // not:  and callerid2=:2
    // not: $stmt->bindValue(2,$callerid2);
    // because IP addr can vary
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result === FALSE)
    {
      // caller ID not known,
      // add it now
      // first add contact
      $stmt = $this->db->prepare('insert into Contacts '
      . ' (name    ,extra  ,parentcontact) '
      . ' VALUES '
      . ' (:1      ,:2     ,:3           ) '
      );
      $stmt->bindValue(1,$name);
      $stmt->bindValue(2,'');
      $stmt->bindValue(3,0);
      $stmt->execute();
      
      $result = array();
      $result["contactid"] = $this->db->lastInsertId('Contacts_id_seq');
      // then add Caller ID
      $stmt = $this->db->prepare('insert into CallerIDs '
      . ' (callerid,callerid2,islocal,contactid,simplified) '
      . ' VALUES '
      . ' (:1       ,:2      ,:3     ,:4,       :5) '
      );
      $stmt->bindValue(1,$callerid);
      $stmt->bindValue(2,$callerid2);
      $stmt->bindValue(3,$islocal);
      $stmt->bindValue(4,$result["contactid"]);
      $stmt->bindValue(5,IdSimplify($callerid));
      $stmt->execute();
      $result["id"] = $this->db->lastInsertId('CallerIDs_id_seq');
    } else {
      if ($islocal && $result['islocal']==0)
      {
        // make local
        $stmt = $this->db->prepare('UPDATE CallerIDs '
        . ' SET islocal=1 WHERE id=:1 '
        );
        $stmt->bindValue(1,$result['id']);
        $stmt->execute();
      }
    }
    return $result;
  }

	function RenameRecorder($recid,$name)
	{
    $result = array();
    $stmt = $this->db->prepare('select id from recorders where recid=:1');
    $stmt->bindValue(1,$recid);
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result === FALSE)
    {
    } else 
		{
      // recorder not known,
      // add it now
      $stmt = $this->db->prepare('UPDATE recorders '
      . ' SET name=:1 '
      . ' WHERE recid=:2 '
      );
      $stmt->bindValue(1,$name);
      $stmt->bindValue(2,$recid);
      $stmt->execute();
    }
    return $result;
	}
	
  // recid is not the database ID, but the identification of the recorder
  // the returned value is the database ID
  function GetOrCreateRecorderId($recid,$name,$rectype=50)
  {
    $result = array();
    $stmt = $this->db->prepare('select id from recorders where recid=:1');
    $stmt->bindValue(1,$recid);
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result === FALSE)
    {
      // recorder not known,
      // add it now
      $stmt = $this->db->prepare('insert into recorders '
      . ' (recid,name,rectype,options) '
      . ' VALUES '
      . ' (:1   ,:2  ,:3     ,:4     ) '
      );
      $stmt->bindValue(1,$recid);
      $stmt->bindValue(2,$name);
      $stmt->bindValue(3,$rectype); // 50=VoIP based on Oreka
      $stmt->bindValue(4,'');
      $stmt->execute();
      $result = $this->db->lastInsertId('recorders_id_seq');
    } else {
      $result = $result["id"];
    }
    return $result;
  }

	function UpdateMaxCalls($numcalls, $hour)
	{
    $result = array();
    $stmt = $this->db->prepare('select numcalls from MaxCalls where hour=:1');
    $stmt->bindValue(1,$hour);
    $stmt->execute();
    $result = $stmt->fetch();
    if ($result === FALSE)
    {
      $stmt = $this->db->prepare('insert into MaxCalls '
      . ' (numcalls,hour) '
      . ' VALUES '
      . ' (:1   ,:2 ) '
      );
      $stmt->bindValue(1,$numcalls);
      $stmt->bindValue(2,$hour);
      $stmt->execute();
		}
		else
		{
			$ret = $result['numcalls'];
			if ($ret < $numcalls)
			{
				$sql = 'UPDATE MaxCalls SET numcalls=:1 WHERE hour=:2';
				$stmt = $this->db->prepare($sql);
				$stmt->bindValue(1,$numcalls);
				$stmt->bindValue(2,$hour);
				if ($stmt->execute()==false)
				{
					$this->OnDbErr();
				}
			}
		}
	}
	
  function GetSubUsersAccIds()
  {
    $result = array();
    // mysubgroups also include sub-subgroups
    foreach ($this->mysubgroups as $groupaccid)
    {
      $members = $this->GetGroupMembers($groupaccid);
      foreach ($members as $member)
      {
        // adding it this way, we prevent duplicates
        $result[ $member ] = true;
      }
    }
    return $result;
  }

  function IsSubGroup($testGroupaccid)
  {
    // mysubgroups also include sub-subgroups
    foreach ($this->mysubgroups as $groupaccid)
    {
      if ($testGroupaccid==$groupaccid)
      {
        return true;
      }
    }
    return false;
  }

  function IsSubUser($testAccid)
  {
    $list = $this->GetSubUsersAccIds();
    foreach ($list as $accid=>$ignore)
    {
      if ($accid==$testAccid)
      {
        return true;
      }
    }
    return false;
  }

  function GetSubUserList()
  {
    $list = $this->GetSubUsersAccIds();
    $result = array();
    foreach ($list as $accid=>$ignore)
    {
      $user = $this->GetUserInfoOnAccId($accid);
      $result[ $user['id'] ] = $user;
    }
    return $result;
  }

  function GetManagedUserList()
  {
    $manager = $this->actuinfo['id'];
    $result = array();
    foreach ($this->db->query('SELECT * '
    . ' FROM Users WHERE manager='.$manager.' ORDER BY name') as $row)
    {
      $result[] = $row;
    }
    return $result;
  }

	function GetAccIdsOfUsersWithoutGroup()
	{
    $sqlstmt = 'SELECT accid FROM Users WHERE accid NOT IN (SELECT accid FROM GroupMembers)';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row['accid'];
    }
    return $result;
	}
	
  function GetTotalUserCount()
  {
    return $this->db->query('SELECT COUNT(id) FROM Users')->fetchColumn(0);
  }

	function GetEditableUserList()
  {
    $all = ($this->actuinfo['permissions']
    & (PM_USERS|PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN) )!=0;
    $result = array();
    foreach ($this->db->query('SELECT * '
    . ' FROM Users ORDER BY name') as $row)
    {
      if ($all==false)
      {
        if (isset($this->accpms[ $row['accid'] ])
        && ($this->accpms[ $row['accid'] ]&PM_USERS)!=0 )
        {
          ; // OK
        } else
        {
          continue;
        }
      }
      $result[] = $row;
    }
    return $result;
  }

  function GetUserIdp($userInfo)
  {
    $groups = $this->GetGroupsOfWhichMember( $userInfo['accid'] );
    foreach ($groups as $group)
    {
      $groupInfo = $this->GetGroupInfoOnAccid( $group );
      if ($groupInfo['idp']!=0) return $groupInfo['idp'];
    }
    return false;
  }

  function GetUserListForApi()
  {
    $all = ($this->actuinfo['permissions']
    & (PM_USERS|PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN|PM_TENANT_ADMIN_AUDIT) )!=0;
    $result = array();
    $tenantid = $this->actuinfo['tenantid'];
    foreach ($this->db->query('SELECT id,accid,usr,name,enabled'
		. ',tenantid,evaluated '
    . ' FROM Users ORDER BY name') as $row)
    {
      if ($all==false)
      {
        if (isset($this->accpms[ $row['accid'] ])
        && ($this->accpms[ $row['accid'] ]&PM_USERS)!=0 )
        {
          ; // OK
        } else
        {
          continue;
        }
      }
      if ($tenantid!=null)
      {
        // skip users from other tenants
        if ($row['tenantid'] != $tenantid) continue;
      }
      
      $result[] = $row;
    }
    return $result;
  }
	
  function GetUserList()
  {
    $all = ($this->actuinfo['permissions']
    & (PM_USERS|PM_CONFIG|PM_CE_EVALUATE|PM_CE_VIEW|PM_CE_MANAGE|PM_CE_SUPERVISOR|PM_CE_SCHEDULING|PM_T2_ADMIN|PM_T3_ADMIN|PM_TENANT_ADMIN_AUDIT|PM_CE_VIEWOWNREPORTS|PM_CE_SELFEVALUATE) )!=0;
    $result = array();
    $tenantid = $this->actuinfo['tenantid'];
    foreach ($this->db->query('SELECT id,accid,usr,name,enabled'
		. ',tenantid,evaluated,iaperiod'
    . ' FROM Users ORDER BY name') as $row)
    {
      if ($all==false)
      {
        if (isset($this->accpms[ $row['accid'] ])
        && ($this->accpms[ $row['accid'] ]&PM_USERS)!=0 )
        {
          ; // OK
        } else
        {
          continue;
        }
      }
      if ($tenantid!=null)
      {
        // skip users from other tenants
        if ($row['tenantid'] != $tenantid) continue;
      }
      $result[] = $row;
    }
    return $result;
  }

  function GetUserInfo($userid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM Users WHERE id=:1');
    $stmt->bindValue(1,$userid);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
  }

  function GetUserInfoOnAccId($accid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM Users WHERE accid=:1');
    $stmt->bindValue(1,$accid);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
  }
	
	function GetUserInfoOnUsername($usr)
	{
    $stmt = $this->db->prepare('SELECT * FROM Users WHERE usr=:1');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
	}

	function GetUserInfoOnEmail($email)
	{
    $stmt = $this->db->prepare('SELECT * FROM Users WHERE email=:1');
    $stmt->bindValue(1,$email);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
	}

  function GetGroupInfo($groupid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM Groups WHERE id=:1');
    $stmt->bindValue(1,$groupid);
    $stmt->execute();
    $result = $stmt->fetch(PDO::FETCH_ASSOC);
    return $result;
  }

  function GetGroupInfoOnAccid($accid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM Groups WHERE accid=:1');
    $stmt->bindValue(1,$accid);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
  }
	
  function GetSingleGroupInfoByName($groupName)
  {
    $groupsWithThisName = $this->GetGroupInfoOnName( $groupName );
    if (count($groupsWithThisName)==0)
    {
      return false;
    }
    return $groupsWithThisName[0];
  }
  
  function GetGroupInfoOnName($name)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM Groups WHERE name=:1');
    $stmt->bindValue(1,$name);
    $stmt->execute();
    $result = array();
    while($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }

    return $result;
  }

	function SetUserRecorderPms($accid,$recPmList)
	{
    $allowed = false;
    $aupm = $this->actuinfo['permissions'];
    if ($aupm&PM_CONFIG) $allowed = true;
    if ($allowed==false
    && ($aupm&(PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))!=0 )
    {
      $allowed = true;
		}
    if ( $allowed==false // not already allowed for other reasons
      && ($this->anyaccpms&PM_USERS)!=0 // we have access to some user accounts
      && ($this->GetAccPm($accid)&PM_USERS)!=0 // access to this account
    )
    {
      /*
       The current user may edit other user/grp accounts.
       He has now trying to change the permissions.
       These restrictions should apply:
       - the user has the permissions already
       - for new permissions:
       - only if the manager has also himself
       */

      $allowed = true;
      // Get current permissions.
      $olddata = $this->GetUserRecorderPms( $accid );
      // Check every permission that is specified
      foreach ($recPmList as $obj)
      {
				$pm = (int) $obj['epermissions'];
        // is it a new permission?
				$oldpm = 0;
        foreach ($olddata as $oldobj)
        {
          if ($obj['recorderid']==$oldobj['recorderid'])
					{
						$oldpm |= (int) $oldobj['epermissions'];
          }
        }
				if ( ($oldpm & $pm) == $pm )
				{
					// the user already had all of these permissions for this recorder
					// old permission are accepted, because it is no change
					continue;
				}
				$pm ^= ($oldpm & $pm);
        // it is a new permission, or at least partially
				// need to check it
				// does manager has it?
				// either globally
				$managerpm = $aupm;
				// or specifically for this recorder
				if (isset($this->recpms[ $obj['recorderid'] ]))
				{
					$managerpm |= $this->recpms[ $obj['recorderid'] ];
				}
				if ( ( $managerpm & $pm ) != $pm )
				{
					// if the manager does not have this permission,
					// then it is not allowed to add it
					$allowed = false;
					break;
				}
      }
    }
		if ($allowed==false)
		{
      $this->lasterr = 'pm-denied';
      return false;
		}
		
    $stmt = $this->db->prepare('DELETE FROM AccRecorders WHERE accid=:1');
    $stmt->bindValue(1,$accid);
    $stmt->execute();

    $stmt = $this->db->prepare('INSERT INTO AccRecorders '
    . ' (accid,recorderid,epermissions) '
    . ' VALUES '
    . ' (:1    ,:2       ,:3          ) '
    );
    foreach($recPmList as $recPm)
    {
      $stmt->bindValue(1,$accid);
      $stmt->bindValue(2,$recPm['recorderid']);
      $stmt->bindValue(3,$recPm['epermissions']);
      $stmt->execute();
    }
		return true;
	}
	
	function GetUserRecorderPms($accid)
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * '
    . ' FROM AccRecorders WHERE accid=:1 '
    );
		$stmt->bindValue(1,$accid);
		$stmt->execute();
		while ($row = $stmt->fetch())
		{
			$result[] = $row;
		}
    return $result;
	}
	
  function GetAccAccPms($accidlist)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * '
    . ' FROM AccAcc WHERE accid=:1 '
    );
    foreach($accidlist as $accid)
    {
      $stmt->bindValue(1,$accid);
      $stmt->execute();
      while ($row = $stmt->fetch())
      {
        $result[] = $row;
      }
    }
    return $result;
  }

  // do we have global permission for this (actuinfo)
  // or permission for any user for his (anyaccpms)
  function HasPmForAny($pm_flag)
  {
    if ( (($this->actuinfo['permissions']& $pm_flag )!=0)
    || (($this->anyaccpms& $pm_flag )!=0) )
    {
      return true;
    }
    return false;
  }
  
  function AmIAdminT123()
  {
    if ( ($this->actuinfo['permissions']&PM_CONFIG)!=0
        || ($this->actuinfo["permissions"]&PM_T2_ADMIN)!=0
        || ($this->actuinfo["permissions"]&PM_T3_ADMIN)!=0
         )
    {
      return true;
    }
    return false;
  }
  
  function IsUserAdminT123($userid)
  {
    $int_userid = (int) $userid;
    $ret = false;
    $db2 = new DBInterface('');
    $db2->cloneInit( $this );
    $db2->SetCurrentUserId( $int_userid );
    if ( ( $db2->actuinfo['permissions'] & (PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN) ) !=0 )
    {
      $ret = true;
    }
    $db2->close();
    return $ret;
  }

  function IsUserAdmin($userid)
  {
    $int_userid = (int) $userid;
    // get info including accessor-id
    $userinfo = $this->GetUserInfo($int_userid);
    if (($this->GetUserPersonalPm($userinfo['accid']) & PM_CONFIG)!=0)
    {
      return true;
    }
  }

  function MayEditUserAccount($userid,$useraccid)
  {
    $allowed = false;
    $aupm = $this->actuinfo['permissions'];
    if ($aupm&PM_CONFIG) $allowed = true;
    if ($allowed==false
    && ($aupm&(PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))!=0 )
    {
      $allowed = true;
      // except
      $db2 = new DBInterface('');
      $db2->cloneInit( $this );
      $theuser = $this->GetUserInfo($userid);
      $db2->SetCurrentUserId( $userid );
      if ( $db2->actuinfo['permissions']&PM_CONFIG )
      {
        $allowed = false;
      }
      if ( $this->IsExtraAdminPm(PM_T2_ADMIN)
      && $db2->actuinfo['permissions']&PM_T2_ADMIN )
      {
        $allowed = false;
      }
      if ( $this->IsExtraAdminPm(PM_T3_ADMIN)
      && $db2->actuinfo['permissions']&PM_T3_ADMIN )
      {
        $allowed = false;
      }
    }
    if ($aupm&PM_SUBADMIN)
    {
      $theuser = $this->GetUserInfo($userid);
      if ($theuser['manager']==$this->actuinfo['id'])
      {
        $allowed = true;
      }
    }
    if ($this->anyaccpms&PM_USERS)
    {
      if ( ($this->GetAccPm($useraccid)&PM_USERS)!=0 )
      {
        $allowed = true;
      }
    }
    if ($allowed==false)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    // below only extra restrictions
    if (defined('DEMO') && $userid==1)
    {
      $this->lasterr = 'demo-editadmin';
      return false;
    }
    if (defined('DEMO') && $userid==2)
    {
      $this->lasterr = 'demo-protecteduser';
      return false;
    }
    return true;
  }

  function EditUserAccount($userid,$data)
  {
    $theuser = $this->GetUserInfo($userid);
		if ( ( int_ensure_defined($data['customlocalid'])==1 && $data['customlocalid'] != $theuser['customlocalid'] )
		|| ( $theuser['customlocalid']==1 && $data['name'] != $theuser['name'] )
		)
		{
			// Setting customlocalid is dangerous.
			// When set, the Name will be used as Local ID.
			// And the Local ID determines the access permissions.
			// So when set, changing the name is also dangerous.
			// Only an admin. can do that.
			if ( ($this->actuinfo['permissions']&PM_CONFIG)==0 )
			{
				$this->lasterr = 'pm-denied';
				return false;
			}
		}
    if ($this->MayEditUserAccount($userid,$theuser['accid'])==false)
    {
      return false;
    }
		return $this->EditUserAccountLC($userid,$data);
	}

	function EditUserAccountUnchecked($userid,$data)
	{
    $sql = 'UPDATE Users SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$userid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return true;		
	}
	
	// not unchecked, but less checked (LC)
	function EditUserAccountLC($userid,$data)
  {
		if (count($data)==0) return false;
    if (isset($data['enabled']) && $data['enabled']==0)
    {
      if ($this->IsLastAdmin($userid))
      {
        // we would be locked out
        // setting of permissions is checked somewhere else
        $this->lasterr = 'lock-out';
        return false;
      }
    }
    $aupm = $this->actuinfo['permissions'];
    if ( ($aupm&(PM_CONFIG|PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))==0 )
    {
      if (isset($data['mphones']))
      {
        $this->lasterr = 'pm-denied';
        return false;
      }
    }
		if (isset($data['rawpwd']))
		{
			$pwd = $data['rawpwd'];
			unset($data['rawpwd']);
			if ($this->ChangePassword($userid,$pwd)==false)
			{
				return false;
			}
		}

    $sql = 'UPDATE Users SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$userid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return true;
  }
	
	function SetUserLastNewLogin($uid, $logintime)
	{
    $sql = 'UPDATE Users SET lastnewlogin=:1 WHERE id=:2';
    $stmt = $this->db->prepare($sql);
		$stmt->bindValue(1,$logintime);
    $stmt->bindValue(2,$uid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}
	
	function SetUserLanguage($uid, $lng)
	{
    $sql = 'UPDATE Users SET lng=:1 WHERE id=:2';
    $stmt = $this->db->prepare($sql);
		$stmt->bindValue(1,$lng);
    $stmt->bindValue(2,$uid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}
	
	function ResetChangePwdFlag($uid)
	{
    $sql = 'UPDATE Users SET changepwd=0 WHERE id=:1';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$uid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}
	
  function IsAccountTempDisabled($usr)
  {
    $stmt = $this->db->prepare('SELECT tempdisabled FROM Users WHERE usr=:1');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
		{
      return true;
    }
    if ($row['tempdisabled'] == 1) return true;
    return false;
  }

  function IsUserCreatedByAD($accid)
  {
    $stmt = $this->db->prepare('SELECT ldapuser FROM Users WHERE accid=:1');
    $stmt->bindValue(1,$accid);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
		{
      return false;
    }
    if ($row['ldapuser'] == 1) return true;
    return false;
  }

	//$number==-1 -> returns number of wrong passwords
	//$number==0 -> resets number and returns 0
	//$number==1 -> increase number and returns new number
	function WrongPassword($user, $number, $MaxPwdFailures)
	{
		if ($this->readonly==false)
		{
			if ($number==0)
			{
				$sql = 'UPDATE Users SET numwrongpwd = 0, tempdisabled = 0 WHERE usr=:1';
			}
			if ($number==1)
			{
				$sql = 'UPDATE Users SET numwrongpwd = numwrongpwd + 1 WHERE usr=:1';
			}
			if ($number != -1)
			{
				$stmt = $this->db->prepare($sql);
				$stmt->bindValue(1,$user);
				if ($stmt->execute()==false)
				{
					$this->OnDbErr();
				}
			}
		}
		//return the current number
    $stmt = $this->db->prepare('SELECT numwrongpwd FROM Users WHERE usr=:1');
    $stmt->bindValue(1,$user);
    $stmt->execute();
    $result = $stmt->fetch();
    $ret = $result['numwrongpwd'];
		if ($ret == $MaxPwdFailures && $number == 1)
		{
			//Disable the account temporarily!
			$disabledtime = time();
			$sql = 'UPDATE Users SET tempdisabled = 1, disabledtime=:1 WHERE usr=:2';
			$stmt = $this->db->prepare($sql);
			$stmt->bindValue(1,$disabledtime);
			$stmt->bindValue(2,$user);
			if ($stmt->execute()==false)
			{
				$this->OnDbErr();
			}
		}
		return $ret;
	}
	
	function NewPassword($usr, $passwd, $mustchangepassword, &$emailaddress, &$friendlyname)
	{
		$emailaddress = '';
		$friendlyname = '';
		$pwdchangedate = time();
		if (!$this->UsrExists($usr)) return false;	//Wrong username!
		
		//Get the emailaddress!
    $stmt = $this->db->prepare('SELECT email, name FROM Users WHERE usr=:1');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $result = $stmt->fetch();
		if ($result['email'] == '')
		{
			return true;
		}
    $emailaddress = $result['email'];
		$friendlyname = $result['name'];
		
		//The password is auto-generated, so we assume it's unique!
		$sql = 'UPDATE Users SET changepwd=:1, pwd=:2, numwrongpwd=0, pwdchangedate=:3, tempdisabled=0 WHERE usr=:4';
		$stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$mustchangepassword);
    $stmt->bindValue(2,$passwd);
		$stmt->bindValue(3,$pwdchangedate);
    $stmt->bindValue(4,$usr);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		//Add the new password to the history!
		$NumPwd = $this->GetConfigValue('PwdHistory');
		if ($NumPwd > 0)
		{
			$pwdhistory = $this->GetPwdHistory($usr);
			$passwords = explode(",", $pwdhistory);
			$num = count($passwords);
			if ($num == 0)
			{
				$pwdhistory = $passwd;
			}
			else
			{
				if ($num >= $NumPwd)
				{
					while($num >= $NumPwd)
					{
						array_shift($passwords);	//Shift the first value out of the array!
						$num--;
					}
				}
				array_push($passwords, $passwd);
				$pwdhistory = implode(",", $passwords);
			}
			$this->SetPwdHistory($usr, $pwdhistory);
		}
		return true;
	}
	
	function GetPwdHistory($usr)
	{
    $stmt = $this->db->prepare('SELECT pwdhistory FROM Users WHERE usr=:1');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result['pwdhistory'];
	}	
	
	function SetPwdHistory($usr, $pwdhistory)
	{
		$sql = 'UPDATE Users SET pwdhistory=:1 WHERE usr=:2';
		$stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$pwdhistory);
    $stmt->bindValue(2,$usr);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}	
	
  function GetPwdChangeDate($id)
  {
    $stmt = $this->db->prepare('SELECT pwdchangedate FROM Users WHERE id=:1');
    $stmt->bindValue(1,$id);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result['pwdchangedate'];
  }
	
	function EnableVisibleColumn($colkey)
	{
    foreach ($this->db->query('SELECT id,usr,colcfg '
    . ' FROM Users WHERE colcfg IS NOT NULL') as $row)
    {
			$colcfg = $row['colcfg'];
			$cols = explode(',',$colcfg);
			$found = false;
			foreach ($cols as $col)
			{
				if (trim($col) == $colkey) $found = true;
			}
			
			if ($found==false)
			{
				$data = array();
				$colcfg = $colcfg . ', ' . $colkey;
				$data['colcfg'] = $colcfg;
				$this->EditUserAccountUnchecked($row['id'], $data );
			}
    }
		
	}
  
  // extra compared to with it has itself
  function IsExtraAdminPm($pm)
  {
    $aupm = $this->actuinfo['permissions'];
    if ( ($aupm&(PM_CONFIG)) == 0)
    {
      if ($pm&(PM_CONFIG|PM_FILTER) )
      {
        return true;
      }
    }
    if ( ($aupm&(PM_CONFIG|PM_T2_ADMIN)) == 0)
    {
      if ($pm&(PM_T2_ADMIN|PM_SYSALARM) )
      {
        return true;
      }
    }
    if ( ($aupm&(PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN)) == 0)
    {
      if ($pm&PM_T3_ADMIN)
      {
        return true;
      }
    }
    if ( ($aupm&(PM_CONFIG|PM_T2_ADMIN|PM_T3_ADMIN|PM_USERS)) == 0)
    {
      if ($pm&(PM_USERS|PM_SUBADMIN) )
      {
        return true;
      }
    }
    return false;
  }
	
  function SetAccAccPms($accid,$data)
  {
    $allowed = false;
    $aupm = $this->actuinfo['permissions'];
    // PM_CONFIG - PM_T2_ADMIN - PM_T3_ADMIN - PM_USERS
    $extraAdminPm = false;
    $defines_admin = false;
    foreach($data as $obj)
    {
      if ($obj['pm']&PM_CONFIG)
      {
        $defines_admin = true;
      }
      if ( $this->IsExtraAdminPm($obj['pm']) )
      {
        $extraAdminPm = true;
      }
    }
    if ($aupm&PM_CONFIG) $allowed = true;
    if ($aupm&(PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))
    {
      if ($extraAdminPm==false)
      {
        $allowed = true;
      }
    }
    if ( $allowed==false // not already allowed for other reasons
      && ($this->anyaccpms&PM_USERS)!=0 // we have access to some user accounts
      && ($this->GetAccPm($accid)&PM_USERS)!=0 // access to this account
    )
    {
      /*
       The current user may edit other user/grp accounts.
       He has now trying to change the permissions.
       These restrictions should apply:
       - the user has the permissions already
       - for new permissions:
       - only if the manager has also himself
       */

        $allowed = true;
      // Get current permissions.
      $olddata = $this->GetAccAccPms( array($accid) );
      // Check every permission that is specified
      foreach ($data as $obj)
      {
        // if it is a new permission
        $isnew = true;
        foreach ($olddata as $oldobj)
        {
          if ($obj['callaccid']==$oldobj['callaccid']
          && $obj['pm']==$oldobj['epermissions']
          && $obj['applytype']==$oldobj['applytype']
          ){
            $isnew = false;
          }
        }
        // old permission are accepted, because it is no change
        if ($isnew==false) continue;
        // it is a new permission, need to check it
        if ($obj['callaccid']==0)
        {
          // it is a general permission
          if ( ($this->actuinfo['permissions'] & $obj['pm']) == $obj['pm'] )
          {
            // OK, manager has it also, and may delegate it.
          } else
          {
            $this->lasterr = 'pm-denied';
            return false;
          }
        } else
        {
          $checkAccId = $obj['callaccid'];
          if ($checkAccId==-1)
          {
            $checkAccId = $accid;
          }
          // it is a specific permission
          // does manager has it?
          $mpm = $this->GetAccPm( $checkAccId );
          if ( ($mpm & $obj['pm']) != $obj['pm'])
          {
            $allowed = false;
            break;
          }
        }
      }
    }

    if ($allowed==false)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }

    $info = $this->GetUserInfoOnAccId($accid);
    if ($info !== false && $this->IsLastAdmin($info['id']))
    {
      if ($defines_admin==false)
      {
        $this->lasterr = 'lock-out';
        return false;
      }
    }

    $stmt = $this->db->prepare('DELETE FROM AccAcc WHERE accid=:1');
    $stmt->bindValue(1,$accid);
    $stmt->execute();

    $stmt = $this->db->prepare('INSERT INTO AccACC '
    . ' (accid,callaccid,epermissions,applytype) '
    . ' VALUES '
    . ' (:1    ,:2       ,:3         ,:4) '
    );
    foreach($data as $obj)
    {
      //echo $obj['contactid'].' '.$obj['pm'];
      $stmt->bindValue(1,$accid);
      $stmt->bindValue(2,$obj['callaccid']);
      $stmt->bindValue(3,$obj['pm']);
      $stmt->bindValue(4,$obj['applytype']);
      $stmt->execute();

    }
    return true;
  }

  function AddUserAccount(&$userid,$data,$maingrp=0)
  {
    $allowed = false;
    $aupm = $this->actuinfo['permissions'];
    if ($aupm&PM_CONFIG) $allowed = true;
    if ($aupm&PM_USERS) $allowed = true;
    if ($aupm&PM_T2_ADMIN) $allowed = true;
    if ($aupm&PM_T3_ADMIN) $allowed = true;

    if (($this->anyaccpms&PM_USERS)!=0 && $allowed==false)
    {
      if ( ($this->GetAccPm($maingrp) &PM_USERS)!=0 )
      {
        $allowed = true;
      }
      	
    }

    if ( ($aupm&PM_SUBADMIN)!=0 && $allowed==false)
    {
      // a sub-admin
      // no further check here
      // this is in other functions such as SetUserPhones
      // only set manager
      $data['manager'] = $this->actuinfo['id'];
      // groups array will be corrected if not allowed
      // by SetGroupsOfWhichMemberLC
      $allowed = true;
    }

		//Add timestamp for password!
		$data['pwdchangedate'] = time();
		$data['tempdisabled'] = 0;
		if ($this->GetConfigValue('PwdHistory') > 0)
		{
			$data['pwdhistory'] = $data['pwd'];	//New account. This password is the first in the history list!
		}
		
    if ($allowed==false)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }

    if (file_exists('../apresa/conf/compact'))
    {
      $cnt = $this->GetTotalUserCount();
      // Compact has a web user limit
      if ($cnt+1 > 25)
      {
        $this->lasterr = 'compact-users';
        return false;
      }
    }

		if (isset($data['rawpwd']))
		{
			$rawpwd = $data['rawpwd'];
			unset($data['rawpwd']);
		}
		
    $stmt = $this->db->prepare('INSERT INTO Accessors(type) VALUES(0)');
    if ($stmt->execute()===false)
		{
			$this->onDbErr();
		}
    $accid = $this->db->lastInsertId('Accessors_id_seq');
    $sql = 'INSERT INTO Users (accid,'.implode(', ',array_keys($data)).') '
    . ' VALUES (?,'.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    } else {
      $ix = 1;
      $stmt->bindValue($ix,$accid);
      foreach($data as $value)
      {
        $ix ++;
        $stmt->bindValue($ix,$value);
      }
      $res = $stmt->execute();
			if ($res==false)
			{
				$this->OnDbErr();
			}
      $userid = $this->db->lastInsertId('Users_id_seq');
      	
      if ($maingrp!=0)
      {
        // With limited check (LC),
        // because check on adding to maingrp
        // has already been made,
        // and the function otherwise assumes
        // we are *editing* a user with no group,
        // might be not allowed,
        // while adding a new user is allowed.
        $this->SetGroupsOfWhichMemberLC($accid,array($maingrp));
      }
			
			if (isset($rawpwd))
			{
				$this->ChangePassword($userid,$rawpwd);
			}
      	
			$this->CalcTenantUsers();
      return true;
    } // stmt is OK
  }

  function RecalcPermissions()
  {
    $this->SetCurrentUserId( $this->actuinfo['id'] );
  }

  function IsLastAdmin($userid)
  {
    $userids = $this->GetAdmins();
    if (count($userids)==1)
    {
      if (isset($userids[$userid])) return true;
      return false;
    }
    return false;
  }

  // returns an array of user-ids of the admins
  // admins are those that have PM_CONFIG permission
  function GetAdmins()
  {
    $result = array();
    $sql = 'SELECT accid FROM AccAcc WHERE callaccid=0 AND (epermissions&'.PM_CONFIG.')>0';
    foreach($this->db->query($sql) as $row)
    {
      $accid = $row['accid'];
      // check is user list
      $sql = 'SELECT id,enabled FROM Users WHERE accid='.(int)$accid;
      $res = $this->db->query($sql);
			if ($res===false) continue;
      $row = $res->fetch();
			if ($row===false) continue;
      $userid = $row['id'];
      if ($row['enabled'])
      {
        $result[$userid] = 1;
      }
    }
    return $result;
  }

  function DeleteUserAccount($userid)
  {
    $theuser = $this->GetUserInfo($userid);
    if ($this->MayEditUserAccount($userid,$theuser['accid'])==false)
    {
      return false;
    }
    $int_userid = (int) $userid;
    // get info including accessor-id
    $userinfo = $this->GetUserInfo($int_userid);
    if (  (($this->actuinfo['permissions']&(PM_CONFIG|PM_SUBADMIN))==0)
    &&    (($this->actuinfo['permissions']&(PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))!=0)   )
    {
      // so PM_USERS or equiv
      // it may not delete admins
      if ($this->IsUserAdmin($userid))
      {
        // it is an admin
        $this->lasterr = 'pm-denied';
        return false;
      }
    }
    if (($this->actuinfo['permissions']&PM_CONFIG)==0
    && ($this->actuinfo['permissions']&PM_SUBADMIN)!=0)
    {
      // a sub-admin may only delete users for which he is the manager
      if ($userinfo['manager']!=$this->actuinfo['id'])
      {
        $this->lasterr = 'pm-denied';
        return false;
      }
    }
    // check if this is the last admin
    if ($this->IsLastAdmin($int_userid))
    {
      // then we are not allowed to delete
      $this->lasterr = 'lock-out';
      return false;
    }
    if (defined('DEMO') && $userid==1)
    {
      $this->lasterr = 'demo-editadmin';
      return false;
    }
    if (defined('DEMO') && $userid==2)
    {
      $this->lasterr = 'demo-protecteduser';
      return false;
    }

    // delete references
    // 1. specific permission
    $stmt = $this->db->prepare('DELETE FROM AccAcc WHERE accid=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$userinfo['accid']);
    $stmt->execute();
    // 1b. permissions to this account
    $stmt = $this->db->prepare('DELETE FROM AccAcc WHERE callaccid=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$userinfo['accid']);
    $stmt->execute();
    // 2. user phones
    $stmt = $this->db->prepare('DELETE FROM UserPhones WHERE accid=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$userinfo['accid']);
    $stmt->execute();
    // 3. group membership
    $stmt = $this->db->prepare('DELETE FROM GroupMembers WHERE accid=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$userinfo['accid']);
    $stmt->execute();
    // delete accessor
    $stmt = $this->db->prepare('DELETE FROM Accessors WHERE id=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$userinfo['accid']);
    $stmt->execute();
    // delete user account
    $stmt = $this->db->prepare('DELETE FROM Users WHERE id=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$userid);
    $stmt->execute();
    return true;
  }

	function IpPmsAddIp($ipaddr, $pm)
	{
    // int cast is important
    if (isset($this->ippms[$ipaddr]))
    {
      $this->ippms[$ipaddr] |= (int) $pm;
    } else
    {
      $this->ippms[$ipaddr] = (int) $pm;
    }
	}
	
	function LinePmsAddLine($linenr, $pm)
	{
    // int cast is important
    if (isset($this->linepms[$linenr]))
    {
      $this->linepms[$linenr] |= (int) $pm;
    } else
    {
      $this->linepms[$linenr] = (int) $pm;
    }
	}
	
  function CtPmsAddCt($ctid, $pm)
  {
    // int cast is important
    if (isset($this->ctpms[$ctid]))
    {
      $this->ctpms[$ctid] |= (int) $pm;
    } else
    {
      $this->ctpms[$ctid] = (int) $pm;
    }
  }

  function GetAccPm($accid)
  {
    if (isset($this->accpms[$accid]))
    {
      return $this->accpms[$accid];
    } else
    {
      return 0;
    }
  }

  function AddAccPms($accid, $pm)
  {
    $this->anyaccpms |= (int) $pm;
    if (isset($this->accpms[$accid]))
    {
      $this->accpms[$accid] |= (int) $pm;
    } else
    {
      $this->accpms[$accid] = (int) $pm;
    }
  }

  function CtPmsAddGroup($accid,$pm)
  {
    // also sub groups
    $groups = array($accid);
    $this->FindSubGroups($groups,$accid);
    // find group members...
    foreach ($groups as $groupaccid)
    {
      $this->AddAccPms($groupaccid,$pm);
      $members = $this->GetGroupMembers($groupaccid);
      foreach ($members as $member)
      {
        $this->CtPmsAddUser($member,$pm);
      }
    }
  }

	function EnsureUserPhones($accid)
	{
    if (isset($this->userphones[$accid]))
		{
			return;
		}

		$phone_list = $this->GetUserPhones($accid);
		$phones = array();
		$lines = array();
		$ips = array();
		foreach ($phone_list as $phone)
		{
			$phones[] = $phone;
			// is it a line contact?
			$contact = $this->GetContactInfo($phone);
			if ($contact['ctype']==1)
			{
				$lines[] = $contact['linenr'];
			}
			if ($contact['ctype']==2)
			{
				$ips[] = $contact['value'];
			}
		}
		$this->userphones[$accid] = $phones;
		$this->userphones_lines[$accid] = $lines;
		$this->userphones_ips[$accid] = $ips;
	}
	
  function CtPmsAddUser($accid,$pm)
  {
    $this->AddAccPms($accid,$pm);
    if ($accid!=$this->actuinfo['accid'])
    {
      $this->otherpm = true;
			if (($pm&PM_LIST)!=0)
			{
				$this->otherlistpm = true;
			}
    }
		$this->EnsureUserPhones($accid);
    foreach ($this->userphones[$accid] as $contactid)
    {
      $this->CtPmsAddCt($contactid,$pm);
    }
		foreach ($this->userphones_lines[$accid] as $linenr)
    {
      $this->LinePmsAddLine($linenr,$pm);
    }
		foreach ($this->userphones_ips[$accid] as $ipaddr)
    {
      $this->IpPmsAddIp($ipaddr,$pm);
    }
  }

	function RecPmsAdd($list)
	{
		foreach ($list as $recpm)
		{
			$this->recpms[ $recpm['recorderid'] ] = int_ensure_defined($this->recpms[ $recpm['recorderid'] ]) | $recpm['epermissions'];
		}
	}
	
  function CtPmsAddAccAcc($accaccs, $supergrp)
  {
    foreach ($accaccs as $accacc)
    {
      if ($supergrp)
      {
        // permissions in super groups only apply
        // if applytype is 1
        if ($accacc['applytype']!=1) continue;
      }
      $accid = $accacc['callaccid'];
      $pm = $accacc['epermissions'];
      if ($pm==0) continue;
      if ($accid>0)
      {
        // a specific user or group
        if (isset($this->allgroups[$accid]))
        {
          // it is a group
          $this->CtPmsAddGroup($accid,$pm);
        } else
        {
          // it is a user
          $this->CtPmsAddUser($accid,$pm);
        }
      } else {
        // special codes
        if ($accid==0)
        {
          if (($pm&(1+2+4+16))!=0)
          {
            $this->otherpm = true;
          }
					if (($pm&PM_LIST)!=0)
					{
						$this->otherlistpm = true;
					}
          $this->actuinfo['permissions'] |= $pm;
        }
        // -1 : self
        if ($accid==-1)
        {
          $this->CtPmsAddUser($this->actuinfo['accid'],$pm);
        }
        // -2 : same group
        if ($accid==-2)
        {
          foreach ($this->mygroups as $group)
          {
            $this->CtPmsAddGroup($group,$pm);
          }
        }
        // -3 : people in sub groups
        if ($accid==-3)
        {
          // add permission to sub groups
          foreach ($this->mysubgroups as $group)
          {
            $this->CtPmsAddGroup($group,$pm);
          }
        }
      }
    }
  }

  // assumes allgroups is already filled in
  // with SetCurrentUserId
  // used for showing inherited permissions
  function GetSuperGroups($groups)
  {
    $supergroups = array();
    foreach ($groups as $group)
    {
      // move to super group
      $group = $this->allgroups[$group];
      while ($group!=0)
      {
        // check if already visited
        if (in_array($group,$supergroups)) break;
        // check if a group
        if (isset($this->allgroups[$group])==false) break;
        // mark visited
        $supergroups[] = $group;
        // move to super group
        $group = $this->allgroups[$group];
      }
    }
    return $supergroups;
  }

	function ClearAccess()
	{
    $this->otherpm = false;
		$this->otherlistpm = false;
    $this->actuinfo = array();
    $this->actuinfo['permissions'] = 0;
    $this->accidlist = null;
    $this->myphones = null;
    $this->hasphones = false;
    $this->userphones = array();
    $this->ctpms = array();
		$this->linepms = array();
		$this->ippms = array();
    $this->accpms = array();
		$this->recpms = array();
    $this->anyaccpms = 0;
    $this->mygroups = array();
    $this->mysubgroups = array();
	}
	
	function RestrictAccess( $accid )
	{
		$this->ClearAccess();
		
		//$this->CtPmsAddAccAcc($this->GetAccAccPms($this->mygroups),false);		
    /*if ($accacc['applytype']!=1) continue;
      }
      $accid = $accacc['callaccid'];
      $pm = $accacc['epermissions'];
		*/
	}
	
	function DetermineAllGroups($force = false)
	{
		if ($force==false && isset($this->allgroups)) return;
    $grouplist = $this->GetGroupList();
		$this->allgroups = array();
    foreach ($grouplist as $group)
    {
      $this->allgroups[$group['accid']] = $group['super'];
    }
	}
	
  function SetCurrentUserId($userid)
  {
    $this->otherpm = false;
		$this->otherlistpm = false;
    $userInfo = $this->GetUserInfo($userid);
    
    if($userInfo === false)
    {
      return;
    }

    $this->actuinfo = $userInfo;
    $this->accidlist = $this->GetAccIdList( $this->actuinfo['accid'] );
    //$this->actaccid = $this->actuinfo['accid'];
    // merge permissions
    //$this->actuinfo['permissions'] |= $this->GetGroupPermissions($this->accidlist);

    // global permissions abolished
    $this->actuinfo['permissions'] = 0;

    $this->myphones = $this->GetUserPhones($this->actuinfo['accid']);
    $this->hasphones = count($this->myphones)>0;

    $this->userphones = array();
    unset( $this->allgroups );
		$this->DetermineAllGroups();

    // clear, will be refilled
    $this->ctpms = array();
		$this->linepms = array();
		$this->ippms = array();
		$this->recpms = array();
    $this->accpms = array();
    $this->anyaccpms = 0;

    $this->mygroups = $this->GetGroupsOfWhichMember( $this->actuinfo['accid'] );

    // Find super groups
    $supergroups = array();
    foreach ($this->mygroups as $group)
    {
      // move to super group
      $group = $this->allgroups[$group];
      while ($group!=0)
      {
        // check if already visited
        if (in_array($group,$supergroups)) break;
        // check if a group
        if (isset($this->allgroups[$group])==false) break;
        // mark visited
        $supergroups[] = $group;
        // move to super group
        $group = $this->allgroups[$group];
      }
    }

    // Find sub groups
    $this->mysubgroups = array();
    foreach ($this->mygroups as $group)
    {
      $this->FindSubGroups($this->mysubgroups,$group);
    }

    //echo '<p>my groups:';
    //var_dump($this->mygroups);
    //echo '<p>';
    //var_dump($this->mysubgroups);

    // Load personal permissions
    $this->CtPmsAddAccAcc($this->GetAccAccPms(array( $this->actuinfo['accid'] )),false);
    // Load group permissions
    $this->CtPmsAddAccAcc($this->GetAccAccPms($this->mygroups),false);
    // Load super group permissions
    //var_dump( $this->GetAccAccPms($supergroups) );
    $this->CtPmsAddAccAcc($this->GetAccAccPms($supergroups),true);
		
		$this->RecPmsAdd( $this->GetUserRecorderPms( $this->actuinfo['accid'] ) );

    // $this->actuinfo['accid']
    //echo '<p>ctmps:<p>';
    //var_dump( $this->ctpms );
    //exit;
    
    $this->EnsureUserPhones( $this->actuinfo['accid'] );

  }

  function FindSubGroups(&$subgroups,$group)
  {
    /*echo "</br>All groups: ";
    var_dump($this->allgroups);
    echo "end all groups</br>";*/
    foreach ($this->allgroups as $accid=>$super)
    {
      if ($group==$super)
      {
        if (in_array($accid,$subgroups)==false)
        {
          $subgroups[] = $accid;
          $this->FindSubGroups($subgroups,$accid);
        }
      }
    }

  }

  function GetActiveUserInfo()
  {
    return $this->actuinfo;
  }

  function NameInUseExcept($name,$accid)
  {
    $stmt = $this->db->prepare('SELECT id FROM Users WHERE name=:1 AND accid<>:2');
    $stmt->bindValue(1,$name);
    $stmt->bindValue(2,$accid);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
    {
      $stmt = $this->db->prepare('SELECT id FROM Groups WHERE name=:1 AND accid<>:2');
      $stmt->bindValue(1,$name);
      $stmt->bindValue(2,$accid);
      $stmt->execute();
      $row = $stmt->fetch();
      if ($row===FALSE)
      {
        return false;
      }
    }
    return true;
  }

  function NameInUse($name)
  {
    $stmt = $this->db->prepare('SELECT id FROM Users WHERE name=:1');
    $stmt->bindValue(1,$name);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
    {
      $stmt = $this->db->prepare('SELECT id FROM Groups WHERE name=:1');
      $stmt->bindValue(1,$name);
      $stmt->execute();
      $row = $stmt->fetch();
      if ($row===FALSE)
      {
        return false;
      }
    }
    return true;
  }

  function UsrExistsExcept($usr,$accid)
  {
    $stmt = $this->db->prepare('SELECT id FROM Users WHERE usr=:1 AND accid<>:2');
    $stmt->bindValue(1,$usr);
    $stmt->bindValue(2,$accid);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
    {
      return false;
    }
    return true;
  }

  function UsrExists($usr)
  {
    $stmt = $this->db->prepare('SELECT id FROM Users WHERE usr=:1');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
    {
      return false;
    }
    return true;
  }

	function ChangePassword($userid,$pwd,$do_checks = true)
	{
		//Double check the password policies!
    if($do_checks === false)
    {
      $PwdLength = 0;
      $PwdAuth = 0;
    }
    else
    {
      $PwdLength = $this->GetConfigValue('MinPwdLength');
		  $PwdAuth = $this->GetConfigValue('StrongPwdAuth');
    }

		if ($PwdLength > 0 && strlen($pwd) < $PwdLength)
		{
			$this->lasterr = 'newpwd-error';
			return false;
		}
		if ($PwdAuth)
		{
			if( !preg_match( '~[A-Z]~', $pwd) || !preg_match( '~[a-z]~', $pwd) || !preg_match( '~\d~', $pwd))
			{
				$this->lasterr = 'newpwd-error';
				return false;
			}
		}
		//Get the password history policy!
		$NumPwd = $this->GetConfigValue('PwdHistory');
		$pwdhash = DBInterface::CreatePwdHash($pwd);
		if ($NumPwd > 0)
		{
			$userinfo = $this->GetUserInfo($userid);
			$usr = $userinfo['usr'];
			$pwdhistory = $this->GetPwdHistory($usr);
			$passwords = explode(",", $pwdhistory);
			$num = count($passwords);
			if ($num == 0)
			{
				$pwdhistory = $pwdhash;
			}
			else
			{
				$usedBefore = false;
        foreach ($passwords as $dbhash)
        {
          // we cannot simply test for equality of pwdhash and dbhash
          // we need to use the verify function
          if (DBInterface::VerifyPwd($pwd,$dbhash))
          {
            $usedBefore = true;
          }
        }
        if ($usedBefore)
				{
					$this->lasterr = 'pwd-used-before';
					return false;		//New password cannot be used!
				}
				else
				{
					//Add the new password to the password history!
					if ($num >= $NumPwd)
					{
						while($num >= $NumPwd)
						{
							array_shift($passwords);	//Shift the first value out of the array!
							$num--;
						}
					}
					array_push($passwords, $pwdhash );
					$pwdhistory = implode(",", $passwords);
				}
				$this->SetPwdHistory($usr, $pwdhistory);
			}
		}
		$pwdchangedate = time();
    $stmt = $this->db->prepare('UPDATE Users SET pwd=:1, pwdchangedate=:2 WHERE id=:3');
    $stmt->bindValue(1,$pwdhash);
		$stmt->bindValue(2,$pwdchangedate);
    $stmt->bindValue(3,$userid);
		if ($stmt->execute()==false)
		{
			$this->OnDbErr();
			return false;
		}
    return true;
	}
	
  // dont use this function to verify a password
  static function CreatePwdHash($pwd)
  {
    if (function_exists('password_hash')==false)
    {
      return md5( $pwd );
    } else
    {
      return password_hash( $pwd , PASSWORD_DEFAULT );
    }
  }
  
  static function VerifyPwd($pwd, $dbhash)
  {
    if (!isset($dbhash) || strlen($dbhash)==0)
    {
      return false;
    }
    if (strlen($dbhash)==32 && $dbhash[0]!='$')
    {
      // MD5
      return (md5($pwd) == $dbhash);
    } else
    {
      return password_verify( $pwd , $dbhash );
    }
  }

  // uid is filled in when OK
  function TryLogin($usr,$pwd,&$userid)
  {
    $stmt = $this->db->prepare('SELECT id,pwd FROM Users WHERE usr=:1 AND enabled=1 AND tempdisabled=0');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
		{
      $this->lasterr = 'login-failed';
      return false;
    }
    if (DBInterface::VerifyPwd($pwd,$row['pwd'])==false)
    {
      $this->lasterr = 'login-failed';
      return false;
    }
    $userid = $row['id'];
    return true;
  }

  //uid is filled in when OK
  function TryLoginEx($usr,$pwd,&$userid,&$pwdnok)
  {
		//Does the user exist?
    $stmt = $this->db->prepare('SELECT id, tempdisabled, disabledtime FROM Users WHERE usr=:1 AND enabled=1');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
    {
      $this->lasterr = 'login-failed';
			$pwdnok = false;
      return false;
    }
		//Check if the account is temporarily disabled. Enable the account after the timeout!
		$tempdisabled = $row['tempdisabled'];
		if ($tempdisabled == 1)		//Temporarily disabled!
		{
			$disabledtime = $row['disabledtime'];
			$tempdisabledtime = $this->GetConfigValue('TempDisabledTime');
			if (time() > $disabledtime + $tempdisabledtime)
			{
				$this->WrongPassword($usr, 0, 0);		//Reset 'numwrongpwd' and 'tempdisabled'!
			}
			else
			{
				//The account is still disabled, so return
				$this->lasterr = 'temporarily disabled';
				return false;
			}
		}
		//The user exists. Is the password ok?
    $stmt = $this->db->prepare('SELECT id,pwd FROM Users WHERE usr=:1 AND enabled=1 AND tempdisabled=0');
    $stmt->bindValue(1,$usr);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
    {
      // should never happen
      $this->lasterr = 'login-failed';
			$pwdnok = false;
      return false;
    }
    if (DBInterface::VerifyPwd($pwd,$row['pwd'])==false)
    {
      $this->lasterr = 'login-failed';
			$pwdnok = true;
      return false;
    }
    $userid = $row['id'];
		$pwdnok = false;
    return true;
  }
	
	// retries a function until no DbError,
	// or until max tries is reached
	// returns the result of the function
	// the function f should contain a transaction
	function TransactionRetry( $f )
	{
		// settings:
		$maxtry = 600;
		$sleepms = 5;
		// init
		$ret = NULL;
		$cnt = 0;
		while (true)
		{
			$cnt ++;
			// We do retries only on the top level,
			// because the top level needs to retry,
			// because a failure in a lower level,
			// might be caused by a higher level,
			// and then it would never end.
			// We have a max try to avoid a freeze.
			try
			{
				$ret = $f();
				// function was performed without exceptions
				// we are ready
				break;
			} catch (DbError $e)
			{
				// debug: "DbError with msg: " . $e->getMessage() . "\n";
				// there was an error
				// cancel the transaction
				// this has to be done always, even on sublevels
				$this->CancelTransaction();
				if ($this->transactionLevel>0)
				{
					// no retry, throw instead
					throw $e;
					break;
				}
				if ($cnt>=$maxtry)
				{
					// TODO: do the same as OnDbErr
					// LogDbErrExit($e)
					// log and exit 
					echo $e->getTraceAsString() . "\r\n";
					var_dump( $this->db->errorInfo() );
					exit;
				}
			}
      catch(PDOException $e)
      {
        $this->CancelTransaction();
        if ($this->transactionLevel>0)
				{
					// no retry, throw instead
					throw $e;
					break;
				}
				if ($cnt>=$maxtry)
				{
					echo $e->getTraceAsString() . "\r\n";
          echo $e->getMessage() . "\r\n";
					var_dump( $this->db->errorInfo() );
					exit;
				}
      }
			// there was an exception, and we need to retry
			// sleep to allow other threads to finish
			usleep( 1000 * $sleepms );
		}
		// debug logging
		if ($cnt>1)
		{
			echo "OK (anyretry) after $cnt tries\n";
		}
		// returning the result of the function
		return $ret;
	}
	
  function StartTransaction()
  {
    if (DB_TYPE=='sqlite3')
    {
      $this->db->exec('BEGIN IMMEDIATE');
    } else
    {
			$this->transactionLevel ++;
			if ($this->transactionLevel==1)
			{
				if ($this->db->beginTransaction()==false)
        {
          // failed to start transaction
          throw new DbError("beginTransaction failed");
        }
				$this->db->exec('SET TRANSACTION ISOLATION LEVEL REPEATABLE READ');
			} else
			{
				$this->db->exec('SAVEPOINT LEVEL' . (int) $this->transactionLevel);
			}
    }
  }

  function CancelTransaction()
  {
    if (DB_TYPE=='sqlite3')
    {
      $this->db->exec('ROLLBACK');
    } else
    {
			if ($this->transactionLevel==1)
			{
				$this->db->rollBack();
			} else
			{
				$this->db->exec('ROLLBACK TO SAVEPOINT LEVEL' . (int) $this->transactionLevel);
			}
			$this->transactionLevel --;
    }
  }

  function CommitTransaction()
  {
    if (DB_TYPE=='sqlite3')
    {
      $this->db->exec('COMMIT');
    } else
    {
			if ($this->transactionLevel==1)
			{
				$this->db->commit();
			} else
			{
				$this->db->exec('RELEASE SAVEPOINT LEVEL' . (int) $this->transactionLevel);
			}
			$this->transactionLevel --;
    }
  }

  function csvDataObj($value, $inside)
  {
    if ($inside==false)
    {
      if ($value=='NULL') return NULL;
    }
    return $value;
  }
  
	// NOTE: very similar to CSV parse in user import.
	// a difference is the handling of NULL.
  function csvSplit($line)
  {
    $ret = array();
    $len = strlen($line);
    $st = 0;
    /* 0: outside data
     * 1: inside "
     * 2: data without "
     */
    $delim = '"';
    $sep = ',';
    $hadquote = false;
    $cell = '';
    for ($i=0;$i<$len;$i++)
    {
      $ch = $line[$i];
      if ($st==0){
        if (ctype_space($ch))
        {
          // do nothing
        } else if ($ch==$delim)
        {
          $st = 1;
          $hadquote = true;
        } else if ($ch==$sep)
        {
          // end of cell
          $ret[] = $this->csvDataObj($cell,$hadquote);
          $cell = "";
          $hadquote = false;
        } else
        {
          $st = 2;
          $cell .= $ch;
        }
      } else if ($st==1){
        if ($ch==$delim)
        {
					if ($i+1<$len && $line[$i+1]==$delim)
					{
						// double delim
						$cell .= $delim;
						$i = $i + 1;
					} else
					{
						$st = 0;
					}
        } else
        {
          $cell .= $ch;
        }
      } else if ($st==2){
        if ($ch==$sep)
        {
          // end of cell
          $ret[] = $this->csvDataObj($cell,$hadquote);
          $cell = "";
          $hadquote = false;
          $st = 0;
        } else
        {
          $cell .= $ch;
        }
      }
    }
		$ret[] = $this->csvDataObj($cell,$hadquote);
    return $ret;
  }
  
  function CsvQuote($t)
  {
    if ($t === NULL) return 'NULL';
    if (is_integer($t))
    {
      return ''.$t;
    }
    // duplicate " inside quotes
    $t = str_replace('"','""',$t);
    return '"'.$t.'"';
  }

  function GetBackupConfigConfig()
  {
    // empty means: use default
		return array
    ( 'Configs'
    => "SELECT * FROM Configs WHERE "
    .	" (NOT param LIKE '%ChCnt') "
    .	" AND (NOT param LIKE 'Lic%') "
    .	" AND (NOT param LIKE 'CompanyName%') "
    .	" AND (NOT param LIKE '%_RecCnt') "
    .	" AND (NOT param LIKE '%_DiskUsage') "
    .	" AND (NOT param LIKE '%_DiskUsageCheckTime') "
    .	" AND (NOT param LIKE '%_BackupRecCnt') "
    .	" AND (NOT param LIKE '%_BackupDiskUsage') "
    .	" AND param<>'crec' "
    .	" AND param<>'calleval' "
    .	" AND param<>'PlayPkey' "
    .	" AND param<>'RecPkey' "
    .	" AND param<>'Version' "
    .	" AND param<>'ApresaVersion' "
    .	" AND param<>'ApresaSerial' "
    .	" AND param<>'ServiceEnd' "
    .	" AND param<>'ServiceSysDescr' "
		.	" AND part<>'dat' "
    // no clustering status data
    .	" AND param<>'SaveB' "
    .	" AND param<>'SaveN' "
    .	" AND param<>'SaveS' "
    .	" AND param<>'ReconfigO' "
    .	" AND param<>'ConfRestartO' "
    .	" AND param<>'ConfFreshO' "
    , 'Users' => ''
    , 'Groups' => ''
		, 'GroupMembers' => ''
    , 'Accessors' => ''
    , 'UserPhones' => "SELECT UserPhones.id, accid, callerid"
		.	" , linenr, ctype, value FROM UserPhones "
    . " LEFT JOIN CallerIDs ON UserPhones.contactid = CallerIDs.contactid "
    . " LEFT JOIN Lines ON UserPhones.contactid = Lines.contactid "
		. " LEFT JOIN Contacts ON UserPhones.contactid = Contacts.id "
    , 'AccAcc' => ''
    );
  }

	// restores a database table from the previous
	// backup (in the form of a (CSV) stream)
  function RestoreConfigTable( $thisTabnm, $stream, $RestoreNetw=false )
  {
    $tabcfg = $this->GetBackupConfigConfig();
		// this also prevents other tables
		// from being overwritten by the restore
    foreach ($tabcfg as $tabnm=>$tabsql)
    {
      // only for one table
      if ($tabnm!=$thisTabnm) continue;
      if ($tabsql=='')
      {
        $tabsql = 'SELECT * FROM '.$tabnm;
      }
      
      // load into memory
      $csvRows = array();
      $csvParser = new CsvParser();
      $csvParser->detectPureNull = true;
      while ( ! feof($stream) )
      {
        $line = fgets( $stream );
        
        $csvParser->addLine( $line );
        $parts = $csvParser->getData();
        if ($parts !== null)
        {
          $csvRows[] = $parts;
        }
      }
      $csvParser->flush();
      $parts = $csvParser->getData();
      if ($parts !== null)
      {
        $csvRows[] = $parts;
      }
      
      if ($tabnm=='UserPhones')
      {
        // needs special handling
				// because export was also special
				// the export contains the caller id,
				// or line number, instead of directly
				// the contact id.
				// we need to convert it to a contact id.

				// cache
				$ctlines = $this->GetLineList();
				$linemap = array();
				foreach ($ctlines as $ctline)
				{
					$linemap[$ctline['linenr']] = $ctline['contactid'];
				}

				// Clear existing content
				$delsql = 'DELETE FROM ' . $tabnm;
				$this->db->exec( $delsql );
				// Fill it with the content in the stream
				$stmt = NULL;
				$addsql = "INSERT INTO $tabnm (id,accid,contactid) "
				. " VALUES(?,?,?)";
				$stmt = $this->db->prepare($addsql);
				if ($stmt===FALSE)
				{
					$this->OnDbErr();
				}
				$firstRecord = true;
        foreach ($csvRows as $parts)
        {
					if ($firstRecord)
					{
						$cols = implode(',',$parts);
						// we expect a certain header
						if ($cols=='id,accid,callerid,linenr,ctype,value')
						{
							$includectype = true;
							$reqcolcnt = 6;
						} else if ($cols=='id,accid,callerid,linenr')
						{
							$includectype = false;
							$reqcolcnt = 4;
						} else if ($cols=='' && feof($stream))
						{
							// accept completely empty
							break;
						} else
						{
							$this->lasterr = 'formaterr';
							return false;
						}
						$firstRecord = false;
						continue;
					}
					$partcnt = count($parts);
					// data must have same number of columns
					if ($partcnt != $reqcolcnt) break;
					// not foreach because we have null values
					$contactid = NULL;
					if ($parts[2]!==NULL)
					{
						$res = $this->GetOrCreateCallerID($parts[2],'',2);
						$contactid = $res["contactid"];
					}
					if ($parts[3]!==NULL)
					{
		        $linenr = $parts[3];
						if (isset($linemap[$linenr])==false)
		        {
							//echo 'Line: '.$parts[3].' not found';
							$this->lasterr = 'line-not-found';
							// this will cancel the complete import
							return false;
						}
          	$contactid = $linemap[$linenr];
					}
					if ($includectype)
					{
						$col_ctype = (int) $parts[4];
						if ($col_ctype!=0)
						{
							$col_value = $parts[5];
							$contactid = $this->GetOrCreateInternalContact( $col_ctype, $col_value, '' );
						}
					}
					if ($contactid!==NULL)
					{
						$stmt->bindValue(1,$parts[0]);
						$stmt->bindValue(2,$parts[1]);
						$stmt->bindValue(3,$contactid);
            if ($stmt->execute()==false)
            {
              $this->OnDbErr();
            }
					}
					$stmt->closeCursor();
				}
        break;
      }
      // Clear existing content
      $delsql = str_replace('SELECT *','DELETE',$tabsql);
      if ($tabnm=='Configs' && $RestoreNetw==false)
      {
        $delsql .=	" AND part<>'n' ";
        $delsql .=	" AND part<>'cluster' ";
      }
      $this->db->exec( $delsql );
      // Fill it with the content in the stream
      $stmt = NULL;
			$testStmt = NULL;
			if ($tabnm=='Configs')
			{
				// special handling to avoid duplicates
				$testsql = "SELECT * FROM Configs WHERE param=?";
				$testStmt = $this->db->prepare($testsql);
				if ($testStmt===FALSE)
				{
					$this->OnDbErr();
				}
			}
			$colnames = $this->GetTableColumns($tabnm);
			$firstRecord = true;
      foreach ($csvRows as $parts)
      {
        if ($firstRecord)
        {
					// use same column order for insert
          $cols = implode(',',$parts);
					if ($cols=='' && feof($stream))
					{
						// completely empty
						break;
					}
          $q = array();
          foreach ($parts as $part)
          {
						if (isset($colnames[$part])==false)
						{
							$this->lasterr = 'formaterr';
							return false;
						}
            $q[] = '?';
          }
          $qs = implode(',',$q);
          $addsql = "INSERT INTO $tabnm ($cols) VALUES($qs)";
          $stmt = $this->db->prepare($addsql);
          if ($stmt===FALSE)
          {
            $this->OnDbErr();
          }
          $firstRecord = false;
          continue;
        }
				$partcnt = count($parts);
				// data must have same number of columns
        if (count($parts) != count($q)) break;

        if ($tabnm=='Configs' && $RestoreNetw==false)
        {
          // skip importing network and cluster settings
          $cfgPart = $parts[2];
          if ($cfgPart=='n' || $cfgPart=='cluster')
          {
            continue;
          }
        }
        
				if ($testStmt!=NULL)
				{
          $skip = false;
					// prevent duplicate Config items
					// param column is the first
					$testStmt->bindValue(1,$parts[0]); 
					if ($testStmt->execute()==false)
					{
						$this->OnDbErr();
					}
					if ($testStmt->fetch() !== false)
					{
						// already exists
						$skip = true;
					}
					$testStmt->closeCursor();
          if ($skip) continue;
				}
        
				// not foreach because we have null values
        for ($partix=0;$partix<$partcnt;$partix++)
				{
					$part = $parts[ $partix ];
          //echo "Part $partix: $part\n";
          $stmt->bindValue($partix+1,$part);
				}
        if ($stmt->execute()==false)
        {
          $this->OnDbErr();
        }
        $stmt->closeCursor();
      }
    }
		foreach ($tabcfg as $tabnm=>$tabsql)
    {
			if ($tabnm=='Configs') continue;
			$this->RestoreSequence( $tabnm );
		}
		return true;
  }

	// export the configuration
	// Returns an array of CSV files as strings.
  function ExportConfig( $Conf )
  {
    $tabcfg = $this->GetBackupConfigConfig();
    $tables = array();
    foreach ($tabcfg as $tabnm=>$tabsql)
    {
      if ($tabsql=='')
      {
        $tabsql = 'SELECT * FROM '.$tabnm;
      }
      if ($tabnm=='Configs')
      {
        if ( $Conf['BckConf'] == 0 ) continue;
      } else
      {
        if ( $Conf['BckUsr'] == 0 ) continue;
      }

      $stmt = $this->db->prepare($tabsql);
      if ($stmt===FALSE)
      {
        $this->OnDbErr();
      }
      if ($stmt->execute()==false)
      {
        $this->OnDbErr();
      }
      $output = '';
      $firstRecord = true;
      while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
      {
        if ($firstRecord)
        {
          $firstRecord = false;
          $first = true;
          $line = '';
					// column order may vary, we can handle that
					// during import
          foreach ($row as $col=>$val)
          {
            if ($first) $first = false;
            else $line .= ',';
            $line .= $this->CsvQuote($col);
          }
          $output .= $line . "\n";
        }
        $first = true;
        $line = '';
        foreach ($row as $col=>$val)
        {
          if ($first) $first = false;
          else $line .= ',';
          $line .= $this->CsvQuote($val);
        }
        $output .= $line . "\n";
      }
      $stmt = NULL;
      $tables[$tabnm] = $output;
    }
    return $tables;
  }

	function GetCachedConfigValue($param)
	{
		if (isset($this->cachedsettings[$param])==false)
		{
			$this->cachedsettings[$param] = $this->GetConfigValue($param);
		}
		return $this->cachedsettings[$param];
	}
	
  function GetConfigValue($param,$part='?')
  {
		$ref_db =& $this->db;
		if ($this->IsMachineCfgKey($param,$part))
		{
			$ref_db =& $this->machinedb;
		}
    $stmt = $ref_db->prepare('SELECT value FROM Configs WHERE param=?');
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$param);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return $stmt->fetchColumn(0);
  }

	function MachineLoadConfigs(&$configs_ref)
	{
    foreach($this->machinedb->query('SELECT param,value FROM Configs') as $row)
    {
      $configs_ref[$row['param']] = $row['value'];
    }
	}
	
	function MachineLoadConfigsPart(&$configs_ref,$part)
	{
		$stmt = $this->machinedb->prepare('SELECT param,value FROM Configs WHERE part=?');
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$part);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch())
    {
      $configs_ref[$row['param']] = $row['value'];
    }
	}

	var $machineCfgKeys = array(
	// the network config. 1
	'DHCP'=>1,'IPaddr'=>1,'IPmask'=>1
	,'IPgate'=>1,'IPname'=>1
	,'DNSaddr'=>1
	// network config. 2
	,'Network2'=>1
	,'n2_DHCP'=>1,'n2_IPaddr'=>1,'n2_IPmask'=>1
	,'n2_IPgate'=>1
	,'n2_DNSaddr'=>1
	,'SMTPhost'=>1,'SMTPfrom'=>1
	,'SMTPusr'=>1,'SMTPpwd'=>1,'SMTPport'=>1
	,'SMTPenc'=>1
	// considered, but not (maybe later)
	//,'WebSPort'=>1
	//,'WebMaxCon'=>1
	
	// licensing:
	,'LicKey'=>1
	,'calleval'=>1
	,'ChCnt'=>1	,'G729ChCnt'=>1	,'DstChCnt'=>1	,'AnalogChCnt'=>1	,'PriChCnt'=>1
	,'MobileChCnt'=>1	,'RtaChCnt'=>1
	,'CompanyName'=>1 , 'CompanyNameSet'=>1
	,'LicStartDay'=>1	,'LicSystem'=>1	,'LicOnce'=>1	,'LicFailMail'=>1
	,'LicFailMailGrace'=>1	,'LicFailDaysLeft'=>1	,'LicFailDays'=>1
	,'LicFailDay'=>1	,'LicGrace'=>1	,'LicTrial'=>1	,'LicTrialDaysLeft'=>1
	,'ServiceEnd'=>1 , 'ServiceSysDescr'=>1
	,'MobileDevConf'=>1 // deduced from licensing
  
  // health and alarms
  ,'HealthIgnoreList'=>1 ,'SimulateError'=>1
  
	);
	
	function IsMachineCfgKey($cfgkey,$part)
	{
		if ($this->sharedb)
		{
			if ($part=='crec')
			{
				return true;
			}
			if (isset($this->machineCfgKeys[$cfgkey]))
			{
				return true;
			}
		}
		return false;
	}
	
  function LoadConfigsPart($part)
  {
    $configs = array();
    $stmt = $this->db->prepare('SELECT param,value FROM Configs WHERE part=?');
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$part);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch())
    {
			if ($this->sharedb)
			{
				if ($this->IsMachineCfgKey($row['param'],$part))
				{
					// skip, but shouldnt be there anyway
					continue;
				}
			}
      $configs[$row['param']] = $row['value'];
    }
		if ($this->sharedb)
		{
			$this->MachineLoadConfigsPart($configs,$part);
		}
    return $configs;
  }
	
  function LoadConfigs()
  {
    $configs = array();
    foreach($this->db->query('SELECT param,value,part FROM Configs') as $row)
    {
			if ($this->sharedb)
			{
				if ($this->IsMachineCfgKey($row['param'],$row['part']))
				{
					// skip, but shouldnt be there anyway
					continue;
				}
			}
      $configs[$row['param']] = $row['value'];
    }
		if ($this->sharedb)
		{
			$this->MachineLoadConfigs($configs);
		}
    return $configs;
  }

	function MachineSetConfigsPart($configs,$part)
	{
    $this->machinedb->exec('BEGIN IMMEDIATE');
    $delStmt = $this->machinedb->prepare('DELETE FROM Configs WHERE param=?');
    if ( ! $delStmt)
    {
      $this->OnDbErr(true);
    }
    $addStmt = $this->machinedb->prepare('INSERT INTO Configs(param,value,part) VALUES(?,?,?)');
    if ( ! $addStmt)
    {
      $this->OnDbErr(true);
    }
    foreach ($configs as $param=>$value)
    {
      $delStmt->bindValue(1,$param);
      if ($delStmt->execute()==false)
      {
        $this->OnDbErr(true);
      }
      $addStmt->bindValue(1,$param);
      $addStmt->bindValue(2,$value);
      $addStmt->bindValue(3,$part);
      if ($addStmt->execute()==false)
      {
        $this->OnDbErr(true);
      }
    }
    $this->machinedb->exec('COMMIT');
    return true;
	}

	function AddToConfig($param,$value,$part)
	{
    if (DB_TYPE=='sqlite3')
    {
			$stmt = $this->db->prepare('UPDATE Configs SET value = value + ? WHERE param=?');
		} else
		{
			$stmt = $this->db->prepare('UPDATE Configs SET value = value::bigint + ? WHERE param=?');
		}
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$value);
    $stmt->bindValue(2,$param);
    if ($stmt->execute()==false) $this->OnDbErr();
		if ($stmt->rowCount()==0)
		{
			// param does not exist in db yet, then add it
			$addStmt = $this->db->prepare('INSERT INTO Configs(param,value,part) VALUES(?,?,?)');
			if ( ! $addStmt) $this->OnDbErr();
      $addStmt->bindValue(1,$param);
      $addStmt->bindValue(2,$value);
      $addStmt->bindValue(3,$part);
      if ($addStmt->execute()==false) $this->OnDbErr();
		}
	}

	function SetConfigsPart($configs,$part)
	{
		$cthis = $this;
		$f = function () use ($cthis,$configs,$part)
		{
			return $cthis->SetConfigsPartT($configs,$part);
		};
		return $this->TransactionRetry( $f );
	}
	
  function SetConfigsPartT($configs,$part)
  {
    $machine_configs = array();
		$this->StartTransaction();
    $delStmt = $this->db->prepare('DELETE FROM Configs WHERE param=?');
    if ( ! $delStmt)
    {
      $this->OnDbErr();
    }
    $addStmt = $this->db->prepare('INSERT INTO Configs(param,value,part) VALUES(?,?,?)');
    if ( ! $addStmt)
    {
      $this->OnDbErr();
    }
    foreach ($configs as $param=>$value)
    {
			if ($this->IsMachineCfgKey($param,$part))
			{
				$machine_configs[ $param ] = $value;
				continue;
			}
      $delStmt->bindValue(1,$param);
      if ($delStmt->execute()==false)
      {
        $this->OnDbErr();
      }
      $addStmt->bindValue(1,$param);
      $addStmt->bindValue(2,$value);
      $addStmt->bindValue(3,$part);
      if ($addStmt->execute()==false)
      {
        $this->OnDbErr();
      }
    }
    $this->CommitTransaction();
		if (count($machine_configs)>0)
		{
			$this->MachineSetConfigsPart($machine_configs,$part);
		}
    return true;
  }

	function DeleteConfig($param)
	{
    $stmt = $this->db->prepare('DELETE FROM Configs WHERE param = ?');
    $stmt->bindValue(1,$param);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}
	
	function DeleteConfigsWithPrefix($prefix)
	{
    $stmt = $this->db->prepare('DELETE FROM Configs WHERE PARAM LIKE ?');
    $stmt->bindValue(1,$prefix . '%');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}
	
	function GetConfigsWithPrefix($prefix)
	{
		$configs = array();
		$stmt = $this->db->prepare('SELECT * FROM Configs WHERE param LIKE ?');
		$stmt->bindValue(1,$prefix . '%');
		if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		
		while($row = $stmt->fetch())
		{
			$start = strlen($prefix);
			$configs[substr($row['param'], $start)] = $row['value'];
		}
		
		return $configs;
	}

	function DeleteFromConfigs($param)
	{
    $stmt = $this->db->prepare('DELETE FROM Configs WHERE param=?');
    $stmt->bindValue(1,$param);
    if($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}
	
  function ClearActiveCalls()
  {
    $stmt = $this->db->prepare('DELETE FROM ActiveCalls');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }

  function RemoveActiveCallOnId($id)
  {
    $stmt = $this->db->prepare('DELETE FROM ActiveCalls WHERE id=?');
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }

  function RemoveActiveCall($remoteid)
  {
    $stmt = $this->db->prepare('DELETE FROM ActiveCalls WHERE remoteid=?');
    $stmt->bindValue(1,$remoteid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }
  
  function DeleteActiveCallSpin($time)
  {
    $stmt = $this->db->prepare('DELETE FROM ActiveCalls WHERE (flags&1024)=1024 AND startdate<=?');
    $stmt->bindValue(1,$time);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }

  function UpdateActiveCall($remoteid,$data)
  {
    $sql = 'UPDATE ActiveCalls SET ';
    $first = true;
    foreach($data as $col=>$val)
    {
      if ($first) $first = false;
      else $sql .= ',';
      $sql .= ' ' . $col . ' = ? ';
    }
    $sql .= ' WHERE remoteid=?';

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix++;
    $stmt->bindValue($ix,$remoteid);
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }

  function InsertActiveCall($data)
  {
    $sql = 'INSERT INTO ActiveCalls ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }

	function GetActivityOtherRecs()
	{
    $result = array();
		$sql = 'SELECT description, lasttry, recorderid FROM ImportTargets WHERE inactivitycheck=1 AND enabled=1';	//lasttry = time lastfile was imported!
    foreach($this->db->query($sql) as $row)
    {
      $result[] = $row;
    }
    return $result;
	}

  function GetCallCountSpecial($startdate,$enddate
			,$linenr,$recorderid,$tenantid
			,$userphones=false
			)
  {
    $startdate = (int) $startdate;
		$enddate = (int) $enddate;
    $linenr = (int) $linenr;
    $sql = 'SELECT COUNT(Id) FROM Calls '
    . ' WHERE startdate>=' . $startdate
    . ' AND startdate+duration>=' . $enddate
    ;
    if ($linenr!=-1) $sql .= ' AND linenr='. $linenr;
		if ($recorderid!=-1) $sql .= ' AND recorderid='. $recorderid;
		if ($tenantid!=-1) $sql .= ' AND tenantid='.$tenantid;
		if ($userphones!==false)
		{
			$sql .= $this->GetEnhContactsFilter($userphones);
		}
    return $this->db->query($sql)->fetchColumn(0);
  }

  // for checking inactivity
	function GetActiveCallsCountPhones($tenantid,$userphones)
	{
		$sql = 'SELECT COUNT(Id) FROM ActiveCalls WHERE ' . $this->GetTrue();
		if ($tenantid!=-1) $sql .= ' AND tenantid='.$tenantid;
		$sql .= $this->GetEnhContactsFilter($userphones);
    return $this->db->query($sql)->fetchColumn(0);
	}
	
  // for checking inactivity
  function GetActiveCallsCountOfLine($linenr)
  {
    $sql = 'SELECT COUNT(Id) FROM ActiveCalls WHERE linenr='.(int) $linenr;
    return $this->db->query($sql)->fetchColumn(0);
  }

  // to check if restart is safe
  function GetActiveCallsCount()
  {
    return $this->db->query('SELECT COUNT(Id) FROM ActiveCalls WHERE (processingstate&128)=0')->fetchColumn(0);
  }
	
	function GetActiveCallsProcessing()
	{
    return $this->db->query('SELECT COUNT(Id) FROM ActiveCalls WHERE (processingstate&4)=4')->fetchColumn(0);
	}

  // for checking inactivity
  function GetActiveCallsCountPerTenant($tenantid)
  {
    return $this->db->query('SELECT COUNT(Id) FROM ActiveCalls WHERE tenantid='.$tenantid)->fetchColumn(0);
  }

  // for checking inactivity
  function GetActiveCallsCountForRecorder($recorderid)
  {
		return $this->db->query('SELECT COUNT(Id) FROM ActiveCalls WHERE recorderid='.$recorderid)->fetchColumn(0);
  }
	
  // for checking inactivity
  function GetOldestActiveCall($ignoreTunnelCalls = true)
  {
		if($ignoreTunnelCalls)
		{
			return $this->db->query('SELECT startdate FROM ActiveCalls ORDER BY startdate ASC LIMIT 1')->fetchColumn(0);
		}
		else
		{
			return $this->db->query('SELECT startdate FROM ActiveCalls WHERE flags & 128 = 0 ORDER BY startdate ASC LIMIT 1')->fetchColumn(0);
		}
  }

  function GetActiveCallsForClientCon()
  {
    $result = array();
    $sqlstmt = 'SELECT ActiveCalls.id, remoteid, startdate, '
    . ' local, remote, localcontact, remotecontact, '
    . ' cloc.name as lname,crem.name as rname, '
    . ' direction, isarchived, callextra, '
    . ' linenr, isarchived, callextra, startms, '
    . ' local2, remote2, tenantid, '
    . ' notes, notes2, category ' // for web client
    . ' FROM ActiveCalls '
    . ' LEFT OUTER JOIN contacts AS cloc ON ActiveCalls.localcontact = cloc.id'
    . ' LEFT OUTER JOIN contacts AS crem ON ActiveCalls.remotecontact = crem.id'
    . ' WHERE (processingstate&132)=0 '
    . ' ORDER BY startdate ';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;    
  }
  
  function GetFullActiveCallList()
  {
    $result = array();
    $sqlstmt = 'SELECT * FROM ActiveCalls '
    . ' ORDER BY startdate ';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
  }

  function GetActiveCallList()
  {
    // Check if and which filtering we need to apply
    // Check global permission
    $filter = ' WHERE (processingstate&132)=0 ';
    $cts = null;
		$recorders = null;
    if (($this->actuinfo['permissions']&PM_LIST)==0)
    {
      $cts = array();
      foreach ($this->ctpms as $contactid=>$pm)
      {
        if (($pm&PM_LIST)==0) continue;
        $cts[] = $contactid;
      }
			$recorders = array();
      foreach ($this->recpms as $recorderid=>$pm)
      {
        if (($pm&PM_LIST)==0) continue;
        $recorders[] = $recorderid;
      }
    }
    return $this->GetActiveCallList_CTs($cts,$recorders);
  }

	// cts: array of contacts
	// returns: SQL part that filters on these contacts
	// can handle caller IDs, lines, and IP addresses.
	function GetEnhContactsFilter($cts, $recorders=null)
	{
		$UsrFilterLocal = $this->GetCachedConfigValue('UsrFilterLocal')!=0;
		$filter = '';
		$iplist = '';
		$linelist = '';
		foreach ($cts as $contactid)
		{
			$ctinfo = $this->GetContactInfo($contactid);
			if ($ctinfo['ctype']>=2)
			{
				if ($iplist!='') $iplist .= ',';
				$iplist .= $this->db->quote($ctinfo['value']);
			}
			if ($ctinfo['ctype']==1)
			{
				if ($linelist!='') $linelist .= ',';
				$linelist .= (int) $ctinfo['linenr'];
			}
		}
		$reclist = '';
		$first = true;
		if ($recorders!=null)
		{
			foreach ($recorders as $recorderid)
			{
				if ($first==false) $reclist .= ',';
				$reclist .= $recorderid;
				$first = false;
			}
		}
		$ctlist = implode(',',$cts);
		$filter .= ' AND ('.$this->GetIn('localcontact',$ctlist);
		if ($UsrFilterLocal==false)
		{
			$filter .= ' OR '.$this->GetIn('remotecontact',$ctlist);
			$filter .= ' OR '.$this->GetIn('connectedcontact',$ctlist);
		}
		if ($linelist!='')
		{
			$filter .= ' OR '.$this->GetIn('linenr',$linelist);
		}
		if ($iplist!='')
		{
			$filter .= ' OR '.$this->GetIn('local2',$iplist);
			if ($UsrFilterLocal==false)
			{
				$filter .= ' OR '.$this->GetIn('remote2',$iplist);
			}
		}
		if ($reclist!='')
		{
			$filter .= ' OR ' . $this->GetIn('recorderid',$reclist);
		}
		$filter .= ') ';
		return $filter;
	}
	
  function GetActiveCallList_CTs($cts, $recorders=null)
  {
    // Check if and which filtering we need to apply
    // Check global permission
    $filter = ' WHERE (processingstate&132)=0 ';
    if ($cts!==null)
    {
			$filter .= $this->GetEnhContactsFilter($cts,$recorders);
    }
		$MultiTenancy = $this->GetCachedConfigValue('MultiTenancy')!=0;
		if ($MultiTenancy)
		{
			if ($this->actuinfo['tenantid'] !== null)
			{
				$filter .= ' AND ActiveCalls.tenantid=' . (int) $this->actuinfo['tenantid'];
			}
		}

    // note: LEFT OUTER JOIN is the same as LEFT JOIN
    
    if ($this->UseTenantContactList())
    {
      $tenantid = (int)$this->actuinfo['tenantid'];
      // tenant contact list
      $contactJoins = ''
      . ' LEFT JOIN TenantContacts AS cloc ON ActiveCalls.localcontact = cloc.contactid AND cloc.tenantid = ' . $tenantid . ' '
      . ' LEFT JOIN TenantContacts AS crem ON ActiveCalls.remotecontact = crem.contactid AND crem.tenantid = ' . $tenantid . ' '
      . ' LEFT JOIN TenantContacts AS ccon ON ActiveCalls.connectedcontact = ccon.contactid AND ccon.tenantid = ' . $tenantid . ' '
      ;
    } else
    {
      // system contact list
      $contactJoins = ''
      . ' LEFT OUTER JOIN contacts AS cloc ON ActiveCalls.localcontact = cloc.id'
      . ' LEFT OUTER JOIN contacts AS crem ON ActiveCalls.remotecontact = crem.id'
      . ' LEFT OUTER JOIN contacts AS ccon ON ActiveCalls.connectedcontact = ccon.id'
      ;
    }

    $result = array();
    $sqlstmt = 'SELECT ActiveCalls.id,ActiveCalls.remoteid,startdate,duration,'
    . ' remote,local,remote2,local2,remotefull,localfull, '
    . ' connected, source, filename, '
    . ' direction,notes,notes2,notes3,notes4,notes5,category,'
    . ' localcontact,remotecontact,connectedcontact, '
    . ' cloc.name as lname,crem.name as rname, '
    . ' ccon.name as cname, '
    . ' linenr, isarchived, callextra, startms, '
		. ' recorderid '
    . ' FROM ActiveCalls '
    . $contactJoins
    . $filter
    . ' ORDER BY startdate ';
    //echo $sqlstmt.'<p>';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch())
    {
      // fill in permission for the call
      $row['pm'] = $this->GetCallPermissions($row);
      $result[] = $row;
    }
    return $result;
  }

  // TODO: fix this mess
  // - we often get useless errors, because the error could be in the statement,
  //   and then the database handle does not have error info
  // - probably better to pass the object that has erred (if we know), than this machine boolean
  //   so we could have db, machinedb, or the statement object
  // - maybe we should only use [2] of the error info object,
  //   because it seems to contain the interesting thing:
  //   as per specs, [2] is the driver-specific error message,
  //   while [0] and [1] are error codes
  function OnDbErr($machine=false)
  {
		$einfo = $this->db->errorInfo();
    // why are we checking on zero, which means no error?
    // maybe we should throw always if transaction level is >0,
    // and not check on zero
		if ($einfo[0]=='00000' && $this->transactionLevel>0)
		{
			throw new DbError($einfo[2]);
		}
		if ($this->errorAsException)
		{
			if ($machine)
			{
				$errt = var_export( $this->machinedb->errorInfo() , true);
			}
			else
			{
				$errt = var_export( $this->db->errorInfo() , true );
			}
			throw new Exception($errt);
		}
    // if we dont throw we log and exit the script
    try
    {
      throw new Exception();
    } catch (Exception $e)
    {
      echo '<pre>';
      echo $e->getTraceAsString() . "\r\n";
    }
		if ($machine)
		{
			var_dump( $this->machinedb->errorInfo() );
		}
		else
		{
			var_dump( $this->db->errorInfo() );
		}
    exit;
  }

	function GetSortedGroupList()
	{
    $result = array();
    foreach ($this->db->query('SELECT * FROM Groups ORDER BY UPPER(name)') as $row)
    {
      $result[] = $row;
    }
    return $result;
	}

  function GetGroupList()
  {
    $result = array();
    foreach ($this->db->query('SELECT * FROM Groups ORDER BY id') as $row)
    {
      $result[] = $row;
    }
    return $result;
  }

	function GetNonTenantGroupList()
	{
		$result = array();
		foreach ($this->db->query('SELECT * FROM Groups WHERE (is_tenant=0 OR is_tenant IS NULL) AND tenantid IS NULL') as $row)
		{
			$result[] = $row;
		}
	return $result;
	}

  function GetTenantList()
  {
    $result = array();
    foreach ($this->db->query('SELECT * FROM Groups WHERE is_tenant=1 ORDER BY name ') as $row)
    {
      //var_dump($row);
      $result[] = $row;
    }
    return $result;
  }

  function IsTenant($id)
  {
    $stmt = $this->db->prepare('SELECT EXISTS(SELECT 1 FROM GROUPS WHERE is_tenant=1 AND tenantid=?)');

    if (!$stmt)
    {
      return false;
    }

    $stmt->bindValue(1,$id);
    if(!$stmt->execute())
    {
      return false;
    }
    
    while($row = $stmt->fetch())
    {
      return $row['exists'];
    }

    return false;
  }

	// argument 'field' must be set in code, in by user!
	// is applies to all users of tenant
	function SetTenantUserOption($groupid, $field, $value)
	{
    $sql = 'UPDATE USERS SET '.$field.'=? WHERE tenantid=?';
    $stmt = $this->db->prepare($sql);
		if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$value);
    $stmt->bindValue(2,$groupid);
    $stmt->execute();
	}
	
	function DeleteTenantUsers($tenantid)
	{
    // get tenant users
		$stmt = $this->db->prepare('SELECT id FROM Users WHERE tenantid=?');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    $stmt->execute();
		$users = array();
    while ($row = $stmt->fetch())
    {
      $users[] = $row['id'];
    }
		foreach ($users as $userid)
		{
			$this->DeleteUserAccount( $userid );
		}
	}
	
	function DeleteTenantSubGroups($tenantid)
	{
		// get sub-groups (so not itself)
		$stmt = $this->db->prepare('SELECT id FROM Groups WHERE tenantid=? AND id<>?');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    $stmt->bindValue(2,$tenantid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$groups = array();
    while ($row = $stmt->fetch())
    {
      $groups[] = $row['id'];
    }
		foreach ($groups as $id)
		{
			$this->DeleteGroup( $id );
		}		
	}

	function UnassignAllTenantCalls()
	{
		$stmt = $this->db->prepare('UPDATE Calls SET tenantid=NULL WHERE tenantid IS NOT NULL');
		if (!$stmt) $this->OnDbErr();
    $stmt->execute();
	}
	
	function UnassignTenantCalls($tenantid)
	{
		$stmt = $this->db->prepare('UPDATE Calls SET tenantid=NULL WHERE tenantid=?');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }

		$stmt = $this->db->prepare('UPDATE ActiveCalls SET tenantid=NULL WHERE tenantid=?');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}

	function DeleteTenantCalls($tenantid)
  {
		$stmt = $this->db->prepare('DELETE FROM Calls WHERE tenantid=?');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }

	function DeleteTenantCallsArchive($tenantid)
  {
		$stmt = $this->db->prepare('DELETE FROM CallsArchive WHERE tenantid=?');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }
	
	function PerformOnTenantCalls($tenantid,$PerformF,$cfg)
	{
    $stmt = $this->db->prepare('SELECT * FROM Calls WHERE tenantid=:1');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    $stmt->execute();
    while ($row = $stmt->fetch())
    {
      $PerformF( $row , $cfg );
    }
	}

  function PerformOnTenantCallsArchive($tenantid,$PerformF,$cfg)
	{
    $stmt = $this->db->prepare('SELECT * FROM CallsArchive WHERE tenantid=:1');
		if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$tenantid);
    $stmt->execute();
    while ($row = $stmt->fetch())
    {
      $PerformF( $row , $cfg );
    }
	}
  
  function UpdateTenantPhones($groupid){
    $groupinfo = $this->GetGroupInfo($groupid);
    if ($groupinfo===false) return false;
    if(int_ensure_defined($groupinfo['is_tenant']) !== 1){
      if(isset($groupinfo['tenantid']))
      {
        $groupid = $groupinfo['tenantid'];
        $groupinfo = $this->GetGroupInfo($groupid);
        if(int_ensure_defined($groupinfo['is_tenant']) !== 1){
          return false;
        }
      }
      else
      {
        return false;
      }
    }
    $numbers = $this->GetTenantNumbers($groupinfo['accid']); 
    $key = 'TN' . $groupid . '_' . 'Numbers';
		$configs_tn_new = array();
    $configs_tn_new[$key] = implode(',',$numbers);
		if ($this->GetConfigValue($key) == $configs_tn_new[$key])
		{
			// already the same, no change
			return false;
		}
		$this->SetConfigsPart($configs_tn_new, 'tn');
		return true;
  }

  function GetAllTenantSettings($id)
  {
    $key = 'TN'. $id . '_%';
    $stmt = $this->db->prepare('SELECT param, value FROM configs WHERE param LIKE :1');
    $stmt->bindValue(1,$key);
    $stmt->execute();
    $result = array();
    while($row = $stmt->fetch())
    {
      $result[$row['param']] = $row['value'];
    }
    return $result;
  }
	
  function AddGroup(&$groupid,$data,$ReturnAccId = false)
  {
    if (($this->actuinfo['permissions']&(PM_CONFIG|PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    // check if we are moving this group to a new supergroup,
    // that has too much permissions
    if (($this->actuinfo['permissions']&(PM_CONFIG))==0
    && isset($data['super']))
    {
      if ( $this->IsExtraAdminPm( $this->GetGroupPm($data['super'],true) ) )
      {
        $this->lasterr = 'pm-denied';
        return false;
      }
    }
    $stmt = $this->db->prepare('INSERT INTO Accessors(type) VALUES(1)');
    $stmt->execute();
    $accid = $this->db->lastInsertId('Accessors_id_seq');
    $sql = 'INSERT INTO Groups (accid,'.implode(', ',array_keys($data)).') '
    . ' VALUES (?,'.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    } else {
      $ix = 1;
      $stmt->bindValue($ix,$accid);
      foreach($data as $value)
      {
        $ix ++;
        $stmt->bindValue($ix,$value);
      }
      $res = $stmt->execute();
      $groupid = $this->db->lastInsertId('Groups_id_seq');
      if($ReturnAccId)
      {
        $groupid = $accid;
      }
			$this->allgroups[$accid] = $data['super'];
			$this->CalcTenantUsers();
      return true;
    }
  }

  function DeleteGroup($groupid)
  {
    if (($this->actuinfo['permissions']&(PM_CONFIG|PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    if (defined('DEMO') && ($groupid==1 || $groupid==2))
    {
      $this->lasterr = 'demo-protectedgroup';
      return false;
    }
    // get info including accessor-id
    $groupinfo = $this->GetGroupInfo($groupid);
    // delete references
    $stmt = $this->db->prepare('DELETE FROM AccAcc WHERE accid=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    }
    $stmt->bindValue(1,$groupinfo['accid']);
    $stmt->execute();
    // delete references
    $stmt = $this->db->prepare('DELETE FROM GroupMembers WHERE grpid=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    }
    $stmt->bindValue(1,$groupinfo['accid']);
    $stmt->execute();
    // delete accessor
    $stmt = $this->db->prepare('DELETE FROM Accessors WHERE id=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    }
    $stmt->bindValue(1,$groupinfo['accid']);
    $stmt->execute();
    // delete group
    $stmt = $this->db->prepare('DELETE FROM Groups WHERE id=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    }
    $stmt->bindValue(1,$groupid);
    $stmt->execute();
		$this->CalcTenantUsers();
    return true;
  }

  // returns true if any tenant is a subgroup of another tenant, which is invalid
  // if so, fills in supert and subt with the group ids of the conflicting tenants
  function TenantInsideTenant(& $ref_supert,& $ref_subt)
  {
		$tenants = $this->GetTenantList();
		foreach ($tenants as $group)
		{
      $superaccid = $group['super'];
      $supergroup = $this->GetGroupInfoOnAccid( $superaccid );
      if ($supergroup['tenantid']!=NULL)
      {
        $ref_supert = $supergroup['tenantid'];
        $ref_subt = $group['id'];
        return true;
      }
    }
    return false;
  }
  
  function MayEditGroup($groupid)
  {
    if (($this->actuinfo['permissions']&(PM_CONFIG|PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    // check if this group has too much permission that we may not edit it
    if (($this->actuinfo['permissions']&(PM_CONFIG))==0)
    {
      $info = $this->GetGroupInfo($groupid);
      if ( $this->IsExtraAdminPm( $this->GetGroupPm($info['accid']) ) )
      {
        $this->lasterr = 'pm-denied';
        return false;
      }
    }
    return true;
  }
  
  function EditGroup($groupid,$data)
  {
    if ($this->MayEditGroup($groupid)==false)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    // check if we are moving this group to a new supergroup,
    // that has too much permissions
    if (($this->actuinfo['permissions']&(PM_CONFIG))==0
    && isset($data['super']))
    {
      if ( $this->IsExtraAdminPm( $this->GetGroupPm($data['super'],true) ) )
      {
        $this->lasterr = 'pm-denied';
        return false;
      }
    }
    if (defined('DEMO') && ($groupid==1 || $groupid==2))
    {
      $this->lasterr = 'demo-protectedgroup';
      return false;
    }
		$this->EditGroupPure($groupid,$data);
		$this->CalcTenantUsers();
    return true;
	}
	
	function EditGroupPure($groupid,$data)
	{
    $sql = 'UPDATE groups SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$groupid);
    $res = $stmt->execute();
    return true;
  }

  function GetAccessorsList()
  {
    $result = array();
    foreach ($this->db->query('SELECT accid,name,usr FROM Users ORDER BY upper(name)') as $row)
    {
      $row['type'] = 0;
      $result[] = $row;
    }
    foreach ($this->db->query('SELECT accid,name FROM Groups ORDER BY upper(name)') as $row)
    {
      $row['type'] = 1;
      $result[] = $row;
    }
    return $result;
  }

  // groups array contain accids
  function SetGroupsOfWhichMember($useraccid,$groups)
  {
    $theuser = $this->GetUserInfoOnAccId($useraccid);
    if ($this->MayEditUserAccount($theuser['id'],$useraccid)==false)
    {
      return false;
    }
    return $this->SetGroupsOfWhichMemberLC($useraccid,$groups);
  }

  // Calculate group permissions recursive
  // if sub is false, for the group itself
  // if sub is true, for a subgroup
  function GetGroupPm($groupaccid,$sub=false)
  {
    $groups = array();
    $origroupaccid = $groupaccid;
    while ($groupaccid!=0)
    {
      $groups[] = $groupaccid;
      $info = $this->GetGroupInfoOnAccid($groupaccid);
      $groupaccid = $info['super'];
    }
    
    $items = $this->GetAccAccPms($groups);
    $pms = 0;
    foreach ($items as $item)
    {
      $apply = false;
      if ($item['accid']==$origroupaccid && $sub==false) $apply = true;
      else if ($item['applytype']==1) $apply = true;
      if ($apply) $pms |= $item['epermissions'];
    }
    return $pms;
  }
  
  function SetGroupsOfWhichMemberLC($useraccid,$groups)
  {
    $allowed = false;
    $aupm = $this->actuinfo['permissions'];
    if ($aupm&PM_CONFIG) $allowed = true;
    if ($aupm&(PM_T2_ADMIN|PM_T3_ADMIN|PM_USERS) )
    {
      // May not move to a group to with higher access
      $oldGroups = $this->GetGroupsOfWhichMember( $useraccid );
      $allowed = true;
      foreach($groups as $value)
      {
        $isNew = true;
        foreach ($oldGroups as $oldGroup)
        {
          if ($oldGroup==$value)
          {
            $isNew = false;
          }
        }
        if ($isNew==false) continue;
        // it is a new group
        $groupPm = $this->GetGroupPm($value);
        if ($this->IsExtraAdminPm($groupPm))
        {
          // that has extra permissions
          $allowed = false;
        }
      }
    }
    if ( $allowed==false // not already allowed for other reasons
      && ($this->anyaccpms&PM_USERS)!=0 // we have access to some user accounts
    // Next line is disabled, because it is already checked for edit,
    // and for new accounts it is not true yet,
    // because not yet member of group to which it would have access.
    // not: && ($this->GetAccPm($useraccid)&PM_USERS)!=0
    )
    {
      // May not move to a group to which no
      // edit access.
      $oldGroups = $this->GetGroupsOfWhichMember( $useraccid );
      $allowed = true;
      foreach($groups as $value)
      {
        $isNew = true;
        foreach ($oldGroups as $oldGroup)
        {
          if ($oldGroup==$value)
          {
            $isNew = false;
          }
        }
        if ($isNew==false) continue;
        if (($this->GetAccPm($value)&PM_USERS)==0)
        {
          $allowed = false;
        }
      }
      if (count($groups)<1 && count($oldGroups)>=1)
      {
        // user must belong to one group at least
        $allowed = false;
      }
    }
    if ($aupm&PM_SUBADMIN && $allowed==false)
    {
      // a sub-admin
      // groups must be same as group of subadmin itself
      $groups = array();
      $arr = $this->GetGroupsOfWhichMember( $this->actuinfo['accid'] );
      if (count($arr)>0)
      {
        $groups = array( $arr[0] );
      }
      // we have modified the group list to ensure it is allowed
      $allowed = true;
    }

    if ($allowed==false)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }

    $stmt = $this->db->prepare('DELETE FROM GroupMembers WHERE accid=:1');
    $stmt->bindValue(1,$useraccid);
    $stmt->execute();
    $stmt = $this->db->prepare('INSERT INTO GroupMembers '
    . ' (grpid,accid) '
    . ' VALUES '
    . ' (:1    ,:2  ) '
    );
    if ($stmt==FALSE)
    {
      $this->OnDbErr();
    }
    foreach($groups as $value)
    {
      $stmt->bindValue(1,$value);
      $stmt->bindValue(2,$useraccid);
      $stmt->execute();

    }
		$this->CalcTenantUsers();
    return true;
  }

  function GetGroupsOfWhichMember($useraccid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT grpid FROM GroupMembers WHERE accid=:1');
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$useraccid);
    $stmt->execute();
    while ($row = $stmt->fetch())
    {
      $grpid = $row['grpid'];
      if ($grpid === null) continue;
      $result[] = $grpid;
    }
    return $result;
  }

	function GetSortedGroupMembers($groupaccid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT Users.accid FROM GroupMembers'
		. ' INNER JOIN Users ON Groupmembers.accid=Users.accid'
		. ' WHERE grpid=:1'
		. ' ORDER BY upper(name)'
		);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$groupaccid);
    $stmt->execute();
    while ($row = $stmt->fetch())
    {
      $result[] = $row[0];
    }
    return $result;
  }
	
  function GetGroupMembers($groupaccid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT accid FROM GroupMembers WHERE grpid=:1');
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$groupaccid);
    $stmt->execute();
    while ($row = $stmt->fetch())
    {
      $result[] = $row['accid'];
    }
    return $result;
  }

  /*
   // typegrp seems not used anymore, is always 0
   function RecalcAllGroup()
   {
   if (($this->actuinfo['permissions']&PM_CONFIG)==0)
   {
   $this->lasterr = 'pm-denied';
   return false;
   }

   // Get AccId of the All group
   $groupaccid = $this->db->query('SELECT accid FROM Groups WHERE typegrp=1')->fetchColumn(0);

   $users = $this->GetUserList();
   $list = array();
   foreach ($users as $user)
   {
   $list[] = $user['accid'];
   }

   $this->SetGroupMembers($groupaccid,$list);
   }
   */

  function SetGroupMembers($groupaccid,$list)
  {
    if (($this->actuinfo['permissions']&(PM_CONFIG|PM_USERS|PM_T2_ADMIN|PM_T3_ADMIN))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    $stmt = $this->db->prepare('DELETE FROM GroupMembers WHERE grpid=:1');
    $stmt->bindValue(1,$groupaccid);
    $stmt->execute();
    $stmt = $this->db->prepare('INSERT INTO GroupMembers '
    . ' (grpid,accid) '
    . ' VALUES '
    . ' (:1    ,:2  ) '
    );
    if ($stmt==FALSE)
    {
      $this->OnDbErr();
    }
    foreach($list as $value)
    {
      //echo $obj['contactid'].' '.$obj['pm'];
      $stmt->bindValue(1,$groupaccid);
      $stmt->bindValue(2,$value);
      $stmt->execute();

    }
		$this->CalcTenantUsers();
    return true;
  }

  function GetAccIdList($accid)
  {
    $list = array();
    $list[] = $accid;
    $newlist = $list;
    $increased = true;
    while ($increased)
    {
      $copy_newlist = $newlist;
      $newlist = array();
      $increased = false;
      // check if new items are part of groups
      foreach ($copy_newlist as $item)
      {
        $stmt = $this->db->prepare('SELECT grpid FROM GroupMembers WHERE accid=:1');
        if (!$stmt)
        {
          $this->OnDbErr();
        }
        $stmt->bindValue(1,$accid);
        $stmt->execute();
        $increased = false;
        while ($row = $stmt->fetch())
        {
          $newaccid = $row['grpid'];
          if ( ! in_array( $newaccid, $list ) )
          {
            $list[] = $newaccid;
            $newlist[] = $newaccid;
            $increased = true;
          }
        }
      } // end for new items
    }
    return $list;
  }

  // not used anymore it seems
  function GetGroupPermissions($list)
  {
    $pm = 0;
    $stmt = $this->db->prepare('SELECT permissions FROM Groups WHERE accid=:1');
    foreach ($list as $accid)
    {
      $result = array();
      $stmt->bindValue(1,$accid);
      $stmt->execute();
      $result = $stmt->fetch();
      if ($result==null)
      {
        continue;
      }
      $pm |= $result['permissions'];
    }
    return $pm;
  }

  function AddLicNotRec($startdate)
  {
    $sqlstmt = 'INSERT INTO LicNotRec(startdate) VALUES(?) ';
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$startdate);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
  }

  function MayViewLicNotRec()
  {
    if (($this->actuinfo['permissions']&PM_CONFIG)==0)
    {
      return false;
    }
    return true;
  }
	
	function EmptyLicNotRec()
	{
		if (($this->actuinfo['permissions']&PM_CONFIG)==0)
		{
			return;
		}
		
		$sqlstmt ='DELETE FROM LicNotRec';
		$stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
		if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
	}

  function GetLicNotRecStats($cfg,$ApplyStatF,$search)
  {
    // Check if and which filtering we need to apply
    $filter = ' WHERE ' . $this->GetTrue() . ' ';
    // Check permission
    if ($this->MayViewLicNotRec()==false)
    {
      return array();
    }
    // searching
    foreach ($search as $searchcol=>$searchvalue)
    {
      switch($searchcol)
      {
        case 'fd':
          $filter .= ' AND startdate >= ? ';
          break;
        case 'ld':
          $filter .= ' AND startdate <= ? ';
          break;
      }
    }

    $sqlstmt = 'SELECT startdate '
    . ' FROM LicNotRec '
    . $filter
    . ' ORDER BY startdate ';
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    // searching params binding
    $ix = 0;
    foreach ($search as $searchcol=>$searchvalue)
    {
      $ix ++;
      switch ($searchcol)
      {
        case 'fd': case 'ld':
          $stmt->bindValue($ix,$searchvalue);
          break;
      }
    }
    $stmt->execute();

    $result = array();
    while ($row = $stmt->fetch())
    {
      $ApplyStatF($cfg,$result,$row);
    }
    return $result;
  }

	// Calculate the users and user groups per tenant,
	// and store in database.
	function CalcTenantUsers()
	{
		// users is a map: user-accid => tenant-group-id
		$users = array();
		// get tenant groups
		$grouptenants = array();
		$tenants = $this->GetTenantList();
    $this->DetermineAllGroups(true);
		foreach ($tenants as $group)
		{
			//echo $group['name'] . ' ';
			// get subgroups
			$groups = array( $group['accid'] );
			$this->FindSubGroups( $groups, $group['accid'] );
			//var_dump( $subgroups );
			// for this and subgroup
			foreach ($groups as $groupaccid)
			{
				$grouptenants[ $groupaccid ] = $group['id'];
				$groupinfo = $this->GetGroupInfoOnAccid( $groupaccid );
				// find and register the members
				$members = $this->GetGroupMembers($groupaccid);
				foreach ($members as $member)
				{
					$users[ $member ] = $group['id'];
				}
			}
		}
		// update Group table
		$grouplist = $this->GetGroupList();
    /*echo "</br></br>";
    var_dump($grouptenants);
    echo "</br></br>";
    var_dump($grouplist);*/
		foreach ($grouplist as $group)
		{
			$old_tenantid = $group['tenantid'];
			$new_tenantid = null;
			if (isset($grouptenants[$group['accid']]))
			{
				$new_tenantid = $grouptenants[$group['accid']];
			}
			if ($old_tenantid !== $new_tenantid)
			{
				$this->EditGroupPure( $group['id']
					, array( 'tenantid' => $new_tenantid)
					);
			}
		}
		// update Users table.
		// we only update what is changed
    // not: $this->StartTransaction();
		// to prevent nested transactions
    $stmt = $this->db->prepare('UPDATE Users SET tenantid = ? WHERE accid = ?');
    if ( ! $stmt)
    {
      $this->OnDbErr();
    }
		// get current user list (with tenant ids)
		$allusers = $this->GetUserList();
		foreach ($allusers as $auser)
		{
			$old_tenantid = $auser['tenantid'];
			//var_dump( $old_tenantid );
			$new_tenantid = null;
			$accid = $auser['accid'];
			if (isset($users[$accid]))
			{
				$new_tenantid = $users[$accid];
			}
			// check if we need to change this record
			if ($old_tenantid !== $new_tenantid)
			{
				//echo ' O: ' . $old_tenantid . ' <> N: ' . $new_tenantid;
				$stmt->bindValue(1,$new_tenantid);
				$stmt->bindValue(2,$accid);
				if ($stmt->execute()==false)
				{
					$this->OnDbErr();
				}
				$this->ApplyTenantSettingsToUser($auser['id'],$new_tenantid);
			}
		}
    // not: $this->CommitTransaction();
	}
	
	function GetTenantMandates( $tenantid )
	{
		$data = array();
		$tn_prefix = 'TN' . $tenantid . '_';
		if (((int)$this->GetCachedConfigValue($tn_prefix . 'RecOnDmdAvail'))==0)
		{
			$data['recondmd'] = '0';
		}
		if (((int)$this->GetCachedConfigValue($tn_prefix . 'SilOnDemandAvail'))==0)
		{
			$data['silondmd'] = '0';
		}
		return $data;
	}

	function ApplyTenantSettings( $tenantid )
	{
		$data = $this->GetTenantMandates( $tenantid );
		foreach ($data as $key => $value)
		{
			$this->SetTenantUserOption( $tenantid , $key , $value );
		}
    
    $newconfigs = array();
    $numbers = $this->GetRecOnDmdUserNumbers();
    $newconfigs['RecOnDmdList'] = implode(",",$numbers);
    
    $numbers = $this->GetSilOnDmdUserNumbers();
    $newconfigs['SilOnDmdList'] = implode(",",$numbers);
    $this->SetConfigsPart($newconfigs,'o');
	}
	
	function ApplyTenantSettingsToUser($userid, $tenantid)
	{
		$data = $this->GetTenantMandates( $tenantid );
		$this->EditUserAccountLC($userid,$data);
	}

	
	function GetSimCallsForHour($hour)
	{
    $stmt = $this->db->prepare('SELECT numcalls FROM MaxCalls WHERE hour=:1');
    $stmt->bindValue(1,$hour);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result ? $result['numcalls'] : 0;
  }
	
	function GetMaxSimCallsStats($cfg,$ApplyStatF,$search)
	{
    //Check if and which filtering we need to apply
    $filter = ' WHERE ' . $this->GetTrue() . ' ';
    //Check permission
    if ($this->MayViewLicNotRec()==false)
    {
      return array();
    }
    //Searching
    foreach ($search as $searchcol=>$searchvalue)
    {
      switch($searchcol)
      {
        case 'fd':
					$starthour = $searchvalue;
          break;
        case 'ld':
					$endhour = $searchvalue;
          break;
      }
    }
		$filter .= ' AND hour >= :1 AND hour <= :2 ';
		
		//Determine start and end time
		if (!isset($search['fd']) && !isset($search['ld']))
		{
			$endhour = time();
			$endhour = intval($endhour/3600);
			$starthour = $endhour - 30*24;	//Default 30 days!
		}
		else
		{
			if (!isset($search['fd']))
			{
				$endhour = intval($endhour/3600);
				$starthour = $endhour - 30*24;	//Default 30 days!
			} else if (!isset($search['ld']))
			{
				$starthour = intval($starthour/3600);
				$endhour = $starthour + 30*24;	//Default 30 days!
			} else
			{
				$starthour = intval($starthour/3600);
				$endhour = intval($endhour/3600);
			}
		}

    $sqlstmt = 'SELECT numcalls, hour '
    . ' FROM MaxCalls '
    . $filter
    . ' ORDER BY hour ';
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
		$stmt->bindValue(1,$starthour);
		$stmt->bindValue(2,$endhour);
    $stmt->execute();
		
		$result = array();
		$newday = 0;
    while ($row = $stmt->fetch())
    {
			if($newday < intval($row['hour']/24))
			{
				$newday = intval($row['hour']/24);
				$result[$newday] = $row['numcalls'];
			}
			else
			{
				if ($result[$newday] < $row['numcalls'])
				{
					$result[$newday] = $row['numcalls'];
				}
			}
		}
		return $result;
	}

	function CallWalker(&$ref_cfg,$FunctionCB)
	{
    $sqlstmt = 'SELECT ' . implode(',',$ref_cfg['cols'])
		. ' FROM ';
    if ( isset($ref_cfg['archive']) && $ref_cfg['archive']==true)
    {
      $sqlstmt .= 'CallsArchive';
    } else {
      $sqlstmt .= 'Calls';
    }
    $sqlstmt .= ' WHERE 1=1';
		if (isset($ref_cfg['tenancy']) && $ref_cfg['tenancy']==true)
		{
			$sqlstmt .= ' AND tenantid IS NOT NULL';
		}
		if (isset($ref_cfg['isarchived']) && $ref_cfg['isarchived']==true)
		{
			$sqlstmt .= ' AND isarchived=1';
		}
		if (isset($ref_cfg['notenant']) && $ref_cfg['notenant']==true)
		{
			$sqlstmt .= ' AND tenantid IS NULL';
		}
		if (isset($ref_cfg['noagent']) && $ref_cfg['noagent']==true)
		{
			$sqlstmt .= ' AND agent IS NULL';
		}
		$sqlstmt .= ' ORDER BY startdate DESC';
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
    $result = array();
    while ($row = $stmt->fetch())
    {
      if ($FunctionCB($ref_cfg,$result,$row)==false)
      {
        // ready..
        break;
      }
    }
    return $result;
	}
	
  /*
   Returns a data array.
   Actual logic of filling of this data array is done in ApplyStatF.
   */
  function GetCallStats($cfg,$ApplyStatF,$search)
  {
    $sf = $this->SearchFilter($search,array());
    $filter = $sf['filter'];
    $tobind = $sf['tobind'];
    $sqlstmt = 'SELECT Calls.id,startdate,duration,waittime,'
    . ' remote,local,remote2,local2,direction,notes, '
    . ' localcontact,remotecontact,connectedcontact, '
    . ' cloc.name as lname,crem.name as rname,ccon.name as cname '
		. ' ,agent '
    . ' FROM Calls '
    . ' JOIN contacts AS cloc ON Calls.localcontact = cloc.id'
    . ' JOIN contacts AS crem ON Calls.remotecontact = crem.id'
    . ' LEFT JOIN contacts AS ccon ON Calls.connectedcontact = ccon.id'
    . $filter
    . ' ORDER BY startdate ';
    if (isset($cfg['sortdir']))
    {
      if ($cfg['sortdir']=='down') $sqlstmt .= 'DESC';
      else $sqlstmt .= 'ASC';
    } else
    {
      $sqlstmt .= 'ASC';
    }
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    // searching params binding
    $ix = 0;
    foreach ($tobind as $bind)
    {
      $ix++; $stmt->bindValue($ix,$bind);
    }
    $stmt->execute();

    $result = array();
    while ($row = $stmt->fetch())
    {
      if ($ApplyStatF($cfg,$result,$row)==false)
      {
        // ready..
        break;
      }
    }
    return $result;
  }

  function GetOlderRecordings($fromtime,$limit,$DeleteMode,$tenantid=FALSE,$backup=false,$MaxId=-1,$retention=false)
  {
    $sqlstmt = 'SELECT id,directory,filename,flags,isarchived,tenantid,screenfn FROM ';
    if ($backup) $sqlstmt .= 'CallsArchive';
    else $sqlstmt .= 'Calls';
    $sqlstmt .= ' WHERE 1=1';
    
    // keep meta
		if ($DeleteMode==1) $sqlstmt .= ' AND isarchived=1 ';
    // delete meta
    if ($DeleteMode==2) $sqlstmt .= ' ';
    // delete meta except for AgEv
    if ($DeleteMode==3) $sqlstmt .= ' AND (isarchived=1 OR NOT EXISTS '
        . '(select recordingid from CECalls where recordingid=calls.id))';
    // screen recordings
    if ($DeleteMode==4) $sqlstmt .= ' AND (flags & 2)=2 AND isarchived=1 ';
    // transcripts
    if ($DeleteMode==5) $sqlstmt .= ' AND EXISTS '
        . '(select call_id from Transcriptions where call_id=calls.id)';
		
		if ($tenantid!==FALSE)
		{
			$sqlstmt .= ' AND tenantid=' . (int) $tenantid;
		}
		if ($fromtime!==false)
		{
			$sqlstmt .= ' AND startdate<:1';
		}
    if ($MaxId!=-1)
    {
      $sqlstmt .= ' AND id <= ' . (int) $MaxId;
    }
    if ($retention!==false)
    {
      if ($retention===null) $sqlstmt .= ' AND retention IS NULL';
      else $sqlstmt .= ' AND retention = ' . (int) $retention;
    }
		$sqlstmt .= ' ORDER BY startdate ASC LIMIT ' . (int) $limit;
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
		if ($fromtime!==false)
		{
			$stmt->bindValue(1,$fromtime);
		}
    $stmt->execute();
    $result = array();
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
  }

  function GetToRestoreList($limit)
  {
    // compared to backup, we start with the *newest* in the database
    // also, we use another flag (8)
    $sqlstmt = 'SELECT id,directory,filename,screenfn,flags,isarchived,tenantid from calls where (processingstate&8)=8 order by startdate DESC LIMIT '.$limit;
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
    $result = array();
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
  }

	// first,last: ID in Calls table
	// used by export in frqtask
	function SetProcessingState($first,$last,$flag,$cfg=array())
	{
    $sqlstmt = "UPDATE Calls SET processingstate=processingstate|$flag "
    . " WHERE id >= $first AND id <= $last ";
    if (isset($cfg['filter']))
    {
			$contacts_filter = $this->GetAccidPhoneContacts( $cfg['filter'] , false );
      $sqlstmt .= $this->GetEnhContactsFilter($contacts_filter);
    }
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
		return $stmt->rowCount();
	}
	
  // IdOffset: this is the old style offset as was used by export
  // The offset can be beyond records that need to be handled,
  // but these would then be marked with a flag.
  // cfg: which flags indicate things to do
  // Returns: the lowest id of a record that (surely) was already handled
  function GetIdHandled($IdOffset,$cfg)
  {
    $IdOffset = (int) $IdOffset;
    $sqlstmt = 'SELECT id FROM calls WHERE 1=1 ';
    if (isset($cfg['AndFlag']))
    {
      $AndFlag = (int) $cfg['AndFlag'];
      $EqFlag = (int) $cfg['EqFlag'];
      $sqlstmt .= " AND (processingstate & $AndFlag) = $EqFlag ";
    }
    // find the lowest ID that we need to handle
    $sqlstmt .= " ORDER BY id ASC LIMIT 1 ";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===false)
    {
      // no record found, there is nothing to handle from a previous round,
      // so we can use the offset
      return $IdOffset;
    } else
    {
      // we have the first id to be handled
      // so one earlier is already handled (-1)
      $IdHandled = $row['id'] - 1;
      // if the offset is lower, than take the offset
      if ($IdOffset<$IdHandled) $IdHandled = $IdOffset;
      return $IdHandled;
    }
  }

  function GetHandlingFilterSql($cfg,&$tobind)
  {
    $sqlstmt = ' AND isarchived=1 ';
    if (isset($cfg['from_ts']) && $cfg['from_ts']!=0)
    {
      $sqlstmt .= ' AND startdate>= ' . (int) $cfg['from_ts'];
    }
    if (isset($cfg['to_ts']) && $cfg['to_ts']!=0)
    {
      $sqlstmt .= ' AND startdate< ' . (int) $cfg['to_ts'];
    }
    if (isset($cfg['tenant']))
    {
      $tenant = (int) $cfg['tenant'];
      $sqlstmt .= " AND tenantid = $tenant ";
    }
    if (isset($cfg['Flag']))
    {
      $Flag = (int) $cfg['Flag'];
      $sqlstmt .= " AND (processingstate & $Flag) = $Flag ";
    }
    if (isset($cfg['accidfilter']))
    {
      $accid = (int) $cfg['accidfilter'];
      $tenantid = null;
      $is_tenant = false;
      $group = $this->GetGroupInfoOnAccid( $accid );
      if ($group !== false)
      {
        if ($group['is_tenant']==1)
        {
          $is_tenant = true;
          $tenantid = $group['id'];
        }
      }
      if ($is_tenant)
      {
        $sqlstmt .= " AND tenantid = $tenantid ";
      } else
      {
        $contacts_filter = $this->GetAccidPhoneContacts( $cfg['accidfilter'] , false );
        $sqlstmt .= $this->GetEnhContactsFilter($contacts_filter);
      }
    }
    if (isset($cfg['telnum']))
    {
            $filter = '';
            $searchvalue = $cfg['telnum'];
            
            $searchitems = explode(',',$searchvalue);
            $first = true;
            // negatives are handled separately
            // otherwise the result would be counter-intuitive
            // none of the negated phone numbers should be present (so AND NOT)
            // but only one of the non-negated phone number is enough (so OR)
            $nots = array();
            foreach ($searchitems as $searchitem)
            {
              if (substr($searchitem,0,1)=='!')
              {
                $searchitem = substr($searchitem,1);
                $nots[] = $searchitem;
                continue;
              }
              if (!$first) $filter .= ' OR ';
              else $filter .= ' AND (';
              $first = false;
              $searchitem = $this->TextLikeSearch($searchitem,true);
              $filter .= ' local '.$this->GetILike()." ?  ".$this->GetEscape()." OR remote ".$this->GetILike()." ?  ".$this->GetEscape()." OR connected ".$this->GetILike()." ?  ".$this->GetEscape()."";
              $tobind[] = $searchitem;
              $tobind[] = $searchitem;
              $tobind[] = $searchitem;
            }
            if ($first==false) $filter .= ') ';
            if (count($nots)>0)
            {
              $filter .= ' AND ';
              $first = true;
              foreach ($nots as $searchitem)
              {
                if (!$first) $filter .= ' AND ';
                $first = false;
                $searchitem = $this->TextLikeSearch($searchitem,true);
                $filter .= ' NOT (local '.$this->GetILike()." ?  ".$this->GetEscape()." OR remote ".$this->GetILike()." ?  ".$this->GetEscape()." OR connected ".$this->GetILike()." ?  ".$this->GetEscape().")";
                $tobind[] = $searchitem;
                $tobind[] = $searchitem;
                $tobind[] = $searchitem;
              }
            }
            $sqlstmt .= $filter;
      
    }
    if (isset($cfg['fdur']))
    {
            $sqlstmt .= ' AND duration >= ? ';
            $tobind[] = $cfg['fdur'];
    }
    if (isset($cfg['ldur']))
    {
            $sqlstmt .= ' AND duration <= ? ';
            $tobind[] = $cfg['ldur'];
    }
    return $sqlstmt;
  }
  
  // IdHandled: ID of already handled
  // Limit: max to return
  // cfg: extra filtered, can be empty array
  function GetToHandleList($IdHandled,$Limit,$cfg)
  {
    $IdHandled = (int) $IdHandled;
    $Limit = (int) $Limit;
    $tobind = array();
    $sqlstmt = 'SELECT id,directory,filename,screenfn,flags,startdate,duration,tenantid FROM calls '
    . " WHERE id > $IdHandled ";
    $sqlstmt .= $this->GetHandlingFilterSql($cfg,$tobind);
    if (isset($cfg['IdMax']))
    {
      $IdMax = (int) $cfg['IdMax'];
      $sqlstmt .= " AND id <= $IdMax ";
    }
    if (isset($cfg['FlagHandled']))
    {
      // skip records already handled
      $Flag = (int) $cfg['FlagHandled'];
      $sqlstmt .= " AND (processingstate & $Flag) = 0 ";
    }
    $sqlstmt .= " ORDER BY id ASC LIMIT $Limit ";
    // echo $sqlstmt . "\n";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt) $this->OnDbErr();
    $ix = 0;
    foreach ($tobind as $bind)
    {
      $ix++; $stmt->bindValue($ix,$bind);
    }
    if (!$stmt->execute()) $this->OnDbErr();
    $result = array();
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
  }
  
  function GetToBackupList($limit,$FlagToBackup,$FlagHandled)
  {
    // this check both flag 1 and flag 2 (hence 3)
    $sqlstmt = 'SELECT id,directory,filename,screenfn,flags,startdate,duration,tenantid from calls where (processingstate&'.($FlagToBackup+$FlagHandled).')='.$FlagToBackup.' order by startdate ASC LIMIT '.$limit;
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
    $result = array();
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
  }

  function GetSingleValue($sql)
  {
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    if (!$stmt->execute()) $this->OnDbErr();
    $row = $stmt->fetch();
    if ($row === false) return NULL;
    return $row[0];
  }
  
  function GetBackupFrame($FlagToBackup,$FlagHandled,$cfg)
  {
    $IdSeen = 0;
    $IdHandled = 0;
    $IdLatest = $this->GetIdOfLatestCall();
    $tobind = array(); // not actually used
    
    $FirstNotHandled = $this->GetSingleValue("SELECT id FROM Calls "
    . ' WHERE (processingstate&'.($FlagHandled).')=0 '
    . $this->GetHandlingFilterSql($cfg,$tobind)
    . " ORDER BY id ASC LIMIT 1");
    if ($FirstNotHandled!==null) $IdHandled = $FirstNotHandled - 1;
    else $IdHandled = $IdLatest; // everything handled
    
    $FirstNotSeen = $this->GetSingleValue("SELECT id FROM Calls "
    . ' WHERE (processingstate&'.($FlagToBackup).')=0 '
    . $this->GetHandlingFilterSql($cfg,$tobind)
    . " ORDER BY id ASC LIMIT 1");
    if ($FirstNotSeen!==null) $IdSeen = $FirstNotSeen - 1;
    else $IdSeen = $IdLatest; // everything seen
    
    return array
    ( 'IdSeen' => $IdSeen
    , 'IdHandled' => $IdHandled
    );
  }
  
  function UpdateBackupRange($IdSeen,$cfg)
  {
    $IdSeen = (int) $IdSeen;
    $Cnt = 0;
    $sqlstmt = "SELECT count(id),max(id) FROM Calls "
    . " WHERE id>$IdSeen ";
    $tobind = array(); // not actually used
    $sqlstmt .= $this->GetHandlingFilterSql($cfg,$tobind);
    if (isset($cfg['FlagToBackup']))
    {
      // skip records already handled
      $Flag = (int) $cfg['FlagToBackup'];
      $sqlstmt .= " AND (processingstate & $Flag) = 0 ";
    }
    // echo $sqlstmt . "\n";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt) $this->OnDbErr();
    if (!$stmt->execute()) $this->OnDbErr();
    $row = $stmt->fetch();
    if ($row[1] !== NULL)
    {
      // if some rows found
      $Cnt = $row[0];
      $IdSeen = $row[1];
    }
    $result = array();
    $result['Cnt'] = $Cnt;
    $result['IdSeen'] = $IdSeen;
    return $result;
  }
  
  function SetBackupRange($from,$to)
  {
    $flag = 1;
    $sqlstmt = "UPDATE Calls SET processingstate=processingstate|$flag "
    . " WHERE isarchived=1 AND (processingstate & $flag)=0";
    if ($from!=0)
    {
      $sqlstmt .= ' AND startdate>= ' . $from;
    }
    if ($to!=0)
    {
      $sqlstmt .= ' AND startdate< ' . $to;
    }
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
    return $stmt->rowCount();
  }

  function GetInArchiveList($idlist)
  {
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "SELECT id FROM CallsArchive WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
    $ret = array();
    while ($row = $stmt->fetch())
    {
      $ret[ $row['id'] ] = true;
    }
    return $ret;
  }

  function DeleteCallsArchiveForIDs($idlist)
  {
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "DELETE FROM CallsArchive WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }
  
  function DeleteCallsForIDs($idlist)
  {
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "DELETE FROM Calls WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }

  function DeleteActiveCallsForIDs($idlist)
  {
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "DELETE FROM ActiveCalls WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }
  
  function SetActiveProcessingFlag($remoteid,$flag)
  {
    $sqlstmt = "UPDATE ActiveCalls SET processingstate=processingstate|$flag WHERE remoteid=:1";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$remoteid);
    $stmt->execute();
  }

	function IntifyArray($arr)
	{
		$ret = array();
		foreach ($arr as $key=>$data)
		{
			$ret[$key] = (int) $data;
		}
		return $ret;
	}
	
  function SetIsArchivedForIDs($idlist,$value)
  {
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "UPDATE Calls SET isarchived=$value WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }

  function SetFlagOnForIDs($idlist,$flag)
  {
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "UPDATE Calls SET flags=flags|$flag WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }

  function SetFlagOffForIDs($idlist,$flag)
  {
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "UPDATE Calls SET flags = flags & ~ $flag WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }
  
  function SetFlagOffForIDsArchive($idlist,$flag)
  {
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "UPDATE CallsArchive SET flags = flags & ~ $flag WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }
	
  function ClearCallsArchive()
  {
    $sqlstmt = "DELETE FROM CallsArchive";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt) $this->OnDbErr();
    $stmt->execute();
  }
  
  // walk through all Calls, and add to CallsArchive of another database
  function CopyCallsToArchiveDb($otherdb)
  {
    $cnt = 0;
    $sqlstmt = 'SELECT * FROM Calls';
    $stmt = $this->db->prepare($sqlstmt);
    $addStmt = null;
    if (!$stmt) $this->OnDbErr();
    $stmt->execute();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $cols = array_keys($row);
      if ($addStmt===null)
      {
        $sql2 = 'INSERT INTO CallsArchive ('.implode(',',$cols).') '
        . ' VALUES ('.str_repeat('?,',sizeof($row)-1).'?) ';
        $addStmt = $otherdb->db->prepare($sql2);
        if (!$addStmt) $otherdb->OnDbErr();
      }
      $ix = 0;
      foreach ($cols as $col)
      {
        $ix ++;
        $addStmt->bindValue($ix,$row[$col]);
      }
      // ignore errors (duplicates)
      if ($addStmt->execute())
      {
        $cnt ++;
      }
    }
    return $cnt;
  }
  
  function CopyCallsToArchive($idlist)
  {
    $colnames = array_keys($this->GetTableColumns('Calls'));
    $cols = implode(',',$colnames) . "\n";
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    // Copy screen recording data, because this might be added delayed.
    // Only add screen recording flag (2), not remove it,
    // because backup copies screen recording, not removes it,
    // except during retention, which is another phase, not here.
    $sqlstmt = "UPDATE CallsArchive AS A "
    . " SET flags = A.flags|(Calls.flags&2) "
    . " , screenfn = Calls.screenfn "
    . " FROM Calls "
    . " WHERE A.id = Calls.id "
    . " AND A.id IN $idarg ";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt) $this->OnDbErr();
    if (!$stmt->execute()) $this->OnDbErr();
    $sqlstmt = "INSERT INTO CallsArchive ($cols) SELECT $cols FROM Calls WHERE id IN $idarg"
    . " AND NOT EXISTS (SELECT id FROM callsarchive WHERE calls.id=callsarchive.id)";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt) $this->OnDbErr();
    if (!$stmt->execute()) $this->OnDbErr();
  }
  
  function SetProcessingFlagOnForIDs($idlist,$flag)
  {
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "UPDATE Calls SET processingstate=processingstate|$flag WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }

  function RemoveProcessingFlagForIDs($idlist,$flag)
  {
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "UPDATE Calls SET processingstate=processingstate&(~$flag) WHERE id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }

  function SetProcessingFlagOnForAll($flag)
  {
		$flag = (int) $flag;
    $sqlstmt = "UPDATE Calls SET processingstate=processingstate|$flag";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }

  function RemoveProcessingFlagForAll($flag)
  {
		$flag = (int) $flag;
    $sqlstmt = "UPDATE Calls SET processingstate=processingstate&(~$flag) "
		. " WHERE (processingstate & $flag) <> 0 "; // for speed-up
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
  }

  function SetAdmin()
  {
    $this->actuinfo['permissions'] = 255
    + PM_VIEW_TRANSCRIPT
    ;
  }
  
  function AllowEditNames()
  {
    $this->actuinfo['permissions'] |= PM_EDITNAMES;
  }

  function GetSessionCount()
  {
    return $this->db->query('SELECT COUNT(id) FROM Sessions')->fetchColumn(0);
  }

  function UpdateSession($sessionid,$lastacc)
  {
		if ($this->readonly) return;
    $sql = 'UPDATE Sessions SET lastacc=? WHERE sessionid=?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$lastacc);
    $stmt->bindValue(2,$sessionid);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
  }

  function InsertSession($data)
  {
    $sql = 'INSERT INTO Sessions ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';

    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
  }

  function DeleteAllSessions()
  {
    if ($this->db->exec('DELETE FROM Sessions')===FALSE)
    {
      $this->OnDbErr();
    }
  }

  function DeleteSession($sessionid)
  {
		if ($this->readonly) return;
    $sql = 'DELETE FROM Sessions WHERE sessionid=?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$sessionid);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
  }

  function GetSessions()
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM Sessions');
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    $stmt->execute();
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
  }

	// returns as the keys of an array
	function GetContactsOfCallForFilter($callinfo, $side=null )
	{
		$UsrFilterLocal = $this->GetCachedConfigValue('UsrFilterLocal')!=0;
		$result = array();
    if ($side===null || $side=='L')
    {
      $result[ $callinfo['localcontact'] ] = true;
    }
		if ($UsrFilterLocal==false)
		{
      if ($side===null || $side=='R')
      {
        $result[ $callinfo['remotecontact'] ] = true;
      }
      if ($side===null)
      {
        if (isset($callinfo['connectedcontact']))
        {
          $result[ $callinfo['connectedcontact'] ] = true;
        }
      }
		}
    if ($side===null || $side=='L')
    {
      $linenr = ensure_defined($callinfo['linenr'], null);
      if ($linenr!==NULL && $linenr!=-1 && $linenr!=0)
      {
        $fuller = $this->GetCallInfo($callinfo['id']);
        $res = $this->GetLineContact( $linenr, $fuller['recorderid']);
        if ($res!=NULL)
        {
          $result[ $res['contactid'] ] = true;
        }
      }
      $res = $this->GetIpAddr( $callinfo['local2'] );
      if ($res!==NULL)
      {
        $result[ $res ] = true;
      }
    }
    if ($side===null || $side=='R')
    {
      if ($UsrFilterLocal==false)
      {
        $res = $this->GetIpAddr( $callinfo['remote2'] );
        if ($res!==NULL)
        {
          $result[ $res ] = true;
        }
      }
    }
		return $result;
	}
	
  function GetUsersOfPhonesFl($contactid,$OnlyEnabled)
  {
    $ret = array();
    $users = $this->GetUsersOfPhones($contactid);
    foreach ($users as $useraccid)
    {
      $userinfo = $this->GetUserInfoOnAccId( $useraccid );
      if ($OnlyEnabled)
      {
        if ($userinfo['enabled']==false) continue;
      }
      // if a free-seating account,
      // don't match on the phones,
      // because they are shared by many.
      if ($userinfo['freeseat']!=0 && $userinfo['customlocalid']==1)
      {
        continue;
      }
      $ret[] = $useraccid;
    }
    return $ret;
  }
  
	// does not check permissions,
	// but if local/remote of call is a phone of user
	function GetUsersOfCall($callinfo, $OnlyEnabled, $side=null)
	{
		// map is used to avoid duplicates
		$resultmap = array();
		$contacts = $this->GetContactsOfCallForFilter( $callinfo , $side );
		foreach ($contacts as $contactid=>$dummy)
		{
			$users = $this->GetUsersOfPhones($contactid);
			foreach ($users as $useraccid)
			{
				$userinfo = $this->GetUserInfoOnAccId( $useraccid );
				if ($OnlyEnabled)
				{
					if ($userinfo['enabled']==false) continue;
				}
				if (isset($callinfo['tenantid']) && $callinfo['tenantid']!=NULL)
				{
					// if the recording belongs to a tenant,
					// then the user must belong to the same tenant
					if ($callinfo['tenantid']!=$userinfo['tenantid']) continue;
				}
				// if a free-seating account,
				// don't match on the phones,
				// because they are shared by many.
				if ($userinfo['freeseat']!=0 && $userinfo['customlocalid']==1)
				{
					continue;
				}
				$resultmap[ $useraccid ] = true;
			}
		}
		$result = array();
		foreach ($resultmap as $useraccid=>$dummy)
		{
			$result[] = $useraccid;
		}
		return $result;
	}
	
  // returns accid, not user id
  function GetUsersOfPhones($contactid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT accid '
    . ' FROM UserPhones WHERE contactid=:1 '
    );
    $stmt->bindValue(1,$contactid);
    $stmt->execute();
    while ($row = $stmt->fetch())
    {
      $result[] = $row['accid'];
    }
    return $result;
  }
	
  function GetTenantUsers($tenantid)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM Users WHERE tenantid=:1');
    $stmt->bindValue(1,$tenantid);
    $stmt->execute();
    while($row = $stmt->fetch())
    {
      $result[] = $row;
    }    
    return $result;
  }
  
  function GetGroupAllProp($accid,$OnlyEnabled,$prop){
		$ret = array();
		$this->DetermineAllGroups();
		if (isset($this->allgroups[$accid]))
    {
			// the accid refers to a group
	    // examine this group
			$groups = array($accid);
			// but also subgroups
			$this->FindSubGroups($groups,$accid);
			// find group members...
			foreach ($groups as $groupaccid)
			{
				$members = $this->GetGroupMembers($groupaccid);
				foreach ($members as $member)
				{
					$ret = array_merge( $ret , $this->GetGroupAllProp($member,$OnlyEnabled,$prop) );
				}
			}
		} else
		{
			// the accid refers to a user
      if ($prop=='name')
      {
        $user = $this->GetUserInfoOnAccId($accid);
        $ret = array( $user['name'] );
      }
		}
		return $ret;
  }
  
  function GetGroupAllNumbers($accid,$OnlyEnabled){
    $result = array();
    $contacts = $this->GetAccidPhoneContacts($accid,$OnlyEnabled);
    $query = 'SELECT distinct callerid FROM callerids WHERE deleted=0 AND ' . $this->GetIn('contactid',implode(',',$contacts));
    $stmt = $this->db->prepare($query);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while($row = $stmt->fetch()){
      array_push($result, $row['callerid']);
    }
    return $result;
  }
  
  function GetTenantNumbers($accid){
    $result = array();
    $TenantPhonesOnlyEnabled = (int) $this->GetConfigValue('TenantPhonesOnlyEnabled');
    $contacts = $this->GetAccidPhoneContacts($accid,$TenantPhonesOnlyEnabled!=0);
    $query = 'SELECT distinct callerid FROM callerids WHERE deleted=0 AND ' . $this->GetIn('contactid',implode(',',$contacts));
    $stmt = $this->db->prepare($query);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while($row = $stmt->fetch()){
      array_push($result, $row['callerid']);
    }
    return $result;
  }
  
	function GetUserPhoneNumbers()
	{
		$result = array();
		$stmt = $this->db->prepare('SELECT distinct c.callerid FROM CallerIDs c, UserPhones u WHERE u.contactid = c.contactid ORDER BY c.callerid');
		$stmt->execute();
		while($row = $stmt->fetch())
		{
			array_push($result, $row['callerid']);
		}
		return $result;
	}
	
	function GetRecOnDmdUserNumbers(){
		$result = array();
		$stmt = $this->db->prepare('SELECT distinct callerid '
		.	'FROM callerids '
		.	'WHERE contactid in( '
			.	'SELECT contactid '
			.	'FROM UserPhones '
			. 'WHERE accid in ( '
				.	'SELECT accid '
				.	'FROM Users '
				.	'WHERE recondmd = 1 ) ) ');
		$stmt->execute();
		while($row = $stmt->fetch())
		{
			array_push($result, $row['callerid']);
		}
		return $result;
	}

	function GetDelOnDmdUserNumbers(){
		$result = array();
		$stmt = $this->db->prepare('SELECT distinct callerid '
		.	'FROM callerids '
		.	'WHERE contactid in( '
			.	'SELECT contactid '
			.	'FROM UserPhones '
			. 'WHERE accid in ( '
				.	'SELECT accid '
				.	'FROM Users '
				.	'WHERE delondmd = 1 ) ) ');
		$stmt->execute();
		while($row = $stmt->fetch())
		{
			array_push($result, $row['callerid']);
		}
		return $result;
	}
	
	function GetSilOnDmdUserNumbers(){
		$result = array();
		$stmt = $this->db->prepare('SELECT distinct callerid '
		.	'FROM callerids '
		.	'WHERE contactid in( '
			.	'SELECT contactid '
			.	'FROM UserPhones '
			. 'WHERE accid in ( '
				.	'SELECT accid '
				.	'FROM Users '
				.	'WHERE silondmd = 1 ) ) ');
		$stmt->execute();
		while($row = $stmt->fetch())
		{
			array_push($result, $row['callerid']);
		}
		return $result;
	}

  function GetUserPersonalPm($accid)
  {
    $pm = 0;
    $accaccs = $this->GetAccAccPms(array($accid));
    foreach ($accaccs as $accacc)
    {
      if ($accacc['callaccid']==0)
      {
        // not specific to contacts
        $pm |= $accacc['epermissions'];
      }
    }
    return $pm;
  }

  function GetUserPhones($accid)
  {
		$items = explode(",",$accid);			
		$query='SELECT contactid FROM UserPhones WHERE accid ';
		
		$first=true;
		$query.= "IN ( ";
		foreach($items as $item){
			if($first){
				$first=false;					
			}else{
				$query.=",";
			}
			$query.="?";				
		}		
		$query.=" )";
		
    $result = array();
		$stmt = $this->db->prepare($query);
		$stmt->execute($items);

    while ($row = $stmt->fetch())
    {
      $result[] = $row['contactid'];
    }
		// getting my own phones
		if ($this->actuinfo['accid']==$accid
		// and this is a free-seating account
		&& isset($this->actuinfo['freeseat']) && $this->actuinfo['freeseat']!=0)
		{
			// In case this is a free-seating account,
			// then the phone list needs to be filtered,
			// based on the pcname.
			$seats = $this->GetSeats( $this->pcname );
			$seat_contactids = array();
			foreach ($seats as $seat)
			{
				$seat_contactids[] = $seat['contactid'];
			}
			$result = array_intersect( $seat_contactids , $result );
			
			// we to do it this way because:
			// - it saves time for who configurates the system
			// - this item cannot be connected to a workstation
			//   so it should by-pass the previous check.
			// - it removes redundancy when the name is changed.
			// - the check on customlocalid is needed,
			//   since customlocalid is properly protected by checks,
			//   but otherwise there is a security risk
			//   of adding a new contact id to your list.
			if ($this->actuinfo['customlocalid']==1)
			{
				// Or the contact that corresponds to the custom local id.
				// It does not need to be in the phone list.
				$result[] = $this->GetCustomLocalContactId();
			}
			
		}
    return $result;
  }
	
	function simplified_stacktrace()
	{
		$arr = debug_backtrace();
		$t = '';
		foreach ( $arr as $item )
		{
			$t .= 'line ' . $item['line'] . ' in ' . $item['file'] . ' '
			. $item['function'] . '(';
			$parts = array();
			foreach ( $item['args'] as $arg )
			{
				if (is_scalar($arg) || is_array($arg))
				{
					$parts[] = (string) $arg;
				}
			}
			$t .= implode(',', $parts);
			$t .= ')' . "\n";
		}
		return $t;
	}

	function GetCustomLocalContactId()
	{
		$CustomLocal = $this->GetOrCreateCallerID( $this->actuinfo['name'] , '', true );
		return $CustomLocal['contactid'];
	}
	
  function ManagerPhoneMatch($expr,$num)
  {
    // Check if expr matches number
    $expr = str_replace('+','\\+',$expr);
    $expr = str_replace('.','\\.',$expr);
    $expr = str_replace('?','.',$expr);
    $expr = str_replace('*','.*',$expr);
    $expr = '/^' . $expr . '$/';
    if (preg_match($expr,$num)!=0) return true;
    return false;
  }

  function SetUserPhones($accid,$data)
  {
    if($this->FromScim($accid))
    {
      $toRemain = array_map(function($obj)
      {
        return $obj['contactid'];
      }, $data);

      $eScimUserPhones = $this->GetScimUserPhones($accid);
      $scimUserPhones = [];
      foreach($eScimUserPhones as $eScim)
      {
        if(in_array($eScim['contactid'], $toRemain))
        {
          $scimUserPhones[] = $eScim;
        }
      }
      
      $scimUserContacts = array_map(function($scimPhone){
        return $scimPhone['contactid'];
      }, $scimUserPhones);
    }

    $theuser = $this->GetUserInfoOnAccId($accid);
    if ($this->MayEditUserAccount($theuser['id'],$accid)==false)
    {
      return false;
    }
    $allowed = false;
    $aupm = $this->actuinfo['permissions'];
    if ($aupm&PM_CONFIG) $allowed = true;
    if ($aupm&PM_USERS) $allowed = true;
    if ($aupm&PM_T2_ADMIN) $allowed = true;
    if ($aupm&PM_T3_ADMIN) $allowed = true;
    if ( $allowed==false // not already allowed for other reasons
      && ($this->anyaccpms&PM_USERS)!=0 // we have access to some user accounts
      && ($this->GetAccPm($accid)&PM_USERS)!=0 // access to this account
    )
    {
      $allowed = true;
      // The new phones must be either:
      // - Already in the list (not new)
      // - In the managed phone list
      // / not: In the list of phones of the manager
      $olddata = $this->GetUserPhones( $accid );
      //$ownPhones = $this->GetUserPhones( $this->actuinfo['accid'] );
      foreach ($data as $obj)
      {
        $isnew = true;
        foreach ($olddata as $oldobj)
        {
          if ($oldobj==$obj['contactid'])
          {
            $isnew = false;
            break;
          }
        }
        // Existing are allowed always
        if ($isnew==false) continue;
        // check it
        $isok = false;
        // Is it the manager's own phone?
        // no longer sufficient condition.
        /*
         foreach ($ownPhones as $phone)
         {
         if ($phone==$obj['contactid'])
         {
         $isok = true;
         break;
         }
         }
         */
        // Look up the callerid (number) of this contact
        $ctinfo = $this->GetContactInfo($obj['contactid']);
        //var_dump($ctinfo);
        // Check in the manager's managed phones list
        $mphones = explode(',',$this->actuinfo['mphones']);
        foreach ($mphones as $phone)
        {
          if ($ctinfo['ctype']==0)
          {
            if ($this->ManagerPhoneMatch($phone,$ctinfo['callerid']))
            {
              // any match is sufficient
              $isok = true;
              break;
            }
          }
        }
				// for ctype==2 (IP addr.), it always fails
        if ($isok==false)
        {
          // if no match, then not allowed
          $allowed = false;
          break;
        }
      }
    } // end pm_users

    if ($aupm&PM_SUBADMIN && $allowed==false)
    {
      $allowed = true;
      // a sub-admin
      // check if all user phones are also phones of the manager
      // others it is not allowed
      $phonelist = $this->GetUserPhones( $this->actuinfo['accid'] );
      foreach ($phonelist as $phone)
      {
        $phonemap[ $phone ] = true;
      }
      foreach($data as $obj)
      {
        if (isset($phonemap[ $obj['contactid'] ])==false)
        {
          $this->lasterr = 'pm-denied';
          $allowed = false;
          break;
        }
      }
    }

    if ($allowed==false)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
		
		if ( (int) $this->GetCachedConfigValue('UniquePhones') != 0 )
		{
			// Phones should be unique per user
			foreach($data as $obj)
			{
				if ($this->PhoneInUseExcept( (int) $obj['contactid'] , $accid ))
				{
					$this->lasterr = 'phone-in-use';
					return false;
				}
			}
		}
		
    $stmt = $this->db->prepare('DELETE FROM UserPhones WHERE accid=:1');
    $stmt->bindValue(1,$accid);
    $stmt->execute();

    $stmt = $this->db->prepare('INSERT INTO UserPhones '
    . ' (accid,contactid) '
    . ' VALUES '
    . ' (:1    ,:2      ) '
    );
    foreach($data as $obj)
    {
      if(isset($scimUserContacts) && in_array($obj['contactid'], $scimUserContacts))
      {
        continue;
      }

      //echo $obj['contactid'].' '.$obj['pm'];
      $stmt->bindValue(1,$accid);
      $stmt->bindValue(2,$obj['contactid']);
      $stmt->execute();
    }

    if(isset($scimUserPhones))
    {
      return $this->SetScimUserPhones($accid, $scimUserPhones);
    }
    else
    { 
      return true;
    }
  }

	function PhoneInUseExcept($contactid, $accid)
	{
		//echo 'PhoneInUseExcept ' . $contactid . ' accid: ' . $accid . "\r\n";
		$sql = 'SELECT accid FROM UserPhones WHERE accid<>:1'
		. ' AND contactid=:2';
		$stmt = $this->db->prepare( $sql );
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
		$stmt->bindValue(1,$accid);
		$stmt->bindValue(2,$contactid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$row = $stmt->fetch();
		if ($row !== false)
		{
			$this->lasterr = 'phone-in-use';
			return $row['accid'];
		}
		return false;
	}			
	
	function SetMobileDevices($mobs)
	{
		$cthis = $this;
		$f = function () use ($cthis,$mobs)
		{
			return $cthis->SetMobileDevicesT($mobs);
		};
		return $this->TransactionRetry( $f );
	}
	
	function SetMobileDevicesT($mobs)
	{
		// do in a transction
		$this->StartTransaction();
		
		if ($this->db->exec('DELETE FROM MobileDevices')===false)
		{
      $this->OnDbErr();
		}

    $stmt = $this->db->prepare('INSERT INTO MobileDevices '
    . ' (imei,phonenum) '
    . ' VALUES '
    . ' (:1    ,:2      ) '
    );
    foreach($mobs as $mob)
    {
      $stmt->bindValue(1,$mob['imei']);
      $stmt->bindValue(2,$mob['phonenum']);
      if ($stmt->execute()===false)
			{
				$this->OnDbErr();
			}
    }

		$this->CommitTransaction();
		return true;
	}
	
	function GetMobileDevices()
	{
    $sqlstmt = 'SELECT * FROM MobileDevices';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
		
	}

	// key functions
	
  function ClearLicKeys()
  {
		$ref_db =& $this->db;
		if ($this->sharedb) $ref_db =& $this->machinedb;
    $stmt = $ref_db->prepare('DELETE FROM LicKeys');
    if (!$stmt->execute())
    {
      $this->OnDbErr();
    }
  }

  function DeleteProductLicKeys($productid)
  {
		$ref_db =& $this->db;
		if ($this->sharedb) $ref_db =& $this->machinedb;
		$productid = (int) $productid;
    $stmt = $ref_db->prepare("DELETE FROM lickeys WHERE LENGTH(usedkey)<30 AND usedkey LIKE '$productid%'");
    if (!$stmt->execute())
    {
      $this->OnDbErr();
    }
  }

  function UsedLicKey($lickey)
  {
		$ref_db =& $this->db;
		if ($this->sharedb) $ref_db =& $this->machinedb;
    $result = array();
    $stmt = $ref_db->prepare('SELECT id FROM LicKeys WHERE usedkey=:1');
    $stmt->bindValue(1,$lickey);
    if (!$stmt->execute())
    {
      $this->OnDbErr();
    }
    if ($stmt->fetch()==false) return false;
    else return true;
  }

  function AddLicKey($lickey)
  {
		$ref_db =& $this->db;
		if ($this->sharedb) $ref_db =& $this->machinedb;
    $stmt = $ref_db->prepare('INSERT INTO LicKeys(usedkey) VALUES(:1)');
    $stmt->bindValue(1,$lickey);
    $stmt->execute();
  }

  // Saved Queries
  function GetQueries()
  {
    $result = array();
    $sqlstmt = 'SELECT * FROM Queries ORDER BY id';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
  }
  function DelQuery($dbid)
  {
    // delete accessor
    $stmt = $this->db->prepare('DELETE FROM Queries WHERE id=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    }
    $stmt->bindValue(1,$dbid);
    $stmt->execute();
  }
  function AddQuery($name,$qspec)
  {
    $stmt = $this->db->prepare('insert into Queries '
    . ' (name,qspec) '
    . ' VALUES '
    . ' (:1       ,:2   ) '
    );
    $stmt->bindValue(1,$name);
    $stmt->bindValue(2,$qspec);
    $stmt->execute();
    return $this->db->lastInsertId('Queries_id_seq');
  }

	// TransferTargets
	
	function GetTransferTargets()
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * from TransferTargets ORDER BY id');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
	}

	function GetTransferTarget( $id )
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * from TransferTargets WHERE ID=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    }
		$stmt->bindValue( 1 , $id );
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return $stmt->fetch(PDO::FETCH_ASSOC);
	}
	
	function AddTransferTarget(&$id,$data)
	{
    $sql = 'INSERT INTO TransferTargets ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    } 
		$ix = 0;
		foreach($data as $value)
		{
			$ix ++;
			$stmt->bindValue($ix,$value);
		}
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    $id = $this->db->lastInsertId('TransferTargets_id_seq');
    return true;
	}
	
	function EditTransferTarget($id, $data)
	{
    $sql = 'UPDATE TransferTargets SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}
	
	function DelTransferTarget($id)
	{
    $sql = 'DELETE FROM TransferTargets WHERE id=:1 ';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}

	//ImportTargets
	
	function GetEnabledImportTargetsOfType($rectype)
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * FROM ImportTargets'
		. ' WHERE rectype=:1 '
		. ' AND enabled=1 '
		. ' ORDER BY id');
    $stmt->bindValue(1,$rectype);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
	}
	
	function GetImportTargets()
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * from ImportTargets ORDER BY id');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
	}

	function GetImportTarget( $id )
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * from ImportTargets WHERE ID=:1');
    if (!$stmt)
    {
      $this->OnDbErr();
      return false;
    }
		$stmt->bindValue( 1 , $id );
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return $stmt->fetch(PDO::FETCH_ASSOC);
	}
	
	function AddImportTarget($data)
	{
    $sql = 'INSERT INTO ImportTargets ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    } 
		$ix = 0;
		foreach($data as $value)
		{
			$ix ++;
			$stmt->bindValue($ix,$value);
		}
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}
	
	function EditImportTarget($id, $data)
	{
    $sql = 'UPDATE ImportTargets SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}
	
	function DelImportTarget($id)
	{
    $sql = 'DELETE FROM ImportTargets WHERE id=:1 ';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}
	
	// AuthKeys
	
	function GetAuthKeys()
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * from AuthKeys ORDER BY id');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
	}
	
	function AddAuthKey($data)
	{
    $sql = 'INSERT INTO AuthKeys ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    } 
		$ix = 0;
		foreach($data as $value)
		{
			$ix ++;
			$stmt->bindValue($ix,$value);
		}
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}
	
	function EditAuthKey($id, $data)
	{
    $sql = 'UPDATE AuthKeys SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}
	
	function DelAuthKey($id)
	{
    $sql = 'DELETE FROM AuthKeys WHERE id=:1 ';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    return true;
	}
	
	// Recorders

	function UpdateRecorderConnection($id, $data)
	{
		$sql = 'UPDATE RecorderConnections SET ';
		$first = true;
		foreach ($data as $key=>$value)
		{
			if (!$first) $sql .= ',';
			$sql .= ' ' . $key . '=?';
			$first = false;
		}
		$sql .= ' WHERE id = ?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
		$ix++;
    $stmt->bindValue($ix,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
	}
	
  function GetRecorderById($id)
  {
    $result = array();
    $stmt = $this->db->prepare('SELECT * from Recorders WHERE id=?');
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==false) $this->OnDbErr();
    return $stmt->fetch();
  }
  
	function GetRecorders()
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * from Recorders ');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
	}
	
	function GetRecorderConnections()
	{
    $result = array();
    $stmt = $this->db->prepare('SELECT * from RecorderConnections');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
    return $result;
	}
	
	// IdMap
	
	function AddToIdMap($theid,$value,$target)
	{
		$sql = 'INSERT INTO IdMap (theid, value, target) '
    . ' VALUES (?,?,?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1, $theid);
    $stmt->bindValue(2, $value);
    $stmt->bindValue(3, $target);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
	}
	
	function EditIdMap($theid,$value,$target)
	{
		$sql = 'DELETE FROM IdMap WHERE theid=? ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1, $theid);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		
		$this->AddToIdMap($theid,$value,$target);
	}
	
	function GetIdMapEntry($theid)
	{
		$sql = 'SELECT * FROM IdMap WHERE theid=? ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1, $theid);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		$arr = $stmt->fetch();
		return $arr;
	}
	
	function ClearIdMap()
	{
    $stmt = $this->db->prepare('DELETE FROM IdMap');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
	}
  
  // ActionCache

  // get all items in the action cache
  function GetActionCacheCount()
  {
    $stmt = $this->db->prepare('SELECT COUNT(id) FROM ActionCache');
    $stmt->execute();
		$res = $stmt->fetchColumn(0);
		if ($res===false) return 0;
		return (int) $res;
  }
  
  // get items in the action cache that match a function
  function GetActionCache($checkFunction, & $ref_functionData)
  {
    $sql = "SELECT * FROM ActionCache";
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    if (!$stmt->execute()) $this->OnDbErr();
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      if ($checkFunction($ref_functionData,$row))
      {
        $result[] = $row;
      }
    }
    return $result;
  }

  function RemoveActionCacheItem($id)
  {
    $id = (int) $id;
    $sql = 'DELETE FROM ActionCache WHERE id=?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$id);
    if (!$stmt->execute()) $this->OnDbErr();
    return true;
  }

  // remove all actions in the cache that are older than
  // the specified number of seconds,
  // and items that are past their own expiration date
  function ExpireActionCache($retentionSeconds)
  {
    $retentionSeconds = (int) $retentionSeconds;
    $maxTime = time() - $retentionSeconds;
    $sql = 'DELETE FROM ActionCache WHERE addtime<?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$maxTime);
    if (!$stmt->execute()) $this->OnDbErr();

    // this will skip NULL entries, which is good
    $sql = 'DELETE FROM ActionCache WHERE expires<?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,time());
    if (!$stmt->execute()) $this->OnDbErr();
    
    return true;
  }

  // add specified action to the cache to be performed later
  function AddToActionCache($action,$expires)
  {
    $sql = 'INSERT INTO ActionCache(action,addtime,expires) VALUES (?,?,?)';
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$action);
    $stmt->bindValue(2,time());
    $stmt->bindValue(3,$expires);
    if (!$stmt->execute()) $this->OnDbErr();
    return true;
  }
	
	// Annotations

	function HasAnnotatePM( $recording_id )
	{
		$info = $this->GetCallInfo($recording_id);
		if (($info['permissions']&PM_NOTES)==0)
		{
			return false;
		}
		return true;
	}
		
	function GetAnnotation($id)
	{
		$sql = 'SELECT * FROM Annotations WHERE id=? ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1, $id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		
    return $stmt->fetch(PDO::FETCH_ASSOC);
	}
	
  function GetTranscriptUC($id)
  {
    $id = (int) $id;
		$sql = 'SELECT transcript FROM Transcriptions WHERE '
		. ' call_id = ?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
    $row = $stmt->fetch(PDO::FETCH_ASSOC);
    if ($row===false) return false;
    return $row['transcript'];
  }
  
  function GetTranscript($id)
  {
    $id = (int) $id;
    $callinfo = $this->GetCallInfo($id);
    if ($callinfo===false)
    {
      $this->lasterr = "not-found";
      return false;
    }
    $pm = $callinfo['permissions'];
    if ( ($pm & PM_VIEW_TRANSCRIPT) == 0 )
    {
      $this->lasterr = "pm-denied";
      return false;
    }
    return $this->GetTranscriptUC($id);
  }
  
  function DeleteTranscripts($idlist)
  {
		$idlist = $this->IntifyArray( $idlist );
    $idarg = '(' . implode(',',$idlist) . ')';
    $sqlstmt = "DELETE FROM Transcriptions WHERE call_id IN $idarg";
    $stmt = $this->db->prepare($sqlstmt);
    if (!$stmt) $this->OnDbErr();
    if ($stmt->execute()==FALSE) $this->OnDbErr();
  }
  
  function DeleteTranscript($id)
  {
    $id = (int) $id;
    $sql = 'DELETE FROM Transcriptions WHERE call_id=?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==FALSE) $this->OnDbErr();
  }
  
  function SetTranscript($id,$text)
  {
    $id = (int) $id;
    $callinfo = $this->GetCallInfo($id);
    if ($callinfo===false)
    {
      $this->lasterr = "not-found";
      return false;
    }
    $pm = $callinfo['permissions'];
    if ( ($pm & PM_VIEW_TRANSCRIPT) == 0 )
    {
      $this->lasterr = "pm-denied";
      return false;
    }
    if ( ($pm & PM_NOTES) == 0 )
    {
      $this->lasterr = "pm-denied";
      return false;
    }
		// check if exists, if we must update or insert
    $sql = 'SELECT call_id FROM Transcriptions WHERE call_id = ?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==FALSE) $this->OnDbErr();
    if ($stmt->fetch(PDO::FETCH_ASSOC)===false)
    {
      $sql = 'INSERT INTO Transcriptions(transcript,call_id) VALUES(?,?)';
    } else
    {
      $sql = 'UPDATE Transcriptions SET transcript=? WHERE call_id = ?';
    }
    $stmt = $this->db->prepare($sql);
    if (!$stmt) $this->OnDbErr();
    $stmt->bindValue(1,$text);
    $stmt->bindValue(2,$id);
    if ($stmt->execute()==FALSE) $this->OnDbErr();
		return true;
  }
  
  function GetHasTranscriptSet($ids)
  {
		$ids = $this->IntifyArray( $ids );
		$sql = 'SELECT call_id FROM Transcriptions WHERE '
		. $this->GetIn('call_id',implode(',',$ids));
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
			$result[ $row['call_id'] ] = true;
    }
		return $result;    
  }
  
	function GetAnnotationSet($ids,$hideAuto=false)
	{
		$ids = $this->IntifyArray( $ids );
		$sql = 'SELECT * FROM Annotations WHERE '
		. $this->GetIn('call_id',implode(',',$ids));
    if ($hideAuto) $sql .= ' AND createdby IS NOT NULL';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
			$result[ $row['call_id'] ][] = $row;
    }
		return $result;
	}
	
	function GetAnnotations($call_id)
	{
		$sql = 'SELECT * FROM Annotations WHERE call_id=? ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1, $call_id);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
		return $result;
	}

	function DelAnnotationPerform( $id , &$ref_retobj )
	{
		$marker = $this->GetAnnotation($id);
		if ($marker==null)
		{
			$ref_retobj['errcode'] = 'not-found';
			return false;
		}
		
		if ($this->HasAnnotatePM($marker['call_id'])==false)
		{
			$ref_retobj['errcode'] = 'pm-denied';
			return false;
		}
		
		// add to database
		$stmt = $this->db->prepare('DELETE FROM Annotations '
		. ' WHERE id=:1 '
		);
		$stmt->bindValue(1,$id);
		$stmt->execute();
		
		return true;
	}
	
  function DelAnnotationsOfCall( $call_id )
  {
		$stmt = $this->db->prepare('DELETE FROM Annotations '
		. ' WHERE call_id=:1 '
		);
		$stmt->bindValue(1,$call_id);
		$stmt->execute();
  }
  
	function DelAnnotation( $id )
	{
		$cthis = $this;
		$f = function () use ($cthis,$id)
		{
			return $cthis->DelAnnotationT($id);
		};
		return $this->TransactionRetry( $f );
	}
	
	function DelAnnotationT( $id )
	{
		$retobj = array();
		$retobj['errcode'] = 'undefined';
		$this->StartTransaction();
		if ($this->DelAnnotationPerform( $id , $retobj ))
		{
			$this->CommitTransaction();
			$retobj['errcode'] = 0;
		} else
		{
			$this->CancelTransaction();
		}
		return $retobj;
	}

	function DetermineAnnotationLabel($call_id)
	{
		// choose a unique label (A-Z)
		$labels_in_use = array();
		$existing = $this->GetAnnotations($call_id);
		foreach ($existing as $obj)
		{
			$labels_in_use[$obj['label']] = true;
		}
		// if all used, then use
		$label = '+';
		// choose first one that is free
		for ($i='A';$i<='Z';$i++)
		{
			if (isset($labels_in_use[$i])==false)
			{
				$label = $i;
				break;
			}
		}

		return $label;
	}
	
	function AddAnnotationPerform( $marker , &$ref_retobj )
	{
		if ($this->HasAnnotatePM($marker['call_id'])==false)
		{
			$ref_retobj['errcode'] = 'pm-denied';
			return false;
		}
		
		$label = $this->DetermineAnnotationLabel($marker['call_id']);
		
		// add to database
		$stmt = $this->db->prepare('insert into Annotations '
		. ' (call_id, descr, timepos, label, createtime, createdby) '
		. ' VALUES '
		. ' (:1     , :2   , :3     , :4   , :5        , :6 ) '
		);
		$stmt->bindValue(1,$marker['call_id']);
		$stmt->bindValue(2,$marker['descr']);
		$stmt->bindValue(3,$marker['timepos']);
		$stmt->bindValue(4,$label);
		$stmt->bindValue(5,time());
		$stmt->bindValue(6,$this->actuinfo['id']);
		$stmt->execute();
		
		// tell back which ID and label the new marker has
		$ref_retobj["id"] = $this->db->lastInsertId('Annotations_id_seq');
		$ref_retobj["label"] = $label;
		
		return true;
	}
	
	function AddAnnotation( $marker )
	{
		$cthis = $this;
		$f = function () use ($cthis,$marker)
		{
			return $cthis->AddAnnotationT($marker);
		};
		return $this->TransactionRetry( $f );
	}
	
	function AddAnnotationT( $marker )
	{
		$retobj = array();
		$retobj['errcode'] = 'undefined';
		$this->StartTransaction();
		if ($this->AddAnnotationPerform($marker, $retobj))
		{
			$this->CommitTransaction();
			$retobj['errcode'] = 0;
		} else
		{
			$this->CancelTransaction();
		}
		return $retobj;
	}

	function EditAnnotationPerform( $id, $data , &$ref_retobj )
	{
		$marker = $this->GetAnnotation($id);
		if ($marker==null)
		{
			$ref_retobj['errcode'] = 'not-found';
			return false;
		}
		
		if ($this->HasAnnotatePM($marker['call_id'])==false)
		{
			$ref_retobj['errcode'] = 'pm-denied';
			return false;
		}
		
		// edit
    $sql = 'UPDATE Annotations SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
		$ix ++;
		if ($ix!=1) $sql .= ',';
		$sql .= " edittime=:$ix ";
		$ix ++;
		if ($ix!=1) $sql .= ',';
		$sql .= " editedby=:$ix ";
    $ix ++;
    $sql .= " WHERE id=:$ix";
    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
		$stmt->bindValue($ix,time());
    $ix ++;
		$stmt->bindValue($ix,$this->actuinfo['id']);
    $ix ++;
		$stmt->bindValue($ix,$id);
		$stmt->execute();

		return true;
	}
	
	function EditAnnotation( $id, $marker )
	{
		$cthis = $this;
		$f = function () use ($cthis,$id,$marker)
		{
			return $cthis->EditAnnotationT($id,$marker);
		};
		return $this->TransactionRetry( $f );
	}
	
	function EditAnnotationT( $id, $marker )
	{
		$retobj = array();
		$retobj['errcode'] = 'undefined';
		$this->StartTransaction();
		if ($this->EditAnnotationPerform($id, $marker, $retobj))
		{
			$this->CommitTransaction();
			$retobj['errcode'] = 0;
		} else 
		{
			$this->CancelTransaction();
		}
		return $retobj;
	}

	function AddFinishedAnnotation($call_id, $marker, $timestamp)
	{
		$stmt = $this->db->prepare('insert into Annotations '
		. ' (call_id, label, timepos, descr, createtime) '
		. ' VALUES '
		. ' (:1     , :2   , :3     , :4   , :5        ) '
		);

		$label = $this->DetermineAnnotationLabel($call_id);
		
		$stmt->bindValue(1,$call_id);
		$stmt->bindValue(2,$label);
		$stmt->bindValue(3,$marker['timepos']);
		$stmt->bindValue(4,$marker['descr']);
		$stmt->bindValue(5,$timestamp);
		$res = $stmt->execute();
		return $res;
	}

	function AddActiveAnnotation($call_id, $marker, $timestamp)
	{
		$stmt = $this->db->prepare('insert into ActiveAnnotations '
		. ' (call_id, descr, timepos, createtime) '
		. ' VALUES '
		. ' (:1     , :2   , :3     , :4        ) '
		);

		$stmt->bindValue(1,$call_id);
		$stmt->bindValue(2,$marker['descr']);
		$stmt->bindValue(3,$marker['timepos']);
		$stmt->bindValue(4,$timestamp);
		$res = $stmt->execute();
		return $res;
	}

	function GetActiveAnnotations($call_id)
	{
		$sql = 'SELECT * FROM ActiveAnnotations WHERE call_id=? ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      return false;
    }
    $stmt->bindValue(1, $call_id);
    if ($stmt->execute()==FALSE)
    {
      return false;
    }
		
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
		return $result;
	}

	function MoveActiveAnnotations($call_id, $old_id)
	{
		$ActiveAnnotations = $this->GetActiveAnnotations($old_id);

		if(count($ActiveAnnotations) === 0)
		{
			return true;
		}

    if (DB_TYPE=='sqlite3')
    {
      $stmt = $this->db->prepare('DELETE FROM ActiveAnnotations WHERE call_id=?');
      $stmt->bindValue(1,$old_id);
      if ($stmt->execute()==false)
      {
        $this->OnDbErr();
      }
    }
    
		$sql = 'insert into Annotations '
		. ' (call_id, label, timepos, descr, createtime) '
		. ' VALUES (?,?,?,?,?) ' ;

		$stmt = $this->db->prepare($sql);
		
		if(!$stmt)
		{
			return false;
		}

		$label = 'A';
		$cnt = 0;
		foreach($ActiveAnnotations as $ActiveAnnotation)
		{		
			
			$off = 1; 

			$stmt->bindValue($off,$call_id);
			$stmt->bindValue($off+1,$label);
			$stmt->bindValue($off+2,$ActiveAnnotation['timepos']);
			$stmt->bindValue($off+3,$ActiveAnnotation['descr']);
			$stmt->bindValue($off+4,$ActiveAnnotation['createtime']);
			
      if($stmt->execute()==false)
      {
        //var_dump($stmt->errorInfo());
        return false;
      }

			if($label < 'Z')
			{
				$label++;
			}
			else
			{
				$label = '+';
			}
			$cnt++;
      
		}
		
    return true;
	}
	
	
	function GetCustomLocalId($source)
	{
		// first check if already exists
		$stmt = $this->db->prepare('SELECT * FROM LocalIdMapping '
		.' WHERE srcid=:1 '
		);
		$stmt->bindValue(1,$source);
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		return $row;
	}
	
	function AddOrUpdateSingleLocalIdMapping($source, $target)
	{
		// first check if already exists
		$stmt = $this->db->prepare('SELECT * FROM LocalIdMapping '
		.' WHERE srcid=:1 '
		);
		$stmt->bindValue(1,$source);
    if (!$stmt)
    {
      $this->OnDbErr();
  	}
		if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row!==false)
		{
			// already exists, update it
			$sql = 'UPDATE LocalIdMapping SET dstid=?, updatetime=? 
			 WHERE srcid=? ';
			$stmt = $this->db->prepare($sql);
			if (!$stmt)
			{
				$this->OnDbErr();
			}
			$stmt->bindValue(1, $target);
			$stmt->bindValue(2, time() );
			$stmt->bindValue(3, $source);
			if ($stmt->execute()==FALSE)
			{
				$this->OnDbErr();
			}
		} else
		{
			// it does not exist, create it
			$sql = 'INSERT INTO LocalIdMapping (srcid, dstid, updatetime) '
			. ' VALUES (?,?,?) ';
			$stmt = $this->db->prepare($sql);
			if (!$stmt)
			{
				$this->OnDbErr();
			}
			$stmt->bindValue(1, $source);
			$stmt->bindValue(2, $target);
			$stmt->bindValue(3, time() );
			if ($stmt->execute()==FALSE)
			{
				$this->OnDbErr();
			}
		}
	}
	
	function AddOrUpdateLocalIdMapping($sources, $target)
	{
		$this->StartTransaction();
		$committed = false;
		try
		{
			foreach ($sources as $ct)
			{
        // mapping unto itself is useless
        if ($ct==$target) continue;
				$this->AddOrUpdateSingleLocalIdMapping($ct, $target);
			}
			$this->CommitTransaction();
			$committed = true;
		} catch(Exception $e)
		{
		}
		if ($committed==false)
		{
			$this->CancelTransaction();
		}
	}
  
  function ExpireLocalIdMapping($time)
  {
    $sql = 'DELETE FROM LocalIdMapping WHERE updatetime < ?';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1, $time);
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
  }
	
	function GetSeats($pcname)
	{
    $sqlstmt = 'SELECT * FROM FreeSeats WHERE pcname=?';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
		$stmt->bindValue(1, $pcname);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
	}
	
	function GetAllSeats()
	{
    $sqlstmt = 'SELECT * FROM FreeSeats';
    $stmt = $this->db->prepare($sqlstmt);
    if ($stmt===FALSE)
    {
      $this->OnDbErr();
    }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
	}
	
	function SetAllSeats($seats)
	{
		$this->StartTransaction();
		$committed = false;
		try
		{
			$sql = 'DELETE FROM FreeSeats';
			$stmt = $this->db->prepare($sql);
			$stmt->execute();
			
			$sql = 'INSERT INTO FreeSeats '
			. ' (pcname,contactid) '
			. ' VALUES '
			. ' (:1    ,:2       ) ';
			$stmt = $this->db->prepare($sql);
			foreach ($seats as $seat)
			{
				$stmt->bindValue(1,$seat['pcname']);
				$stmt->bindValue(2,$seat['contactid']);
				$stmt->execute();
			}
			
			$this->CommitTransaction();
			$committed = true;
		} catch(Exception $e)
		{
		}
		if ($committed==false)
		{
			$this->CancelTransaction();
		}
	}
	
	
	// Normalisation between postgresql and sqlite3.
	
  function GetFalse()
  {
    if (DB_TYPE=='postgresql')
    {
      return 'false';
    }
    if (DB_TYPE=='sqlite3')
    {
      return '0';
    }
  }

  function GetTrue()
  {
    if (DB_TYPE=='postgresql')
    {
      return 'true';
    }
    if (DB_TYPE=='sqlite3')
    {
      return '1';
    }
  }
	
	function RestoreSequence($tabnm)
	{
    if (DB_TYPE=='postgresql')
    {
			$sql = "SELECT setval('".$tabnm."_id_seq', (SELECT MAX(id)+1 FROM ".$tabnm.") )";
	    $stmt = $this->db->prepare($sql);
			if ($stmt->execute()==false)
			{
				$this->onDbErr();
			}
		}
	}

  function GetEscape()
  {
    if (DB_TYPE=='postgresql')
    {
      return "";
			// previous it was: ESCAPE '\\\\'
			// but that causes an error on postgresql v.9.1.9
    }
    if (DB_TYPE=='sqlite3')
    {
      return "ESCAPE '\\'";
    }
  }

  function GetILike()
  {
    if (DB_TYPE=='postgresql')
    {
      return 'ILIKE';
    }
    if (DB_TYPE=='sqlite3')
    {
      return 'LIKE';
    }
  }

  function GetIn($col,$list)
  {
    if ($list=='') return $this->GetFalse();
    else return "$col IN ($list)";
  }

  function GetPKeyIndx($tablename)
  {
    if (DB_TYPE=='postgresql')
    {
      return 'true';
    }
    if (DB_TYPE=='sqlite3')
    {
      return '1';
    }
  }
	
	function GetRecIdsNext()
	{
		$id = 0;
    if (DB_TYPE=='postgresql')
    {
			// _sq instead of _seq, 
			// because _seq is for auto-generated table seq.
      $id = $this->db->query("select nextval('recids_sq')")->fetchColumn(0);
    }
    if (DB_TYPE=='sqlite3')
    {
      $id = rand();
    }
		return sprintf("%04d",$id % 1000);
	}
	
	function GetTableColumns( $table )
	{
    if (DB_TYPE=='sqlite3')
		{
			$result = array();
			$stmt = $this->db->prepare('PRAGMA table_info('.$table.')');
			if ($stmt==false)
			{
				$this->OnDbErr();
			}
			if ($stmt->execute()==false)
			{
				$this->OnDbErr();
			}
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
			{
				$result[ $row['name'] ] = 1;
			}
			return $result;
		}
    if (DB_TYPE=='postgresql')
    {
			$result = array();
			$stmt = $this->db->prepare('select column_name '
			. ' from information_schema.columns '
			. ' where table_schema=:1 and table_name=:2 ');
			$stmt->bindValue( 1, 'public' );
			$stmt->bindValue( 2, strtolower($table) );
			if ($stmt->execute()==false)
			{
				$this->OnDbErr();
			}
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
			{
				$result[ $row['column_name'] ] = 1;
			}
			return $result;
		}
		return null;
	}

	function MergeCallsTables($CallBackNewCall,$FromDate,$ToDate)
	{
    $FromDate = (int) $FromDate;
    $ToDate = (int) $ToDate;
		if (DB_TYPE=='postgresql')
		{
			$dbbu = new DBInterface(CMD_WEB_RELPATH . DATA_PATH, 'apresabu');
		}
		else return null;
		
    $sqlbu = 'SELECT * FROM Calls WHERE 1=1 ';
    if ($FromDate!=0) $sqlbu .= ' AND startdate >= ? ';
    if ($ToDate!=0)   $sqlbu .= ' AND startdate <= ? ';
    $sqlbu .= ' ORDER BY remoteid ASC';
		$stmtbu = $dbbu->db->prepare($sqlbu);
    if ($stmtbu===FALSE)
    {
      return null;
    }
    $ix = 1;
    if ($FromDate!=0) { $stmtbu->bindValue($ix,$FromDate); $ix ++; }
    if ($ToDate  !=0) { $stmtbu->bindValue($ix,$ToDate);   $ix ++; }
    if ($stmtbu->execute()==false)
    {
      return null;
    }
    $sql = 'SELECT remoteid, processingstate, flags FROM Calls WHERE 1=1';
    if ($FromDate!=0) $sql .= ' AND startdate >= ? ';
    if ($ToDate!=0)   $sql .= ' AND startdate <= ? ';
    $sql .= ' ORDER BY remoteid ASC';
		$stmt = $this->db->prepare($sql);
    if ($stmt===FALSE)
    {
      return null;
    }
    $ix = 1;
    if ($FromDate!=0) { $stmt->bindValue($ix,$FromDate); $ix ++; }
    if ($ToDate  !=0) { $stmt->bindValue($ix,$ToDate);   $ix ++; }
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
      return null;
    }
		
		$fieldnames = array();
		$stmtfn = $dbbu->db->prepare("select column_name "
		. " from information_schema.columns "
		. " where table_schema='public' and table_name='calls' ");
		if ($stmtfn->execute()==false)
		{
			return null;
		}
		lg('MergeCallsTables: Get fieldnames.');
		while ($rowfn = $stmtfn->fetch(PDO::FETCH_ASSOC))
		{
			$fieldnames[] = $rowfn['column_name'];
		}

		//Do in a transction
		$this->StartTransaction();
		$number = 0;
		$first = true;
		lg('MergeCallsTables: Start merge.');
		while ($rowbu = $stmtbu->fetch(PDO::FETCH_ASSOC))
		{
			if(!$first)
			{
				if($rowbu['remoteid'] <= $row['remoteid'])
				{
					if($rowbu['remoteid'] < $row['remoteid'])
					{
						if($this->InsertCallsRecord($rowbu, $fieldnames, $dbbu, $CallBackNewCall))
						{
							$number++;	//Add record to database!
							if(!($number%100)) lg('1. Inserted record number: '.$number);
						}
					}
					else
					{
						$this->SetCallsFlags($row['remoteid'], $row['processingstate']
						, $row['flags'], $rowbu['processingstate'], $rowbu['flags']
						, $row['isarchived'], $rowbu['isarchived']
						);
					}
					continue;
				}
			}
			else $first = false;
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
			{
				if($rowbu['remoteid'] <= $row['remoteid'])
				{
					if($rowbu['remoteid'] < $row['remoteid'])
					{
						if($this->InsertCallsRecord($rowbu, $fieldnames, $dbbu, $CallBackNewCall))
						{
							$number++;	//Add record to database!
							if(!($number%100)) lg('2. Inserted record number: '.$number);
						}
					}
					else
					{
						$this->SetCallsFlags($row['remoteid'], $row['processingstate']
						, $row['flags'], $rowbu['processingstate'], $rowbu['flags']
						, $row['isarchived'], $rowbu['isarchived']
						);
					}
					continue 2;
				}
				else continue;
			}
			if(!$row)
			{
				if($this->InsertCallsRecord($rowbu, $fieldnames, $dbbu, $CallBackNewCall))
				{
					$number++;	//Add record to database!
					if(!($number%100)) lg('3. Inserted record number: '.$number);
				}
				continue;
			}
		}
		$this->CommitTransaction();
		return $number;
	}
  
  /*
  * @param DBInterface $ddbu 
  * @return int
  */
	function InsertCallsRecord($rowbu, $fieldnames, $dbbu, $CallBackNewCall)
	{
		$sqlkeys = "";
		$sqlvalues = "";
		$values = array();
		$first = true;

		//Get or create the localcallerid
		if ($rowbu['localcallerid'] != null)
		{
			$ret = $dbbu->GetCallerID($rowbu['localcallerid']);
			$callerid = $ret['callerid'];
			$callerid2 = $ret['callerid2'];
			$islocal = $ret['islocal'];
			$ret2 = $dbbu->GetContactInfo($rowbu['localcallerid']);
			$name = $ret2['name'];
			$ret3 = $this->GetOrCreateCallerID($callerid, $callerid2, $islocal, $name);
			$localcalleridnew = $ret3['id'];
			$localcontactidnew = $ret3['contactid'];
		}
		else
		{
			$localcalleridnew = null;
			$localcontactidnew = null;
		}
		
		//Get or create the remotecallerid
		$ret = $dbbu->GetCallerID($rowbu['remotecallerid']);
		$callerid = $ret['callerid'];
		$callerid2 = $ret['callerid2'];
		$islocal = $ret['islocal'];
		$ret2 = $dbbu->GetContactInfo($rowbu['remotecallerid']);
		$name = $ret2['name'];
		$ret3 = $this->GetOrCreateCallerID($callerid, $callerid2, $islocal, $name);
		$remotecalleridnew = $ret3['id'];
		$remotecontactidnew = $ret3['contactid'];

		//Get or create the connectedcallerid
		if ($rowbu['connectedcallerid'] != null)
		{
			$ret = $dbbu->GetCallerID($rowbu['connectedcallerid']);
			$callerid = $ret['callerid'];
			$callerid2 = $ret['callerid2'];
			$islocal = $ret['islocal'];
			$ret2 = $dbbu->GetContactInfo($rowbu['connectedcallerid']);
			$name = $ret2['name'];
			$ret3 = $this->GetOrCreateCallerID($callerid, $callerid2, $islocal, $name);
			$connectedcalleridnew = $ret3['id'];
			$connectedcontactidnew = $ret3['contactid'];
		}
		else
		{
			$connectedcalleridnew = null;
			$connectedcontactidnew = null;
		}

		//Get or create the recorder
		$recorderidnew = 0;
		if ($rowbu['recorderid'] != null)
		{
			$ret = $dbbu->GetRecorders();
			foreach ($ret as $recorder)
			{
				if ($recorder['id'] == $rowbu['recorderid'])
				{
					$recid = $recorder['recid'];
					$recname = $recorder['name'];
					$rectype = $recorder['rectype'];
					$recorderidnew = $this->GetOrCreateRecorderId($recid, $recname, $rectype);
					break;
				}
			}
		}
		
		//Get or create the line
		$lineidnew = 0;
		if ($rowbu['lineid'] > 0)
		{
			$linename = $dbbu->GetLineName($rowbu['lineid']);
			$ret4 = $this->GetOrCreateLineContact($rowbu['linenr'], $recorderidnew, $linename);
			$lineidnew = $ret4['id'];
			$linecontactidnew = $ret4['contactid'];
			if ($localcontactidnew == null)
			{
				$localcontactidnew = $linecontactidnew;
			}
		}
		
		// tenant?
		$tenantidnew = null;
		if (isset($rowbu['tenantid']) && $rowbu['tenantid']!==null)
		{
			// backup has tenant set
			// check if tenant has same name
			$tenantid = $rowbu['tenantid'];
			$buTenant = $dbbu->GetGroupInfo( $tenantid );
			$thisTenant = $this->GetGroupInfo( $tenantid );
			if ($buTenant !== false && $thisTenant !== false
			&& $buTenant['name'] == $thisTenant['name'])
			{
				$tenantidnew = $tenantid;
			}
    }
    
    if($rowbu['flags'] & 2048)
    {
      $callkey = $dbbu->GetCallKey( $rowbu['id'] );
      if($callkey)
      {
        $privKey = $this->FindCallEncryptionKeyOnPrivKey($callkey['privkey']);
        if(!$privKey)
        {
          $privKeybu = $dbbu->GetCallEncrytionKeyOnId($callkey['privid']);

          if($privKeybu)
          {
            $privid = $this->AddCallEncryptionKey([
              "accid" => $tenantidnew ? $tenantidnew : -1,
              "inuse" => 0,
              "pubkey" => $privKeybu['pubkey'],
              "privkey" => $privKeybu['privkey'],
              "iv" => $privKeybu['iv'],
              "salt" => $privKeybu['salt'],
              "deleted" => 1,
            ]);
          }
          else
          {
            lg("Encrypted call error: could not retrieve private key from backup");
          }
        }
        else
        {
          $privid = $privKey['id'];
        }

        if(isset($privid))
        {
          $callkeynew = [
            "keyid" => $privid,
            "callkey" => $callkey['callkey'],
            "iv" => $callkey['iv']
          ];
        }
      }
      else
      {
        lg("Encrypted call error: could not get callkey from backup database");
      }
    }
		
		foreach ($fieldnames as $fn)
		{
			if ($fn != 'id')
			{
				if ($fn == 'recyclebin' && $rowbu[$fn] == '') $rowbu[$fn] = '0';	//Recyclebin is new field. Cannot be '' must always be '0' after restore!
				if ($fn == 'processingstate') $rowbu[$fn] |= 8;										//Set 'to restore from backup' bit!
				if ($fn == 'localcallerid') $rowbu[$fn] = $localcalleridnew;
				if ($fn == 'remotecallerid') $rowbu[$fn] = $remotecalleridnew;
				if ($fn == 'connectedcallerid') $rowbu[$fn] = $connectedcalleridnew;
				if ($fn == 'recorderid') $rowbu[$fn] = $recorderidnew;
				if ($fn == 'lineid') $rowbu[$fn] = $lineidnew;
				if ($fn == 'isactive') $rowbu[$fn] = 0;														//Cannot be ""!
				if ($fn == 'localcontact') $rowbu[$fn] = $localcontactidnew;
				if ($fn == 'remotecontact') $rowbu[$fn] = $remotecontactidnew;
				if ($fn == 'connectedcontact') $rowbu[$fn] = $connectedcontactidnew;
				if ($fn == 'tenantid') $rowbu[$fn] = $tenantidnew;
				
				if ($rowbu[$fn] !== null)	//It's already null!
				{
					if($first)
					{
						$first = false;
						$sqlkeys = "(".$fn;
						$sqlvalues = "(?";
						$values[] = $rowbu[$fn];
					}
					else
					{
						$sqlkeys = $sqlkeys.",".$fn;
						$sqlvalues = $sqlvalues.",?";
						$values[] = $rowbu[$fn];
					}
				}
			}
		}
		
		$sql = "INSERT INTO Calls ".$sqlkeys.") VALUES ".$sqlvalues.")";
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
			$this->OnDbErr();
      return false;
    }
		$ix = 1;
		foreach ($values as $value)
		{
			$stmt->bindValue($ix,$value);
			$ix++;
		}
    if ($stmt->execute()==FALSE)
    {
			$this->OnDbErr();
      return false;
    }

    if(isset($callkeynew))
    {
      $callkeynew['callid'] = $this->db->lastInsertId('Calls_id_seq');
      $this->AddCallKey($callkeynew);
    }

		$CallBackNewCall($rowbu);
		return true;
	}
	
	function SetCallsFlags($remoteid, $procstate, $flags, $procstatebu, $flagsbu, $isarchived, $isarchivedbu)
	{
		$procstate |= 8;
		$autoDeletebit = ($flagsbu >> 2) & 1;
		if ($autoDeletebit) $flags |= 4;
		else $flags &= ~4;
		
		if ($isarchivedbu==1)
		{
			$isarchived = 1;
		}

    $stmt = $this->db->prepare('UPDATE Calls SET processingstate=:1, flags=:2, isarchived=:3 WHERE remoteid=:4');
    if (!$stmt)
    {
      return false;
    }
    $stmt->bindValue(1,$procstate);
    $stmt->bindValue(2,$flags);
		$stmt->bindValue(3,(int)$isarchived);
		$stmt->bindValue(4,$remoteid);
    $res = $stmt->execute();
	}
	
	function ToshibaSMDRUpdateNr($local, $remote, $direction, $node_id)
	{
		
		$this->StartTransaction();
		
		$min_lookback = 2;
		$last = time() - $min_lookback*60;
		
		$sql = 'SELECT id,remoteid,local,local2,callextra FROM calls 
						WHERE direction=:1 AND remote=:2 AND (startdate+duration)>=:3 AND flags&256=0 
						ORDER by startdate ASC LIMIT 1';
		
		$stmt = $this->db->prepare($sql);
				
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, (int) $direction);
		$stmt->bindValue(2,$remote);
		$stmt->bindValue(3,$last);
		$stmt->execute();
		
		$stmt->execute();
    $result = $stmt->fetch();
		
		if($result == false)
		{
			$this->CancelTransaction();
			// There is a possibility that the call has not been moved yet, so check for active calls as well
			return $this->ToshibaSMDRUpdateNrActive($local, $remote, $direction, $node_id);
		}
		
		$callextra = StringToArray($result['callextra']);
		$callextra['toshiba_nodeid'] = $node_id;
		$callextra_str = ArrayToString($callextra);
		
		$callerid = $this->GetOrCreateCallerID($local,$result['local2'],1);
	
		$sql = 'UPDATE calls SET local=:1,flags=flags|256,callextra=:2,localcontact=:3,localcallerid=:4,localfull=:5 WHERE id=:6';
		
		$stmt = $this->db->prepare($sql);
		
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, $local);
		$stmt->bindValue(2, $callextra_str);
		$stmt->bindValue(3, $callerid['contactid']);
		$stmt->bindValue(4, $callerid['id']);
		$stmt->bindValue(5, $local);
		$stmt->bindValue(6, $result['id']);
				
		$res = $stmt->execute();
		
		if(!$res)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$this->CommitTransaction();
		
		return $result;
	}
	
	function ToshibaSMDRUpdateConnected($local, $remote, $direction, $connected, $node_id)
	{	
		$this->StartTransaction();
		
		$min_lookback = 2;
		$last = time() - $min_lookback*60;
		
		$sql = 'SELECT id,callextra,remoteid FROM calls 
						WHERE direction=:1 AND remote=:2 AND local=:3 AND (startdate+duration)>=:4 AND flags&256>0';
			
		$stmt = $this->db->prepare($sql);
				
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, (int) $direction);
		$stmt->bindValue(2,$remote);
		$stmt->bindValue(3,$local);
		$stmt->bindValue(4,$last);
		
		$stmt->execute();
		
		$foundCall = false;
		
		while($result = $stmt->fetch())
		{
			$callextra = StringToArray($result['callextra']);
			if($callextra['toshiba_nodeid'] == $node_id)
			{
				$foundCall = true;
				break;
			}
		}
		
		if(!$foundCall)
		{
			$this->CancelTransaction();
			// There is a possibility that the call has not been moved yet, so check for active calls as well
			return $this->ToshibaSMDRUpdateConnectedActive($local, $remote, $direction, $connected, $node_id);
		}
		
		$callerid = $this->GetOrCreateCallerID($connected, $result['local2'], 1);
		
		$sql = 'UPDATE calls SET connected=:1,connectedcontact=:2,connectedcallerid=:3 WHERE id=:4';
		
		$stmt = $this->db->prepare($sql);
		
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, $connected);
		$stmt->bindValue(2, $callerid['contactid']);
		$stmt->bindValue(3, $callerid['id']);
		$stmt->bindValue(4, $result['id']);
		
		$res = $stmt->execute();
		
		if(!$res)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$this->CommitTransaction();
		
		return $result;
	}
	
	function ToshibaSMDRUpdateNrActive($local, $remote, $direction, $node_id)
	{
		$this->StartTransaction();
		
		$min_lookback = 2;
		$last = time() - $min_lookback*60;
		
		$sql = 'SELECT id,remoteid,local,callextra FROM activecalls 
						WHERE direction=:1 AND remote=:2 AND (startdate+duration)>=:3 AND flags&256=0 
						ORDER by startdate ASC LIMIT 1';
		
		$stmt = $this->db->prepare($sql);
				
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, (int) $direction);
		$stmt->bindValue(2,$remote);
		$stmt->bindValue(3,$last);
		$stmt->execute();
		
		$stmt->execute();
    $result = $stmt->fetch();
		
		if($result == false)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$callextra = StringToArray($result['callextra']);
		$callextra['toshiba_nodeid'] = $node_id;
		$callextra_str = ArrayToString($callextra);
		
		$callerid = $this->GetOrCreateCallerID($local,$result['local2'],1);
	
		$sql = 'UPDATE calls SET local=:1,flags=flags|256,callextra=:2,localcontact=:3,localcallerid=:4,localfull=:5 WHERE id=:6';
		
		$stmt = $this->db->prepare($sql);
		
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, $local);
		$stmt->bindValue(2, $callextra_str);
		$stmt->bindValue(3, $callerid['contactid']);
		$stmt->bindValue(4, $callerid['id']);
		$stmt->bindValue(5, $local);
		$stmt->bindValue(6, $result['id']);
				
		$res = $stmt->execute();
		
		if(!$res)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$this->CommitTransaction();
		return $result;
	}
	
	function ToshibaSMDRUpdateConnectedActive($local, $remote, $direction, $connected, $node_id)
	{		
		$this->StartTransaction();
		
		$min_lookback = 2;
		$last = time() - $min_lookback*60;
		
		$sql = 'SELECT id,callextra,remoteid FROM activecalls 
						WHERE direction=:1 AND remote=:2 AND local=:3 AND (startdate+duration)>=:4 AND flags&256>0';
			
		$stmt = $this->db->prepare($sql);
				
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, (int) $direction);
		$stmt->bindValue(2,$remote);
		$stmt->bindValue(3,$local);
		$stmt->bindValue(4,$last);
		
		$stmt->execute();
		
		$foundCall = false;
		
		while($result = $stmt->fetch())
		{
			$callextra = StringToArray($result['callextra']);
			if($callextra['toshiba_nodeid'] == $node_id)
			{
				$foundCall = true;
				break;
			}
		}
		
		if(!$foundCall)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$callerid = $this->GetOrCreateCallerID($connected, $result['local2'], 1);
		
		$sql = 'UPDATE calls SET connected=:1,connectedcontact=:2,connectedcallerid=:3 WHERE id=:4';
		
		$stmt = $this->db->prepare($sql);
		
		if(!$stmt)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$stmt->bindValue(1, $connected);
		$stmt->bindValue(2, $callerid['contactid']);
		$stmt->bindValue(3, $callerid['id']);
		$stmt->bindValue(4, $result['id']);
		
		$res = $stmt->execute();
		
		if(!$res)
		{
			$this->CancelTransaction();
			return false;
		}
		
		$this->CommitTransaction();
		
		return $result;
	}
  
  // LiveCalls
  
  function GetLiveCalls()
  {
    if (($this->actuinfo['permissions']&(PM_CONFIG))==0)
    {
      $this->lasterr = 'pm-denied';
      return false;
    }
    $result = array();
    $stmt = $this->db->prepare('SELECT * from LiveCalls ORDER BY id');
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
  }
  
  function AddLiveCall($remoteid,$data)
  {
    $sql = 'INSERT INTO LiveCalls (remoteid,'.implode(', ',array_keys($data)).') '
    . ' VALUES (?,'.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    $ix ++;
    $stmt->bindValue($ix,$remoteid);
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    if ($stmt->execute()==FALSE)
    {
      $this->OnDbErr();
    }
		return $this->db->lastInsertId('LiveCalls_id_seq');
  }
  
  function EditLiveCall($remoteid,$data)
  {
    $sql = 'UPDATE LiveCalls SET ';
    $first = true;
    foreach($data as $col=>$val)
    {
      if ($first) $first = false;
      else $sql .= ',';
      $sql .= ' ' . $col . ' = ? ';
    }
    $sql .= ' WHERE remoteid=?';

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix++;
    $stmt->bindValue($ix,$remoteid);
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }
  
  function DelLiveCall($remoteid)
  {
    $stmt = $this->db->prepare('DELETE FROM LiveCalls WHERE remoteid=?');
    $stmt->bindValue(1,$remoteid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }

	// Chat
	
	function UpdateChat($id, $data)
	{
		// update things in the database
		$sql = 'UPDATE Chat SET ';
		$ix = 0;
		foreach ($data as $key=>$value)
		{
			$ix ++;
			if ($ix!=1) $sql .= ',';
			$sql .= " $key=? ";
		}
		$sql .= ' WHERE id=?';
		$stmt = $this->db->prepare($sql);
		if(!$stmt) $this->onDbErr();
		$ix = 0;
		foreach($data as $value)
		{
			$ix ++;
			$stmt->bindValue($ix,$value);
		}
		$stmt->bindValue($ix+1,$id);
		if ($stmt->execute()==false) $this->onDbErr();
	}

	function SetChatMembers($chatid, $memberList)
	{
		$membercols = array('chat','member','membercallerid','membercontact');
		$sql = 'INSERT INTO ChatMembers('
		. implode(',',$membercols) .')'
		. ' VALUES ('.str_repeat('?,',sizeof($membercols)-1).'?)'
		. ' ON CONFLICT DO NOTHING ';
		$stmt = $this->db->prepare($sql);
		if(!$stmt) $this->onDbErr();
		foreach ($memberList as $member)
		{
			$member['chat'] = $chatid;
			$ix = 0;
			foreach($membercols as $col)
			{
				$ix ++;
				$stmt->bindValue($ix,$member[$col]);
			}
			if ($stmt->execute()==false) $this->onDbErr();
		}
	}
	
	function SetChat($chatdata, $memberList, $msgs)
	{
		//var_dump( $chatdata );
		
		$cols = array
		( 'chatid'
		, 'tenantid'
		, 'eid'
		, 'type'
		, 'topic'
		, 'chatid'
		, 'teamid'
		, 'channelid'
		, 'teamname'
		, 'channelname'
		);
		$dbdata = array();
		foreach($cols as $col)
		{
			if (isset($chatdata[$col]))
			{
				$dbdata[$col] = $chatdata[$col];
			}
		}
		$lastupdated = 0;
		foreach ($msgs as $msg)
		{
			if ($msg['created']!=null)
			{
				$lastupdated = max($lastupdated,$msg['created']);
			}
			if ($msg['modified']!=null)
			{
				$lastupdated = max($lastupdated,$msg['modified']);
			}
			if ($msg['deleted']!=null)
			{
				$lastupdated = max($lastupdated,$msg['deleted']);
			}
		}

		// get the id of the chat so we can reference it
		$sql = 'SELECT id FROM Chat WHERE eid=?';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
		$stmt->bindValue(1,$dbdata['eid']);
		if ($stmt->execute()==false) $this->onDbErr();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row===false)
		{
			if ($lastupdated!=0)
			{
				$dbdata['lastupdated'] = $lastupdated;
			}
			// it does not exist yet
			// so add it
			// created the chat if it did not yet exist
			$sql = 'INSERT INTO Chat('
			. implode(',',array_keys($dbdata)) .')'
			. ' VALUES ('.str_repeat('?,',sizeof($dbdata)-1).'?)';
			$stmt = $this->db->prepare($sql);
			if(!$stmt) $this->onDbErr();
			$ix = 0;
			foreach($dbdata as $value)
			{
				$ix ++;
				$stmt->bindValue($ix,$value);
			}
			if ($stmt->execute()==false) $this->onDbErr();
			$chatid = $this->db->lastInsertId('Chat_id_seq');
			
			$this->SetChatMembers($chatid,$memberList);
		} else
		{
			$chatid = $row['id'];
			if ($lastupdated!=0)
			{
				if ( $row['lastupdated']==null
				|| $lastupdated > $row['lastupdated'] )
				{
					$this->UpdateChat($row['id'],array('lastupdated'=>$lastupdated));
				}
			}
			// add new members
			$this->SetChatMembers($chatid,$memberList);
		}

		foreach ($msgs as $msg)
		{
			$action = $msg['action'];
			unset($msg['action']);
			// the message might already exist
			// if so, it needs to be updated, or deleted
			$sql = 'SELECT id FROM ChatMessages WHERE chat=? AND messageid=?';
			$stmt = $this->db->prepare($sql);
			if(!$stmt) $this->onDbErr();
			$stmt->bindValue(1,$chatid);
			$stmt->bindValue(2,$msg['messageid']);
			if ($stmt->execute()==false) $this->onDbErr();
			$row = $stmt->fetch(PDO::FETCH_ASSOC);
			$cols = array
			( 'sender'
			, 'sendercallerid'
			, 'sendercontact'
			, 'content'
			, 'created'
			, 'modified'
			, 'deleted' );
			// create chat message data for in database
			$msgdata = array();
			foreach ($cols as $col)
			{
				if (isset($msg[$col]))
				{
					$msgdata[$col] = $msg[$col];
				}
			}
			if ($row===false)
			{
				$msgdata['chat'] = $chatid;
				$msgdata['messageid'] = $msg['messageid'];
				// message not in database
				if ($action=='created' || $action=='updated')
				{
					$sql = 'INSERT INTO ChatMessages('
					. implode(',',array_keys($msgdata)) .')'
					. ' VALUES ('.str_repeat('?,',sizeof($msgdata)-1).'?)';
					$stmt = $this->db->prepare($sql);
					if(!$stmt) $this->onDbErr();
					$ix = 0;
					foreach($msgdata as $value)
					{
						$ix ++;
						$stmt->bindValue($ix,$value);
					}
					if ($stmt->execute()==false) $this->onDbErr();
				} else if ($action=='deleted')
				{
					// unknown message is deleted
					// nothing to do
				}
			} else
			{
				if ($action=='created'); // should not happen
				else if ($action=='updated'
				|| $action=='deleted')
				{
					// clear content if deleted
					// update things in the database
					$sql = 'UPDATE ChatMessages SET ';
					$ix = 0;
					foreach ($msgdata as $key=>$value)
					{
						$ix ++;
						if ($ix!=1) $sql .= ',';
						$sql .= " $key=? ";
					}
					$sql .= ' WHERE id=?';
					$stmt = $this->db->prepare($sql);
					if(!$stmt) $this->onDbErr();
					$ix = 0;
					foreach($msgdata as $value)
					{
						$ix ++;
						$stmt->bindValue($ix,$value);
					}
					$stmt->bindValue($ix+1,$row['id']);
					if ($stmt->execute()==false) $this->onDbErr();
				}
			}
		}
	}
	
	function GetChats($offset,$limit)
	{
		$sql = 'SELECT * FROM Chat ORDER BY lastupdated DESC '
		. ' LIMIT ' . (int) $limit . ' OFFSET ' . (int) $offset;
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
		if ($stmt->execute()==false) $this->onDbErr();
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
		return $result;
	}
	
	function GetChatMemberList($chatid, $maxmembers)
	{
		$sql = 'SELECT * FROM ChatMembers WHERE chat=?';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
		$stmt->bindValue(1,(int) $chatid);
		if ($stmt->execute()==false) $this->onDbErr();
		$result = array();
		$memcnt = 0;
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row['member'];
			$memcnt++;
			if ($memcnt > $maxmembers) break;
		}
		return $result;
	}
	
	function GetChatMesssages($options, $chatid)
	{
		$limit  = $options['limit'];
		$offset = $options['offset'];
		$sql = 'SELECT * FROM ChatMessages WHERE chat=? ORDER BY created DESC '
		. ' LIMIT ' . (int) $limit . ' OFFSET ' . (int) $offset;
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
		$stmt->bindValue(1,(int) $chatid);
		if ($stmt->execute()==false) $this->onDbErr();
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
		return $result;
		
	}
  
  // Retention policies
  
  function GetRetentionPolicies()
  {
    // get m all
    $sql = 'SELECT * FROM RetentionPolicies';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    if ($stmt->execute()==false) $this->onDbErr();
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
		return $result;
  }
  
  function IsRetentionPolicy($nr)
  {
    $sql = 'SELECT * FROM RetentionPolicies WHERE nr=?';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $stmt->bindValue(1,$nr);
    if ($stmt->execute()==false) $this->onDbErr();
    if ($stmt->fetch(PDO::FETCH_ASSOC)===false) return false;
    return true;
  }
  
  function AddRetentionPolicy($data)
  {
    $sql = 'INSERT INTO RetentionPolicies ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    if ($stmt->execute()==FALSE)
    {
      return false;
    }
    return true;
  }
  
  function DeleteRetentionPolicy($nr)
  {
    $sql = 'DELETE FROM RetentionPolicies WHERE nr = ? ';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $stmt->bindValue(1,$nr);
    if ($stmt->execute()==false) $this->onDbErr();
  }
  
  function ModifyRetentionPolicy($data)
  {
    $sql = 'UPDATE RetentionPolicies SET ';
    $ix = 0;
    $to_bind = [];
    foreach ($data as $key=>$value)
    {
      if ($key=='nr') continue;
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
      $to_bind[] = $value;
    }
    $ix ++;
    $sql .= " WHERE nr=:$ix";
    $to_bind[] = $data['nr'];

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($to_bind as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    if ($stmt->execute()==false) $this->onDbErr();
    return true;
  }  
  
  // Transcription plans
  
  function GetTScriptPlans()
  {
    // get m all
    $sql = 'SELECT * FROM TScriptPlans';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    if ($stmt->execute()==false) $this->onDbErr();
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
		return $result;
  }
  
  function AddTScriptPlan($data)
  {
    $sql = 'INSERT INTO TScriptPlans '
         . ' (name,enabled,maxdur,filter) '
         . ' VALUES(?,?,?,?) ';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $tobind = array
      ( $data['name'] 
      , $data['enabled']
      , $data['maxdur']
      , $data['filter']
      );
    $ix = 1;
    foreach ($tobind as $value)
    {
      $stmt->bindValue($ix,$value);
      $ix++;
    }
    if ($stmt->execute()==false) $this->onDbErr();
  }
  
  function ModifyTScriptPlan($id,$data)
  {
    $sql = 'UPDATE TScriptPlans SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE id=:$ix";

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$id);
    if ($stmt->execute()==false) $this->onDbErr();
    return true;
  }
  
  function EditTScriptPlan($id,$data)
  {
    $sql = 'UPDATE TScriptPlans '
         . ' SET name = ? '
         . ' , enabled = ? '
         . ' , maxdur = ? '
         . ' , filter = ? '
         . ' WHERE id = ? ';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $tobind = array
      ( $data['name'] 
      , $data['enabled']
      , $data['maxdur']
      , $data['filter']
      , $id 
      );
    $ix = 1;
    foreach ($tobind as $value)
    {
      $stmt->bindValue($ix,$value);
      $ix++;
    }
    if ($stmt->execute()==false) $this->onDbErr();
  }
  
  function DeleteTScriptPlan($id)
  {
    $sql = 'DELETE FROM TScriptPlans '
         . ' WHERE id = ? ';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $tobind = array
      ( $id 
      );
    $ix = 1;
    foreach ($tobind as $value)
    {
      $stmt->bindValue($ix,$value);
      $ix++;
    }
    if ($stmt->execute()==false) $this->onDbErr();
  }
	
  // Transcription queue
  
  function GetTScriptQ()
  {
    $sql = 'SELECT * FROM TScriptQ';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    if ($stmt->execute()==false) $this->onDbErr();
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
		return $result;    
  }
  
  function AddTScriptQ($data)
  {
    $sql = 'INSERT INTO TScriptQ ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    if ($stmt->execute()==false) $this->onDbErr();    
  }
  
  function DelTScriptQ($id)
  {
    $sql = 'DELETE FROM TScriptQ '
         . ' WHERE id = ? ';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==false) $this->onDbErr();
  }
  
  // Audit Trail

  function AddAuditTrailEntry($moment,$ip,$userid,$descr)
  {
    $stmt = $this->db->prepare('insert into AuditTrail '
    . ' (moment,ipaddr,userid,descr) '
    . ' VALUES '
    . ' (?,?,?,?) '
    );
    if(!$stmt) $this->onDbErr();
    $stmt->bindValue(1,$moment);
    $stmt->bindValue(2,$ip);
    $stmt->bindValue(3,$userid);
    $stmt->bindValue(4,$descr);
    $stmt->execute();
  }
  
  function GetAuditTrailEntries($search,$limit,$offset,$sortby)
  {
    $params = array();
    $sql = 'SELECT * FROM AuditTrail';
    $sql .= ' WHERE 1=1 ';
    if (isset($search['fd']))
    {
      $sql .= ' AND moment >= ? ';
      $params[] = (int) $search['fd'];
    }
    if (isset($search['ld']))
    {
      $sql .= ' AND moment <= ? ';
      $params[] = (int) $search['ld'];
    }
    if (isset($search['ft']))
    {
      if (DB_TYPE=='sqlite3')
      {
        $sql .= ' AND time(moment,\'unixepoch\',\'localtime\') >= ? ';
      } else
      {
        $sql .= ' AND to_timestamp(moment)::time >= ? ';
      }
      $params[] = $search['ft'];
    }
    if (isset($search['lt']))
    {
      if (DB_TYPE=='sqlite3')
      {
        $sql .= ' AND time(moment,\'unixepoch\',\'localtime\') <= ? ';
      } else
      {
        $sql .= ' AND to_timestamp(moment)::time <= ? ';
      }
      $params[] = $search['lt'];
    }
    if (isset($search['tenantid']))
    {
      // filter on all tenant users
      $users = $this->GetTenantUsers( $search['tenantid'] );
      $sql .= ' AND (';
      $first = true;
      foreach ($users as $user)
      {
        if ($first==false)
        {
          $sql .= ' OR ';
        }
        $first = false;
        $sql .= 'userid = ?';
        $params[] = $user['id'];
      }
      $sql .= ')';
    }
    if (isset($search['userid']))
    {
      $sql .= ' AND userid = ?';
      $params[] = $search['userid'];
    }
    if (isset($search['descr']))
    {
      $sql .= ' AND descr ' . $this->GetILike() . ' ? ';
      $params[] = '%' . $search['descr'] . '%';
    }
    $sql .= ' ORDER BY ' . $sortby['key'] . ' ' . ($sortby['ascending']?'ASC':'DESC');
    $sql .= ' , id DESC ';
    $sql .= ' LIMIT ?';
    $params[] = (int) $limit;
    $sql .= ' OFFSET ?';
    $params[] = (int) $offset;
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $ix = 1;
    foreach ($params as $param)
    {
      $stmt->bindValue($ix,$param);
      $ix++;
    }
    if ($stmt->execute()==false) $this->onDbErr();
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
		return $result;
  }
  
  function AutoTrailAutoDel($before)
  {
    $sql = 'DELETE FROM AuditTrail';
    $sql .= ' WHERE moment < ?';
    $stmt = $this->db->prepare($sql);
    if(!$stmt) $this->onDbErr();
    $stmt->bindValue(1,$before);
    if ($stmt->execute()==false) $this->onDbErr();

    return $stmt->rowCount();
  }
  
  // end of Audit Trail block
  // Begin CjAccounts
  
  function GetCjAccounts()
  {
    $result = array();
		$sql = 'SELECT * FROM cjaccounts ORDER BY id';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}	
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
    return $result;
  }
  
  function AddCjAccount($data)
  {
    $sql = 'INSERT INTO cjaccounts ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }
  
  function EditCjAccount($id, $data)
  {
    $sql = 'UPDATE cjaccounts SET ';
    $first = true;
    foreach($data as $col=>$val)
    {
      if ($first) $first = false;
      else $sql .= ',';
      $sql .= ' ' . $col . ' = ? ';
    }
    $sql .= ' WHERE id=?';
    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix++;
    $stmt->bindValue($ix,$id);
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }
  
  function DeleteCjAccount($id)
  {
    $stmt = $this->db->prepare('DELETE FROM CjAccounts WHERE id=?');
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }
  
  // End CjAccounts
  
  // Begin AzureApp
  
  function GetAzureApps()
  {
    $result = array();
		$sql = 'SELECT * FROM AzureApps ORDER BY id';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}	
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
    return $result;
  }  

  function GetAzureApp($id)
  {
    $result = array();
		$sql = 'SELECT * FROM AzureApps WHERE id=?';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}
    $stmt->bindValue(1,$id);
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		return $stmt->fetch(PDO::FETCH_ASSOC);
  }
  
  function AddAzureApp($data)
  {
    $sql = 'INSERT INTO AzureApps ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';

    $stmt = $this->db->prepare($sql);
    if ($stmt==false) $this->OnDbErr();
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
    return $this->db->lastInsertId('AzureApps_id_seq');
  }
  
  function EditAzureApp($id, $data)
  {
    $sql = 'UPDATE AzureApps SET ';
    $first = true;
    foreach($data as $col=>$val)
    {
      if ($first) $first = false;
      else $sql .= ',';
      $sql .= ' ' . $col . ' = ? ';
    }
    $sql .= ' WHERE id=?';
    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix++;
    $stmt->bindValue($ix,$id);
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }
  
  function DeleteAzureApp($id)
  {
    $stmt = $this->db->prepare('DELETE FROM AzureApps WHERE id=?');
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }
  
  // End AzureApp
  
  // Begin ExtIdP

  function GetExtIdPs()
  {
    $result = array();
		$sql = 'SELECT * FROM ExtIdP ORDER BY id';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}	
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
    return $result;
  }

  function GetExtIdP($id)
  {
    $result = array();
		$sql = 'SELECT * FROM ExtIdP WHERE id=?';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}
    $stmt->bindValue(1,$id);
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		return $stmt->fetch(PDO::FETCH_ASSOC);
  }
  
  function AddExtIdP($data)
  {
    $sql = 'INSERT INTO ExtIdP ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';

    $stmt = $this->db->prepare($sql);
    if ($stmt==false) $this->OnDbErr();
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
    return $this->db->lastInsertId('ExtIdP_id_seq');
  }

  function EditExtIdP($id, $data)
  {
    $sql = 'UPDATE ExtIdP SET ';
    $first = true;
    foreach($data as $col=>$val)
    {
      if ($first) $first = false;
      else $sql .= ',';
      $sql .= ' ' . $col . ' = ? ';
    }
    $sql .= ' WHERE id=?';
    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix++;
    $stmt->bindValue($ix,$id);
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }

  function DeleteExtIdP($id)
  {
    $stmt = $this->db->prepare('DELETE FROM ExtIdP WHERE id=?');
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }

  // End ExtIdP
  // Begin ReportSchedules
  
  function GetReportSchedules()
  {
    $result = array();
		$sql = 'SELECT * FROM ReportSchedules ORDER BY id';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}	
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		$result = array();
		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}
    return $result;
  }
  
  function AddReportSchedule($data)
  {
    $sql = 'INSERT INTO ReportSchedules ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';

    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
    return $this->db->lastInsertId('ReportSchedules_id_seq');
  }
  
  function EditReportSchedule($id,$data)
  {
    $sql = 'UPDATE ReportSchedules SET ';
    $first = true;
    foreach($data as $col=>$val)
    {
      if ($first) $first = false;
      else $sql .= ',';
      $sql .= ' ' . $col . ' = ? ';
    }
    $sql .= ' WHERE id=?';
    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix++;
    $stmt->bindValue($ix,$id);
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
  }
  
  function DelReportSchedule($id)
  {
    $stmt = $this->db->prepare('DELETE FROM ReportSchedules WHERE id=?');
    $stmt->bindValue(1,$id);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
  }
  
  // End ReportSchedules
  
  function GetTmpFile($filepath)
  {
    $result = array();
		$sql = 'SELECT * FROM TmpFiles WHERE filepath=?';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}
    $stmt->bindValue(1,$filepath);
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		return $stmt->fetch(PDO::FETCH_ASSOC);
  }
  
  function AddTmpFile($data)
  {
    $sql = 'INSERT INTO TmpFiles ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';

    $stmt = $this->db->prepare($sql);
    if ($stmt==false) $this->OnDbErr();
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }
    return $this->db->lastInsertId('AddTmpFile_id_seq');
  }
  
  function EditTmpFile($filepath,$data)
  {
    $sql = 'UPDATE TmpFiles SET ';
    $first = true;
    foreach($data as $col=>$val)
    {
      if ($first) $first = false;
      else $sql .= ',';
      $sql .= ' ' . $col . ' = ? ';
    }
    $sql .= ' WHERE filepath=?';
    $stmt = $this->db->prepare($sql);
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix++;
    $stmt->bindValue($ix,$filepath);
    $res = $stmt->execute();
    if ($res==false)
    {
      $this->OnDbErr();
    }    
  }
	
	function RemoveTmpFile($path)
	{
		$sql = 'DELETE FROM TmpFiles WHERE filepath=?';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}
    $stmt->bindValue(1,$path);
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
	}
  
  function DeleteExpiredTmpFiles($threshold)
  {
    $result = array();
		$sql = 'DELETE FROM TmpFiles WHERE expiry<=?';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}
    $stmt->bindValue(1,$threshold);
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
  }
  
  function GetExpiredTmpFiles($threshold)
  {
    $result = array();
		$sql = 'SELECT * FROM TmpFiles WHERE expiry<=?';
		$stmt = $this->db->prepare($sql);
		if(!$stmt)
		{
			$this->onDbErr();
		}
    $stmt->bindValue(1,$threshold);
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
    $result = array();
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      $result[] = $row;
    }
    return $result;
  }
  
  // V-Taps
  
	function GetVTaps()
	{
		$sql = 'SELECT vtap_id,password,tenantid,name FROM vtaps ORDER BY id';
		
		$stmt = $this->db->prepare($sql);
		
		if(!$stmt)
		{
			$this->onDbErr();
		}	
		if ($stmt->execute()==false)
		{
			$this->onDbErr();
		}
		
		$result = array();
		while($row = $stmt->fetch())
		{
			$result[] = $row;
		}
		
		return $result;
	}
	
	function UpdateOrCreateVTap($vtap)
	{
		$sql = 'SELECT id FROM vtaps WHERE vtap_id=:1';
		$stmt = $this->db->prepare($sql);
		
		if(!$stmt) $this->onDbErr();
		$stmt->bindValue(1, $vtap['vtap_id']);
		if($stmt->execute() == false) $this->onDbErr();
		$result = $stmt->fetch();
		
		if($result === false)
		{
			$sql = 'INSERT INTO vtaps(';
			
			$first = true;
			foreach($vtap as $key=>$value)
			{
				if($first === false) $sql .= ',';
				
				$first = false;
				
				$sql .= $key;
			}
			$sql .= ') VALUES(';
			
			$ix = 0;
			foreach($vtap as $value)
			{
				$ix++;
				if($ix != 1) $sql .= ',';
				$sql .= ":$ix";
			}
			$sql .= ');';
			
			$stmt = $this->db->prepare($sql);
			
			if(!$stmt) $this->onDbErr();
			
			$ix = 0;
			foreach($vtap as $value)
			{
				$ix++;
				$stmt->bindValue($ix, $value);
			}
			$stmt->execute();
		}
		else
		{
			$sql = 'UPDATE vtaps SET ';
			
			$ix = 0;
			foreach($vtap as $key=>$value)
			{
				$ix++;
				if($ix != 1) $sql .= ',';
				$sql .= "$key=:$ix";
			}
			
			
			$ix++;
			$sql .= " WHERE id=:$ix";
						
			$stmt = $this->db->prepare($sql);
			
			if(!$stmt) $this->onDbErr();
			
			$ix=0;
			foreach($vtap as $value)
			{
				$ix++;
				$stmt->bindValue($ix, $value);
			}
			$ix++;
			$stmt->bindValue($ix, $result['id']);
			
			$stmt->execute();
		}
	}
	
	function DeleteVTap($vtap_id)
	{
		$sql = 'DELETE FROM vtaps WHERE vtap_id=:1';
		
		$stmt = $this->db->prepare($sql);
		
		if(!$stmt) $this->onDbErr();
		
		$stmt->bindValue(1, $vtap_id);
		
		$stmt->execute();
	}
	
	function GetVTap($vtap_id)
	{
		$sql = 'SELECT id,vtap_id,tenantid,name FROM vtaps WHERE vtap_id=:1';
		
		$stmt = $this->db->prepare($sql);
		if(!$stmt) return false;
		$stmt->bindValue(1, $vtap_id);
		if($stmt->execute() === false) return false;
		$result = $stmt->fetch();
		return $result;
	}
	
	function UnassignVTapTenant($tenant_id)
	{
		$sql = 'UPDATE vtaps SET tenantid=NULL WHERE tenantid=:1';
		
		$stmt = $this->db->prepare($sql);
		if(!$stmt){
			$this->onDbErr();
		}
		
		$stmt->bindValue(1, $tenant_id);
		if($stmt->execute() == false)
		{
			$this->onDBErr();
		}
	}
	
	function GetTunnelCallsWithRecorder()
	{
		$rows = array();
		$result = $this->db->query(
			'SELECT calls.id, calls.callextra, recorders.recid 
			FROM calls INNER JOIN recorders ON calls.recorderid = recorders.id 
			WHERE (calls.flags & 128) > 0;');
		
		if($result === false) return $rows;
		while($row = $result->fetch())
		{
			$rows[] = $row;
		}
		return $rows;
	}
	
	function InsertCertificate($certificate)
	{
		$sql = 'INSERT INTO certificates(';
		$first = true;
		foreach($certificate as $key=>$value)
		{
			if($first === false) $sql .= ',';
				
			$first = false;
				
			$sql .= $key;
		}
		
		$sql .= ') VALUES(';
		
		$ix = 0;
		foreach($certificate as $value)
		{
			$ix++;
			if($ix != 1) $sql .= ',';
			$sql .= ":$ix";
		}
		$sql .= ');';
		
		$stmt = $this->db->prepare($sql);
			
		if(!$stmt) $this->onDbErr();
		
		$ix = 0;
		foreach($certificate as $value)
		{
			$ix++;
			$stmt->bindValue($ix, $value);
		}
		
		$stmt->execute();
		
		return $this->db->lastInsertId('Certificates_id_seq');
	}
	
	function GetCertificates()
	{
		$result = array();
    $sql = 'SELECT id, name, subject, valid_to, privatekey, trusted FROM certificates';
		$res = $this->db->query($sql);
		
		if($res === false) return $result;
		
    foreach($res as $row)
    {
			$result[] = $row;
		}	
		return $result;
  }
	
	function GetCertificateFiles()
  {
    $result = array();
    $sql = 'SELECT cert_location, key_location, intermediate_location, trusted, webkey FROM certificates';
		$res = $this->db->query($sql);
		
		if($res === false) return $result;
		
    foreach($res as $row)
    {
			$result[] = $row;
		}
		
		return $result;
  }
	
	
	function GetCertificate($id)
	{
		$stmt = $this->db->prepare('SELECT * FROM certificates WHERE id=:1');
		$stmt->bindValue(1,$id);
    $stmt->execute();
		return $stmt->fetch(PDO::FETCH_ASSOC);
	}
	
	function DeleteCertificate($id)
	{
		$stmt = $this->db->prepare('DELETE FROM certificates WHERE id=:1');
		$stmt->bindValue(1,$id);
    return $stmt->execute();
	}
	
	function UpdateCertificate($id, $cert)
	{
		$sql = 'UPDATE certificates SET ';
		$ix = 0;
		foreach($cert as $key=>$value)
		{
			$ix++;
			if($ix != 1) $sql .= ',';
			$sql .= "$key=:$ix";
		}
		
		$ix++;
		$sql .= " WHERE id=:$ix";
						
		$stmt = $this->db->prepare($sql);
			
		if(!$stmt) $this->onDbErr();
			
		$ix=0;
		foreach($cert as $value)
		{
			$ix++;
			$stmt->bindValue($ix, $value);
		}
		$ix++;
		$stmt->bindValue($ix, $id);
			
		$stmt->execute();
	}
	
	function GetCertificatesWithPrivateKey()
	{
		$result = array();
    $sql = 'SELECT id, name, subject FROM certificates WHERE privatekey=1';
    foreach($this->db->query($sql) as $row)
    {
			$result[] = $row;
		}
		return $result;
	}
	
	function SetCertificateUse($use, $certificateid)
	{
		$sql = 'SELECT * FROM certificateuse WHERE usedfor=:1';
		
		$stmt = $this->db->prepare($sql);
		
		if($stmt === false) $this->onDbErr();
		
		$stmt->bindValue(1, $use);
		$stmt->execute();
		$result = $stmt->fetch();
		
		if($result === false)
		{
			$sql = 'INSERT INTO certificateuse(certificateid, usedfor) VALUES(:1,:2)';
			$stmt = $this->db->prepare($sql);
		
			if($stmt === false) $this->onDbErr();
			
			$stmt->bindValue(1, $certificateid);
			$stmt->bindValue(2, $use);
			
			if($stmt->execute() === false) $this->onDBErr();
			
			return true;
		}
		else if($result['certificateid'] != $certificateid)
		{
			$sql = 'UPDATE certificateuse SET certificateid=:1 WHERE usedfor=:2';
			$stmt = $this->db->prepare($sql);
			
			if($stmt === false) $this->onDbErr();
			
			$stmt->bindValue(1, $certificateid);
			$stmt->bindValue(2, $use);
			
			if($stmt->execute() === false) $this->onDBErr();
			
			return true;
		}
		
		return false;
	}
	
	function DeleteCertificateUse($use)
	{
		$sql = 'DELETE FROM certificateuse WHERE usedfor=:1';
		$stmt = $this->db->prepare($sql);
		if($stmt === false) $this->onDbErr();
		$stmt->bindValue(1, $use);
		if($stmt->execute() === false) $this->onDbErr();
 	}
	
	function GetCertificateFor($use)
	{
		$sql = 'SELECT certificateid FROM certificateuse WHERE usedfor=:1';
		$stmt = $this->db->prepare($sql);
		if($stmt === false) return $this->onDbErr();
		$stmt->bindValue(1, $use);
		$stmt->execute();
		$result = $stmt->fetch();
		if($result === false) return false;
		return $result['certificateid'];
	}
	
	function GetCertificateUses($certificateid)
	{
		$result = array();
		
		$sql = 'SELECT usedfor FROM certificateuse WHERE certificateid=:1';
		$stmt = $this->db->prepare($sql);
		if($stmt == false) return $this->onDbErr();
		
		$stmt->bindValue(1, $certificateid);
		$stmt->execute();
		
		while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
		
		return $result;
	}
	
	function CertificateInUse($certificateid)
	{
		return sizeof($this->GetCertificateUses($certificateid)) > 0;
	}

	function InsertCertificateRequest($certificate)
	{
		$sql = 'INSERT INTO certificaterequests(';
		$first = true;
		foreach($certificate as $key=>$value)
		{
			if($first === false) $sql .= ',';
				
			$first = false;
				
			$sql .= $key;
		}
		
		$sql .= ') VALUES(';
		
		$ix = 0;
		foreach($certificate as $value)
		{
			$ix++;
			if($ix != 1) $sql .= ',';
			$sql .= ":$ix";
		}
		$sql .= ');';
		
		$stmt = $this->db->prepare($sql);
			
		if(!$stmt) $this->onDbErr();
		
		$ix = 0;
		foreach($certificate as $value)
		{
			$ix++;
			$stmt->bindValue($ix, $value);
		}
		
		$stmt->execute();
		
		return $this->db->lastInsertId('certificaterequests_id_seq');
  }
	
	function GetCertificateRequests()
	{
		$result = array();
    $sql = 'SELECT id, name, subject FROM certificaterequests';
		$res = $this->db->query($sql);
		
		if($res === false) return $result;
		
    foreach($res as $row)
    {
			$result[] = $row;
		}
		
		return $result;
	}
	
	function GetCertificateRequestFiles()
  {
    $result = array();
    $sql = 'SELECT request_location, key_location FROM certificaterequests';
		$res = $this->db->query($sql);
		
		if($res === false) return $result;
		
    foreach($res as $row)
    {
			$result[] = $row;
		}
		
		return $result;
  }
	
	function GetCertificateRequest($id)
	{
		$stmt = $this->db->prepare('SELECT * FROM certificaterequests WHERE id=:1');
		$stmt->bindValue(1,$id);
    $stmt->execute();
		return $stmt->fetch(PDO::FETCH_ASSOC);
	}
	
	function DeleteCertificateRequest($id)
	{
		$stmt = $this->db->prepare('DELETE FROM certificaterequests WHERE id=:1');
		$stmt->bindValue(1,$id);
    return $stmt->execute();
	}
	
	function UpdateCertificateRequest($id, $cert)
	{
		$sql = 'UPDATE certificaterequests SET ';
		$ix = 0;
		foreach($cert as $key=>$value)
		{
			$ix++;
			if($ix != 1) $sql .= ',';
			$sql .= "$key=:$ix";
		}
		
		$ix++;
		$sql .= " WHERE id=:$ix";
						
		$stmt = $this->db->prepare($sql);
			
		if(!$stmt) $this->onDbErr();
			
		$ix=0;
		foreach($cert as $value)
		{
			$ix++;
			$stmt->bindValue($ix, $value);
		}
		$ix++;
		$stmt->bindValue($ix, $id);
			
		$stmt->execute();
  }
  
  function GetExpiringLetsEncryptCerts($days)
  {
		$result = array();

		$stmt = $this->db->prepare("SELECT id, name FROM certificates where lets_encrypt=1 and DATE_PART('day',TO_TIMESTAMP(valid_to) - NOW()) < :1");
		
		if(!$stmt) return $result;

		$stmt->bindValue(1, $days);
		$res = $stmt->execute();

		if($res === false) return $result;

		while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
		
		return $result;
  }

  function GetExpringCerts($days, $WithPrivKey, $WithoutPrivKey)
  {
    $result = [];
    
    if(!$WithPrivKey  && !$WithoutPrivKey)
    {
      return $result;
    }

    $sql = "SELECT id, name, valid_to FROM certificates where DATE_PART('day',TO_TIMESTAMP(valid_to) - NOW()) < :1";
    
    if($WithPrivKey && !$WithoutPrivKey)
    {
      $sql .= ' and privatekey=1';
    }
    else if(!$WithPrivKey && $WithoutPrivKey)
    {
      $sql .= ' and (privatekey=0 or privatekey IS NULL)';
    }

    $stmt = $this->db->prepare($sql);

    if(!$stmt) return $result;

    $stmt->bindValue(1, $days);
		$res = $stmt->execute();
		if($res === false) return $result;

    while ($row = $stmt->fetch())
    {
      $result[] = $row;
    }
		
		return $result;
  }
  
  function UpdateWebClientSettings($accid, $settings)
  {
    $stmt = $this->db->prepare('SELECT * from webclientsettings WHERE accid=:1');
    
    if(!$stmt) $this->OnDbErr();
    
    $stmt->bindValue(1, $accid);
   
    $stmt->execute();

    if(!$stmt) $this->OnDbErr();

    $exists = $stmt->fetch(PDO::FETCH_ASSOC);

    if(!$exists)
    {
      $stmt2 = $this->db->prepare('INSERT INTO webclientsettings (accid, settings) VALUES (:1, :2)');
      if(!$stmt2) $this->OnDbErr();
      $stmt2->bindValue(1, $accid);
      $stmt2->bindValue(2, $settings);
      $stmt2->execute();
      if(!$stmt2) $this->OnDbErr();
    }
    else
    {
      $stmt2 = $this->db->prepare('UPDATE webclientsettings SET settings=:1 WHERE accid=:2');
      if(!$stmt2) $this->OnDbErr();
      $stmt2->bindValue(1, $settings);
      $stmt2->bindValue(2, $accid);
      $stmt2->execute();
      if(!$stmt2) $this->OnDbErr();
    }
  }

  function GetWebClientSettings($accid)
  {
    $stmt = $this->db->prepare('SELECT * FROM webclientsettings WHERE accid=:1');
    if(!$stmt) $this->onDbErr();
    $stmt->bindValue(1, $accid);
    $stmt->execute();
    if(!$stmt) $this->onDbErr();
    $row = $stmt->fetch(PDO::FETCH_ASSOC);
    if($row === false) return false;
    else return $row['settings'];
  }

  function DeleteWebClientSettings($accid)
  {
    $stmt = $this->db->prepare('DELETE FROM webclientsettings WHERE accid=:1');
    if(!$stmt) $this->onDbErr();
    $stmt->bindValue(1, $accid);
    $stmt->execute();
    if(!$stmt) $this->onDbErr();
  }

  	function GetWebClientSettingsUser($accid)
	{
		$sql = 'SELECT webclientsettings FROM Users where accid=:1';
		$stmt = $this->db->prepare($sql);
		$stmt->bindValue(1,$accid);
		$stmt->execute();
		$result = $stmt->fetch();
		
		if($result == false)
		{
			return false;
		}
		
		return $result['webclientsettings'] ? $result['webclientsettings'] : false;
	}
	
	function SetWebClientSettingsUser($accid, $settings)
	{
    $sql = 'UPDATE Users SET webclientsettings=:1 WHERE accid=:2';
    $stmt = $this->db->prepare($sql);
    if(!$stmt)
    {
      return false;
    }
    $stmt->bindValue(1,(int) $settings);
    $stmt->bindValue(2, $accid);
    $res = $stmt->execute();
    return $res;
  }

  function GetCallEncryptionKey($accid)
  {
    $sql = 'SELECT * FROM callencryptionkeys where accid=:1 and deleted=0';
		$stmt = $this->db->prepare($sql);
		$stmt->bindValue(1,$accid);
    $stmt->execute();
    $result = $stmt->fetch();

    return $result;
  }

  function GetCallEncrytionKeyOnId($id)
  {
    $sql = 'SELECT * FROM callencryptionkeys WHERE id=:1';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1, $id);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
  }

  function FindCallEncryptionKeyOnPrivKey($privKey)
  {
    $sql = 'SELECT * from callencryptionkeys WHERE privKey=:1';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1, $privKey);
    $stmt->execute();
    $result = $stmt->fetch();
    return $result;
  }

  function UpdateCallEncryptionKey($accid, $conf)
  {
    $sql = 'UPDATE CallEncryptionKeys SET ';
    $ix = 0;
    foreach($conf as $key=>$value)
    {
      $ix++;
      if($ix != 1) $sql .= ',';
      $sql .= "$key=:$ix";
    }
    
    $ix++;
    $sql .= " WHERE accid=:$ix and deleted=0";
            
    $stmt = $this->db->prepare($sql);
      
    if(!$stmt) $this->onDbErr();
      
    $ix=0;
    foreach($conf as $value)
    {
      $ix++;
      $stmt->bindValue($ix, $value);
    }
    $ix++;
    $stmt->bindValue($ix, $accid);
      
    $stmt->execute(); 
  }

  function AddCallEncryptionKey($newKey)
  {
    $sql = 'INSERT INTO CallEncryptionKeys(';
		$first = true;
		foreach($newKey as $key=>$value)
		{
			if($first === false) $sql .= ',';
				
			$first = false;
				
			$sql .= $key;
    }
    
		$sql .= ') VALUES(';
		
		$ix = 0;
		foreach($newKey as $value)
		{
			$ix++;
			if($ix != 1) $sql .= ',';
			$sql .= ":$ix";
		}
    $sql .= ');';
		$stmt = $this->db->prepare($sql);
			
		if(!$stmt) $this->onDbErr();
		
		$ix = 0;
		foreach($newKey as $value)
		{
			$ix++;
			$stmt->bindValue($ix, $value);
		}
		
    $stmt->execute();
    return $this->db->lastInsertId('callencryptionkeys_id_seq');
  }

  function AddActiveCallKey($newKey)
  {
    $sql = 'INSERT INTO ActiveCallKeys(';
		$first = true;
		foreach($newKey as $key=>$value)
		{
			if($first === false) $sql .= ',';
				
			$first = false;
				
			$sql .= $key;
    }
    
		$sql .= ') VALUES(';
		
		$ix = 0;
		foreach($newKey as $value)
		{
			$ix++;
			if($ix != 1) $sql .= ',';
			$sql .= ":$ix";
		}
    $sql .= ');';
		$stmt = $this->db->prepare($sql);
			
		if(!$stmt) $this->onDbErr();
		
		$ix = 0;
		foreach($newKey as $value)
		{
			$ix++;
			$stmt->bindValue($ix, $value);
		}
		
		$stmt->execute();
  }
  
  function AddCallKey($newKey)
  {
    $sql = 'INSERT INTO CallKeys(';
		$first = true;
		foreach($newKey as $key=>$value)
		{
			if($first === false) $sql .= ',';
				
			$first = false;
				
			$sql .= $key;
    }
    
		$sql .= ') VALUES(';
		
		$ix = 0;
		foreach($newKey as $value)
		{
			$ix++;
			if($ix != 1) $sql .= ',';
			$sql .= ":$ix";
		}
    $sql .= ');';
		$stmt = $this->db->prepare($sql);
			
		if(!$stmt) $this->onDbErr();
		
		$ix = 0;
		foreach($newKey as $value)
		{
			$ix++;
			$stmt->bindValue($ix, $value);
		}
		
		$stmt->execute();
  }

  function MoveActiveCallKey($call_id, $old_id)
  {
		$callKey = $this->GetActiveCallKeyBasic($old_id);
		if($callKey === false) return true;
		$sql = 'INSERT INTO CallKeys '
		. ' (callid, keyid, callkey, iv) '
		. ' VALUES (?,?,?,?) ' ;
		$stmt = $this->db->prepare($sql);
		if(!$stmt) return false;
    $stmt->bindValue(1,$call_id);
    $stmt->bindValue(2,$callKey['keyid']);
    $stmt->bindValue(3,$callKey['callkey']);
    $stmt->bindValue(4,$callKey['iv']);
    if($stmt->execute()==false)
    {
      return false;
    }
    return true;    
  }
  
  function GetActiveCallKeyBasic($callid)
  {
    $sql = 'SELECT * FROM ActiveCallKeys WHERE callid=:1';
    $stmt = $this->db->prepare($sql);
    $stmt->bindValue(1,$callid);
    $stmt->execute();
    $result = $stmt->fetch();

    return $result;
  }
  
  function GetCallKey($callid)
  {
    $sql = 'SELECT callkeys.id, callkeys.callkey, callkeys.iv, 
      callencryptionkeys.id as privid, callencryptionkeys.privKey, callencryptionkeys.salt, callencryptionkeys.accid, callencryptionkeys.iv as keyiv
      FROM callkeys INNER JOIN callencryptionkeys ON callkeys.keyid = callencryptionkeys.id WHERE callid=:1';

    $stmt = $this->db->prepare($sql);

    $stmt->bindValue(1,$callid);
    $stmt->execute();
    $result = $stmt->fetch();

    return $result;
  }

  function RemoveCallKey($id)
  {
    $sql = 'DELETE FROM callkeys WHERE id=:1';
    $stmt = $this->db->prepare($sql);

    $stmt->bindValue(1, $id);
    $stmt->execute();
  }

  function GetCallerIDSearchIDs($filter, $tobind)
  {
    $sqlstmt = 'SELECT id FROM callerids WHERE ' . $filter;
    $stmt = $this->db->prepare($sqlstmt);
    if(!$stmt)
    {
      $this->OnDbErr();
    }

    $ix = 0;
    foreach($tobind as $bind)
    {
      $ix++;
      $stmt->bindValue($ix, $bind);
    }

    if($stmt->execute() == false)
    {
      $this->OnDbErr();
    }

    $results = [];
    while($row = $stmt->fetch())
    {
      $results[] = $row['id'];
    }
    
    return $results;
  }

  function GetContactIDSearchIDs($searchitem)
  {
    if ($this->UseTenantContactList())
    {
      $tenantid = (int)$this->actuinfo['tenantid'];
      $sqlstmt = 'SELECT contactid as id FROM tenantcontacts WHERE (name '. $this->GetILike(). " :1 " . $this->GetEscape() . ") AND tenantid=:2";
    } else
    {
      $sqlstmt = 'SELECT id FROM contacts WHERE (name '. $this->GetILike(). " :1 " . $this->GetEscape() . ")";
    }

    $stmt = $this->db->prepare($sqlstmt);
    if(!$stmt)
    {
      $this->OnDbErr();
    }

    $stmt->bindValue(1, $searchitem);
    if(isset($tenantid))
    {
      $stmt->bindValue(2, $tenantid);
    }

    if($stmt->execute() == false)
    {
      $this->OnDbErr();
    }

    $results = [];
    while($row = $stmt->fetch())
    {
      $results[] = $row['id'];
    }

    return $results;
  }

  function InsertScimData($data)
  {
    $sql = 'INSERT INTO scimdata ('.implode(', ',array_keys($data)).') '
    . ' VALUES ('.str_repeat('?,',sizeof($data)-1).'?) ';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    if ($stmt->execute()==FALSE)
    {
      $this->lasterr = $stmt->errorInfo()[2];
      return false;
    }

		return true;
  }

  function GetScimUser($id, $scimparent)
  {
    $sql = 'SELECT u.id, u.enabled, u.usr, u.accid, u.name, u.extra, u.email, s.created, s.updated, s.externalid, s.scimparent
		FROM users AS u INNER JOIN scimdata AS s ON u.accid = s.accid 
		WHERE u.id = :1 AND s.scimparent = :2';

    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$id);
		$stmt->bindValue(2,$scimparent);

    if ($stmt->execute()==FALSE)
    {
      return false;
    }

    if($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      return $row;
    }

    return false;
  }

	function GetScimUserIdFromAccid($accid)
	{
		$sql = 'SELECT u.id FROM users AS u INNER JOIN scimdata AS s ON u.accid = s.accid WHERE u.accid = :1';
    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$accid);

    if ($stmt->execute()==FALSE)
    {
      return false;
    }

    if($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      return $row["id"];
    }

    return false;
	}

	function EditScimData($accid, $data)
	{
    $sql = 'UPDATE scimdata SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE accid=:$ix";

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
    {
      return false;
    }
    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      $stmt->bindValue($ix,$value);
    }
    $ix ++;
    $stmt->bindValue($ix,$accid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }
    return true;		
	}

  function GetScimTotalUsers($scimparent, $filter=null)
  {
    $sql = 'SELECT count(u.id)
    FROM users AS u INNER JOIN scimdata AS s ON u.accid = s.accid WHERE s.scimparent=?';

    if(isset($filter["filter"]))
    {
      $sql .= " AND (" . $filter["filter"] . ")";
    }

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
		{
			return false;
		}

		$ix = 1;
		$stmt->bindValue($ix, $scimparent);

    if(isset($filter["toBind"]))
    {
      foreach($filter["toBind"] as $bindVal)
      {
        $ix++;
        $stmt->bindValue($ix, $bindVal);
      }
    }

		if ($stmt->execute()==FALSE)
		{
			return false;
		}

    while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			return $row['count'];
		}

		return 0;
  }

	function GetScimUsers($scimparent, $filter = null)
	{
		$sql = 'SELECT u.id, u.enabled, u.usr, u.accid, 
			u.name, u.extra, u.email, s.created, s.updated, s.externalid, s.scimparent
			FROM users AS u INNER JOIN scimdata AS s ON u.accid = s.accid WHERE s.scimparent=?';

    if(isset($filter["filter"]))
    {
      $sql .= " AND (" . $filter["filter"] . ")";
    }
        
		$sql .= " ORDER BY u.id";

    if(isset($filter['count']))
    {
      $sql .= ' LIMIT ' . $filter['count'];
      if(isset($filter['startIndex']))
      {
        $sql .= ' OFFSET ' . $filter['startIndex'];
      }
    }
    
		$stmt = $this->db->prepare($sql);
		if (!$stmt)
		{
			return false;
		}

		$ix = 1;
		$stmt->bindValue($ix, $scimparent);

    if(isset($filter["toBind"]))
    {
      foreach($filter["toBind"] as $bindVal)
      {
        $ix++;
        $stmt->bindValue($ix, $bindVal);
      }
    }

		if ($stmt->execute()==FALSE)
		{
			return false;
		}

		$result = [];

		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}

		return $result;
	}

	function GetScimTotalGroups($scimparent, $filter = null)
  {
    $sql = 'SELECT count(g.id)
    FROM groups AS g INNER JOIN scimdata AS s ON g.accid = s.accid
		WHERE s.scimparent=?';

    if(isset($filter["filter"]))
    {
      $sql .= " AND (" . $filter["filter"] . ")";
    }

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
		{
			return false;
		}

		$ix = 1;
		$stmt->bindValue($ix, $scimparent);

    if(isset($filter["toBind"]))
    {
      foreach($filter["toBind"] as $bindVal)
      {
        $ix++;
        $stmt->bindValue($ix, $bindVal);
      }
    }

		if ($stmt->execute()==FALSE)
		{
			return false;
		}

    while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			return $row['count'];
		}

		return 0;
  }

	function GetScimGroups($scimparent, $filter = null)
	{
		$sql = 'SELECT g.id, g.name, g.accid, s.created, s.updated, s.externalid 
			FROM groups AS g INNER JOIN scimdata AS s ON g.accid = s.accid
			WHERE s.scimparent=?';

    if(isset($filter["filter"]))
    {
      $sql .= " AND (" . $filter["filter"] . ")";
    }
        
		$sql .= " ORDER BY g.id";

    if(isset($filter['count']))
    {
      $sql .= ' LIMIT ' . $filter['count'];
      if(isset($filter['startIndex']))
      {
        $sql .= ' OFFSET ' . $filter['startIndex'];
      }
    }
    
		$stmt = $this->db->prepare($sql);
		if (!$stmt)
		{
			return false;
		}

		$ix = 1;
		$stmt->bindValue($ix, $scimparent);
    if(isset($filter["toBind"]))
    {
      foreach($filter["toBind"] as $bindVal)
      {
        $ix++;
        $stmt->bindValue($ix, $bindVal);
      }
    }

		if ($stmt->execute()==FALSE)
		{
			return false;
		}

		$result = [];

		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}

		return $result;
	}

	function GetScimGroup($id, $scimparent)
  {
    $sql = 'SELECT g.id, g.name, g.accid, s.created, s.updated, s.externalid, s.scimparent
		FROM groups AS g INNER JOIN scimdata AS s ON g.accid = s.accid 
		WHERE g.id = :1 AND s.scimparent = :2';

    $stmt = $this->db->prepare($sql);
    if (!$stmt)
    {
      $this->OnDbErr();
    }
    $stmt->bindValue(1,$id);
		$stmt->bindValue(2,$scimparent);

    if ($stmt->execute()==FALSE)
    {
      return false;
    }

    if($row = $stmt->fetch(PDO::FETCH_ASSOC))
    {
      return $row;
    }

    return false;
  }

  function GetScimGroupMembers($groupaccid, $filter)
  {
    $sql = 'SELECT u.id, u.accid, u.usr 
    FROM groupmembers AS g INNER JOIN users AS u ON u.accid = g.accid 
    WHERE g.grpid=?';

    if(isset($filter["filter"]))
    {
      $sql .= " AND (" . $filter["filter"] . ")";
    }

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
		{
			return false;
		}

    $stmt->bindValue(1, $groupaccid);

    if(isset($filter["toBind"]))
    {
      $ix = 1;
      foreach($filter["toBind"] as $bindVal)
      {
        $ix++;
        $stmt->bindValue($ix, $bindVal);
      }
    }
    
    if ($stmt->execute()==FALSE)
    {
      return false;
    }

    $result = [];

		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = $row;
		}

		return $result;
  }

  function GetScimUserPhones($useraccid, $filter=null)
  {
    $sql = 'SELECT u.id as userphoneid, c.id, cd.callerid, s.is_primary, s.type FROM userphones AS u
    INNER JOIN scimuserphones AS s ON u.id = s.userphoneid 
    INNER JOIN contacts AS c ON u.contactid = c.id 
    INNER JOIN callerids AS cd ON cd.contactid = c.id 
    WHERE accid=?';

    if(isset($filter["filter"]))
    {
      $sql .= " AND (" . $filter["filter"] . ")";
    }

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
		{
			return false;
		}

    $stmt->bindValue(1, $useraccid);

    if(isset($filter["toBind"]))
    {
      $ix = 1;
      foreach($filter["toBind"] as $bindVal)
      {
        $ix++;
        if(is_bool($bindVal))
        {
          $stmt->bindValue($ix,$bindVal,PDO::PARAM_BOOL);
        }
        else
        {
          $stmt->bindValue($ix,$bindVal);
        }
      }
    }
    
    if ($stmt->execute()==FALSE)
    {
      return false;
    }

    $result = [];

		while($row = $stmt->fetch(PDO::FETCH_ASSOC))
		{
			$result[] = [
        "contactid" => $row["id"],
        "callerid" => $row["callerid"],
        "userphoneid" => $row["userphoneid"],
        "scim_data" => [
          "is_primary" => $row["is_primary"],
          "type" => $row["type"]
        ]
      ];
		}

		return $result;
  }

  function GroupExists($name)
  {
    $stmt = $this->db->prepare('SELECT id FROM Groups WHERE name=:1');
    $stmt->bindValue(1,$name);
    $stmt->execute();
    $row = $stmt->fetch();
    if ($row===FALSE)
    {
      return false;
    }
    return true;
  }

  function UpdateScimUserPhone($userphoneid, $data)
  {
    $sql = 'UPDATE ScimUserPhones SET ';
    $ix = 0;
    foreach ($data as $key=>$value)
    {
      $ix ++;
      if ($ix!=1) $sql .= ',';
      $sql .= " $key=:$ix ";
    }
    $ix ++;
    $sql .= " WHERE userphoneid=:$ix";

    $stmt = $this->db->prepare($sql);
		if (!$stmt)
    {
      return false;
    }

    $ix = 0;
    foreach($data as $value)
    {
      $ix ++;
      if(is_bool($value))
      {
        $stmt->bindValue($ix,$value,PDO::PARAM_BOOL);
      }
      else
      {
        $stmt->bindValue($ix,$value);
      }
    }

    $ix ++;
    $stmt->bindValue($ix,$userphoneid);
    if ($stmt->execute()==false)
    {
      $this->OnDbErr();
    }

    return true;
  }

  function RemoveNonScimUserPhones($accid, $data)
  {
    if(count($data) == 0) return;

    $query='DELETE FROM UserPhones WHERE contactid ';
		
		$first=true;
		$query.= "IN ( ";
		foreach($data as $obj){
			if($first){
				$first=false;					
			}else{
				$query.=",";
			}
			$query.="?";				
		}		
		$query.=" ) AND accid=?";

    $stmt = $this->db->prepare($query);

    $ix = 0;
    foreach($data as $obj)
    {
      $ix ++;
      $stmt->bindValue($ix,$obj['contactid']);
    }
    $ix ++;
    $stmt->bindValue($ix,$accid);

    if($stmt->execute() === false)
    {
      $this->lasterr = $stmt->errorInfo()[2];
      return false;
    }
  }

  function SetScimUserPhones($accid,$data)
  {    
		if ( (int) $this->GetCachedConfigValue('UniquePhones') != 0 )
		{
			foreach($data as $obj)
			{
				if ($this->PhoneInUseExcept( (int) $obj['contactid'] , $accid ))
				{
					$this->lasterr = 'phone-in-use';
					return false;
				}
			}
		}
		
    $stmt = $this->db->prepare('DELETE FROM UserPhones WHERE id IN 
      (SELECT s.userphoneid FROM userphones AS u INNER JOIN 
      scimuserphones AS s ON u.id=s.userphoneid WHERE accid=:1)');

    $stmt->bindValue(1,$accid);
    $stmt->execute();

    // Remove any manually assigned phones that are now being added through scim to prevent duplicates
    $this->RemoveNonScimUserPhones($accid, $data);

    $stmt = $this->db->prepare('INSERT INTO UserPhones '
    . ' (accid,contactid) '
    . ' VALUES '
    . ' (:1    ,:2      ) '
    );

    $stmt2 = $this->db->prepare('INSERT INTO ScimUserPhones (is_primary, type, userphoneid) VALUES (:1,:2,:3)');

    foreach($data as $obj)
    {
      $stmt->bindValue(1,$accid);
      $stmt->bindValue(2,$obj['contactid']);
      if($stmt->execute() === false)
      {
        $this->lasterr = $stmt->errorInfo()[2];
        return false;
      }

      $userphoneid = $this->db->lastInsertId('userphones_id_seq');

      $stmt2->bindValue(1, $obj['scim_data']['is_primary'], PDO::PARAM_BOOL);
      $stmt2->bindValue(2, $obj['scim_data']['type']);
      $stmt2->bindValue(3, $userphoneid);
      if($stmt2->execute() === false)
      {
        $this->lasterr = $stmt2->errorInfo()[2];
        return false;
      }
    }
    return true;
  }

  function FromScim($accid)
  {
    $stmt = $this->db->prepare('SELECT EXISTS(SELECT 1 FROM ScimData WHERE accid=:1)');
    $stmt->bindValue(1, $accid);
    $stmt->execute();
    
    while($row = $stmt->fetch())
    {
      return $row['exists'];
    }

    return false;
  }

	function GetScimParent($accid)
	{
		$stmt = $this->db->prepare('SELECT scimparent FROM ScimData WHERE accid=:1');
		$stmt->bindValue(1, $accid);
    $stmt->execute();
    
    while($row = $stmt->fetch())
    {
      return $row['scimparent'];
    }

    return false;
	}
};

?>
